/* Generated by Cython 0.29 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "depends": [
            "/export/ctlhomes/oper/4pult/include/cda.h",
            "/export/ctlhomes/oper/4pult/include/cx.h",
            "/export/ctlhomes/oper/4pult/include/cx_common_types.h",
            "/export/ctlhomes/oper/4pult/include/cxscheduler.h",
            "/export/ctlhomes/oper/4pult/include/misc_types.h",
            "/export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/numpy/core/include/numpy/arrayobject.h",
            "/export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/numpy/core/include/numpy/ufuncobject.h"
        ],
        "include_dirs": [
            "/export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/numpy/core/include",
            "/export/ctlhomes/oper/4pult/include"
        ],
        "libraries": [
            "cda",
            "cx_async",
            "useful",
            "misc",
            "cxscheduler"
        ],
        "library_dirs": [
            "/export/ctlhomes/oper/4pult/lib"
        ],
        "name": "pycx4.pycda",
        "sources": [
            "./pycx4/pycda.pyx"
        ]
    },
    "module_name": "pycx4.pycda"
}
END: Cython Metadata */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_29"
#define CYTHON_HEX_VERSION 0x001D00F0
#define CYTHON_FUTURE_DIVISION 1
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
  #ifndef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS (PY_VERSION_HEX >= 0x030600B1)
  #endif
  #ifndef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK (PY_VERSION_HEX >= 0x030700A3)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if CYTHON_USE_DICT_VERSIONS
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
        static PY_UINT64_T __pyx_dict_version = 0;\
        static PyObject *__pyx_dict_cached_value = NULL;\
        if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
            (VAR) = __pyx_dict_cached_value;\
        } else {\
            (VAR) = __pyx_dict_cached_value = (LOOKUP);\
            __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
        }\
    }
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030400A1
  #define PyMem_RawMalloc(n)           PyMem_Malloc(n)
  #define PyMem_RawRealloc(p, n)       PyMem_Realloc(p, n)
  #define PyMem_RawFree(p)             PyMem_Free(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0; // PyThread_create_key reports success always
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif // TSS (Thread Specific Storage) API
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStr(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
#else
#define __Pyx_PyDict_GetItemStr(dict, name)  PyDict_GetItem(dict, name)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None || (PyString_Check(b) && !PyString_CheckExact(b)))) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
  #define PyObject_Unicode             PyObject_Str
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
#else
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : (Py_INCREF(func), func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif


#define __PYX_ERR(f_index, lineno, Ln_error) \
{ \
  __pyx_filename = __pyx_f[f_index]; __pyx_lineno = lineno; __pyx_clineno = __LINE__; goto Ln_error; \
}

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__pycx4__pycda
#define __PYX_HAVE_API__pycx4__pycda
/* Early includes */
#include <string.h>
#include <stdio.h>
#include "numpy/arrayobject.h"
#include "numpy/ufuncobject.h"
#include <stdlib.h>
#include "misc_types.h"
#include "cx.h"
#include "cx_common_types.h"
#include "cda.h"
#include <sys/types.h>
#include "cxscheduler.h"
#include "pythread.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 1
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT 0
#define __PYX_DEFAULT_STRING_ENCODING "ascii"
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c) + 1);
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime = NULL;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;

/* Header.proto */
#if !defined(CYTHON_CCOMPLEX)
  #if defined(__cplusplus)
    #define CYTHON_CCOMPLEX 1
  #elif defined(_Complex_I)
    #define CYTHON_CCOMPLEX 1
  #else
    #define CYTHON_CCOMPLEX 0
  #endif
#endif
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #include <complex>
  #else
    #include <complex.h>
  #endif
#endif
#if CYTHON_CCOMPLEX && !defined(__cplusplus) && defined(__sun__) && defined(__GNUC__)
  #undef _Complex_I
  #define _Complex_I 1.0fj
#endif


static const char *__pyx_f[] = {
  "pycx4/signal.pxi",
  "stringsource",
  "pycx4/timer.pxi",
  "pycx4/context.pxi",
  "pycx4/scheduler.pxi",
  "pycx4/fd_event.pxi",
  "pycx4/cxdtype.pxi",
  "pycx4/rflags.pxi",
  "pycx4/cdaobject.pxi",
  "pycx4/basechan.pxi",
  "pycx4/dchan.pxi",
  "pycx4/ichan.pxi",
  "pycx4/chan.pxi",
  "pycx4/vchan.pxi",
  "pycx4/strchan.pxi",
  "pycx4/chan_factory.pxi",
  "pycx4/gw.pxi",
  "__init__.pxd",
  "pycx4/pycda.pyx",
  "type.pxd",
  "bool.pxd",
  "complex.pxd",
  "pycx4/imports.pxi",
  "pycx4/cda_common.pxi",
  "pycx4/cda_all.pxi",
};
/* ForceInitThreads.proto */
#ifndef __PYX_FORCE_INIT_THREADS
  #define __PYX_FORCE_INIT_THREADS 0
#endif

/* NoFastGil.proto */
#define __Pyx_PyGILState_Ensure PyGILState_Ensure
#define __Pyx_PyGILState_Release PyGILState_Release
#define __Pyx_FastGIL_Remember()
#define __Pyx_FastGIL_Forget()
#define __Pyx_FastGilFuncInit()


/* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":776
 * # in Cython to enable them only on the right systems.
 * 
 * ctypedef npy_int8       int8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 */
typedef npy_int8 __pyx_t_5numpy_int8_t;

/* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":777
 * 
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t
 */
typedef npy_int16 __pyx_t_5numpy_int16_t;

/* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":778
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int64      int64_t
 * #ctypedef npy_int96      int96_t
 */
typedef npy_int32 __pyx_t_5numpy_int32_t;

/* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":779
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_int96      int96_t
 * #ctypedef npy_int128     int128_t
 */
typedef npy_int64 __pyx_t_5numpy_int64_t;

/* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":783
 * #ctypedef npy_int128     int128_t
 * 
 * ctypedef npy_uint8      uint8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 */
typedef npy_uint8 __pyx_t_5numpy_uint8_t;

/* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":784
 * 
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t
 */
typedef npy_uint16 __pyx_t_5numpy_uint16_t;

/* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":785
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint64     uint64_t
 * #ctypedef npy_uint96     uint96_t
 */
typedef npy_uint32 __pyx_t_5numpy_uint32_t;

/* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":786
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_uint96     uint96_t
 * #ctypedef npy_uint128    uint128_t
 */
typedef npy_uint64 __pyx_t_5numpy_uint64_t;

/* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":790
 * #ctypedef npy_uint128    uint128_t
 * 
 * ctypedef npy_float32    float32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_float64    float64_t
 * #ctypedef npy_float80    float80_t
 */
typedef npy_float32 __pyx_t_5numpy_float32_t;

/* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":791
 * 
 * ctypedef npy_float32    float32_t
 * ctypedef npy_float64    float64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_float80    float80_t
 * #ctypedef npy_float128   float128_t
 */
typedef npy_float64 __pyx_t_5numpy_float64_t;

/* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":800
 * # The int types are mapped a bit surprising --
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   long_t
 * ctypedef npy_longlong   longlong_t
 */
typedef npy_long __pyx_t_5numpy_int_t;

/* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":801
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   long_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   longlong_t
 * 
 */
typedef npy_longlong __pyx_t_5numpy_long_t;

/* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":802
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   long_t
 * ctypedef npy_longlong   longlong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_ulong      uint_t
 */
typedef npy_longlong __pyx_t_5numpy_longlong_t;

/* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":804
 * ctypedef npy_longlong   longlong_t
 * 
 * ctypedef npy_ulong      uint_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulong_t
 * ctypedef npy_ulonglong  ulonglong_t
 */
typedef npy_ulong __pyx_t_5numpy_uint_t;

/* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":805
 * 
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulong_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 */
typedef npy_ulonglong __pyx_t_5numpy_ulong_t;

/* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":806
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulong_t
 * ctypedef npy_ulonglong  ulonglong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_intp       intp_t
 */
typedef npy_ulonglong __pyx_t_5numpy_ulonglong_t;

/* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":808
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 * ctypedef npy_intp       intp_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uintp      uintp_t
 * 
 */
typedef npy_intp __pyx_t_5numpy_intp_t;

/* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":809
 * 
 * ctypedef npy_intp       intp_t
 * ctypedef npy_uintp      uintp_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_double     float_t
 */
typedef npy_uintp __pyx_t_5numpy_uintp_t;

/* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":811
 * ctypedef npy_uintp      uintp_t
 * 
 * ctypedef npy_double     float_t             # <<<<<<<<<<<<<<
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t
 */
typedef npy_double __pyx_t_5numpy_float_t;

/* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":812
 * 
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longdouble longdouble_t
 * 
 */
typedef npy_double __pyx_t_5numpy_double_t;

/* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":813
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cfloat      cfloat_t
 */
typedef npy_longdouble __pyx_t_5numpy_longdouble_t;
/* Declarations.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< float > __pyx_t_float_complex;
  #else
    typedef float _Complex __pyx_t_float_complex;
  #endif
#else
    typedef struct { float real, imag; } __pyx_t_float_complex;
#endif
static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float, float);

/* Declarations.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< double > __pyx_t_double_complex;
  #else
    typedef double _Complex __pyx_t_double_complex;
  #endif
#else
    typedef struct { double real, imag; } __pyx_t_double_complex;
#endif
static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double, double);


/*--- Type declarations ---*/
struct __pyx_obj_5pycx4_5pycda_Signal;
struct __pyx_obj_5pycx4_5pycda_Timer;
struct __pyx_obj_5pycx4_5pycda_FdEvent;
struct __pyx_obj_5pycx4_5pycda_CdaObject;
struct __pyx_obj_5pycx4_5pycda_Context;
struct __pyx_obj_5pycx4_5pycda_BaseChan;
struct __pyx_obj_5pycx4_5pycda_DChan;
struct __pyx_obj_5pycx4_5pycda_IChan;
struct __pyx_obj_5pycx4_5pycda_Chan;
struct __pyx_obj_5pycx4_5pycda_VChan;
struct __pyx_obj_5pycx4_5pycda_StrChan;
struct __pyx_obj_5pycx4_5pycda_ChanFactory;
struct __pyx_obj_5pycx4_5pycda_PassGW;
struct __pyx_obj_5pycx4_5pycda_ProcGW;

/* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":815
 * ctypedef npy_longdouble longdouble_t
 * 
 * ctypedef npy_cfloat      cfloat_t             # <<<<<<<<<<<<<<
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t
 */
typedef npy_cfloat __pyx_t_5numpy_cfloat_t;

/* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":816
 * 
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t             # <<<<<<<<<<<<<<
 * ctypedef npy_clongdouble clongdouble_t
 * 
 */
typedef npy_cdouble __pyx_t_5numpy_cdouble_t;

/* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":817
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cdouble     complex_t
 */
typedef npy_clongdouble __pyx_t_5numpy_clongdouble_t;

/* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":819
 * ctypedef npy_clongdouble clongdouble_t
 * 
 * ctypedef npy_cdouble     complex_t             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 */
typedef npy_cdouble __pyx_t_5numpy_complex_t;
struct __pyx_t_3cx4_11cxscheduler_timeval;
typedef struct __pyx_t_3cx4_11cxscheduler_timeval __pyx_t_3cx4_11cxscheduler_timeval;

/* "cx4/cxscheduler.pxd":6
 * # Dew to somehow incorrect timeval definition in posix.time
 * # if timeval in posix.time - not a type
 * ctypedef struct timeval:             # <<<<<<<<<<<<<<
 *         time_t      tv_sec
 *         suseconds_t tv_usec
 */
struct __pyx_t_3cx4_11cxscheduler_timeval {
  time_t tv_sec;
  suseconds_t tv_usec;
};
struct __pyx_opt_args_5pycx4_5pycda_5Timer_singleShot;
struct __pyx_t_5pycx4_5pycda_event;
typedef struct __pyx_t_5pycx4_5pycda_event __pyx_t_5pycx4_5pycda_event;

/* "pycx4/timer.pxi":36
 *         self.tid = sl_enq_tout_after(0, NULL, self.usec, sltimer_proc, <void*>self)
 * 
 *     cpdef singleShot(self, int msec=0, proc=None):             # <<<<<<<<<<<<<<
 *         if msec > 0:
 *             self.setInterval(msec)
 */
struct __pyx_opt_args_5pycx4_5pycda_5Timer_singleShot {
  int __pyx_n;
  int msec;
  PyObject *proc;
};

/* "pycx4/event.pxi":4
 * 
 * # struct to extend private pointer
 * ctypedef struct event:             # <<<<<<<<<<<<<<
 *     int evmask    # cda eventmask
 *     void *evproc  # pointer to proc function
 */
struct __pyx_t_5pycx4_5pycda_event {
  int evmask;
  void *evproc;
  void *objptr;
  void *userptr;
};

/* "pycx4/signal.pxi":4
 * from cpython cimport Py_INCREF,Py_DECREF
 * 
 * cdef class Signal:             # <<<<<<<<<<<<<<
 *     cdef:
 *         void **callbacks
 */
struct __pyx_obj_5pycx4_5pycda_Signal {
  PyObject_HEAD
  struct __pyx_vtabstruct_5pycx4_5pycda_Signal *__pyx_vtab;
  void **callbacks;
  int cnum;
};


/* "pycx4/timer.pxi":12
 * 
 * 
 * cdef class Timer:             # <<<<<<<<<<<<<<
 *     cdef readonly:
 *         int usec
 */
struct __pyx_obj_5pycx4_5pycda_Timer {
  PyObject_HEAD
  struct __pyx_vtabstruct_5pycx4_5pycda_Timer *__pyx_vtab;
  int usec;
  int repeat;
  int active;
  sl_tid_t tid;
  struct __pyx_obj_5pycx4_5pycda_Signal *timeout;
};


/* "pycx4/fd_event.pxi":8
 * 
 * 
 * cdef class FdEvent:             # <<<<<<<<<<<<<<
 *     cdef readonly:
 *         object file, ready
 */
struct __pyx_obj_5pycx4_5pycda_FdEvent {
  PyObject_HEAD
  PyObject *file;
  PyObject *ready;
  int mask;
  sl_fdh_t fhd;
};


/* "pycx4/cdaobject.pxi":3
 * 
 * 
 * cdef class CdaObject:             # <<<<<<<<<<<<<<
 *     cdef:
 *         event **events
 */
struct __pyx_obj_5pycx4_5pycda_CdaObject {
  PyObject_HEAD
  struct __pyx_vtabstruct_5pycx4_5pycda_CdaObject *__pyx_vtab;
  __pyx_t_5pycx4_5pycda_event **events;
  int evnum;
};


/* "pycx4/context.pxi":9
 * 
 * 
 * cdef class Context(CdaObject):             # <<<<<<<<<<<<<<
 *     cdef readonly:
 *         cda_context_t cid
 */
struct __pyx_obj_5pycx4_5pycda_Context {
  struct __pyx_obj_5pycx4_5pycda_CdaObject __pyx_base;
  cda_context_t cid;
  PyObject *defpfx;
  PyObject *serverCycle;
  void **chans;
  int channum;
};


/* "pycx4/basechan.pxi":71
 * 
 * # wrapper-class for low-level functions and channel registration
 * cdef class BaseChan(CdaObject):             # <<<<<<<<<<<<<<
 *     cdef readonly:
 *         cda_dataref_t ref      # chan id
 */
struct __pyx_obj_5pycx4_5pycda_BaseChan {
  struct __pyx_obj_5pycx4_5pycda_CdaObject __pyx_base;
  cda_dataref_t ref;
  PyObject *name;
  int max_nelems;
  cxdtype_t dtype;
  size_t itemsize;
  int64 time;
  int64 prev_time;
  long rslv_stat;
  PyObject *rslv_str;
  double quant;
  int registered;
  int first_cycle;
  int lock_state;
  struct __pyx_obj_5pycx4_5pycda_Context *context;
  rflags_t rflags;
  PyObject *rng;
  char *ident;
  char *label;
  char *tip;
  char *comment;
  char *geoinfo;
  char *rsrvd6;
  char *units;
  char *dpyfmt;
  PyObject *valueMeasured;
  PyObject *valueChanged;
  PyObject *resolve;
  PyObject *stringsUpdate;
  PyObject *lockStateUpdate;
};


/* "pycx4/dchan.pxi":2
 * # scalar double channel
 * cdef class DChan(BaseChan):             # <<<<<<<<<<<<<<
 *     """
 *     One-double channel
 */
struct __pyx_obj_5pycx4_5pycda_DChan {
  struct __pyx_obj_5pycx4_5pycda_BaseChan __pyx_base;
  double val;
  double prev_val;
  double tolerance;
};


/* "pycx4/ichan.pxi":2
 * # scalar int32 channel with simplified interface
 * cdef class IChan(BaseChan):             # <<<<<<<<<<<<<<
 *     cdef:
 *         # all general properties defined in base classes
 */
struct __pyx_obj_5pycx4_5pycda_IChan {
  struct __pyx_obj_5pycx4_5pycda_BaseChan __pyx_base;
  int val;
  int prev_val;
  int tolerance;
};


/* "pycx4/chan.pxi":2
 * # general channel (any cx type)
 * cdef class Chan(BaseChan):             # <<<<<<<<<<<<<<
 *     cdef:
 *         readonly object val, prev_val
 */
struct __pyx_obj_5pycx4_5pycda_Chan {
  struct __pyx_obj_5pycx4_5pycda_BaseChan __pyx_base;
  PyObject *val;
  PyObject *prev_val;
};


/* "pycx4/vchan.pxi":4
 * 
 * # vector-data channel class
 * cdef class VChan(BaseChan):             # <<<<<<<<<<<<<<
 *     cdef:
 *         readonly np.ndarray val
 */
struct __pyx_obj_5pycx4_5pycda_VChan {
  struct __pyx_obj_5pycx4_5pycda_BaseChan __pyx_base;
  PyArrayObject *val;
  PyObject *npdtype;
  int nelems;
  int change_sign;
};


/* "pycx4/strchan.pxi":2
 * # chan to transfer python string
 * cdef class StrChan(BaseChan):             # <<<<<<<<<<<<<<
 *     cdef:
 *         readonly str val, prev_val
 */
struct __pyx_obj_5pycx4_5pycda_StrChan {
  struct __pyx_obj_5pycx4_5pycda_BaseChan __pyx_base;
  PyObject *val;
  PyObject *prev_val;
  char *cval;
};


/* "pycx4/chan_factory.pxi":2
 * 
 * cdef class ChanFactory:             # <<<<<<<<<<<<<<
 *     cdef:
 *         dict single_chans, vector_chans
 */
struct __pyx_obj_5pycx4_5pycda_ChanFactory {
  PyObject_HEAD
  PyObject *single_chans;
  PyObject *vector_chans;
};


/* "pycx4/gw.pxi":6
 * 
 * # pass value
 * cdef class PassGW:             # <<<<<<<<<<<<<<
 *     cdef:
 *         object in_chan, out_chan, last_val
 */
struct __pyx_obj_5pycx4_5pycda_PassGW {
  PyObject_HEAD
  struct __pyx_vtabstruct_5pycx4_5pycda_PassGW *__pyx_vtab;
  PyObject *in_chan;
  PyObject *out_chan;
  PyObject *last_val;
};


/* "pycx4/gw.pxi":32
 * 
 * 
 * cdef class ProcGW:             # <<<<<<<<<<<<<<
 *     cdef:
 *         object in_chan, out_chan
 */
struct __pyx_obj_5pycx4_5pycda_ProcGW {
  PyObject_HEAD
  struct __pyx_vtabstruct_5pycx4_5pycda_ProcGW *__pyx_vtab;
  PyObject *in_chan;
  PyObject *out_chan;
};



/* "pycx4/signal.pxi":4
 * from cpython cimport Py_INCREF,Py_DECREF
 * 
 * cdef class Signal:             # <<<<<<<<<<<<<<
 *     cdef:
 *         void **callbacks
 */

struct __pyx_vtabstruct_5pycx4_5pycda_Signal {
  PyObject *(*connect)(struct __pyx_obj_5pycx4_5pycda_Signal *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*disconnect)(struct __pyx_obj_5pycx4_5pycda_Signal *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_5pycx4_5pycda_Signal *__pyx_vtabptr_5pycx4_5pycda_Signal;


/* "pycx4/timer.pxi":12
 * 
 * 
 * cdef class Timer:             # <<<<<<<<<<<<<<
 *     cdef readonly:
 *         int usec
 */

struct __pyx_vtabstruct_5pycx4_5pycda_Timer {
  PyObject *(*stop)(struct __pyx_obj_5pycx4_5pycda_Timer *, int __pyx_skip_dispatch);
  PyObject *(*start)(struct __pyx_obj_5pycx4_5pycda_Timer *, int __pyx_skip_dispatch);
  PyObject *(*singleShot)(struct __pyx_obj_5pycx4_5pycda_Timer *, int __pyx_skip_dispatch, struct __pyx_opt_args_5pycx4_5pycda_5Timer_singleShot *__pyx_optional_args);
  int (*interval)(struct __pyx_obj_5pycx4_5pycda_Timer *, int __pyx_skip_dispatch);
  void (*setInterval)(struct __pyx_obj_5pycx4_5pycda_Timer *, int, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_5pycx4_5pycda_Timer *__pyx_vtabptr_5pycx4_5pycda_Timer;


/* "pycx4/cdaobject.pxi":3
 * 
 * 
 * cdef class CdaObject:             # <<<<<<<<<<<<<<
 *     cdef:
 *         event **events
 */

struct __pyx_vtabstruct_5pycx4_5pycda_CdaObject {
  int (*add_event)(struct __pyx_obj_5pycx4_5pycda_CdaObject *, int, void *, void *, void *);
  int (*del_event)(struct __pyx_obj_5pycx4_5pycda_CdaObject *, int, void *, void *, void *);
  void (*register_event)(struct __pyx_obj_5pycx4_5pycda_CdaObject *, __pyx_t_5pycx4_5pycda_event *);
  void (*unregister_event)(struct __pyx_obj_5pycx4_5pycda_CdaObject *, __pyx_t_5pycx4_5pycda_event *);
};
static struct __pyx_vtabstruct_5pycx4_5pycda_CdaObject *__pyx_vtabptr_5pycx4_5pycda_CdaObject;


/* "pycx4/context.pxi":9
 * 
 * 
 * cdef class Context(CdaObject):             # <<<<<<<<<<<<<<
 *     cdef readonly:
 *         cda_context_t cid
 */

struct __pyx_vtabstruct_5pycx4_5pycda_Context {
  struct __pyx_vtabstruct_5pycx4_5pycda_CdaObject __pyx_base;
  void (*save_chan)(struct __pyx_obj_5pycx4_5pycda_Context *, void *);
  void (*drop_chan)(struct __pyx_obj_5pycx4_5pycda_Context *, void *);
  void (*check_exception)(struct __pyx_obj_5pycx4_5pycda_Context *, int);
};
static struct __pyx_vtabstruct_5pycx4_5pycda_Context *__pyx_vtabptr_5pycx4_5pycda_Context;


/* "pycx4/basechan.pxi":71
 * 
 * # wrapper-class for low-level functions and channel registration
 * cdef class BaseChan(CdaObject):             # <<<<<<<<<<<<<<
 *     cdef readonly:
 *         cda_dataref_t ref      # chan id
 */

struct __pyx_vtabstruct_5pycx4_5pycda_BaseChan {
  struct __pyx_vtabstruct_5pycx4_5pycda_CdaObject __pyx_base;
  void (*cb)(struct __pyx_obj_5pycx4_5pycda_BaseChan *);
  int (*snd_data)(struct __pyx_obj_5pycx4_5pycda_BaseChan *, cxdtype_t, int, void *);
  int (*get_data)(struct __pyx_obj_5pycx4_5pycda_BaseChan *, size_t, size_t, void *);
  int (*current_nelems)(struct __pyx_obj_5pycx4_5pycda_BaseChan *);
  void (*get_src)(struct __pyx_obj_5pycx4_5pycda_BaseChan *, char const **);
  void (*check_exception)(struct __pyx_obj_5pycx4_5pycda_BaseChan *, int);
  PyObject *(*is_available)(struct __pyx_obj_5pycx4_5pycda_BaseChan *, int __pyx_skip_dispatch);
  PyObject *(*rflags_text)(struct __pyx_obj_5pycx4_5pycda_BaseChan *, int __pyx_skip_dispatch);
  PyObject *(*get_range)(struct __pyx_obj_5pycx4_5pycda_BaseChan *, int __pyx_skip_dispatch);
  PyObject *(*get_strings)(struct __pyx_obj_5pycx4_5pycda_BaseChan *, int __pyx_skip_dispatch);
  PyObject *(*get_lockstate)(struct __pyx_obj_5pycx4_5pycda_BaseChan *, int __pyx_skip_dispatch);
  PyObject *(*lock)(struct __pyx_obj_5pycx4_5pycda_BaseChan *, int __pyx_skip_dispatch);
  PyObject *(*unlock)(struct __pyx_obj_5pycx4_5pycda_BaseChan *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_5pycx4_5pycda_BaseChan *__pyx_vtabptr_5pycx4_5pycda_BaseChan;


/* "pycx4/dchan.pxi":2
 * # scalar double channel
 * cdef class DChan(BaseChan):             # <<<<<<<<<<<<<<
 *     """
 *     One-double channel
 */

struct __pyx_vtabstruct_5pycx4_5pycda_DChan {
  struct __pyx_vtabstruct_5pycx4_5pycda_BaseChan __pyx_base;
  void (*setValue)(struct __pyx_obj_5pycx4_5pycda_DChan *, double, int __pyx_skip_dispatch);
  void (*setTolerance)(struct __pyx_obj_5pycx4_5pycda_DChan *, double, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_5pycx4_5pycda_DChan *__pyx_vtabptr_5pycx4_5pycda_DChan;


/* "pycx4/ichan.pxi":2
 * # scalar int32 channel with simplified interface
 * cdef class IChan(BaseChan):             # <<<<<<<<<<<<<<
 *     cdef:
 *         # all general properties defined in base classes
 */

struct __pyx_vtabstruct_5pycx4_5pycda_IChan {
  struct __pyx_vtabstruct_5pycx4_5pycda_BaseChan __pyx_base;
  void (*setValue)(struct __pyx_obj_5pycx4_5pycda_IChan *, int, int __pyx_skip_dispatch);
  void (*setTolerance)(struct __pyx_obj_5pycx4_5pycda_IChan *, int, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_5pycx4_5pycda_IChan *__pyx_vtabptr_5pycx4_5pycda_IChan;


/* "pycx4/chan.pxi":2
 * # general channel (any cx type)
 * cdef class Chan(BaseChan):             # <<<<<<<<<<<<<<
 *     cdef:
 *         readonly object val, prev_val
 */

struct __pyx_vtabstruct_5pycx4_5pycda_Chan {
  struct __pyx_vtabstruct_5pycx4_5pycda_BaseChan __pyx_base;
  void (*setValue)(struct __pyx_obj_5pycx4_5pycda_Chan *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_5pycx4_5pycda_Chan *__pyx_vtabptr_5pycx4_5pycda_Chan;


/* "pycx4/vchan.pxi":4
 * 
 * # vector-data channel class
 * cdef class VChan(BaseChan):             # <<<<<<<<<<<<<<
 *     cdef:
 *         readonly np.ndarray val
 */

struct __pyx_vtabstruct_5pycx4_5pycda_VChan {
  struct __pyx_vtabstruct_5pycx4_5pycda_BaseChan __pyx_base;
  void (*setValue)(struct __pyx_obj_5pycx4_5pycda_VChan *, PyArrayObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_5pycx4_5pycda_VChan *__pyx_vtabptr_5pycx4_5pycda_VChan;


/* "pycx4/strchan.pxi":2
 * # chan to transfer python string
 * cdef class StrChan(BaseChan):             # <<<<<<<<<<<<<<
 *     cdef:
 *         readonly str val, prev_val
 */

struct __pyx_vtabstruct_5pycx4_5pycda_StrChan {
  struct __pyx_vtabstruct_5pycx4_5pycda_BaseChan __pyx_base;
  void (*setValue)(struct __pyx_obj_5pycx4_5pycda_StrChan *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_5pycx4_5pycda_StrChan *__pyx_vtabptr_5pycx4_5pycda_StrChan;


/* "pycx4/gw.pxi":6
 * 
 * # pass value
 * cdef class PassGW:             # <<<<<<<<<<<<<<
 *     cdef:
 *         object in_chan, out_chan, last_val
 */

struct __pyx_vtabstruct_5pycx4_5pycda_PassGW {
  void (*in_c_proc)(struct __pyx_obj_5pycx4_5pycda_PassGW *, PyObject *, int __pyx_skip_dispatch);
  void (*out_c_proc)(struct __pyx_obj_5pycx4_5pycda_PassGW *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_5pycx4_5pycda_PassGW *__pyx_vtabptr_5pycx4_5pycda_PassGW;


/* "pycx4/gw.pxi":32
 * 
 * 
 * cdef class ProcGW:             # <<<<<<<<<<<<<<
 *     cdef:
 *         object in_chan, out_chan
 */

struct __pyx_vtabstruct_5pycx4_5pycda_ProcGW {
  void (*in_c_proc)(struct __pyx_obj_5pycx4_5pycda_ProcGW *, PyObject *, int __pyx_skip_dispatch);
  void (*out_c_proc)(struct __pyx_obj_5pycx4_5pycda_ProcGW *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_5pycx4_5pycda_ProcGW *__pyx_vtabptr_5pycx4_5pycda_ProcGW;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
#define __Pyx_GetModuleGlobalNameUncached(var, name)  {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = PyFrame_Type.tp_basicsize - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* KeywordStringCheck.proto */
static int __Pyx_CheckKeywordStrings(PyObject *kwdict, const char* function_name, int kw_allowed);

/* PyObjectCall2Args.proto */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* CallableCheck.proto */
#if CYTHON_USE_TYPE_SLOTS && PY_MAJOR_VERSION >= 3
#define __Pyx_PyCallable_Check(obj)   ((obj)->ob_type->tp_call != NULL)
#else
#define __Pyx_PyCallable_Check(obj)   PyCallable_Check(obj)
#endif

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* GetAttr.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *, PyObject *);

/* GetAttr3.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *, PyObject *, PyObject *);

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        Py_SIZE(list) = len+1;
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* PyDictContains.proto */
static CYTHON_INLINE int __Pyx_PyDict_ContainsTF(PyObject* item, PyObject* dict, int eq) {
    int result = PyDict_Contains(dict, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* DictGetItem.proto */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key);
#define __Pyx_PyObject_Dict_GetItem(obj, name)\
    (likely(PyDict_CheckExact(obj)) ?\
     __Pyx_PyDict_GetItem(obj, name) : PyObject_GetItem(obj, name))
#else
#define __Pyx_PyDict_GetItem(d, key) PyObject_GetItem(d, key)
#define __Pyx_PyObject_Dict_GetItem(obj, name)  PyObject_GetItem(obj, name)
#endif

/* PyObjectFormat.proto */
#if CYTHON_USE_UNICODE_WRITER
static PyObject* __Pyx_PyObject_Format(PyObject* s, PyObject* f);
#else
#define __Pyx_PyObject_Format(s, f) PyObject_Format(s, f)
#endif

/* PyUnicode_Unicode.proto */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_Unicode(PyObject *obj);

/* BuildPyUnicode.proto */
static PyObject* __Pyx_PyUnicode_BuildFromAscii(Py_ssize_t ulength, char* chars, int clength,
                                                int prepend_sign, char padding_char);

/* CIntToPyUnicode.proto */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_From_int(int value, Py_ssize_t width, char padding_char, char format_char);

/* IncludeStringH.proto */
#include <string.h>

/* JoinPyUnicode.proto */
static PyObject* __Pyx_PyUnicode_Join(PyObject* value_tuple, Py_ssize_t value_count, Py_ssize_t result_ulength,
                                      Py_UCS4 max_char);

/* PyObjectFormatAndDecref.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatSimpleAndDecref(PyObject* s, PyObject* f);
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatAndDecref(PyObject* s, PyObject* f);

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely((Py_TYPE(obj) == type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* dict_getitem_default.proto */
static PyObject* __Pyx_PyDict_GetItemDefault(PyObject* d, PyObject* key, PyObject* default_value);

/* UnpackUnboundCMethod.proto */
typedef struct {
    PyObject *type;
    PyObject **method_name;
    PyCFunction func;
    PyObject *method;
    int flag;
} __Pyx_CachedCFunction;

/* CallUnboundCMethod1.proto */
static PyObject* __Pyx__CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg);
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg);
#else
#define __Pyx_CallUnboundCMethod1(cfunc, self, arg)  __Pyx__CallUnboundCMethod1(cfunc, self, arg)
#endif

/* CallUnboundCMethod2.proto */
static PyObject* __Pyx__CallUnboundCMethod2(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg1, PyObject* arg2);
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030600B1
static CYTHON_INLINE PyObject *__Pyx_CallUnboundCMethod2(__Pyx_CachedCFunction *cfunc, PyObject *self, PyObject *arg1, PyObject *arg2);
#else
#define __Pyx_CallUnboundCMethod2(cfunc, self, arg1, arg2)  __Pyx__CallUnboundCMethod2(cfunc, self, arg1, arg2)
#endif

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* ObjectGetItem.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key);
#else
#define __Pyx_PyObject_GetItem(obj, key)  PyObject_GetItem(obj, key)
#endif

/* decode_c_string_utf16.proto */
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_DecodeUTF16(const char *s, Py_ssize_t size, const char *errors) {
    int byteorder = 0;
    return PyUnicode_DecodeUTF16(s, size, errors, &byteorder);
}
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_DecodeUTF16LE(const char *s, Py_ssize_t size, const char *errors) {
    int byteorder = -1;
    return PyUnicode_DecodeUTF16(s, size, errors, &byteorder);
}
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_DecodeUTF16BE(const char *s, Py_ssize_t size, const char *errors) {
    int byteorder = 1;
    return PyUnicode_DecodeUTF16(s, size, errors, &byteorder);
}

/* decode_c_bytes.proto */
static CYTHON_INLINE PyObject* __Pyx_decode_c_bytes(
         const char* cstring, Py_ssize_t length, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors));

/* decode_bytes.proto */
static CYTHON_INLINE PyObject* __Pyx_decode_bytes(
         PyObject* string, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors)) {
    return __Pyx_decode_c_bytes(
        PyBytes_AS_STRING(string), PyBytes_GET_SIZE(string),
        start, stop, encoding, errors, decode_func);
}

/* BytesEquals.proto */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* PySequenceContains.proto */
static CYTHON_INLINE int __Pyx_PySequence_ContainsTF(PyObject* item, PyObject* seq, int eq) {
    int result = PySequence_Contains(seq, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* PyIntCompare.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, long intval, long inplace);

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* HasAttr.proto */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* RaiseNoneIterError.proto */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void);

/* GetTopmostException.proto */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem * __Pyx_PyErr_GetTopmostException(PyThreadState *tstate);
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* CallNextTpTraverse.proto */
static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse);

/* CallNextTpClear.proto */
static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_dealloc);

/* PyObject_GenericGetAttrNoDict.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttrNoDict PyObject_GenericGetAttr
#endif

/* PyObject_GenericGetAttr.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttr PyObject_GenericGetAttr
#endif

/* SetVTable.proto */
static int __Pyx_SetVtable(PyObject *dict, void *vtable);

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto
#define __PYX_HAVE_RT_ImportType_proto
enum __Pyx_ImportType_CheckSize {
   __Pyx_ImportType_CheckSize_Error = 0,
   __Pyx_ImportType_CheckSize_Warn = 1,
   __Pyx_ImportType_CheckSize_Ignore = 2
};
static PyTypeObject *__Pyx_ImportType(PyObject* module, const char *module_name, const char *class_name, size_t size, enum __Pyx_ImportType_CheckSize check_size);
#endif

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_sl_tid_t(sl_tid_t value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_sl_fdh_t(sl_fdh_t value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int32(int32 value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int8(int8 value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int16(int16 value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int64(int64 value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint8(uint8 value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint16(uint16 value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint32(uint32 value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint64(uint64 value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_char8(char8 value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_cda_context_t(cda_context_t value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_CxDataRef_t(CxDataRef_t value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* RealImag.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #define __Pyx_CREAL(z) ((z).real())
    #define __Pyx_CIMAG(z) ((z).imag())
  #else
    #define __Pyx_CREAL(z) (__real__(z))
    #define __Pyx_CIMAG(z) (__imag__(z))
  #endif
#else
    #define __Pyx_CREAL(z) ((z).real)
    #define __Pyx_CIMAG(z) ((z).imag)
#endif
#if defined(__cplusplus) && CYTHON_CCOMPLEX\
        && (defined(_WIN32) || defined(__clang__) || (defined(__GNUC__) && (__GNUC__ >= 5 || __GNUC__ == 4 && __GNUC_MINOR__ >= 4 )) || __cplusplus >= 201103)
    #define __Pyx_SET_CREAL(z,x) ((z).real(x))
    #define __Pyx_SET_CIMAG(z,y) ((z).imag(y))
#else
    #define __Pyx_SET_CREAL(z,x) __Pyx_CREAL(z) = (x)
    #define __Pyx_SET_CIMAG(z,y) __Pyx_CIMAG(z) = (y)
#endif

/* Arithmetic.proto */
#if CYTHON_CCOMPLEX
    #define __Pyx_c_eq_float(a, b)   ((a)==(b))
    #define __Pyx_c_sum_float(a, b)  ((a)+(b))
    #define __Pyx_c_diff_float(a, b) ((a)-(b))
    #define __Pyx_c_prod_float(a, b) ((a)*(b))
    #define __Pyx_c_quot_float(a, b) ((a)/(b))
    #define __Pyx_c_neg_float(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero_float(z) ((z)==(float)0)
    #define __Pyx_c_conj_float(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs_float(z)     (::std::abs(z))
        #define __Pyx_c_pow_float(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero_float(z) ((z)==0)
    #define __Pyx_c_conj_float(z)    (conjf(z))
    #if 1
        #define __Pyx_c_abs_float(z)     (cabsf(z))
        #define __Pyx_c_pow_float(a, b)  (cpowf(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sum_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_diff_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prod_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_neg_float(__pyx_t_float_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero_float(__pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conj_float(__pyx_t_float_complex);
    #if 1
        static CYTHON_INLINE float __Pyx_c_abs_float(__pyx_t_float_complex);
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_pow_float(__pyx_t_float_complex, __pyx_t_float_complex);
    #endif
#endif

/* Arithmetic.proto */
#if CYTHON_CCOMPLEX
    #define __Pyx_c_eq_double(a, b)   ((a)==(b))
    #define __Pyx_c_sum_double(a, b)  ((a)+(b))
    #define __Pyx_c_diff_double(a, b) ((a)-(b))
    #define __Pyx_c_prod_double(a, b) ((a)*(b))
    #define __Pyx_c_quot_double(a, b) ((a)/(b))
    #define __Pyx_c_neg_double(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero_double(z) ((z)==(double)0)
    #define __Pyx_c_conj_double(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs_double(z)     (::std::abs(z))
        #define __Pyx_c_pow_double(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero_double(z) ((z)==0)
    #define __Pyx_c_conj_double(z)    (conj(z))
    #if 1
        #define __Pyx_c_abs_double(z)     (cabs(z))
        #define __Pyx_c_pow_double(a, b)  (cpow(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg_double(__pyx_t_double_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero_double(__pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj_double(__pyx_t_double_complex);
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs_double(__pyx_t_double_complex);
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow_double(__pyx_t_double_complex, __pyx_t_double_complex);
    #endif
#endif

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__NPY_TYPES(enum NPY_TYPES value);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE sl_tid_t __Pyx_PyInt_As_sl_tid_t(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE sl_fdh_t __Pyx_PyInt_As_sl_fdh_t(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE int32 __Pyx_PyInt_As_int32(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE int8 __Pyx_PyInt_As_int8(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE int16 __Pyx_PyInt_As_int16(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE int64 __Pyx_PyInt_As_int64(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE uint8 __Pyx_PyInt_As_uint8(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE uint16 __Pyx_PyInt_As_uint16(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE uint32 __Pyx_PyInt_As_uint32(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE uint64 __Pyx_PyInt_As_uint64(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE char8 __Pyx_PyInt_As_char8(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE size_t __Pyx_PyInt_As_size_t(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static PyObject *__pyx_f_5pycx4_5pycda_6Signal_connect(struct __pyx_obj_5pycx4_5pycda_Signal *__pyx_v_self, PyObject *__pyx_v_slot, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_5pycx4_5pycda_6Signal_disconnect(struct __pyx_obj_5pycx4_5pycda_Signal *__pyx_v_self, PyObject *__pyx_v_callback, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_5pycx4_5pycda_5Timer_stop(struct __pyx_obj_5pycx4_5pycda_Timer *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_5pycx4_5pycda_5Timer_start(struct __pyx_obj_5pycx4_5pycda_Timer *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_5pycx4_5pycda_5Timer_singleShot(struct __pyx_obj_5pycx4_5pycda_Timer *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_5pycx4_5pycda_5Timer_singleShot *__pyx_optional_args); /* proto*/
static int __pyx_f_5pycx4_5pycda_5Timer_interval(struct __pyx_obj_5pycx4_5pycda_Timer *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static void __pyx_f_5pycx4_5pycda_5Timer_setInterval(struct __pyx_obj_5pycx4_5pycda_Timer *__pyx_v_self, int __pyx_v_msec, int __pyx_skip_dispatch); /* proto*/
static int __pyx_f_5pycx4_5pycda_9CdaObject_add_event(struct __pyx_obj_5pycx4_5pycda_CdaObject *__pyx_v_self, int __pyx_v_evmask, void *__pyx_v_evproc, void *__pyx_v_objptr, void *__pyx_v_userptr); /* proto*/
static int __pyx_f_5pycx4_5pycda_9CdaObject_del_event(struct __pyx_obj_5pycx4_5pycda_CdaObject *__pyx_v_self, int __pyx_v_evmask, void *__pyx_v_evproc, void *__pyx_v_objptr, void *__pyx_v_userptr); /* proto*/
static void __pyx_f_5pycx4_5pycda_9CdaObject_register_event(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_CdaObject *__pyx_v_self, CYTHON_UNUSED __pyx_t_5pycx4_5pycda_event *__pyx_v_ev); /* proto*/
static void __pyx_f_5pycx4_5pycda_9CdaObject_unregister_event(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_CdaObject *__pyx_v_self, CYTHON_UNUSED __pyx_t_5pycx4_5pycda_event *__pyx_v_ev); /* proto*/
static void __pyx_f_5pycx4_5pycda_7Context_save_chan(struct __pyx_obj_5pycx4_5pycda_Context *__pyx_v_self, void *__pyx_v_chan); /* proto*/
static void __pyx_f_5pycx4_5pycda_7Context_drop_chan(struct __pyx_obj_5pycx4_5pycda_Context *__pyx_v_self, void *__pyx_v_chan); /* proto*/
static void __pyx_f_5pycx4_5pycda_7Context_register_event(struct __pyx_obj_5pycx4_5pycda_Context *__pyx_v_self, __pyx_t_5pycx4_5pycda_event *__pyx_v_ev); /* proto*/
static void __pyx_f_5pycx4_5pycda_7Context_unregister_event(struct __pyx_obj_5pycx4_5pycda_Context *__pyx_v_self, __pyx_t_5pycx4_5pycda_event *__pyx_v_ev); /* proto*/
static void __pyx_f_5pycx4_5pycda_7Context_check_exception(struct __pyx_obj_5pycx4_5pycda_Context *__pyx_v_self, int __pyx_v_c_res); /* proto*/
static void __pyx_f_5pycx4_5pycda_8BaseChan_cb(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto*/
static int __pyx_f_5pycx4_5pycda_8BaseChan_snd_data(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self, cxdtype_t __pyx_v_dtype, int __pyx_v_nelems, void *__pyx_v_data_p); /* proto*/
static int __pyx_f_5pycx4_5pycda_8BaseChan_get_data(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self, size_t __pyx_v_ofs, size_t __pyx_v_size, void *__pyx_v_buf); /* proto*/
static int __pyx_f_5pycx4_5pycda_8BaseChan_current_nelems(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto*/
static void __pyx_f_5pycx4_5pycda_8BaseChan_get_src(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self, char const **__pyx_v_src_p); /* proto*/
static void __pyx_f_5pycx4_5pycda_8BaseChan_register_event(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self, __pyx_t_5pycx4_5pycda_event *__pyx_v_ev); /* proto*/
static void __pyx_f_5pycx4_5pycda_8BaseChan_unregister_event(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self, __pyx_t_5pycx4_5pycda_event *__pyx_v_ev); /* proto*/
static void __pyx_f_5pycx4_5pycda_8BaseChan_check_exception(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self, int __pyx_v_c_res); /* proto*/
static PyObject *__pyx_f_5pycx4_5pycda_8BaseChan_is_available(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_5pycx4_5pycda_8BaseChan_rflags_text(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_5pycx4_5pycda_8BaseChan_get_range(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_5pycx4_5pycda_8BaseChan_get_strings(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_5pycx4_5pycda_8BaseChan_get_lockstate(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_5pycx4_5pycda_8BaseChan_lock(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_5pycx4_5pycda_8BaseChan_unlock(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static void __pyx_f_5pycx4_5pycda_5DChan_cb(struct __pyx_obj_5pycx4_5pycda_DChan *__pyx_v_self); /* proto*/
static void __pyx_f_5pycx4_5pycda_5DChan_setValue(struct __pyx_obj_5pycx4_5pycda_DChan *__pyx_v_self, double __pyx_v_value, int __pyx_skip_dispatch); /* proto*/
static void __pyx_f_5pycx4_5pycda_5DChan_setTolerance(struct __pyx_obj_5pycx4_5pycda_DChan *__pyx_v_self, double __pyx_v_new_tolerance, int __pyx_skip_dispatch); /* proto*/
static void __pyx_f_5pycx4_5pycda_5IChan_cb(struct __pyx_obj_5pycx4_5pycda_IChan *__pyx_v_self); /* proto*/
static void __pyx_f_5pycx4_5pycda_5IChan_setValue(struct __pyx_obj_5pycx4_5pycda_IChan *__pyx_v_self, int __pyx_v_value, int __pyx_skip_dispatch); /* proto*/
static void __pyx_f_5pycx4_5pycda_5IChan_setTolerance(struct __pyx_obj_5pycx4_5pycda_IChan *__pyx_v_self, int __pyx_v_new_tolerance, int __pyx_skip_dispatch); /* proto*/
static void __pyx_f_5pycx4_5pycda_4Chan_cb(struct __pyx_obj_5pycx4_5pycda_Chan *__pyx_v_self); /* proto*/
static void __pyx_f_5pycx4_5pycda_4Chan_setValue(struct __pyx_obj_5pycx4_5pycda_Chan *__pyx_v_self, PyObject *__pyx_v_value, int __pyx_skip_dispatch); /* proto*/
static void __pyx_f_5pycx4_5pycda_5VChan_cb(struct __pyx_obj_5pycx4_5pycda_VChan *__pyx_v_self); /* proto*/
static void __pyx_f_5pycx4_5pycda_5VChan_setValue(struct __pyx_obj_5pycx4_5pycda_VChan *__pyx_v_self, PyArrayObject *__pyx_v_value, int __pyx_skip_dispatch); /* proto*/
static void __pyx_f_5pycx4_5pycda_7StrChan_cb(struct __pyx_obj_5pycx4_5pycda_StrChan *__pyx_v_self); /* proto*/
static void __pyx_f_5pycx4_5pycda_7StrChan_setValue(struct __pyx_obj_5pycx4_5pycda_StrChan *__pyx_v_self, PyObject *__pyx_v_value, int __pyx_skip_dispatch); /* proto*/
static void __pyx_f_5pycx4_5pycda_6PassGW_in_c_proc(struct __pyx_obj_5pycx4_5pycda_PassGW *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_chan, int __pyx_skip_dispatch); /* proto*/
static void __pyx_f_5pycx4_5pycda_6PassGW_out_c_proc(struct __pyx_obj_5pycx4_5pycda_PassGW *__pyx_v_self, PyObject *__pyx_v_chan, int __pyx_skip_dispatch); /* proto*/
static void __pyx_f_5pycx4_5pycda_6ProcGW_in_c_proc(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_ProcGW *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_chan, int __pyx_skip_dispatch); /* proto*/
static void __pyx_f_5pycx4_5pycda_6ProcGW_out_c_proc(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_ProcGW *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_chan, int __pyx_skip_dispatch); /* proto*/

/* Module declarations from 'cpython.buffer' */

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.type' */
static PyTypeObject *__pyx_ptype_7cpython_4type_type = 0;

/* Module declarations from 'cpython.version' */

/* Module declarations from 'cpython.exc' */

/* Module declarations from 'cpython.module' */

/* Module declarations from 'cpython.mem' */

/* Module declarations from 'cpython.tuple' */

/* Module declarations from 'cpython.list' */

/* Module declarations from 'cpython.sequence' */

/* Module declarations from 'cpython.mapping' */

/* Module declarations from 'cpython.iterator' */

/* Module declarations from 'cpython.number' */

/* Module declarations from 'cpython.int' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.bool' */
static PyTypeObject *__pyx_ptype_7cpython_4bool_bool = 0;

/* Module declarations from 'cpython.long' */

/* Module declarations from 'cpython.float' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.complex' */
static PyTypeObject *__pyx_ptype_7cpython_7complex_complex = 0;

/* Module declarations from 'cpython.string' */

/* Module declarations from 'cpython.unicode' */

/* Module declarations from 'cpython.dict' */

/* Module declarations from 'cpython.instance' */

/* Module declarations from 'cpython.function' */

/* Module declarations from 'cpython.method' */

/* Module declarations from 'cpython.weakref' */

/* Module declarations from 'cpython.getargs' */

/* Module declarations from 'cpython.pythread' */

/* Module declarations from 'cpython.pystate' */

/* Module declarations from 'cpython.cobject' */

/* Module declarations from 'cpython.oldbuffer' */

/* Module declarations from 'cpython.set' */

/* Module declarations from 'cpython.bytes' */

/* Module declarations from 'cpython.pycapsule' */

/* Module declarations from 'cpython' */

/* Module declarations from 'cpython.object' */

/* Module declarations from 'cpython.ref' */

/* Module declarations from 'numpy' */

/* Module declarations from 'numpy' */
static PyTypeObject *__pyx_ptype_5numpy_dtype = 0;
static PyTypeObject *__pyx_ptype_5numpy_flatiter = 0;
static PyTypeObject *__pyx_ptype_5numpy_broadcast = 0;
static PyTypeObject *__pyx_ptype_5numpy_ndarray = 0;
static PyTypeObject *__pyx_ptype_5numpy_ufunc = 0;
static CYTHON_INLINE char *__pyx_f_5numpy__util_dtypestring(PyArray_Descr *, char *, char *, int *); /*proto*/
static CYTHON_INLINE int __pyx_f_5numpy_import_array(void); /*proto*/

/* Module declarations from 'libc.stdlib' */

/* Module declarations from 'cx4.misc_types' */

/* Module declarations from 'cx4.cx' */

/* Module declarations from 'cx4.cx_common_types' */

/* Module declarations from 'cx4.cda' */

/* Module declarations from 'posix.types' */

/* Module declarations from 'cx4.cxscheduler' */

/* Module declarations from 'pycx4.pycda' */
static PyTypeObject *__pyx_ptype_5pycx4_5pycda_Signal = 0;
static PyTypeObject *__pyx_ptype_5pycx4_5pycda_Timer = 0;
static PyTypeObject *__pyx_ptype_5pycx4_5pycda_FdEvent = 0;
static PyTypeObject *__pyx_ptype_5pycx4_5pycda_CdaObject = 0;
static PyTypeObject *__pyx_ptype_5pycx4_5pycda_Context = 0;
static PyTypeObject *__pyx_ptype_5pycx4_5pycda_BaseChan = 0;
static PyTypeObject *__pyx_ptype_5pycx4_5pycda_DChan = 0;
static PyTypeObject *__pyx_ptype_5pycx4_5pycda_IChan = 0;
static PyTypeObject *__pyx_ptype_5pycx4_5pycda_Chan = 0;
static PyTypeObject *__pyx_ptype_5pycx4_5pycda_VChan = 0;
static PyTypeObject *__pyx_ptype_5pycx4_5pycda_StrChan = 0;
static PyTypeObject *__pyx_ptype_5pycx4_5pycda_ChanFactory = 0;
static PyTypeObject *__pyx_ptype_5pycx4_5pycda_PassGW = 0;
static PyTypeObject *__pyx_ptype_5pycx4_5pycda_ProcGW = 0;
static struct __pyx_obj_5pycx4_5pycda_Context *__pyx_v_5pycx4_5pycda_default_context = 0;
static PyObject *__pyx_f_5pycx4_5pycda_main_loop(int __pyx_skip_dispatch); /*proto*/
static PyObject *__pyx_f_5pycx4_5pycda_break_(int __pyx_skip_dispatch); /*proto*/
static void __pyx_f_5pycx4_5pycda_at_select(void); /*proto*/
static void __pyx_f_5pycx4_5pycda_sltimer_proc(int, void *, sl_tid_t, void *); /*proto*/
static void __pyx_f_5pycx4_5pycda_fd_event_proc(int, void *, sl_fdh_t, int, int, void *); /*proto*/
static PyObject *__pyx_f_5pycx4_5pycda_aval_value(CxAnyVal_t *, cxdtype_t); /*proto*/
static void __pyx_f_5pycx4_5pycda_aval_setvalue(PyObject *, CxAnyVal_t *, cxdtype_t); /*proto*/
static PyObject *__pyx_f_5pycx4_5pycda_cxdtype2np(cxdtype_t); /*proto*/
static cxdtype_t __pyx_f_5pycx4_5pycda_np2cxdtype(PyObject *); /*proto*/
static PyObject *__pyx_f_5pycx4_5pycda_rflags_text(rflags_t); /*proto*/
static void __pyx_f_5pycx4_5pycda_evproc_cont_cycle(int, void *, cda_context_t, int, int, void *); /*proto*/
static void __pyx_f_5pycx4_5pycda_evproc_rslvstat(int, void *, cda_dataref_t, int, void *, void *); /*proto*/
static void __pyx_f_5pycx4_5pycda_evproc_update(int, void *, cda_dataref_t, int, void *, void *); /*proto*/
static void __pyx_f_5pycx4_5pycda_quant_update(int, void *, cda_dataref_t, int, void *, void *); /*proto*/
static void __pyx_f_5pycx4_5pycda_range_update(int, void *, cda_dataref_t, int, void *, void *); /*proto*/
static void __pyx_f_5pycx4_5pycda_strs_update(int, void *, cda_dataref_t, int, void *, void *); /*proto*/
static void __pyx_f_5pycx4_5pycda_lockstat_update(int, void *, cda_dataref_t, int, void *, void *); /*proto*/
static PyObject *__pyx_f_5pycx4_5pycda___pyx_unpickle_Timer__set_state(struct __pyx_obj_5pycx4_5pycda_Timer *, PyObject *); /*proto*/
static PyObject *__pyx_f_5pycx4_5pycda___pyx_unpickle_FdEvent__set_state(struct __pyx_obj_5pycx4_5pycda_FdEvent *, PyObject *); /*proto*/
static PyObject *__pyx_f_5pycx4_5pycda___pyx_unpickle_ChanFactory__set_state(struct __pyx_obj_5pycx4_5pycda_ChanFactory *, PyObject *); /*proto*/
static PyObject *__pyx_f_5pycx4_5pycda___pyx_unpickle_PassGW__set_state(struct __pyx_obj_5pycx4_5pycda_PassGW *, PyObject *); /*proto*/
static PyObject *__pyx_f_5pycx4_5pycda___pyx_unpickle_ProcGW__set_state(struct __pyx_obj_5pycx4_5pycda_ProcGW *, PyObject *); /*proto*/
#define __Pyx_MODULE_NAME "pycx4.pycda"
extern int __pyx_module_is_main_pycx4__pycda;
int __pyx_module_is_main_pycx4__pycda = 0;

/* Implementation of 'pycx4.pycda' */
static PyObject *__pyx_builtin_range;
static PyObject *__pyx_builtin_MemoryError;
static PyObject *__pyx_builtin_TypeError;
static PyObject *__pyx_builtin_object;
static PyObject *__pyx_builtin_super;
static PyObject *__pyx_builtin_ValueError;
static PyObject *__pyx_builtin_RuntimeError;
static PyObject *__pyx_builtin_ImportError;
static const char __pyx_k_C[] = "C";
static const char __pyx_k_d[] = "d";
static const char __pyx_k_cx[] = "cx::";
static const char __pyx_k_np[] = "np";
static const char __pyx_k__10[] = ">";
static const char __pyx_k__11[] = ", ";
static const char __pyx_k__19[] = ".";
static const char __pyx_k__20[] = "@";
static const char __pyx_k__32[] = "";
static const char __pyx_k_all[] = "__all__";
static const char __pyx_k_get[] = "get";
static const char __pyx_k_int[] = "int";
static const char __pyx_k_new[] = "__new__";
static const char __pyx_k_shy[] = "shy";
static const char __pyx_k_str[] = "str";
static const char __pyx_k_val[] = "val";
static const char __pyx_k_Chan[] = "Chan";
static const char __pyx_k_None[] = "None";
static const char __pyx_k_dict[] = "__dict__";
static const char __pyx_k_emit[] = "emit";
static const char __pyx_k_file[] = "file";
static const char __pyx_k_init[] = "__init__";
static const char __pyx_k_int8[] = "int8";
static const char __pyx_k_lock[] = "lock";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_mask[] = "mask";
static const char __pyx_k_msec[] = "msec";
static const char __pyx_k_name[] = "name";
static const char __pyx_k_proc[] = "proc";
static const char __pyx_k_size[] = "size";
static const char __pyx_k_stop[] = "stop";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_text[] = "text";
static const char __pyx_k_uint[] = "uint";
static const char __pyx_k_Alarm[] = "Alarm!";
static const char __pyx_k_DChan[] = "DChan";
static const char __pyx_k_IChan[] = "IChan";
static const char __pyx_k_Timer[] = "Timer";
static const char __pyx_k_VChan[] = "VChan";
static const char __pyx_k_ascii[] = "ascii";
static const char __pyx_k_break[] = "break_";
static const char __pyx_k_debug[] = "debug";
static const char __pyx_k_dsize[] = "dsize";
static const char __pyx_k_dtype[] = "dtype";
static const char __pyx_k_float[] = "float";
static const char __pyx_k_found[] = "found";
static const char __pyx_k_int16[] = "int16";
static const char __pyx_k_int32[] = "int32";
static const char __pyx_k_int64[] = "int64";
static const char __pyx_k_numpy[] = "numpy";
static const char __pyx_k_order[] = "order";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_split[] = "split";
static const char __pyx_k_start[] = "start";
static const char __pyx_k_super[] = "super";
static const char __pyx_k_uint8[] = "uint8";
static const char __pyx_k_zeros[] = "zeros";
static const char __pyx_k_PassGW[] = "PassGW";
static const char __pyx_k_ProcGW[] = "ProcGW";
static const char __pyx_k_Signal[] = "Signal";
static const char __pyx_k_defpfx[] = "defpfx";
static const char __pyx_k_double[] = "double";
static const char __pyx_k_encode[] = "encode";
static const char __pyx_k_fileno[] = "fileno";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_name_2[] = ", name=";
static const char __pyx_k_name_3[] = "__name__";
static const char __pyx_k_object[] = "object";
static const char __pyx_k_pickle[] = "pickle";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_resize[] = "resize";
static const char __pyx_k_single[] = "single";
static const char __pyx_k_uctext[] = "uctext";
static const char __pyx_k_uint16[] = "uint16";
static const char __pyx_k_uint32[] = "uint32";
static const char __pyx_k_uint64[] = "uint64";
static const char __pyx_k_unlock[] = "unlock";
static const char __pyx_k_update[] = "update";
static const char __pyx_k_Context[] = "Context";
static const char __pyx_k_FdEvent[] = "FdEvent";
static const char __pyx_k_StrChan[] = "StrChan";
static const char __pyx_k_channum[] = ", channum=";
static const char __pyx_k_connect[] = "connect";
static const char __pyx_k_context[] = "context";
static const char __pyx_k_errcode[] = ", errcode=";
static const char __pyx_k_in_name[] = "in_name";
static const char __pyx_k_private[] = "private";
static const char __pyx_k_BaseChan[] = "BaseChan";
static const char __pyx_k_cfactory[] = "cfactory";
static const char __pyx_k_defpfx_2[] = ", defpfx=";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_interval[] = "interval";
static const char __pyx_k_out_name[] = "out_name";
static const char __pyx_k_pyx_type[] = "__pyx_type";
static const char __pyx_k_setValue[] = "setValue";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_CdaObject[] = "CdaObject";
static const char __pyx_k_TypeError[] = "TypeError";
static const char __pyx_k_find_only[] = "find_only";
static const char __pyx_k_get_range[] = "get_range";
static const char __pyx_k_in_c_proc[] = "in_c_proc";
static const char __pyx_k_main_loop[] = "main_loop";
static const char __pyx_k_not_found[] = "not found";
static const char __pyx_k_on_update[] = "on_update";
static const char __pyx_k_pyx_state[] = "__pyx_state";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_searching[] = "searching";
static const char __pyx_k_DTYPE_INT8[] = "DTYPE_INT8";
static const char __pyx_k_DTYPE_TEXT[] = "DTYPE_TEXT";
static const char __pyx_k_ValueError[] = "ValueError";
static const char __pyx_k_disconnect[] = "disconnect";
static const char __pyx_k_get_curval[] = "get_curval";
static const char __pyx_k_max_nelems[] = "max_nelems";
static const char __pyx_k_no_rd_conv[] = "no_rd_conv";
static const char __pyx_k_no_wr_wait[] = "no_wr_wait";
static const char __pyx_k_out_c_proc[] = "out_c_proc";
static const char __pyx_k_pyx_result[] = "__pyx_result";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_singleShot[] = "singleShot";
static const char __pyx_k_ChanFactory[] = "ChanFactory";
static const char __pyx_k_DTYPE_INT16[] = "DTYPE_INT16";
static const char __pyx_k_DTYPE_INT32[] = "DTYPE_INT32";
static const char __pyx_k_DTYPE_INT64[] = "DTYPE_INT64";
static const char __pyx_k_DTYPE_UINT8[] = "DTYPE_UINT8";
static const char __pyx_k_ImportError[] = "ImportError";
static const char __pyx_k_MemoryError[] = "MemoryError";
static const char __pyx_k_PickleError[] = "PickleError";
static const char __pyx_k_change_sign[] = "change_sign";
static const char __pyx_k_create_chan[] = "create_chan";
static const char __pyx_k_get_strings[] = "get_strings";
static const char __pyx_k_pycx4_pycda[] = "pycx4.pycda";
static const char __pyx_k_rflags_text[] = "rflags_text";
static const char __pyx_k_setInterval[] = "setInterval";
static const char __pyx_k_DTYPE_DOUBLE[] = "DTYPE_DOUBLE";
static const char __pyx_k_DTYPE_SINGLE[] = "DTYPE_SINGLE";
static const char __pyx_k_DTYPE_UCTEXT[] = "DTYPE_UCTEXT";
static const char __pyx_k_DTYPE_UINT16[] = "DTYPE_UINT16";
static const char __pyx_k_DTYPE_UINT32[] = "DTYPE_UINT32";
static const char __pyx_k_DTYPE_UINT64[] = "DTYPE_UINT64";
static const char __pyx_k_RuntimeError[] = "RuntimeError";
static const char __pyx_k_Wrong_device[] = "Wrong device";
static const char __pyx_k_cda_chan_ref[] = "<cda_chan: ref=";
static const char __pyx_k_cx_dtype_map[] = "cx_dtype_map";
static const char __pyx_k_is_available[] = "is_available";
static const char __pyx_k_pyx_checksum[] = "__pyx_checksum";
static const char __pyx_k_setTolerance[] = "setTolerance";
static const char __pyx_k_stringsource[] = "stringsource";
static const char __pyx_k_DTYPE_UNKNOWN[] = "DTYPE_UNKNOWN";
static const char __pyx_k_get_lockstate[] = "get_lockstate";
static const char __pyx_k_ignore_update[] = "ignore_update";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_valueMeasured[] = "valueMeasured";
static const char __pyx_k_CdaContext_cid[] = "<CdaContext: cid=";
static const char __pyx_k_RSLVSTAT_FOUND[] = "RSLVSTAT_FOUND";
static const char __pyx_k_Value_is_weird[] = "Value is weird";
static const char __pyx_k_other_opp_flag[] = "other_opp_flag";
static const char __pyx_k_Defunct_channel[] = "Defunct channel";
static const char __pyx_k_pyx_PickleError[] = "__pyx_PickleError";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_Channel_not_found[] = "Channel not found";
static const char __pyx_k_Device_is_offline[] = "Device is offline";
static const char __pyx_k_Invalid_parameter[] = "Invalid parameter";
static const char __pyx_k_RSLVSTAT_NOTFOUND[] = "RSLVSTAT_NOTFOUND";
static const char __pyx_k_Value_in_red_zone[] = "Value in red zone";
static const char __pyx_k_RSLVSTAT_SEARCHING[] = "RSLVSTAT_SEARCHING";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_pyx_unpickle_Timer[] = "__pyx_unpickle_Timer";
static const char __pyx_k_I_O_timeout_expired[] = "I/O timeout expired";
static const char __pyx_k_pycx4_scheduler_pxi[] = "pycx4/scheduler.pxi";
static const char __pyx_k_pyx_unpickle_PassGW[] = "__pyx_unpickle_PassGW";
static const char __pyx_k_pyx_unpickle_ProcGW[] = "__pyx_unpickle_ProcGW";
static const char __pyx_k_Relaxing_after_alarm[] = "Relaxing after alarm";
static const char __pyx_k_Value_in_yellow_zone[] = "Value in yellow zone";
static const char __pyx_k_cda_chan_error_cname[] = "cda chan error: cname=";
static const char __pyx_k_pyx_unpickle_FdEvent[] = "__pyx_unpickle_FdEvent";
static const char __pyx_k_Configuration_problem[] = "Configuration problem";
static const char __pyx_k_cda_context_error_cid[] = "cda context error: cid=";
static const char __pyx_k_Driver_loading_problem[] = "Driver loading problem";
static const char __pyx_k_Input_channel_overload[] = "Input channel overload";
static const char __pyx_k_No_Q_from_CAMAC_device[] = "No Q from CAMAC device";
static const char __pyx_k_No_X_from_CAMAC_device[] = "No X from CAMAC device";
static const char __pyx_k_A_function_was_expected[] = "A function was expected";
static const char __pyx_k_Driver_internal_problem[] = "Driver internal problem";
static const char __pyx_k_select_interrupt_action[] = "select_interrupt_action";
static const char __pyx_k_Other_operator_is_active[] = "Other operator is active";
static const char __pyx_k_pyx_unpickle_ChanFactory[] = "__pyx_unpickle_ChanFactory";
static const char __pyx_k_Formula_calculation_error[] = "Formula calculation error";
static const char __pyx_k_Remote_controller_problem[] = "Remote controller problem";
static const char __pyx_k_Unsupported_feature_channel[] = "Unsupported feature/channel";
static const char __pyx_k_ndarray_is_not_C_contiguous[] = "ndarray is not C contiguous";
static const char __pyx_k_numpy_core_multiarray_failed_to[] = "numpy.core.multiarray failed to import";
static const char __pyx_k_self_events_cannot_be_converted[] = "self.events cannot be converted to a Python object for pickling";
static const char __pyx_k_unknown_dtype_code_in_numpy_pxd[] = "unknown dtype code in numpy.pxd (%d)";
static const char __pyx_k_value_size_greater_than_channel[] = "value size greater than channel.max_nelems";
static const char __pyx_k_Channel_was_programmatically_cha[] = "Channel was programmatically changed";
static const char __pyx_k_Format_string_allocated_too_shor[] = "Format string allocated too short, see comment in numpy.pxd";
static const char __pyx_k_Incompatible_checksums_s_vs_0x18[] = "Incompatible checksums (%s vs 0x183067a = (fhd, file, mask, ready))";
static const char __pyx_k_Incompatible_checksums_s_vs_0x57[] = "Incompatible checksums (%s vs 0x57ffd47 = (single_chans, vector_chans))";
static const char __pyx_k_Incompatible_checksums_s_vs_0x5a[] = "Incompatible checksums (%s vs 0x5a33863 = (in_chan, last_val, out_chan))";
static const char __pyx_k_Incompatible_checksums_s_vs_0x77[] = "Incompatible checksums (%s vs 0x777971d = (in_chan, out_chan))";
static const char __pyx_k_Incompatible_checksums_s_vs_0xef[] = "Incompatible checksums (%s vs 0xef1b4d4 = (active, repeat, tid, timeout, usec))";
static const char __pyx_k_Non_native_byte_order_not_suppor[] = "Non-native byte order not supported";
static const char __pyx_k_ndarray_is_not_Fortran_contiguou[] = "ndarray is not Fortran contiguous";
static const char __pyx_k_no_default___reduce___due_to_non[] = "no default __reduce__ due to non-trivial __cinit__";
static const char __pyx_k_numpy_core_umath_failed_to_impor[] = "numpy.core.umath failed to import";
static const char __pyx_k_self_chans_self_events_cannot_be[] = "self.chans,self.events cannot be converted to a Python object for pickling";
static const char __pyx_k_Format_string_allocated_too_shor_2[] = "Format string allocated too short.";
static PyObject *__pyx_kp_u_A_function_was_expected;
static PyObject *__pyx_kp_u_Alarm;
static PyObject *__pyx_n_s_BaseChan;
static PyObject *__pyx_n_u_C;
static PyObject *__pyx_kp_u_CdaContext_cid;
static PyObject *__pyx_n_s_CdaObject;
static PyObject *__pyx_n_s_Chan;
static PyObject *__pyx_n_u_Chan;
static PyObject *__pyx_n_s_ChanFactory;
static PyObject *__pyx_n_u_ChanFactory;
static PyObject *__pyx_kp_u_Channel_not_found;
static PyObject *__pyx_kp_u_Channel_was_programmatically_cha;
static PyObject *__pyx_kp_u_Configuration_problem;
static PyObject *__pyx_n_s_Context;
static PyObject *__pyx_n_u_Context;
static PyObject *__pyx_n_s_DChan;
static PyObject *__pyx_n_u_DChan;
static PyObject *__pyx_n_s_DTYPE_DOUBLE;
static PyObject *__pyx_n_u_DTYPE_DOUBLE;
static PyObject *__pyx_n_s_DTYPE_INT16;
static PyObject *__pyx_n_u_DTYPE_INT16;
static PyObject *__pyx_n_s_DTYPE_INT32;
static PyObject *__pyx_n_u_DTYPE_INT32;
static PyObject *__pyx_n_s_DTYPE_INT64;
static PyObject *__pyx_n_u_DTYPE_INT64;
static PyObject *__pyx_n_s_DTYPE_INT8;
static PyObject *__pyx_n_u_DTYPE_INT8;
static PyObject *__pyx_n_s_DTYPE_SINGLE;
static PyObject *__pyx_n_u_DTYPE_SINGLE;
static PyObject *__pyx_n_s_DTYPE_TEXT;
static PyObject *__pyx_n_u_DTYPE_TEXT;
static PyObject *__pyx_n_s_DTYPE_UCTEXT;
static PyObject *__pyx_n_u_DTYPE_UCTEXT;
static PyObject *__pyx_n_s_DTYPE_UINT16;
static PyObject *__pyx_n_u_DTYPE_UINT16;
static PyObject *__pyx_n_s_DTYPE_UINT32;
static PyObject *__pyx_n_u_DTYPE_UINT32;
static PyObject *__pyx_n_s_DTYPE_UINT64;
static PyObject *__pyx_n_u_DTYPE_UINT64;
static PyObject *__pyx_n_s_DTYPE_UINT8;
static PyObject *__pyx_n_u_DTYPE_UINT8;
static PyObject *__pyx_n_s_DTYPE_UNKNOWN;
static PyObject *__pyx_n_u_DTYPE_UNKNOWN;
static PyObject *__pyx_kp_u_Defunct_channel;
static PyObject *__pyx_kp_u_Device_is_offline;
static PyObject *__pyx_kp_u_Driver_internal_problem;
static PyObject *__pyx_kp_u_Driver_loading_problem;
static PyObject *__pyx_n_s_FdEvent;
static PyObject *__pyx_n_u_FdEvent;
static PyObject *__pyx_kp_u_Format_string_allocated_too_shor;
static PyObject *__pyx_kp_u_Format_string_allocated_too_shor_2;
static PyObject *__pyx_kp_u_Formula_calculation_error;
static PyObject *__pyx_n_s_IChan;
static PyObject *__pyx_n_u_IChan;
static PyObject *__pyx_kp_u_I_O_timeout_expired;
static PyObject *__pyx_n_s_ImportError;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x18;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x57;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x5a;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x77;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0xef;
static PyObject *__pyx_kp_u_Input_channel_overload;
static PyObject *__pyx_kp_u_Invalid_parameter;
static PyObject *__pyx_n_s_MemoryError;
static PyObject *__pyx_kp_u_No_Q_from_CAMAC_device;
static PyObject *__pyx_kp_u_No_X_from_CAMAC_device;
static PyObject *__pyx_kp_u_Non_native_byte_order_not_suppor;
static PyObject *__pyx_kp_u_None;
static PyObject *__pyx_kp_u_Other_operator_is_active;
static PyObject *__pyx_n_s_PassGW;
static PyObject *__pyx_n_u_PassGW;
static PyObject *__pyx_n_s_PickleError;
static PyObject *__pyx_n_s_ProcGW;
static PyObject *__pyx_n_s_RSLVSTAT_FOUND;
static PyObject *__pyx_n_u_RSLVSTAT_FOUND;
static PyObject *__pyx_n_s_RSLVSTAT_NOTFOUND;
static PyObject *__pyx_n_u_RSLVSTAT_NOTFOUND;
static PyObject *__pyx_n_s_RSLVSTAT_SEARCHING;
static PyObject *__pyx_n_u_RSLVSTAT_SEARCHING;
static PyObject *__pyx_kp_u_Relaxing_after_alarm;
static PyObject *__pyx_kp_u_Remote_controller_problem;
static PyObject *__pyx_n_s_RuntimeError;
static PyObject *__pyx_n_s_Signal;
static PyObject *__pyx_n_u_Signal;
static PyObject *__pyx_n_s_StrChan;
static PyObject *__pyx_n_u_StrChan;
static PyObject *__pyx_n_s_Timer;
static PyObject *__pyx_n_u_Timer;
static PyObject *__pyx_n_s_TypeError;
static PyObject *__pyx_kp_u_Unsupported_feature_channel;
static PyObject *__pyx_n_s_VChan;
static PyObject *__pyx_n_u_VChan;
static PyObject *__pyx_n_s_ValueError;
static PyObject *__pyx_kp_u_Value_in_red_zone;
static PyObject *__pyx_kp_u_Value_in_yellow_zone;
static PyObject *__pyx_kp_u_Value_is_weird;
static PyObject *__pyx_kp_u_Wrong_device;
static PyObject *__pyx_kp_u__10;
static PyObject *__pyx_kp_u__11;
static PyObject *__pyx_kp_u__19;
static PyObject *__pyx_kp_u__20;
static PyObject *__pyx_kp_u__32;
static PyObject *__pyx_n_s_all;
static PyObject *__pyx_n_u_ascii;
static PyObject *__pyx_n_u_break;
static PyObject *__pyx_kp_u_cda_chan_error_cname;
static PyObject *__pyx_kp_u_cda_chan_ref;
static PyObject *__pyx_kp_u_cda_context_error_cid;
static PyObject *__pyx_n_s_cfactory;
static PyObject *__pyx_n_u_cfactory;
static PyObject *__pyx_n_u_change_sign;
static PyObject *__pyx_kp_u_channum;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_connect;
static PyObject *__pyx_n_u_context;
static PyObject *__pyx_n_s_create_chan;
static PyObject *__pyx_kp_u_cx;
static PyObject *__pyx_n_s_cx_dtype_map;
static PyObject *__pyx_n_u_d;
static PyObject *__pyx_n_u_debug;
static PyObject *__pyx_n_s_defpfx;
static PyObject *__pyx_kp_u_defpfx_2;
static PyObject *__pyx_n_s_dict;
static PyObject *__pyx_n_s_disconnect;
static PyObject *__pyx_n_s_double;
static PyObject *__pyx_n_u_double;
static PyObject *__pyx_n_s_dsize;
static PyObject *__pyx_n_s_dtype;
static PyObject *__pyx_n_u_dtype;
static PyObject *__pyx_n_s_emit;
static PyObject *__pyx_n_s_encode;
static PyObject *__pyx_kp_u_errcode;
static PyObject *__pyx_n_s_file;
static PyObject *__pyx_n_s_fileno;
static PyObject *__pyx_n_u_find_only;
static PyObject *__pyx_n_u_float;
static PyObject *__pyx_n_u_found;
static PyObject *__pyx_n_s_get;
static PyObject *__pyx_n_u_get_curval;
static PyObject *__pyx_n_s_get_lockstate;
static PyObject *__pyx_n_s_get_range;
static PyObject *__pyx_n_s_get_strings;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_n_u_ignore_update;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_in_c_proc;
static PyObject *__pyx_n_s_in_name;
static PyObject *__pyx_n_s_init;
static PyObject *__pyx_n_u_int;
static PyObject *__pyx_n_s_int16;
static PyObject *__pyx_n_u_int16;
static PyObject *__pyx_n_s_int32;
static PyObject *__pyx_n_u_int32;
static PyObject *__pyx_n_s_int64;
static PyObject *__pyx_n_u_int64;
static PyObject *__pyx_n_s_int8;
static PyObject *__pyx_n_u_int8;
static PyObject *__pyx_n_s_interval;
static PyObject *__pyx_n_s_is_available;
static PyObject *__pyx_n_s_lock;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_u_main_loop;
static PyObject *__pyx_n_s_mask;
static PyObject *__pyx_n_u_max_nelems;
static PyObject *__pyx_n_s_msec;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_kp_u_name_2;
static PyObject *__pyx_n_s_name_3;
static PyObject *__pyx_kp_u_ndarray_is_not_C_contiguous;
static PyObject *__pyx_kp_u_ndarray_is_not_Fortran_contiguou;
static PyObject *__pyx_n_s_new;
static PyObject *__pyx_kp_s_no_default___reduce___due_to_non;
static PyObject *__pyx_n_u_no_rd_conv;
static PyObject *__pyx_n_u_no_wr_wait;
static PyObject *__pyx_kp_u_not_found;
static PyObject *__pyx_n_s_np;
static PyObject *__pyx_n_s_numpy;
static PyObject *__pyx_kp_u_numpy_core_multiarray_failed_to;
static PyObject *__pyx_kp_u_numpy_core_umath_failed_to_impor;
static PyObject *__pyx_n_s_object;
static PyObject *__pyx_n_u_on_update;
static PyObject *__pyx_n_s_order;
static PyObject *__pyx_n_u_other_opp_flag;
static PyObject *__pyx_n_s_out_c_proc;
static PyObject *__pyx_n_s_out_name;
static PyObject *__pyx_n_s_pickle;
static PyObject *__pyx_n_u_private;
static PyObject *__pyx_n_s_proc;
static PyObject *__pyx_n_s_pycx4_pycda;
static PyObject *__pyx_kp_s_pycx4_scheduler_pxi;
static PyObject *__pyx_n_s_pyx_PickleError;
static PyObject *__pyx_n_s_pyx_checksum;
static PyObject *__pyx_n_s_pyx_result;
static PyObject *__pyx_n_s_pyx_state;
static PyObject *__pyx_n_s_pyx_type;
static PyObject *__pyx_n_s_pyx_unpickle_ChanFactory;
static PyObject *__pyx_n_s_pyx_unpickle_FdEvent;
static PyObject *__pyx_n_s_pyx_unpickle_PassGW;
static PyObject *__pyx_n_s_pyx_unpickle_ProcGW;
static PyObject *__pyx_n_s_pyx_unpickle_Timer;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_n_s_resize;
static PyObject *__pyx_n_s_rflags_text;
static PyObject *__pyx_n_u_searching;
static PyObject *__pyx_n_s_select_interrupt_action;
static PyObject *__pyx_kp_s_self_chans_self_events_cannot_be;
static PyObject *__pyx_kp_s_self_events_cannot_be_converted;
static PyObject *__pyx_n_s_setInterval;
static PyObject *__pyx_n_s_setTolerance;
static PyObject *__pyx_n_s_setValue;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_n_u_shy;
static PyObject *__pyx_n_s_single;
static PyObject *__pyx_n_u_single;
static PyObject *__pyx_n_s_singleShot;
static PyObject *__pyx_n_s_size;
static PyObject *__pyx_n_s_split;
static PyObject *__pyx_n_s_start;
static PyObject *__pyx_n_s_stop;
static PyObject *__pyx_n_u_str;
static PyObject *__pyx_kp_s_stringsource;
static PyObject *__pyx_n_s_super;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_u_text;
static PyObject *__pyx_n_u_uctext;
static PyObject *__pyx_n_u_uint;
static PyObject *__pyx_n_s_uint16;
static PyObject *__pyx_n_u_uint16;
static PyObject *__pyx_n_s_uint32;
static PyObject *__pyx_n_u_uint32;
static PyObject *__pyx_n_s_uint64;
static PyObject *__pyx_n_u_uint64;
static PyObject *__pyx_n_s_uint8;
static PyObject *__pyx_n_u_uint8;
static PyObject *__pyx_kp_u_unknown_dtype_code_in_numpy_pxd;
static PyObject *__pyx_n_s_unlock;
static PyObject *__pyx_n_s_update;
static PyObject *__pyx_n_s_val;
static PyObject *__pyx_n_s_valueMeasured;
static PyObject *__pyx_kp_u_value_size_greater_than_channel;
static PyObject *__pyx_n_s_zeros;
static PyObject *__pyx_pf_5pycx4_5pycda_main_loop(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_2break_(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_4select_interrupt_action(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static int __pyx_pf_5pycx4_5pycda_6Signal___cinit__(struct __pyx_obj_5pycx4_5pycda_Signal *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_args); /* proto */
static void __pyx_pf_5pycx4_5pycda_6Signal_2__dealloc__(struct __pyx_obj_5pycx4_5pycda_Signal *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_6Signal_4connect(struct __pyx_obj_5pycx4_5pycda_Signal *__pyx_v_self, PyObject *__pyx_v_slot); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_6Signal_6disconnect(struct __pyx_obj_5pycx4_5pycda_Signal *__pyx_v_self, PyObject *__pyx_v_callback); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_6Signal_8emit(struct __pyx_obj_5pycx4_5pycda_Signal *__pyx_v_self, PyObject *__pyx_v_args); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_6Signal_10__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_Signal *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_6Signal_12__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_Signal *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_5pycx4_5pycda_5Timer___init__(struct __pyx_obj_5pycx4_5pycda_Timer *__pyx_v_self, int __pyx_v_msec); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_5Timer_2stop(struct __pyx_obj_5pycx4_5pycda_Timer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_5Timer_4start(struct __pyx_obj_5pycx4_5pycda_Timer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_5Timer_6singleShot(struct __pyx_obj_5pycx4_5pycda_Timer *__pyx_v_self, int __pyx_v_msec, PyObject *__pyx_v_proc); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_5Timer_8interval(struct __pyx_obj_5pycx4_5pycda_Timer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_5Timer_10setInterval(struct __pyx_obj_5pycx4_5pycda_Timer *__pyx_v_self, int __pyx_v_msec); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_5Timer_4usec___get__(struct __pyx_obj_5pycx4_5pycda_Timer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_5Timer_6repeat___get__(struct __pyx_obj_5pycx4_5pycda_Timer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_5Timer_6active___get__(struct __pyx_obj_5pycx4_5pycda_Timer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_5Timer_3tid___get__(struct __pyx_obj_5pycx4_5pycda_Timer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_5Timer_7timeout___get__(struct __pyx_obj_5pycx4_5pycda_Timer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_5Timer_12__reduce_cython__(struct __pyx_obj_5pycx4_5pycda_Timer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_5Timer_14__setstate_cython__(struct __pyx_obj_5pycx4_5pycda_Timer *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_5pycx4_5pycda_7FdEvent___init__(struct __pyx_obj_5pycx4_5pycda_FdEvent *__pyx_v_self, PyObject *__pyx_v_file, PyObject *__pyx_v_mask); /* proto */
static void __pyx_pf_5pycx4_5pycda_7FdEvent_2__dealloc__(struct __pyx_obj_5pycx4_5pycda_FdEvent *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_7FdEvent_4file___get__(struct __pyx_obj_5pycx4_5pycda_FdEvent *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_7FdEvent_5ready___get__(struct __pyx_obj_5pycx4_5pycda_FdEvent *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_7FdEvent_4mask___get__(struct __pyx_obj_5pycx4_5pycda_FdEvent *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_7FdEvent_3fhd___get__(struct __pyx_obj_5pycx4_5pycda_FdEvent *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_7FdEvent_4__reduce_cython__(struct __pyx_obj_5pycx4_5pycda_FdEvent *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_7FdEvent_6__setstate_cython__(struct __pyx_obj_5pycx4_5pycda_FdEvent *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_5pycx4_5pycda_9CdaObject___init__(struct __pyx_obj_5pycx4_5pycda_CdaObject *__pyx_v_self); /* proto */
static void __pyx_pf_5pycx4_5pycda_9CdaObject_2__dealloc__(struct __pyx_obj_5pycx4_5pycda_CdaObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_9CdaObject_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_CdaObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_9CdaObject_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_CdaObject *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_5pycx4_5pycda_7Context___init__(struct __pyx_obj_5pycx4_5pycda_Context *__pyx_v_self, PyObject *__pyx_v_defpfx, PyObject *__pyx_v_kwargs); /* proto */
static void __pyx_pf_5pycx4_5pycda_7Context_2__dealloc__(struct __pyx_obj_5pycx4_5pycda_Context *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_7Context_4__str__(struct __pyx_obj_5pycx4_5pycda_Context *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_7Context_6enable_serverCycle(struct __pyx_obj_5pycx4_5pycda_Context *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_7Context_3cid___get__(struct __pyx_obj_5pycx4_5pycda_Context *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_7Context_6defpfx___get__(struct __pyx_obj_5pycx4_5pycda_Context *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_7Context_11serverCycle___get__(struct __pyx_obj_5pycx4_5pycda_Context *__pyx_v_self); /* proto */
static int __pyx_pf_5pycx4_5pycda_7Context_11serverCycle_2__set__(struct __pyx_obj_5pycx4_5pycda_Context *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5pycx4_5pycda_7Context_11serverCycle_4__del__(struct __pyx_obj_5pycx4_5pycda_Context *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_7Context_8__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_Context *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_7Context_10__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_Context *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_5pycx4_5pycda_8BaseChan___init__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_kwargs); /* proto */
static void __pyx_pf_5pycx4_5pycda_8BaseChan_2__dealloc__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_4__str__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_6short_name(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_8is_available(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_10rflags_text(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_12get_range(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_14get_strings(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_16get_lockstate(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_18lock(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_20unlock(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_3ref___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_4name___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_10max_nelems___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_5dtype___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_8itemsize___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_4time___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_9prev_time___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_9rslv_stat___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_8rslv_str___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_5quant___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_10registered___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_11first_cycle___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_10lock_state___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_7context___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_6rflags___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_3rng___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_5ident___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_5label___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_3tip___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_7comment___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_7geoinfo___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_6rsrvd6___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_5units___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_6dpyfmt___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_13valueMeasured___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static int __pyx_pf_5pycx4_5pycda_8BaseChan_13valueMeasured_2__set__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5pycx4_5pycda_8BaseChan_13valueMeasured_4__del__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_12valueChanged___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static int __pyx_pf_5pycx4_5pycda_8BaseChan_12valueChanged_2__set__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5pycx4_5pycda_8BaseChan_12valueChanged_4__del__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_7resolve___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static int __pyx_pf_5pycx4_5pycda_8BaseChan_7resolve_2__set__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5pycx4_5pycda_8BaseChan_7resolve_4__del__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_13stringsUpdate___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static int __pyx_pf_5pycx4_5pycda_8BaseChan_13stringsUpdate_2__set__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5pycx4_5pycda_8BaseChan_13stringsUpdate_4__del__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_15lockStateUpdate___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static int __pyx_pf_5pycx4_5pycda_8BaseChan_15lockStateUpdate_2__set__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5pycx4_5pycda_8BaseChan_15lockStateUpdate_4__del__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_22__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_24__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_5pycx4_5pycda_5DChan___init__(struct __pyx_obj_5pycx4_5pycda_DChan *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_5DChan_2setValue(struct __pyx_obj_5pycx4_5pycda_DChan *__pyx_v_self, double __pyx_v_value); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_5DChan_4setTolerance(struct __pyx_obj_5pycx4_5pycda_DChan *__pyx_v_self, double __pyx_v_new_tolerance); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_5DChan_3val___get__(struct __pyx_obj_5pycx4_5pycda_DChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_5DChan_8prev_val___get__(struct __pyx_obj_5pycx4_5pycda_DChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_5DChan_9tolerance___get__(struct __pyx_obj_5pycx4_5pycda_DChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_5DChan_6__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_DChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_5DChan_8__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_DChan *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_5pycx4_5pycda_5IChan___init__(struct __pyx_obj_5pycx4_5pycda_IChan *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_5IChan_2setValue(struct __pyx_obj_5pycx4_5pycda_IChan *__pyx_v_self, int __pyx_v_value); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_5IChan_4setTolerance(struct __pyx_obj_5pycx4_5pycda_IChan *__pyx_v_self, int __pyx_v_new_tolerance); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_5IChan_3val___get__(struct __pyx_obj_5pycx4_5pycda_IChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_5IChan_8prev_val___get__(struct __pyx_obj_5pycx4_5pycda_IChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_5IChan_9tolerance___get__(struct __pyx_obj_5pycx4_5pycda_IChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_5IChan_6__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_IChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_5IChan_8__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_IChan *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_5pycx4_5pycda_4Chan___init__(struct __pyx_obj_5pycx4_5pycda_Chan *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_4Chan_2setValue(struct __pyx_obj_5pycx4_5pycda_Chan *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_4Chan_3val___get__(struct __pyx_obj_5pycx4_5pycda_Chan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_4Chan_8prev_val___get__(struct __pyx_obj_5pycx4_5pycda_Chan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_4Chan_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_Chan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_4Chan_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_Chan *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_5pycx4_5pycda_5VChan___init__(struct __pyx_obj_5pycx4_5pycda_VChan *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_5VChan_2setValue(struct __pyx_obj_5pycx4_5pycda_VChan *__pyx_v_self, PyArrayObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_5VChan_3val___get__(struct __pyx_obj_5pycx4_5pycda_VChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_5VChan_7npdtype___get__(struct __pyx_obj_5pycx4_5pycda_VChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_5VChan_6nelems___get__(struct __pyx_obj_5pycx4_5pycda_VChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_5VChan_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_VChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_5VChan_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_VChan *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_5pycx4_5pycda_7StrChan___init__(struct __pyx_obj_5pycx4_5pycda_StrChan *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_kwargs); /* proto */
static void __pyx_pf_5pycx4_5pycda_7StrChan_2__dealloc__(struct __pyx_obj_5pycx4_5pycda_StrChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_7StrChan_4setValue(struct __pyx_obj_5pycx4_5pycda_StrChan *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_7StrChan_3val___get__(struct __pyx_obj_5pycx4_5pycda_StrChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_7StrChan_8prev_val___get__(struct __pyx_obj_5pycx4_5pycda_StrChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_7StrChan_6__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_StrChan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_7StrChan_8__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_StrChan *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_5pycx4_5pycda_11ChanFactory___init__(struct __pyx_obj_5pycx4_5pycda_ChanFactory *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_11ChanFactory_2create_chan(struct __pyx_obj_5pycx4_5pycda_ChanFactory *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_dtype, PyObject *__pyx_v_dsize, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_11ChanFactory_4__reduce_cython__(struct __pyx_obj_5pycx4_5pycda_ChanFactory *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_11ChanFactory_6__setstate_cython__(struct __pyx_obj_5pycx4_5pycda_ChanFactory *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_5pycx4_5pycda_6PassGW___init__(struct __pyx_obj_5pycx4_5pycda_PassGW *__pyx_v_self, PyObject *__pyx_v_in_name, PyObject *__pyx_v_out_name, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_6PassGW_2in_c_proc(struct __pyx_obj_5pycx4_5pycda_PassGW *__pyx_v_self, PyObject *__pyx_v_chan); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_6PassGW_4out_c_proc(struct __pyx_obj_5pycx4_5pycda_PassGW *__pyx_v_self, PyObject *__pyx_v_chan); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_6PassGW_6__reduce_cython__(struct __pyx_obj_5pycx4_5pycda_PassGW *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_6PassGW_8__setstate_cython__(struct __pyx_obj_5pycx4_5pycda_PassGW *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_5pycx4_5pycda_6ProcGW___init__(struct __pyx_obj_5pycx4_5pycda_ProcGW *__pyx_v_self, PyObject *__pyx_v_in_name, PyObject *__pyx_v_out_name, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_6ProcGW_2in_c_proc(struct __pyx_obj_5pycx4_5pycda_ProcGW *__pyx_v_self, PyObject *__pyx_v_chan); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_6ProcGW_4out_c_proc(struct __pyx_obj_5pycx4_5pycda_ProcGW *__pyx_v_self, PyObject *__pyx_v_chan); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_6ProcGW_6in2out(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_ProcGW *__pyx_v_in_val); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_6ProcGW_8__reduce_cython__(struct __pyx_obj_5pycx4_5pycda_ProcGW *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_6ProcGW_10__setstate_cython__(struct __pyx_obj_5pycx4_5pycda_ProcGW *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_6__pyx_unpickle_Timer(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_8__pyx_unpickle_FdEvent(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_10__pyx_unpickle_ChanFactory(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_12__pyx_unpickle_PassGW(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_14__pyx_unpickle_ProcGW(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_5numpy_7ndarray___getbuffer__(PyArrayObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /* proto */
static void __pyx_pf_5numpy_7ndarray_2__releasebuffer__(PyArrayObject *__pyx_v_self, Py_buffer *__pyx_v_info); /* proto */
static PyObject *__pyx_tp_new_5pycx4_5pycda_Signal(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5pycx4_5pycda_Timer(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5pycx4_5pycda_FdEvent(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5pycx4_5pycda_CdaObject(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5pycx4_5pycda_Context(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5pycx4_5pycda_BaseChan(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5pycx4_5pycda_DChan(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5pycx4_5pycda_IChan(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5pycx4_5pycda_Chan(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5pycx4_5pycda_VChan(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5pycx4_5pycda_StrChan(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5pycx4_5pycda_ChanFactory(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5pycx4_5pycda_PassGW(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5pycx4_5pycda_ProcGW(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_get = {0, &__pyx_n_s_get, 0, 0, 0};
static PyObject *__pyx_int_1;
static PyObject *__pyx_int_25364090;
static PyObject *__pyx_int_92273991;
static PyObject *__pyx_int_94582883;
static PyObject *__pyx_int_125277981;
static PyObject *__pyx_int_250721492;
static PyObject *__pyx_int_neg_1;
static PyObject *__pyx_k__5;
static PyObject *__pyx_tuple_;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__7;
static PyObject *__pyx_tuple__8;
static PyObject *__pyx_tuple__9;
static PyObject *__pyx_tuple__12;
static PyObject *__pyx_tuple__13;
static PyObject *__pyx_tuple__14;
static PyObject *__pyx_tuple__15;
static PyObject *__pyx_tuple__16;
static PyObject *__pyx_tuple__17;
static PyObject *__pyx_tuple__18;
static PyObject *__pyx_tuple__21;
static PyObject *__pyx_tuple__22;
static PyObject *__pyx_tuple__23;
static PyObject *__pyx_tuple__24;
static PyObject *__pyx_tuple__25;
static PyObject *__pyx_tuple__26;
static PyObject *__pyx_tuple__27;
static PyObject *__pyx_tuple__28;
static PyObject *__pyx_tuple__29;
static PyObject *__pyx_tuple__30;
static PyObject *__pyx_tuple__31;
static PyObject *__pyx_tuple__33;
static PyObject *__pyx_tuple__34;
static PyObject *__pyx_tuple__35;
static PyObject *__pyx_tuple__36;
static PyObject *__pyx_tuple__37;
static PyObject *__pyx_tuple__38;
static PyObject *__pyx_tuple__39;
static PyObject *__pyx_tuple__40;
static PyObject *__pyx_tuple__41;
static PyObject *__pyx_tuple__43;
static PyObject *__pyx_tuple__45;
static PyObject *__pyx_tuple__47;
static PyObject *__pyx_tuple__49;
static PyObject *__pyx_tuple__51;
static PyObject *__pyx_codeobj__42;
static PyObject *__pyx_codeobj__44;
static PyObject *__pyx_codeobj__46;
static PyObject *__pyx_codeobj__48;
static PyObject *__pyx_codeobj__50;
static PyObject *__pyx_codeobj__52;
/* Late includes */

/* "pycx4/scheduler.pxi":3
 * from cx4.cxscheduler cimport *
 * 
 * cpdef main_loop():             # <<<<<<<<<<<<<<
 *     sl_main_loop()
 * 
 */

static PyObject *__pyx_pw_5pycx4_5pycda_1main_loop(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_5pycx4_5pycda_main_loop(CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("main_loop", 0);

  /* "pycx4/scheduler.pxi":4
 * 
 * cpdef main_loop():
 *     sl_main_loop()             # <<<<<<<<<<<<<<
 * 
 * cpdef break_():
 */
  (void)(sl_main_loop());

  /* "pycx4/scheduler.pxi":3
 * from cx4.cxscheduler cimport *
 * 
 * cpdef main_loop():             # <<<<<<<<<<<<<<
 *     sl_main_loop()
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_1main_loop(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_1main_loop(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("main_loop (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_main_loop(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_main_loop(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("main_loop", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_5pycx4_5pycda_main_loop(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.main_loop", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/scheduler.pxi":6
 *     sl_main_loop()
 * 
 * cpdef break_():             # <<<<<<<<<<<<<<
 *     sl_break()
 * 
 */

static PyObject *__pyx_pw_5pycx4_5pycda_3break_(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_5pycx4_5pycda_break_(CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("break_", 0);

  /* "pycx4/scheduler.pxi":7
 * 
 * cpdef break_():
 *     sl_break()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  (void)(sl_break());

  /* "pycx4/scheduler.pxi":6
 *     sl_main_loop()
 * 
 * cpdef break_():             # <<<<<<<<<<<<<<
 *     sl_break()
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_3break_(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_3break_(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("break_ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_2break_(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_2break_(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("break_", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_5pycx4_5pycda_break_(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.break_", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/scheduler.pxi":11
 * 
 * #  required to react to signals in case of daemon
 * def select_interrupt_action():             # <<<<<<<<<<<<<<
 *     pass
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_5select_interrupt_action(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_5pycx4_5pycda_5select_interrupt_action = {"select_interrupt_action", (PyCFunction)__pyx_pw_5pycx4_5pycda_5select_interrupt_action, METH_NOARGS, 0};
static PyObject *__pyx_pw_5pycx4_5pycda_5select_interrupt_action(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("select_interrupt_action (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_4select_interrupt_action(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_4select_interrupt_action(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("select_interrupt_action", 0);

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/scheduler.pxi":14
 *     pass
 * 
 * cdef void at_select() with gil:             # <<<<<<<<<<<<<<
 *     select_interrupt_action()
 * 
 */

static void __pyx_f_5pycx4_5pycda_at_select(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("at_select", 0);

  /* "pycx4/scheduler.pxi":15
 * 
 * cdef void at_select() with gil:
 *     select_interrupt_action()             # <<<<<<<<<<<<<<
 * 
 * sl_set_select_behaviour(<sl_at_select_proc>NULL, <sl_at_select_proc>at_select, 0)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_select_interrupt_action); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/scheduler.pxi":14
 *     pass
 * 
 * cdef void at_select() with gil:             # <<<<<<<<<<<<<<
 *     select_interrupt_action()
 * 
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_WriteUnraisable("pycx4.pycda.at_select", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
}

/* "pycx4/signal.pxi":10
 * 
 * # let's rely on default definitions, ha-ha. if any problems - will change it
 *     def __cinit__(self, *args):             # <<<<<<<<<<<<<<
 *         self.callbacks = NULL
 *         self.cnum = 0
 */

/* Python wrapper */
static int __pyx_pw_5pycx4_5pycda_6Signal_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5pycx4_5pycda_6Signal_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  CYTHON_UNUSED PyObject *__pyx_v_args = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__cinit__", 0))) return -1;
  __Pyx_INCREF(__pyx_args);
  __pyx_v_args = __pyx_args;
  __pyx_r = __pyx_pf_5pycx4_5pycda_6Signal___cinit__(((struct __pyx_obj_5pycx4_5pycda_Signal *)__pyx_v_self), __pyx_v_args);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_args);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pycx4_5pycda_6Signal___cinit__(struct __pyx_obj_5pycx4_5pycda_Signal *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_args) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "pycx4/signal.pxi":11
 * # let's rely on default definitions, ha-ha. if any problems - will change it
 *     def __cinit__(self, *args):
 *         self.callbacks = NULL             # <<<<<<<<<<<<<<
 *         self.cnum = 0
 * 
 */
  __pyx_v_self->callbacks = NULL;

  /* "pycx4/signal.pxi":12
 *     def __cinit__(self, *args):
 *         self.callbacks = NULL
 *         self.cnum = 0             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
 */
  __pyx_v_self->cnum = 0;

  /* "pycx4/signal.pxi":10
 * 
 * # let's rely on default definitions, ha-ha. if any problems - will change it
 *     def __cinit__(self, *args):             # <<<<<<<<<<<<<<
 *         self.callbacks = NULL
 *         self.cnum = 0
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/signal.pxi":14
 *         self.cnum = 0
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         cdef int ind
 *         for ind in range(self.cnum):
 */

/* Python wrapper */
static void __pyx_pw_5pycx4_5pycda_6Signal_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_5pycx4_5pycda_6Signal_3__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_5pycx4_5pycda_6Signal_2__dealloc__(((struct __pyx_obj_5pycx4_5pycda_Signal *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_5pycx4_5pycda_6Signal_2__dealloc__(struct __pyx_obj_5pycx4_5pycda_Signal *__pyx_v_self) {
  int __pyx_v_ind;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "pycx4/signal.pxi":16
 *     def __dealloc__(self):
 *         cdef int ind
 *         for ind in range(self.cnum):             # <<<<<<<<<<<<<<
 *             Py_DECREF(<object>(self.callbacks[ind]))
 *         free(self.callbacks)
 */
  __pyx_t_1 = __pyx_v_self->cnum;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_ind = __pyx_t_3;

    /* "pycx4/signal.pxi":17
 *         cdef int ind
 *         for ind in range(self.cnum):
 *             Py_DECREF(<object>(self.callbacks[ind]))             # <<<<<<<<<<<<<<
 *         free(self.callbacks)
 * 
 */
    __pyx_t_4 = ((PyObject *)(__pyx_v_self->callbacks[__pyx_v_ind]));
    __Pyx_INCREF(__pyx_t_4);
    Py_DECREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }

  /* "pycx4/signal.pxi":18
 *         for ind in range(self.cnum):
 *             Py_DECREF(<object>(self.callbacks[ind]))
 *         free(self.callbacks)             # <<<<<<<<<<<<<<
 * 
 *     cpdef connect(self, slot):
 */
  free(__pyx_v_self->callbacks);

  /* "pycx4/signal.pxi":14
 *         self.cnum = 0
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         cdef int ind
 *         for ind in range(self.cnum):
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/signal.pxi":20
 *         free(self.callbacks)
 * 
 *     cpdef connect(self, slot):             # <<<<<<<<<<<<<<
 *         cdef:
 *             void *tmp
 */

static PyObject *__pyx_pw_5pycx4_5pycda_6Signal_5connect(PyObject *__pyx_v_self, PyObject *__pyx_v_slot); /*proto*/
static PyObject *__pyx_f_5pycx4_5pycda_6Signal_connect(struct __pyx_obj_5pycx4_5pycda_Signal *__pyx_v_self, PyObject *__pyx_v_slot, int __pyx_skip_dispatch) {
  void *__pyx_v_tmp;
  int __pyx_v_ind;
  PyObject *__pyx_v_callback = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  __Pyx_RefNannySetupContext("connect", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    static PY_UINT64_T tp_dict_version = 0, obj_dict_version = 0;
    if (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict && tp_dict_version == __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) && (!Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset || obj_dict_version == __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))))));
    else {
      PY_UINT64_T type_dict_guard = (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict)) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_connect); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 20, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_5pycx4_5pycda_6Signal_5connect)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_slot) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_slot);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 20, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
      tp_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      obj_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset) ? __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))) : 0;
      if (unlikely(type_dict_guard != tp_dict_version)) {
        tp_dict_version = obj_dict_version = 0;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    }
    #endif
  }

  /* "pycx4/signal.pxi":24
 *             void *tmp
 *             int ind
 *         if isinstance(slot, Signal):             # <<<<<<<<<<<<<<
 *             callback = slot.emit
 *         elif callable(slot):
 */
  __pyx_t_5 = __Pyx_TypeCheck(__pyx_v_slot, __pyx_ptype_5pycx4_5pycda_Signal); 
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "pycx4/signal.pxi":25
 *             int ind
 *         if isinstance(slot, Signal):
 *             callback = slot.emit             # <<<<<<<<<<<<<<
 *         elif callable(slot):
 *             callback = slot
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_slot, __pyx_n_s_emit); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 25, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_callback = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "pycx4/signal.pxi":24
 *             void *tmp
 *             int ind
 *         if isinstance(slot, Signal):             # <<<<<<<<<<<<<<
 *             callback = slot.emit
 *         elif callable(slot):
 */
    goto __pyx_L3;
  }

  /* "pycx4/signal.pxi":26
 *         if isinstance(slot, Signal):
 *             callback = slot.emit
 *         elif callable(slot):             # <<<<<<<<<<<<<<
 *             callback = slot
 *         else:
 */
  __pyx_t_6 = __Pyx_PyCallable_Check(__pyx_v_slot); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(0, 26, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_6 != 0);
  if (likely(__pyx_t_5)) {

    /* "pycx4/signal.pxi":27
 *             callback = slot.emit
 *         elif callable(slot):
 *             callback = slot             # <<<<<<<<<<<<<<
 *         else:
 *             raise Exception('A function was expected')
 */
    __Pyx_INCREF(__pyx_v_slot);
    __pyx_v_callback = __pyx_v_slot;

    /* "pycx4/signal.pxi":26
 *         if isinstance(slot, Signal):
 *             callback = slot.emit
 *         elif callable(slot):             # <<<<<<<<<<<<<<
 *             callback = slot
 *         else:
 */
    goto __pyx_L3;
  }

  /* "pycx4/signal.pxi":29
 *             callback = slot
 *         else:
 *             raise Exception('A function was expected')             # <<<<<<<<<<<<<<
 *         for ind in range(self.cnum):
 *             if callback == <object>self.callbacks[ind]:
 */
  /*else*/ {
    __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])), __pyx_tuple_, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 29, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 29, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "pycx4/signal.pxi":30
 *         else:
 *             raise Exception('A function was expected')
 *         for ind in range(self.cnum):             # <<<<<<<<<<<<<<
 *             if callback == <object>self.callbacks[ind]:
 *                 return
 */
  __pyx_t_7 = __pyx_v_self->cnum;
  __pyx_t_8 = __pyx_t_7;
  for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
    __pyx_v_ind = __pyx_t_9;

    /* "pycx4/signal.pxi":31
 *             raise Exception('A function was expected')
 *         for ind in range(self.cnum):
 *             if callback == <object>self.callbacks[ind]:             # <<<<<<<<<<<<<<
 *                 return
 *         tmp = realloc(<void*>self.callbacks, sizeof(void*) * (self.cnum+1))
 */
    __pyx_t_1 = PyObject_RichCompare(__pyx_v_callback, ((PyObject *)(__pyx_v_self->callbacks[__pyx_v_ind])), Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 31, __pyx_L1_error)
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 31, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_5) {

      /* "pycx4/signal.pxi":32
 *         for ind in range(self.cnum):
 *             if callback == <object>self.callbacks[ind]:
 *                 return             # <<<<<<<<<<<<<<
 *         tmp = realloc(<void*>self.callbacks, sizeof(void*) * (self.cnum+1))
 *         if not tmp: raise MemoryError()
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_r = Py_None; __Pyx_INCREF(Py_None);
      goto __pyx_L0;

      /* "pycx4/signal.pxi":31
 *             raise Exception('A function was expected')
 *         for ind in range(self.cnum):
 *             if callback == <object>self.callbacks[ind]:             # <<<<<<<<<<<<<<
 *                 return
 *         tmp = realloc(<void*>self.callbacks, sizeof(void*) * (self.cnum+1))
 */
    }
  }

  /* "pycx4/signal.pxi":33
 *             if callback == <object>self.callbacks[ind]:
 *                 return
 *         tmp = realloc(<void*>self.callbacks, sizeof(void*) * (self.cnum+1))             # <<<<<<<<<<<<<<
 *         if not tmp: raise MemoryError()
 *         self.callbacks = <void**>tmp
 */
  __pyx_v_tmp = realloc(((void *)__pyx_v_self->callbacks), ((sizeof(void *)) * (__pyx_v_self->cnum + 1)));

  /* "pycx4/signal.pxi":34
 *                 return
 *         tmp = realloc(<void*>self.callbacks, sizeof(void*) * (self.cnum+1))
 *         if not tmp: raise MemoryError()             # <<<<<<<<<<<<<<
 *         self.callbacks = <void**>tmp
 *         self.callbacks[self.cnum] = <void*>callback
 */
  __pyx_t_5 = ((!(__pyx_v_tmp != 0)) != 0);
  if (unlikely(__pyx_t_5)) {
    PyErr_NoMemory(); __PYX_ERR(0, 34, __pyx_L1_error)
  }

  /* "pycx4/signal.pxi":35
 *         tmp = realloc(<void*>self.callbacks, sizeof(void*) * (self.cnum+1))
 *         if not tmp: raise MemoryError()
 *         self.callbacks = <void**>tmp             # <<<<<<<<<<<<<<
 *         self.callbacks[self.cnum] = <void*>callback
 *         self.cnum += 1
 */
  __pyx_v_self->callbacks = ((void **)__pyx_v_tmp);

  /* "pycx4/signal.pxi":36
 *         if not tmp: raise MemoryError()
 *         self.callbacks = <void**>tmp
 *         self.callbacks[self.cnum] = <void*>callback             # <<<<<<<<<<<<<<
 *         self.cnum += 1
 *         Py_INCREF(callback)
 */
  (__pyx_v_self->callbacks[__pyx_v_self->cnum]) = ((void *)__pyx_v_callback);

  /* "pycx4/signal.pxi":37
 *         self.callbacks = <void**>tmp
 *         self.callbacks[self.cnum] = <void*>callback
 *         self.cnum += 1             # <<<<<<<<<<<<<<
 *         Py_INCREF(callback)
 * 
 */
  __pyx_v_self->cnum = (__pyx_v_self->cnum + 1);

  /* "pycx4/signal.pxi":38
 *         self.callbacks[self.cnum] = <void*>callback
 *         self.cnum += 1
 *         Py_INCREF(callback)             # <<<<<<<<<<<<<<
 * 
 *     cpdef disconnect(self, callback):
 */
  Py_INCREF(__pyx_v_callback);

  /* "pycx4/signal.pxi":20
 *         free(self.callbacks)
 * 
 *     cpdef connect(self, slot):             # <<<<<<<<<<<<<<
 *         cdef:
 *             void *tmp
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pycx4.pycda.Signal.connect", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_callback);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_6Signal_5connect(PyObject *__pyx_v_self, PyObject *__pyx_v_slot); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_6Signal_5connect(PyObject *__pyx_v_self, PyObject *__pyx_v_slot) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("connect (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_6Signal_4connect(((struct __pyx_obj_5pycx4_5pycda_Signal *)__pyx_v_self), ((PyObject *)__pyx_v_slot));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_6Signal_4connect(struct __pyx_obj_5pycx4_5pycda_Signal *__pyx_v_self, PyObject *__pyx_v_slot) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("connect", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_5pycx4_5pycda_6Signal_connect(__pyx_v_self, __pyx_v_slot, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 20, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.Signal.connect", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/signal.pxi":40
 *         Py_INCREF(callback)
 * 
 *     cpdef disconnect(self, callback):             # <<<<<<<<<<<<<<
 *         cdef:
 *             void *tmp
 */

static PyObject *__pyx_pw_5pycx4_5pycda_6Signal_7disconnect(PyObject *__pyx_v_self, PyObject *__pyx_v_callback); /*proto*/
static PyObject *__pyx_f_5pycx4_5pycda_6Signal_disconnect(struct __pyx_obj_5pycx4_5pycda_Signal *__pyx_v_self, PyObject *__pyx_v_callback, int __pyx_skip_dispatch) {
  void *__pyx_v_tmp;
  int __pyx_v_ind;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  __Pyx_RefNannySetupContext("disconnect", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    static PY_UINT64_T tp_dict_version = 0, obj_dict_version = 0;
    if (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict && tp_dict_version == __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) && (!Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset || obj_dict_version == __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))))));
    else {
      PY_UINT64_T type_dict_guard = (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict)) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_disconnect); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 40, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_5pycx4_5pycda_6Signal_7disconnect)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_callback) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_callback);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 40, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
      tp_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      obj_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset) ? __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))) : 0;
      if (unlikely(type_dict_guard != tp_dict_version)) {
        tp_dict_version = obj_dict_version = 0;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    }
    #endif
  }

  /* "pycx4/signal.pxi":44
 *             void *tmp
 *             int ind
 *         for ind in range(self.cnum):             # <<<<<<<<<<<<<<
 *             if callback == <object>self.callbacks[ind]:
 *                 if self.cnum == 1:
 */
  __pyx_t_5 = __pyx_v_self->cnum;
  __pyx_t_6 = __pyx_t_5;
  for (__pyx_t_7 = 0; __pyx_t_7 < __pyx_t_6; __pyx_t_7+=1) {
    __pyx_v_ind = __pyx_t_7;

    /* "pycx4/signal.pxi":45
 *             int ind
 *         for ind in range(self.cnum):
 *             if callback == <object>self.callbacks[ind]:             # <<<<<<<<<<<<<<
 *                 if self.cnum == 1:
 *                     free(self.callbacks)
 */
    __pyx_t_1 = PyObject_RichCompare(__pyx_v_callback, ((PyObject *)(__pyx_v_self->callbacks[__pyx_v_ind])), Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 45, __pyx_L1_error)
    __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 45, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_8) {

      /* "pycx4/signal.pxi":46
 *         for ind in range(self.cnum):
 *             if callback == <object>self.callbacks[ind]:
 *                 if self.cnum == 1:             # <<<<<<<<<<<<<<
 *                     free(self.callbacks)
 *                 else:
 */
      __pyx_t_8 = ((__pyx_v_self->cnum == 1) != 0);
      if (__pyx_t_8) {

        /* "pycx4/signal.pxi":47
 *             if callback == <object>self.callbacks[ind]:
 *                 if self.cnum == 1:
 *                     free(self.callbacks)             # <<<<<<<<<<<<<<
 *                 else:
 *                     memmove(&(self.callbacks[ind]), &(self.callbacks[self.cnum-1]), sizeof(void*))
 */
        free(__pyx_v_self->callbacks);

        /* "pycx4/signal.pxi":46
 *         for ind in range(self.cnum):
 *             if callback == <object>self.callbacks[ind]:
 *                 if self.cnum == 1:             # <<<<<<<<<<<<<<
 *                     free(self.callbacks)
 *                 else:
 */
        goto __pyx_L6;
      }

      /* "pycx4/signal.pxi":49
 *                     free(self.callbacks)
 *                 else:
 *                     memmove(&(self.callbacks[ind]), &(self.callbacks[self.cnum-1]), sizeof(void*))             # <<<<<<<<<<<<<<
 *                     tmp = realloc(self.callbacks, sizeof(void*) * (self.cnum-1))
 *                     if not tmp: raise MemoryError()
 */
      /*else*/ {
        (void)(memmove((&(__pyx_v_self->callbacks[__pyx_v_ind])), (&(__pyx_v_self->callbacks[(__pyx_v_self->cnum - 1)])), (sizeof(void *))));

        /* "pycx4/signal.pxi":50
 *                 else:
 *                     memmove(&(self.callbacks[ind]), &(self.callbacks[self.cnum-1]), sizeof(void*))
 *                     tmp = realloc(self.callbacks, sizeof(void*) * (self.cnum-1))             # <<<<<<<<<<<<<<
 *                     if not tmp: raise MemoryError()
 *                     self.callbacks = <void**>tmp
 */
        __pyx_v_tmp = realloc(__pyx_v_self->callbacks, ((sizeof(void *)) * (__pyx_v_self->cnum - 1)));

        /* "pycx4/signal.pxi":51
 *                     memmove(&(self.callbacks[ind]), &(self.callbacks[self.cnum-1]), sizeof(void*))
 *                     tmp = realloc(self.callbacks, sizeof(void*) * (self.cnum-1))
 *                     if not tmp: raise MemoryError()             # <<<<<<<<<<<<<<
 *                     self.callbacks = <void**>tmp
 *                 self.cnum -= 1
 */
        __pyx_t_8 = ((!(__pyx_v_tmp != 0)) != 0);
        if (unlikely(__pyx_t_8)) {
          PyErr_NoMemory(); __PYX_ERR(0, 51, __pyx_L1_error)
        }

        /* "pycx4/signal.pxi":52
 *                     tmp = realloc(self.callbacks, sizeof(void*) * (self.cnum-1))
 *                     if not tmp: raise MemoryError()
 *                     self.callbacks = <void**>tmp             # <<<<<<<<<<<<<<
 *                 self.cnum -= 1
 *         Py_DECREF(callback)
 */
        __pyx_v_self->callbacks = ((void **)__pyx_v_tmp);
      }
      __pyx_L6:;

      /* "pycx4/signal.pxi":53
 *                     if not tmp: raise MemoryError()
 *                     self.callbacks = <void**>tmp
 *                 self.cnum -= 1             # <<<<<<<<<<<<<<
 *         Py_DECREF(callback)
 * 
 */
      __pyx_v_self->cnum = (__pyx_v_self->cnum - 1);

      /* "pycx4/signal.pxi":45
 *             int ind
 *         for ind in range(self.cnum):
 *             if callback == <object>self.callbacks[ind]:             # <<<<<<<<<<<<<<
 *                 if self.cnum == 1:
 *                     free(self.callbacks)
 */
    }
  }

  /* "pycx4/signal.pxi":54
 *                     self.callbacks = <void**>tmp
 *                 self.cnum -= 1
 *         Py_DECREF(callback)             # <<<<<<<<<<<<<<
 * 
 *     def emit(self, *args):
 */
  Py_DECREF(__pyx_v_callback);

  /* "pycx4/signal.pxi":40
 *         Py_INCREF(callback)
 * 
 *     cpdef disconnect(self, callback):             # <<<<<<<<<<<<<<
 *         cdef:
 *             void *tmp
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pycx4.pycda.Signal.disconnect", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_6Signal_7disconnect(PyObject *__pyx_v_self, PyObject *__pyx_v_callback); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_6Signal_7disconnect(PyObject *__pyx_v_self, PyObject *__pyx_v_callback) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("disconnect (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_6Signal_6disconnect(((struct __pyx_obj_5pycx4_5pycda_Signal *)__pyx_v_self), ((PyObject *)__pyx_v_callback));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_6Signal_6disconnect(struct __pyx_obj_5pycx4_5pycda_Signal *__pyx_v_self, PyObject *__pyx_v_callback) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("disconnect", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_5pycx4_5pycda_6Signal_disconnect(__pyx_v_self, __pyx_v_callback, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.Signal.disconnect", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/signal.pxi":56
 *         Py_DECREF(callback)
 * 
 *     def emit(self, *args):             # <<<<<<<<<<<<<<
 *         cdef int ind
 *         for ind in range(self.cnum):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_6Signal_9emit(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_6Signal_9emit(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("emit (wrapper)", 0);
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "emit", 0))) return NULL;
  __Pyx_INCREF(__pyx_args);
  __pyx_v_args = __pyx_args;
  __pyx_r = __pyx_pf_5pycx4_5pycda_6Signal_8emit(((struct __pyx_obj_5pycx4_5pycda_Signal *)__pyx_v_self), __pyx_v_args);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_args);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_6Signal_8emit(struct __pyx_obj_5pycx4_5pycda_Signal *__pyx_v_self, PyObject *__pyx_v_args) {
  int __pyx_v_ind;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("emit", 0);

  /* "pycx4/signal.pxi":58
 *     def emit(self, *args):
 *         cdef int ind
 *         for ind in range(self.cnum):             # <<<<<<<<<<<<<<
 *             (<object>(self.callbacks[ind]))(*args)
 */
  __pyx_t_1 = __pyx_v_self->cnum;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_ind = __pyx_t_3;

    /* "pycx4/signal.pxi":59
 *         cdef int ind
 *         for ind in range(self.cnum):
 *             (<object>(self.callbacks[ind]))(*args)             # <<<<<<<<<<<<<<
 */
    __pyx_t_4 = __Pyx_PyObject_Call(((PyObject *)(__pyx_v_self->callbacks[__pyx_v_ind])), __pyx_v_args, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 59, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }

  /* "pycx4/signal.pxi":56
 *         Py_DECREF(callback)
 * 
 *     def emit(self, *args):             # <<<<<<<<<<<<<<
 *         cdef int ind
 *         for ind in range(self.cnum):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pycx4.pycda.Signal.emit", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_6Signal_11__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_6Signal_11__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_6Signal_10__reduce_cython__(((struct __pyx_obj_5pycx4_5pycda_Signal *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_6Signal_10__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_Signal *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.Signal.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_6Signal_13__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_6Signal_13__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_6Signal_12__setstate_cython__(((struct __pyx_obj_5pycx4_5pycda_Signal *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_6Signal_12__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_Signal *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.Signal.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/timer.pxi":3
 * from cx4.cxscheduler cimport sl_tout_proc, sl_enq_tout_after, sl_tid_t, sl_deq_tout
 * 
 * cdef void sltimer_proc(int uniq, void *privptr1, sl_tid_t tid, void *privptr2) with gil:             # <<<<<<<<<<<<<<
 *     cdef Timer t = <Timer>privptr2
 *     if t.repeat:
 */

static void __pyx_f_5pycx4_5pycda_sltimer_proc(CYTHON_UNUSED int __pyx_v_uniq, CYTHON_UNUSED void *__pyx_v_privptr1, CYTHON_UNUSED sl_tid_t __pyx_v_tid, void *__pyx_v_privptr2) {
  struct __pyx_obj_5pycx4_5pycda_Timer *__pyx_v_t = 0;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("sltimer_proc", 0);

  /* "pycx4/timer.pxi":4
 * 
 * cdef void sltimer_proc(int uniq, void *privptr1, sl_tid_t tid, void *privptr2) with gil:
 *     cdef Timer t = <Timer>privptr2             # <<<<<<<<<<<<<<
 *     if t.repeat:
 *         t.tid = sl_enq_tout_after(0, NULL, t.usec, sltimer_proc, privptr2)
 */
  __pyx_t_1 = ((PyObject *)__pyx_v_privptr2);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_t = ((struct __pyx_obj_5pycx4_5pycda_Timer *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pycx4/timer.pxi":5
 * cdef void sltimer_proc(int uniq, void *privptr1, sl_tid_t tid, void *privptr2) with gil:
 *     cdef Timer t = <Timer>privptr2
 *     if t.repeat:             # <<<<<<<<<<<<<<
 *         t.tid = sl_enq_tout_after(0, NULL, t.usec, sltimer_proc, privptr2)
 *     else:
 */
  __pyx_t_2 = (__pyx_v_t->repeat != 0);
  if (__pyx_t_2) {

    /* "pycx4/timer.pxi":6
 *     cdef Timer t = <Timer>privptr2
 *     if t.repeat:
 *         t.tid = sl_enq_tout_after(0, NULL, t.usec, sltimer_proc, privptr2)             # <<<<<<<<<<<<<<
 *     else:
 *         t.active = 0
 */
    __pyx_v_t->tid = sl_enq_tout_after(0, NULL, __pyx_v_t->usec, __pyx_f_5pycx4_5pycda_sltimer_proc, __pyx_v_privptr2);

    /* "pycx4/timer.pxi":5
 * cdef void sltimer_proc(int uniq, void *privptr1, sl_tid_t tid, void *privptr2) with gil:
 *     cdef Timer t = <Timer>privptr2
 *     if t.repeat:             # <<<<<<<<<<<<<<
 *         t.tid = sl_enq_tout_after(0, NULL, t.usec, sltimer_proc, privptr2)
 *     else:
 */
    goto __pyx_L3;
  }

  /* "pycx4/timer.pxi":8
 *         t.tid = sl_enq_tout_after(0, NULL, t.usec, sltimer_proc, privptr2)
 *     else:
 *         t.active = 0             # <<<<<<<<<<<<<<
 *     t.timeout.emit(t)
 * 
 */
  /*else*/ {
    __pyx_v_t->active = 0;
  }
  __pyx_L3:;

  /* "pycx4/timer.pxi":9
 *     else:
 *         t.active = 0
 *     t.timeout.emit(t)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_t->timeout), __pyx_n_s_emit); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, ((PyObject *)__pyx_v_t)) : __Pyx_PyObject_CallOneArg(__pyx_t_3, ((PyObject *)__pyx_v_t));
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/timer.pxi":3
 * from cx4.cxscheduler cimport sl_tout_proc, sl_enq_tout_after, sl_tid_t, sl_deq_tout
 * 
 * cdef void sltimer_proc(int uniq, void *privptr1, sl_tid_t tid, void *privptr2) with gil:             # <<<<<<<<<<<<<<
 *     cdef Timer t = <Timer>privptr2
 *     if t.repeat:
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("pycx4.pycda.sltimer_proc", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_t);
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
}

/* "pycx4/timer.pxi":20
 *         Signal timeout
 * 
 *     def __init__(self, int msec=1000):             # <<<<<<<<<<<<<<
 *         self.setInterval(msec)
 *         self.repeat = 1
 */

/* Python wrapper */
static int __pyx_pw_5pycx4_5pycda_5Timer_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5pycx4_5pycda_5Timer_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_v_msec;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_msec,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_msec);
          if (value) { values[0] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(2, 20, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    if (values[0]) {
      __pyx_v_msec = __Pyx_PyInt_As_int(values[0]); if (unlikely((__pyx_v_msec == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 20, __pyx_L3_error)
    } else {
      __pyx_v_msec = ((int)0x3E8);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 0, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 20, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pycx4.pycda.Timer.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pycx4_5pycda_5Timer___init__(((struct __pyx_obj_5pycx4_5pycda_Timer *)__pyx_v_self), __pyx_v_msec);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pycx4_5pycda_5Timer___init__(struct __pyx_obj_5pycx4_5pycda_Timer *__pyx_v_self, int __pyx_v_msec) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "pycx4/timer.pxi":21
 * 
 *     def __init__(self, int msec=1000):
 *         self.setInterval(msec)             # <<<<<<<<<<<<<<
 *         self.repeat = 1
 *         self.timeout = Signal(object)
 */
  ((struct __pyx_vtabstruct_5pycx4_5pycda_Timer *)__pyx_v_self->__pyx_vtab)->setInterval(__pyx_v_self, __pyx_v_msec, 0);

  /* "pycx4/timer.pxi":22
 *     def __init__(self, int msec=1000):
 *         self.setInterval(msec)
 *         self.repeat = 1             # <<<<<<<<<<<<<<
 *         self.timeout = Signal(object)
 * 
 */
  __pyx_v_self->repeat = 1;

  /* "pycx4/timer.pxi":23
 *         self.setInterval(msec)
 *         self.repeat = 1
 *         self.timeout = Signal(object)             # <<<<<<<<<<<<<<
 * 
 *     cpdef stop(self):
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5pycx4_5pycda_Signal), __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 23, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->timeout);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->timeout));
  __pyx_v_self->timeout = ((struct __pyx_obj_5pycx4_5pycda_Signal *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pycx4/timer.pxi":20
 *         Signal timeout
 * 
 *     def __init__(self, int msec=1000):             # <<<<<<<<<<<<<<
 *         self.setInterval(msec)
 *         self.repeat = 1
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.Timer.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/timer.pxi":25
 *         self.timeout = Signal(object)
 * 
 *     cpdef stop(self):             # <<<<<<<<<<<<<<
 *         if self.active == 1:
 *             self.active = 0
 */

static PyObject *__pyx_pw_5pycx4_5pycda_5Timer_3stop(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_5pycx4_5pycda_5Timer_stop(struct __pyx_obj_5pycx4_5pycda_Timer *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  __Pyx_RefNannySetupContext("stop", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    static PY_UINT64_T tp_dict_version = 0, obj_dict_version = 0;
    if (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict && tp_dict_version == __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) && (!Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset || obj_dict_version == __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))))));
    else {
      PY_UINT64_T type_dict_guard = (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict)) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_stop); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 25, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_5pycx4_5pycda_5Timer_3stop)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 25, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
      tp_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      obj_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset) ? __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))) : 0;
      if (unlikely(type_dict_guard != tp_dict_version)) {
        tp_dict_version = obj_dict_version = 0;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    }
    #endif
  }

  /* "pycx4/timer.pxi":26
 * 
 *     cpdef stop(self):
 *         if self.active == 1:             # <<<<<<<<<<<<<<
 *             self.active = 0
 *             sl_deq_tout(self.tid)
 */
  __pyx_t_5 = ((__pyx_v_self->active == 1) != 0);
  if (__pyx_t_5) {

    /* "pycx4/timer.pxi":27
 *     cpdef stop(self):
 *         if self.active == 1:
 *             self.active = 0             # <<<<<<<<<<<<<<
 *             sl_deq_tout(self.tid)
 * 
 */
    __pyx_v_self->active = 0;

    /* "pycx4/timer.pxi":28
 *         if self.active == 1:
 *             self.active = 0
 *             sl_deq_tout(self.tid)             # <<<<<<<<<<<<<<
 * 
 *     cpdef start(self):
 */
    (void)(sl_deq_tout(__pyx_v_self->tid));

    /* "pycx4/timer.pxi":26
 * 
 *     cpdef stop(self):
 *         if self.active == 1:             # <<<<<<<<<<<<<<
 *             self.active = 0
 *             sl_deq_tout(self.tid)
 */
  }

  /* "pycx4/timer.pxi":25
 *         self.timeout = Signal(object)
 * 
 *     cpdef stop(self):             # <<<<<<<<<<<<<<
 *         if self.active == 1:
 *             self.active = 0
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pycx4.pycda.Timer.stop", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_5Timer_3stop(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_5Timer_3stop(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("stop (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_5Timer_2stop(((struct __pyx_obj_5pycx4_5pycda_Timer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_5Timer_2stop(struct __pyx_obj_5pycx4_5pycda_Timer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("stop", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_5pycx4_5pycda_5Timer_stop(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.Timer.stop", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/timer.pxi":30
 *             sl_deq_tout(self.tid)
 * 
 *     cpdef start(self):             # <<<<<<<<<<<<<<
 *         if self.active == 1:
 *             return
 */

static PyObject *__pyx_pw_5pycx4_5pycda_5Timer_5start(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_5pycx4_5pycda_5Timer_start(struct __pyx_obj_5pycx4_5pycda_Timer *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  __Pyx_RefNannySetupContext("start", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    static PY_UINT64_T tp_dict_version = 0, obj_dict_version = 0;
    if (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict && tp_dict_version == __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) && (!Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset || obj_dict_version == __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))))));
    else {
      PY_UINT64_T type_dict_guard = (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict)) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_start); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 30, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_5pycx4_5pycda_5Timer_5start)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 30, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
      tp_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      obj_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset) ? __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))) : 0;
      if (unlikely(type_dict_guard != tp_dict_version)) {
        tp_dict_version = obj_dict_version = 0;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    }
    #endif
  }

  /* "pycx4/timer.pxi":31
 * 
 *     cpdef start(self):
 *         if self.active == 1:             # <<<<<<<<<<<<<<
 *             return
 *         self.active = 1
 */
  __pyx_t_5 = ((__pyx_v_self->active == 1) != 0);
  if (__pyx_t_5) {

    /* "pycx4/timer.pxi":32
 *     cpdef start(self):
 *         if self.active == 1:
 *             return             # <<<<<<<<<<<<<<
 *         self.active = 1
 *         self.tid = sl_enq_tout_after(0, NULL, self.usec, sltimer_proc, <void*>self)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "pycx4/timer.pxi":31
 * 
 *     cpdef start(self):
 *         if self.active == 1:             # <<<<<<<<<<<<<<
 *             return
 *         self.active = 1
 */
  }

  /* "pycx4/timer.pxi":33
 *         if self.active == 1:
 *             return
 *         self.active = 1             # <<<<<<<<<<<<<<
 *         self.tid = sl_enq_tout_after(0, NULL, self.usec, sltimer_proc, <void*>self)
 * 
 */
  __pyx_v_self->active = 1;

  /* "pycx4/timer.pxi":34
 *             return
 *         self.active = 1
 *         self.tid = sl_enq_tout_after(0, NULL, self.usec, sltimer_proc, <void*>self)             # <<<<<<<<<<<<<<
 * 
 *     cpdef singleShot(self, int msec=0, proc=None):
 */
  __pyx_v_self->tid = sl_enq_tout_after(0, NULL, __pyx_v_self->usec, __pyx_f_5pycx4_5pycda_sltimer_proc, ((void *)__pyx_v_self));

  /* "pycx4/timer.pxi":30
 *             sl_deq_tout(self.tid)
 * 
 *     cpdef start(self):             # <<<<<<<<<<<<<<
 *         if self.active == 1:
 *             return
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pycx4.pycda.Timer.start", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_5Timer_5start(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_5Timer_5start(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("start (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_5Timer_4start(((struct __pyx_obj_5pycx4_5pycda_Timer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_5Timer_4start(struct __pyx_obj_5pycx4_5pycda_Timer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("start", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_5pycx4_5pycda_5Timer_start(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 30, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.Timer.start", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/timer.pxi":36
 *         self.tid = sl_enq_tout_after(0, NULL, self.usec, sltimer_proc, <void*>self)
 * 
 *     cpdef singleShot(self, int msec=0, proc=None):             # <<<<<<<<<<<<<<
 *         if msec > 0:
 *             self.setInterval(msec)
 */

static PyObject *__pyx_pw_5pycx4_5pycda_5Timer_7singleShot(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_5pycx4_5pycda_5Timer_singleShot(struct __pyx_obj_5pycx4_5pycda_Timer *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_5pycx4_5pycda_5Timer_singleShot *__pyx_optional_args) {
  int __pyx_v_msec = ((int)0);
  PyObject *__pyx_v_proc = ((PyObject *)Py_None);
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  int __pyx_t_9;
  __Pyx_RefNannySetupContext("singleShot", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_msec = __pyx_optional_args->msec;
      if (__pyx_optional_args->__pyx_n > 1) {
        __pyx_v_proc = __pyx_optional_args->proc;
      }
    }
  }
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    static PY_UINT64_T tp_dict_version = 0, obj_dict_version = 0;
    if (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict && tp_dict_version == __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) && (!Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset || obj_dict_version == __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))))));
    else {
      PY_UINT64_T type_dict_guard = (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict)) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_singleShot); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 36, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_5pycx4_5pycda_5Timer_7singleShot)) {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_msec); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 36, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; __pyx_t_5 = NULL;
        __pyx_t_6 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
          if (likely(__pyx_t_5)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
            __Pyx_INCREF(__pyx_t_5);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_4, function);
            __pyx_t_6 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_4)) {
          PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_v_proc};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 36, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
          PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_v_proc};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 36, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        } else
        #endif
        {
          __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 36, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          if (__pyx_t_5) {
            __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
          }
          __Pyx_GIVEREF(__pyx_t_3);
          PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_t_3);
          __Pyx_INCREF(__pyx_v_proc);
          __Pyx_GIVEREF(__pyx_v_proc);
          PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_v_proc);
          __pyx_t_3 = 0;
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_7, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 36, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
      tp_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      obj_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset) ? __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))) : 0;
      if (unlikely(type_dict_guard != tp_dict_version)) {
        tp_dict_version = obj_dict_version = 0;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    }
    #endif
  }

  /* "pycx4/timer.pxi":37
 * 
 *     cpdef singleShot(self, int msec=0, proc=None):
 *         if msec > 0:             # <<<<<<<<<<<<<<
 *             self.setInterval(msec)
 *         if proc is not None:
 */
  __pyx_t_8 = ((__pyx_v_msec > 0) != 0);
  if (__pyx_t_8) {

    /* "pycx4/timer.pxi":38
 *     cpdef singleShot(self, int msec=0, proc=None):
 *         if msec > 0:
 *             self.setInterval(msec)             # <<<<<<<<<<<<<<
 *         if proc is not None:
 *             self.timeout.connect(proc)
 */
    ((struct __pyx_vtabstruct_5pycx4_5pycda_Timer *)__pyx_v_self->__pyx_vtab)->setInterval(__pyx_v_self, __pyx_v_msec, 0);

    /* "pycx4/timer.pxi":37
 * 
 *     cpdef singleShot(self, int msec=0, proc=None):
 *         if msec > 0:             # <<<<<<<<<<<<<<
 *             self.setInterval(msec)
 *         if proc is not None:
 */
  }

  /* "pycx4/timer.pxi":39
 *         if msec > 0:
 *             self.setInterval(msec)
 *         if proc is not None:             # <<<<<<<<<<<<<<
 *             self.timeout.connect(proc)
 *         self.repeat = 0
 */
  __pyx_t_8 = (__pyx_v_proc != Py_None);
  __pyx_t_9 = (__pyx_t_8 != 0);
  if (__pyx_t_9) {

    /* "pycx4/timer.pxi":40
 *             self.setInterval(msec)
 *         if proc is not None:
 *             self.timeout.connect(proc)             # <<<<<<<<<<<<<<
 *         self.repeat = 0
 *         self.start()
 */
    __pyx_t_1 = ((struct __pyx_vtabstruct_5pycx4_5pycda_Signal *)__pyx_v_self->timeout->__pyx_vtab)->connect(__pyx_v_self->timeout, __pyx_v_proc, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 40, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pycx4/timer.pxi":39
 *         if msec > 0:
 *             self.setInterval(msec)
 *         if proc is not None:             # <<<<<<<<<<<<<<
 *             self.timeout.connect(proc)
 *         self.repeat = 0
 */
  }

  /* "pycx4/timer.pxi":41
 *         if proc is not None:
 *             self.timeout.connect(proc)
 *         self.repeat = 0             # <<<<<<<<<<<<<<
 *         self.start()
 * 
 */
  __pyx_v_self->repeat = 0;

  /* "pycx4/timer.pxi":42
 *             self.timeout.connect(proc)
 *         self.repeat = 0
 *         self.start()             # <<<<<<<<<<<<<<
 * 
 *     cpdef int interval(self):
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_5pycx4_5pycda_Timer *)__pyx_v_self->__pyx_vtab)->start(__pyx_v_self, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 42, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/timer.pxi":36
 *         self.tid = sl_enq_tout_after(0, NULL, self.usec, sltimer_proc, <void*>self)
 * 
 *     cpdef singleShot(self, int msec=0, proc=None):             # <<<<<<<<<<<<<<
 *         if msec > 0:
 *             self.setInterval(msec)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("pycx4.pycda.Timer.singleShot", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_5Timer_7singleShot(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_5Timer_7singleShot(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_v_msec;
  PyObject *__pyx_v_proc = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("singleShot (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_msec,&__pyx_n_s_proc,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_msec);
          if (value) { values[0] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_proc);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "singleShot") < 0)) __PYX_ERR(2, 36, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    if (values[0]) {
      __pyx_v_msec = __Pyx_PyInt_As_int(values[0]); if (unlikely((__pyx_v_msec == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 36, __pyx_L3_error)
    } else {
      __pyx_v_msec = ((int)0);
    }
    __pyx_v_proc = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("singleShot", 0, 0, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 36, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pycx4.pycda.Timer.singleShot", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pycx4_5pycda_5Timer_6singleShot(((struct __pyx_obj_5pycx4_5pycda_Timer *)__pyx_v_self), __pyx_v_msec, __pyx_v_proc);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_5Timer_6singleShot(struct __pyx_obj_5pycx4_5pycda_Timer *__pyx_v_self, int __pyx_v_msec, PyObject *__pyx_v_proc) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_5pycx4_5pycda_5Timer_singleShot __pyx_t_2;
  __Pyx_RefNannySetupContext("singleShot", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 2;
  __pyx_t_2.msec = __pyx_v_msec;
  __pyx_t_2.proc = __pyx_v_proc;
  __pyx_t_1 = __pyx_vtabptr_5pycx4_5pycda_Timer->singleShot(__pyx_v_self, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 36, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.Timer.singleShot", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/timer.pxi":44
 *         self.start()
 * 
 *     cpdef int interval(self):             # <<<<<<<<<<<<<<
 *         return <int>(self.usec/1000)
 * 
 */

static PyObject *__pyx_pw_5pycx4_5pycda_5Timer_9interval(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static int __pyx_f_5pycx4_5pycda_5Timer_interval(struct __pyx_obj_5pycx4_5pycda_Timer *__pyx_v_self, int __pyx_skip_dispatch) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  __Pyx_RefNannySetupContext("interval", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    static PY_UINT64_T tp_dict_version = 0, obj_dict_version = 0;
    if (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict && tp_dict_version == __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) && (!Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset || obj_dict_version == __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))))));
    else {
      PY_UINT64_T type_dict_guard = (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict)) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_interval); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 44, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_5pycx4_5pycda_5Timer_9interval)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 44, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 44, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
      tp_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      obj_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset) ? __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))) : 0;
      if (unlikely(type_dict_guard != tp_dict_version)) {
        tp_dict_version = obj_dict_version = 0;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    }
    #endif
  }

  /* "pycx4/timer.pxi":45
 * 
 *     cpdef int interval(self):
 *         return <int>(self.usec/1000)             # <<<<<<<<<<<<<<
 * 
 *     cpdef void setInterval(self, int msec):
 */
  __pyx_r = ((int)(((long)__pyx_v_self->usec) / 0x3E8));
  goto __pyx_L0;

  /* "pycx4/timer.pxi":44
 *         self.start()
 * 
 *     cpdef int interval(self):             # <<<<<<<<<<<<<<
 *         return <int>(self.usec/1000)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("pycx4.pycda.Timer.interval", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_5Timer_9interval(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_5Timer_9interval(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("interval (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_5Timer_8interval(((struct __pyx_obj_5pycx4_5pycda_Timer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_5Timer_8interval(struct __pyx_obj_5pycx4_5pycda_Timer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("interval", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_f_5pycx4_5pycda_5Timer_interval(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 44, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.Timer.interval", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/timer.pxi":47
 *         return <int>(self.usec/1000)
 * 
 *     cpdef void setInterval(self, int msec):             # <<<<<<<<<<<<<<
 *         self.usec = 1000 * msec
 * 
 */

static PyObject *__pyx_pw_5pycx4_5pycda_5Timer_11setInterval(PyObject *__pyx_v_self, PyObject *__pyx_arg_msec); /*proto*/
static void __pyx_f_5pycx4_5pycda_5Timer_setInterval(struct __pyx_obj_5pycx4_5pycda_Timer *__pyx_v_self, int __pyx_v_msec, int __pyx_skip_dispatch) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("setInterval", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    static PY_UINT64_T tp_dict_version = 0, obj_dict_version = 0;
    if (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict && tp_dict_version == __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) && (!Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset || obj_dict_version == __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))))));
    else {
      PY_UINT64_T type_dict_guard = (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict)) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_setInterval); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 47, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_5pycx4_5pycda_5Timer_11setInterval)) {
        __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_msec); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 47, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; __pyx_t_5 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
          if (likely(__pyx_t_5)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
            __Pyx_INCREF(__pyx_t_5);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_4, function);
          }
        }
        __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_5, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_3);
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 47, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
      tp_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      obj_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset) ? __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))) : 0;
      if (unlikely(type_dict_guard != tp_dict_version)) {
        tp_dict_version = obj_dict_version = 0;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    }
    #endif
  }

  /* "pycx4/timer.pxi":48
 * 
 *     cpdef void setInterval(self, int msec):
 *         self.usec = 1000 * msec             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_v_self->usec = (0x3E8 * __pyx_v_msec);

  /* "pycx4/timer.pxi":47
 *         return <int>(self.usec/1000)
 * 
 *     cpdef void setInterval(self, int msec):             # <<<<<<<<<<<<<<
 *         self.usec = 1000 * msec
 * 
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_WriteUnraisable("pycx4.pycda.Timer.setInterval", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_5Timer_11setInterval(PyObject *__pyx_v_self, PyObject *__pyx_arg_msec); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_5Timer_11setInterval(PyObject *__pyx_v_self, PyObject *__pyx_arg_msec) {
  int __pyx_v_msec;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("setInterval (wrapper)", 0);
  assert(__pyx_arg_msec); {
    __pyx_v_msec = __Pyx_PyInt_As_int(__pyx_arg_msec); if (unlikely((__pyx_v_msec == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 47, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pycx4.pycda.Timer.setInterval", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pycx4_5pycda_5Timer_10setInterval(((struct __pyx_obj_5pycx4_5pycda_Timer *)__pyx_v_self), ((int)__pyx_v_msec));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_5Timer_10setInterval(struct __pyx_obj_5pycx4_5pycda_Timer *__pyx_v_self, int __pyx_v_msec) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("setInterval", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_void_to_None(__pyx_f_5pycx4_5pycda_5Timer_setInterval(__pyx_v_self, __pyx_v_msec, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 47, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.Timer.setInterval", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/timer.pxi":14
 * cdef class Timer:
 *     cdef readonly:
 *         int usec             # <<<<<<<<<<<<<<
 *         int repeat
 *         int active
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_5Timer_4usec_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_5Timer_4usec_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_5Timer_4usec___get__(((struct __pyx_obj_5pycx4_5pycda_Timer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_5Timer_4usec___get__(struct __pyx_obj_5pycx4_5pycda_Timer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->usec); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.Timer.usec.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/timer.pxi":15
 *     cdef readonly:
 *         int usec
 *         int repeat             # <<<<<<<<<<<<<<
 *         int active
 *         sl_tid_t tid
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_5Timer_6repeat_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_5Timer_6repeat_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_5Timer_6repeat___get__(((struct __pyx_obj_5pycx4_5pycda_Timer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_5Timer_6repeat___get__(struct __pyx_obj_5pycx4_5pycda_Timer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->repeat); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.Timer.repeat.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/timer.pxi":16
 *         int usec
 *         int repeat
 *         int active             # <<<<<<<<<<<<<<
 *         sl_tid_t tid
 *         Signal timeout
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_5Timer_6active_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_5Timer_6active_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_5Timer_6active___get__(((struct __pyx_obj_5pycx4_5pycda_Timer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_5Timer_6active___get__(struct __pyx_obj_5pycx4_5pycda_Timer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->active); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.Timer.active.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/timer.pxi":17
 *         int repeat
 *         int active
 *         sl_tid_t tid             # <<<<<<<<<<<<<<
 *         Signal timeout
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_5Timer_3tid_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_5Timer_3tid_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_5Timer_3tid___get__(((struct __pyx_obj_5pycx4_5pycda_Timer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_5Timer_3tid___get__(struct __pyx_obj_5pycx4_5pycda_Timer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_sl_tid_t(__pyx_v_self->tid); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.Timer.tid.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/timer.pxi":18
 *         int active
 *         sl_tid_t tid
 *         Signal timeout             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, int msec=1000):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_5Timer_7timeout_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_5Timer_7timeout_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_5Timer_7timeout___get__(((struct __pyx_obj_5pycx4_5pycda_Timer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_5Timer_7timeout___get__(struct __pyx_obj_5pycx4_5pycda_Timer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->timeout));
  __pyx_r = ((PyObject *)__pyx_v_self->timeout);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef bint use_setstate
 *     state = (self.active, self.repeat, self.tid, self.timeout, self.usec)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_5Timer_13__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_5Timer_13__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_5Timer_12__reduce_cython__(((struct __pyx_obj_5pycx4_5pycda_Timer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_5Timer_12__reduce_cython__(struct __pyx_obj_5pycx4_5pycda_Timer *__pyx_v_self) {
  int __pyx_v_use_setstate;
  PyObject *__pyx_v_state = NULL;
  PyObject *__pyx_v__dict = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_t_7;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     cdef bint use_setstate
 *     state = (self.active, self.repeat, self.tid, self.timeout, self.usec)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->active); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->repeat); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_sl_tid_t(__pyx_v_self->tid); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_self->usec); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(5); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->timeout));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->timeout));
  PyTuple_SET_ITEM(__pyx_t_5, 3, ((PyObject *)__pyx_v_self->timeout));
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 4, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_5);
  __pyx_t_5 = 0;

  /* "(tree fragment)":4
 *     cdef bint use_setstate
 *     state = (self.active, self.repeat, self.tid, self.timeout, self.usec)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_5 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_v__dict = __pyx_t_5;
  __pyx_t_5 = 0;

  /* "(tree fragment)":5
 *     state = (self.active, self.repeat, self.tid, self.timeout, self.usec)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_6 = (__pyx_v__dict != Py_None);
  __pyx_t_7 = (__pyx_t_6 != 0);
  if (__pyx_t_7) {

    /* "(tree fragment)":6
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v__dict);
    __pyx_t_4 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "(tree fragment)":7
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.timeout is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":5
 *     state = (self.active, self.repeat, self.tid, self.timeout, self.usec)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":9
 *         use_setstate = True
 *     else:
 *         use_setstate = self.timeout is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_Timer, (type(self), 0xef1b4d4, None), state
 */
  /*else*/ {
    __pyx_t_7 = (((PyObject *)__pyx_v_self->timeout) != Py_None);
    __pyx_v_use_setstate = __pyx_t_7;
  }
  __pyx_L3:;

  /* "(tree fragment)":10
 *     else:
 *         use_setstate = self.timeout is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_Timer, (type(self), 0xef1b4d4, None), state
 *     else:
 */
  __pyx_t_7 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_7) {

    /* "(tree fragment)":11
 *         use_setstate = self.timeout is not None
 *     if use_setstate:
 *         return __pyx_unpickle_Timer, (type(self), 0xef1b4d4, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_Timer, (type(self), 0xef1b4d4, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_Timer); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_5, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_250721492);
    __Pyx_GIVEREF(__pyx_int_250721492);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_int_250721492);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_5, 2, Py_None);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_5);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_state);
    __pyx_t_4 = 0;
    __pyx_t_5 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":10
 *     else:
 *         use_setstate = self.timeout is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_Timer, (type(self), 0xef1b4d4, None), state
 *     else:
 */
  }

  /* "(tree fragment)":13
 *         return __pyx_unpickle_Timer, (type(self), 0xef1b4d4, None), state
 *     else:
 *         return __pyx_unpickle_Timer, (type(self), 0xef1b4d4, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_Timer__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_pyx_unpickle_Timer); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_5, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_250721492);
    __Pyx_GIVEREF(__pyx_int_250721492);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_int_250721492);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_v_state);
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_5);
    __pyx_t_3 = 0;
    __pyx_t_5 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef bint use_setstate
 *     state = (self.active, self.repeat, self.tid, self.timeout, self.usec)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pycx4.pycda.Timer.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":14
 *     else:
 *         return __pyx_unpickle_Timer, (type(self), 0xef1b4d4, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_Timer__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_5Timer_15__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_5Timer_15__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_5Timer_14__setstate_cython__(((struct __pyx_obj_5pycx4_5pycda_Timer *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_5Timer_14__setstate_cython__(struct __pyx_obj_5pycx4_5pycda_Timer *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":15
 *         return __pyx_unpickle_Timer, (type(self), 0xef1b4d4, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_Timer__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 15, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_5pycx4_5pycda___pyx_unpickle_Timer__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":14
 *     else:
 *         return __pyx_unpickle_Timer, (type(self), 0xef1b4d4, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_Timer__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.Timer.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/fd_event.pxi":3
 * 
 * 
 * cdef void fd_event_proc(int uniq, void *privptr1, sl_fdh_t fdh, int fd, int mask, void *privptr2) with gil:             # <<<<<<<<<<<<<<
 *     cdef FdEvent fev = <FdEvent>privptr1
 *     fev.ready.emit(fev)
 */

static void __pyx_f_5pycx4_5pycda_fd_event_proc(CYTHON_UNUSED int __pyx_v_uniq, void *__pyx_v_privptr1, CYTHON_UNUSED sl_fdh_t __pyx_v_fdh, CYTHON_UNUSED int __pyx_v_fd, CYTHON_UNUSED int __pyx_v_mask, CYTHON_UNUSED void *__pyx_v_privptr2) {
  struct __pyx_obj_5pycx4_5pycda_FdEvent *__pyx_v_fev = 0;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("fd_event_proc", 0);

  /* "pycx4/fd_event.pxi":4
 * 
 * cdef void fd_event_proc(int uniq, void *privptr1, sl_fdh_t fdh, int fd, int mask, void *privptr2) with gil:
 *     cdef FdEvent fev = <FdEvent>privptr1             # <<<<<<<<<<<<<<
 *     fev.ready.emit(fev)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_v_privptr1);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_fev = ((struct __pyx_obj_5pycx4_5pycda_FdEvent *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pycx4/fd_event.pxi":5
 * cdef void fd_event_proc(int uniq, void *privptr1, sl_fdh_t fdh, int fd, int mask, void *privptr2) with gil:
 *     cdef FdEvent fev = <FdEvent>privptr1
 *     fev.ready.emit(fev)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_fev->ready, __pyx_n_s_emit); if (unlikely(!__pyx_t_2)) __PYX_ERR(5, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, ((PyObject *)__pyx_v_fev)) : __Pyx_PyObject_CallOneArg(__pyx_t_2, ((PyObject *)__pyx_v_fev));
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(5, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/fd_event.pxi":3
 * 
 * 
 * cdef void fd_event_proc(int uniq, void *privptr1, sl_fdh_t fdh, int fd, int mask, void *privptr2) with gil:             # <<<<<<<<<<<<<<
 *     cdef FdEvent fev = <FdEvent>privptr1
 *     fev.ready.emit(fev)
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_WriteUnraisable("pycx4.pycda.fd_event_proc", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_fev);
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
}

/* "pycx4/fd_event.pxi":14
 *         sl_fdh_t fhd
 * 
 *     def __init__(self, file, mask=SL_RD):             # <<<<<<<<<<<<<<
 *         self.file = file
 *         self.mask = mask
 */

/* Python wrapper */
static int __pyx_pw_5pycx4_5pycda_7FdEvent_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5pycx4_5pycda_7FdEvent_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_file = 0;
  PyObject *__pyx_v_mask = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_file,&__pyx_n_s_mask,0};
    PyObject* values[2] = {0,0};
    values[1] = __pyx_k__5;
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_file)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_mask);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(5, 14, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_file = values[0];
    __pyx_v_mask = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(5, 14, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pycx4.pycda.FdEvent.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pycx4_5pycda_7FdEvent___init__(((struct __pyx_obj_5pycx4_5pycda_FdEvent *)__pyx_v_self), __pyx_v_file, __pyx_v_mask);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pycx4_5pycda_7FdEvent___init__(struct __pyx_obj_5pycx4_5pycda_FdEvent *__pyx_v_self, PyObject *__pyx_v_file, PyObject *__pyx_v_mask) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "pycx4/fd_event.pxi":15
 * 
 *     def __init__(self, file, mask=SL_RD):
 *         self.file = file             # <<<<<<<<<<<<<<
 *         self.mask = mask
 *         self.ready = Signal(object)
 */
  __Pyx_INCREF(__pyx_v_file);
  __Pyx_GIVEREF(__pyx_v_file);
  __Pyx_GOTREF(__pyx_v_self->file);
  __Pyx_DECREF(__pyx_v_self->file);
  __pyx_v_self->file = __pyx_v_file;

  /* "pycx4/fd_event.pxi":16
 *     def __init__(self, file, mask=SL_RD):
 *         self.file = file
 *         self.mask = mask             # <<<<<<<<<<<<<<
 *         self.ready = Signal(object)
 *         self.fhd = sl_add_fd(0, <void*>self, file.fileno(), mask, fd_event_proc, NULL)
 */
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_mask); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(5, 16, __pyx_L1_error)
  __pyx_v_self->mask = __pyx_t_1;

  /* "pycx4/fd_event.pxi":17
 *         self.file = file
 *         self.mask = mask
 *         self.ready = Signal(object)             # <<<<<<<<<<<<<<
 *         self.fhd = sl_add_fd(0, <void*>self, file.fileno(), mask, fd_event_proc, NULL)
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5pycx4_5pycda_Signal), __pyx_tuple__6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(5, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->ready);
  __Pyx_DECREF(__pyx_v_self->ready);
  __pyx_v_self->ready = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "pycx4/fd_event.pxi":18
 *         self.mask = mask
 *         self.ready = Signal(object)
 *         self.fhd = sl_add_fd(0, <void*>self, file.fileno(), mask, fd_event_proc, NULL)             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_file, __pyx_n_s_fileno); if (unlikely(!__pyx_t_3)) __PYX_ERR(5, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(5, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(5, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_v_mask); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(5, 18, __pyx_L1_error)
  __pyx_v_self->fhd = sl_add_fd(0, ((void *)__pyx_v_self), __pyx_t_1, __pyx_t_5, __pyx_f_5pycx4_5pycda_fd_event_proc, NULL);

  /* "pycx4/fd_event.pxi":14
 *         sl_fdh_t fhd
 * 
 *     def __init__(self, file, mask=SL_RD):             # <<<<<<<<<<<<<<
 *         self.file = file
 *         self.mask = mask
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pycx4.pycda.FdEvent.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/fd_event.pxi":20
 *         self.fhd = sl_add_fd(0, <void*>self, file.fileno(), mask, fd_event_proc, NULL)
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         sl_del_fd(self.fhd)
 */

/* Python wrapper */
static void __pyx_pw_5pycx4_5pycda_7FdEvent_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_5pycx4_5pycda_7FdEvent_3__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_5pycx4_5pycda_7FdEvent_2__dealloc__(((struct __pyx_obj_5pycx4_5pycda_FdEvent *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_5pycx4_5pycda_7FdEvent_2__dealloc__(struct __pyx_obj_5pycx4_5pycda_FdEvent *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "pycx4/fd_event.pxi":21
 * 
 *     def __dealloc__(self):
 *         sl_del_fd(self.fhd)             # <<<<<<<<<<<<<<
 */
  (void)(sl_del_fd(__pyx_v_self->fhd));

  /* "pycx4/fd_event.pxi":20
 *         self.fhd = sl_add_fd(0, <void*>self, file.fileno(), mask, fd_event_proc, NULL)
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         sl_del_fd(self.fhd)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/fd_event.pxi":10
 * cdef class FdEvent:
 *     cdef readonly:
 *         object file, ready             # <<<<<<<<<<<<<<
 *         int mask
 *         sl_fdh_t fhd
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_7FdEvent_4file_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_7FdEvent_4file_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_7FdEvent_4file___get__(((struct __pyx_obj_5pycx4_5pycda_FdEvent *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_7FdEvent_4file___get__(struct __pyx_obj_5pycx4_5pycda_FdEvent *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->file);
  __pyx_r = __pyx_v_self->file;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_7FdEvent_5ready_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_7FdEvent_5ready_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_7FdEvent_5ready___get__(((struct __pyx_obj_5pycx4_5pycda_FdEvent *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_7FdEvent_5ready___get__(struct __pyx_obj_5pycx4_5pycda_FdEvent *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->ready);
  __pyx_r = __pyx_v_self->ready;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/fd_event.pxi":11
 *     cdef readonly:
 *         object file, ready
 *         int mask             # <<<<<<<<<<<<<<
 *         sl_fdh_t fhd
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_7FdEvent_4mask_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_7FdEvent_4mask_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_7FdEvent_4mask___get__(((struct __pyx_obj_5pycx4_5pycda_FdEvent *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_7FdEvent_4mask___get__(struct __pyx_obj_5pycx4_5pycda_FdEvent *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->mask); if (unlikely(!__pyx_t_1)) __PYX_ERR(5, 11, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.FdEvent.mask.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/fd_event.pxi":12
 *         object file, ready
 *         int mask
 *         sl_fdh_t fhd             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, file, mask=SL_RD):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_7FdEvent_3fhd_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_7FdEvent_3fhd_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_7FdEvent_3fhd___get__(((struct __pyx_obj_5pycx4_5pycda_FdEvent *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_7FdEvent_3fhd___get__(struct __pyx_obj_5pycx4_5pycda_FdEvent *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_sl_fdh_t(__pyx_v_self->fhd); if (unlikely(!__pyx_t_1)) __PYX_ERR(5, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.FdEvent.fhd.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef bint use_setstate
 *     state = (self.fhd, self.file, self.mask, self.ready)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_7FdEvent_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_7FdEvent_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_7FdEvent_4__reduce_cython__(((struct __pyx_obj_5pycx4_5pycda_FdEvent *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_7FdEvent_4__reduce_cython__(struct __pyx_obj_5pycx4_5pycda_FdEvent *__pyx_v_self) {
  int __pyx_v_use_setstate;
  PyObject *__pyx_v_state = NULL;
  PyObject *__pyx_v__dict = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     cdef bint use_setstate
 *     state = (self.fhd, self.file, self.mask, self.ready)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyInt_From_sl_fdh_t(__pyx_v_self->fhd); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->mask); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(4); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->file);
  __Pyx_GIVEREF(__pyx_v_self->file);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v_self->file);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->ready);
  __Pyx_GIVEREF(__pyx_v_self->ready);
  PyTuple_SET_ITEM(__pyx_t_3, 3, __pyx_v_self->ready);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "(tree fragment)":4
 *     cdef bint use_setstate
 *     state = (self.fhd, self.file, self.mask, self.ready)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_3 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v__dict = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":5
 *     state = (self.fhd, self.file, self.mask, self.ready)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_4 = (__pyx_v__dict != Py_None);
  __pyx_t_5 = (__pyx_t_4 != 0);
  if (__pyx_t_5) {

    /* "(tree fragment)":6
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v__dict);
    __pyx_t_2 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "(tree fragment)":7
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.file is not None or self.ready is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":5
 *     state = (self.fhd, self.file, self.mask, self.ready)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":9
 *         use_setstate = True
 *     else:
 *         use_setstate = self.file is not None or self.ready is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_FdEvent, (type(self), 0x183067a, None), state
 */
  /*else*/ {
    __pyx_t_4 = (__pyx_v_self->file != Py_None);
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (__pyx_v_self->ready != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    __pyx_t_5 = __pyx_t_4;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_5;
  }
  __pyx_L3:;

  /* "(tree fragment)":10
 *     else:
 *         use_setstate = self.file is not None or self.ready is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_FdEvent, (type(self), 0x183067a, None), state
 *     else:
 */
  __pyx_t_5 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_5) {

    /* "(tree fragment)":11
 *         use_setstate = self.file is not None or self.ready is not None
 *     if use_setstate:
 *         return __pyx_unpickle_FdEvent, (type(self), 0x183067a, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_FdEvent, (type(self), 0x183067a, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_FdEvent); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_25364090);
    __Pyx_GIVEREF(__pyx_int_25364090);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_25364090);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_3, 2, Py_None);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_3);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state);
    __pyx_t_2 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":10
 *     else:
 *         use_setstate = self.file is not None or self.ready is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_FdEvent, (type(self), 0x183067a, None), state
 *     else:
 */
  }

  /* "(tree fragment)":13
 *         return __pyx_unpickle_FdEvent, (type(self), 0x183067a, None), state
 *     else:
 *         return __pyx_unpickle_FdEvent, (type(self), 0x183067a, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_FdEvent__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_pyx_unpickle_FdEvent); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_25364090);
    __Pyx_GIVEREF(__pyx_int_25364090);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_25364090);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_state);
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3);
    __pyx_t_1 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef bint use_setstate
 *     state = (self.fhd, self.file, self.mask, self.ready)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("pycx4.pycda.FdEvent.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":14
 *     else:
 *         return __pyx_unpickle_FdEvent, (type(self), 0x183067a, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_FdEvent__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_7FdEvent_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_7FdEvent_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_7FdEvent_6__setstate_cython__(((struct __pyx_obj_5pycx4_5pycda_FdEvent *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_7FdEvent_6__setstate_cython__(struct __pyx_obj_5pycx4_5pycda_FdEvent *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":15
 *         return __pyx_unpickle_FdEvent, (type(self), 0x183067a, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_FdEvent__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 15, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_5pycx4_5pycda___pyx_unpickle_FdEvent__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":14
 *     else:
 *         return __pyx_unpickle_FdEvent, (type(self), 0x183067a, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_FdEvent__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.FdEvent.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/cxdtype.pxi":40
 * # not to effective, but not a bottleneck
 * cdef:
 *     aval_value(CxAnyVal_t *aval, cxdtype_t dtype):             # <<<<<<<<<<<<<<
 *         if dtype == CXDTYPE_DOUBLE: return aval.f64
 *         elif dtype == CXDTYPE_INT32:  return aval.i32
 */

static PyObject *__pyx_f_5pycx4_5pycda_aval_value(CxAnyVal_t *__pyx_v_aval, cxdtype_t __pyx_v_dtype) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("aval_value", 0);

  /* "pycx4/cxdtype.pxi":41
 * cdef:
 *     aval_value(CxAnyVal_t *aval, cxdtype_t dtype):
 *         if dtype == CXDTYPE_DOUBLE: return aval.f64             # <<<<<<<<<<<<<<
 *         elif dtype == CXDTYPE_INT32:  return aval.i32
 *         elif dtype == CXDTYPE_INT8:   return aval.i8
 */
  switch (__pyx_v_dtype) {
    case CXDTYPE_DOUBLE:
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = PyFloat_FromDouble(__pyx_v_aval->f64); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 41, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;
    break;
    case CXDTYPE_INT32:

    /* "pycx4/cxdtype.pxi":42
 *     aval_value(CxAnyVal_t *aval, cxdtype_t dtype):
 *         if dtype == CXDTYPE_DOUBLE: return aval.f64
 *         elif dtype == CXDTYPE_INT32:  return aval.i32             # <<<<<<<<<<<<<<
 *         elif dtype == CXDTYPE_INT8:   return aval.i8
 *         elif dtype == CXDTYPE_INT16:  return aval.i16
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_int32(__pyx_v_aval->i32); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 42, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;
    break;
    case CXDTYPE_INT8:

    /* "pycx4/cxdtype.pxi":43
 *         if dtype == CXDTYPE_DOUBLE: return aval.f64
 *         elif dtype == CXDTYPE_INT32:  return aval.i32
 *         elif dtype == CXDTYPE_INT8:   return aval.i8             # <<<<<<<<<<<<<<
 *         elif dtype == CXDTYPE_INT16:  return aval.i16
 *         elif dtype == CXDTYPE_INT64:  return aval.i64
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_int8(__pyx_v_aval->i8); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 43, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;
    break;
    case CXDTYPE_INT16:

    /* "pycx4/cxdtype.pxi":44
 *         elif dtype == CXDTYPE_INT32:  return aval.i32
 *         elif dtype == CXDTYPE_INT8:   return aval.i8
 *         elif dtype == CXDTYPE_INT16:  return aval.i16             # <<<<<<<<<<<<<<
 *         elif dtype == CXDTYPE_INT64:  return aval.i64
 *         elif dtype == CXDTYPE_UINT8:  return aval.u8
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_int16(__pyx_v_aval->i16); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 44, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;
    break;
    case CXDTYPE_INT64:

    /* "pycx4/cxdtype.pxi":45
 *         elif dtype == CXDTYPE_INT8:   return aval.i8
 *         elif dtype == CXDTYPE_INT16:  return aval.i16
 *         elif dtype == CXDTYPE_INT64:  return aval.i64             # <<<<<<<<<<<<<<
 *         elif dtype == CXDTYPE_UINT8:  return aval.u8
 *         elif dtype == CXDTYPE_UINT16: return aval.u16
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_int64(__pyx_v_aval->i64); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 45, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;
    break;
    case CXDTYPE_UINT8:

    /* "pycx4/cxdtype.pxi":46
 *         elif dtype == CXDTYPE_INT16:  return aval.i16
 *         elif dtype == CXDTYPE_INT64:  return aval.i64
 *         elif dtype == CXDTYPE_UINT8:  return aval.u8             # <<<<<<<<<<<<<<
 *         elif dtype == CXDTYPE_UINT16: return aval.u16
 *         elif dtype == CXDTYPE_UINT32: return aval.u32
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_uint8(__pyx_v_aval->u8); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 46, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;
    break;
    case CXDTYPE_UINT16:

    /* "pycx4/cxdtype.pxi":47
 *         elif dtype == CXDTYPE_INT64:  return aval.i64
 *         elif dtype == CXDTYPE_UINT8:  return aval.u8
 *         elif dtype == CXDTYPE_UINT16: return aval.u16             # <<<<<<<<<<<<<<
 *         elif dtype == CXDTYPE_UINT32: return aval.u32
 *         elif dtype == CXDTYPE_UINT64: return aval.u64
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_uint16(__pyx_v_aval->u16); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 47, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;
    break;
    case CXDTYPE_UINT32:

    /* "pycx4/cxdtype.pxi":48
 *         elif dtype == CXDTYPE_UINT8:  return aval.u8
 *         elif dtype == CXDTYPE_UINT16: return aval.u16
 *         elif dtype == CXDTYPE_UINT32: return aval.u32             # <<<<<<<<<<<<<<
 *         elif dtype == CXDTYPE_UINT64: return aval.u64
 *         elif dtype == CXDTYPE_SINGLE: return aval.f32
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_uint32(__pyx_v_aval->u32); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 48, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;
    break;
    case CXDTYPE_UINT64:

    /* "pycx4/cxdtype.pxi":49
 *         elif dtype == CXDTYPE_UINT16: return aval.u16
 *         elif dtype == CXDTYPE_UINT32: return aval.u32
 *         elif dtype == CXDTYPE_UINT64: return aval.u64             # <<<<<<<<<<<<<<
 *         elif dtype == CXDTYPE_SINGLE: return aval.f32
 *         elif dtype == CXDTYPE_TEXT:   return aval.c8
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_uint64(__pyx_v_aval->u64); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 49, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;
    break;
    case CXDTYPE_SINGLE:

    /* "pycx4/cxdtype.pxi":50
 *         elif dtype == CXDTYPE_UINT32: return aval.u32
 *         elif dtype == CXDTYPE_UINT64: return aval.u64
 *         elif dtype == CXDTYPE_SINGLE: return aval.f32             # <<<<<<<<<<<<<<
 *         elif dtype == CXDTYPE_TEXT:   return aval.c8
 *         elif dtype == CXDTYPE_UCTEXT: return aval.c32
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = PyFloat_FromDouble(__pyx_v_aval->f32); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 50, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;
    break;
    case CXDTYPE_TEXT:

    /* "pycx4/cxdtype.pxi":51
 *         elif dtype == CXDTYPE_UINT64: return aval.u64
 *         elif dtype == CXDTYPE_SINGLE: return aval.f32
 *         elif dtype == CXDTYPE_TEXT:   return aval.c8             # <<<<<<<<<<<<<<
 *         elif dtype == CXDTYPE_UCTEXT: return aval.c32
 *         return None
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_char8(__pyx_v_aval->c8); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 51, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;
    break;
    case CXDTYPE_UCTEXT:

    /* "pycx4/cxdtype.pxi":52
 *         elif dtype == CXDTYPE_SINGLE: return aval.f32
 *         elif dtype == CXDTYPE_TEXT:   return aval.c8
 *         elif dtype == CXDTYPE_UCTEXT: return aval.c32             # <<<<<<<<<<<<<<
 *         return None
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_uint32(__pyx_v_aval->c32); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 52, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;
    break;
    default: break;
  }

  /* "pycx4/cxdtype.pxi":53
 *         elif dtype == CXDTYPE_TEXT:   return aval.c8
 *         elif dtype == CXDTYPE_UCTEXT: return aval.c32
 *         return None             # <<<<<<<<<<<<<<
 * 
 *     void aval_setvalue(value, CxAnyVal_t *aval, cxdtype_t dtype):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "pycx4/cxdtype.pxi":40
 * # not to effective, but not a bottleneck
 * cdef:
 *     aval_value(CxAnyVal_t *aval, cxdtype_t dtype):             # <<<<<<<<<<<<<<
 *         if dtype == CXDTYPE_DOUBLE: return aval.f64
 *         elif dtype == CXDTYPE_INT32:  return aval.i32
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.aval_value", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/cxdtype.pxi":55
 *         return None
 * 
 *     void aval_setvalue(value, CxAnyVal_t *aval, cxdtype_t dtype):             # <<<<<<<<<<<<<<
 *         if dtype == CXDTYPE_DOUBLE: aval.f64 = value
 *         elif dtype == CXDTYPE_INT32:  aval.i32 = value
 */

static void __pyx_f_5pycx4_5pycda_aval_setvalue(PyObject *__pyx_v_value, CxAnyVal_t *__pyx_v_aval, cxdtype_t __pyx_v_dtype) {
  __Pyx_RefNannyDeclarations
  float64 __pyx_t_1;
  int32 __pyx_t_2;
  int8 __pyx_t_3;
  int16 __pyx_t_4;
  int64 __pyx_t_5;
  uint8 __pyx_t_6;
  uint16 __pyx_t_7;
  uint32 __pyx_t_8;
  uint64 __pyx_t_9;
  float32 __pyx_t_10;
  char8 __pyx_t_11;
  char32 __pyx_t_12;
  __Pyx_RefNannySetupContext("aval_setvalue", 0);

  /* "pycx4/cxdtype.pxi":56
 * 
 *     void aval_setvalue(value, CxAnyVal_t *aval, cxdtype_t dtype):
 *         if dtype == CXDTYPE_DOUBLE: aval.f64 = value             # <<<<<<<<<<<<<<
 *         elif dtype == CXDTYPE_INT32:  aval.i32 = value
 *         elif dtype == CXDTYPE_INT8:   aval.i8 = value
 */
  switch (__pyx_v_dtype) {
    case CXDTYPE_DOUBLE:
    __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == ((float64)-1)) && PyErr_Occurred())) __PYX_ERR(6, 56, __pyx_L1_error)
    __pyx_v_aval->f64 = __pyx_t_1;
    break;
    case CXDTYPE_INT32:

    /* "pycx4/cxdtype.pxi":57
 *     void aval_setvalue(value, CxAnyVal_t *aval, cxdtype_t dtype):
 *         if dtype == CXDTYPE_DOUBLE: aval.f64 = value
 *         elif dtype == CXDTYPE_INT32:  aval.i32 = value             # <<<<<<<<<<<<<<
 *         elif dtype == CXDTYPE_INT8:   aval.i8 = value
 *         elif dtype == CXDTYPE_INT16:  aval.i16 = value
 */
    __pyx_t_2 = __Pyx_PyInt_As_int32(__pyx_v_value); if (unlikely((__pyx_t_2 == ((int32)-1)) && PyErr_Occurred())) __PYX_ERR(6, 57, __pyx_L1_error)
    __pyx_v_aval->i32 = __pyx_t_2;
    break;
    case CXDTYPE_INT8:

    /* "pycx4/cxdtype.pxi":58
 *         if dtype == CXDTYPE_DOUBLE: aval.f64 = value
 *         elif dtype == CXDTYPE_INT32:  aval.i32 = value
 *         elif dtype == CXDTYPE_INT8:   aval.i8 = value             # <<<<<<<<<<<<<<
 *         elif dtype == CXDTYPE_INT16:  aval.i16 = value
 *         elif dtype == CXDTYPE_INT64:  aval.i64 = value
 */
    __pyx_t_3 = __Pyx_PyInt_As_int8(__pyx_v_value); if (unlikely((__pyx_t_3 == ((int8)-1)) && PyErr_Occurred())) __PYX_ERR(6, 58, __pyx_L1_error)
    __pyx_v_aval->i8 = __pyx_t_3;
    break;
    case CXDTYPE_INT16:

    /* "pycx4/cxdtype.pxi":59
 *         elif dtype == CXDTYPE_INT32:  aval.i32 = value
 *         elif dtype == CXDTYPE_INT8:   aval.i8 = value
 *         elif dtype == CXDTYPE_INT16:  aval.i16 = value             # <<<<<<<<<<<<<<
 *         elif dtype == CXDTYPE_INT64:  aval.i64 = value
 *         elif dtype == CXDTYPE_UINT8:  aval.u8 = value
 */
    __pyx_t_4 = __Pyx_PyInt_As_int16(__pyx_v_value); if (unlikely((__pyx_t_4 == ((int16)-1)) && PyErr_Occurred())) __PYX_ERR(6, 59, __pyx_L1_error)
    __pyx_v_aval->i16 = __pyx_t_4;
    break;
    case CXDTYPE_INT64:

    /* "pycx4/cxdtype.pxi":60
 *         elif dtype == CXDTYPE_INT8:   aval.i8 = value
 *         elif dtype == CXDTYPE_INT16:  aval.i16 = value
 *         elif dtype == CXDTYPE_INT64:  aval.i64 = value             # <<<<<<<<<<<<<<
 *         elif dtype == CXDTYPE_UINT8:  aval.u8 = value
 *         elif dtype == CXDTYPE_UINT16: aval.u16 = value
 */
    __pyx_t_5 = __Pyx_PyInt_As_int64(__pyx_v_value); if (unlikely((__pyx_t_5 == ((int64)-1)) && PyErr_Occurred())) __PYX_ERR(6, 60, __pyx_L1_error)
    __pyx_v_aval->i64 = __pyx_t_5;
    break;
    case CXDTYPE_UINT8:

    /* "pycx4/cxdtype.pxi":61
 *         elif dtype == CXDTYPE_INT16:  aval.i16 = value
 *         elif dtype == CXDTYPE_INT64:  aval.i64 = value
 *         elif dtype == CXDTYPE_UINT8:  aval.u8 = value             # <<<<<<<<<<<<<<
 *         elif dtype == CXDTYPE_UINT16: aval.u16 = value
 *         elif dtype == CXDTYPE_UINT32: aval.u32 = value
 */
    __pyx_t_6 = __Pyx_PyInt_As_uint8(__pyx_v_value); if (unlikely((__pyx_t_6 == ((uint8)-1)) && PyErr_Occurred())) __PYX_ERR(6, 61, __pyx_L1_error)
    __pyx_v_aval->u8 = __pyx_t_6;
    break;
    case CXDTYPE_UINT16:

    /* "pycx4/cxdtype.pxi":62
 *         elif dtype == CXDTYPE_INT64:  aval.i64 = value
 *         elif dtype == CXDTYPE_UINT8:  aval.u8 = value
 *         elif dtype == CXDTYPE_UINT16: aval.u16 = value             # <<<<<<<<<<<<<<
 *         elif dtype == CXDTYPE_UINT32: aval.u32 = value
 *         elif dtype == CXDTYPE_UINT64: aval.u64 = value
 */
    __pyx_t_7 = __Pyx_PyInt_As_uint16(__pyx_v_value); if (unlikely((__pyx_t_7 == ((uint16)-1)) && PyErr_Occurred())) __PYX_ERR(6, 62, __pyx_L1_error)
    __pyx_v_aval->u16 = __pyx_t_7;
    break;
    case CXDTYPE_UINT32:

    /* "pycx4/cxdtype.pxi":63
 *         elif dtype == CXDTYPE_UINT8:  aval.u8 = value
 *         elif dtype == CXDTYPE_UINT16: aval.u16 = value
 *         elif dtype == CXDTYPE_UINT32: aval.u32 = value             # <<<<<<<<<<<<<<
 *         elif dtype == CXDTYPE_UINT64: aval.u64 = value
 *         elif dtype == CXDTYPE_SINGLE: aval.f32 = value
 */
    __pyx_t_8 = __Pyx_PyInt_As_uint32(__pyx_v_value); if (unlikely((__pyx_t_8 == ((uint32)-1)) && PyErr_Occurred())) __PYX_ERR(6, 63, __pyx_L1_error)
    __pyx_v_aval->u32 = __pyx_t_8;
    break;
    case CXDTYPE_UINT64:

    /* "pycx4/cxdtype.pxi":64
 *         elif dtype == CXDTYPE_UINT16: aval.u16 = value
 *         elif dtype == CXDTYPE_UINT32: aval.u32 = value
 *         elif dtype == CXDTYPE_UINT64: aval.u64 = value             # <<<<<<<<<<<<<<
 *         elif dtype == CXDTYPE_SINGLE: aval.f32 = value
 *         elif dtype == CXDTYPE_TEXT:   aval.c8 = value
 */
    __pyx_t_9 = __Pyx_PyInt_As_uint64(__pyx_v_value); if (unlikely((__pyx_t_9 == ((uint64)-1)) && PyErr_Occurred())) __PYX_ERR(6, 64, __pyx_L1_error)
    __pyx_v_aval->u64 = __pyx_t_9;
    break;
    case CXDTYPE_SINGLE:

    /* "pycx4/cxdtype.pxi":65
 *         elif dtype == CXDTYPE_UINT32: aval.u32 = value
 *         elif dtype == CXDTYPE_UINT64: aval.u64 = value
 *         elif dtype == CXDTYPE_SINGLE: aval.f32 = value             # <<<<<<<<<<<<<<
 *         elif dtype == CXDTYPE_TEXT:   aval.c8 = value
 *         elif dtype == CXDTYPE_UCTEXT: aval.c32 = value
 */
    __pyx_t_10 = __pyx_PyFloat_AsFloat(__pyx_v_value); if (unlikely((__pyx_t_10 == ((float32)-1)) && PyErr_Occurred())) __PYX_ERR(6, 65, __pyx_L1_error)
    __pyx_v_aval->f32 = __pyx_t_10;
    break;
    case CXDTYPE_TEXT:

    /* "pycx4/cxdtype.pxi":66
 *         elif dtype == CXDTYPE_UINT64: aval.u64 = value
 *         elif dtype == CXDTYPE_SINGLE: aval.f32 = value
 *         elif dtype == CXDTYPE_TEXT:   aval.c8 = value             # <<<<<<<<<<<<<<
 *         elif dtype == CXDTYPE_UCTEXT: aval.c32 = value
 * 
 */
    __pyx_t_11 = __Pyx_PyInt_As_char8(__pyx_v_value); if (unlikely((__pyx_t_11 == ((char8)-1)) && PyErr_Occurred())) __PYX_ERR(6, 66, __pyx_L1_error)
    __pyx_v_aval->c8 = __pyx_t_11;
    break;
    case CXDTYPE_UCTEXT:

    /* "pycx4/cxdtype.pxi":67
 *         elif dtype == CXDTYPE_SINGLE: aval.f32 = value
 *         elif dtype == CXDTYPE_TEXT:   aval.c8 = value
 *         elif dtype == CXDTYPE_UCTEXT: aval.c32 = value             # <<<<<<<<<<<<<<
 * 
 *     # dtype conversion numpy <--> CX
 */
    __pyx_t_12 = __Pyx_PyInt_As_uint32(__pyx_v_value); if (unlikely((__pyx_t_12 == ((char32)-1)) && PyErr_Occurred())) __PYX_ERR(6, 67, __pyx_L1_error)
    __pyx_v_aval->c32 = __pyx_t_12;
    break;
    default: break;
  }

  /* "pycx4/cxdtype.pxi":55
 *         return None
 * 
 *     void aval_setvalue(value, CxAnyVal_t *aval, cxdtype_t dtype):             # <<<<<<<<<<<<<<
 *         if dtype == CXDTYPE_DOUBLE: aval.f64 = value
 *         elif dtype == CXDTYPE_INT32:  aval.i32 = value
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("pycx4.pycda.aval_setvalue", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/cxdtype.pxi":72
 *     # there are no text or unicode text.
 *     # Cause text and unicode text can't be mixed with numbers in a good way
 *     object cxdtype2np(cxdtype_t dtype):             # <<<<<<<<<<<<<<
 *         if dtype == CXDTYPE_DOUBLE: return np.double
 *         elif dtype == CXDTYPE_INT32:  return np.int32
 */

static PyObject *__pyx_f_5pycx4_5pycda_cxdtype2np(cxdtype_t __pyx_v_dtype) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("cxdtype2np", 0);

  /* "pycx4/cxdtype.pxi":73
 *     # Cause text and unicode text can't be mixed with numbers in a good way
 *     object cxdtype2np(cxdtype_t dtype):
 *         if dtype == CXDTYPE_DOUBLE: return np.double             # <<<<<<<<<<<<<<
 *         elif dtype == CXDTYPE_INT32:  return np.int32
 *         elif dtype == CXDTYPE_INT8:   return np.int8
 */
  switch (__pyx_v_dtype) {
    case CXDTYPE_DOUBLE:
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 73, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_double); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 73, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
    break;
    case CXDTYPE_INT32:

    /* "pycx4/cxdtype.pxi":74
 *     object cxdtype2np(cxdtype_t dtype):
 *         if dtype == CXDTYPE_DOUBLE: return np.double
 *         elif dtype == CXDTYPE_INT32:  return np.int32             # <<<<<<<<<<<<<<
 *         elif dtype == CXDTYPE_INT8:   return np.int8
 *         elif dtype == CXDTYPE_INT16:  return np.int16
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 74, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_int32); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 74, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;
    break;
    case CXDTYPE_INT8:

    /* "pycx4/cxdtype.pxi":75
 *         if dtype == CXDTYPE_DOUBLE: return np.double
 *         elif dtype == CXDTYPE_INT32:  return np.int32
 *         elif dtype == CXDTYPE_INT8:   return np.int8             # <<<<<<<<<<<<<<
 *         elif dtype == CXDTYPE_INT16:  return np.int16
 *         elif dtype == CXDTYPE_INT64:  return np.int64
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 75, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_int8); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 75, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
    break;
    case CXDTYPE_INT16:

    /* "pycx4/cxdtype.pxi":76
 *         elif dtype == CXDTYPE_INT32:  return np.int32
 *         elif dtype == CXDTYPE_INT8:   return np.int8
 *         elif dtype == CXDTYPE_INT16:  return np.int16             # <<<<<<<<<<<<<<
 *         elif dtype == CXDTYPE_INT64:  return np.int64
 *         elif dtype == CXDTYPE_UINT8:  return np.uint8
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 76, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_int16); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 76, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;
    break;
    case CXDTYPE_INT64:

    /* "pycx4/cxdtype.pxi":77
 *         elif dtype == CXDTYPE_INT8:   return np.int8
 *         elif dtype == CXDTYPE_INT16:  return np.int16
 *         elif dtype == CXDTYPE_INT64:  return np.int64             # <<<<<<<<<<<<<<
 *         elif dtype == CXDTYPE_UINT8:  return np.uint8
 *         elif dtype == CXDTYPE_UINT16: return np.uint16
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 77, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_int64); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 77, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
    break;
    case CXDTYPE_UINT8:

    /* "pycx4/cxdtype.pxi":78
 *         elif dtype == CXDTYPE_INT16:  return np.int16
 *         elif dtype == CXDTYPE_INT64:  return np.int64
 *         elif dtype == CXDTYPE_UINT8:  return np.uint8             # <<<<<<<<<<<<<<
 *         elif dtype == CXDTYPE_UINT16: return np.uint16
 *         elif dtype == CXDTYPE_UINT32: return np.uint32
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 78, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_uint8); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 78, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;
    break;
    case CXDTYPE_UINT16:

    /* "pycx4/cxdtype.pxi":79
 *         elif dtype == CXDTYPE_INT64:  return np.int64
 *         elif dtype == CXDTYPE_UINT8:  return np.uint8
 *         elif dtype == CXDTYPE_UINT16: return np.uint16             # <<<<<<<<<<<<<<
 *         elif dtype == CXDTYPE_UINT32: return np.uint32
 *         elif dtype == CXDTYPE_UINT64: return np.uint64
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 79, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_uint16); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 79, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
    break;
    case CXDTYPE_UINT32:

    /* "pycx4/cxdtype.pxi":80
 *         elif dtype == CXDTYPE_UINT8:  return np.uint8
 *         elif dtype == CXDTYPE_UINT16: return np.uint16
 *         elif dtype == CXDTYPE_UINT32: return np.uint32             # <<<<<<<<<<<<<<
 *         elif dtype == CXDTYPE_UINT64: return np.uint64
 *         elif dtype == CXDTYPE_SINGLE: return np.single
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 80, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_uint32); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 80, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;
    break;
    case CXDTYPE_UINT64:

    /* "pycx4/cxdtype.pxi":81
 *         elif dtype == CXDTYPE_UINT16: return np.uint16
 *         elif dtype == CXDTYPE_UINT32: return np.uint32
 *         elif dtype == CXDTYPE_UINT64: return np.uint64             # <<<<<<<<<<<<<<
 *         elif dtype == CXDTYPE_SINGLE: return np.single
 *         return None
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 81, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_uint64); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 81, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
    break;
    case CXDTYPE_SINGLE:

    /* "pycx4/cxdtype.pxi":82
 *         elif dtype == CXDTYPE_UINT32: return np.uint32
 *         elif dtype == CXDTYPE_UINT64: return np.uint64
 *         elif dtype == CXDTYPE_SINGLE: return np.single             # <<<<<<<<<<<<<<
 *         return None
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 82, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_single); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 82, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;
    break;
    default: break;
  }

  /* "pycx4/cxdtype.pxi":83
 *         elif dtype == CXDTYPE_UINT64: return np.uint64
 *         elif dtype == CXDTYPE_SINGLE: return np.single
 *         return None             # <<<<<<<<<<<<<<
 * 
 *     cxdtype_t np2cxdtype(object dtype):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "pycx4/cxdtype.pxi":72
 *     # there are no text or unicode text.
 *     # Cause text and unicode text can't be mixed with numbers in a good way
 *     object cxdtype2np(cxdtype_t dtype):             # <<<<<<<<<<<<<<
 *         if dtype == CXDTYPE_DOUBLE: return np.double
 *         elif dtype == CXDTYPE_INT32:  return np.int32
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("pycx4.pycda.cxdtype2np", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/cxdtype.pxi":85
 *         return None
 * 
 *     cxdtype_t np2cxdtype(object dtype):             # <<<<<<<<<<<<<<
 *         if dtype == np.double:      return CXDTYPE_DOUBLE
 *         elif dtype == np.int32:       return CXDTYPE_INT32
 */

static cxdtype_t __pyx_f_5pycx4_5pycda_np2cxdtype(PyObject *__pyx_v_dtype) {
  cxdtype_t __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  __Pyx_RefNannySetupContext("np2cxdtype", 0);

  /* "pycx4/cxdtype.pxi":86
 * 
 *     cxdtype_t np2cxdtype(object dtype):
 *         if dtype == np.double:      return CXDTYPE_DOUBLE             # <<<<<<<<<<<<<<
 *         elif dtype == np.int32:       return CXDTYPE_INT32
 *         elif dtype == np.int8:        return CXDTYPE_INT8
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 86, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_double); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 86, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_dtype, __pyx_t_2, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 86, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(6, 86, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_3) {
    __pyx_r = CXDTYPE_DOUBLE;
    goto __pyx_L0;
  }

  /* "pycx4/cxdtype.pxi":87
 *     cxdtype_t np2cxdtype(object dtype):
 *         if dtype == np.double:      return CXDTYPE_DOUBLE
 *         elif dtype == np.int32:       return CXDTYPE_INT32             # <<<<<<<<<<<<<<
 *         elif dtype == np.int8:        return CXDTYPE_INT8
 *         elif dtype == np.int16:       return CXDTYPE_INT16
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 87, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_int32); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 87, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_dtype, __pyx_t_2, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 87, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(6, 87, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_3) {
    __pyx_r = CXDTYPE_INT32;
    goto __pyx_L0;
  }

  /* "pycx4/cxdtype.pxi":88
 *         if dtype == np.double:      return CXDTYPE_DOUBLE
 *         elif dtype == np.int32:       return CXDTYPE_INT32
 *         elif dtype == np.int8:        return CXDTYPE_INT8             # <<<<<<<<<<<<<<
 *         elif dtype == np.int16:       return CXDTYPE_INT16
 *         elif dtype == np.int64:       return CXDTYPE_INT64
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 88, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_int8); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 88, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_dtype, __pyx_t_2, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 88, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(6, 88, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_3) {
    __pyx_r = CXDTYPE_INT8;
    goto __pyx_L0;
  }

  /* "pycx4/cxdtype.pxi":89
 *         elif dtype == np.int32:       return CXDTYPE_INT32
 *         elif dtype == np.int8:        return CXDTYPE_INT8
 *         elif dtype == np.int16:       return CXDTYPE_INT16             # <<<<<<<<<<<<<<
 *         elif dtype == np.int64:       return CXDTYPE_INT64
 *         elif dtype == np.uint8:       return CXDTYPE_UINT8
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 89, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_int16); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 89, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_dtype, __pyx_t_2, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 89, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(6, 89, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_3) {
    __pyx_r = CXDTYPE_INT16;
    goto __pyx_L0;
  }

  /* "pycx4/cxdtype.pxi":90
 *         elif dtype == np.int8:        return CXDTYPE_INT8
 *         elif dtype == np.int16:       return CXDTYPE_INT16
 *         elif dtype == np.int64:       return CXDTYPE_INT64             # <<<<<<<<<<<<<<
 *         elif dtype == np.uint8:       return CXDTYPE_UINT8
 *         elif dtype == np.uint16:      return CXDTYPE_UINT16
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 90, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_int64); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 90, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_dtype, __pyx_t_2, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 90, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(6, 90, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_3) {
    __pyx_r = CXDTYPE_INT64;
    goto __pyx_L0;
  }

  /* "pycx4/cxdtype.pxi":91
 *         elif dtype == np.int16:       return CXDTYPE_INT16
 *         elif dtype == np.int64:       return CXDTYPE_INT64
 *         elif dtype == np.uint8:       return CXDTYPE_UINT8             # <<<<<<<<<<<<<<
 *         elif dtype == np.uint16:      return CXDTYPE_UINT16
 *         elif dtype == np.uint32:      return CXDTYPE_UINT32
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 91, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_uint8); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 91, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_dtype, __pyx_t_2, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 91, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(6, 91, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_3) {
    __pyx_r = CXDTYPE_UINT8;
    goto __pyx_L0;
  }

  /* "pycx4/cxdtype.pxi":92
 *         elif dtype == np.int64:       return CXDTYPE_INT64
 *         elif dtype == np.uint8:       return CXDTYPE_UINT8
 *         elif dtype == np.uint16:      return CXDTYPE_UINT16             # <<<<<<<<<<<<<<
 *         elif dtype == np.uint32:      return CXDTYPE_UINT32
 *         elif dtype == np.uint64:      return CXDTYPE_UINT64
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 92, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_uint16); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 92, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_dtype, __pyx_t_2, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 92, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(6, 92, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_3) {
    __pyx_r = CXDTYPE_UINT16;
    goto __pyx_L0;
  }

  /* "pycx4/cxdtype.pxi":93
 *         elif dtype == np.uint8:       return CXDTYPE_UINT8
 *         elif dtype == np.uint16:      return CXDTYPE_UINT16
 *         elif dtype == np.uint32:      return CXDTYPE_UINT32             # <<<<<<<<<<<<<<
 *         elif dtype == np.uint64:      return CXDTYPE_UINT64
 *         elif dtype == np.single:      return CXDTYPE_SINGLE
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 93, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_uint32); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 93, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_dtype, __pyx_t_2, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 93, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(6, 93, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_3) {
    __pyx_r = CXDTYPE_UINT32;
    goto __pyx_L0;
  }

  /* "pycx4/cxdtype.pxi":94
 *         elif dtype == np.uint16:      return CXDTYPE_UINT16
 *         elif dtype == np.uint32:      return CXDTYPE_UINT32
 *         elif dtype == np.uint64:      return CXDTYPE_UINT64             # <<<<<<<<<<<<<<
 *         elif dtype == np.single:      return CXDTYPE_SINGLE
 *         return 0
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 94, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_uint64); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 94, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_dtype, __pyx_t_2, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 94, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(6, 94, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_3) {
    __pyx_r = CXDTYPE_UINT64;
    goto __pyx_L0;
  }

  /* "pycx4/cxdtype.pxi":95
 *         elif dtype == np.uint32:      return CXDTYPE_UINT32
 *         elif dtype == np.uint64:      return CXDTYPE_UINT64
 *         elif dtype == np.single:      return CXDTYPE_SINGLE             # <<<<<<<<<<<<<<
 *         return 0
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 95, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_single); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 95, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_dtype, __pyx_t_2, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 95, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(6, 95, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_3) {
    __pyx_r = CXDTYPE_SINGLE;
    goto __pyx_L0;
  }

  /* "pycx4/cxdtype.pxi":96
 *         elif dtype == np.uint64:      return CXDTYPE_UINT64
 *         elif dtype == np.single:      return CXDTYPE_SINGLE
 *         return 0             # <<<<<<<<<<<<<<
 * 
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "pycx4/cxdtype.pxi":85
 *         return None
 * 
 *     cxdtype_t np2cxdtype(object dtype):             # <<<<<<<<<<<<<<
 *         if dtype == np.double:      return CXDTYPE_DOUBLE
 *         elif dtype == np.int32:       return CXDTYPE_INT32
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_WriteUnraisable("pycx4.pycda.np2cxdtype", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/rflags.pxi":2
 * 
 * cdef list rflags_text(rflags_t rflags):             # <<<<<<<<<<<<<<
 *     ret = []
 *     if (rflags & CXRF_CAMAC_NO_X) > 0:
 */

static PyObject *__pyx_f_5pycx4_5pycda_rflags_text(rflags_t __pyx_v_rflags) {
  PyObject *__pyx_v_ret = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  __Pyx_RefNannySetupContext("rflags_text", 0);

  /* "pycx4/rflags.pxi":3
 * 
 * cdef list rflags_text(rflags_t rflags):
 *     ret = []             # <<<<<<<<<<<<<<
 *     if (rflags & CXRF_CAMAC_NO_X) > 0:
 *         ret.append('No X from CAMAC device')
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_ret = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pycx4/rflags.pxi":4
 * cdef list rflags_text(rflags_t rflags):
 *     ret = []
 *     if (rflags & CXRF_CAMAC_NO_X) > 0:             # <<<<<<<<<<<<<<
 *         ret.append('No X from CAMAC device')
 * 
 */
  __pyx_t_2 = (((__pyx_v_rflags & CXRF_CAMAC_NO_X) > 0) != 0);
  if (__pyx_t_2) {

    /* "pycx4/rflags.pxi":5
 *     ret = []
 *     if (rflags & CXRF_CAMAC_NO_X) > 0:
 *         ret.append('No X from CAMAC device')             # <<<<<<<<<<<<<<
 * 
 *     if (rflags & CXRF_CAMAC_NO_Q) > 0:
 */
    __pyx_t_3 = __Pyx_PyList_Append(__pyx_v_ret, __pyx_kp_u_No_X_from_CAMAC_device); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(7, 5, __pyx_L1_error)

    /* "pycx4/rflags.pxi":4
 * cdef list rflags_text(rflags_t rflags):
 *     ret = []
 *     if (rflags & CXRF_CAMAC_NO_X) > 0:             # <<<<<<<<<<<<<<
 *         ret.append('No X from CAMAC device')
 * 
 */
  }

  /* "pycx4/rflags.pxi":7
 *         ret.append('No X from CAMAC device')
 * 
 *     if (rflags & CXRF_CAMAC_NO_Q) > 0:             # <<<<<<<<<<<<<<
 *         ret.append('No Q from CAMAC device')
 * 
 */
  __pyx_t_2 = (((__pyx_v_rflags & CXRF_CAMAC_NO_Q) > 0) != 0);
  if (__pyx_t_2) {

    /* "pycx4/rflags.pxi":8
 * 
 *     if (rflags & CXRF_CAMAC_NO_Q) > 0:
 *         ret.append('No Q from CAMAC device')             # <<<<<<<<<<<<<<
 * 
 *     if (rflags & CXRF_IO_TIMEOUT) > 0:
 */
    __pyx_t_3 = __Pyx_PyList_Append(__pyx_v_ret, __pyx_kp_u_No_Q_from_CAMAC_device); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(7, 8, __pyx_L1_error)

    /* "pycx4/rflags.pxi":7
 *         ret.append('No X from CAMAC device')
 * 
 *     if (rflags & CXRF_CAMAC_NO_Q) > 0:             # <<<<<<<<<<<<<<
 *         ret.append('No Q from CAMAC device')
 * 
 */
  }

  /* "pycx4/rflags.pxi":10
 *         ret.append('No Q from CAMAC device')
 * 
 *     if (rflags & CXRF_IO_TIMEOUT) > 0:             # <<<<<<<<<<<<<<
 *         ret.append('I/O timeout expired')
 * 
 */
  __pyx_t_2 = (((__pyx_v_rflags & CXRF_IO_TIMEOUT) > 0) != 0);
  if (__pyx_t_2) {

    /* "pycx4/rflags.pxi":11
 * 
 *     if (rflags & CXRF_IO_TIMEOUT) > 0:
 *         ret.append('I/O timeout expired')             # <<<<<<<<<<<<<<
 * 
 *     if (rflags & CXRF_REM_C_PROBL) > 0:
 */
    __pyx_t_3 = __Pyx_PyList_Append(__pyx_v_ret, __pyx_kp_u_I_O_timeout_expired); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(7, 11, __pyx_L1_error)

    /* "pycx4/rflags.pxi":10
 *         ret.append('No Q from CAMAC device')
 * 
 *     if (rflags & CXRF_IO_TIMEOUT) > 0:             # <<<<<<<<<<<<<<
 *         ret.append('I/O timeout expired')
 * 
 */
  }

  /* "pycx4/rflags.pxi":13
 *         ret.append('I/O timeout expired')
 * 
 *     if (rflags & CXRF_REM_C_PROBL) > 0:             # <<<<<<<<<<<<<<
 *         ret.append('Remote controller problem')
 * 
 */
  __pyx_t_2 = (((__pyx_v_rflags & CXRF_REM_C_PROBL) > 0) != 0);
  if (__pyx_t_2) {

    /* "pycx4/rflags.pxi":14
 * 
 *     if (rflags & CXRF_REM_C_PROBL) > 0:
 *         ret.append('Remote controller problem')             # <<<<<<<<<<<<<<
 * 
 *     if (rflags & CXRF_OVERLOAD) > 0:
 */
    __pyx_t_3 = __Pyx_PyList_Append(__pyx_v_ret, __pyx_kp_u_Remote_controller_problem); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(7, 14, __pyx_L1_error)

    /* "pycx4/rflags.pxi":13
 *         ret.append('I/O timeout expired')
 * 
 *     if (rflags & CXRF_REM_C_PROBL) > 0:             # <<<<<<<<<<<<<<
 *         ret.append('Remote controller problem')
 * 
 */
  }

  /* "pycx4/rflags.pxi":16
 *         ret.append('Remote controller problem')
 * 
 *     if (rflags & CXRF_OVERLOAD) > 0:             # <<<<<<<<<<<<<<
 *         ret.append('Input channel overload')
 * 
 */
  __pyx_t_2 = (((__pyx_v_rflags & CXRF_OVERLOAD) > 0) != 0);
  if (__pyx_t_2) {

    /* "pycx4/rflags.pxi":17
 * 
 *     if (rflags & CXRF_OVERLOAD) > 0:
 *         ret.append('Input channel overload')             # <<<<<<<<<<<<<<
 * 
 *     if (rflags & CXRF_UNSUPPORTED) > 0:
 */
    __pyx_t_3 = __Pyx_PyList_Append(__pyx_v_ret, __pyx_kp_u_Input_channel_overload); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(7, 17, __pyx_L1_error)

    /* "pycx4/rflags.pxi":16
 *         ret.append('Remote controller problem')
 * 
 *     if (rflags & CXRF_OVERLOAD) > 0:             # <<<<<<<<<<<<<<
 *         ret.append('Input channel overload')
 * 
 */
  }

  /* "pycx4/rflags.pxi":19
 *         ret.append('Input channel overload')
 * 
 *     if (rflags & CXRF_UNSUPPORTED) > 0:             # <<<<<<<<<<<<<<
 *         ret.append('Unsupported feature/channel')
 * 
 */
  __pyx_t_2 = (((__pyx_v_rflags & CXRF_UNSUPPORTED) > 0) != 0);
  if (__pyx_t_2) {

    /* "pycx4/rflags.pxi":20
 * 
 *     if (rflags & CXRF_UNSUPPORTED) > 0:
 *         ret.append('Unsupported feature/channel')             # <<<<<<<<<<<<<<
 * 
 *     if (rflags & CXRF_INVAL) > 0:
 */
    __pyx_t_3 = __Pyx_PyList_Append(__pyx_v_ret, __pyx_kp_u_Unsupported_feature_channel); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(7, 20, __pyx_L1_error)

    /* "pycx4/rflags.pxi":19
 *         ret.append('Input channel overload')
 * 
 *     if (rflags & CXRF_UNSUPPORTED) > 0:             # <<<<<<<<<<<<<<
 *         ret.append('Unsupported feature/channel')
 * 
 */
  }

  /* "pycx4/rflags.pxi":22
 *         ret.append('Unsupported feature/channel')
 * 
 *     if (rflags & CXRF_INVAL) > 0:             # <<<<<<<<<<<<<<
 *         ret.append('Invalid parameter')
 * 
 */
  __pyx_t_2 = (((__pyx_v_rflags & CXRF_INVAL) > 0) != 0);
  if (__pyx_t_2) {

    /* "pycx4/rflags.pxi":23
 * 
 *     if (rflags & CXRF_INVAL) > 0:
 *         ret.append('Invalid parameter')             # <<<<<<<<<<<<<<
 * 
 *     if (rflags & CXRF_WRONG_DEV) > 0:
 */
    __pyx_t_3 = __Pyx_PyList_Append(__pyx_v_ret, __pyx_kp_u_Invalid_parameter); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(7, 23, __pyx_L1_error)

    /* "pycx4/rflags.pxi":22
 *         ret.append('Unsupported feature/channel')
 * 
 *     if (rflags & CXRF_INVAL) > 0:             # <<<<<<<<<<<<<<
 *         ret.append('Invalid parameter')
 * 
 */
  }

  /* "pycx4/rflags.pxi":25
 *         ret.append('Invalid parameter')
 * 
 *     if (rflags & CXRF_WRONG_DEV) > 0:             # <<<<<<<<<<<<<<
 *         ret.append('Wrong device')
 * 
 */
  __pyx_t_2 = (((__pyx_v_rflags & CXRF_WRONG_DEV) > 0) != 0);
  if (__pyx_t_2) {

    /* "pycx4/rflags.pxi":26
 * 
 *     if (rflags & CXRF_WRONG_DEV) > 0:
 *         ret.append('Wrong device')             # <<<<<<<<<<<<<<
 * 
 *     if (rflags & CXRF_CFG_PROBL) > 0:
 */
    __pyx_t_3 = __Pyx_PyList_Append(__pyx_v_ret, __pyx_kp_u_Wrong_device); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(7, 26, __pyx_L1_error)

    /* "pycx4/rflags.pxi":25
 *         ret.append('Invalid parameter')
 * 
 *     if (rflags & CXRF_WRONG_DEV) > 0:             # <<<<<<<<<<<<<<
 *         ret.append('Wrong device')
 * 
 */
  }

  /* "pycx4/rflags.pxi":28
 *         ret.append('Wrong device')
 * 
 *     if (rflags & CXRF_CFG_PROBL) > 0:             # <<<<<<<<<<<<<<
 *         ret.append('Configuration problem')
 * 
 */
  __pyx_t_2 = (((__pyx_v_rflags & CXRF_CFG_PROBL) > 0) != 0);
  if (__pyx_t_2) {

    /* "pycx4/rflags.pxi":29
 * 
 *     if (rflags & CXRF_CFG_PROBL) > 0:
 *         ret.append('Configuration problem')             # <<<<<<<<<<<<<<
 * 
 *     if (rflags & CXRF_DRV_PROBL) > 0:
 */
    __pyx_t_3 = __Pyx_PyList_Append(__pyx_v_ret, __pyx_kp_u_Configuration_problem); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(7, 29, __pyx_L1_error)

    /* "pycx4/rflags.pxi":28
 *         ret.append('Wrong device')
 * 
 *     if (rflags & CXRF_CFG_PROBL) > 0:             # <<<<<<<<<<<<<<
 *         ret.append('Configuration problem')
 * 
 */
  }

  /* "pycx4/rflags.pxi":31
 *         ret.append('Configuration problem')
 * 
 *     if (rflags & CXRF_DRV_PROBL) > 0:             # <<<<<<<<<<<<<<
 *         ret.append('Driver internal problem')
 * 
 */
  __pyx_t_2 = (((__pyx_v_rflags & CXRF_DRV_PROBL) > 0) != 0);
  if (__pyx_t_2) {

    /* "pycx4/rflags.pxi":32
 * 
 *     if (rflags & CXRF_DRV_PROBL) > 0:
 *         ret.append('Driver internal problem')             # <<<<<<<<<<<<<<
 * 
 *     if (rflags & CXRF_NO_DRV) > 0:
 */
    __pyx_t_3 = __Pyx_PyList_Append(__pyx_v_ret, __pyx_kp_u_Driver_internal_problem); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(7, 32, __pyx_L1_error)

    /* "pycx4/rflags.pxi":31
 *         ret.append('Configuration problem')
 * 
 *     if (rflags & CXRF_DRV_PROBL) > 0:             # <<<<<<<<<<<<<<
 *         ret.append('Driver internal problem')
 * 
 */
  }

  /* "pycx4/rflags.pxi":34
 *         ret.append('Driver internal problem')
 * 
 *     if (rflags & CXRF_NO_DRV) > 0:             # <<<<<<<<<<<<<<
 *         ret.append('Driver loading problem')
 * 
 */
  __pyx_t_2 = (((__pyx_v_rflags & CXRF_NO_DRV) > 0) != 0);
  if (__pyx_t_2) {

    /* "pycx4/rflags.pxi":35
 * 
 *     if (rflags & CXRF_NO_DRV) > 0:
 *         ret.append('Driver loading problem')             # <<<<<<<<<<<<<<
 * 
 *     if (rflags & CXRF_OFFLINE) > 0:
 */
    __pyx_t_3 = __Pyx_PyList_Append(__pyx_v_ret, __pyx_kp_u_Driver_loading_problem); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(7, 35, __pyx_L1_error)

    /* "pycx4/rflags.pxi":34
 *         ret.append('Driver internal problem')
 * 
 *     if (rflags & CXRF_NO_DRV) > 0:             # <<<<<<<<<<<<<<
 *         ret.append('Driver loading problem')
 * 
 */
  }

  /* "pycx4/rflags.pxi":37
 *         ret.append('Driver loading problem')
 * 
 *     if (rflags & CXRF_OFFLINE) > 0:             # <<<<<<<<<<<<<<
 *         ret.append('Device is offline')
 * 
 */
  __pyx_t_2 = (((__pyx_v_rflags & CXRF_OFFLINE) > 0) != 0);
  if (__pyx_t_2) {

    /* "pycx4/rflags.pxi":38
 * 
 *     if (rflags & CXRF_OFFLINE) > 0:
 *         ret.append('Device is offline')             # <<<<<<<<<<<<<<
 * 
 *     if (rflags & CXCF_FLAG_CALCERR) > 0:
 */
    __pyx_t_3 = __Pyx_PyList_Append(__pyx_v_ret, __pyx_kp_u_Device_is_offline); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(7, 38, __pyx_L1_error)

    /* "pycx4/rflags.pxi":37
 *         ret.append('Driver loading problem')
 * 
 *     if (rflags & CXRF_OFFLINE) > 0:             # <<<<<<<<<<<<<<
 *         ret.append('Device is offline')
 * 
 */
  }

  /* "pycx4/rflags.pxi":40
 *         ret.append('Device is offline')
 * 
 *     if (rflags & CXCF_FLAG_CALCERR) > 0:             # <<<<<<<<<<<<<<
 *         ret.append('Formula calculation error')
 * 
 */
  __pyx_t_2 = (((__pyx_v_rflags & CXCF_FLAG_CALCERR) > 0) != 0);
  if (__pyx_t_2) {

    /* "pycx4/rflags.pxi":41
 * 
 *     if (rflags & CXCF_FLAG_CALCERR) > 0:
 *         ret.append('Formula calculation error')             # <<<<<<<<<<<<<<
 * 
 *     if (rflags & CXCF_FLAG_DEFUNCT) > 0:
 */
    __pyx_t_3 = __Pyx_PyList_Append(__pyx_v_ret, __pyx_kp_u_Formula_calculation_error); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(7, 41, __pyx_L1_error)

    /* "pycx4/rflags.pxi":40
 *         ret.append('Device is offline')
 * 
 *     if (rflags & CXCF_FLAG_CALCERR) > 0:             # <<<<<<<<<<<<<<
 *         ret.append('Formula calculation error')
 * 
 */
  }

  /* "pycx4/rflags.pxi":43
 *         ret.append('Formula calculation error')
 * 
 *     if (rflags & CXCF_FLAG_DEFUNCT) > 0:             # <<<<<<<<<<<<<<
 *         ret.append('Defunct channel')
 * 
 */
  __pyx_t_2 = (((__pyx_v_rflags & CXCF_FLAG_DEFUNCT) > 0) != 0);
  if (__pyx_t_2) {

    /* "pycx4/rflags.pxi":44
 * 
 *     if (rflags & CXCF_FLAG_DEFUNCT) > 0:
 *         ret.append('Defunct channel')             # <<<<<<<<<<<<<<
 * 
 *     if (rflags & CXCF_FLAG_OTHEROP) > 0:
 */
    __pyx_t_3 = __Pyx_PyList_Append(__pyx_v_ret, __pyx_kp_u_Defunct_channel); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(7, 44, __pyx_L1_error)

    /* "pycx4/rflags.pxi":43
 *         ret.append('Formula calculation error')
 * 
 *     if (rflags & CXCF_FLAG_DEFUNCT) > 0:             # <<<<<<<<<<<<<<
 *         ret.append('Defunct channel')
 * 
 */
  }

  /* "pycx4/rflags.pxi":46
 *         ret.append('Defunct channel')
 * 
 *     if (rflags & CXCF_FLAG_OTHEROP) > 0:             # <<<<<<<<<<<<<<
 *         ret.append('Other operator is active')
 * 
 */
  __pyx_t_2 = (((__pyx_v_rflags & CXCF_FLAG_OTHEROP) > 0) != 0);
  if (__pyx_t_2) {

    /* "pycx4/rflags.pxi":47
 * 
 *     if (rflags & CXCF_FLAG_OTHEROP) > 0:
 *         ret.append('Other operator is active')             # <<<<<<<<<<<<<<
 * 
 *     if (rflags & CXCF_FLAG_PRGLYCHG) > 0:
 */
    __pyx_t_3 = __Pyx_PyList_Append(__pyx_v_ret, __pyx_kp_u_Other_operator_is_active); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(7, 47, __pyx_L1_error)

    /* "pycx4/rflags.pxi":46
 *         ret.append('Defunct channel')
 * 
 *     if (rflags & CXCF_FLAG_OTHEROP) > 0:             # <<<<<<<<<<<<<<
 *         ret.append('Other operator is active')
 * 
 */
  }

  /* "pycx4/rflags.pxi":49
 *         ret.append('Other operator is active')
 * 
 *     if (rflags & CXCF_FLAG_PRGLYCHG) > 0:             # <<<<<<<<<<<<<<
 *         ret.append('Channel was programmatically changed')
 * 
 */
  __pyx_t_2 = (((__pyx_v_rflags & CXCF_FLAG_PRGLYCHG) > 0) != 0);
  if (__pyx_t_2) {

    /* "pycx4/rflags.pxi":50
 * 
 *     if (rflags & CXCF_FLAG_PRGLYCHG) > 0:
 *         ret.append('Channel was programmatically changed')             # <<<<<<<<<<<<<<
 * 
 *     if (rflags & CXCF_FLAG_NOTFOUND) > 0:
 */
    __pyx_t_3 = __Pyx_PyList_Append(__pyx_v_ret, __pyx_kp_u_Channel_was_programmatically_cha); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(7, 50, __pyx_L1_error)

    /* "pycx4/rflags.pxi":49
 *         ret.append('Other operator is active')
 * 
 *     if (rflags & CXCF_FLAG_PRGLYCHG) > 0:             # <<<<<<<<<<<<<<
 *         ret.append('Channel was programmatically changed')
 * 
 */
  }

  /* "pycx4/rflags.pxi":52
 *         ret.append('Channel was programmatically changed')
 * 
 *     if (rflags & CXCF_FLAG_NOTFOUND) > 0:             # <<<<<<<<<<<<<<
 *         ret.append('Channel not found')
 * 
 */
  __pyx_t_2 = (((__pyx_v_rflags & CXCF_FLAG_NOTFOUND) > 0) != 0);
  if (__pyx_t_2) {

    /* "pycx4/rflags.pxi":53
 * 
 *     if (rflags & CXCF_FLAG_NOTFOUND) > 0:
 *         ret.append('Channel not found')             # <<<<<<<<<<<<<<
 * 
 *     if (rflags & CXCF_FLAG_COLOR_WEIRD) > 0:
 */
    __pyx_t_3 = __Pyx_PyList_Append(__pyx_v_ret, __pyx_kp_u_Channel_not_found); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(7, 53, __pyx_L1_error)

    /* "pycx4/rflags.pxi":52
 *         ret.append('Channel was programmatically changed')
 * 
 *     if (rflags & CXCF_FLAG_NOTFOUND) > 0:             # <<<<<<<<<<<<<<
 *         ret.append('Channel not found')
 * 
 */
  }

  /* "pycx4/rflags.pxi":55
 *         ret.append('Channel not found')
 * 
 *     if (rflags & CXCF_FLAG_COLOR_WEIRD) > 0:             # <<<<<<<<<<<<<<
 *         ret.append('Value is weird')
 * 
 */
  __pyx_t_2 = (((__pyx_v_rflags & CXCF_FLAG_COLOR_WEIRD) > 0) != 0);
  if (__pyx_t_2) {

    /* "pycx4/rflags.pxi":56
 * 
 *     if (rflags & CXCF_FLAG_COLOR_WEIRD) > 0:
 *         ret.append('Value is weird')             # <<<<<<<<<<<<<<
 * 
 *     if (rflags & CXCF_FLAG_ALARM_ALARM) > 0:
 */
    __pyx_t_3 = __Pyx_PyList_Append(__pyx_v_ret, __pyx_kp_u_Value_is_weird); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(7, 56, __pyx_L1_error)

    /* "pycx4/rflags.pxi":55
 *         ret.append('Channel not found')
 * 
 *     if (rflags & CXCF_FLAG_COLOR_WEIRD) > 0:             # <<<<<<<<<<<<<<
 *         ret.append('Value is weird')
 * 
 */
  }

  /* "pycx4/rflags.pxi":58
 *         ret.append('Value is weird')
 * 
 *     if (rflags & CXCF_FLAG_ALARM_ALARM) > 0:             # <<<<<<<<<<<<<<
 *         ret.append('Alarm!')
 * 
 */
  __pyx_t_2 = (((__pyx_v_rflags & CXCF_FLAG_ALARM_ALARM) > 0) != 0);
  if (__pyx_t_2) {

    /* "pycx4/rflags.pxi":59
 * 
 *     if (rflags & CXCF_FLAG_ALARM_ALARM) > 0:
 *         ret.append('Alarm!')             # <<<<<<<<<<<<<<
 * 
 *     if (rflags & CXCF_FLAG_ALARM_RELAX) > 0:
 */
    __pyx_t_3 = __Pyx_PyList_Append(__pyx_v_ret, __pyx_kp_u_Alarm); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(7, 59, __pyx_L1_error)

    /* "pycx4/rflags.pxi":58
 *         ret.append('Value is weird')
 * 
 *     if (rflags & CXCF_FLAG_ALARM_ALARM) > 0:             # <<<<<<<<<<<<<<
 *         ret.append('Alarm!')
 * 
 */
  }

  /* "pycx4/rflags.pxi":61
 *         ret.append('Alarm!')
 * 
 *     if (rflags & CXCF_FLAG_ALARM_RELAX) > 0:             # <<<<<<<<<<<<<<
 *         ret.append('Relaxing after alarm')
 * 
 */
  __pyx_t_2 = (((__pyx_v_rflags & CXCF_FLAG_ALARM_RELAX) > 0) != 0);
  if (__pyx_t_2) {

    /* "pycx4/rflags.pxi":62
 * 
 *     if (rflags & CXCF_FLAG_ALARM_RELAX) > 0:
 *         ret.append('Relaxing after alarm')             # <<<<<<<<<<<<<<
 * 
 *     if (rflags & CXCF_FLAG_COLOR_RED) > 0:
 */
    __pyx_t_3 = __Pyx_PyList_Append(__pyx_v_ret, __pyx_kp_u_Relaxing_after_alarm); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(7, 62, __pyx_L1_error)

    /* "pycx4/rflags.pxi":61
 *         ret.append('Alarm!')
 * 
 *     if (rflags & CXCF_FLAG_ALARM_RELAX) > 0:             # <<<<<<<<<<<<<<
 *         ret.append('Relaxing after alarm')
 * 
 */
  }

  /* "pycx4/rflags.pxi":64
 *         ret.append('Relaxing after alarm')
 * 
 *     if (rflags & CXCF_FLAG_COLOR_RED) > 0:             # <<<<<<<<<<<<<<
 *         ret.append('Value in red zone')
 * 
 */
  __pyx_t_2 = (((__pyx_v_rflags & CXCF_FLAG_COLOR_RED) > 0) != 0);
  if (__pyx_t_2) {

    /* "pycx4/rflags.pxi":65
 * 
 *     if (rflags & CXCF_FLAG_COLOR_RED) > 0:
 *         ret.append('Value in red zone')             # <<<<<<<<<<<<<<
 * 
 *     if (rflags & CXCF_FLAG_COLOR_YELLOW) > 0:
 */
    __pyx_t_3 = __Pyx_PyList_Append(__pyx_v_ret, __pyx_kp_u_Value_in_red_zone); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(7, 65, __pyx_L1_error)

    /* "pycx4/rflags.pxi":64
 *         ret.append('Relaxing after alarm')
 * 
 *     if (rflags & CXCF_FLAG_COLOR_RED) > 0:             # <<<<<<<<<<<<<<
 *         ret.append('Value in red zone')
 * 
 */
  }

  /* "pycx4/rflags.pxi":67
 *         ret.append('Value in red zone')
 * 
 *     if (rflags & CXCF_FLAG_COLOR_YELLOW) > 0:             # <<<<<<<<<<<<<<
 *         ret.append('Value in yellow zone')
 * 
 */
  __pyx_t_2 = (((__pyx_v_rflags & CXCF_FLAG_COLOR_YELLOW) > 0) != 0);
  if (__pyx_t_2) {

    /* "pycx4/rflags.pxi":68
 * 
 *     if (rflags & CXCF_FLAG_COLOR_YELLOW) > 0:
 *         ret.append('Value in yellow zone')             # <<<<<<<<<<<<<<
 * 
 *     return ret
 */
    __pyx_t_3 = __Pyx_PyList_Append(__pyx_v_ret, __pyx_kp_u_Value_in_yellow_zone); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(7, 68, __pyx_L1_error)

    /* "pycx4/rflags.pxi":67
 *         ret.append('Value in red zone')
 * 
 *     if (rflags & CXCF_FLAG_COLOR_YELLOW) > 0:             # <<<<<<<<<<<<<<
 *         ret.append('Value in yellow zone')
 * 
 */
  }

  /* "pycx4/rflags.pxi":70
 *         ret.append('Value in yellow zone')
 * 
 *     return ret             # <<<<<<<<<<<<<<
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_ret);
  __pyx_r = __pyx_v_ret;
  goto __pyx_L0;

  /* "pycx4/rflags.pxi":2
 * 
 * cdef list rflags_text(rflags_t rflags):             # <<<<<<<<<<<<<<
 *     ret = []
 *     if (rflags & CXRF_CAMAC_NO_X) > 0:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.rflags_text", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ret);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/cdaobject.pxi":8
 *         int evnum
 * 
 *     def __init__(self):             # <<<<<<<<<<<<<<
 *         self.events = NULL
 *         self.evnum = 0
 */

/* Python wrapper */
static int __pyx_pw_5pycx4_5pycda_9CdaObject_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5pycx4_5pycda_9CdaObject_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("__init__", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return -1;}
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__init__", 0))) return -1;
  __pyx_r = __pyx_pf_5pycx4_5pycda_9CdaObject___init__(((struct __pyx_obj_5pycx4_5pycda_CdaObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pycx4_5pycda_9CdaObject___init__(struct __pyx_obj_5pycx4_5pycda_CdaObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "pycx4/cdaobject.pxi":9
 * 
 *     def __init__(self):
 *         self.events = NULL             # <<<<<<<<<<<<<<
 *         self.evnum = 0
 * 
 */
  __pyx_v_self->events = NULL;

  /* "pycx4/cdaobject.pxi":10
 *     def __init__(self):
 *         self.events = NULL
 *         self.evnum = 0             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
 */
  __pyx_v_self->evnum = 0;

  /* "pycx4/cdaobject.pxi":8
 *         int evnum
 * 
 *     def __init__(self):             # <<<<<<<<<<<<<<
 *         self.events = NULL
 *         self.evnum = 0
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/cdaobject.pxi":12
 *         self.evnum = 0
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if self.evnum > 0:
 *             free(self.events)
 */

/* Python wrapper */
static void __pyx_pw_5pycx4_5pycda_9CdaObject_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_5pycx4_5pycda_9CdaObject_3__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_5pycx4_5pycda_9CdaObject_2__dealloc__(((struct __pyx_obj_5pycx4_5pycda_CdaObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_5pycx4_5pycda_9CdaObject_2__dealloc__(struct __pyx_obj_5pycx4_5pycda_CdaObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "pycx4/cdaobject.pxi":13
 * 
 *     def __dealloc__(self):
 *         if self.evnum > 0:             # <<<<<<<<<<<<<<
 *             free(self.events)
 *             self.events = NULL
 */
  __pyx_t_1 = ((__pyx_v_self->evnum > 0) != 0);
  if (__pyx_t_1) {

    /* "pycx4/cdaobject.pxi":14
 *     def __dealloc__(self):
 *         if self.evnum > 0:
 *             free(self.events)             # <<<<<<<<<<<<<<
 *             self.events = NULL
 *             self.evnum = 0
 */
    free(__pyx_v_self->events);

    /* "pycx4/cdaobject.pxi":15
 *         if self.evnum > 0:
 *             free(self.events)
 *             self.events = NULL             # <<<<<<<<<<<<<<
 *             self.evnum = 0
 * 
 */
    __pyx_v_self->events = NULL;

    /* "pycx4/cdaobject.pxi":16
 *             free(self.events)
 *             self.events = NULL
 *             self.evnum = 0             # <<<<<<<<<<<<<<
 * 
 *     cdef int add_event(self,int evmask, void *evproc, void *objptr, void *userptr):
 */
    __pyx_v_self->evnum = 0;

    /* "pycx4/cdaobject.pxi":13
 * 
 *     def __dealloc__(self):
 *         if self.evnum > 0:             # <<<<<<<<<<<<<<
 *             free(self.events)
 *             self.events = NULL
 */
  }

  /* "pycx4/cdaobject.pxi":12
 *         self.evnum = 0
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if self.evnum > 0:
 *             free(self.events)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/cdaobject.pxi":18
 *             self.evnum = 0
 * 
 *     cdef int add_event(self,int evmask, void *evproc, void *objptr, void *userptr):             # <<<<<<<<<<<<<<
 *         cdef:
 *             int x
 */

static int __pyx_f_5pycx4_5pycda_9CdaObject_add_event(struct __pyx_obj_5pycx4_5pycda_CdaObject *__pyx_v_self, int __pyx_v_evmask, void *__pyx_v_evproc, void *__pyx_v_objptr, void *__pyx_v_userptr) {
  int __pyx_v_x;
  __pyx_t_5pycx4_5pycda_event *__pyx_v_ev_t;
  __pyx_t_5pycx4_5pycda_event *__pyx_v_ev;
  void *__pyx_v_tmp;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  void *__pyx_t_6;
  void *__pyx_t_7;
  void *__pyx_t_8;
  __Pyx_RefNannySetupContext("add_event", 0);

  /* "pycx4/cdaobject.pxi":25
 *             void *tmp
 * 
 *         if evmask == 0 or evproc == NULL:             # <<<<<<<<<<<<<<
 *             # event will not run
 *             return 0
 */
  __pyx_t_2 = ((__pyx_v_evmask == 0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_evproc == NULL) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "pycx4/cdaobject.pxi":27
 *         if evmask == 0 or evproc == NULL:
 *             # event will not run
 *             return 0             # <<<<<<<<<<<<<<
 *         for x in range(self.evnum):
 *             ev_t = self.events[x]
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "pycx4/cdaobject.pxi":25
 *             void *tmp
 * 
 *         if evmask == 0 or evproc == NULL:             # <<<<<<<<<<<<<<
 *             # event will not run
 *             return 0
 */
  }

  /* "pycx4/cdaobject.pxi":28
 *             # event will not run
 *             return 0
 *         for x in range(self.evnum):             # <<<<<<<<<<<<<<
 *             ev_t = self.events[x]
 *             if ev_t.evmask == evmask and ev_t.evproc == evproc and \
 */
  __pyx_t_3 = __pyx_v_self->evnum;
  __pyx_t_4 = __pyx_t_3;
  for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
    __pyx_v_x = __pyx_t_5;

    /* "pycx4/cdaobject.pxi":29
 *             return 0
 *         for x in range(self.evnum):
 *             ev_t = self.events[x]             # <<<<<<<<<<<<<<
 *             if ev_t.evmask == evmask and ev_t.evproc == evproc and \
 *             ev_t.objptr == objptr and ev_t.userptr == userptr:
 */
    __pyx_v_ev_t = (__pyx_v_self->events[__pyx_v_x]);

    /* "pycx4/cdaobject.pxi":30
 *         for x in range(self.evnum):
 *             ev_t = self.events[x]
 *             if ev_t.evmask == evmask and ev_t.evproc == evproc and \             # <<<<<<<<<<<<<<
 *             ev_t.objptr == objptr and ev_t.userptr == userptr:
 *                 # event already exists
 */
    __pyx_t_2 = ((__pyx_v_ev_t->evmask == __pyx_v_evmask) != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L9_bool_binop_done;
    }
    __pyx_t_2 = ((__pyx_v_ev_t->evproc == __pyx_v_evproc) != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L9_bool_binop_done;
    }

    /* "pycx4/cdaobject.pxi":31
 *             ev_t = self.events[x]
 *             if ev_t.evmask == evmask and ev_t.evproc == evproc and \
 *             ev_t.objptr == objptr and ev_t.userptr == userptr:             # <<<<<<<<<<<<<<
 *                 # event already exists
 *                 return 0
 */
    __pyx_t_2 = ((__pyx_v_ev_t->objptr == __pyx_v_objptr) != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L9_bool_binop_done;
    }
    __pyx_t_2 = ((__pyx_v_ev_t->userptr == __pyx_v_userptr) != 0);
    __pyx_t_1 = __pyx_t_2;
    __pyx_L9_bool_binop_done:;

    /* "pycx4/cdaobject.pxi":30
 *         for x in range(self.evnum):
 *             ev_t = self.events[x]
 *             if ev_t.evmask == evmask and ev_t.evproc == evproc and \             # <<<<<<<<<<<<<<
 *             ev_t.objptr == objptr and ev_t.userptr == userptr:
 *                 # event already exists
 */
    if (__pyx_t_1) {

      /* "pycx4/cdaobject.pxi":33
 *             ev_t.objptr == objptr and ev_t.userptr == userptr:
 *                 # event already exists
 *                 return 0             # <<<<<<<<<<<<<<
 * 
 *         ev = <event*>malloc(sizeof(event))
 */
      __pyx_r = 0;
      goto __pyx_L0;

      /* "pycx4/cdaobject.pxi":30
 *         for x in range(self.evnum):
 *             ev_t = self.events[x]
 *             if ev_t.evmask == evmask and ev_t.evproc == evproc and \             # <<<<<<<<<<<<<<
 *             ev_t.objptr == objptr and ev_t.userptr == userptr:
 *                 # event already exists
 */
    }
  }

  /* "pycx4/cdaobject.pxi":35
 *                 return 0
 * 
 *         ev = <event*>malloc(sizeof(event))             # <<<<<<<<<<<<<<
 *         if not ev: raise MemoryError()
 * 
 */
  __pyx_v_ev = ((__pyx_t_5pycx4_5pycda_event *)malloc((sizeof(__pyx_t_5pycx4_5pycda_event))));

  /* "pycx4/cdaobject.pxi":36
 * 
 *         ev = <event*>malloc(sizeof(event))
 *         if not ev: raise MemoryError()             # <<<<<<<<<<<<<<
 * 
 *         ev.evmask, ev.evproc, ev.objptr, ev.userptr = evmask, evproc, objptr, userptr
 */
  __pyx_t_1 = ((!(__pyx_v_ev != 0)) != 0);
  if (unlikely(__pyx_t_1)) {
    PyErr_NoMemory(); __PYX_ERR(8, 36, __pyx_L1_error)
  }

  /* "pycx4/cdaobject.pxi":38
 *         if not ev: raise MemoryError()
 * 
 *         ev.evmask, ev.evproc, ev.objptr, ev.userptr = evmask, evproc, objptr, userptr             # <<<<<<<<<<<<<<
 * 
 *         tmp = realloc(self.events, sizeof(event*) * (self.evnum + 1) )
 */
  __pyx_t_3 = __pyx_v_evmask;
  __pyx_t_6 = __pyx_v_evproc;
  __pyx_t_7 = __pyx_v_objptr;
  __pyx_t_8 = __pyx_v_userptr;
  __pyx_v_ev->evmask = __pyx_t_3;
  __pyx_v_ev->evproc = __pyx_t_6;
  __pyx_v_ev->objptr = __pyx_t_7;
  __pyx_v_ev->userptr = __pyx_t_8;

  /* "pycx4/cdaobject.pxi":40
 *         ev.evmask, ev.evproc, ev.objptr, ev.userptr = evmask, evproc, objptr, userptr
 * 
 *         tmp = realloc(self.events, sizeof(event*) * (self.evnum + 1) )             # <<<<<<<<<<<<<<
 *         if not tmp: raise MemoryError()
 *         self.events = <event**>tmp
 */
  __pyx_v_tmp = realloc(__pyx_v_self->events, ((sizeof(__pyx_t_5pycx4_5pycda_event *)) * (__pyx_v_self->evnum + 1)));

  /* "pycx4/cdaobject.pxi":41
 * 
 *         tmp = realloc(self.events, sizeof(event*) * (self.evnum + 1) )
 *         if not tmp: raise MemoryError()             # <<<<<<<<<<<<<<
 *         self.events = <event**>tmp
 *         self.events[self.evnum] = ev
 */
  __pyx_t_1 = ((!(__pyx_v_tmp != 0)) != 0);
  if (unlikely(__pyx_t_1)) {
    PyErr_NoMemory(); __PYX_ERR(8, 41, __pyx_L1_error)
  }

  /* "pycx4/cdaobject.pxi":42
 *         tmp = realloc(self.events, sizeof(event*) * (self.evnum + 1) )
 *         if not tmp: raise MemoryError()
 *         self.events = <event**>tmp             # <<<<<<<<<<<<<<
 *         self.events[self.evnum] = ev
 * 
 */
  __pyx_v_self->events = ((__pyx_t_5pycx4_5pycda_event **)__pyx_v_tmp);

  /* "pycx4/cdaobject.pxi":43
 *         if not tmp: raise MemoryError()
 *         self.events = <event**>tmp
 *         self.events[self.evnum] = ev             # <<<<<<<<<<<<<<
 * 
 *         self.register_event(ev)
 */
  (__pyx_v_self->events[__pyx_v_self->evnum]) = __pyx_v_ev;

  /* "pycx4/cdaobject.pxi":45
 *         self.events[self.evnum] = ev
 * 
 *         self.register_event(ev)             # <<<<<<<<<<<<<<
 *         self.evnum += 1
 *         return 1
 */
  ((struct __pyx_vtabstruct_5pycx4_5pycda_CdaObject *)__pyx_v_self->__pyx_vtab)->register_event(__pyx_v_self, __pyx_v_ev);

  /* "pycx4/cdaobject.pxi":46
 * 
 *         self.register_event(ev)
 *         self.evnum += 1             # <<<<<<<<<<<<<<
 *         return 1
 * 
 */
  __pyx_v_self->evnum = (__pyx_v_self->evnum + 1);

  /* "pycx4/cdaobject.pxi":47
 *         self.register_event(ev)
 *         self.evnum += 1
 *         return 1             # <<<<<<<<<<<<<<
 * 
 *     cdef int del_event(self, int evmask, void *evproc, void *objptr, void *userptr):
 */
  __pyx_r = 1;
  goto __pyx_L0;

  /* "pycx4/cdaobject.pxi":18
 *             self.evnum = 0
 * 
 *     cdef int add_event(self,int evmask, void *evproc, void *objptr, void *userptr):             # <<<<<<<<<<<<<<
 *         cdef:
 *             int x
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("pycx4.pycda.CdaObject.add_event", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/cdaobject.pxi":49
 *         return 1
 * 
 *     cdef int del_event(self, int evmask, void *evproc, void *objptr, void *userptr):             # <<<<<<<<<<<<<<
 *         cdef:
 *             event ev
 */

static int __pyx_f_5pycx4_5pycda_9CdaObject_del_event(struct __pyx_obj_5pycx4_5pycda_CdaObject *__pyx_v_self, int __pyx_v_evmask, void *__pyx_v_evproc, void *__pyx_v_objptr, void *__pyx_v_userptr) {
  int __pyx_v_ev_ind;
  int __pyx_v_x;
  void *__pyx_v_tmp;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  __Pyx_RefNannySetupContext("del_event", 0);

  /* "pycx4/cdaobject.pxi":52
 *         cdef:
 *             event ev
 *             int ev_ind = -1             # <<<<<<<<<<<<<<
 *             int x
 * 
 */
  __pyx_v_ev_ind = -1;

  /* "pycx4/cdaobject.pxi":55
 *             int x
 * 
 *         for x in range(self.evnum):             # <<<<<<<<<<<<<<
 *             if self.events[x].evmask == evmask and self.events[x].evproc == evproc and \
 *             self.events[x].objptr == objptr and self.events[x].userptr == userptr:
 */
  __pyx_t_1 = __pyx_v_self->evnum;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_x = __pyx_t_3;

    /* "pycx4/cdaobject.pxi":56
 * 
 *         for x in range(self.evnum):
 *             if self.events[x].evmask == evmask and self.events[x].evproc == evproc and \             # <<<<<<<<<<<<<<
 *             self.events[x].objptr == objptr and self.events[x].userptr == userptr:
 *                 ev_ind = x
 */
    __pyx_t_5 = (((__pyx_v_self->events[__pyx_v_x])->evmask == __pyx_v_evmask) != 0);
    if (__pyx_t_5) {
    } else {
      __pyx_t_4 = __pyx_t_5;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_5 = (((__pyx_v_self->events[__pyx_v_x])->evproc == __pyx_v_evproc) != 0);
    if (__pyx_t_5) {
    } else {
      __pyx_t_4 = __pyx_t_5;
      goto __pyx_L6_bool_binop_done;
    }

    /* "pycx4/cdaobject.pxi":57
 *         for x in range(self.evnum):
 *             if self.events[x].evmask == evmask and self.events[x].evproc == evproc and \
 *             self.events[x].objptr == objptr and self.events[x].userptr == userptr:             # <<<<<<<<<<<<<<
 *                 ev_ind = x
 *                 break
 */
    __pyx_t_5 = (((__pyx_v_self->events[__pyx_v_x])->objptr == __pyx_v_objptr) != 0);
    if (__pyx_t_5) {
    } else {
      __pyx_t_4 = __pyx_t_5;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_5 = (((__pyx_v_self->events[__pyx_v_x])->userptr == __pyx_v_userptr) != 0);
    __pyx_t_4 = __pyx_t_5;
    __pyx_L6_bool_binop_done:;

    /* "pycx4/cdaobject.pxi":56
 * 
 *         for x in range(self.evnum):
 *             if self.events[x].evmask == evmask and self.events[x].evproc == evproc and \             # <<<<<<<<<<<<<<
 *             self.events[x].objptr == objptr and self.events[x].userptr == userptr:
 *                 ev_ind = x
 */
    if (__pyx_t_4) {

      /* "pycx4/cdaobject.pxi":58
 *             if self.events[x].evmask == evmask and self.events[x].evproc == evproc and \
 *             self.events[x].objptr == objptr and self.events[x].userptr == userptr:
 *                 ev_ind = x             # <<<<<<<<<<<<<<
 *                 break
 * 
 */
      __pyx_v_ev_ind = __pyx_v_x;

      /* "pycx4/cdaobject.pxi":59
 *             self.events[x].objptr == objptr and self.events[x].userptr == userptr:
 *                 ev_ind = x
 *                 break             # <<<<<<<<<<<<<<
 * 
 *         if ev_ind < 0: # event does not exist
 */
      goto __pyx_L4_break;

      /* "pycx4/cdaobject.pxi":56
 * 
 *         for x in range(self.evnum):
 *             if self.events[x].evmask == evmask and self.events[x].evproc == evproc and \             # <<<<<<<<<<<<<<
 *             self.events[x].objptr == objptr and self.events[x].userptr == userptr:
 *                 ev_ind = x
 */
    }
  }
  __pyx_L4_break:;

  /* "pycx4/cdaobject.pxi":61
 *                 break
 * 
 *         if ev_ind < 0: # event does not exist             # <<<<<<<<<<<<<<
 *             return 0
 *         self.unregister_event(self.events[ev_ind])
 */
  __pyx_t_4 = ((__pyx_v_ev_ind < 0) != 0);
  if (__pyx_t_4) {

    /* "pycx4/cdaobject.pxi":62
 * 
 *         if ev_ind < 0: # event does not exist
 *             return 0             # <<<<<<<<<<<<<<
 *         self.unregister_event(self.events[ev_ind])
 *         if self.evnum == 1:
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "pycx4/cdaobject.pxi":61
 *                 break
 * 
 *         if ev_ind < 0: # event does not exist             # <<<<<<<<<<<<<<
 *             return 0
 *         self.unregister_event(self.events[ev_ind])
 */
  }

  /* "pycx4/cdaobject.pxi":63
 *         if ev_ind < 0: # event does not exist
 *             return 0
 *         self.unregister_event(self.events[ev_ind])             # <<<<<<<<<<<<<<
 *         if self.evnum == 1:
 *             free(self.events[ev_ind])
 */
  ((struct __pyx_vtabstruct_5pycx4_5pycda_CdaObject *)__pyx_v_self->__pyx_vtab)->unregister_event(__pyx_v_self, (__pyx_v_self->events[__pyx_v_ev_ind]));

  /* "pycx4/cdaobject.pxi":64
 *             return 0
 *         self.unregister_event(self.events[ev_ind])
 *         if self.evnum == 1:             # <<<<<<<<<<<<<<
 *             free(self.events[ev_ind])
 *             # looks like self.events[ev_ind] = NULL - not needed here
 */
  __pyx_t_4 = ((__pyx_v_self->evnum == 1) != 0);
  if (__pyx_t_4) {

    /* "pycx4/cdaobject.pxi":65
 *         self.unregister_event(self.events[ev_ind])
 *         if self.evnum == 1:
 *             free(self.events[ev_ind])             # <<<<<<<<<<<<<<
 *             # looks like self.events[ev_ind] = NULL - not needed here
 *             free(self.events)
 */
    free((__pyx_v_self->events[__pyx_v_ev_ind]));

    /* "pycx4/cdaobject.pxi":67
 *             free(self.events[ev_ind])
 *             # looks like self.events[ev_ind] = NULL - not needed here
 *             free(self.events)             # <<<<<<<<<<<<<<
 *             self.events = NULL
 *         else:
 */
    free(__pyx_v_self->events);

    /* "pycx4/cdaobject.pxi":68
 *             # looks like self.events[ev_ind] = NULL - not needed here
 *             free(self.events)
 *             self.events = NULL             # <<<<<<<<<<<<<<
 *         else:
 *             memmove(&(self.events[ev_ind]), &(self.events[self.evnum-1]), sizeof(event*))
 */
    __pyx_v_self->events = NULL;

    /* "pycx4/cdaobject.pxi":64
 *             return 0
 *         self.unregister_event(self.events[ev_ind])
 *         if self.evnum == 1:             # <<<<<<<<<<<<<<
 *             free(self.events[ev_ind])
 *             # looks like self.events[ev_ind] = NULL - not needed here
 */
    goto __pyx_L11;
  }

  /* "pycx4/cdaobject.pxi":70
 *             self.events = NULL
 *         else:
 *             memmove(&(self.events[ev_ind]), &(self.events[self.evnum-1]), sizeof(event*))             # <<<<<<<<<<<<<<
 *             tmp = realloc(self.events, sizeof(event*) * (self.evnum - 1))
 *             if not tmp: raise MemoryError()
 */
  /*else*/ {
    (void)(memmove((&(__pyx_v_self->events[__pyx_v_ev_ind])), (&(__pyx_v_self->events[(__pyx_v_self->evnum - 1)])), (sizeof(__pyx_t_5pycx4_5pycda_event *))));

    /* "pycx4/cdaobject.pxi":71
 *         else:
 *             memmove(&(self.events[ev_ind]), &(self.events[self.evnum-1]), sizeof(event*))
 *             tmp = realloc(self.events, sizeof(event*) * (self.evnum - 1))             # <<<<<<<<<<<<<<
 *             if not tmp: raise MemoryError()
 *             self.events = <event**>tmp
 */
    __pyx_v_tmp = realloc(__pyx_v_self->events, ((sizeof(__pyx_t_5pycx4_5pycda_event *)) * (__pyx_v_self->evnum - 1)));

    /* "pycx4/cdaobject.pxi":72
 *             memmove(&(self.events[ev_ind]), &(self.events[self.evnum-1]), sizeof(event*))
 *             tmp = realloc(self.events, sizeof(event*) * (self.evnum - 1))
 *             if not tmp: raise MemoryError()             # <<<<<<<<<<<<<<
 *             self.events = <event**>tmp
 *         self.evnum -= 1
 */
    __pyx_t_4 = ((!(__pyx_v_tmp != 0)) != 0);
    if (unlikely(__pyx_t_4)) {
      PyErr_NoMemory(); __PYX_ERR(8, 72, __pyx_L1_error)
    }

    /* "pycx4/cdaobject.pxi":73
 *             tmp = realloc(self.events, sizeof(event*) * (self.evnum - 1))
 *             if not tmp: raise MemoryError()
 *             self.events = <event**>tmp             # <<<<<<<<<<<<<<
 *         self.evnum -= 1
 *         return 1
 */
    __pyx_v_self->events = ((__pyx_t_5pycx4_5pycda_event **)__pyx_v_tmp);
  }
  __pyx_L11:;

  /* "pycx4/cdaobject.pxi":74
 *             if not tmp: raise MemoryError()
 *             self.events = <event**>tmp
 *         self.evnum -= 1             # <<<<<<<<<<<<<<
 *         return 1
 * 
 */
  __pyx_v_self->evnum = (__pyx_v_self->evnum - 1);

  /* "pycx4/cdaobject.pxi":75
 *             self.events = <event**>tmp
 *         self.evnum -= 1
 *         return 1             # <<<<<<<<<<<<<<
 * 
 *     cdef void register_event(self, event *ev):
 */
  __pyx_r = 1;
  goto __pyx_L0;

  /* "pycx4/cdaobject.pxi":49
 *         return 1
 * 
 *     cdef int del_event(self, int evmask, void *evproc, void *objptr, void *userptr):             # <<<<<<<<<<<<<<
 *         cdef:
 *             event ev
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("pycx4.pycda.CdaObject.del_event", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/cdaobject.pxi":77
 *         return 1
 * 
 *     cdef void register_event(self, event *ev):             # <<<<<<<<<<<<<<
 *         return
 * 
 */

static void __pyx_f_5pycx4_5pycda_9CdaObject_register_event(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_CdaObject *__pyx_v_self, CYTHON_UNUSED __pyx_t_5pycx4_5pycda_event *__pyx_v_ev) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("register_event", 0);

  /* "pycx4/cdaobject.pxi":78
 * 
 *     cdef void register_event(self, event *ev):
 *         return             # <<<<<<<<<<<<<<
 * 
 *     cdef void unregister_event(self, event *ev):
 */
  goto __pyx_L0;

  /* "pycx4/cdaobject.pxi":77
 *         return 1
 * 
 *     cdef void register_event(self, event *ev):             # <<<<<<<<<<<<<<
 *         return
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/cdaobject.pxi":80
 *         return
 * 
 *     cdef void unregister_event(self, event *ev):             # <<<<<<<<<<<<<<
 *         return
 */

static void __pyx_f_5pycx4_5pycda_9CdaObject_unregister_event(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_CdaObject *__pyx_v_self, CYTHON_UNUSED __pyx_t_5pycx4_5pycda_event *__pyx_v_ev) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("unregister_event", 0);

  /* "pycx4/cdaobject.pxi":81
 * 
 *     cdef void unregister_event(self, event *ev):
 *         return             # <<<<<<<<<<<<<<
 */
  goto __pyx_L0;

  /* "pycx4/cdaobject.pxi":80
 *         return
 * 
 *     cdef void unregister_event(self, event *ev):             # <<<<<<<<<<<<<<
 *         return
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_9CdaObject_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_9CdaObject_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_9CdaObject_4__reduce_cython__(((struct __pyx_obj_5pycx4_5pycda_CdaObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_9CdaObject_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_CdaObject *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.CdaObject.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_9CdaObject_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_9CdaObject_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_9CdaObject_6__setstate_cython__(((struct __pyx_obj_5pycx4_5pycda_CdaObject *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_9CdaObject_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_CdaObject *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.CdaObject.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/context.pxi":3
 * 
 * # C callback for context
 * cdef void evproc_cont_cycle(int uniq, void *privptr1, cda_context_t cid, int reason,             # <<<<<<<<<<<<<<
 *                       int info_int, void *privptr2) with gil:
 *     cdef Context cont=<Context>(<event*>privptr2).objptr
 */

static void __pyx_f_5pycx4_5pycda_evproc_cont_cycle(CYTHON_UNUSED int __pyx_v_uniq, CYTHON_UNUSED void *__pyx_v_privptr1, CYTHON_UNUSED cda_context_t __pyx_v_cid, CYTHON_UNUSED int __pyx_v_reason, CYTHON_UNUSED int __pyx_v_info_int, void *__pyx_v_privptr2) {
  struct __pyx_obj_5pycx4_5pycda_Context *__pyx_v_cont = 0;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("evproc_cont_cycle", 0);

  /* "pycx4/context.pxi":5
 * cdef void evproc_cont_cycle(int uniq, void *privptr1, cda_context_t cid, int reason,
 *                       int info_int, void *privptr2) with gil:
 *     cdef Context cont=<Context>(<event*>privptr2).objptr             # <<<<<<<<<<<<<<
 *     cont.serverCycle.emit(cont)
 * 
 */
  __pyx_t_1 = ((PyObject *)((__pyx_t_5pycx4_5pycda_event *)__pyx_v_privptr2)->objptr);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_cont = ((struct __pyx_obj_5pycx4_5pycda_Context *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pycx4/context.pxi":6
 *                       int info_int, void *privptr2) with gil:
 *     cdef Context cont=<Context>(<event*>privptr2).objptr
 *     cont.serverCycle.emit(cont)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_cont->serverCycle, __pyx_n_s_emit); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, ((PyObject *)__pyx_v_cont)) : __Pyx_PyObject_CallOneArg(__pyx_t_2, ((PyObject *)__pyx_v_cont));
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/context.pxi":3
 * 
 * # C callback for context
 * cdef void evproc_cont_cycle(int uniq, void *privptr1, cda_context_t cid, int reason,             # <<<<<<<<<<<<<<
 *                       int info_int, void *privptr2) with gil:
 *     cdef Context cont=<Context>(<event*>privptr2).objptr
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_WriteUnraisable("pycx4.pycda.evproc_cont_cycle", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_cont);
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
}

/* "pycx4/context.pxi":19
 *         int channum
 * 
 *     def __init__(self, defpfx="cx::", **kwargs):             # <<<<<<<<<<<<<<
 *         super().__init__()
 *         cdef:
 */

/* Python wrapper */
static int __pyx_pw_5pycx4_5pycda_7Context_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5pycx4_5pycda_7Context_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_defpfx = 0;
  PyObject *__pyx_v_kwargs = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  __pyx_v_kwargs = PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return -1;
  __Pyx_GOTREF(__pyx_v_kwargs);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_defpfx,0};
    PyObject* values[1] = {0};
    values[0] = ((PyObject *)__pyx_kp_u_cx);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_defpfx);
          if (value) { values[0] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, __pyx_v_kwargs, values, pos_args, "__init__") < 0)) __PYX_ERR(3, 19, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_defpfx = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 0, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(3, 19, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_DECREF(__pyx_v_kwargs); __pyx_v_kwargs = 0;
  __Pyx_AddTraceback("pycx4.pycda.Context.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pycx4_5pycda_7Context___init__(((struct __pyx_obj_5pycx4_5pycda_Context *)__pyx_v_self), __pyx_v_defpfx, __pyx_v_kwargs);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pycx4_5pycda_7Context___init__(struct __pyx_obj_5pycx4_5pycda_Context *__pyx_v_self, PyObject *__pyx_v_defpfx, PyObject *__pyx_v_kwargs) {
  int __pyx_v_ret;
  int __pyx_v_options;
  char *__pyx_v_c_defpfx;
  PyObject *__pyx_v_ascii_pfx = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  char *__pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  void *__pyx_t_8;
  int __pyx_t_9;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "pycx4/context.pxi":20
 * 
 *     def __init__(self, defpfx="cx::", **kwargs):
 *         super().__init__()             # <<<<<<<<<<<<<<
 *         cdef:
 *             int ret
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 20, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_5pycx4_5pycda_Context));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_5pycx4_5pycda_Context));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_5pycx4_5pycda_Context));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 20, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 20, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 20, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/context.pxi":23
 *         cdef:
 *             int ret
 *             int options = 0             # <<<<<<<<<<<<<<
 *             char *c_defpfx
 *         ascii_pfx = defpfx.encode("ascii") # encode to ascii
 */
  __pyx_v_options = 0;

  /* "pycx4/context.pxi":25
 *             int options = 0
 *             char *c_defpfx
 *         ascii_pfx = defpfx.encode("ascii") # encode to ascii             # <<<<<<<<<<<<<<
 *         c_defpfx = ascii_pfx  # convert to char*
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_defpfx, __pyx_n_s_encode); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_n_u_ascii) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_n_u_ascii);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_ascii_pfx = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pycx4/context.pxi":26
 *             char *c_defpfx
 *         ascii_pfx = defpfx.encode("ascii") # encode to ascii
 *         c_defpfx = ascii_pfx  # convert to char*             # <<<<<<<<<<<<<<
 * 
 *         if 'other_opp_flag' in kwargs:
 */
  __pyx_t_4 = __Pyx_PyObject_AsWritableString(__pyx_v_ascii_pfx); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(3, 26, __pyx_L1_error)
  __pyx_v_c_defpfx = __pyx_t_4;

  /* "pycx4/context.pxi":28
 *         c_defpfx = ascii_pfx  # convert to char*
 * 
 *         if 'other_opp_flag' in kwargs:             # <<<<<<<<<<<<<<
 *             if kwargs['other_opp_flag']:
 *                 options += CDA_CONTEXT_OPT_NO_OTHEROP
 */
  __pyx_t_5 = (__Pyx_PyDict_ContainsTF(__pyx_n_u_other_opp_flag, __pyx_v_kwargs, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(3, 28, __pyx_L1_error)
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "pycx4/context.pxi":29
 * 
 *         if 'other_opp_flag' in kwargs:
 *             if kwargs['other_opp_flag']:             # <<<<<<<<<<<<<<
 *                 options += CDA_CONTEXT_OPT_NO_OTHEROP
 * 
 */
    __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_kwargs, __pyx_n_u_other_opp_flag); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 29, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(3, 29, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_6) {

      /* "pycx4/context.pxi":30
 *         if 'other_opp_flag' in kwargs:
 *             if kwargs['other_opp_flag']:
 *                 options += CDA_CONTEXT_OPT_NO_OTHEROP             # <<<<<<<<<<<<<<
 * 
 *         if 'ignore_update' in kwargs:
 */
      __pyx_v_options = (__pyx_v_options + CDA_CONTEXT_OPT_NO_OTHEROP);

      /* "pycx4/context.pxi":29
 * 
 *         if 'other_opp_flag' in kwargs:
 *             if kwargs['other_opp_flag']:             # <<<<<<<<<<<<<<
 *                 options += CDA_CONTEXT_OPT_NO_OTHEROP
 * 
 */
    }

    /* "pycx4/context.pxi":28
 *         c_defpfx = ascii_pfx  # convert to char*
 * 
 *         if 'other_opp_flag' in kwargs:             # <<<<<<<<<<<<<<
 *             if kwargs['other_opp_flag']:
 *                 options += CDA_CONTEXT_OPT_NO_OTHEROP
 */
  }

  /* "pycx4/context.pxi":32
 *                 options += CDA_CONTEXT_OPT_NO_OTHEROP
 * 
 *         if 'ignore_update' in kwargs:             # <<<<<<<<<<<<<<
 *             if kwargs['ignore_update']:
 *                 options += CDA_CONTEXT_OPT_IGN_UPDATE
 */
  __pyx_t_6 = (__Pyx_PyDict_ContainsTF(__pyx_n_u_ignore_update, __pyx_v_kwargs, Py_EQ)); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(3, 32, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_6 != 0);
  if (__pyx_t_5) {

    /* "pycx4/context.pxi":33
 * 
 *         if 'ignore_update' in kwargs:
 *             if kwargs['ignore_update']:             # <<<<<<<<<<<<<<
 *                 options += CDA_CONTEXT_OPT_IGN_UPDATE
 * 
 */
    __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_kwargs, __pyx_n_u_ignore_update); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 33, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(3, 33, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_5) {

      /* "pycx4/context.pxi":34
 *         if 'ignore_update' in kwargs:
 *             if kwargs['ignore_update']:
 *                 options += CDA_CONTEXT_OPT_IGN_UPDATE             # <<<<<<<<<<<<<<
 * 
 *         ret = cda_new_context(0, NULL, c_defpfx, options, NULL, 0, <cda_context_evproc_t>NULL, NULL)
 */
      __pyx_v_options = (__pyx_v_options + CDA_CONTEXT_OPT_IGN_UPDATE);

      /* "pycx4/context.pxi":33
 * 
 *         if 'ignore_update' in kwargs:
 *             if kwargs['ignore_update']:             # <<<<<<<<<<<<<<
 *                 options += CDA_CONTEXT_OPT_IGN_UPDATE
 * 
 */
    }

    /* "pycx4/context.pxi":32
 *                 options += CDA_CONTEXT_OPT_NO_OTHEROP
 * 
 *         if 'ignore_update' in kwargs:             # <<<<<<<<<<<<<<
 *             if kwargs['ignore_update']:
 *                 options += CDA_CONTEXT_OPT_IGN_UPDATE
 */
  }

  /* "pycx4/context.pxi":36
 *                 options += CDA_CONTEXT_OPT_IGN_UPDATE
 * 
 *         ret = cda_new_context(0, NULL, c_defpfx, options, NULL, 0, <cda_context_evproc_t>NULL, NULL)             # <<<<<<<<<<<<<<
 *         self.check_exception(ret)
 *         self.cid, self.defpfx, self.chans, self.channum = ret, defpfx, NULL, 0
 */
  __pyx_v_ret = cda_new_context(0, NULL, __pyx_v_c_defpfx, __pyx_v_options, NULL, 0, ((cda_context_evproc_t)NULL), NULL);

  /* "pycx4/context.pxi":37
 * 
 *         ret = cda_new_context(0, NULL, c_defpfx, options, NULL, 0, <cda_context_evproc_t>NULL, NULL)
 *         self.check_exception(ret)             # <<<<<<<<<<<<<<
 *         self.cid, self.defpfx, self.chans, self.channum = ret, defpfx, NULL, 0
 * 
 */
  ((struct __pyx_vtabstruct_5pycx4_5pycda_Context *)__pyx_v_self->__pyx_base.__pyx_vtab)->check_exception(__pyx_v_self, __pyx_v_ret);

  /* "pycx4/context.pxi":38
 *         ret = cda_new_context(0, NULL, c_defpfx, options, NULL, 0, <cda_context_evproc_t>NULL, NULL)
 *         self.check_exception(ret)
 *         self.cid, self.defpfx, self.chans, self.channum = ret, defpfx, NULL, 0             # <<<<<<<<<<<<<<
 * 
 *         self.serverCycle = Signal(object)
 */
  __pyx_t_7 = __pyx_v_ret;
  if (!(likely(PyUnicode_CheckExact(__pyx_v_defpfx))||((__pyx_v_defpfx) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_v_defpfx)->tp_name), 0))) __PYX_ERR(3, 38, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_defpfx;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_8 = NULL;
  __pyx_t_9 = 0;
  __pyx_v_self->cid = __pyx_t_7;
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->defpfx);
  __Pyx_DECREF(__pyx_v_self->defpfx);
  __pyx_v_self->defpfx = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_v_self->chans = __pyx_t_8;
  __pyx_v_self->channum = __pyx_t_9;

  /* "pycx4/context.pxi":40
 *         self.cid, self.defpfx, self.chans, self.channum = ret, defpfx, NULL, 0
 * 
 *         self.serverCycle = Signal(object)             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5pycx4_5pycda_Signal), __pyx_tuple__9, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->serverCycle);
  __Pyx_DECREF(__pyx_v_self->serverCycle);
  __pyx_v_self->serverCycle = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pycx4/context.pxi":19
 *         int channum
 * 
 *     def __init__(self, defpfx="cx::", **kwargs):             # <<<<<<<<<<<<<<
 *         super().__init__()
 *         cdef:
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("pycx4.pycda.Context.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ascii_pfx);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/context.pxi":42
 *         self.serverCycle = Signal(object)
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if self.cid > 0:
 *             self.check_exception( cda_del_context(self.cid) )
 */

/* Python wrapper */
static void __pyx_pw_5pycx4_5pycda_7Context_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_5pycx4_5pycda_7Context_3__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_5pycx4_5pycda_7Context_2__dealloc__(((struct __pyx_obj_5pycx4_5pycda_Context *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_5pycx4_5pycda_7Context_2__dealloc__(struct __pyx_obj_5pycx4_5pycda_Context *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "pycx4/context.pxi":43
 * 
 *     def __dealloc__(self):
 *         if self.cid > 0:             # <<<<<<<<<<<<<<
 *             self.check_exception( cda_del_context(self.cid) )
 *             self.cid = 0
 */
  __pyx_t_1 = ((__pyx_v_self->cid > 0) != 0);
  if (__pyx_t_1) {

    /* "pycx4/context.pxi":44
 *     def __dealloc__(self):
 *         if self.cid > 0:
 *             self.check_exception( cda_del_context(self.cid) )             # <<<<<<<<<<<<<<
 *             self.cid = 0
 *         if self.channum > 0:
 */
    ((struct __pyx_vtabstruct_5pycx4_5pycda_Context *)__pyx_v_self->__pyx_base.__pyx_vtab)->check_exception(__pyx_v_self, cda_del_context(__pyx_v_self->cid));

    /* "pycx4/context.pxi":45
 *         if self.cid > 0:
 *             self.check_exception( cda_del_context(self.cid) )
 *             self.cid = 0             # <<<<<<<<<<<<<<
 *         if self.channum > 0:
 *             free(self.chans)
 */
    __pyx_v_self->cid = 0;

    /* "pycx4/context.pxi":43
 * 
 *     def __dealloc__(self):
 *         if self.cid > 0:             # <<<<<<<<<<<<<<
 *             self.check_exception( cda_del_context(self.cid) )
 *             self.cid = 0
 */
  }

  /* "pycx4/context.pxi":46
 *             self.check_exception( cda_del_context(self.cid) )
 *             self.cid = 0
 *         if self.channum > 0:             # <<<<<<<<<<<<<<
 *             free(self.chans)
 *             self.chans = NULL
 */
  __pyx_t_1 = ((__pyx_v_self->channum > 0) != 0);
  if (__pyx_t_1) {

    /* "pycx4/context.pxi":47
 *             self.cid = 0
 *         if self.channum > 0:
 *             free(self.chans)             # <<<<<<<<<<<<<<
 *             self.chans = NULL
 *             self.channum = 0
 */
    free(__pyx_v_self->chans);

    /* "pycx4/context.pxi":48
 *         if self.channum > 0:
 *             free(self.chans)
 *             self.chans = NULL             # <<<<<<<<<<<<<<
 *             self.channum = 0
 * 
 */
    __pyx_v_self->chans = NULL;

    /* "pycx4/context.pxi":49
 *             free(self.chans)
 *             self.chans = NULL
 *             self.channum = 0             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
    __pyx_v_self->channum = 0;

    /* "pycx4/context.pxi":46
 *             self.check_exception( cda_del_context(self.cid) )
 *             self.cid = 0
 *         if self.channum > 0:             # <<<<<<<<<<<<<<
 *             free(self.chans)
 *             self.chans = NULL
 */
  }

  /* "pycx4/context.pxi":42
 *         self.serverCycle = Signal(object)
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if self.cid > 0:
 *             self.check_exception( cda_del_context(self.cid) )
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/context.pxi":51
 *             self.channum = 0
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return '<CdaContext: cid=%d, defpfx=%s, channum=%d>' % (self.cid, self.defpfx, self.channum)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_7Context_5__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_7Context_5__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_7Context_4__str__(((struct __pyx_obj_5pycx4_5pycda_Context *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_7Context_4__str__(struct __pyx_obj_5pycx4_5pycda_Context *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  Py_UCS4 __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "pycx4/context.pxi":52
 * 
 *     def __str__(self):
 *         return '<CdaContext: cid=%d, defpfx=%s, channum=%d>' % (self.cid, self.defpfx, self.channum)             # <<<<<<<<<<<<<<
 * 
 *     def enable_serverCycle(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyTuple_New(7); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 52, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = 0;
  __pyx_t_3 = 127;
  __Pyx_INCREF(__pyx_kp_u_CdaContext_cid);
  __pyx_t_2 += 17;
  __Pyx_GIVEREF(__pyx_kp_u_CdaContext_cid);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_kp_u_CdaContext_cid);
  __pyx_t_4 = __Pyx_PyInt_From_cda_context_t(__pyx_v_self->cid); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 52, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_Format(__pyx_t_4, __pyx_n_u_d); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 52, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_3 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_5) > __pyx_t_3) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_5) : __pyx_t_3;
  __pyx_t_2 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_5);
  __pyx_t_5 = 0;
  __Pyx_INCREF(__pyx_kp_u_defpfx_2);
  __pyx_t_2 += 9;
  __Pyx_GIVEREF(__pyx_kp_u_defpfx_2);
  PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_kp_u_defpfx_2);
  __pyx_t_5 = __Pyx_PyUnicode_Unicode(__pyx_v_self->defpfx); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 52, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_5) > __pyx_t_3) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_5) : __pyx_t_3;
  __pyx_t_2 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_1, 3, __pyx_t_5);
  __pyx_t_5 = 0;
  __Pyx_INCREF(__pyx_kp_u_channum);
  __pyx_t_2 += 10;
  __Pyx_GIVEREF(__pyx_kp_u_channum);
  PyTuple_SET_ITEM(__pyx_t_1, 4, __pyx_kp_u_channum);
  __pyx_t_5 = __Pyx_PyUnicode_From_int(__pyx_v_self->channum, 0, ' ', 'd'); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 52, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_1, 5, __pyx_t_5);
  __pyx_t_5 = 0;
  __Pyx_INCREF(__pyx_kp_u__10);
  __pyx_t_2 += 1;
  __Pyx_GIVEREF(__pyx_kp_u__10);
  PyTuple_SET_ITEM(__pyx_t_1, 6, __pyx_kp_u__10);
  __pyx_t_5 = __Pyx_PyUnicode_Join(__pyx_t_1, 7, __pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 52, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "pycx4/context.pxi":51
 *             self.channum = 0
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return '<CdaContext: cid=%d, defpfx=%s, channum=%d>' % (self.cid, self.defpfx, self.channum)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pycx4.pycda.Context.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/context.pxi":54
 *         return '<CdaContext: cid=%d, defpfx=%s, channum=%d>' % (self.cid, self.defpfx, self.channum)
 * 
 *     def enable_serverCycle(self):             # <<<<<<<<<<<<<<
 *         self.add_event(CDA_CTX_EVMASK_CYCLE, <void*>evproc_cont_cycle, <void*>self, NULL)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_7Context_7enable_serverCycle(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_7Context_7enable_serverCycle(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("enable_serverCycle (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_7Context_6enable_serverCycle(((struct __pyx_obj_5pycx4_5pycda_Context *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_7Context_6enable_serverCycle(struct __pyx_obj_5pycx4_5pycda_Context *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("enable_serverCycle", 0);

  /* "pycx4/context.pxi":55
 * 
 *     def enable_serverCycle(self):
 *         self.add_event(CDA_CTX_EVMASK_CYCLE, <void*>evproc_cont_cycle, <void*>self, NULL)             # <<<<<<<<<<<<<<
 * 
 *     cdef void save_chan(self, void *chan):
 */
  (void)(((struct __pyx_vtabstruct_5pycx4_5pycda_Context *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base.add_event(((struct __pyx_obj_5pycx4_5pycda_CdaObject *)__pyx_v_self), CDA_CTX_EVMASK_CYCLE, ((void *)__pyx_f_5pycx4_5pycda_evproc_cont_cycle), ((void *)__pyx_v_self), NULL));

  /* "pycx4/context.pxi":54
 *         return '<CdaContext: cid=%d, defpfx=%s, channum=%d>' % (self.cid, self.defpfx, self.channum)
 * 
 *     def enable_serverCycle(self):             # <<<<<<<<<<<<<<
 *         self.add_event(CDA_CTX_EVMASK_CYCLE, <void*>evproc_cont_cycle, <void*>self, NULL)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/context.pxi":57
 *         self.add_event(CDA_CTX_EVMASK_CYCLE, <void*>evproc_cont_cycle, <void*>self, NULL)
 * 
 *     cdef void save_chan(self, void *chan):             # <<<<<<<<<<<<<<
 *         cdef:
 *             void *tmp = realloc(<void*>self.chans, sizeof(void*) * (self.channum+1))
 */

static void __pyx_f_5pycx4_5pycda_7Context_save_chan(struct __pyx_obj_5pycx4_5pycda_Context *__pyx_v_self, void *__pyx_v_chan) {
  void *__pyx_v_tmp;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("save_chan", 0);

  /* "pycx4/context.pxi":59
 *     cdef void save_chan(self, void *chan):
 *         cdef:
 *             void *tmp = realloc(<void*>self.chans, sizeof(void*) * (self.channum+1))             # <<<<<<<<<<<<<<
 *         if not tmp: raise MemoryError()
 *         self.chans = <void**>tmp
 */
  __pyx_v_tmp = realloc(((void *)__pyx_v_self->chans), ((sizeof(void *)) * (__pyx_v_self->channum + 1)));

  /* "pycx4/context.pxi":60
 *         cdef:
 *             void *tmp = realloc(<void*>self.chans, sizeof(void*) * (self.channum+1))
 *         if not tmp: raise MemoryError()             # <<<<<<<<<<<<<<
 *         self.chans = <void**>tmp
 *         self.chans[self.channum] = chan
 */
  __pyx_t_1 = ((!(__pyx_v_tmp != 0)) != 0);
  if (unlikely(__pyx_t_1)) {
    PyErr_NoMemory(); __PYX_ERR(3, 60, __pyx_L1_error)
  }

  /* "pycx4/context.pxi":61
 *             void *tmp = realloc(<void*>self.chans, sizeof(void*) * (self.channum+1))
 *         if not tmp: raise MemoryError()
 *         self.chans = <void**>tmp             # <<<<<<<<<<<<<<
 *         self.chans[self.channum] = chan
 *         self.channum += 1
 */
  __pyx_v_self->chans = ((void **)__pyx_v_tmp);

  /* "pycx4/context.pxi":62
 *         if not tmp: raise MemoryError()
 *         self.chans = <void**>tmp
 *         self.chans[self.channum] = chan             # <<<<<<<<<<<<<<
 *         self.channum += 1
 * 
 */
  (__pyx_v_self->chans[__pyx_v_self->channum]) = __pyx_v_chan;

  /* "pycx4/context.pxi":63
 *         self.chans = <void**>tmp
 *         self.chans[self.channum] = chan
 *         self.channum += 1             # <<<<<<<<<<<<<<
 * 
 *     cdef void drop_chan(self, void *chan):
 */
  __pyx_v_self->channum = (__pyx_v_self->channum + 1);

  /* "pycx4/context.pxi":57
 *         self.add_event(CDA_CTX_EVMASK_CYCLE, <void*>evproc_cont_cycle, <void*>self, NULL)
 * 
 *     cdef void save_chan(self, void *chan):             # <<<<<<<<<<<<<<
 *         cdef:
 *             void *tmp = realloc(<void*>self.chans, sizeof(void*) * (self.channum+1))
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("pycx4.pycda.Context.save_chan", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/context.pxi":65
 *         self.channum += 1
 * 
 *     cdef void drop_chan(self, void *chan):             # <<<<<<<<<<<<<<
 *         if not self.cid: return
 *         cdef:
 */

static void __pyx_f_5pycx4_5pycda_7Context_drop_chan(struct __pyx_obj_5pycx4_5pycda_Context *__pyx_v_self, void *__pyx_v_chan) {
  void *__pyx_v_tmp;
  int __pyx_v_ind;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  __Pyx_RefNannySetupContext("drop_chan", 0);

  /* "pycx4/context.pxi":66
 * 
 *     cdef void drop_chan(self, void *chan):
 *         if not self.cid: return             # <<<<<<<<<<<<<<
 *         cdef:
 *             void *tmp
 */
  __pyx_t_1 = ((!(__pyx_v_self->cid != 0)) != 0);
  if (__pyx_t_1) {
    goto __pyx_L0;
  }

  /* "pycx4/context.pxi":70
 *             void *tmp
 *             int ind
 *         for ind in range(self.channum):             # <<<<<<<<<<<<<<
 *             if chan == self.chans[ind]:
 *                 if self.channum == 1:
 */
  __pyx_t_2 = __pyx_v_self->channum;
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_ind = __pyx_t_4;

    /* "pycx4/context.pxi":71
 *             int ind
 *         for ind in range(self.channum):
 *             if chan == self.chans[ind]:             # <<<<<<<<<<<<<<
 *                 if self.channum == 1:
 *                     free(self.chans)
 */
    __pyx_t_1 = ((__pyx_v_chan == (__pyx_v_self->chans[__pyx_v_ind])) != 0);
    if (__pyx_t_1) {

      /* "pycx4/context.pxi":72
 *         for ind in range(self.channum):
 *             if chan == self.chans[ind]:
 *                 if self.channum == 1:             # <<<<<<<<<<<<<<
 *                     free(self.chans)
 *                     self.chans = NULL
 */
      __pyx_t_1 = ((__pyx_v_self->channum == 1) != 0);
      if (__pyx_t_1) {

        /* "pycx4/context.pxi":73
 *             if chan == self.chans[ind]:
 *                 if self.channum == 1:
 *                     free(self.chans)             # <<<<<<<<<<<<<<
 *                     self.chans = NULL
 *                 else:
 */
        free(__pyx_v_self->chans);

        /* "pycx4/context.pxi":74
 *                 if self.channum == 1:
 *                     free(self.chans)
 *                     self.chans = NULL             # <<<<<<<<<<<<<<
 *                 else:
 *                     memmove(&(self.chans[ind]), &(self.chans[self.channum-1]), sizeof(void*) )
 */
        __pyx_v_self->chans = NULL;

        /* "pycx4/context.pxi":72
 *         for ind in range(self.channum):
 *             if chan == self.chans[ind]:
 *                 if self.channum == 1:             # <<<<<<<<<<<<<<
 *                     free(self.chans)
 *                     self.chans = NULL
 */
        goto __pyx_L7;
      }

      /* "pycx4/context.pxi":76
 *                     self.chans = NULL
 *                 else:
 *                     memmove(&(self.chans[ind]), &(self.chans[self.channum-1]), sizeof(void*) )             # <<<<<<<<<<<<<<
 *                     tmp = realloc(<void*>self.chans, sizeof(void*) * (self.channum-1))
 *                     if not tmp and self.channum>1: raise MemoryError()
 */
      /*else*/ {
        (void)(memmove((&(__pyx_v_self->chans[__pyx_v_ind])), (&(__pyx_v_self->chans[(__pyx_v_self->channum - 1)])), (sizeof(void *))));

        /* "pycx4/context.pxi":77
 *                 else:
 *                     memmove(&(self.chans[ind]), &(self.chans[self.channum-1]), sizeof(void*) )
 *                     tmp = realloc(<void*>self.chans, sizeof(void*) * (self.channum-1))             # <<<<<<<<<<<<<<
 *                     if not tmp and self.channum>1: raise MemoryError()
 *                     self.chans = <void**>tmp
 */
        __pyx_v_tmp = realloc(((void *)__pyx_v_self->chans), ((sizeof(void *)) * (__pyx_v_self->channum - 1)));

        /* "pycx4/context.pxi":78
 *                     memmove(&(self.chans[ind]), &(self.chans[self.channum-1]), sizeof(void*) )
 *                     tmp = realloc(<void*>self.chans, sizeof(void*) * (self.channum-1))
 *                     if not tmp and self.channum>1: raise MemoryError()             # <<<<<<<<<<<<<<
 *                     self.chans = <void**>tmp
 *                 self.channum -= 1
 */
        __pyx_t_5 = ((!(__pyx_v_tmp != 0)) != 0);
        if (__pyx_t_5) {
        } else {
          __pyx_t_1 = __pyx_t_5;
          goto __pyx_L9_bool_binop_done;
        }
        __pyx_t_5 = ((__pyx_v_self->channum > 1) != 0);
        __pyx_t_1 = __pyx_t_5;
        __pyx_L9_bool_binop_done:;
        if (unlikely(__pyx_t_1)) {
          PyErr_NoMemory(); __PYX_ERR(3, 78, __pyx_L1_error)
        }

        /* "pycx4/context.pxi":79
 *                     tmp = realloc(<void*>self.chans, sizeof(void*) * (self.channum-1))
 *                     if not tmp and self.channum>1: raise MemoryError()
 *                     self.chans = <void**>tmp             # <<<<<<<<<<<<<<
 *                 self.channum -= 1
 *                 return
 */
        __pyx_v_self->chans = ((void **)__pyx_v_tmp);
      }
      __pyx_L7:;

      /* "pycx4/context.pxi":80
 *                     if not tmp and self.channum>1: raise MemoryError()
 *                     self.chans = <void**>tmp
 *                 self.channum -= 1             # <<<<<<<<<<<<<<
 *                 return
 * 
 */
      __pyx_v_self->channum = (__pyx_v_self->channum - 1);

      /* "pycx4/context.pxi":81
 *                     self.chans = <void**>tmp
 *                 self.channum -= 1
 *                 return             # <<<<<<<<<<<<<<
 * 
 *     cdef void register_event(self, event *ev):
 */
      goto __pyx_L0;

      /* "pycx4/context.pxi":71
 *             int ind
 *         for ind in range(self.channum):
 *             if chan == self.chans[ind]:             # <<<<<<<<<<<<<<
 *                 if self.channum == 1:
 *                     free(self.chans)
 */
    }
  }

  /* "pycx4/context.pxi":65
 *         self.channum += 1
 * 
 *     cdef void drop_chan(self, void *chan):             # <<<<<<<<<<<<<<
 *         if not self.cid: return
 *         cdef:
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("pycx4.pycda.Context.drop_chan", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/context.pxi":83
 *                 return
 * 
 *     cdef void register_event(self, event *ev):             # <<<<<<<<<<<<<<
 *         self.check_exception( cda_add_context_evproc(self.cid, ev.evmask, <cda_context_evproc_t>ev.evproc, ev) )
 * 
 */

static void __pyx_f_5pycx4_5pycda_7Context_register_event(struct __pyx_obj_5pycx4_5pycda_Context *__pyx_v_self, __pyx_t_5pycx4_5pycda_event *__pyx_v_ev) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("register_event", 0);

  /* "pycx4/context.pxi":84
 * 
 *     cdef void register_event(self, event *ev):
 *         self.check_exception( cda_add_context_evproc(self.cid, ev.evmask, <cda_context_evproc_t>ev.evproc, ev) )             # <<<<<<<<<<<<<<
 * 
 *     cdef void unregister_event(self, event *ev):
 */
  ((struct __pyx_vtabstruct_5pycx4_5pycda_Context *)__pyx_v_self->__pyx_base.__pyx_vtab)->check_exception(__pyx_v_self, cda_add_context_evproc(__pyx_v_self->cid, __pyx_v_ev->evmask, ((cda_context_evproc_t)__pyx_v_ev->evproc), __pyx_v_ev));

  /* "pycx4/context.pxi":83
 *                 return
 * 
 *     cdef void register_event(self, event *ev):             # <<<<<<<<<<<<<<
 *         self.check_exception( cda_add_context_evproc(self.cid, ev.evmask, <cda_context_evproc_t>ev.evproc, ev) )
 * 
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/context.pxi":86
 *         self.check_exception( cda_add_context_evproc(self.cid, ev.evmask, <cda_context_evproc_t>ev.evproc, ev) )
 * 
 *     cdef void unregister_event(self, event *ev):             # <<<<<<<<<<<<<<
 *         self.check_exception( cda_del_context_evproc(self.cid, ev.evmask, <cda_context_evproc_t>ev.evproc, ev) )
 * 
 */

static void __pyx_f_5pycx4_5pycda_7Context_unregister_event(struct __pyx_obj_5pycx4_5pycda_Context *__pyx_v_self, __pyx_t_5pycx4_5pycda_event *__pyx_v_ev) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("unregister_event", 0);

  /* "pycx4/context.pxi":87
 * 
 *     cdef void unregister_event(self, event *ev):
 *         self.check_exception( cda_del_context_evproc(self.cid, ev.evmask, <cda_context_evproc_t>ev.evproc, ev) )             # <<<<<<<<<<<<<<
 * 
 *     cdef void check_exception(self, int c_res):
 */
  ((struct __pyx_vtabstruct_5pycx4_5pycda_Context *)__pyx_v_self->__pyx_base.__pyx_vtab)->check_exception(__pyx_v_self, cda_del_context_evproc(__pyx_v_self->cid, __pyx_v_ev->evmask, ((cda_context_evproc_t)__pyx_v_ev->evproc), __pyx_v_ev));

  /* "pycx4/context.pxi":86
 *         self.check_exception( cda_add_context_evproc(self.cid, ev.evmask, <cda_context_evproc_t>ev.evproc, ev) )
 * 
 *     cdef void unregister_event(self, event *ev):             # <<<<<<<<<<<<<<
 *         self.check_exception( cda_del_context_evproc(self.cid, ev.evmask, <cda_context_evproc_t>ev.evproc, ev) )
 * 
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/context.pxi":89
 *         self.check_exception( cda_del_context_evproc(self.cid, ev.evmask, <cda_context_evproc_t>ev.evproc, ev) )
 * 
 *     cdef void check_exception(self, int c_res):             # <<<<<<<<<<<<<<
 *         if c_res < 0:
 *             raise Exception("cda context error: cid=%s, %s, errcode=%s" % (self.cid, cda_last_err(), c_res ))
 */

static void __pyx_f_5pycx4_5pycda_7Context_check_exception(struct __pyx_obj_5pycx4_5pycda_Context *__pyx_v_self, int __pyx_v_c_res) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  Py_UCS4 __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("check_exception", 0);

  /* "pycx4/context.pxi":90
 * 
 *     cdef void check_exception(self, int c_res):
 *         if c_res < 0:             # <<<<<<<<<<<<<<
 *             raise Exception("cda context error: cid=%s, %s, errcode=%s" % (self.cid, cda_last_err(), c_res ))
 */
  __pyx_t_1 = ((__pyx_v_c_res < 0) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "pycx4/context.pxi":91
 *     cdef void check_exception(self, int c_res):
 *         if c_res < 0:
 *             raise Exception("cda context error: cid=%s, %s, errcode=%s" % (self.cid, cda_last_err(), c_res ))             # <<<<<<<<<<<<<<
 */
    __pyx_t_2 = PyTuple_New(6); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 91, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = 0;
    __pyx_t_4 = 127;
    __Pyx_INCREF(__pyx_kp_u_cda_context_error_cid);
    __pyx_t_3 += 23;
    __Pyx_GIVEREF(__pyx_kp_u_cda_context_error_cid);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_kp_u_cda_context_error_cid);
    __pyx_t_5 = __Pyx_PyInt_From_cda_context_t(__pyx_v_self->cid); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 91, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Unicode(__pyx_t_5), __pyx_empty_unicode); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 91, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_4 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_6) > __pyx_t_4) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_6) : __pyx_t_4;
    __pyx_t_3 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_6);
    __pyx_t_6 = 0;
    __Pyx_INCREF(__pyx_kp_u__11);
    __pyx_t_3 += 2;
    __Pyx_GIVEREF(__pyx_kp_u__11);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_kp_u__11);
    __pyx_t_6 = __Pyx_PyBytes_FromString(cda_last_err()); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 91, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_5 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Unicode(__pyx_t_6), __pyx_empty_unicode); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 91, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_4 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_5) > __pyx_t_4) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_5) : __pyx_t_4;
    __pyx_t_3 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_2, 3, __pyx_t_5);
    __pyx_t_5 = 0;
    __Pyx_INCREF(__pyx_kp_u_errcode);
    __pyx_t_3 += 10;
    __Pyx_GIVEREF(__pyx_kp_u_errcode);
    PyTuple_SET_ITEM(__pyx_t_2, 4, __pyx_kp_u_errcode);
    __pyx_t_5 = __Pyx_PyUnicode_From_int(__pyx_v_c_res, 0, ' ', 'd'); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 91, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_3 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_2, 5, __pyx_t_5);
    __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyUnicode_Join(__pyx_t_2, 6, __pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 91, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])), __pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 91, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(3, 91, __pyx_L1_error)

    /* "pycx4/context.pxi":90
 * 
 *     cdef void check_exception(self, int c_res):
 *         if c_res < 0:             # <<<<<<<<<<<<<<
 *             raise Exception("cda context error: cid=%s, %s, errcode=%s" % (self.cid, cda_last_err(), c_res ))
 */
  }

  /* "pycx4/context.pxi":89
 *         self.check_exception( cda_del_context_evproc(self.cid, ev.evmask, <cda_context_evproc_t>ev.evproc, ev) )
 * 
 *     cdef void check_exception(self, int c_res):             # <<<<<<<<<<<<<<
 *         if c_res < 0:
 *             raise Exception("cda context error: cid=%s, %s, errcode=%s" % (self.cid, cda_last_err(), c_res ))
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_WriteUnraisable("pycx4.pycda.Context.check_exception", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/context.pxi":11
 * cdef class Context(CdaObject):
 *     cdef readonly:
 *         cda_context_t cid             # <<<<<<<<<<<<<<
 *         str defpfx
 *     cdef public:
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_7Context_3cid_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_7Context_3cid_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_7Context_3cid___get__(((struct __pyx_obj_5pycx4_5pycda_Context *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_7Context_3cid___get__(struct __pyx_obj_5pycx4_5pycda_Context *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_cda_context_t(__pyx_v_self->cid); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 11, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.Context.cid.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/context.pxi":12
 *     cdef readonly:
 *         cda_context_t cid
 *         str defpfx             # <<<<<<<<<<<<<<
 *     cdef public:
 *         object serverCycle
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_7Context_6defpfx_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_7Context_6defpfx_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_7Context_6defpfx___get__(((struct __pyx_obj_5pycx4_5pycda_Context *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_7Context_6defpfx___get__(struct __pyx_obj_5pycx4_5pycda_Context *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->defpfx);
  __pyx_r = __pyx_v_self->defpfx;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/context.pxi":14
 *         str defpfx
 *     cdef public:
 *         object serverCycle             # <<<<<<<<<<<<<<
 *     cdef:
 *         void **chans
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_7Context_11serverCycle_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_7Context_11serverCycle_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_7Context_11serverCycle___get__(((struct __pyx_obj_5pycx4_5pycda_Context *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_7Context_11serverCycle___get__(struct __pyx_obj_5pycx4_5pycda_Context *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->serverCycle);
  __pyx_r = __pyx_v_self->serverCycle;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5pycx4_5pycda_7Context_11serverCycle_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5pycx4_5pycda_7Context_11serverCycle_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_7Context_11serverCycle_2__set__(((struct __pyx_obj_5pycx4_5pycda_Context *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pycx4_5pycda_7Context_11serverCycle_2__set__(struct __pyx_obj_5pycx4_5pycda_Context *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->serverCycle);
  __Pyx_DECREF(__pyx_v_self->serverCycle);
  __pyx_v_self->serverCycle = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5pycx4_5pycda_7Context_11serverCycle_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5pycx4_5pycda_7Context_11serverCycle_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_7Context_11serverCycle_4__del__(((struct __pyx_obj_5pycx4_5pycda_Context *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pycx4_5pycda_7Context_11serverCycle_4__del__(struct __pyx_obj_5pycx4_5pycda_Context *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->serverCycle);
  __Pyx_DECREF(__pyx_v_self->serverCycle);
  __pyx_v_self->serverCycle = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.chans,self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_7Context_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_7Context_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_7Context_8__reduce_cython__(((struct __pyx_obj_5pycx4_5pycda_Context *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_7Context_8__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_Context *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.chans,self.events cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.chans,self.events cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__12, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.chans,self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.Context.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.chans,self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.chans,self.events cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_7Context_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_7Context_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_7Context_10__setstate_cython__(((struct __pyx_obj_5pycx4_5pycda_Context *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_7Context_10__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_Context *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self.chans,self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.chans,self.events cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__13, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.chans,self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.chans,self.events cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.Context.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/basechan.pxi":4
 * 
 * # C callback function for ref's (channels)
 * cdef void evproc_rslvstat(int uniq, void *privptr1, cda_dataref_t ref, int reason,             # <<<<<<<<<<<<<<
 *                           void *info_ptr, void *privptr2) with gil:
 *     cdef BaseChan chan = <BaseChan>(<event*>privptr2).objptr
 */

static void __pyx_f_5pycx4_5pycda_evproc_rslvstat(CYTHON_UNUSED int __pyx_v_uniq, CYTHON_UNUSED void *__pyx_v_privptr1, CYTHON_UNUSED cda_dataref_t __pyx_v_ref, CYTHON_UNUSED int __pyx_v_reason, void *__pyx_v_info_ptr, void *__pyx_v_privptr2) {
  struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_chan = 0;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("evproc_rslvstat", 0);

  /* "pycx4/basechan.pxi":6
 * cdef void evproc_rslvstat(int uniq, void *privptr1, cda_dataref_t ref, int reason,
 *                           void *info_ptr, void *privptr2) with gil:
 *     cdef BaseChan chan = <BaseChan>(<event*>privptr2).objptr             # <<<<<<<<<<<<<<
 *     chan.rslv_stat = <long>info_ptr # resolve status passed through this pointer
 * 
 */
  __pyx_t_1 = ((PyObject *)((__pyx_t_5pycx4_5pycda_event *)__pyx_v_privptr2)->objptr);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_chan = ((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pycx4/basechan.pxi":7
 *                           void *info_ptr, void *privptr2) with gil:
 *     cdef BaseChan chan = <BaseChan>(<event*>privptr2).objptr
 *     chan.rslv_stat = <long>info_ptr # resolve status passed through this pointer             # <<<<<<<<<<<<<<
 * 
 *     if chan.rslv_stat == CDA_RSLVSTAT_NOTFOUND:
 */
  __pyx_v_chan->rslv_stat = ((long)__pyx_v_info_ptr);

  /* "pycx4/basechan.pxi":9
 *     chan.rslv_stat = <long>info_ptr # resolve status passed through this pointer
 * 
 *     if chan.rslv_stat == CDA_RSLVSTAT_NOTFOUND:             # <<<<<<<<<<<<<<
 *         chan.rslv_str = 'not found'
 *     elif chan.rslv_stat == CDA_RSLVSTAT_SEARCHING:
 */
  switch (__pyx_v_chan->rslv_stat) {
    case CDA_RSLVSTAT_NOTFOUND:

    /* "pycx4/basechan.pxi":10
 * 
 *     if chan.rslv_stat == CDA_RSLVSTAT_NOTFOUND:
 *         chan.rslv_str = 'not found'             # <<<<<<<<<<<<<<
 *     elif chan.rslv_stat == CDA_RSLVSTAT_SEARCHING:
 *         chan.rslv_str = 'searching'
 */
    __Pyx_INCREF(__pyx_kp_u_not_found);
    __Pyx_GIVEREF(__pyx_kp_u_not_found);
    __Pyx_GOTREF(__pyx_v_chan->rslv_str);
    __Pyx_DECREF(__pyx_v_chan->rslv_str);
    __pyx_v_chan->rslv_str = __pyx_kp_u_not_found;

    /* "pycx4/basechan.pxi":9
 *     chan.rslv_stat = <long>info_ptr # resolve status passed through this pointer
 * 
 *     if chan.rslv_stat == CDA_RSLVSTAT_NOTFOUND:             # <<<<<<<<<<<<<<
 *         chan.rslv_str = 'not found'
 *     elif chan.rslv_stat == CDA_RSLVSTAT_SEARCHING:
 */
    break;
    case CDA_RSLVSTAT_SEARCHING:

    /* "pycx4/basechan.pxi":12
 *         chan.rslv_str = 'not found'
 *     elif chan.rslv_stat == CDA_RSLVSTAT_SEARCHING:
 *         chan.rslv_str = 'searching'             # <<<<<<<<<<<<<<
 *     elif chan.rslv_stat == CDA_RSLVSTAT_FOUND:
 *         chan.rslv_str = 'found'
 */
    __Pyx_INCREF(__pyx_n_u_searching);
    __Pyx_GIVEREF(__pyx_n_u_searching);
    __Pyx_GOTREF(__pyx_v_chan->rslv_str);
    __Pyx_DECREF(__pyx_v_chan->rslv_str);
    __pyx_v_chan->rslv_str = __pyx_n_u_searching;

    /* "pycx4/basechan.pxi":11
 *     if chan.rslv_stat == CDA_RSLVSTAT_NOTFOUND:
 *         chan.rslv_str = 'not found'
 *     elif chan.rslv_stat == CDA_RSLVSTAT_SEARCHING:             # <<<<<<<<<<<<<<
 *         chan.rslv_str = 'searching'
 *     elif chan.rslv_stat == CDA_RSLVSTAT_FOUND:
 */
    break;
    case CDA_RSLVSTAT_FOUND:

    /* "pycx4/basechan.pxi":14
 *         chan.rslv_str = 'searching'
 *     elif chan.rslv_stat == CDA_RSLVSTAT_FOUND:
 *         chan.rslv_str = 'found'             # <<<<<<<<<<<<<<
 *     chan.resolve.emit(chan)
 * 
 */
    __Pyx_INCREF(__pyx_n_u_found);
    __Pyx_GIVEREF(__pyx_n_u_found);
    __Pyx_GOTREF(__pyx_v_chan->rslv_str);
    __Pyx_DECREF(__pyx_v_chan->rslv_str);
    __pyx_v_chan->rslv_str = __pyx_n_u_found;

    /* "pycx4/basechan.pxi":13
 *     elif chan.rslv_stat == CDA_RSLVSTAT_SEARCHING:
 *         chan.rslv_str = 'searching'
 *     elif chan.rslv_stat == CDA_RSLVSTAT_FOUND:             # <<<<<<<<<<<<<<
 *         chan.rslv_str = 'found'
 *     chan.resolve.emit(chan)
 */
    break;
    default: break;
  }

  /* "pycx4/basechan.pxi":15
 *     elif chan.rslv_stat == CDA_RSLVSTAT_FOUND:
 *         chan.rslv_str = 'found'
 *     chan.resolve.emit(chan)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_chan->resolve, __pyx_n_s_emit); if (unlikely(!__pyx_t_2)) __PYX_ERR(9, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, ((PyObject *)__pyx_v_chan)) : __Pyx_PyObject_CallOneArg(__pyx_t_2, ((PyObject *)__pyx_v_chan));
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/basechan.pxi":4
 * 
 * # C callback function for ref's (channels)
 * cdef void evproc_rslvstat(int uniq, void *privptr1, cda_dataref_t ref, int reason,             # <<<<<<<<<<<<<<
 *                           void *info_ptr, void *privptr2) with gil:
 *     cdef BaseChan chan = <BaseChan>(<event*>privptr2).objptr
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_WriteUnraisable("pycx4.pycda.evproc_rslvstat", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_chan);
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
}

/* "pycx4/basechan.pxi":18
 * 
 * 
 * cdef void evproc_update(int uniq, void *privptr1, cda_dataref_t ref, int reason,             # <<<<<<<<<<<<<<
 *                         void *info_ptr, void *privptr2) with gil:
 *     cdef:
 */

static void __pyx_f_5pycx4_5pycda_evproc_update(CYTHON_UNUSED int __pyx_v_uniq, CYTHON_UNUSED void *__pyx_v_privptr1, cda_dataref_t __pyx_v_ref, CYTHON_UNUSED int __pyx_v_reason, CYTHON_UNUSED void *__pyx_v_info_ptr, void *__pyx_v_privptr2) {
  cx_time_t __pyx_v_timestr;
  struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_chan = 0;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int64 __pyx_t_2;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("evproc_update", 0);

  /* "pycx4/basechan.pxi":22
 *     cdef:
 *         cx_time_t timestr
 *         BaseChan chan = <BaseChan>(<event*>privptr2).objptr             # <<<<<<<<<<<<<<
 *     chan.check_exception( cda_get_ref_stat(ref, &chan.rflags, &timestr) )
 * 
 */
  __pyx_t_1 = ((PyObject *)((__pyx_t_5pycx4_5pycda_event *)__pyx_v_privptr2)->objptr);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_chan = ((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pycx4/basechan.pxi":23
 *         cx_time_t timestr
 *         BaseChan chan = <BaseChan>(<event*>privptr2).objptr
 *     chan.check_exception( cda_get_ref_stat(ref, &chan.rflags, &timestr) )             # <<<<<<<<<<<<<<
 * 
 *     chan.prev_time = chan.time
 */
  ((struct __pyx_vtabstruct_5pycx4_5pycda_BaseChan *)__pyx_v_chan->__pyx_base.__pyx_vtab)->check_exception(__pyx_v_chan, cda_get_ref_stat(__pyx_v_ref, (&__pyx_v_chan->rflags), (&__pyx_v_timestr)));

  /* "pycx4/basechan.pxi":25
 *     chan.check_exception( cda_get_ref_stat(ref, &chan.rflags, &timestr) )
 * 
 *     chan.prev_time = chan.time             # <<<<<<<<<<<<<<
 *     chan.time = <int64>timestr.sec * 1000000 + timestr.nsec / 1000
 *     chan.cb()
 */
  __pyx_t_2 = __pyx_v_chan->time;
  __pyx_v_chan->prev_time = __pyx_t_2;

  /* "pycx4/basechan.pxi":26
 * 
 *     chan.prev_time = chan.time
 *     chan.time = <int64>timestr.sec * 1000000 + timestr.nsec / 1000             # <<<<<<<<<<<<<<
 *     chan.cb()
 * 
 */
  __pyx_v_chan->time = ((((int64)__pyx_v_timestr.sec) * 0xF4240) + (((long)__pyx_v_timestr.nsec) / 0x3E8));

  /* "pycx4/basechan.pxi":27
 *     chan.prev_time = chan.time
 *     chan.time = <int64>timestr.sec * 1000000 + timestr.nsec / 1000
 *     chan.cb()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  ((struct __pyx_vtabstruct_5pycx4_5pycda_BaseChan *)__pyx_v_chan->__pyx_base.__pyx_vtab)->cb(__pyx_v_chan);

  /* "pycx4/basechan.pxi":18
 * 
 * 
 * cdef void evproc_update(int uniq, void *privptr1, cda_dataref_t ref, int reason,             # <<<<<<<<<<<<<<
 *                         void *info_ptr, void *privptr2) with gil:
 *     cdef:
 */

  /* function exit code */
  __Pyx_XDECREF((PyObject *)__pyx_v_chan);
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
}

/* "pycx4/basechan.pxi":30
 * 
 * 
 * cdef void quant_update(int uniq, void *privptr1, cda_dataref_t ref, int reason,             # <<<<<<<<<<<<<<
 *                         void *info_ptr, void *privptr2) with gil:
 *     cdef:
 */

static void __pyx_f_5pycx4_5pycda_quant_update(CYTHON_UNUSED int __pyx_v_uniq, CYTHON_UNUSED void *__pyx_v_privptr1, cda_dataref_t __pyx_v_ref, CYTHON_UNUSED int __pyx_v_reason, CYTHON_UNUSED void *__pyx_v_info_ptr, void *__pyx_v_privptr2) {
  CYTHON_UNUSED int __pyx_v_res;
  CxAnyVal_t __pyx_v_quant_raw;
  cxdtype_t __pyx_v_quant_dtype;
  struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_chan = 0;
  double __pyx_v_quant_draw;
  double __pyx_v_val;
  double __pyx_v_shift;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  double __pyx_t_2;
  int __pyx_t_3;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("quant_update", 0);

  /* "pycx4/basechan.pxi":36
 *         CxAnyVal_t quant_raw
 *         cxdtype_t quant_dtype
 *         BaseChan chan = <BaseChan>(<event*>privptr2).objptr             # <<<<<<<<<<<<<<
 *         double quant_draw, val, shift
 * 
 */
  __pyx_t_1 = ((PyObject *)((__pyx_t_5pycx4_5pycda_event *)__pyx_v_privptr2)->objptr);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_chan = ((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pycx4/basechan.pxi":39
 *         double quant_draw, val, shift
 * 
 *     res = cda_quant_of_ref(chan.ref, &quant_raw, &quant_dtype)             # <<<<<<<<<<<<<<
 *     quant_draw = aval_value(&quant_raw, quant_dtype)
 *     if quant_draw == 0:
 */
  __pyx_v_res = cda_quant_of_ref(__pyx_v_chan->ref, (&__pyx_v_quant_raw), (&__pyx_v_quant_dtype));

  /* "pycx4/basechan.pxi":40
 * 
 *     res = cda_quant_of_ref(chan.ref, &quant_raw, &quant_dtype)
 *     quant_draw = aval_value(&quant_raw, quant_dtype)             # <<<<<<<<<<<<<<
 *     if quant_draw == 0:
 *         chan.quant = 0
 */
  __pyx_t_1 = __pyx_f_5pycx4_5pycda_aval_value((&__pyx_v_quant_raw), __pyx_v_quant_dtype); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(9, 40, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_quant_draw = __pyx_t_2;

  /* "pycx4/basechan.pxi":41
 *     res = cda_quant_of_ref(chan.ref, &quant_raw, &quant_dtype)
 *     quant_draw = aval_value(&quant_raw, quant_dtype)
 *     if quant_draw == 0:             # <<<<<<<<<<<<<<
 *         chan.quant = 0
 *         return
 */
  __pyx_t_3 = ((__pyx_v_quant_draw == 0.0) != 0);
  if (__pyx_t_3) {

    /* "pycx4/basechan.pxi":42
 *     quant_draw = aval_value(&quant_raw, quant_dtype)
 *     if quant_draw == 0:
 *         chan.quant = 0             # <<<<<<<<<<<<<<
 *         return
 *     else:
 */
    __pyx_v_chan->quant = 0.0;

    /* "pycx4/basechan.pxi":43
 *     if quant_draw == 0:
 *         chan.quant = 0
 *         return             # <<<<<<<<<<<<<<
 *     else:
 *         res = cda_rd_convert(ref, 0, &shift)
 */
    goto __pyx_L0;

    /* "pycx4/basechan.pxi":41
 *     res = cda_quant_of_ref(chan.ref, &quant_raw, &quant_dtype)
 *     quant_draw = aval_value(&quant_raw, quant_dtype)
 *     if quant_draw == 0:             # <<<<<<<<<<<<<<
 *         chan.quant = 0
 *         return
 */
  }

  /* "pycx4/basechan.pxi":45
 *         return
 *     else:
 *         res = cda_rd_convert(ref, 0, &shift)             # <<<<<<<<<<<<<<
 *         res = cda_rd_convert(ref, quant_draw, &val)
 *         chan.quant = val - shift
 */
  /*else*/ {
    __pyx_v_res = cda_rd_convert(__pyx_v_ref, 0.0, (&__pyx_v_shift));

    /* "pycx4/basechan.pxi":46
 *     else:
 *         res = cda_rd_convert(ref, 0, &shift)
 *         res = cda_rd_convert(ref, quant_draw, &val)             # <<<<<<<<<<<<<<
 *         chan.quant = val - shift
 * 
 */
    __pyx_v_res = cda_rd_convert(__pyx_v_ref, __pyx_v_quant_draw, (&__pyx_v_val));

    /* "pycx4/basechan.pxi":47
 *         res = cda_rd_convert(ref, 0, &shift)
 *         res = cda_rd_convert(ref, quant_draw, &val)
 *         chan.quant = val - shift             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_v_chan->quant = (__pyx_v_val - __pyx_v_shift);
  }

  /* "pycx4/basechan.pxi":30
 * 
 * 
 * cdef void quant_update(int uniq, void *privptr1, cda_dataref_t ref, int reason,             # <<<<<<<<<<<<<<
 *                         void *info_ptr, void *privptr2) with gil:
 *     cdef:
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_WriteUnraisable("pycx4.pycda.quant_update", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_chan);
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
}

/* "pycx4/basechan.pxi":50
 * 
 * 
 * cdef void range_update(int uniq, void *privptr1, cda_dataref_t ref, int reason,             # <<<<<<<<<<<<<<
 *                         void *info_ptr, void *privptr2) with gil:
 *     cdef:
 */

static void __pyx_f_5pycx4_5pycda_range_update(CYTHON_UNUSED int __pyx_v_uniq, CYTHON_UNUSED void *__pyx_v_privptr1, CYTHON_UNUSED cda_dataref_t __pyx_v_ref, CYTHON_UNUSED int __pyx_v_reason, CYTHON_UNUSED void *__pyx_v_info_ptr, void *__pyx_v_privptr2) {
  struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_chan = 0;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("range_update", 0);

  /* "pycx4/basechan.pxi":53
 *                         void *info_ptr, void *privptr2) with gil:
 *     cdef:
 *         BaseChan chan = <BaseChan>(<event*>privptr2).objptr             # <<<<<<<<<<<<<<
 *     chan.get_range()
 * 
 */
  __pyx_t_1 = ((PyObject *)((__pyx_t_5pycx4_5pycda_event *)__pyx_v_privptr2)->objptr);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_chan = ((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pycx4/basechan.pxi":54
 *     cdef:
 *         BaseChan chan = <BaseChan>(<event*>privptr2).objptr
 *     chan.get_range()             # <<<<<<<<<<<<<<
 * 
 * cdef void strs_update(int uniq, void *privptr1, cda_dataref_t ref, int reason,
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_5pycx4_5pycda_BaseChan *)__pyx_v_chan->__pyx_base.__pyx_vtab)->get_range(__pyx_v_chan, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 54, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/basechan.pxi":50
 * 
 * 
 * cdef void range_update(int uniq, void *privptr1, cda_dataref_t ref, int reason,             # <<<<<<<<<<<<<<
 *                         void *info_ptr, void *privptr2) with gil:
 *     cdef:
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_WriteUnraisable("pycx4.pycda.range_update", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_chan);
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
}

/* "pycx4/basechan.pxi":56
 *     chan.get_range()
 * 
 * cdef void strs_update(int uniq, void *privptr1, cda_dataref_t ref, int reason,             # <<<<<<<<<<<<<<
 *                         void *info_ptr, void *privptr2) with gil:
 *     cdef:
 */

static void __pyx_f_5pycx4_5pycda_strs_update(CYTHON_UNUSED int __pyx_v_uniq, CYTHON_UNUSED void *__pyx_v_privptr1, CYTHON_UNUSED cda_dataref_t __pyx_v_ref, CYTHON_UNUSED int __pyx_v_reason, CYTHON_UNUSED void *__pyx_v_info_ptr, void *__pyx_v_privptr2) {
  struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_chan = 0;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("strs_update", 0);

  /* "pycx4/basechan.pxi":59
 *                         void *info_ptr, void *privptr2) with gil:
 *     cdef:
 *         BaseChan chan = <BaseChan>(<event*>privptr2).objptr             # <<<<<<<<<<<<<<
 *     chan.get_strings()
 * 
 */
  __pyx_t_1 = ((PyObject *)((__pyx_t_5pycx4_5pycda_event *)__pyx_v_privptr2)->objptr);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_chan = ((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pycx4/basechan.pxi":60
 *     cdef:
 *         BaseChan chan = <BaseChan>(<event*>privptr2).objptr
 *     chan.get_strings()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_5pycx4_5pycda_BaseChan *)__pyx_v_chan->__pyx_base.__pyx_vtab)->get_strings(__pyx_v_chan, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 60, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/basechan.pxi":56
 *     chan.get_range()
 * 
 * cdef void strs_update(int uniq, void *privptr1, cda_dataref_t ref, int reason,             # <<<<<<<<<<<<<<
 *                         void *info_ptr, void *privptr2) with gil:
 *     cdef:
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_WriteUnraisable("pycx4.pycda.strs_update", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_chan);
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
}

/* "pycx4/basechan.pxi":63
 * 
 * 
 * cdef void lockstat_update(int uniq, void *privptr1, cda_dataref_t ref, int reason,             # <<<<<<<<<<<<<<
 *                         void *info_ptr, void *privptr2) with gil:
 *     cdef:
 */

static void __pyx_f_5pycx4_5pycda_lockstat_update(CYTHON_UNUSED int __pyx_v_uniq, CYTHON_UNUSED void *__pyx_v_privptr1, CYTHON_UNUSED cda_dataref_t __pyx_v_ref, CYTHON_UNUSED int __pyx_v_reason, CYTHON_UNUSED void *__pyx_v_info_ptr, void *__pyx_v_privptr2) {
  struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_chan = 0;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("lockstat_update", 0);

  /* "pycx4/basechan.pxi":66
 *                         void *info_ptr, void *privptr2) with gil:
 *     cdef:
 *         BaseChan chan = <BaseChan>(<event*>privptr2).objptr             # <<<<<<<<<<<<<<
 *     chan.get_lockstate()
 * 
 */
  __pyx_t_1 = ((PyObject *)((__pyx_t_5pycx4_5pycda_event *)__pyx_v_privptr2)->objptr);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_chan = ((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pycx4/basechan.pxi":67
 *     cdef:
 *         BaseChan chan = <BaseChan>(<event*>privptr2).objptr
 *     chan.get_lockstate()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_5pycx4_5pycda_BaseChan *)__pyx_v_chan->__pyx_base.__pyx_vtab)->get_lockstate(__pyx_v_chan, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 67, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/basechan.pxi":63
 * 
 * 
 * cdef void lockstat_update(int uniq, void *privptr1, cda_dataref_t ref, int reason,             # <<<<<<<<<<<<<<
 *                         void *info_ptr, void *privptr2) with gil:
 *     cdef:
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_WriteUnraisable("pycx4.pycda.lockstat_update", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_chan);
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
}

/* "pycx4/basechan.pxi":98
 *         object valueMeasured, valueChanged, resolve, stringsUpdate, lockStateUpdate
 * 
 *     def __init__(self, str name, **kwargs):             # <<<<<<<<<<<<<<
 *         CdaObject.__init__(self)
 *         self.context = kwargs.get('context', default_context)
 */

/* Python wrapper */
static int __pyx_pw_5pycx4_5pycda_8BaseChan_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5pycx4_5pycda_8BaseChan_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_kwargs = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  __pyx_v_kwargs = PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return -1;
  __Pyx_GOTREF(__pyx_v_kwargs);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_name,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_name)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, __pyx_v_kwargs, values, pos_args, "__init__") < 0)) __PYX_ERR(9, 98, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_name = ((PyObject*)values[0]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(9, 98, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_DECREF(__pyx_v_kwargs); __pyx_v_kwargs = 0;
  __Pyx_AddTraceback("pycx4.pycda.BaseChan.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_name), (&PyUnicode_Type), 1, "name", 1))) __PYX_ERR(9, 98, __pyx_L1_error)
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan___init__(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self), __pyx_v_name, __pyx_v_kwargs);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pycx4_5pycda_8BaseChan___init__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_kwargs) {
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_v_b_name = NULL;
  char *__pyx_v_c_name;
  int __pyx_v_ret;
  unsigned int __pyx_v_options;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  cxdtype_t __pyx_t_6;
  char *__pyx_t_7;
  size_t __pyx_t_8;
  int __pyx_t_9;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "pycx4/basechan.pxi":99
 * 
 *     def __init__(self, str name, **kwargs):
 *         CdaObject.__init__(self)             # <<<<<<<<<<<<<<
 *         self.context = kwargs.get('context', default_context)
 *         self.max_nelems = kwargs.get('max_nelems', 1)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_5pycx4_5pycda_CdaObject), __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(9, 99, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, ((PyObject *)__pyx_v_self)) : __Pyx_PyObject_CallOneArg(__pyx_t_2, ((PyObject *)__pyx_v_self));
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 99, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/basechan.pxi":100
 *     def __init__(self, str name, **kwargs):
 *         CdaObject.__init__(self)
 *         self.context = kwargs.get('context', default_context)             # <<<<<<<<<<<<<<
 *         self.max_nelems = kwargs.get('max_nelems', 1)
 *         dtype = kwargs.get('dtype', CXDTYPE_DOUBLE)
 */
  __pyx_t_1 = __Pyx_PyDict_GetItemDefault(__pyx_v_kwargs, __pyx_n_u_context, ((PyObject *)__pyx_v_5pycx4_5pycda_default_context)); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 100, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5pycx4_5pycda_Context))))) __PYX_ERR(9, 100, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->context);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->context));
  __pyx_v_self->context = ((struct __pyx_obj_5pycx4_5pycda_Context *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pycx4/basechan.pxi":101
 *         CdaObject.__init__(self)
 *         self.context = kwargs.get('context', default_context)
 *         self.max_nelems = kwargs.get('max_nelems', 1)             # <<<<<<<<<<<<<<
 *         dtype = kwargs.get('dtype', CXDTYPE_DOUBLE)
 *         if type(dtype) == str:
 */
  __pyx_t_1 = __Pyx_PyDict_GetItemDefault(__pyx_v_kwargs, __pyx_n_u_max_nelems, __pyx_int_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 101, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_4 == (int)-1) && PyErr_Occurred())) __PYX_ERR(9, 101, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->max_nelems = __pyx_t_4;

  /* "pycx4/basechan.pxi":102
 *         self.context = kwargs.get('context', default_context)
 *         self.max_nelems = kwargs.get('max_nelems', 1)
 *         dtype = kwargs.get('dtype', CXDTYPE_DOUBLE)             # <<<<<<<<<<<<<<
 *         if type(dtype) == str:
 *             self.dtype = cx_dtype_map[dtype]
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(CXDTYPE_DOUBLE); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 102, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyDict_GetItemDefault(__pyx_v_kwargs, __pyx_n_u_dtype, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(9, 102, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_dtype = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "pycx4/basechan.pxi":103
 *         self.max_nelems = kwargs.get('max_nelems', 1)
 *         dtype = kwargs.get('dtype', CXDTYPE_DOUBLE)
 *         if type(dtype) == str:             # <<<<<<<<<<<<<<
 *             self.dtype = cx_dtype_map[dtype]
 *         else:
 */
  __pyx_t_2 = PyObject_RichCompare(((PyObject *)Py_TYPE(__pyx_v_dtype)), ((PyObject *)(&PyUnicode_Type)), Py_EQ); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(9, 103, __pyx_L1_error)
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(9, 103, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_5) {

    /* "pycx4/basechan.pxi":104
 *         dtype = kwargs.get('dtype', CXDTYPE_DOUBLE)
 *         if type(dtype) == str:
 *             self.dtype = cx_dtype_map[dtype]             # <<<<<<<<<<<<<<
 *         else:
 *             self.dtype = dtype
 */
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_cx_dtype_map); if (unlikely(!__pyx_t_2)) __PYX_ERR(9, 104, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_t_2, __pyx_v_dtype); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 104, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_6 = __Pyx_PyInt_As_uint32(__pyx_t_1); if (unlikely((__pyx_t_6 == ((cxdtype_t)-1)) && PyErr_Occurred())) __PYX_ERR(9, 104, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_self->dtype = __pyx_t_6;

    /* "pycx4/basechan.pxi":103
 *         self.max_nelems = kwargs.get('max_nelems', 1)
 *         dtype = kwargs.get('dtype', CXDTYPE_DOUBLE)
 *         if type(dtype) == str:             # <<<<<<<<<<<<<<
 *             self.dtype = cx_dtype_map[dtype]
 *         else:
 */
    goto __pyx_L3;
  }

  /* "pycx4/basechan.pxi":106
 *             self.dtype = cx_dtype_map[dtype]
 *         else:
 *             self.dtype = dtype             # <<<<<<<<<<<<<<
 * 
 *         self.valueChanged, self.valueMeasured, self.resolve = Signal(object), Signal(object), Signal(object)
 */
  /*else*/ {
    __pyx_t_6 = __Pyx_PyInt_As_uint32(__pyx_v_dtype); if (unlikely((__pyx_t_6 == ((cxdtype_t)-1)) && PyErr_Occurred())) __PYX_ERR(9, 106, __pyx_L1_error)
    __pyx_v_self->dtype = __pyx_t_6;
  }
  __pyx_L3:;

  /* "pycx4/basechan.pxi":108
 *             self.dtype = dtype
 * 
 *         self.valueChanged, self.valueMeasured, self.resolve = Signal(object), Signal(object), Signal(object)             # <<<<<<<<<<<<<<
 *         self.stringsUpdate, self.lockStateUpdate = Signal(object), Signal(object)
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5pycx4_5pycda_Signal), __pyx_tuple__14, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 108, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5pycx4_5pycda_Signal), __pyx_tuple__15, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(9, 108, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5pycx4_5pycda_Signal), __pyx_tuple__16, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(9, 108, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->valueChanged);
  __Pyx_DECREF(__pyx_v_self->valueChanged);
  __pyx_v_self->valueChanged = __pyx_t_1;
  __pyx_t_1 = 0;
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->valueMeasured);
  __Pyx_DECREF(__pyx_v_self->valueMeasured);
  __pyx_v_self->valueMeasured = __pyx_t_2;
  __pyx_t_2 = 0;
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->resolve);
  __Pyx_DECREF(__pyx_v_self->resolve);
  __pyx_v_self->resolve = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pycx4/basechan.pxi":109
 * 
 *         self.valueChanged, self.valueMeasured, self.resolve = Signal(object), Signal(object), Signal(object)
 *         self.stringsUpdate, self.lockStateUpdate = Signal(object), Signal(object)             # <<<<<<<<<<<<<<
 * 
 *         b_name = name.encode("ascii")
 */
  __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5pycx4_5pycda_Signal), __pyx_tuple__17, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(9, 109, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5pycx4_5pycda_Signal), __pyx_tuple__18, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(9, 109, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->stringsUpdate);
  __Pyx_DECREF(__pyx_v_self->stringsUpdate);
  __pyx_v_self->stringsUpdate = __pyx_t_3;
  __pyx_t_3 = 0;
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->lockStateUpdate);
  __Pyx_DECREF(__pyx_v_self->lockStateUpdate);
  __pyx_v_self->lockStateUpdate = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "pycx4/basechan.pxi":111
 *         self.stringsUpdate, self.lockStateUpdate = Signal(object), Signal(object)
 * 
 *         b_name = name.encode("ascii")             # <<<<<<<<<<<<<<
 *         cdef:
 *             char *c_name = b_name
 */
  if (unlikely(__pyx_v_name == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "encode");
    __PYX_ERR(9, 111, __pyx_L1_error)
  }
  __pyx_t_2 = PyUnicode_AsASCIIString(__pyx_v_name); if (unlikely(!__pyx_t_2)) __PYX_ERR(9, 111, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_b_name = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "pycx4/basechan.pxi":113
 *         b_name = name.encode("ascii")
 *         cdef:
 *             char *c_name = b_name             # <<<<<<<<<<<<<<
 *             int ret
 *             unsigned int options = 0
 */
  __pyx_t_7 = __Pyx_PyObject_AsWritableString(__pyx_v_b_name); if (unlikely((!__pyx_t_7) && PyErr_Occurred())) __PYX_ERR(9, 113, __pyx_L1_error)
  __pyx_v_c_name = __pyx_t_7;

  /* "pycx4/basechan.pxi":115
 *             char *c_name = b_name
 *             int ret
 *             unsigned int options = 0             # <<<<<<<<<<<<<<
 * 
 *         # may be cycle?
 */
  __pyx_v_options = 0;

  /* "pycx4/basechan.pxi":118
 * 
 *         # may be cycle?
 *         if kwargs.get('private', False):             # <<<<<<<<<<<<<<
 *             options += CDA_DATAREF_OPT_PRIVATE
 * 
 */
  __pyx_t_2 = __Pyx_PyDict_GetItemDefault(__pyx_v_kwargs, __pyx_n_u_private, Py_False); if (unlikely(!__pyx_t_2)) __PYX_ERR(9, 118, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(9, 118, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_5) {

    /* "pycx4/basechan.pxi":119
 *         # may be cycle?
 *         if kwargs.get('private', False):
 *             options += CDA_DATAREF_OPT_PRIVATE             # <<<<<<<<<<<<<<
 * 
 *         if kwargs.get('no_rd_conv', False):
 */
    __pyx_v_options = (__pyx_v_options + CDA_DATAREF_OPT_PRIVATE);

    /* "pycx4/basechan.pxi":118
 * 
 *         # may be cycle?
 *         if kwargs.get('private', False):             # <<<<<<<<<<<<<<
 *             options += CDA_DATAREF_OPT_PRIVATE
 * 
 */
  }

  /* "pycx4/basechan.pxi":121
 *             options += CDA_DATAREF_OPT_PRIVATE
 * 
 *         if kwargs.get('no_rd_conv', False):             # <<<<<<<<<<<<<<
 *             options += CDA_DATAREF_OPT_NO_RD_CONV
 * 
 */
  __pyx_t_2 = __Pyx_PyDict_GetItemDefault(__pyx_v_kwargs, __pyx_n_u_no_rd_conv, Py_False); if (unlikely(!__pyx_t_2)) __PYX_ERR(9, 121, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(9, 121, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_5) {

    /* "pycx4/basechan.pxi":122
 * 
 *         if kwargs.get('no_rd_conv', False):
 *             options += CDA_DATAREF_OPT_NO_RD_CONV             # <<<<<<<<<<<<<<
 * 
 *         if kwargs.get('shy', False):
 */
    __pyx_v_options = (__pyx_v_options + CDA_DATAREF_OPT_NO_RD_CONV);

    /* "pycx4/basechan.pxi":121
 *             options += CDA_DATAREF_OPT_PRIVATE
 * 
 *         if kwargs.get('no_rd_conv', False):             # <<<<<<<<<<<<<<
 *             options += CDA_DATAREF_OPT_NO_RD_CONV
 * 
 */
  }

  /* "pycx4/basechan.pxi":124
 *             options += CDA_DATAREF_OPT_NO_RD_CONV
 * 
 *         if kwargs.get('shy', False):             # <<<<<<<<<<<<<<
 *             options += CDA_DATAREF_OPT_SHY
 * 
 */
  __pyx_t_2 = __Pyx_PyDict_GetItemDefault(__pyx_v_kwargs, __pyx_n_u_shy, Py_False); if (unlikely(!__pyx_t_2)) __PYX_ERR(9, 124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(9, 124, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_5) {

    /* "pycx4/basechan.pxi":125
 * 
 *         if kwargs.get('shy', False):
 *             options += CDA_DATAREF_OPT_SHY             # <<<<<<<<<<<<<<
 * 
 *         if kwargs.get('find_only', False):
 */
    __pyx_v_options = (__pyx_v_options + CDA_DATAREF_OPT_SHY);

    /* "pycx4/basechan.pxi":124
 *             options += CDA_DATAREF_OPT_NO_RD_CONV
 * 
 *         if kwargs.get('shy', False):             # <<<<<<<<<<<<<<
 *             options += CDA_DATAREF_OPT_SHY
 * 
 */
  }

  /* "pycx4/basechan.pxi":127
 *             options += CDA_DATAREF_OPT_SHY
 * 
 *         if kwargs.get('find_only', False):             # <<<<<<<<<<<<<<
 *             options += CDA_DATAREF_OPT_FIND_ONLY
 * 
 */
  __pyx_t_2 = __Pyx_PyDict_GetItemDefault(__pyx_v_kwargs, __pyx_n_u_find_only, Py_False); if (unlikely(!__pyx_t_2)) __PYX_ERR(9, 127, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(9, 127, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_5) {

    /* "pycx4/basechan.pxi":128
 * 
 *         if kwargs.get('find_only', False):
 *             options += CDA_DATAREF_OPT_FIND_ONLY             # <<<<<<<<<<<<<<
 * 
 *         if kwargs.get('on_update', False):
 */
    __pyx_v_options = (__pyx_v_options + CDA_DATAREF_OPT_FIND_ONLY);

    /* "pycx4/basechan.pxi":127
 *             options += CDA_DATAREF_OPT_SHY
 * 
 *         if kwargs.get('find_only', False):             # <<<<<<<<<<<<<<
 *             options += CDA_DATAREF_OPT_FIND_ONLY
 * 
 */
  }

  /* "pycx4/basechan.pxi":130
 *             options += CDA_DATAREF_OPT_FIND_ONLY
 * 
 *         if kwargs.get('on_update', False):             # <<<<<<<<<<<<<<
 *             options += CDA_DATAREF_OPT_ON_UPDATE
 * 
 */
  __pyx_t_2 = __Pyx_PyDict_GetItemDefault(__pyx_v_kwargs, __pyx_n_u_on_update, Py_False); if (unlikely(!__pyx_t_2)) __PYX_ERR(9, 130, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(9, 130, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_5) {

    /* "pycx4/basechan.pxi":131
 * 
 *         if kwargs.get('on_update', False):
 *             options += CDA_DATAREF_OPT_ON_UPDATE             # <<<<<<<<<<<<<<
 * 
 *         if kwargs.get('no_wr_wait', False):
 */
    __pyx_v_options = (__pyx_v_options + CDA_DATAREF_OPT_ON_UPDATE);

    /* "pycx4/basechan.pxi":130
 *             options += CDA_DATAREF_OPT_FIND_ONLY
 * 
 *         if kwargs.get('on_update', False):             # <<<<<<<<<<<<<<
 *             options += CDA_DATAREF_OPT_ON_UPDATE
 * 
 */
  }

  /* "pycx4/basechan.pxi":133
 *             options += CDA_DATAREF_OPT_ON_UPDATE
 * 
 *         if kwargs.get('no_wr_wait', False):             # <<<<<<<<<<<<<<
 *             options += CDA_DATAREF_OPT_NO_WR_WAIT
 * 
 */
  __pyx_t_2 = __Pyx_PyDict_GetItemDefault(__pyx_v_kwargs, __pyx_n_u_no_wr_wait, Py_False); if (unlikely(!__pyx_t_2)) __PYX_ERR(9, 133, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(9, 133, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_5) {

    /* "pycx4/basechan.pxi":134
 * 
 *         if kwargs.get('no_wr_wait', False):
 *             options += CDA_DATAREF_OPT_NO_WR_WAIT             # <<<<<<<<<<<<<<
 * 
 *         if kwargs.get('debug', False):
 */
    __pyx_v_options = (__pyx_v_options + CDA_DATAREF_OPT_NO_WR_WAIT);

    /* "pycx4/basechan.pxi":133
 *             options += CDA_DATAREF_OPT_ON_UPDATE
 * 
 *         if kwargs.get('no_wr_wait', False):             # <<<<<<<<<<<<<<
 *             options += CDA_DATAREF_OPT_NO_WR_WAIT
 * 
 */
  }

  /* "pycx4/basechan.pxi":136
 *             options += CDA_DATAREF_OPT_NO_WR_WAIT
 * 
 *         if kwargs.get('debug', False):             # <<<<<<<<<<<<<<
 *             options += CDA_DATAREF_OPT_DEBUG
 * 
 */
  __pyx_t_2 = __Pyx_PyDict_GetItemDefault(__pyx_v_kwargs, __pyx_n_u_debug, Py_False); if (unlikely(!__pyx_t_2)) __PYX_ERR(9, 136, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(9, 136, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_5) {

    /* "pycx4/basechan.pxi":137
 * 
 *         if kwargs.get('debug', False):
 *             options += CDA_DATAREF_OPT_DEBUG             # <<<<<<<<<<<<<<
 * 
 *         ret = cda_add_chan((<Context>self.context).cid, NULL, c_name, options, self.dtype, self.max_nelems,
 */
    __pyx_v_options = (__pyx_v_options + CDA_DATAREF_OPT_DEBUG);

    /* "pycx4/basechan.pxi":136
 *             options += CDA_DATAREF_OPT_NO_WR_WAIT
 * 
 *         if kwargs.get('debug', False):             # <<<<<<<<<<<<<<
 *             options += CDA_DATAREF_OPT_DEBUG
 * 
 */
  }

  /* "pycx4/basechan.pxi":139
 *             options += CDA_DATAREF_OPT_DEBUG
 * 
 *         ret = cda_add_chan((<Context>self.context).cid, NULL, c_name, options, self.dtype, self.max_nelems,             # <<<<<<<<<<<<<<
 *                            0, <cda_dataref_evproc_t>NULL, NULL)
 *         self.check_exception(ret)
 */
  __pyx_v_ret = cda_add_chan(__pyx_v_self->context->cid, NULL, __pyx_v_c_name, __pyx_v_options, __pyx_v_self->dtype, __pyx_v_self->max_nelems, 0, ((cda_dataref_evproc_t)NULL), NULL);

  /* "pycx4/basechan.pxi":141
 *         ret = cda_add_chan((<Context>self.context).cid, NULL, c_name, options, self.dtype, self.max_nelems,
 *                            0, <cda_dataref_evproc_t>NULL, NULL)
 *         self.check_exception(ret)             # <<<<<<<<<<<<<<
 *         self.ref, self.name, self.itemsize = ret, name, sizeof_cxdtype(dtype)
 * 
 */
  ((struct __pyx_vtabstruct_5pycx4_5pycda_BaseChan *)__pyx_v_self->__pyx_base.__pyx_vtab)->check_exception(__pyx_v_self, __pyx_v_ret);

  /* "pycx4/basechan.pxi":142
 *                            0, <cda_dataref_evproc_t>NULL, NULL)
 *         self.check_exception(ret)
 *         self.ref, self.name, self.itemsize = ret, name, sizeof_cxdtype(dtype)             # <<<<<<<<<<<<<<
 * 
 *         (<Context>self.context).save_chan(<void*>self)
 */
  __pyx_t_4 = __pyx_v_ret;
  __pyx_t_2 = __pyx_v_name;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_6 = __Pyx_PyInt_As_uint32(__pyx_v_dtype); if (unlikely((__pyx_t_6 == ((cxdtype_t)-1)) && PyErr_Occurred())) __PYX_ERR(9, 142, __pyx_L1_error)
  __pyx_t_8 = sizeof_cxdtype(__pyx_t_6);
  __pyx_v_self->ref = __pyx_t_4;
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->name);
  __Pyx_DECREF(__pyx_v_self->name);
  __pyx_v_self->name = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_v_self->itemsize = __pyx_t_8;

  /* "pycx4/basechan.pxi":144
 *         self.ref, self.name, self.itemsize = ret, name, sizeof_cxdtype(dtype)
 * 
 *         (<Context>self.context).save_chan(<void*>self)             # <<<<<<<<<<<<<<
 * 
 *         # events which can be regular
 */
  ((struct __pyx_vtabstruct_5pycx4_5pycda_Context *)((struct __pyx_obj_5pycx4_5pycda_Context *)__pyx_v_self->context)->__pyx_base.__pyx_vtab)->save_chan(((struct __pyx_obj_5pycx4_5pycda_Context *)__pyx_v_self->context), ((void *)__pyx_v_self));

  /* "pycx4/basechan.pxi":147
 * 
 *         # events which can be regular
 *         self.add_event(CDA_REF_EVMASK_RSLVSTAT, <void*>evproc_rslvstat, <void*>self, NULL)             # <<<<<<<<<<<<<<
 *         self.add_event(CDA_REF_EVMASK_UPDATE, <void*>evproc_update, <void*>self, NULL)
 * 
 */
  (void)(((struct __pyx_vtabstruct_5pycx4_5pycda_BaseChan *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base.add_event(((struct __pyx_obj_5pycx4_5pycda_CdaObject *)__pyx_v_self), CDA_REF_EVMASK_RSLVSTAT, ((void *)__pyx_f_5pycx4_5pycda_evproc_rslvstat), ((void *)__pyx_v_self), NULL));

  /* "pycx4/basechan.pxi":148
 *         # events which can be regular
 *         self.add_event(CDA_REF_EVMASK_RSLVSTAT, <void*>evproc_rslvstat, <void*>self, NULL)
 *         self.add_event(CDA_REF_EVMASK_UPDATE, <void*>evproc_update, <void*>self, NULL)             # <<<<<<<<<<<<<<
 * 
 *         self.add_event(CDA_REF_EVMASK_QUANTCHG, <void*>quant_update, <void*>self, NULL)
 */
  (void)(((struct __pyx_vtabstruct_5pycx4_5pycda_BaseChan *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base.add_event(((struct __pyx_obj_5pycx4_5pycda_CdaObject *)__pyx_v_self), CDA_REF_EVMASK_UPDATE, ((void *)__pyx_f_5pycx4_5pycda_evproc_update), ((void *)__pyx_v_self), NULL));

  /* "pycx4/basechan.pxi":150
 *         self.add_event(CDA_REF_EVMASK_UPDATE, <void*>evproc_update, <void*>self, NULL)
 * 
 *         self.add_event(CDA_REF_EVMASK_QUANTCHG, <void*>quant_update, <void*>self, NULL)             # <<<<<<<<<<<<<<
 *         self.add_event(CDA_REF_EVMASK_RANGECHG, <void*>range_update, <void*>self, NULL)
 * 
 */
  (void)(((struct __pyx_vtabstruct_5pycx4_5pycda_BaseChan *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base.add_event(((struct __pyx_obj_5pycx4_5pycda_CdaObject *)__pyx_v_self), CDA_REF_EVMASK_QUANTCHG, ((void *)__pyx_f_5pycx4_5pycda_quant_update), ((void *)__pyx_v_self), NULL));

  /* "pycx4/basechan.pxi":151
 * 
 *         self.add_event(CDA_REF_EVMASK_QUANTCHG, <void*>quant_update, <void*>self, NULL)
 *         self.add_event(CDA_REF_EVMASK_RANGECHG, <void*>range_update, <void*>self, NULL)             # <<<<<<<<<<<<<<
 * 
 *         self.add_event(CDA_REF_EVMASK_STRSCHG, <void*>strs_update, <void*>self, NULL)
 */
  (void)(((struct __pyx_vtabstruct_5pycx4_5pycda_BaseChan *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base.add_event(((struct __pyx_obj_5pycx4_5pycda_CdaObject *)__pyx_v_self), CDA_REF_EVMASK_RANGECHG, ((void *)__pyx_f_5pycx4_5pycda_range_update), ((void *)__pyx_v_self), NULL));

  /* "pycx4/basechan.pxi":153
 *         self.add_event(CDA_REF_EVMASK_RANGECHG, <void*>range_update, <void*>self, NULL)
 * 
 *         self.add_event(CDA_REF_EVMASK_STRSCHG, <void*>strs_update, <void*>self, NULL)             # <<<<<<<<<<<<<<
 * 
 *         self.add_event(CDA_REF_EVMASK_LOCKSTAT, <void*>lockstat_update, <void*>self, NULL)
 */
  (void)(((struct __pyx_vtabstruct_5pycx4_5pycda_BaseChan *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base.add_event(((struct __pyx_obj_5pycx4_5pycda_CdaObject *)__pyx_v_self), CDA_REF_EVMASK_STRSCHG, ((void *)__pyx_f_5pycx4_5pycda_strs_update), ((void *)__pyx_v_self), NULL));

  /* "pycx4/basechan.pxi":155
 *         self.add_event(CDA_REF_EVMASK_STRSCHG, <void*>strs_update, <void*>self, NULL)
 * 
 *         self.add_event(CDA_REF_EVMASK_LOCKSTAT, <void*>lockstat_update, <void*>self, NULL)             # <<<<<<<<<<<<<<
 * 
 *         if kwargs.get('get_curval', False):
 */
  (void)(((struct __pyx_vtabstruct_5pycx4_5pycda_BaseChan *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base.add_event(((struct __pyx_obj_5pycx4_5pycda_CdaObject *)__pyx_v_self), CDA_REF_EVMASK_LOCKSTAT, ((void *)__pyx_f_5pycx4_5pycda_lockstat_update), ((void *)__pyx_v_self), NULL));

  /* "pycx4/basechan.pxi":157
 *         self.add_event(CDA_REF_EVMASK_LOCKSTAT, <void*>lockstat_update, <void*>self, NULL)
 * 
 *         if kwargs.get('get_curval', False):             # <<<<<<<<<<<<<<
 *             self.add_event(CDA_REF_EVMASK_CURVAL, <void*>evproc_update, <void*>self, NULL)
 * 
 */
  __pyx_t_2 = __Pyx_PyDict_GetItemDefault(__pyx_v_kwargs, __pyx_n_u_get_curval, Py_False); if (unlikely(!__pyx_t_2)) __PYX_ERR(9, 157, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(9, 157, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_5) {

    /* "pycx4/basechan.pxi":158
 * 
 *         if kwargs.get('get_curval', False):
 *             self.add_event(CDA_REF_EVMASK_CURVAL, <void*>evproc_update, <void*>self, NULL)             # <<<<<<<<<<<<<<
 * 
 *         # CX event masks
 */
    (void)(((struct __pyx_vtabstruct_5pycx4_5pycda_BaseChan *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base.add_event(((struct __pyx_obj_5pycx4_5pycda_CdaObject *)__pyx_v_self), CDA_REF_EVMASK_CURVAL, ((void *)__pyx_f_5pycx4_5pycda_evproc_update), ((void *)__pyx_v_self), NULL));

    /* "pycx4/basechan.pxi":157
 *         self.add_event(CDA_REF_EVMASK_LOCKSTAT, <void*>lockstat_update, <void*>self, NULL)
 * 
 *         if kwargs.get('get_curval', False):             # <<<<<<<<<<<<<<
 *             self.add_event(CDA_REF_EVMASK_CURVAL, <void*>evproc_update, <void*>self, NULL)
 * 
 */
  }

  /* "pycx4/basechan.pxi":177
 *         # initialization events (will be unregistered when get)
 * 
 *         self.registered, self.first_cycle = True, True             # <<<<<<<<<<<<<<
 *         self.rslv_stat = CDA_RSLVSTAT_SEARCHING
 *         self.rslv_str = 'searching'
 */
  __pyx_t_5 = 1;
  __pyx_t_9 = 1;
  __pyx_v_self->registered = __pyx_t_5;
  __pyx_v_self->first_cycle = __pyx_t_9;

  /* "pycx4/basechan.pxi":178
 * 
 *         self.registered, self.first_cycle = True, True
 *         self.rslv_stat = CDA_RSLVSTAT_SEARCHING             # <<<<<<<<<<<<<<
 *         self.rslv_str = 'searching'
 *         self.rng = []
 */
  __pyx_v_self->rslv_stat = CDA_RSLVSTAT_SEARCHING;

  /* "pycx4/basechan.pxi":179
 *         self.registered, self.first_cycle = True, True
 *         self.rslv_stat = CDA_RSLVSTAT_SEARCHING
 *         self.rslv_str = 'searching'             # <<<<<<<<<<<<<<
 *         self.rng = []
 * 
 */
  __Pyx_INCREF(__pyx_n_u_searching);
  __Pyx_GIVEREF(__pyx_n_u_searching);
  __Pyx_GOTREF(__pyx_v_self->rslv_str);
  __Pyx_DECREF(__pyx_v_self->rslv_str);
  __pyx_v_self->rslv_str = __pyx_n_u_searching;

  /* "pycx4/basechan.pxi":180
 *         self.rslv_stat = CDA_RSLVSTAT_SEARCHING
 *         self.rslv_str = 'searching'
 *         self.rng = []             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
 */
  __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(9, 180, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->rng);
  __Pyx_DECREF(__pyx_v_self->rng);
  __pyx_v_self->rng = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pycx4/basechan.pxi":98
 *         object valueMeasured, valueChanged, resolve, stringsUpdate, lockStateUpdate
 * 
 *     def __init__(self, str name, **kwargs):             # <<<<<<<<<<<<<<
 *         CdaObject.__init__(self)
 *         self.context = kwargs.get('context', default_context)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("pycx4.pycda.BaseChan.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_b_name);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/basechan.pxi":182
 *         self.rng = []
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if self.registered:
 *             (<Context>self.context).drop_chan(<void*>self)
 */

/* Python wrapper */
static void __pyx_pw_5pycx4_5pycda_8BaseChan_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_5pycx4_5pycda_8BaseChan_3__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_5pycx4_5pycda_8BaseChan_2__dealloc__(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_5pycx4_5pycda_8BaseChan_2__dealloc__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "pycx4/basechan.pxi":183
 * 
 *     def __dealloc__(self):
 *         if self.registered:             # <<<<<<<<<<<<<<
 *             (<Context>self.context).drop_chan(<void*>self)
 *             cda_del_chan(self.ref)
 */
  __pyx_t_1 = (__pyx_v_self->registered != 0);
  if (__pyx_t_1) {

    /* "pycx4/basechan.pxi":184
 *     def __dealloc__(self):
 *         if self.registered:
 *             (<Context>self.context).drop_chan(<void*>self)             # <<<<<<<<<<<<<<
 *             cda_del_chan(self.ref)
 *             self.registered = 0
 */
    ((struct __pyx_vtabstruct_5pycx4_5pycda_Context *)((struct __pyx_obj_5pycx4_5pycda_Context *)__pyx_v_self->context)->__pyx_base.__pyx_vtab)->drop_chan(((struct __pyx_obj_5pycx4_5pycda_Context *)__pyx_v_self->context), ((void *)__pyx_v_self));

    /* "pycx4/basechan.pxi":185
 *         if self.registered:
 *             (<Context>self.context).drop_chan(<void*>self)
 *             cda_del_chan(self.ref)             # <<<<<<<<<<<<<<
 *             self.registered = 0
 * 
 */
    (void)(cda_del_chan(__pyx_v_self->ref));

    /* "pycx4/basechan.pxi":186
 *             (<Context>self.context).drop_chan(<void*>self)
 *             cda_del_chan(self.ref)
 *             self.registered = 0             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
    __pyx_v_self->registered = 0;

    /* "pycx4/basechan.pxi":183
 * 
 *     def __dealloc__(self):
 *         if self.registered:             # <<<<<<<<<<<<<<
 *             (<Context>self.context).drop_chan(<void*>self)
 *             cda_del_chan(self.ref)
 */
  }

  /* "pycx4/basechan.pxi":182
 *         self.rng = []
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if self.registered:
 *             (<Context>self.context).drop_chan(<void*>self)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/basechan.pxi":188
 *             self.registered = 0
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return '<cda_chan: ref=%d, name=%s>' % (self.ref, self.name)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_5__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_5__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_4__str__(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_4__str__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  Py_UCS4 __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "pycx4/basechan.pxi":189
 * 
 *     def __str__(self):
 *         return '<cda_chan: ref=%d, name=%s>' % (self.ref, self.name)             # <<<<<<<<<<<<<<
 * 
 *     def short_name(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyTuple_New(5); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 189, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = 0;
  __pyx_t_3 = 127;
  __Pyx_INCREF(__pyx_kp_u_cda_chan_ref);
  __pyx_t_2 += 15;
  __Pyx_GIVEREF(__pyx_kp_u_cda_chan_ref);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_kp_u_cda_chan_ref);
  __pyx_t_4 = __Pyx_PyInt_From_CxDataRef_t(__pyx_v_self->ref); if (unlikely(!__pyx_t_4)) __PYX_ERR(9, 189, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_Format(__pyx_t_4, __pyx_n_u_d); if (unlikely(!__pyx_t_5)) __PYX_ERR(9, 189, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_3 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_5) > __pyx_t_3) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_5) : __pyx_t_3;
  __pyx_t_2 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_5);
  __pyx_t_5 = 0;
  __Pyx_INCREF(__pyx_kp_u_name_2);
  __pyx_t_2 += 7;
  __Pyx_GIVEREF(__pyx_kp_u_name_2);
  PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_kp_u_name_2);
  __pyx_t_5 = __Pyx_PyUnicode_Unicode(__pyx_v_self->name); if (unlikely(!__pyx_t_5)) __PYX_ERR(9, 189, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_5) > __pyx_t_3) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_5) : __pyx_t_3;
  __pyx_t_2 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_1, 3, __pyx_t_5);
  __pyx_t_5 = 0;
  __Pyx_INCREF(__pyx_kp_u__10);
  __pyx_t_2 += 1;
  __Pyx_GIVEREF(__pyx_kp_u__10);
  PyTuple_SET_ITEM(__pyx_t_1, 4, __pyx_kp_u__10);
  __pyx_t_5 = __Pyx_PyUnicode_Join(__pyx_t_1, 5, __pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(9, 189, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "pycx4/basechan.pxi":188
 *             self.registered = 0
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return '<cda_chan: ref=%d, name=%s>' % (self.ref, self.name)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pycx4.pycda.BaseChan.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/basechan.pxi":191
 *         return '<cda_chan: ref=%d, name=%s>' % (self.ref, self.name)
 * 
 *     def short_name(self):             # <<<<<<<<<<<<<<
 *         a = self.name.split('.')
 *         return a[len(a)-1].split('@')[0]
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_7short_name(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_7short_name(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("short_name (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_6short_name(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_6short_name(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  PyObject *__pyx_v_a = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("short_name", 0);

  /* "pycx4/basechan.pxi":192
 * 
 *     def short_name(self):
 *         a = self.name.split('.')             # <<<<<<<<<<<<<<
 *         return a[len(a)-1].split('@')[0]
 * 
 */
  if (unlikely(__pyx_v_self->name == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "split");
    __PYX_ERR(9, 192, __pyx_L1_error)
  }
  __pyx_t_1 = PyUnicode_Split(__pyx_v_self->name, __pyx_kp_u__19, -1L); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 192, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_a = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pycx4/basechan.pxi":193
 *     def short_name(self):
 *         a = self.name.split('.')
 *         return a[len(a)-1].split('@')[0]             # <<<<<<<<<<<<<<
 * 
 *     cdef void cb(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = PyObject_Length(__pyx_v_a); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(9, 193, __pyx_L1_error)
  __pyx_t_3 = (__pyx_t_2 - 1);
  __pyx_t_4 = __Pyx_GetItemInt(__pyx_v_a, __pyx_t_3, Py_ssize_t, 1, PyInt_FromSsize_t, 0, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(9, 193, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_split); if (unlikely(!__pyx_t_5)) __PYX_ERR(9, 193, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_4, __pyx_kp_u__20) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_kp_u__20);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 193, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_GetItemInt(__pyx_t_1, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(9, 193, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "pycx4/basechan.pxi":191
 *         return '<cda_chan: ref=%d, name=%s>' % (self.ref, self.name)
 * 
 *     def short_name(self):             # <<<<<<<<<<<<<<
 *         a = self.name.split('.')
 *         return a[len(a)-1].split('@')[0]
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pycx4.pycda.BaseChan.short_name", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_a);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/basechan.pxi":195
 *         return a[len(a)-1].split('@')[0]
 * 
 *     cdef void cb(self):             # <<<<<<<<<<<<<<
 *         #empty callback for overrideing
 *         pass
 */

static void __pyx_f_5pycx4_5pycda_8BaseChan_cb(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("cb", 0);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/basechan.pxi":199
 *         pass
 * 
 *     cdef int snd_data(self, cxdtype_t dtype, int nelems, void* data_p):             # <<<<<<<<<<<<<<
 *         cdef int res = cda_snd_ref_data(self.ref, dtype, nelems, data_p)
 *         self.check_exception(res)
 */

static int __pyx_f_5pycx4_5pycda_8BaseChan_snd_data(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self, cxdtype_t __pyx_v_dtype, int __pyx_v_nelems, void *__pyx_v_data_p) {
  int __pyx_v_res;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("snd_data", 0);

  /* "pycx4/basechan.pxi":200
 * 
 *     cdef int snd_data(self, cxdtype_t dtype, int nelems, void* data_p):
 *         cdef int res = cda_snd_ref_data(self.ref, dtype, nelems, data_p)             # <<<<<<<<<<<<<<
 *         self.check_exception(res)
 * 
 */
  __pyx_v_res = cda_snd_ref_data(__pyx_v_self->ref, __pyx_v_dtype, __pyx_v_nelems, __pyx_v_data_p);

  /* "pycx4/basechan.pxi":201
 *     cdef int snd_data(self, cxdtype_t dtype, int nelems, void* data_p):
 *         cdef int res = cda_snd_ref_data(self.ref, dtype, nelems, data_p)
 *         self.check_exception(res)             # <<<<<<<<<<<<<<
 * 
 *     cdef int get_data(self, size_t ofs, size_t size, void* buf):
 */
  ((struct __pyx_vtabstruct_5pycx4_5pycda_BaseChan *)__pyx_v_self->__pyx_base.__pyx_vtab)->check_exception(__pyx_v_self, __pyx_v_res);

  /* "pycx4/basechan.pxi":199
 *         pass
 * 
 *     cdef int snd_data(self, cxdtype_t dtype, int nelems, void* data_p):             # <<<<<<<<<<<<<<
 *         cdef int res = cda_snd_ref_data(self.ref, dtype, nelems, data_p)
 *         self.check_exception(res)
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/basechan.pxi":203
 *         self.check_exception(res)
 * 
 *     cdef int get_data(self, size_t ofs, size_t size, void* buf):             # <<<<<<<<<<<<<<
 *         cdef int res = cda_get_ref_data(self.ref, ofs, size, buf)
 *         self.check_exception(res)
 */

static int __pyx_f_5pycx4_5pycda_8BaseChan_get_data(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self, size_t __pyx_v_ofs, size_t __pyx_v_size, void *__pyx_v_buf) {
  int __pyx_v_res;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_data", 0);

  /* "pycx4/basechan.pxi":204
 * 
 *     cdef int get_data(self, size_t ofs, size_t size, void* buf):
 *         cdef int res = cda_get_ref_data(self.ref, ofs, size, buf)             # <<<<<<<<<<<<<<
 *         self.check_exception(res)
 *         return res
 */
  __pyx_v_res = cda_get_ref_data(__pyx_v_self->ref, __pyx_v_ofs, __pyx_v_size, __pyx_v_buf);

  /* "pycx4/basechan.pxi":205
 *     cdef int get_data(self, size_t ofs, size_t size, void* buf):
 *         cdef int res = cda_get_ref_data(self.ref, ofs, size, buf)
 *         self.check_exception(res)             # <<<<<<<<<<<<<<
 *         return res
 * 
 */
  ((struct __pyx_vtabstruct_5pycx4_5pycda_BaseChan *)__pyx_v_self->__pyx_base.__pyx_vtab)->check_exception(__pyx_v_self, __pyx_v_res);

  /* "pycx4/basechan.pxi":206
 *         cdef int res = cda_get_ref_data(self.ref, ofs, size, buf)
 *         self.check_exception(res)
 *         return res             # <<<<<<<<<<<<<<
 * 
 *     cdef int current_nelems(self):
 */
  __pyx_r = __pyx_v_res;
  goto __pyx_L0;

  /* "pycx4/basechan.pxi":203
 *         self.check_exception(res)
 * 
 *     cdef int get_data(self, size_t ofs, size_t size, void* buf):             # <<<<<<<<<<<<<<
 *         cdef int res = cda_get_ref_data(self.ref, ofs, size, buf)
 *         self.check_exception(res)
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/basechan.pxi":208
 *         return res
 * 
 *     cdef int current_nelems(self):             # <<<<<<<<<<<<<<
 *         return cda_current_nelems_of_ref(self.ref)
 * 
 */

static int __pyx_f_5pycx4_5pycda_8BaseChan_current_nelems(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("current_nelems", 0);

  /* "pycx4/basechan.pxi":209
 * 
 *     cdef int current_nelems(self):
 *         return cda_current_nelems_of_ref(self.ref)             # <<<<<<<<<<<<<<
 * 
 *     cdef void get_src(self, const char **src_p):
 */
  __pyx_r = cda_current_nelems_of_ref(__pyx_v_self->ref);
  goto __pyx_L0;

  /* "pycx4/basechan.pxi":208
 *         return res
 * 
 *     cdef int current_nelems(self):             # <<<<<<<<<<<<<<
 *         return cda_current_nelems_of_ref(self.ref)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/basechan.pxi":211
 *         return cda_current_nelems_of_ref(self.ref)
 * 
 *     cdef void get_src(self, const char **src_p):             # <<<<<<<<<<<<<<
 *         self.check_exception( cda_src_of_ref(self.ref, src_p) )
 * 
 */

static void __pyx_f_5pycx4_5pycda_8BaseChan_get_src(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self, char const **__pyx_v_src_p) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_src", 0);

  /* "pycx4/basechan.pxi":212
 * 
 *     cdef void get_src(self, const char **src_p):
 *         self.check_exception( cda_src_of_ref(self.ref, src_p) )             # <<<<<<<<<<<<<<
 * 
 *     cdef void register_event(self, event *ev):
 */
  ((struct __pyx_vtabstruct_5pycx4_5pycda_BaseChan *)__pyx_v_self->__pyx_base.__pyx_vtab)->check_exception(__pyx_v_self, cda_src_of_ref(__pyx_v_self->ref, __pyx_v_src_p));

  /* "pycx4/basechan.pxi":211
 *         return cda_current_nelems_of_ref(self.ref)
 * 
 *     cdef void get_src(self, const char **src_p):             # <<<<<<<<<<<<<<
 *         self.check_exception( cda_src_of_ref(self.ref, src_p) )
 * 
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/basechan.pxi":214
 *         self.check_exception( cda_src_of_ref(self.ref, src_p) )
 * 
 *     cdef void register_event(self, event *ev):             # <<<<<<<<<<<<<<
 *         self.check_exception( cda_add_dataref_evproc(self.ref, ev.evmask, <cda_dataref_evproc_t>ev.evproc, ev) )
 * 
 */

static void __pyx_f_5pycx4_5pycda_8BaseChan_register_event(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self, __pyx_t_5pycx4_5pycda_event *__pyx_v_ev) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("register_event", 0);

  /* "pycx4/basechan.pxi":215
 * 
 *     cdef void register_event(self, event *ev):
 *         self.check_exception( cda_add_dataref_evproc(self.ref, ev.evmask, <cda_dataref_evproc_t>ev.evproc, ev) )             # <<<<<<<<<<<<<<
 * 
 *     cdef void unregister_event(self, event *ev):
 */
  ((struct __pyx_vtabstruct_5pycx4_5pycda_BaseChan *)__pyx_v_self->__pyx_base.__pyx_vtab)->check_exception(__pyx_v_self, cda_add_dataref_evproc(__pyx_v_self->ref, __pyx_v_ev->evmask, ((cda_dataref_evproc_t)__pyx_v_ev->evproc), __pyx_v_ev));

  /* "pycx4/basechan.pxi":214
 *         self.check_exception( cda_src_of_ref(self.ref, src_p) )
 * 
 *     cdef void register_event(self, event *ev):             # <<<<<<<<<<<<<<
 *         self.check_exception( cda_add_dataref_evproc(self.ref, ev.evmask, <cda_dataref_evproc_t>ev.evproc, ev) )
 * 
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/basechan.pxi":217
 *         self.check_exception( cda_add_dataref_evproc(self.ref, ev.evmask, <cda_dataref_evproc_t>ev.evproc, ev) )
 * 
 *     cdef void unregister_event(self, event *ev):             # <<<<<<<<<<<<<<
 *         self.check_exception( cda_del_dataref_evproc(self.ref, ev.evmask, <cda_dataref_evproc_t>ev.evproc, ev) )
 * 
 */

static void __pyx_f_5pycx4_5pycda_8BaseChan_unregister_event(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self, __pyx_t_5pycx4_5pycda_event *__pyx_v_ev) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("unregister_event", 0);

  /* "pycx4/basechan.pxi":218
 * 
 *     cdef void unregister_event(self, event *ev):
 *         self.check_exception( cda_del_dataref_evproc(self.ref, ev.evmask, <cda_dataref_evproc_t>ev.evproc, ev) )             # <<<<<<<<<<<<<<
 * 
 *     cdef void check_exception(self, int c_res):
 */
  ((struct __pyx_vtabstruct_5pycx4_5pycda_BaseChan *)__pyx_v_self->__pyx_base.__pyx_vtab)->check_exception(__pyx_v_self, cda_del_dataref_evproc(__pyx_v_self->ref, __pyx_v_ev->evmask, ((cda_dataref_evproc_t)__pyx_v_ev->evproc), __pyx_v_ev));

  /* "pycx4/basechan.pxi":217
 *         self.check_exception( cda_add_dataref_evproc(self.ref, ev.evmask, <cda_dataref_evproc_t>ev.evproc, ev) )
 * 
 *     cdef void unregister_event(self, event *ev):             # <<<<<<<<<<<<<<
 *         self.check_exception( cda_del_dataref_evproc(self.ref, ev.evmask, <cda_dataref_evproc_t>ev.evproc, ev) )
 * 
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/basechan.pxi":220
 *         self.check_exception( cda_del_dataref_evproc(self.ref, ev.evmask, <cda_dataref_evproc_t>ev.evproc, ev) )
 * 
 *     cdef void check_exception(self, int c_res):             # <<<<<<<<<<<<<<
 *         if c_res < 0:
 *             raise Exception("cda chan error: cname=%s, %s, errcode=%s" % (self.name, cda_last_err(), c_res ))
 */

static void __pyx_f_5pycx4_5pycda_8BaseChan_check_exception(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self, int __pyx_v_c_res) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  Py_UCS4 __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("check_exception", 0);

  /* "pycx4/basechan.pxi":221
 * 
 *     cdef void check_exception(self, int c_res):
 *         if c_res < 0:             # <<<<<<<<<<<<<<
 *             raise Exception("cda chan error: cname=%s, %s, errcode=%s" % (self.name, cda_last_err(), c_res ))
 * 
 */
  __pyx_t_1 = ((__pyx_v_c_res < 0) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "pycx4/basechan.pxi":222
 *     cdef void check_exception(self, int c_res):
 *         if c_res < 0:
 *             raise Exception("cda chan error: cname=%s, %s, errcode=%s" % (self.name, cda_last_err(), c_res ))             # <<<<<<<<<<<<<<
 * 
 *     cpdef is_available(self):
 */
    __pyx_t_2 = PyTuple_New(6); if (unlikely(!__pyx_t_2)) __PYX_ERR(9, 222, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = 0;
    __pyx_t_4 = 127;
    __Pyx_INCREF(__pyx_kp_u_cda_chan_error_cname);
    __pyx_t_3 += 22;
    __Pyx_GIVEREF(__pyx_kp_u_cda_chan_error_cname);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_kp_u_cda_chan_error_cname);
    __pyx_t_5 = __Pyx_PyUnicode_Unicode(__pyx_v_self->name); if (unlikely(!__pyx_t_5)) __PYX_ERR(9, 222, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_5) > __pyx_t_4) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_5) : __pyx_t_4;
    __pyx_t_3 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_5);
    __pyx_t_5 = 0;
    __Pyx_INCREF(__pyx_kp_u__11);
    __pyx_t_3 += 2;
    __Pyx_GIVEREF(__pyx_kp_u__11);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_kp_u__11);
    __pyx_t_5 = __Pyx_PyBytes_FromString(cda_last_err()); if (unlikely(!__pyx_t_5)) __PYX_ERR(9, 222, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Unicode(__pyx_t_5), __pyx_empty_unicode); if (unlikely(!__pyx_t_6)) __PYX_ERR(9, 222, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_4 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_6) > __pyx_t_4) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_6) : __pyx_t_4;
    __pyx_t_3 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_2, 3, __pyx_t_6);
    __pyx_t_6 = 0;
    __Pyx_INCREF(__pyx_kp_u_errcode);
    __pyx_t_3 += 10;
    __Pyx_GIVEREF(__pyx_kp_u_errcode);
    PyTuple_SET_ITEM(__pyx_t_2, 4, __pyx_kp_u_errcode);
    __pyx_t_6 = __Pyx_PyUnicode_From_int(__pyx_v_c_res, 0, ' ', 'd'); if (unlikely(!__pyx_t_6)) __PYX_ERR(9, 222, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_3 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_2, 5, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_6 = __Pyx_PyUnicode_Join(__pyx_t_2, 6, __pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_6)) __PYX_ERR(9, 222, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])), __pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(9, 222, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(9, 222, __pyx_L1_error)

    /* "pycx4/basechan.pxi":221
 * 
 *     cdef void check_exception(self, int c_res):
 *         if c_res < 0:             # <<<<<<<<<<<<<<
 *             raise Exception("cda chan error: cname=%s, %s, errcode=%s" % (self.name, cda_last_err(), c_res ))
 * 
 */
  }

  /* "pycx4/basechan.pxi":220
 *         self.check_exception( cda_del_dataref_evproc(self.ref, ev.evmask, <cda_dataref_evproc_t>ev.evproc, ev) )
 * 
 *     cdef void check_exception(self, int c_res):             # <<<<<<<<<<<<<<
 *         if c_res < 0:
 *             raise Exception("cda chan error: cname=%s, %s, errcode=%s" % (self.name, cda_last_err(), c_res ))
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_WriteUnraisable("pycx4.pycda.BaseChan.check_exception", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/basechan.pxi":224
 *             raise Exception("cda chan error: cname=%s, %s, errcode=%s" % (self.name, cda_last_err(), c_res ))
 * 
 *     cpdef is_available(self):             # <<<<<<<<<<<<<<
 *         if self.rslv_stat == CDA_RSLVSTAT_FOUND:
 *             return True
 */

static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_9is_available(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_5pycx4_5pycda_8BaseChan_is_available(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  __Pyx_RefNannySetupContext("is_available", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    static PY_UINT64_T tp_dict_version = 0, obj_dict_version = 0;
    if (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict && tp_dict_version == __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) && (!Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset || obj_dict_version == __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))))));
    else {
      PY_UINT64_T type_dict_guard = (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict)) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_is_available); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 224, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_5pycx4_5pycda_8BaseChan_9is_available)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(9, 224, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
      tp_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      obj_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset) ? __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))) : 0;
      if (unlikely(type_dict_guard != tp_dict_version)) {
        tp_dict_version = obj_dict_version = 0;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    }
    #endif
  }

  /* "pycx4/basechan.pxi":225
 * 
 *     cpdef is_available(self):
 *         if self.rslv_stat == CDA_RSLVSTAT_FOUND:             # <<<<<<<<<<<<<<
 *             return True
 *         return False
 */
  __pyx_t_5 = ((__pyx_v_self->rslv_stat == CDA_RSLVSTAT_FOUND) != 0);
  if (__pyx_t_5) {

    /* "pycx4/basechan.pxi":226
 *     cpdef is_available(self):
 *         if self.rslv_stat == CDA_RSLVSTAT_FOUND:
 *             return True             # <<<<<<<<<<<<<<
 *         return False
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_True);
    __pyx_r = Py_True;
    goto __pyx_L0;

    /* "pycx4/basechan.pxi":225
 * 
 *     cpdef is_available(self):
 *         if self.rslv_stat == CDA_RSLVSTAT_FOUND:             # <<<<<<<<<<<<<<
 *             return True
 *         return False
 */
  }

  /* "pycx4/basechan.pxi":227
 *         if self.rslv_stat == CDA_RSLVSTAT_FOUND:
 *             return True
 *         return False             # <<<<<<<<<<<<<<
 * 
 *     cpdef rflags_text(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(Py_False);
  __pyx_r = Py_False;
  goto __pyx_L0;

  /* "pycx4/basechan.pxi":224
 *             raise Exception("cda chan error: cname=%s, %s, errcode=%s" % (self.name, cda_last_err(), c_res ))
 * 
 *     cpdef is_available(self):             # <<<<<<<<<<<<<<
 *         if self.rslv_stat == CDA_RSLVSTAT_FOUND:
 *             return True
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pycx4.pycda.BaseChan.is_available", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_9is_available(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_9is_available(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("is_available (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_8is_available(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_8is_available(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("is_available", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_5pycx4_5pycda_8BaseChan_is_available(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 224, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.BaseChan.is_available", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/basechan.pxi":229
 *         return False
 * 
 *     cpdef rflags_text(self):             # <<<<<<<<<<<<<<
 *         return rflags_text(self.rflags)
 * 
 */

static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_11rflags_text(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_5pycx4_5pycda_8BaseChan_rflags_text(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("rflags_text", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    static PY_UINT64_T tp_dict_version = 0, obj_dict_version = 0;
    if (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict && tp_dict_version == __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) && (!Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset || obj_dict_version == __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))))));
    else {
      PY_UINT64_T type_dict_guard = (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict)) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_rflags_text); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 229, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_5pycx4_5pycda_8BaseChan_11rflags_text)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(9, 229, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
      tp_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      obj_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset) ? __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))) : 0;
      if (unlikely(type_dict_guard != tp_dict_version)) {
        tp_dict_version = obj_dict_version = 0;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    }
    #endif
  }

  /* "pycx4/basechan.pxi":230
 * 
 *     cpdef rflags_text(self):
 *         return rflags_text(self.rflags)             # <<<<<<<<<<<<<<
 * 
 *     # TESTING functions, not yet fully implemented
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_5pycx4_5pycda_rflags_text(__pyx_v_self->rflags); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 230, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pycx4/basechan.pxi":229
 *         return False
 * 
 *     cpdef rflags_text(self):             # <<<<<<<<<<<<<<
 *         return rflags_text(self.rflags)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pycx4.pycda.BaseChan.rflags_text", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_11rflags_text(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_11rflags_text(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("rflags_text (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_10rflags_text(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_10rflags_text(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("rflags_text", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_5pycx4_5pycda_8BaseChan_rflags_text(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 229, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.BaseChan.rflags_text", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/basechan.pxi":233
 * 
 *     # TESTING functions, not yet fully implemented
 *     cpdef get_range(self):             # <<<<<<<<<<<<<<
 *         cdef CxAnyVal_t r[2]
 *         cdef cxdtype_t dt
 */

static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_13get_range(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_5pycx4_5pycda_8BaseChan_get_range(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self, int __pyx_skip_dispatch) {
  CxAnyVal_t __pyx_v_r[2];
  cxdtype_t __pyx_v_dt;
  CYTHON_UNUSED int __pyx_v_c_res;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  __Pyx_RefNannySetupContext("get_range", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    static PY_UINT64_T tp_dict_version = 0, obj_dict_version = 0;
    if (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict && tp_dict_version == __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) && (!Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset || obj_dict_version == __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))))));
    else {
      PY_UINT64_T type_dict_guard = (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict)) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_get_range); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 233, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_5pycx4_5pycda_8BaseChan_13get_range)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(9, 233, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
      tp_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      obj_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset) ? __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))) : 0;
      if (unlikely(type_dict_guard != tp_dict_version)) {
        tp_dict_version = obj_dict_version = 0;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    }
    #endif
  }

  /* "pycx4/basechan.pxi":236
 *         cdef CxAnyVal_t r[2]
 *         cdef cxdtype_t dt
 *         c_res = cda_range_of_ref(self.ref, r, &dt)             # <<<<<<<<<<<<<<
 *         if dt == CXDTYPE_UNKNOWN:
 *             self.rng = []
 */
  __pyx_v_c_res = cda_range_of_ref(__pyx_v_self->ref, __pyx_v_r, (&__pyx_v_dt));

  /* "pycx4/basechan.pxi":237
 *         cdef cxdtype_t dt
 *         c_res = cda_range_of_ref(self.ref, r, &dt)
 *         if dt == CXDTYPE_UNKNOWN:             # <<<<<<<<<<<<<<
 *             self.rng = []
 *         else:
 */
  __pyx_t_5 = ((__pyx_v_dt == CXDTYPE_UNKNOWN) != 0);
  if (__pyx_t_5) {

    /* "pycx4/basechan.pxi":238
 *         c_res = cda_range_of_ref(self.ref, r, &dt)
 *         if dt == CXDTYPE_UNKNOWN:
 *             self.rng = []             # <<<<<<<<<<<<<<
 *         else:
 *             self.rng = [aval_value(&(r[0]), dt), aval_value(&(r[1]), dt)]
 */
    __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 238, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->rng);
    __Pyx_DECREF(__pyx_v_self->rng);
    __pyx_v_self->rng = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "pycx4/basechan.pxi":237
 *         cdef cxdtype_t dt
 *         c_res = cda_range_of_ref(self.ref, r, &dt)
 *         if dt == CXDTYPE_UNKNOWN:             # <<<<<<<<<<<<<<
 *             self.rng = []
 *         else:
 */
    goto __pyx_L3;
  }

  /* "pycx4/basechan.pxi":240
 *             self.rng = []
 *         else:
 *             self.rng = [aval_value(&(r[0]), dt), aval_value(&(r[1]), dt)]             # <<<<<<<<<<<<<<
 * 
 *     cpdef get_strings(self):
 */
  /*else*/ {
    __pyx_t_1 = __pyx_f_5pycx4_5pycda_aval_value((&(__pyx_v_r[0])), __pyx_v_dt); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 240, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __pyx_f_5pycx4_5pycda_aval_value((&(__pyx_v_r[1])), __pyx_v_dt); if (unlikely(!__pyx_t_2)) __PYX_ERR(9, 240, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyList_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(9, 240, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_1);
    PyList_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    PyList_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
    __pyx_t_1 = 0;
    __pyx_t_2 = 0;
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_v_self->rng);
    __Pyx_DECREF(__pyx_v_self->rng);
    __pyx_v_self->rng = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;
  }
  __pyx_L3:;

  /* "pycx4/basechan.pxi":233
 * 
 *     # TESTING functions, not yet fully implemented
 *     cpdef get_range(self):             # <<<<<<<<<<<<<<
 *         cdef CxAnyVal_t r[2]
 *         cdef cxdtype_t dt
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pycx4.pycda.BaseChan.get_range", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_13get_range(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_13get_range(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_range (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_12get_range(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_12get_range(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("get_range", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_5pycx4_5pycda_8BaseChan_get_range(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 233, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.BaseChan.get_range", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/basechan.pxi":242
 *             self.rng = [aval_value(&(r[0]), dt), aval_value(&(r[1]), dt)]
 * 
 *     cpdef get_strings(self):             # <<<<<<<<<<<<<<
 *         c_res = cda_strings_of_ref(self.ref, &self.ident, &self.label, &self.tip, &self.comment,
 *                                    &self.geoinfo, &self.rsrvd6, &self.units, &self.dpyfmt)
 */

static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_15get_strings(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_5pycx4_5pycda_8BaseChan_get_strings(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self, int __pyx_skip_dispatch) {
  CYTHON_UNUSED int __pyx_v_c_res;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("get_strings", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    static PY_UINT64_T tp_dict_version = 0, obj_dict_version = 0;
    if (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict && tp_dict_version == __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) && (!Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset || obj_dict_version == __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))))));
    else {
      PY_UINT64_T type_dict_guard = (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict)) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_get_strings); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 242, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_5pycx4_5pycda_8BaseChan_15get_strings)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(9, 242, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
      tp_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      obj_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset) ? __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))) : 0;
      if (unlikely(type_dict_guard != tp_dict_version)) {
        tp_dict_version = obj_dict_version = 0;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    }
    #endif
  }

  /* "pycx4/basechan.pxi":243
 * 
 *     cpdef get_strings(self):
 *         c_res = cda_strings_of_ref(self.ref, &self.ident, &self.label, &self.tip, &self.comment,             # <<<<<<<<<<<<<<
 *                                    &self.geoinfo, &self.rsrvd6, &self.units, &self.dpyfmt)
 *         self.stringsUpdate.emit(self)
 */
  __pyx_v_c_res = cda_strings_of_ref(__pyx_v_self->ref, (&__pyx_v_self->ident), (&__pyx_v_self->label), (&__pyx_v_self->tip), (&__pyx_v_self->comment), (&__pyx_v_self->geoinfo), (&__pyx_v_self->rsrvd6), (&__pyx_v_self->units), (&__pyx_v_self->dpyfmt));

  /* "pycx4/basechan.pxi":245
 *         c_res = cda_strings_of_ref(self.ref, &self.ident, &self.label, &self.tip, &self.comment,
 *                                    &self.geoinfo, &self.rsrvd6, &self.units, &self.dpyfmt)
 *         self.stringsUpdate.emit(self)             # <<<<<<<<<<<<<<
 * 
 *     cpdef get_lockstate(self):
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->stringsUpdate, __pyx_n_s_emit); if (unlikely(!__pyx_t_2)) __PYX_ERR(9, 245, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, ((PyObject *)__pyx_v_self)) : __Pyx_PyObject_CallOneArg(__pyx_t_2, ((PyObject *)__pyx_v_self));
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 245, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/basechan.pxi":242
 *             self.rng = [aval_value(&(r[0]), dt), aval_value(&(r[1]), dt)]
 * 
 *     cpdef get_strings(self):             # <<<<<<<<<<<<<<
 *         c_res = cda_strings_of_ref(self.ref, &self.ident, &self.label, &self.tip, &self.comment,
 *                                    &self.geoinfo, &self.rsrvd6, &self.units, &self.dpyfmt)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pycx4.pycda.BaseChan.get_strings", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_15get_strings(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_15get_strings(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_strings (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_14get_strings(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_14get_strings(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("get_strings", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_5pycx4_5pycda_8BaseChan_get_strings(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 242, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.BaseChan.get_strings", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/basechan.pxi":247
 *         self.stringsUpdate.emit(self)
 * 
 *     cpdef get_lockstate(self):             # <<<<<<<<<<<<<<
 *         self.lock_state = cda_lock_stat_of_ref(self.ref)
 *         self.lockStateUpdate.emit(self)
 */

static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_17get_lockstate(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_5pycx4_5pycda_8BaseChan_get_lockstate(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("get_lockstate", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    static PY_UINT64_T tp_dict_version = 0, obj_dict_version = 0;
    if (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict && tp_dict_version == __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) && (!Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset || obj_dict_version == __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))))));
    else {
      PY_UINT64_T type_dict_guard = (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict)) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_get_lockstate); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 247, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_5pycx4_5pycda_8BaseChan_17get_lockstate)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(9, 247, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
      tp_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      obj_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset) ? __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))) : 0;
      if (unlikely(type_dict_guard != tp_dict_version)) {
        tp_dict_version = obj_dict_version = 0;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    }
    #endif
  }

  /* "pycx4/basechan.pxi":248
 * 
 *     cpdef get_lockstate(self):
 *         self.lock_state = cda_lock_stat_of_ref(self.ref)             # <<<<<<<<<<<<<<
 *         self.lockStateUpdate.emit(self)
 * 
 */
  __pyx_v_self->lock_state = cda_lock_stat_of_ref(__pyx_v_self->ref);

  /* "pycx4/basechan.pxi":249
 *     cpdef get_lockstate(self):
 *         self.lock_state = cda_lock_stat_of_ref(self.ref)
 *         self.lockStateUpdate.emit(self)             # <<<<<<<<<<<<<<
 * 
 *     cpdef lock(self):
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->lockStateUpdate, __pyx_n_s_emit); if (unlikely(!__pyx_t_2)) __PYX_ERR(9, 249, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, ((PyObject *)__pyx_v_self)) : __Pyx_PyObject_CallOneArg(__pyx_t_2, ((PyObject *)__pyx_v_self));
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 249, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/basechan.pxi":247
 *         self.stringsUpdate.emit(self)
 * 
 *     cpdef get_lockstate(self):             # <<<<<<<<<<<<<<
 *         self.lock_state = cda_lock_stat_of_ref(self.ref)
 *         self.lockStateUpdate.emit(self)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pycx4.pycda.BaseChan.get_lockstate", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_17get_lockstate(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_17get_lockstate(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_lockstate (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_16get_lockstate(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_16get_lockstate(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("get_lockstate", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_5pycx4_5pycda_8BaseChan_get_lockstate(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 247, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.BaseChan.get_lockstate", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/basechan.pxi":251
 *         self.lockStateUpdate.emit(self)
 * 
 *     cpdef lock(self):             # <<<<<<<<<<<<<<
 *         cdef int ret = cda_lock_chans(1, &self.ref, CX_LOCK_WRITE_SET, 0)
 * 
 */

static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_19lock(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_5pycx4_5pycda_8BaseChan_lock(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self, int __pyx_skip_dispatch) {
  CYTHON_UNUSED int __pyx_v_ret;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("lock", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    static PY_UINT64_T tp_dict_version = 0, obj_dict_version = 0;
    if (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict && tp_dict_version == __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) && (!Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset || obj_dict_version == __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))))));
    else {
      PY_UINT64_T type_dict_guard = (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict)) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_lock); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 251, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_5pycx4_5pycda_8BaseChan_19lock)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(9, 251, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
      tp_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      obj_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset) ? __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))) : 0;
      if (unlikely(type_dict_guard != tp_dict_version)) {
        tp_dict_version = obj_dict_version = 0;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    }
    #endif
  }

  /* "pycx4/basechan.pxi":252
 * 
 *     cpdef lock(self):
 *         cdef int ret = cda_lock_chans(1, &self.ref, CX_LOCK_WRITE_SET, 0)             # <<<<<<<<<<<<<<
 * 
 *     cpdef unlock(self):
 */
  __pyx_v_ret = cda_lock_chans(1, (&__pyx_v_self->ref), CX_LOCK_WRITE_SET, 0);

  /* "pycx4/basechan.pxi":251
 *         self.lockStateUpdate.emit(self)
 * 
 *     cpdef lock(self):             # <<<<<<<<<<<<<<
 *         cdef int ret = cda_lock_chans(1, &self.ref, CX_LOCK_WRITE_SET, 0)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pycx4.pycda.BaseChan.lock", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_19lock(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_19lock(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("lock (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_18lock(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_18lock(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("lock", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_5pycx4_5pycda_8BaseChan_lock(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 251, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.BaseChan.lock", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/basechan.pxi":254
 *         cdef int ret = cda_lock_chans(1, &self.ref, CX_LOCK_WRITE_SET, 0)
 * 
 *     cpdef unlock(self):             # <<<<<<<<<<<<<<
 *         cdef int ret = cda_lock_chans(1, &self.ref, CX_LOCK_WRITE_RLS, 0)
 * 
 */

static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_21unlock(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_5pycx4_5pycda_8BaseChan_unlock(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self, int __pyx_skip_dispatch) {
  CYTHON_UNUSED int __pyx_v_ret;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("unlock", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    static PY_UINT64_T tp_dict_version = 0, obj_dict_version = 0;
    if (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict && tp_dict_version == __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) && (!Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset || obj_dict_version == __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))))));
    else {
      PY_UINT64_T type_dict_guard = (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict)) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_unlock); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 254, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_5pycx4_5pycda_8BaseChan_21unlock)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(9, 254, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
      tp_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      obj_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset) ? __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))) : 0;
      if (unlikely(type_dict_guard != tp_dict_version)) {
        tp_dict_version = obj_dict_version = 0;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    }
    #endif
  }

  /* "pycx4/basechan.pxi":255
 * 
 *     cpdef unlock(self):
 *         cdef int ret = cda_lock_chans(1, &self.ref, CX_LOCK_WRITE_RLS, 0)             # <<<<<<<<<<<<<<
 * 
 */
  __pyx_v_ret = cda_lock_chans(1, (&__pyx_v_self->ref), CX_LOCK_WRITE_RLS, 0);

  /* "pycx4/basechan.pxi":254
 *         cdef int ret = cda_lock_chans(1, &self.ref, CX_LOCK_WRITE_SET, 0)
 * 
 *     cpdef unlock(self):             # <<<<<<<<<<<<<<
 *         cdef int ret = cda_lock_chans(1, &self.ref, CX_LOCK_WRITE_RLS, 0)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pycx4.pycda.BaseChan.unlock", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_21unlock(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_21unlock(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("unlock (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_20unlock(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_20unlock(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("unlock", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_5pycx4_5pycda_8BaseChan_unlock(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 254, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.BaseChan.unlock", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/basechan.pxi":73
 * cdef class BaseChan(CdaObject):
 *     cdef readonly:
 *         cda_dataref_t ref      # chan id             # <<<<<<<<<<<<<<
 *         str name               # chan name
 *         int max_nelems         # max number of elements when registered
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_3ref_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_3ref_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_3ref___get__(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_3ref___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_CxDataRef_t(__pyx_v_self->ref); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 73, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.BaseChan.ref.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/basechan.pxi":74
 *     cdef readonly:
 *         cda_dataref_t ref      # chan id
 *         str name               # chan name             # <<<<<<<<<<<<<<
 *         int max_nelems         # max number of elements when registered
 *         cxdtype_t dtype        # data type
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_4name_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_4name_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_4name___get__(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_4name___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->name);
  __pyx_r = __pyx_v_self->name;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/basechan.pxi":75
 *         cda_dataref_t ref      # chan id
 *         str name               # chan name
 *         int max_nelems         # max number of elements when registered             # <<<<<<<<<<<<<<
 *         cxdtype_t dtype        # data type
 *         size_t itemsize        # item size in bytes
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_10max_nelems_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_10max_nelems_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_10max_nelems___get__(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_10max_nelems___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->max_nelems); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 75, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.BaseChan.max_nelems.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/basechan.pxi":76
 *         str name               # chan name
 *         int max_nelems         # max number of elements when registered
 *         cxdtype_t dtype        # data type             # <<<<<<<<<<<<<<
 *         size_t itemsize        # item size in bytes
 *         int64 time, prev_time  # last and prev data update time
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_5dtype_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_5dtype_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_5dtype___get__(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_5dtype___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_uint32(__pyx_v_self->dtype); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 76, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.BaseChan.dtype.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/basechan.pxi":77
 *         int max_nelems         # max number of elements when registered
 *         cxdtype_t dtype        # data type
 *         size_t itemsize        # item size in bytes             # <<<<<<<<<<<<<<
 *         int64 time, prev_time  # last and prev data update time
 *         long rslv_stat         # resolving status
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_8itemsize_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_8itemsize_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_8itemsize___get__(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_8itemsize___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_FromSize_t(__pyx_v_self->itemsize); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 77, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.BaseChan.itemsize.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/basechan.pxi":78
 *         cxdtype_t dtype        # data type
 *         size_t itemsize        # item size in bytes
 *         int64 time, prev_time  # last and prev data update time             # <<<<<<<<<<<<<<
 *         long rslv_stat         # resolving status
 *         str rslv_str           # resolving string
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_4time_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_4time_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_4time___get__(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_4time___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int64(__pyx_v_self->time); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 78, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.BaseChan.time.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_9prev_time_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_9prev_time_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_9prev_time___get__(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_9prev_time___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int64(__pyx_v_self->prev_time); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 78, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.BaseChan.prev_time.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/basechan.pxi":79
 *         size_t itemsize        # item size in bytes
 *         int64 time, prev_time  # last and prev data update time
 *         long rslv_stat         # resolving status             # <<<<<<<<<<<<<<
 *         str rslv_str           # resolving string
 *         double quant
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_9rslv_stat_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_9rslv_stat_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_9rslv_stat___get__(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_9rslv_stat___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_v_self->rslv_stat); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 79, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.BaseChan.rslv_stat.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/basechan.pxi":80
 *         int64 time, prev_time  # last and prev data update time
 *         long rslv_stat         # resolving status
 *         str rslv_str           # resolving string             # <<<<<<<<<<<<<<
 *         double quant
 *         int registered, first_cycle, lock_state
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_8rslv_str_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_8rslv_str_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_8rslv_str___get__(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_8rslv_str___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->rslv_str);
  __pyx_r = __pyx_v_self->rslv_str;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/basechan.pxi":81
 *         long rslv_stat         # resolving status
 *         str rslv_str           # resolving string
 *         double quant             # <<<<<<<<<<<<<<
 *         int registered, first_cycle, lock_state
 *         Context context
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_5quant_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_5quant_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_5quant___get__(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_5quant___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->quant); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.BaseChan.quant.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/basechan.pxi":82
 *         str rslv_str           # resolving string
 *         double quant
 *         int registered, first_cycle, lock_state             # <<<<<<<<<<<<<<
 *         Context context
 *         rflags_t rflags        # flags
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_10registered_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_10registered_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_10registered___get__(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_10registered___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->registered); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 82, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.BaseChan.registered.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_11first_cycle_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_11first_cycle_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_11first_cycle___get__(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_11first_cycle___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->first_cycle); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 82, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.BaseChan.first_cycle.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_10lock_state_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_10lock_state_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_10lock_state___get__(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_10lock_state___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->lock_state); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 82, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.BaseChan.lock_state.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/basechan.pxi":83
 *         double quant
 *         int registered, first_cycle, lock_state
 *         Context context             # <<<<<<<<<<<<<<
 *         rflags_t rflags        # flags
 *         list rng               # ranges
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_7context_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_7context_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_7context___get__(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_7context___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->context));
  __pyx_r = ((PyObject *)__pyx_v_self->context);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/basechan.pxi":84
 *         int registered, first_cycle, lock_state
 *         Context context
 *         rflags_t rflags        # flags             # <<<<<<<<<<<<<<
 *         list rng               # ranges
 *         # strings of ref
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_6rflags_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_6rflags_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_6rflags___get__(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_6rflags___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_uint32(__pyx_v_self->rflags); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 84, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.BaseChan.rflags.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/basechan.pxi":85
 *         Context context
 *         rflags_t rflags        # flags
 *         list rng               # ranges             # <<<<<<<<<<<<<<
 *         # strings of ref
 *         char *ident
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_3rng_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_3rng_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_3rng___get__(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_3rng___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->rng);
  __pyx_r = __pyx_v_self->rng;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/basechan.pxi":87
 *         list rng               # ranges
 *         # strings of ref
 *         char *ident             # <<<<<<<<<<<<<<
 *         char *label
 *         char *tip
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_5ident_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_5ident_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_5ident___get__(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_5ident___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBytes_FromString(__pyx_v_self->ident); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 87, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.BaseChan.ident.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/basechan.pxi":88
 *         # strings of ref
 *         char *ident
 *         char *label             # <<<<<<<<<<<<<<
 *         char *tip
 *         char *comment
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_5label_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_5label_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_5label___get__(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_5label___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBytes_FromString(__pyx_v_self->label); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 88, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.BaseChan.label.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/basechan.pxi":89
 *         char *ident
 *         char *label
 *         char *tip             # <<<<<<<<<<<<<<
 *         char *comment
 *         char *geoinfo
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_3tip_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_3tip_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_3tip___get__(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_3tip___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBytes_FromString(__pyx_v_self->tip); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 89, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.BaseChan.tip.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/basechan.pxi":90
 *         char *label
 *         char *tip
 *         char *comment             # <<<<<<<<<<<<<<
 *         char *geoinfo
 *         char *rsrvd6
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_7comment_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_7comment_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_7comment___get__(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_7comment___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBytes_FromString(__pyx_v_self->comment); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 90, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.BaseChan.comment.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/basechan.pxi":91
 *         char *tip
 *         char *comment
 *         char *geoinfo             # <<<<<<<<<<<<<<
 *         char *rsrvd6
 *         char *units
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_7geoinfo_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_7geoinfo_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_7geoinfo___get__(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_7geoinfo___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBytes_FromString(__pyx_v_self->geoinfo); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 91, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.BaseChan.geoinfo.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/basechan.pxi":92
 *         char *comment
 *         char *geoinfo
 *         char *rsrvd6             # <<<<<<<<<<<<<<
 *         char *units
 *         char *dpyfmt
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_6rsrvd6_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_6rsrvd6_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_6rsrvd6___get__(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_6rsrvd6___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBytes_FromString(__pyx_v_self->rsrvd6); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 92, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.BaseChan.rsrvd6.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/basechan.pxi":93
 *         char *geoinfo
 *         char *rsrvd6
 *         char *units             # <<<<<<<<<<<<<<
 *         char *dpyfmt
 *     cdef public:
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_5units_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_5units_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_5units___get__(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_5units___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBytes_FromString(__pyx_v_self->units); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 93, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.BaseChan.units.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/basechan.pxi":94
 *         char *rsrvd6
 *         char *units
 *         char *dpyfmt             # <<<<<<<<<<<<<<
 *     cdef public:
 *         object valueMeasured, valueChanged, resolve, stringsUpdate, lockStateUpdate
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_6dpyfmt_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_6dpyfmt_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_6dpyfmt___get__(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_6dpyfmt___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBytes_FromString(__pyx_v_self->dpyfmt); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 94, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.BaseChan.dpyfmt.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/basechan.pxi":96
 *         char *dpyfmt
 *     cdef public:
 *         object valueMeasured, valueChanged, resolve, stringsUpdate, lockStateUpdate             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, str name, **kwargs):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_13valueMeasured_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_13valueMeasured_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_13valueMeasured___get__(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_13valueMeasured___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->valueMeasured);
  __pyx_r = __pyx_v_self->valueMeasured;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5pycx4_5pycda_8BaseChan_13valueMeasured_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5pycx4_5pycda_8BaseChan_13valueMeasured_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_13valueMeasured_2__set__(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pycx4_5pycda_8BaseChan_13valueMeasured_2__set__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->valueMeasured);
  __Pyx_DECREF(__pyx_v_self->valueMeasured);
  __pyx_v_self->valueMeasured = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5pycx4_5pycda_8BaseChan_13valueMeasured_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5pycx4_5pycda_8BaseChan_13valueMeasured_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_13valueMeasured_4__del__(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pycx4_5pycda_8BaseChan_13valueMeasured_4__del__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->valueMeasured);
  __Pyx_DECREF(__pyx_v_self->valueMeasured);
  __pyx_v_self->valueMeasured = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_12valueChanged_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_12valueChanged_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_12valueChanged___get__(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_12valueChanged___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->valueChanged);
  __pyx_r = __pyx_v_self->valueChanged;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5pycx4_5pycda_8BaseChan_12valueChanged_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5pycx4_5pycda_8BaseChan_12valueChanged_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_12valueChanged_2__set__(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pycx4_5pycda_8BaseChan_12valueChanged_2__set__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->valueChanged);
  __Pyx_DECREF(__pyx_v_self->valueChanged);
  __pyx_v_self->valueChanged = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5pycx4_5pycda_8BaseChan_12valueChanged_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5pycx4_5pycda_8BaseChan_12valueChanged_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_12valueChanged_4__del__(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pycx4_5pycda_8BaseChan_12valueChanged_4__del__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->valueChanged);
  __Pyx_DECREF(__pyx_v_self->valueChanged);
  __pyx_v_self->valueChanged = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_7resolve_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_7resolve_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_7resolve___get__(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_7resolve___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->resolve);
  __pyx_r = __pyx_v_self->resolve;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5pycx4_5pycda_8BaseChan_7resolve_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5pycx4_5pycda_8BaseChan_7resolve_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_7resolve_2__set__(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pycx4_5pycda_8BaseChan_7resolve_2__set__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->resolve);
  __Pyx_DECREF(__pyx_v_self->resolve);
  __pyx_v_self->resolve = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5pycx4_5pycda_8BaseChan_7resolve_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5pycx4_5pycda_8BaseChan_7resolve_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_7resolve_4__del__(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pycx4_5pycda_8BaseChan_7resolve_4__del__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->resolve);
  __Pyx_DECREF(__pyx_v_self->resolve);
  __pyx_v_self->resolve = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_13stringsUpdate_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_13stringsUpdate_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_13stringsUpdate___get__(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_13stringsUpdate___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->stringsUpdate);
  __pyx_r = __pyx_v_self->stringsUpdate;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5pycx4_5pycda_8BaseChan_13stringsUpdate_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5pycx4_5pycda_8BaseChan_13stringsUpdate_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_13stringsUpdate_2__set__(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pycx4_5pycda_8BaseChan_13stringsUpdate_2__set__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->stringsUpdate);
  __Pyx_DECREF(__pyx_v_self->stringsUpdate);
  __pyx_v_self->stringsUpdate = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5pycx4_5pycda_8BaseChan_13stringsUpdate_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5pycx4_5pycda_8BaseChan_13stringsUpdate_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_13stringsUpdate_4__del__(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pycx4_5pycda_8BaseChan_13stringsUpdate_4__del__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->stringsUpdate);
  __Pyx_DECREF(__pyx_v_self->stringsUpdate);
  __pyx_v_self->stringsUpdate = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_15lockStateUpdate_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_15lockStateUpdate_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_15lockStateUpdate___get__(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_15lockStateUpdate___get__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->lockStateUpdate);
  __pyx_r = __pyx_v_self->lockStateUpdate;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5pycx4_5pycda_8BaseChan_15lockStateUpdate_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5pycx4_5pycda_8BaseChan_15lockStateUpdate_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_15lockStateUpdate_2__set__(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pycx4_5pycda_8BaseChan_15lockStateUpdate_2__set__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->lockStateUpdate);
  __Pyx_DECREF(__pyx_v_self->lockStateUpdate);
  __pyx_v_self->lockStateUpdate = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5pycx4_5pycda_8BaseChan_15lockStateUpdate_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5pycx4_5pycda_8BaseChan_15lockStateUpdate_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_15lockStateUpdate_4__del__(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pycx4_5pycda_8BaseChan_15lockStateUpdate_4__del__(struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->lockStateUpdate);
  __Pyx_DECREF(__pyx_v_self->lockStateUpdate);
  __pyx_v_self->lockStateUpdate = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_23__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_23__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_22__reduce_cython__(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_22__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__21, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.BaseChan.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_25__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_8BaseChan_25__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_8BaseChan_24__setstate_cython__(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_8BaseChan_24__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_BaseChan *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__22, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.BaseChan.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/dchan.pxi":14
 *         readonly double val, prev_val, tolerance
 * 
 *     def __init__(self, str name, **kwargs):             # <<<<<<<<<<<<<<
 *         kwargs['dtype'] = CXDTYPE_DOUBLE
 *         BaseChan.__init__(self, name, **kwargs)
 */

/* Python wrapper */
static int __pyx_pw_5pycx4_5pycda_5DChan_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5pycx4_5pycda_5DChan_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_kwargs = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  __pyx_v_kwargs = PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return -1;
  __Pyx_GOTREF(__pyx_v_kwargs);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_name,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_name)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, __pyx_v_kwargs, values, pos_args, "__init__") < 0)) __PYX_ERR(10, 14, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_name = ((PyObject*)values[0]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(10, 14, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_DECREF(__pyx_v_kwargs); __pyx_v_kwargs = 0;
  __Pyx_AddTraceback("pycx4.pycda.DChan.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_name), (&PyUnicode_Type), 1, "name", 1))) __PYX_ERR(10, 14, __pyx_L1_error)
  __pyx_r = __pyx_pf_5pycx4_5pycda_5DChan___init__(((struct __pyx_obj_5pycx4_5pycda_DChan *)__pyx_v_self), __pyx_v_name, __pyx_v_kwargs);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pycx4_5pycda_5DChan___init__(struct __pyx_obj_5pycx4_5pycda_DChan *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_kwargs) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  double __pyx_t_4;
  double __pyx_t_5;
  double __pyx_t_6;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "pycx4/dchan.pxi":15
 * 
 *     def __init__(self, str name, **kwargs):
 *         kwargs['dtype'] = CXDTYPE_DOUBLE             # <<<<<<<<<<<<<<
 *         BaseChan.__init__(self, name, **kwargs)
 *         self.val, self.prev_val, self.tolerance = 0.0, 0.0, 0.0
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(CXDTYPE_DOUBLE); if (unlikely(!__pyx_t_1)) __PYX_ERR(10, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (unlikely(PyDict_SetItem(__pyx_v_kwargs, __pyx_n_u_dtype, __pyx_t_1) < 0)) __PYX_ERR(10, 15, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/dchan.pxi":16
 *     def __init__(self, str name, **kwargs):
 *         kwargs['dtype'] = CXDTYPE_DOUBLE
 *         BaseChan.__init__(self, name, **kwargs)             # <<<<<<<<<<<<<<
 *         self.val, self.prev_val, self.tolerance = 0.0, 0.0, 0.0
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_5pycx4_5pycda_BaseChan), __pyx_n_s_init); if (unlikely(!__pyx_t_1)) __PYX_ERR(10, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(10, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_v_self));
  __Pyx_INCREF(__pyx_v_name);
  __Pyx_GIVEREF(__pyx_v_name);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_name);
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_2, __pyx_v_kwargs); if (unlikely(!__pyx_t_3)) __PYX_ERR(10, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "pycx4/dchan.pxi":17
 *         kwargs['dtype'] = CXDTYPE_DOUBLE
 *         BaseChan.__init__(self, name, **kwargs)
 *         self.val, self.prev_val, self.tolerance = 0.0, 0.0, 0.0             # <<<<<<<<<<<<<<
 * 
 *     cdef void cb(self):
 */
  __pyx_t_4 = 0.0;
  __pyx_t_5 = 0.0;
  __pyx_t_6 = 0.0;
  __pyx_v_self->val = __pyx_t_4;
  __pyx_v_self->prev_val = __pyx_t_5;
  __pyx_v_self->tolerance = __pyx_t_6;

  /* "pycx4/dchan.pxi":14
 *         readonly double val, prev_val, tolerance
 * 
 *     def __init__(self, str name, **kwargs):             # <<<<<<<<<<<<<<
 *         kwargs['dtype'] = CXDTYPE_DOUBLE
 *         BaseChan.__init__(self, name, **kwargs)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("pycx4.pycda.DChan.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/dchan.pxi":19
 *         self.val, self.prev_val, self.tolerance = 0.0, 0.0, 0.0
 * 
 *     cdef void cb(self):             # <<<<<<<<<<<<<<
 *         self.prev_val = self.val
 *         self.check_exception(cda_get_dcval(self.ref, &self.val))
 */

static void __pyx_f_5pycx4_5pycda_5DChan_cb(struct __pyx_obj_5pycx4_5pycda_DChan *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("cb", 0);

  /* "pycx4/dchan.pxi":20
 * 
 *     cdef void cb(self):
 *         self.prev_val = self.val             # <<<<<<<<<<<<<<
 *         self.check_exception(cda_get_dcval(self.ref, &self.val))
 *         if abs(self.val - self.prev_val) > self.tolerance or self.first_cycle:
 */
  __pyx_t_1 = __pyx_v_self->val;
  __pyx_v_self->prev_val = __pyx_t_1;

  /* "pycx4/dchan.pxi":21
 *     cdef void cb(self):
 *         self.prev_val = self.val
 *         self.check_exception(cda_get_dcval(self.ref, &self.val))             # <<<<<<<<<<<<<<
 *         if abs(self.val - self.prev_val) > self.tolerance or self.first_cycle:
 *             self.valueChanged.emit(self)
 */
  ((struct __pyx_vtabstruct_5pycx4_5pycda_DChan *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.check_exception(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self), cda_get_dcval(__pyx_v_self->__pyx_base.ref, (&__pyx_v_self->val)));

  /* "pycx4/dchan.pxi":22
 *         self.prev_val = self.val
 *         self.check_exception(cda_get_dcval(self.ref, &self.val))
 *         if abs(self.val - self.prev_val) > self.tolerance or self.first_cycle:             # <<<<<<<<<<<<<<
 *             self.valueChanged.emit(self)
 *         self.valueMeasured.emit(self)
 */
  __pyx_t_3 = ((fabs((__pyx_v_self->val - __pyx_v_self->prev_val)) > __pyx_v_self->tolerance) != 0);
  if (!__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = (__pyx_v_self->__pyx_base.first_cycle != 0);
  __pyx_t_2 = __pyx_t_3;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "pycx4/dchan.pxi":23
 *         self.check_exception(cda_get_dcval(self.ref, &self.val))
 *         if abs(self.val - self.prev_val) > self.tolerance or self.first_cycle:
 *             self.valueChanged.emit(self)             # <<<<<<<<<<<<<<
 *         self.valueMeasured.emit(self)
 *         self.first_cycle = False
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->__pyx_base.valueChanged, __pyx_n_s_emit); if (unlikely(!__pyx_t_5)) __PYX_ERR(10, 23, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    __pyx_t_4 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_6, ((PyObject *)__pyx_v_self)) : __Pyx_PyObject_CallOneArg(__pyx_t_5, ((PyObject *)__pyx_v_self));
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(10, 23, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "pycx4/dchan.pxi":22
 *         self.prev_val = self.val
 *         self.check_exception(cda_get_dcval(self.ref, &self.val))
 *         if abs(self.val - self.prev_val) > self.tolerance or self.first_cycle:             # <<<<<<<<<<<<<<
 *             self.valueChanged.emit(self)
 *         self.valueMeasured.emit(self)
 */
  }

  /* "pycx4/dchan.pxi":24
 *         if abs(self.val - self.prev_val) > self.tolerance or self.first_cycle:
 *             self.valueChanged.emit(self)
 *         self.valueMeasured.emit(self)             # <<<<<<<<<<<<<<
 *         self.first_cycle = False
 * 
 */
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->__pyx_base.valueMeasured, __pyx_n_s_emit); if (unlikely(!__pyx_t_5)) __PYX_ERR(10, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  __pyx_t_4 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_6, ((PyObject *)__pyx_v_self)) : __Pyx_PyObject_CallOneArg(__pyx_t_5, ((PyObject *)__pyx_v_self));
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (unlikely(!__pyx_t_4)) __PYX_ERR(10, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pycx4/dchan.pxi":25
 *             self.valueChanged.emit(self)
 *         self.valueMeasured.emit(self)
 *         self.first_cycle = False             # <<<<<<<<<<<<<<
 * 
 *     cpdef void setValue(self, double value):
 */
  __pyx_v_self->__pyx_base.first_cycle = 0;

  /* "pycx4/dchan.pxi":19
 *         self.val, self.prev_val, self.tolerance = 0.0, 0.0, 0.0
 * 
 *     cdef void cb(self):             # <<<<<<<<<<<<<<
 *         self.prev_val = self.val
 *         self.check_exception(cda_get_dcval(self.ref, &self.val))
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_WriteUnraisable("pycx4.pycda.DChan.cb", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/dchan.pxi":27
 *         self.first_cycle = False
 * 
 *     cpdef void setValue(self, double value):             # <<<<<<<<<<<<<<
 *         self.check_exception( cda_set_dcval(self.ref, value) )
 * 
 */

static PyObject *__pyx_pw_5pycx4_5pycda_5DChan_3setValue(PyObject *__pyx_v_self, PyObject *__pyx_arg_value); /*proto*/
static void __pyx_f_5pycx4_5pycda_5DChan_setValue(struct __pyx_obj_5pycx4_5pycda_DChan *__pyx_v_self, double __pyx_v_value, int __pyx_skip_dispatch) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("setValue", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    static PY_UINT64_T tp_dict_version = 0, obj_dict_version = 0;
    if (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict && tp_dict_version == __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) && (!Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset || obj_dict_version == __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))))));
    else {
      PY_UINT64_T type_dict_guard = (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict)) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_setValue); if (unlikely(!__pyx_t_1)) __PYX_ERR(10, 27, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_5pycx4_5pycda_5DChan_3setValue)) {
        __pyx_t_3 = PyFloat_FromDouble(__pyx_v_value); if (unlikely(!__pyx_t_3)) __PYX_ERR(10, 27, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; __pyx_t_5 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
          if (likely(__pyx_t_5)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
            __Pyx_INCREF(__pyx_t_5);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_4, function);
          }
        }
        __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_5, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_3);
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(10, 27, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
      tp_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      obj_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset) ? __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))) : 0;
      if (unlikely(type_dict_guard != tp_dict_version)) {
        tp_dict_version = obj_dict_version = 0;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    }
    #endif
  }

  /* "pycx4/dchan.pxi":28
 * 
 *     cpdef void setValue(self, double value):
 *         self.check_exception( cda_set_dcval(self.ref, value) )             # <<<<<<<<<<<<<<
 * 
 *     cpdef void setTolerance(self, double new_tolerance):
 */
  ((struct __pyx_vtabstruct_5pycx4_5pycda_DChan *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.check_exception(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self), cda_set_dcval(__pyx_v_self->__pyx_base.ref, __pyx_v_value));

  /* "pycx4/dchan.pxi":27
 *         self.first_cycle = False
 * 
 *     cpdef void setValue(self, double value):             # <<<<<<<<<<<<<<
 *         self.check_exception( cda_set_dcval(self.ref, value) )
 * 
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_WriteUnraisable("pycx4.pycda.DChan.setValue", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_5DChan_3setValue(PyObject *__pyx_v_self, PyObject *__pyx_arg_value); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_5DChan_3setValue(PyObject *__pyx_v_self, PyObject *__pyx_arg_value) {
  double __pyx_v_value;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("setValue (wrapper)", 0);
  assert(__pyx_arg_value); {
    __pyx_v_value = __pyx_PyFloat_AsDouble(__pyx_arg_value); if (unlikely((__pyx_v_value == (double)-1) && PyErr_Occurred())) __PYX_ERR(10, 27, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pycx4.pycda.DChan.setValue", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pycx4_5pycda_5DChan_2setValue(((struct __pyx_obj_5pycx4_5pycda_DChan *)__pyx_v_self), ((double)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_5DChan_2setValue(struct __pyx_obj_5pycx4_5pycda_DChan *__pyx_v_self, double __pyx_v_value) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("setValue", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_void_to_None(__pyx_f_5pycx4_5pycda_5DChan_setValue(__pyx_v_self, __pyx_v_value, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(10, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.DChan.setValue", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/dchan.pxi":30
 *         self.check_exception( cda_set_dcval(self.ref, value) )
 * 
 *     cpdef void setTolerance(self, double new_tolerance):             # <<<<<<<<<<<<<<
 *         self.tolerance = new_tolerance
 */

static PyObject *__pyx_pw_5pycx4_5pycda_5DChan_5setTolerance(PyObject *__pyx_v_self, PyObject *__pyx_arg_new_tolerance); /*proto*/
static void __pyx_f_5pycx4_5pycda_5DChan_setTolerance(struct __pyx_obj_5pycx4_5pycda_DChan *__pyx_v_self, double __pyx_v_new_tolerance, int __pyx_skip_dispatch) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("setTolerance", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    static PY_UINT64_T tp_dict_version = 0, obj_dict_version = 0;
    if (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict && tp_dict_version == __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) && (!Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset || obj_dict_version == __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))))));
    else {
      PY_UINT64_T type_dict_guard = (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict)) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_setTolerance); if (unlikely(!__pyx_t_1)) __PYX_ERR(10, 30, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_5pycx4_5pycda_5DChan_5setTolerance)) {
        __pyx_t_3 = PyFloat_FromDouble(__pyx_v_new_tolerance); if (unlikely(!__pyx_t_3)) __PYX_ERR(10, 30, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; __pyx_t_5 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
          if (likely(__pyx_t_5)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
            __Pyx_INCREF(__pyx_t_5);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_4, function);
          }
        }
        __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_5, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_3);
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(10, 30, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
      tp_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      obj_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset) ? __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))) : 0;
      if (unlikely(type_dict_guard != tp_dict_version)) {
        tp_dict_version = obj_dict_version = 0;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    }
    #endif
  }

  /* "pycx4/dchan.pxi":31
 * 
 *     cpdef void setTolerance(self, double new_tolerance):
 *         self.tolerance = new_tolerance             # <<<<<<<<<<<<<<
 */
  __pyx_v_self->tolerance = __pyx_v_new_tolerance;

  /* "pycx4/dchan.pxi":30
 *         self.check_exception( cda_set_dcval(self.ref, value) )
 * 
 *     cpdef void setTolerance(self, double new_tolerance):             # <<<<<<<<<<<<<<
 *         self.tolerance = new_tolerance
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_WriteUnraisable("pycx4.pycda.DChan.setTolerance", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_5DChan_5setTolerance(PyObject *__pyx_v_self, PyObject *__pyx_arg_new_tolerance); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_5DChan_5setTolerance(PyObject *__pyx_v_self, PyObject *__pyx_arg_new_tolerance) {
  double __pyx_v_new_tolerance;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("setTolerance (wrapper)", 0);
  assert(__pyx_arg_new_tolerance); {
    __pyx_v_new_tolerance = __pyx_PyFloat_AsDouble(__pyx_arg_new_tolerance); if (unlikely((__pyx_v_new_tolerance == (double)-1) && PyErr_Occurred())) __PYX_ERR(10, 30, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pycx4.pycda.DChan.setTolerance", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pycx4_5pycda_5DChan_4setTolerance(((struct __pyx_obj_5pycx4_5pycda_DChan *)__pyx_v_self), ((double)__pyx_v_new_tolerance));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_5DChan_4setTolerance(struct __pyx_obj_5pycx4_5pycda_DChan *__pyx_v_self, double __pyx_v_new_tolerance) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("setTolerance", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_void_to_None(__pyx_f_5pycx4_5pycda_5DChan_setTolerance(__pyx_v_self, __pyx_v_new_tolerance, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(10, 30, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.DChan.setTolerance", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/dchan.pxi":12
 *     cdef:
 *         # all general properties defined in base classes
 *         readonly double val, prev_val, tolerance             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, str name, **kwargs):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_5DChan_3val_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_5DChan_3val_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_5DChan_3val___get__(((struct __pyx_obj_5pycx4_5pycda_DChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_5DChan_3val___get__(struct __pyx_obj_5pycx4_5pycda_DChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->val); if (unlikely(!__pyx_t_1)) __PYX_ERR(10, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.DChan.val.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_5DChan_8prev_val_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_5DChan_8prev_val_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_5DChan_8prev_val___get__(((struct __pyx_obj_5pycx4_5pycda_DChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_5DChan_8prev_val___get__(struct __pyx_obj_5pycx4_5pycda_DChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->prev_val); if (unlikely(!__pyx_t_1)) __PYX_ERR(10, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.DChan.prev_val.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_5DChan_9tolerance_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_5DChan_9tolerance_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_5DChan_9tolerance___get__(((struct __pyx_obj_5pycx4_5pycda_DChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_5DChan_9tolerance___get__(struct __pyx_obj_5pycx4_5pycda_DChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->tolerance); if (unlikely(!__pyx_t_1)) __PYX_ERR(10, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.DChan.tolerance.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_5DChan_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_5DChan_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_5DChan_6__reduce_cython__(((struct __pyx_obj_5pycx4_5pycda_DChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_5DChan_6__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_DChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__23, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.DChan.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_5DChan_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_5DChan_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_5DChan_8__setstate_cython__(((struct __pyx_obj_5pycx4_5pycda_DChan *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_5DChan_8__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_DChan *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__24, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.DChan.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/ichan.pxi":7
 *         readonly int val, prev_val, tolerance
 * 
 *     def __init__(self, str name, **kwargs):             # <<<<<<<<<<<<<<
 *         kwargs['dtype'] = CXDTYPE_UINT32
 *         BaseChan.__init__(self, name, **kwargs)
 */

/* Python wrapper */
static int __pyx_pw_5pycx4_5pycda_5IChan_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5pycx4_5pycda_5IChan_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_kwargs = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  __pyx_v_kwargs = PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return -1;
  __Pyx_GOTREF(__pyx_v_kwargs);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_name,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_name)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, __pyx_v_kwargs, values, pos_args, "__init__") < 0)) __PYX_ERR(11, 7, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_name = ((PyObject*)values[0]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(11, 7, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_DECREF(__pyx_v_kwargs); __pyx_v_kwargs = 0;
  __Pyx_AddTraceback("pycx4.pycda.IChan.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_name), (&PyUnicode_Type), 1, "name", 1))) __PYX_ERR(11, 7, __pyx_L1_error)
  __pyx_r = __pyx_pf_5pycx4_5pycda_5IChan___init__(((struct __pyx_obj_5pycx4_5pycda_IChan *)__pyx_v_self), __pyx_v_name, __pyx_v_kwargs);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pycx4_5pycda_5IChan___init__(struct __pyx_obj_5pycx4_5pycda_IChan *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_kwargs) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "pycx4/ichan.pxi":8
 * 
 *     def __init__(self, str name, **kwargs):
 *         kwargs['dtype'] = CXDTYPE_UINT32             # <<<<<<<<<<<<<<
 *         BaseChan.__init__(self, name, **kwargs)
 *         self.tolerance = 0
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(CXDTYPE_UINT32); if (unlikely(!__pyx_t_1)) __PYX_ERR(11, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (unlikely(PyDict_SetItem(__pyx_v_kwargs, __pyx_n_u_dtype, __pyx_t_1) < 0)) __PYX_ERR(11, 8, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/ichan.pxi":9
 *     def __init__(self, str name, **kwargs):
 *         kwargs['dtype'] = CXDTYPE_UINT32
 *         BaseChan.__init__(self, name, **kwargs)             # <<<<<<<<<<<<<<
 *         self.tolerance = 0
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_5pycx4_5pycda_BaseChan), __pyx_n_s_init); if (unlikely(!__pyx_t_1)) __PYX_ERR(11, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(11, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_v_self));
  __Pyx_INCREF(__pyx_v_name);
  __Pyx_GIVEREF(__pyx_v_name);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_name);
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_2, __pyx_v_kwargs); if (unlikely(!__pyx_t_3)) __PYX_ERR(11, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "pycx4/ichan.pxi":10
 *         kwargs['dtype'] = CXDTYPE_UINT32
 *         BaseChan.__init__(self, name, **kwargs)
 *         self.tolerance = 0             # <<<<<<<<<<<<<<
 * 
 *     cdef void cb(self):
 */
  __pyx_v_self->tolerance = 0;

  /* "pycx4/ichan.pxi":7
 *         readonly int val, prev_val, tolerance
 * 
 *     def __init__(self, str name, **kwargs):             # <<<<<<<<<<<<<<
 *         kwargs['dtype'] = CXDTYPE_UINT32
 *         BaseChan.__init__(self, name, **kwargs)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("pycx4.pycda.IChan.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/ichan.pxi":12
 *         self.tolerance = 0
 * 
 *     cdef void cb(self):             # <<<<<<<<<<<<<<
 *         self.prev_val = self.val
 *         self.check_exception(cda_get_icval(self.ref, &self.val))
 */

static void __pyx_f_5pycx4_5pycda_5IChan_cb(struct __pyx_obj_5pycx4_5pycda_IChan *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("cb", 0);

  /* "pycx4/ichan.pxi":13
 * 
 *     cdef void cb(self):
 *         self.prev_val = self.val             # <<<<<<<<<<<<<<
 *         self.check_exception(cda_get_icval(self.ref, &self.val))
 *         if abs(self.val - self.prev_val) > self.tolerance or self.first_cycle:
 */
  __pyx_t_1 = __pyx_v_self->val;
  __pyx_v_self->prev_val = __pyx_t_1;

  /* "pycx4/ichan.pxi":14
 *     cdef void cb(self):
 *         self.prev_val = self.val
 *         self.check_exception(cda_get_icval(self.ref, &self.val))             # <<<<<<<<<<<<<<
 *         if abs(self.val - self.prev_val) > self.tolerance or self.first_cycle:
 *             self.valueChanged.emit(self)
 */
  ((struct __pyx_vtabstruct_5pycx4_5pycda_IChan *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.check_exception(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self), cda_get_icval(__pyx_v_self->__pyx_base.ref, (&__pyx_v_self->val)));

  /* "pycx4/ichan.pxi":15
 *         self.prev_val = self.val
 *         self.check_exception(cda_get_icval(self.ref, &self.val))
 *         if abs(self.val - self.prev_val) > self.tolerance or self.first_cycle:             # <<<<<<<<<<<<<<
 *             self.valueChanged.emit(self)
 *         self.valueMeasured.emit(self)
 */
  __pyx_t_1 = abs((__pyx_v_self->val - __pyx_v_self->prev_val)); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(11, 15, __pyx_L1_error)
  __pyx_t_3 = ((__pyx_t_1 > __pyx_v_self->tolerance) != 0);
  if (!__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = (__pyx_v_self->__pyx_base.first_cycle != 0);
  __pyx_t_2 = __pyx_t_3;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "pycx4/ichan.pxi":16
 *         self.check_exception(cda_get_icval(self.ref, &self.val))
 *         if abs(self.val - self.prev_val) > self.tolerance or self.first_cycle:
 *             self.valueChanged.emit(self)             # <<<<<<<<<<<<<<
 *         self.valueMeasured.emit(self)
 *         self.first_cycle = False
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->__pyx_base.valueChanged, __pyx_n_s_emit); if (unlikely(!__pyx_t_5)) __PYX_ERR(11, 16, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    __pyx_t_4 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_6, ((PyObject *)__pyx_v_self)) : __Pyx_PyObject_CallOneArg(__pyx_t_5, ((PyObject *)__pyx_v_self));
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(11, 16, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "pycx4/ichan.pxi":15
 *         self.prev_val = self.val
 *         self.check_exception(cda_get_icval(self.ref, &self.val))
 *         if abs(self.val - self.prev_val) > self.tolerance or self.first_cycle:             # <<<<<<<<<<<<<<
 *             self.valueChanged.emit(self)
 *         self.valueMeasured.emit(self)
 */
  }

  /* "pycx4/ichan.pxi":17
 *         if abs(self.val - self.prev_val) > self.tolerance or self.first_cycle:
 *             self.valueChanged.emit(self)
 *         self.valueMeasured.emit(self)             # <<<<<<<<<<<<<<
 *         self.first_cycle = False
 * 
 */
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->__pyx_base.valueMeasured, __pyx_n_s_emit); if (unlikely(!__pyx_t_5)) __PYX_ERR(11, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  __pyx_t_4 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_6, ((PyObject *)__pyx_v_self)) : __Pyx_PyObject_CallOneArg(__pyx_t_5, ((PyObject *)__pyx_v_self));
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (unlikely(!__pyx_t_4)) __PYX_ERR(11, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pycx4/ichan.pxi":18
 *             self.valueChanged.emit(self)
 *         self.valueMeasured.emit(self)
 *         self.first_cycle = False             # <<<<<<<<<<<<<<
 * 
 *     cpdef void setValue(self, int value):
 */
  __pyx_v_self->__pyx_base.first_cycle = 0;

  /* "pycx4/ichan.pxi":12
 *         self.tolerance = 0
 * 
 *     cdef void cb(self):             # <<<<<<<<<<<<<<
 *         self.prev_val = self.val
 *         self.check_exception(cda_get_icval(self.ref, &self.val))
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_WriteUnraisable("pycx4.pycda.IChan.cb", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/ichan.pxi":20
 *         self.first_cycle = False
 * 
 *     cpdef void setValue(self, int value):             # <<<<<<<<<<<<<<
 *         self.check_exception( cda_set_icval(self.ref, value) )
 * 
 */

static PyObject *__pyx_pw_5pycx4_5pycda_5IChan_3setValue(PyObject *__pyx_v_self, PyObject *__pyx_arg_value); /*proto*/
static void __pyx_f_5pycx4_5pycda_5IChan_setValue(struct __pyx_obj_5pycx4_5pycda_IChan *__pyx_v_self, int __pyx_v_value, int __pyx_skip_dispatch) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("setValue", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    static PY_UINT64_T tp_dict_version = 0, obj_dict_version = 0;
    if (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict && tp_dict_version == __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) && (!Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset || obj_dict_version == __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))))));
    else {
      PY_UINT64_T type_dict_guard = (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict)) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_setValue); if (unlikely(!__pyx_t_1)) __PYX_ERR(11, 20, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_5pycx4_5pycda_5IChan_3setValue)) {
        __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_value); if (unlikely(!__pyx_t_3)) __PYX_ERR(11, 20, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; __pyx_t_5 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
          if (likely(__pyx_t_5)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
            __Pyx_INCREF(__pyx_t_5);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_4, function);
          }
        }
        __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_5, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_3);
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(11, 20, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
      tp_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      obj_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset) ? __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))) : 0;
      if (unlikely(type_dict_guard != tp_dict_version)) {
        tp_dict_version = obj_dict_version = 0;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    }
    #endif
  }

  /* "pycx4/ichan.pxi":21
 * 
 *     cpdef void setValue(self, int value):
 *         self.check_exception( cda_set_icval(self.ref, value) )             # <<<<<<<<<<<<<<
 * 
 *     cpdef void setTolerance(self, int new_tolerance):
 */
  ((struct __pyx_vtabstruct_5pycx4_5pycda_IChan *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.check_exception(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self), cda_set_icval(__pyx_v_self->__pyx_base.ref, __pyx_v_value));

  /* "pycx4/ichan.pxi":20
 *         self.first_cycle = False
 * 
 *     cpdef void setValue(self, int value):             # <<<<<<<<<<<<<<
 *         self.check_exception( cda_set_icval(self.ref, value) )
 * 
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_WriteUnraisable("pycx4.pycda.IChan.setValue", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_5IChan_3setValue(PyObject *__pyx_v_self, PyObject *__pyx_arg_value); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_5IChan_3setValue(PyObject *__pyx_v_self, PyObject *__pyx_arg_value) {
  int __pyx_v_value;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("setValue (wrapper)", 0);
  assert(__pyx_arg_value); {
    __pyx_v_value = __Pyx_PyInt_As_int(__pyx_arg_value); if (unlikely((__pyx_v_value == (int)-1) && PyErr_Occurred())) __PYX_ERR(11, 20, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pycx4.pycda.IChan.setValue", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pycx4_5pycda_5IChan_2setValue(((struct __pyx_obj_5pycx4_5pycda_IChan *)__pyx_v_self), ((int)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_5IChan_2setValue(struct __pyx_obj_5pycx4_5pycda_IChan *__pyx_v_self, int __pyx_v_value) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("setValue", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_void_to_None(__pyx_f_5pycx4_5pycda_5IChan_setValue(__pyx_v_self, __pyx_v_value, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(11, 20, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.IChan.setValue", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/ichan.pxi":23
 *         self.check_exception( cda_set_icval(self.ref, value) )
 * 
 *     cpdef void setTolerance(self, int new_tolerance):             # <<<<<<<<<<<<<<
 *         self.tolerance = new_tolerance
 */

static PyObject *__pyx_pw_5pycx4_5pycda_5IChan_5setTolerance(PyObject *__pyx_v_self, PyObject *__pyx_arg_new_tolerance); /*proto*/
static void __pyx_f_5pycx4_5pycda_5IChan_setTolerance(struct __pyx_obj_5pycx4_5pycda_IChan *__pyx_v_self, int __pyx_v_new_tolerance, int __pyx_skip_dispatch) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("setTolerance", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    static PY_UINT64_T tp_dict_version = 0, obj_dict_version = 0;
    if (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict && tp_dict_version == __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) && (!Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset || obj_dict_version == __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))))));
    else {
      PY_UINT64_T type_dict_guard = (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict)) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_setTolerance); if (unlikely(!__pyx_t_1)) __PYX_ERR(11, 23, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_5pycx4_5pycda_5IChan_5setTolerance)) {
        __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_new_tolerance); if (unlikely(!__pyx_t_3)) __PYX_ERR(11, 23, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; __pyx_t_5 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
          if (likely(__pyx_t_5)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
            __Pyx_INCREF(__pyx_t_5);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_4, function);
          }
        }
        __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_5, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_3);
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(11, 23, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
      tp_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      obj_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset) ? __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))) : 0;
      if (unlikely(type_dict_guard != tp_dict_version)) {
        tp_dict_version = obj_dict_version = 0;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    }
    #endif
  }

  /* "pycx4/ichan.pxi":24
 * 
 *     cpdef void setTolerance(self, int new_tolerance):
 *         self.tolerance = new_tolerance             # <<<<<<<<<<<<<<
 */
  __pyx_v_self->tolerance = __pyx_v_new_tolerance;

  /* "pycx4/ichan.pxi":23
 *         self.check_exception( cda_set_icval(self.ref, value) )
 * 
 *     cpdef void setTolerance(self, int new_tolerance):             # <<<<<<<<<<<<<<
 *         self.tolerance = new_tolerance
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_WriteUnraisable("pycx4.pycda.IChan.setTolerance", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_5IChan_5setTolerance(PyObject *__pyx_v_self, PyObject *__pyx_arg_new_tolerance); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_5IChan_5setTolerance(PyObject *__pyx_v_self, PyObject *__pyx_arg_new_tolerance) {
  int __pyx_v_new_tolerance;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("setTolerance (wrapper)", 0);
  assert(__pyx_arg_new_tolerance); {
    __pyx_v_new_tolerance = __Pyx_PyInt_As_int(__pyx_arg_new_tolerance); if (unlikely((__pyx_v_new_tolerance == (int)-1) && PyErr_Occurred())) __PYX_ERR(11, 23, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pycx4.pycda.IChan.setTolerance", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pycx4_5pycda_5IChan_4setTolerance(((struct __pyx_obj_5pycx4_5pycda_IChan *)__pyx_v_self), ((int)__pyx_v_new_tolerance));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_5IChan_4setTolerance(struct __pyx_obj_5pycx4_5pycda_IChan *__pyx_v_self, int __pyx_v_new_tolerance) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("setTolerance", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_void_to_None(__pyx_f_5pycx4_5pycda_5IChan_setTolerance(__pyx_v_self, __pyx_v_new_tolerance, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(11, 23, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.IChan.setTolerance", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/ichan.pxi":5
 *     cdef:
 *         # all general properties defined in base classes
 *         readonly int val, prev_val, tolerance             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, str name, **kwargs):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_5IChan_3val_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_5IChan_3val_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_5IChan_3val___get__(((struct __pyx_obj_5pycx4_5pycda_IChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_5IChan_3val___get__(struct __pyx_obj_5pycx4_5pycda_IChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->val); if (unlikely(!__pyx_t_1)) __PYX_ERR(11, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.IChan.val.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_5IChan_8prev_val_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_5IChan_8prev_val_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_5IChan_8prev_val___get__(((struct __pyx_obj_5pycx4_5pycda_IChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_5IChan_8prev_val___get__(struct __pyx_obj_5pycx4_5pycda_IChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->prev_val); if (unlikely(!__pyx_t_1)) __PYX_ERR(11, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.IChan.prev_val.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_5IChan_9tolerance_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_5IChan_9tolerance_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_5IChan_9tolerance___get__(((struct __pyx_obj_5pycx4_5pycda_IChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_5IChan_9tolerance___get__(struct __pyx_obj_5pycx4_5pycda_IChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->tolerance); if (unlikely(!__pyx_t_1)) __PYX_ERR(11, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.IChan.tolerance.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_5IChan_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_5IChan_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_5IChan_6__reduce_cython__(((struct __pyx_obj_5pycx4_5pycda_IChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_5IChan_6__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_IChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__25, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.IChan.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_5IChan_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_5IChan_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_5IChan_8__setstate_cython__(((struct __pyx_obj_5pycx4_5pycda_IChan *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_5IChan_8__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_IChan *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__26, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.IChan.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/chan.pxi":6
 *         readonly object val, prev_val
 * 
 *     def __init__(self, str name, **kwargs):             # <<<<<<<<<<<<<<
 *         BaseChan.__init__(self, name, **kwargs)
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pycx4_5pycda_4Chan_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5pycx4_5pycda_4Chan_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_kwargs = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  __pyx_v_kwargs = PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return -1;
  __Pyx_GOTREF(__pyx_v_kwargs);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_name,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_name)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, __pyx_v_kwargs, values, pos_args, "__init__") < 0)) __PYX_ERR(12, 6, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_name = ((PyObject*)values[0]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(12, 6, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_DECREF(__pyx_v_kwargs); __pyx_v_kwargs = 0;
  __Pyx_AddTraceback("pycx4.pycda.Chan.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_name), (&PyUnicode_Type), 1, "name", 1))) __PYX_ERR(12, 6, __pyx_L1_error)
  __pyx_r = __pyx_pf_5pycx4_5pycda_4Chan___init__(((struct __pyx_obj_5pycx4_5pycda_Chan *)__pyx_v_self), __pyx_v_name, __pyx_v_kwargs);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pycx4_5pycda_4Chan___init__(struct __pyx_obj_5pycx4_5pycda_Chan *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_kwargs) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "pycx4/chan.pxi":7
 * 
 *     def __init__(self, str name, **kwargs):
 *         BaseChan.__init__(self, name, **kwargs)             # <<<<<<<<<<<<<<
 * 
 *     cdef void cb(self):
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_5pycx4_5pycda_BaseChan), __pyx_n_s_init); if (unlikely(!__pyx_t_1)) __PYX_ERR(12, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(12, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_v_self));
  __Pyx_INCREF(__pyx_v_name);
  __Pyx_GIVEREF(__pyx_v_name);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_name);
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_2, __pyx_v_kwargs); if (unlikely(!__pyx_t_3)) __PYX_ERR(12, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "pycx4/chan.pxi":6
 *         readonly object val, prev_val
 * 
 *     def __init__(self, str name, **kwargs):             # <<<<<<<<<<<<<<
 *         BaseChan.__init__(self, name, **kwargs)
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("pycx4.pycda.Chan.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/chan.pxi":9
 *         BaseChan.__init__(self, name, **kwargs)
 * 
 *     cdef void cb(self):             # <<<<<<<<<<<<<<
 *         cdef CxAnyVal_t aval
 *         self.prev_val = self.val
 */

static void __pyx_f_5pycx4_5pycda_4Chan_cb(struct __pyx_obj_5pycx4_5pycda_Chan *__pyx_v_self) {
  CxAnyVal_t __pyx_v_aval;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("cb", 0);

  /* "pycx4/chan.pxi":11
 *     cdef void cb(self):
 *         cdef CxAnyVal_t aval
 *         self.prev_val = self.val             # <<<<<<<<<<<<<<
 *         self.get_data(0, self.itemsize, <void*>&aval)
 *         self.val = aval_value(&aval, self.dtype)
 */
  __pyx_t_1 = __pyx_v_self->val;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->prev_val);
  __Pyx_DECREF(__pyx_v_self->prev_val);
  __pyx_v_self->prev_val = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pycx4/chan.pxi":12
 *         cdef CxAnyVal_t aval
 *         self.prev_val = self.val
 *         self.get_data(0, self.itemsize, <void*>&aval)             # <<<<<<<<<<<<<<
 *         self.val = aval_value(&aval, self.dtype)
 *         if self.val != self.prev_val or self.first_cycle:
 */
  (void)(((struct __pyx_vtabstruct_5pycx4_5pycda_Chan *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.get_data(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self), 0, __pyx_v_self->__pyx_base.itemsize, ((void *)(&__pyx_v_aval))));

  /* "pycx4/chan.pxi":13
 *         self.prev_val = self.val
 *         self.get_data(0, self.itemsize, <void*>&aval)
 *         self.val = aval_value(&aval, self.dtype)             # <<<<<<<<<<<<<<
 *         if self.val != self.prev_val or self.first_cycle:
 *             self.valueChanged.emit(self)
 */
  __pyx_t_1 = __pyx_f_5pycx4_5pycda_aval_value((&__pyx_v_aval), __pyx_v_self->__pyx_base.dtype); if (unlikely(!__pyx_t_1)) __PYX_ERR(12, 13, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->val);
  __Pyx_DECREF(__pyx_v_self->val);
  __pyx_v_self->val = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pycx4/chan.pxi":14
 *         self.get_data(0, self.itemsize, <void*>&aval)
 *         self.val = aval_value(&aval, self.dtype)
 *         if self.val != self.prev_val or self.first_cycle:             # <<<<<<<<<<<<<<
 *             self.valueChanged.emit(self)
 *             self.first_cycle = False
 */
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_self->val, __pyx_v_self->prev_val, Py_NE); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(12, 14, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(12, 14, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (!__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = (__pyx_v_self->__pyx_base.first_cycle != 0);
  __pyx_t_2 = __pyx_t_3;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "pycx4/chan.pxi":15
 *         self.val = aval_value(&aval, self.dtype)
 *         if self.val != self.prev_val or self.first_cycle:
 *             self.valueChanged.emit(self)             # <<<<<<<<<<<<<<
 *             self.first_cycle = False
 *         self.valueMeasured.emit(self)
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->__pyx_base.valueChanged, __pyx_n_s_emit); if (unlikely(!__pyx_t_4)) __PYX_ERR(12, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    __pyx_t_1 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_5, ((PyObject *)__pyx_v_self)) : __Pyx_PyObject_CallOneArg(__pyx_t_4, ((PyObject *)__pyx_v_self));
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(12, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pycx4/chan.pxi":16
 *         if self.val != self.prev_val or self.first_cycle:
 *             self.valueChanged.emit(self)
 *             self.first_cycle = False             # <<<<<<<<<<<<<<
 *         self.valueMeasured.emit(self)
 * 
 */
    __pyx_v_self->__pyx_base.first_cycle = 0;

    /* "pycx4/chan.pxi":14
 *         self.get_data(0, self.itemsize, <void*>&aval)
 *         self.val = aval_value(&aval, self.dtype)
 *         if self.val != self.prev_val or self.first_cycle:             # <<<<<<<<<<<<<<
 *             self.valueChanged.emit(self)
 *             self.first_cycle = False
 */
  }

  /* "pycx4/chan.pxi":17
 *             self.valueChanged.emit(self)
 *             self.first_cycle = False
 *         self.valueMeasured.emit(self)             # <<<<<<<<<<<<<<
 * 
 *     cpdef void setValue(self, value):
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->__pyx_base.valueMeasured, __pyx_n_s_emit); if (unlikely(!__pyx_t_4)) __PYX_ERR(12, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_1 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_5, ((PyObject *)__pyx_v_self)) : __Pyx_PyObject_CallOneArg(__pyx_t_4, ((PyObject *)__pyx_v_self));
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(12, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/chan.pxi":9
 *         BaseChan.__init__(self, name, **kwargs)
 * 
 *     cdef void cb(self):             # <<<<<<<<<<<<<<
 *         cdef CxAnyVal_t aval
 *         self.prev_val = self.val
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_WriteUnraisable("pycx4.pycda.Chan.cb", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/chan.pxi":19
 *         self.valueMeasured.emit(self)
 * 
 *     cpdef void setValue(self, value):             # <<<<<<<<<<<<<<
 *         cdef CxAnyVal_t aval
 *         aval_setvalue(value, &aval, self.dtype)
 */

static PyObject *__pyx_pw_5pycx4_5pycda_4Chan_3setValue(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static void __pyx_f_5pycx4_5pycda_4Chan_setValue(struct __pyx_obj_5pycx4_5pycda_Chan *__pyx_v_self, PyObject *__pyx_v_value, int __pyx_skip_dispatch) {
  CxAnyVal_t __pyx_v_aval;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("setValue", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    static PY_UINT64_T tp_dict_version = 0, obj_dict_version = 0;
    if (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict && tp_dict_version == __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) && (!Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset || obj_dict_version == __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))))));
    else {
      PY_UINT64_T type_dict_guard = (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict)) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_setValue); if (unlikely(!__pyx_t_1)) __PYX_ERR(12, 19, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_5pycx4_5pycda_4Chan_3setValue)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_value) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_value);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(12, 19, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
      tp_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      obj_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset) ? __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))) : 0;
      if (unlikely(type_dict_guard != tp_dict_version)) {
        tp_dict_version = obj_dict_version = 0;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    }
    #endif
  }

  /* "pycx4/chan.pxi":21
 *     cpdef void setValue(self, value):
 *         cdef CxAnyVal_t aval
 *         aval_setvalue(value, &aval, self.dtype)             # <<<<<<<<<<<<<<
 *         self.snd_data(self.dtype, 1, <void*>&aval)
 */
  __pyx_f_5pycx4_5pycda_aval_setvalue(__pyx_v_value, (&__pyx_v_aval), __pyx_v_self->__pyx_base.dtype);

  /* "pycx4/chan.pxi":22
 *         cdef CxAnyVal_t aval
 *         aval_setvalue(value, &aval, self.dtype)
 *         self.snd_data(self.dtype, 1, <void*>&aval)             # <<<<<<<<<<<<<<
 */
  (void)(((struct __pyx_vtabstruct_5pycx4_5pycda_Chan *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.snd_data(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self), __pyx_v_self->__pyx_base.dtype, 1, ((void *)(&__pyx_v_aval))));

  /* "pycx4/chan.pxi":19
 *         self.valueMeasured.emit(self)
 * 
 *     cpdef void setValue(self, value):             # <<<<<<<<<<<<<<
 *         cdef CxAnyVal_t aval
 *         aval_setvalue(value, &aval, self.dtype)
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("pycx4.pycda.Chan.setValue", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_4Chan_3setValue(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_4Chan_3setValue(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("setValue (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_4Chan_2setValue(((struct __pyx_obj_5pycx4_5pycda_Chan *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_4Chan_2setValue(struct __pyx_obj_5pycx4_5pycda_Chan *__pyx_v_self, PyObject *__pyx_v_value) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("setValue", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_void_to_None(__pyx_f_5pycx4_5pycda_4Chan_setValue(__pyx_v_self, __pyx_v_value, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(12, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.Chan.setValue", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/chan.pxi":4
 * cdef class Chan(BaseChan):
 *     cdef:
 *         readonly object val, prev_val             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, str name, **kwargs):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_4Chan_3val_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_4Chan_3val_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_4Chan_3val___get__(((struct __pyx_obj_5pycx4_5pycda_Chan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_4Chan_3val___get__(struct __pyx_obj_5pycx4_5pycda_Chan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->val);
  __pyx_r = __pyx_v_self->val;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_4Chan_8prev_val_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_4Chan_8prev_val_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_4Chan_8prev_val___get__(((struct __pyx_obj_5pycx4_5pycda_Chan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_4Chan_8prev_val___get__(struct __pyx_obj_5pycx4_5pycda_Chan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->prev_val);
  __pyx_r = __pyx_v_self->prev_val;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_4Chan_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_4Chan_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_4Chan_4__reduce_cython__(((struct __pyx_obj_5pycx4_5pycda_Chan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_4Chan_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_Chan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__27, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.Chan.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_4Chan_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_4Chan_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_4Chan_6__setstate_cython__(((struct __pyx_obj_5pycx4_5pycda_Chan *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_4Chan_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_Chan *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__28, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.Chan.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/vchan.pxi":11
 *         int change_sign
 * 
 *     def __init__(self, str name, **kwargs):             # <<<<<<<<<<<<<<
 *         BaseChan.__init__(self, name, **kwargs)
 *         self.change_sign = kwargs.get('change_sign', False)
 */

/* Python wrapper */
static int __pyx_pw_5pycx4_5pycda_5VChan_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5pycx4_5pycda_5VChan_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_kwargs = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  __pyx_v_kwargs = PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return -1;
  __Pyx_GOTREF(__pyx_v_kwargs);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_name,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_name)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, __pyx_v_kwargs, values, pos_args, "__init__") < 0)) __PYX_ERR(13, 11, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_name = ((PyObject*)values[0]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(13, 11, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_DECREF(__pyx_v_kwargs); __pyx_v_kwargs = 0;
  __Pyx_AddTraceback("pycx4.pycda.VChan.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_name), (&PyUnicode_Type), 1, "name", 1))) __PYX_ERR(13, 11, __pyx_L1_error)
  __pyx_r = __pyx_pf_5pycx4_5pycda_5VChan___init__(((struct __pyx_obj_5pycx4_5pycda_VChan *)__pyx_v_self), __pyx_v_name, __pyx_v_kwargs);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pycx4_5pycda_5VChan___init__(struct __pyx_obj_5pycx4_5pycda_VChan *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_kwargs) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "pycx4/vchan.pxi":12
 * 
 *     def __init__(self, str name, **kwargs):
 *         BaseChan.__init__(self, name, **kwargs)             # <<<<<<<<<<<<<<
 *         self.change_sign = kwargs.get('change_sign', False)
 *         self.npdtype = cxdtype2np(self.dtype)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_5pycx4_5pycda_BaseChan), __pyx_n_s_init); if (unlikely(!__pyx_t_1)) __PYX_ERR(13, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(13, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_v_self));
  __Pyx_INCREF(__pyx_v_name);
  __Pyx_GIVEREF(__pyx_v_name);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_name);
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_2, __pyx_v_kwargs); if (unlikely(!__pyx_t_3)) __PYX_ERR(13, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "pycx4/vchan.pxi":13
 *     def __init__(self, str name, **kwargs):
 *         BaseChan.__init__(self, name, **kwargs)
 *         self.change_sign = kwargs.get('change_sign', False)             # <<<<<<<<<<<<<<
 *         self.npdtype = cxdtype2np(self.dtype)
 *         self.val = np.zeros(1, self.npdtype, order='C')
 */
  __pyx_t_3 = __Pyx_PyDict_GetItemDefault(__pyx_v_kwargs, __pyx_n_u_change_sign, Py_False); if (unlikely(!__pyx_t_3)) __PYX_ERR(13, 13, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_4 == (int)-1) && PyErr_Occurred())) __PYX_ERR(13, 13, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_self->change_sign = __pyx_t_4;

  /* "pycx4/vchan.pxi":14
 *         BaseChan.__init__(self, name, **kwargs)
 *         self.change_sign = kwargs.get('change_sign', False)
 *         self.npdtype = cxdtype2np(self.dtype)             # <<<<<<<<<<<<<<
 *         self.val = np.zeros(1, self.npdtype, order='C')
 *         self.nelems = 1
 */
  __pyx_t_3 = __pyx_f_5pycx4_5pycda_cxdtype2np(__pyx_v_self->__pyx_base.dtype); if (unlikely(!__pyx_t_3)) __PYX_ERR(13, 14, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->npdtype);
  __Pyx_DECREF(__pyx_v_self->npdtype);
  __pyx_v_self->npdtype = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pycx4/vchan.pxi":15
 *         self.change_sign = kwargs.get('change_sign', False)
 *         self.npdtype = cxdtype2np(self.dtype)
 *         self.val = np.zeros(1, self.npdtype, order='C')             # <<<<<<<<<<<<<<
 *         self.nelems = 1
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(13, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) __PYX_ERR(13, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(13, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_int_1);
  __Pyx_GIVEREF(__pyx_int_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_int_1);
  __Pyx_INCREF(__pyx_v_self->npdtype);
  __Pyx_GIVEREF(__pyx_v_self->npdtype);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v_self->npdtype);
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(13, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_order, __pyx_n_u_C) < 0) __PYX_ERR(13, 15, __pyx_L1_error)
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, __pyx_t_1); if (unlikely(!__pyx_t_5)) __PYX_ERR(13, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (!(likely(((__pyx_t_5) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_5, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(13, 15, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_5);
  __Pyx_GOTREF(__pyx_v_self->val);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->val));
  __pyx_v_self->val = ((PyArrayObject *)__pyx_t_5);
  __pyx_t_5 = 0;

  /* "pycx4/vchan.pxi":16
 *         self.npdtype = cxdtype2np(self.dtype)
 *         self.val = np.zeros(1, self.npdtype, order='C')
 *         self.nelems = 1             # <<<<<<<<<<<<<<
 * 
 *     cdef void cb(self):
 */
  __pyx_v_self->nelems = 1;

  /* "pycx4/vchan.pxi":11
 *         int change_sign
 * 
 *     def __init__(self, str name, **kwargs):             # <<<<<<<<<<<<<<
 *         BaseChan.__init__(self, name, **kwargs)
 *         self.change_sign = kwargs.get('change_sign', False)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pycx4.pycda.VChan.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/vchan.pxi":18
 *         self.nelems = 1
 * 
 *     cdef void cb(self):             # <<<<<<<<<<<<<<
 *         nelems = self.current_nelems()
 *         if self.nelems != nelems:
 */

static void __pyx_f_5pycx4_5pycda_5VChan_cb(struct __pyx_obj_5pycx4_5pycda_VChan *__pyx_v_self) {
  PyObject *__pyx_v_nelems = NULL;
  PyObject *__pyx_v_c_len = NULL;
  CYTHON_UNUSED PyObject *__pyx_v_nelems_read = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  __Pyx_RefNannySetupContext("cb", 0);

  /* "pycx4/vchan.pxi":19
 * 
 *     cdef void cb(self):
 *         nelems = self.current_nelems()             # <<<<<<<<<<<<<<
 *         if self.nelems != nelems:
 *             self.nelems = nelems
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(((struct __pyx_vtabstruct_5pycx4_5pycda_VChan *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.current_nelems(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self))); if (unlikely(!__pyx_t_1)) __PYX_ERR(13, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_nelems = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pycx4/vchan.pxi":20
 *     cdef void cb(self):
 *         nelems = self.current_nelems()
 *         if self.nelems != nelems:             # <<<<<<<<<<<<<<
 *             self.nelems = nelems
 *             self.val.resize(nelems)
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->nelems); if (unlikely(!__pyx_t_1)) __PYX_ERR(13, 20, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_RichCompare(__pyx_t_1, __pyx_v_nelems, Py_NE); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(13, 20, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(13, 20, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_3) {

    /* "pycx4/vchan.pxi":21
 *         nelems = self.current_nelems()
 *         if self.nelems != nelems:
 *             self.nelems = nelems             # <<<<<<<<<<<<<<
 *             self.val.resize(nelems)
 *         c_len = self.get_data(0, self.itemsize * nelems, <void*>self.val.data)
 */
    __pyx_t_4 = __Pyx_PyInt_As_int(__pyx_v_nelems); if (unlikely((__pyx_t_4 == (int)-1) && PyErr_Occurred())) __PYX_ERR(13, 21, __pyx_L1_error)
    __pyx_v_self->nelems = __pyx_t_4;

    /* "pycx4/vchan.pxi":22
 *         if self.nelems != nelems:
 *             self.nelems = nelems
 *             self.val.resize(nelems)             # <<<<<<<<<<<<<<
 *         c_len = self.get_data(0, self.itemsize * nelems, <void*>self.val.data)
 *         nelems_read = int(c_len/self.itemsize)
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->val), __pyx_n_s_resize); if (unlikely(!__pyx_t_1)) __PYX_ERR(13, 22, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_5, __pyx_v_nelems) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_v_nelems);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(13, 22, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pycx4/vchan.pxi":20
 *     cdef void cb(self):
 *         nelems = self.current_nelems()
 *         if self.nelems != nelems:             # <<<<<<<<<<<<<<
 *             self.nelems = nelems
 *             self.val.resize(nelems)
 */
  }

  /* "pycx4/vchan.pxi":23
 *             self.nelems = nelems
 *             self.val.resize(nelems)
 *         c_len = self.get_data(0, self.itemsize * nelems, <void*>self.val.data)             # <<<<<<<<<<<<<<
 *         nelems_read = int(c_len/self.itemsize)
 *         if self.change_sign:
 */
  __pyx_t_2 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.itemsize); if (unlikely(!__pyx_t_2)) __PYX_ERR(13, 23, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyNumber_Multiply(__pyx_t_2, __pyx_v_nelems); if (unlikely(!__pyx_t_1)) __PYX_ERR(13, 23, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_6 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_6 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(13, 23, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyInt_From_int(((struct __pyx_vtabstruct_5pycx4_5pycda_VChan *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.get_data(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self), 0, __pyx_t_6, ((void *)__pyx_v_self->val->data))); if (unlikely(!__pyx_t_1)) __PYX_ERR(13, 23, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_c_len = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pycx4/vchan.pxi":24
 *             self.val.resize(nelems)
 *         c_len = self.get_data(0, self.itemsize * nelems, <void*>self.val.data)
 *         nelems_read = int(c_len/self.itemsize)             # <<<<<<<<<<<<<<
 *         if self.change_sign:
 *             self.val *= -1
 */
  __pyx_t_1 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.itemsize); if (unlikely(!__pyx_t_1)) __PYX_ERR(13, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyNumber_Divide(__pyx_v_c_len, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(13, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyNumber_Int(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(13, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_nelems_read = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pycx4/vchan.pxi":25
 *         c_len = self.get_data(0, self.itemsize * nelems, <void*>self.val.data)
 *         nelems_read = int(c_len/self.itemsize)
 *         if self.change_sign:             # <<<<<<<<<<<<<<
 *             self.val *= -1
 * 
 */
  __pyx_t_3 = (__pyx_v_self->change_sign != 0);
  if (__pyx_t_3) {

    /* "pycx4/vchan.pxi":26
 *         nelems_read = int(c_len/self.itemsize)
 *         if self.change_sign:
 *             self.val *= -1             # <<<<<<<<<<<<<<
 * 
 *         self.valueMeasured.emit(self)
 */
    __pyx_t_1 = PyNumber_InPlaceMultiply(((PyObject *)__pyx_v_self->val), __pyx_int_neg_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(13, 26, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(13, 26, __pyx_L1_error)
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->val);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->val));
    __pyx_v_self->val = ((PyArrayObject *)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "pycx4/vchan.pxi":25
 *         c_len = self.get_data(0, self.itemsize * nelems, <void*>self.val.data)
 *         nelems_read = int(c_len/self.itemsize)
 *         if self.change_sign:             # <<<<<<<<<<<<<<
 *             self.val *= -1
 * 
 */
  }

  /* "pycx4/vchan.pxi":28
 *             self.val *= -1
 * 
 *         self.valueMeasured.emit(self)             # <<<<<<<<<<<<<<
 *         self.valueChanged.emit(self)
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->__pyx_base.valueMeasured, __pyx_n_s_emit); if (unlikely(!__pyx_t_2)) __PYX_ERR(13, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, ((PyObject *)__pyx_v_self)) : __Pyx_PyObject_CallOneArg(__pyx_t_2, ((PyObject *)__pyx_v_self));
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(13, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/vchan.pxi":29
 * 
 *         self.valueMeasured.emit(self)
 *         self.valueChanged.emit(self)             # <<<<<<<<<<<<<<
 * 
 *     cpdef void setValue(self, np.ndarray value):
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->__pyx_base.valueChanged, __pyx_n_s_emit); if (unlikely(!__pyx_t_2)) __PYX_ERR(13, 29, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, ((PyObject *)__pyx_v_self)) : __Pyx_PyObject_CallOneArg(__pyx_t_2, ((PyObject *)__pyx_v_self));
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(13, 29, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/vchan.pxi":18
 *         self.nelems = 1
 * 
 *     cdef void cb(self):             # <<<<<<<<<<<<<<
 *         nelems = self.current_nelems()
 *         if self.nelems != nelems:
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_WriteUnraisable("pycx4.pycda.VChan.cb", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_nelems);
  __Pyx_XDECREF(__pyx_v_c_len);
  __Pyx_XDECREF(__pyx_v_nelems_read);
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/vchan.pxi":31
 *         self.valueChanged.emit(self)
 * 
 *     cpdef void setValue(self, np.ndarray value):             # <<<<<<<<<<<<<<
 *         if value.size > self.max_nelems:
 *             raise Exception('value size greater than channel.max_nelems')
 */

static PyObject *__pyx_pw_5pycx4_5pycda_5VChan_3setValue(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static void __pyx_f_5pycx4_5pycda_5VChan_setValue(struct __pyx_obj_5pycx4_5pycda_VChan *__pyx_v_self, PyArrayObject *__pyx_v_value, int __pyx_skip_dispatch) {
  cxdtype_t __pyx_v_dtype;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  __Pyx_RefNannySetupContext("setValue", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    static PY_UINT64_T tp_dict_version = 0, obj_dict_version = 0;
    if (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict && tp_dict_version == __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) && (!Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset || obj_dict_version == __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))))));
    else {
      PY_UINT64_T type_dict_guard = (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict)) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_setValue); if (unlikely(!__pyx_t_1)) __PYX_ERR(13, 31, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_5pycx4_5pycda_5VChan_3setValue)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, ((PyObject *)__pyx_v_value)) : __Pyx_PyObject_CallOneArg(__pyx_t_3, ((PyObject *)__pyx_v_value));
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(13, 31, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
      tp_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      obj_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset) ? __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))) : 0;
      if (unlikely(type_dict_guard != tp_dict_version)) {
        tp_dict_version = obj_dict_version = 0;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    }
    #endif
  }

  /* "pycx4/vchan.pxi":32
 * 
 *     cpdef void setValue(self, np.ndarray value):
 *         if value.size > self.max_nelems:             # <<<<<<<<<<<<<<
 *             raise Exception('value size greater than channel.max_nelems')
 *         dtype = np2cxdtype(value.dtype)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_value), __pyx_n_s_size); if (unlikely(!__pyx_t_1)) __PYX_ERR(13, 32, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->__pyx_base.max_nelems); if (unlikely(!__pyx_t_2)) __PYX_ERR(13, 32, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyObject_RichCompare(__pyx_t_1, __pyx_t_2, Py_GT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(13, 32, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(13, 32, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(__pyx_t_5)) {

    /* "pycx4/vchan.pxi":33
 *     cpdef void setValue(self, np.ndarray value):
 *         if value.size > self.max_nelems:
 *             raise Exception('value size greater than channel.max_nelems')             # <<<<<<<<<<<<<<
 *         dtype = np2cxdtype(value.dtype)
 *         self.snd_data(dtype, value.size, <void*>value.data)
 */
    __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])), __pyx_tuple__29, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(13, 33, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(13, 33, __pyx_L1_error)

    /* "pycx4/vchan.pxi":32
 * 
 *     cpdef void setValue(self, np.ndarray value):
 *         if value.size > self.max_nelems:             # <<<<<<<<<<<<<<
 *             raise Exception('value size greater than channel.max_nelems')
 *         dtype = np2cxdtype(value.dtype)
 */
  }

  /* "pycx4/vchan.pxi":34
 *         if value.size > self.max_nelems:
 *             raise Exception('value size greater than channel.max_nelems')
 *         dtype = np2cxdtype(value.dtype)             # <<<<<<<<<<<<<<
 *         self.snd_data(dtype, value.size, <void*>value.data)
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_value), __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) __PYX_ERR(13, 34, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v_dtype = __pyx_f_5pycx4_5pycda_np2cxdtype(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "pycx4/vchan.pxi":35
 *             raise Exception('value size greater than channel.max_nelems')
 *         dtype = np2cxdtype(value.dtype)
 *         self.snd_data(dtype, value.size, <void*>value.data)             # <<<<<<<<<<<<<<
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_value), __pyx_n_s_size); if (unlikely(!__pyx_t_3)) __PYX_ERR(13, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_6 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_6 == (int)-1) && PyErr_Occurred())) __PYX_ERR(13, 35, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  (void)(((struct __pyx_vtabstruct_5pycx4_5pycda_VChan *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.snd_data(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self), __pyx_v_dtype, __pyx_t_6, ((void *)__pyx_v_value->data)));

  /* "pycx4/vchan.pxi":31
 *         self.valueChanged.emit(self)
 * 
 *     cpdef void setValue(self, np.ndarray value):             # <<<<<<<<<<<<<<
 *         if value.size > self.max_nelems:
 *             raise Exception('value size greater than channel.max_nelems')
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("pycx4.pycda.VChan.setValue", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_5VChan_3setValue(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_5VChan_3setValue(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("setValue (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_value), __pyx_ptype_5numpy_ndarray, 1, "value", 0))) __PYX_ERR(13, 31, __pyx_L1_error)
  __pyx_r = __pyx_pf_5pycx4_5pycda_5VChan_2setValue(((struct __pyx_obj_5pycx4_5pycda_VChan *)__pyx_v_self), ((PyArrayObject *)__pyx_v_value));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_5VChan_2setValue(struct __pyx_obj_5pycx4_5pycda_VChan *__pyx_v_self, PyArrayObject *__pyx_v_value) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("setValue", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_void_to_None(__pyx_f_5pycx4_5pycda_5VChan_setValue(__pyx_v_self, __pyx_v_value, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(13, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.VChan.setValue", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/vchan.pxi":6
 * cdef class VChan(BaseChan):
 *     cdef:
 *         readonly np.ndarray val             # <<<<<<<<<<<<<<
 *         readonly object npdtype
 *         readonly int nelems
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_5VChan_3val_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_5VChan_3val_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_5VChan_3val___get__(((struct __pyx_obj_5pycx4_5pycda_VChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_5VChan_3val___get__(struct __pyx_obj_5pycx4_5pycda_VChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->val));
  __pyx_r = ((PyObject *)__pyx_v_self->val);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/vchan.pxi":7
 *     cdef:
 *         readonly np.ndarray val
 *         readonly object npdtype             # <<<<<<<<<<<<<<
 *         readonly int nelems
 *         int change_sign
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_5VChan_7npdtype_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_5VChan_7npdtype_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_5VChan_7npdtype___get__(((struct __pyx_obj_5pycx4_5pycda_VChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_5VChan_7npdtype___get__(struct __pyx_obj_5pycx4_5pycda_VChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->npdtype);
  __pyx_r = __pyx_v_self->npdtype;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/vchan.pxi":8
 *         readonly np.ndarray val
 *         readonly object npdtype
 *         readonly int nelems             # <<<<<<<<<<<<<<
 *         int change_sign
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_5VChan_6nelems_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_5VChan_6nelems_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_5VChan_6nelems___get__(((struct __pyx_obj_5pycx4_5pycda_VChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_5VChan_6nelems___get__(struct __pyx_obj_5pycx4_5pycda_VChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->nelems); if (unlikely(!__pyx_t_1)) __PYX_ERR(13, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.VChan.nelems.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_5VChan_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_5VChan_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_5VChan_4__reduce_cython__(((struct __pyx_obj_5pycx4_5pycda_VChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_5VChan_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_VChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__30, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.VChan.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_5VChan_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_5VChan_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_5VChan_6__setstate_cython__(((struct __pyx_obj_5pycx4_5pycda_VChan *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_5VChan_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_VChan *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__31, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.VChan.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/strchan.pxi":7
 *         char *cval
 * 
 *     def __init__(self, str name, **kwargs):             # <<<<<<<<<<<<<<
 *         kwargs['dtype'] = CXDTYPE_TEXT
 *         BaseChan.__init__(self, name, **kwargs)
 */

/* Python wrapper */
static int __pyx_pw_5pycx4_5pycda_7StrChan_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5pycx4_5pycda_7StrChan_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_kwargs = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  __pyx_v_kwargs = PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return -1;
  __Pyx_GOTREF(__pyx_v_kwargs);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_name,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_name)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, __pyx_v_kwargs, values, pos_args, "__init__") < 0)) __PYX_ERR(14, 7, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_name = ((PyObject*)values[0]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(14, 7, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_DECREF(__pyx_v_kwargs); __pyx_v_kwargs = 0;
  __Pyx_AddTraceback("pycx4.pycda.StrChan.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_name), (&PyUnicode_Type), 1, "name", 1))) __PYX_ERR(14, 7, __pyx_L1_error)
  __pyx_r = __pyx_pf_5pycx4_5pycda_7StrChan___init__(((struct __pyx_obj_5pycx4_5pycda_StrChan *)__pyx_v_self), __pyx_v_name, __pyx_v_kwargs);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pycx4_5pycda_7StrChan___init__(struct __pyx_obj_5pycx4_5pycda_StrChan *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_kwargs) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "pycx4/strchan.pxi":8
 * 
 *     def __init__(self, str name, **kwargs):
 *         kwargs['dtype'] = CXDTYPE_TEXT             # <<<<<<<<<<<<<<
 *         BaseChan.__init__(self, name, **kwargs)
 *         self.cval = <char*>malloc(self.max_nelems)
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(CXDTYPE_TEXT); if (unlikely(!__pyx_t_1)) __PYX_ERR(14, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (unlikely(PyDict_SetItem(__pyx_v_kwargs, __pyx_n_u_dtype, __pyx_t_1) < 0)) __PYX_ERR(14, 8, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/strchan.pxi":9
 *     def __init__(self, str name, **kwargs):
 *         kwargs['dtype'] = CXDTYPE_TEXT
 *         BaseChan.__init__(self, name, **kwargs)             # <<<<<<<<<<<<<<
 *         self.cval = <char*>malloc(self.max_nelems)
 *         if not self.cval: raise MemoryError()
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_5pycx4_5pycda_BaseChan), __pyx_n_s_init); if (unlikely(!__pyx_t_1)) __PYX_ERR(14, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(14, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_v_self));
  __Pyx_INCREF(__pyx_v_name);
  __Pyx_GIVEREF(__pyx_v_name);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_name);
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_2, __pyx_v_kwargs); if (unlikely(!__pyx_t_3)) __PYX_ERR(14, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "pycx4/strchan.pxi":10
 *         kwargs['dtype'] = CXDTYPE_TEXT
 *         BaseChan.__init__(self, name, **kwargs)
 *         self.cval = <char*>malloc(self.max_nelems)             # <<<<<<<<<<<<<<
 *         if not self.cval: raise MemoryError()
 *         self.val,self.prev_val = '', ''
 */
  __pyx_v_self->cval = ((char *)malloc(__pyx_v_self->__pyx_base.max_nelems));

  /* "pycx4/strchan.pxi":11
 *         BaseChan.__init__(self, name, **kwargs)
 *         self.cval = <char*>malloc(self.max_nelems)
 *         if not self.cval: raise MemoryError()             # <<<<<<<<<<<<<<
 *         self.val,self.prev_val = '', ''
 * 
 */
  __pyx_t_4 = ((!(__pyx_v_self->cval != 0)) != 0);
  if (unlikely(__pyx_t_4)) {
    PyErr_NoMemory(); __PYX_ERR(14, 11, __pyx_L1_error)
  }

  /* "pycx4/strchan.pxi":12
 *         self.cval = <char*>malloc(self.max_nelems)
 *         if not self.cval: raise MemoryError()
 *         self.val,self.prev_val = '', ''             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
 */
  __pyx_t_3 = __pyx_kp_u__32;
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_2 = __pyx_kp_u__32;
  __Pyx_INCREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->val);
  __Pyx_DECREF(__pyx_v_self->val);
  __pyx_v_self->val = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->prev_val);
  __Pyx_DECREF(__pyx_v_self->prev_val);
  __pyx_v_self->prev_val = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pycx4/strchan.pxi":7
 *         char *cval
 * 
 *     def __init__(self, str name, **kwargs):             # <<<<<<<<<<<<<<
 *         kwargs['dtype'] = CXDTYPE_TEXT
 *         BaseChan.__init__(self, name, **kwargs)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("pycx4.pycda.StrChan.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/strchan.pxi":14
 *         self.val,self.prev_val = '', ''
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if not self.cval:
 *             free(self.cval)
 */

/* Python wrapper */
static void __pyx_pw_5pycx4_5pycda_7StrChan_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_5pycx4_5pycda_7StrChan_3__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_5pycx4_5pycda_7StrChan_2__dealloc__(((struct __pyx_obj_5pycx4_5pycda_StrChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_5pycx4_5pycda_7StrChan_2__dealloc__(struct __pyx_obj_5pycx4_5pycda_StrChan *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "pycx4/strchan.pxi":15
 * 
 *     def __dealloc__(self):
 *         if not self.cval:             # <<<<<<<<<<<<<<
 *             free(self.cval)
 *             self.cval = NULL
 */
  __pyx_t_1 = ((!(__pyx_v_self->cval != 0)) != 0);
  if (__pyx_t_1) {

    /* "pycx4/strchan.pxi":16
 *     def __dealloc__(self):
 *         if not self.cval:
 *             free(self.cval)             # <<<<<<<<<<<<<<
 *             self.cval = NULL
 * 
 */
    free(__pyx_v_self->cval);

    /* "pycx4/strchan.pxi":17
 *         if not self.cval:
 *             free(self.cval)
 *             self.cval = NULL             # <<<<<<<<<<<<<<
 * 
 *     cdef void cb(self):
 */
    __pyx_v_self->cval = NULL;

    /* "pycx4/strchan.pxi":15
 * 
 *     def __dealloc__(self):
 *         if not self.cval:             # <<<<<<<<<<<<<<
 *             free(self.cval)
 *             self.cval = NULL
 */
  }

  /* "pycx4/strchan.pxi":14
 *         self.val,self.prev_val = '', ''
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if not self.cval:
 *             free(self.cval)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/strchan.pxi":19
 *             self.cval = NULL
 * 
 *     cdef void cb(self):             # <<<<<<<<<<<<<<
 *         self.prev_val = self.val
 *         c_len = self.get_data(0, self.max_nelems, <void*>self.cval)
 */

static void __pyx_f_5pycx4_5pycda_7StrChan_cb(struct __pyx_obj_5pycx4_5pycda_StrChan *__pyx_v_self) {
  int __pyx_v_c_len;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  __Pyx_RefNannySetupContext("cb", 0);

  /* "pycx4/strchan.pxi":20
 * 
 *     cdef void cb(self):
 *         self.prev_val = self.val             # <<<<<<<<<<<<<<
 *         c_len = self.get_data(0, self.max_nelems, <void*>self.cval)
 *         if PY_MAJOR_VERSION > 2:
 */
  __pyx_t_1 = __pyx_v_self->val;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->prev_val);
  __Pyx_DECREF(__pyx_v_self->prev_val);
  __pyx_v_self->prev_val = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pycx4/strchan.pxi":21
 *     cdef void cb(self):
 *         self.prev_val = self.val
 *         c_len = self.get_data(0, self.max_nelems, <void*>self.cval)             # <<<<<<<<<<<<<<
 *         if PY_MAJOR_VERSION > 2:
 *             self.val = (<bytes>self.cval[:c_len]).decode('UTF-8')
 */
  __pyx_v_c_len = ((struct __pyx_vtabstruct_5pycx4_5pycda_StrChan *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.get_data(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self), 0, __pyx_v_self->__pyx_base.max_nelems, ((void *)__pyx_v_self->cval));

  /* "pycx4/strchan.pxi":22
 *         self.prev_val = self.val
 *         c_len = self.get_data(0, self.max_nelems, <void*>self.cval)
 *         if PY_MAJOR_VERSION > 2:             # <<<<<<<<<<<<<<
 *             self.val = (<bytes>self.cval[:c_len]).decode('UTF-8')
 *         else:
 */
  __pyx_t_2 = ((PY_MAJOR_VERSION > 2) != 0);
  if (__pyx_t_2) {

    /* "pycx4/strchan.pxi":23
 *         c_len = self.get_data(0, self.max_nelems, <void*>self.cval)
 *         if PY_MAJOR_VERSION > 2:
 *             self.val = (<bytes>self.cval[:c_len]).decode('UTF-8')             # <<<<<<<<<<<<<<
 *         else:
 *             self.val = self.cval[:c_len]
 */
    __pyx_t_1 = __Pyx_PyBytes_FromStringAndSize(__pyx_v_self->cval + 0, __pyx_v_c_len - 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(14, 23, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (unlikely(__pyx_t_1 == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "decode");
      __PYX_ERR(14, 23, __pyx_L1_error)
    }
    __pyx_t_3 = __Pyx_decode_bytes(((PyObject*)__pyx_t_1), 0, PY_SSIZE_T_MAX, NULL, NULL, PyUnicode_DecodeUTF8); if (unlikely(!__pyx_t_3)) __PYX_ERR(14, 23, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_v_self->val);
    __Pyx_DECREF(__pyx_v_self->val);
    __pyx_v_self->val = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "pycx4/strchan.pxi":22
 *         self.prev_val = self.val
 *         c_len = self.get_data(0, self.max_nelems, <void*>self.cval)
 *         if PY_MAJOR_VERSION > 2:             # <<<<<<<<<<<<<<
 *             self.val = (<bytes>self.cval[:c_len]).decode('UTF-8')
 *         else:
 */
    goto __pyx_L3;
  }

  /* "pycx4/strchan.pxi":25
 *             self.val = (<bytes>self.cval[:c_len]).decode('UTF-8')
 *         else:
 *             self.val = self.cval[:c_len]             # <<<<<<<<<<<<<<
 *         self.valueMeasured.emit(self)
 *         if self.val != self.prev_val:
 */
  /*else*/ {
    __pyx_t_3 = __Pyx_PyUnicode_FromStringAndSize(__pyx_v_self->cval + 0, __pyx_v_c_len - 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(14, 25, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_v_self->val);
    __Pyx_DECREF(__pyx_v_self->val);
    __pyx_v_self->val = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;
  }
  __pyx_L3:;

  /* "pycx4/strchan.pxi":26
 *         else:
 *             self.val = self.cval[:c_len]
 *         self.valueMeasured.emit(self)             # <<<<<<<<<<<<<<
 *         if self.val != self.prev_val:
 *             self.valueChanged.emit(self)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->__pyx_base.valueMeasured, __pyx_n_s_emit); if (unlikely(!__pyx_t_1)) __PYX_ERR(14, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_4, ((PyObject *)__pyx_v_self)) : __Pyx_PyObject_CallOneArg(__pyx_t_1, ((PyObject *)__pyx_v_self));
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(14, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "pycx4/strchan.pxi":27
 *             self.val = self.cval[:c_len]
 *         self.valueMeasured.emit(self)
 *         if self.val != self.prev_val:             # <<<<<<<<<<<<<<
 *             self.valueChanged.emit(self)
 * 
 */
  __pyx_t_2 = (__Pyx_PyUnicode_Equals(__pyx_v_self->val, __pyx_v_self->prev_val, Py_NE)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(14, 27, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_2 != 0);
  if (__pyx_t_5) {

    /* "pycx4/strchan.pxi":28
 *         self.valueMeasured.emit(self)
 *         if self.val != self.prev_val:
 *             self.valueChanged.emit(self)             # <<<<<<<<<<<<<<
 * 
 *     cpdef void setValue(self, str value):
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->__pyx_base.valueChanged, __pyx_n_s_emit); if (unlikely(!__pyx_t_1)) __PYX_ERR(14, 28, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_4, ((PyObject *)__pyx_v_self)) : __Pyx_PyObject_CallOneArg(__pyx_t_1, ((PyObject *)__pyx_v_self));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(14, 28, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "pycx4/strchan.pxi":27
 *             self.val = self.cval[:c_len]
 *         self.valueMeasured.emit(self)
 *         if self.val != self.prev_val:             # <<<<<<<<<<<<<<
 *             self.valueChanged.emit(self)
 * 
 */
  }

  /* "pycx4/strchan.pxi":19
 *             self.cval = NULL
 * 
 *     cdef void cb(self):             # <<<<<<<<<<<<<<
 *         self.prev_val = self.val
 *         c_len = self.get_data(0, self.max_nelems, <void*>self.cval)
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("pycx4.pycda.StrChan.cb", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/strchan.pxi":30
 *             self.valueChanged.emit(self)
 * 
 *     cpdef void setValue(self, str value):             # <<<<<<<<<<<<<<
 *         if PY_MAJOR_VERSION > 2:
 *             bv = value.encode('UTF-8')
 */

static PyObject *__pyx_pw_5pycx4_5pycda_7StrChan_5setValue(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static void __pyx_f_5pycx4_5pycda_7StrChan_setValue(struct __pyx_obj_5pycx4_5pycda_StrChan *__pyx_v_self, PyObject *__pyx_v_value, int __pyx_skip_dispatch) {
  PyObject *__pyx_v_bv = NULL;
  char *__pyx_v_v;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  char *__pyx_t_6;
  Py_ssize_t __pyx_t_7;
  __Pyx_RefNannySetupContext("setValue", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    static PY_UINT64_T tp_dict_version = 0, obj_dict_version = 0;
    if (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict && tp_dict_version == __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) && (!Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset || obj_dict_version == __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))))));
    else {
      PY_UINT64_T type_dict_guard = (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict)) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_setValue); if (unlikely(!__pyx_t_1)) __PYX_ERR(14, 30, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_5pycx4_5pycda_7StrChan_5setValue)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_value) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_value);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(14, 30, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
      tp_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      obj_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset) ? __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))) : 0;
      if (unlikely(type_dict_guard != tp_dict_version)) {
        tp_dict_version = obj_dict_version = 0;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    }
    #endif
  }

  /* "pycx4/strchan.pxi":31
 * 
 *     cpdef void setValue(self, str value):
 *         if PY_MAJOR_VERSION > 2:             # <<<<<<<<<<<<<<
 *             bv = value.encode('UTF-8')
 *         else:
 */
  __pyx_t_5 = ((PY_MAJOR_VERSION > 2) != 0);
  if (__pyx_t_5) {

    /* "pycx4/strchan.pxi":32
 *     cpdef void setValue(self, str value):
 *         if PY_MAJOR_VERSION > 2:
 *             bv = value.encode('UTF-8')             # <<<<<<<<<<<<<<
 *         else:
 *             bv = unicode(value).encode('UTF-8')
 */
    if (unlikely(__pyx_v_value == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "encode");
      __PYX_ERR(14, 32, __pyx_L1_error)
    }
    __pyx_t_1 = PyUnicode_AsUTF8String(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(14, 32, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_bv = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "pycx4/strchan.pxi":31
 * 
 *     cpdef void setValue(self, str value):
 *         if PY_MAJOR_VERSION > 2:             # <<<<<<<<<<<<<<
 *             bv = value.encode('UTF-8')
 *         else:
 */
    goto __pyx_L3;
  }

  /* "pycx4/strchan.pxi":34
 *             bv = value.encode('UTF-8')
 *         else:
 *             bv = unicode(value).encode('UTF-8')             # <<<<<<<<<<<<<<
 * 
 *         cdef char *v = bv
 */
  /*else*/ {
    __pyx_t_1 = __Pyx_PyUnicode_Unicode(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(14, 34, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PyUnicode_AsUTF8String(((PyObject*)__pyx_t_1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(14, 34, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_bv = __pyx_t_2;
    __pyx_t_2 = 0;
  }
  __pyx_L3:;

  /* "pycx4/strchan.pxi":36
 *             bv = unicode(value).encode('UTF-8')
 * 
 *         cdef char *v = bv             # <<<<<<<<<<<<<<
 *         self.snd_data(CXDTYPE_TEXT, len(bv), <void*>v)
 * 
 */
  __pyx_t_6 = __Pyx_PyObject_AsWritableString(__pyx_v_bv); if (unlikely((!__pyx_t_6) && PyErr_Occurred())) __PYX_ERR(14, 36, __pyx_L1_error)
  __pyx_v_v = __pyx_t_6;

  /* "pycx4/strchan.pxi":37
 * 
 *         cdef char *v = bv
 *         self.snd_data(CXDTYPE_TEXT, len(bv), <void*>v)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_7 = PyObject_Length(__pyx_v_bv); if (unlikely(__pyx_t_7 == ((Py_ssize_t)-1))) __PYX_ERR(14, 37, __pyx_L1_error)
  (void)(((struct __pyx_vtabstruct_5pycx4_5pycda_StrChan *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.snd_data(((struct __pyx_obj_5pycx4_5pycda_BaseChan *)__pyx_v_self), CXDTYPE_TEXT, __pyx_t_7, ((void *)__pyx_v_v)));

  /* "pycx4/strchan.pxi":30
 *             self.valueChanged.emit(self)
 * 
 *     cpdef void setValue(self, str value):             # <<<<<<<<<<<<<<
 *         if PY_MAJOR_VERSION > 2:
 *             bv = value.encode('UTF-8')
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("pycx4.pycda.StrChan.setValue", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_bv);
  __Pyx_RefNannyFinishContext();
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_7StrChan_5setValue(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_7StrChan_5setValue(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("setValue (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_value), (&PyUnicode_Type), 1, "value", 1))) __PYX_ERR(14, 30, __pyx_L1_error)
  __pyx_r = __pyx_pf_5pycx4_5pycda_7StrChan_4setValue(((struct __pyx_obj_5pycx4_5pycda_StrChan *)__pyx_v_self), ((PyObject*)__pyx_v_value));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_7StrChan_4setValue(struct __pyx_obj_5pycx4_5pycda_StrChan *__pyx_v_self, PyObject *__pyx_v_value) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("setValue", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_void_to_None(__pyx_f_5pycx4_5pycda_7StrChan_setValue(__pyx_v_self, __pyx_v_value, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(14, 30, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.StrChan.setValue", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/strchan.pxi":4
 * cdef class StrChan(BaseChan):
 *     cdef:
 *         readonly str val, prev_val             # <<<<<<<<<<<<<<
 *         char *cval
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_7StrChan_3val_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_7StrChan_3val_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_7StrChan_3val___get__(((struct __pyx_obj_5pycx4_5pycda_StrChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_7StrChan_3val___get__(struct __pyx_obj_5pycx4_5pycda_StrChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->val);
  __pyx_r = __pyx_v_self->val;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_7StrChan_8prev_val_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_7StrChan_8prev_val_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_7StrChan_8prev_val___get__(((struct __pyx_obj_5pycx4_5pycda_StrChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_7StrChan_8prev_val___get__(struct __pyx_obj_5pycx4_5pycda_StrChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->prev_val);
  __pyx_r = __pyx_v_self->prev_val;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_7StrChan_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_7StrChan_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_7StrChan_6__reduce_cython__(((struct __pyx_obj_5pycx4_5pycda_StrChan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_7StrChan_6__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_StrChan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__33, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.StrChan.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_7StrChan_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_7StrChan_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_7StrChan_8__setstate_cython__(((struct __pyx_obj_5pycx4_5pycda_StrChan *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_7StrChan_8__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_StrChan *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__34, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.StrChan.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/chan_factory.pxi":6
 *         dict single_chans, vector_chans
 * 
 *     def __init__(self):             # <<<<<<<<<<<<<<
 *         self.single_chans = {
 *             'double': DChan,
 */

/* Python wrapper */
static int __pyx_pw_5pycx4_5pycda_11ChanFactory_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5pycx4_5pycda_11ChanFactory_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("__init__", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return -1;}
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__init__", 0))) return -1;
  __pyx_r = __pyx_pf_5pycx4_5pycda_11ChanFactory___init__(((struct __pyx_obj_5pycx4_5pycda_ChanFactory *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pycx4_5pycda_11ChanFactory___init__(struct __pyx_obj_5pycx4_5pycda_ChanFactory *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "pycx4/chan_factory.pxi":8
 *     def __init__(self):
 *         self.single_chans = {
 *             'double': DChan,             # <<<<<<<<<<<<<<
 *             'int': IChan,
 *             'int32': IChan
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(15, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_double, ((PyObject *)__pyx_ptype_5pycx4_5pycda_DChan)) < 0) __PYX_ERR(15, 8, __pyx_L1_error)

  /* "pycx4/chan_factory.pxi":9
 *         self.single_chans = {
 *             'double': DChan,
 *             'int': IChan,             # <<<<<<<<<<<<<<
 *             'int32': IChan
 *         }
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_int, ((PyObject *)__pyx_ptype_5pycx4_5pycda_IChan)) < 0) __PYX_ERR(15, 8, __pyx_L1_error)

  /* "pycx4/chan_factory.pxi":10
 *             'double': DChan,
 *             'int': IChan,
 *             'int32': IChan             # <<<<<<<<<<<<<<
 *         }
 *         self.vector_chans = {
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_int32, ((PyObject *)__pyx_ptype_5pycx4_5pycda_IChan)) < 0) __PYX_ERR(15, 8, __pyx_L1_error)

  /* "pycx4/chan_factory.pxi":7
 * 
 *     def __init__(self):
 *         self.single_chans = {             # <<<<<<<<<<<<<<
 *             'double': DChan,
 *             'int': IChan,
 */
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->single_chans);
  __Pyx_DECREF(__pyx_v_self->single_chans);
  __pyx_v_self->single_chans = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pycx4/chan_factory.pxi":13
 *         }
 *         self.vector_chans = {
 *             'str': StrChan,             # <<<<<<<<<<<<<<
 *         }
 * 
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(15, 13, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_str, ((PyObject *)__pyx_ptype_5pycx4_5pycda_StrChan)) < 0) __PYX_ERR(15, 13, __pyx_L1_error)

  /* "pycx4/chan_factory.pxi":12
 *             'int32': IChan
 *         }
 *         self.vector_chans = {             # <<<<<<<<<<<<<<
 *             'str': StrChan,
 *         }
 */
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->vector_chans);
  __Pyx_DECREF(__pyx_v_self->vector_chans);
  __pyx_v_self->vector_chans = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pycx4/chan_factory.pxi":6
 *         dict single_chans, vector_chans
 * 
 *     def __init__(self):             # <<<<<<<<<<<<<<
 *         self.single_chans = {
 *             'double': DChan,
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.ChanFactory.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/chan_factory.pxi":16
 *         }
 * 
 *     def create_chan(self, name, dtype, dsize, **kwargs):             # <<<<<<<<<<<<<<
 *         if dtype not in cx_dtype_map:
 *             return None
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_11ChanFactory_3create_chan(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_11ChanFactory_3create_chan(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_dtype = 0;
  PyObject *__pyx_v_dsize = 0;
  PyObject *__pyx_v_kwargs = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("create_chan (wrapper)", 0);
  __pyx_v_kwargs = PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return NULL;
  __Pyx_GOTREF(__pyx_v_kwargs);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_name,&__pyx_n_s_dtype,&__pyx_n_s_dsize,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_name)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_dtype)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("create_chan", 1, 3, 3, 1); __PYX_ERR(15, 16, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_dsize)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("create_chan", 1, 3, 3, 2); __PYX_ERR(15, 16, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, __pyx_v_kwargs, values, pos_args, "create_chan") < 0)) __PYX_ERR(15, 16, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_name = values[0];
    __pyx_v_dtype = values[1];
    __pyx_v_dsize = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("create_chan", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(15, 16, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_DECREF(__pyx_v_kwargs); __pyx_v_kwargs = 0;
  __Pyx_AddTraceback("pycx4.pycda.ChanFactory.create_chan", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pycx4_5pycda_11ChanFactory_2create_chan(((struct __pyx_obj_5pycx4_5pycda_ChanFactory *)__pyx_v_self), __pyx_v_name, __pyx_v_dtype, __pyx_v_dsize, __pyx_v_kwargs);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_11ChanFactory_2create_chan(struct __pyx_obj_5pycx4_5pycda_ChanFactory *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_dtype, PyObject *__pyx_v_dsize, PyObject *__pyx_v_kwargs) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  __Pyx_RefNannySetupContext("create_chan", 0);

  /* "pycx4/chan_factory.pxi":17
 * 
 *     def create_chan(self, name, dtype, dsize, **kwargs):
 *         if dtype not in cx_dtype_map:             # <<<<<<<<<<<<<<
 *             return None
 *         if dsize == 1:
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_cx_dtype_map); if (unlikely(!__pyx_t_1)) __PYX_ERR(15, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = (__Pyx_PySequence_ContainsTF(__pyx_v_dtype, __pyx_t_1, Py_NE)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(15, 17, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "pycx4/chan_factory.pxi":18
 *     def create_chan(self, name, dtype, dsize, **kwargs):
 *         if dtype not in cx_dtype_map:
 *             return None             # <<<<<<<<<<<<<<
 *         if dsize == 1:
 *             if dtype in self.single_chans:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "pycx4/chan_factory.pxi":17
 * 
 *     def create_chan(self, name, dtype, dsize, **kwargs):
 *         if dtype not in cx_dtype_map:             # <<<<<<<<<<<<<<
 *             return None
 *         if dsize == 1:
 */
  }

  /* "pycx4/chan_factory.pxi":19
 *         if dtype not in cx_dtype_map:
 *             return None
 *         if dsize == 1:             # <<<<<<<<<<<<<<
 *             if dtype in self.single_chans:
 *                 return self.single_chans[dtype](name, **kwargs)
 */
  __pyx_t_1 = __Pyx_PyInt_EqObjC(__pyx_v_dsize, __pyx_int_1, 1, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(15, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(15, 19, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_3) {

    /* "pycx4/chan_factory.pxi":20
 *             return None
 *         if dsize == 1:
 *             if dtype in self.single_chans:             # <<<<<<<<<<<<<<
 *                 return self.single_chans[dtype](name, **kwargs)
 *             else:
 */
    if (unlikely(__pyx_v_self->single_chans == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
      __PYX_ERR(15, 20, __pyx_L1_error)
    }
    __pyx_t_3 = (__Pyx_PyDict_ContainsTF(__pyx_v_dtype, __pyx_v_self->single_chans, Py_EQ)); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(15, 20, __pyx_L1_error)
    __pyx_t_2 = (__pyx_t_3 != 0);
    if (__pyx_t_2) {

      /* "pycx4/chan_factory.pxi":21
 *         if dsize == 1:
 *             if dtype in self.single_chans:
 *                 return self.single_chans[dtype](name, **kwargs)             # <<<<<<<<<<<<<<
 *             else:
 *                 return Chan(name, cx_dtype_map[dtype])
 */
      __Pyx_XDECREF(__pyx_r);
      if (unlikely(__pyx_v_self->single_chans == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(15, 21, __pyx_L1_error)
      }
      __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_self->single_chans, __pyx_v_dtype); if (unlikely(!__pyx_t_1)) __PYX_ERR(15, 21, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(15, 21, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_INCREF(__pyx_v_name);
      __Pyx_GIVEREF(__pyx_v_name);
      PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_name);
      __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_4, __pyx_v_kwargs); if (unlikely(!__pyx_t_5)) __PYX_ERR(15, 21, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_r = __pyx_t_5;
      __pyx_t_5 = 0;
      goto __pyx_L0;

      /* "pycx4/chan_factory.pxi":20
 *             return None
 *         if dsize == 1:
 *             if dtype in self.single_chans:             # <<<<<<<<<<<<<<
 *                 return self.single_chans[dtype](name, **kwargs)
 *             else:
 */
    }

    /* "pycx4/chan_factory.pxi":23
 *                 return self.single_chans[dtype](name, **kwargs)
 *             else:
 *                 return Chan(name, cx_dtype_map[dtype])             # <<<<<<<<<<<<<<
 *         elif dtype in self.vector_chans:
 *             return self.vector_chans[dtype](name, dsize)
 */
    /*else*/ {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_cx_dtype_map); if (unlikely(!__pyx_t_5)) __PYX_ERR(15, 23, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_4 = __Pyx_PyObject_GetItem(__pyx_t_5, __pyx_v_dtype); if (unlikely(!__pyx_t_4)) __PYX_ERR(15, 23, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(15, 23, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_INCREF(__pyx_v_name);
      __Pyx_GIVEREF(__pyx_v_name);
      PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v_name);
      __Pyx_GIVEREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_4);
      __pyx_t_4 = 0;
      __pyx_t_4 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5pycx4_5pycda_Chan), __pyx_t_5, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(15, 23, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_r = __pyx_t_4;
      __pyx_t_4 = 0;
      goto __pyx_L0;
    }

    /* "pycx4/chan_factory.pxi":19
 *         if dtype not in cx_dtype_map:
 *             return None
 *         if dsize == 1:             # <<<<<<<<<<<<<<
 *             if dtype in self.single_chans:
 *                 return self.single_chans[dtype](name, **kwargs)
 */
  }

  /* "pycx4/chan_factory.pxi":24
 *             else:
 *                 return Chan(name, cx_dtype_map[dtype])
 *         elif dtype in self.vector_chans:             # <<<<<<<<<<<<<<
 *             return self.vector_chans[dtype](name, dsize)
 *         return VChan(name, cx_dtype_map[dtype], dsize, **kwargs)
 */
  if (unlikely(__pyx_v_self->vector_chans == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(15, 24, __pyx_L1_error)
  }
  __pyx_t_2 = (__Pyx_PyDict_ContainsTF(__pyx_v_dtype, __pyx_v_self->vector_chans, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(15, 24, __pyx_L1_error)
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "pycx4/chan_factory.pxi":25
 *                 return Chan(name, cx_dtype_map[dtype])
 *         elif dtype in self.vector_chans:
 *             return self.vector_chans[dtype](name, dsize)             # <<<<<<<<<<<<<<
 *         return VChan(name, cx_dtype_map[dtype], dsize, **kwargs)
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(__pyx_v_self->vector_chans == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(15, 25, __pyx_L1_error)
    }
    __pyx_t_5 = __Pyx_PyDict_GetItem(__pyx_v_self->vector_chans, __pyx_v_dtype); if (unlikely(!__pyx_t_5)) __PYX_ERR(15, 25, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = NULL;
    __pyx_t_6 = 0;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
        __pyx_t_6 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_name, __pyx_v_dsize};
      __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(15, 25, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_4);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_name, __pyx_v_dsize};
      __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(15, 25, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_4);
    } else
    #endif
    {
      __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(15, 25, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      if (__pyx_t_1) {
        __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_1); __pyx_t_1 = NULL;
      }
      __Pyx_INCREF(__pyx_v_name);
      __Pyx_GIVEREF(__pyx_v_name);
      PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_v_name);
      __Pyx_INCREF(__pyx_v_dsize);
      __Pyx_GIVEREF(__pyx_v_dsize);
      PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_v_dsize);
      __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_7, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(15, 25, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "pycx4/chan_factory.pxi":24
 *             else:
 *                 return Chan(name, cx_dtype_map[dtype])
 *         elif dtype in self.vector_chans:             # <<<<<<<<<<<<<<
 *             return self.vector_chans[dtype](name, dsize)
 *         return VChan(name, cx_dtype_map[dtype], dsize, **kwargs)
 */
  }

  /* "pycx4/chan_factory.pxi":26
 *         elif dtype in self.vector_chans:
 *             return self.vector_chans[dtype](name, dsize)
 *         return VChan(name, cx_dtype_map[dtype], dsize, **kwargs)             # <<<<<<<<<<<<<<
 * 
 * cfactory = ChanFactory()
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_cx_dtype_map); if (unlikely(!__pyx_t_4)) __PYX_ERR(15, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetItem(__pyx_t_4, __pyx_v_dtype); if (unlikely(!__pyx_t_5)) __PYX_ERR(15, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(15, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_v_name);
  __Pyx_GIVEREF(__pyx_v_name);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_name);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_5);
  __Pyx_INCREF(__pyx_v_dsize);
  __Pyx_GIVEREF(__pyx_v_dsize);
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_dsize);
  __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5pycx4_5pycda_VChan), __pyx_t_4, __pyx_v_kwargs); if (unlikely(!__pyx_t_5)) __PYX_ERR(15, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "pycx4/chan_factory.pxi":16
 *         }
 * 
 *     def create_chan(self, name, dtype, dsize, **kwargs):             # <<<<<<<<<<<<<<
 *         if dtype not in cx_dtype_map:
 *             return None
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("pycx4.pycda.ChanFactory.create_chan", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef bint use_setstate
 *     state = (self.single_chans, self.vector_chans)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_11ChanFactory_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_11ChanFactory_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_11ChanFactory_4__reduce_cython__(((struct __pyx_obj_5pycx4_5pycda_ChanFactory *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_11ChanFactory_4__reduce_cython__(struct __pyx_obj_5pycx4_5pycda_ChanFactory *__pyx_v_self) {
  int __pyx_v_use_setstate;
  PyObject *__pyx_v_state = NULL;
  PyObject *__pyx_v__dict = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     cdef bint use_setstate
 *     state = (self.single_chans, self.vector_chans)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->single_chans);
  __Pyx_GIVEREF(__pyx_v_self->single_chans);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_self->single_chans);
  __Pyx_INCREF(__pyx_v_self->vector_chans);
  __Pyx_GIVEREF(__pyx_v_self->vector_chans);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_self->vector_chans);
  __pyx_v_state = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":4
 *     cdef bint use_setstate
 *     state = (self.single_chans, self.vector_chans)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_1 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v__dict = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":5
 *     state = (self.single_chans, self.vector_chans)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_2 = (__pyx_v__dict != Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "(tree fragment)":6
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v__dict);
    __pyx_t_4 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "(tree fragment)":7
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.single_chans is not None or self.vector_chans is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":5
 *     state = (self.single_chans, self.vector_chans)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":9
 *         use_setstate = True
 *     else:
 *         use_setstate = self.single_chans is not None or self.vector_chans is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_ChanFactory, (type(self), 0x57ffd47, None), state
 */
  /*else*/ {
    __pyx_t_2 = (__pyx_v_self->single_chans != ((PyObject*)Py_None));
    __pyx_t_5 = (__pyx_t_2 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_3 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (__pyx_v_self->vector_chans != ((PyObject*)Py_None));
    __pyx_t_2 = (__pyx_t_5 != 0);
    __pyx_t_3 = __pyx_t_2;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_3;
  }
  __pyx_L3:;

  /* "(tree fragment)":10
 *     else:
 *         use_setstate = self.single_chans is not None or self.vector_chans is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_ChanFactory, (type(self), 0x57ffd47, None), state
 *     else:
 */
  __pyx_t_3 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_3) {

    /* "(tree fragment)":11
 *         use_setstate = self.single_chans is not None or self.vector_chans is not None
 *     if use_setstate:
 *         return __pyx_unpickle_ChanFactory, (type(self), 0x57ffd47, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_ChanFactory, (type(self), 0x57ffd47, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_ChanFactory); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_92273991);
    __Pyx_GIVEREF(__pyx_int_92273991);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_int_92273991);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_1, 2, Py_None);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);
    __pyx_t_4 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_6;
    __pyx_t_6 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":10
 *     else:
 *         use_setstate = self.single_chans is not None or self.vector_chans is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_ChanFactory, (type(self), 0x57ffd47, None), state
 *     else:
 */
  }

  /* "(tree fragment)":13
 *         return __pyx_unpickle_ChanFactory, (type(self), 0x57ffd47, None), state
 *     else:
 *         return __pyx_unpickle_ChanFactory, (type(self), 0x57ffd47, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_ChanFactory__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_pyx_unpickle_ChanFactory); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_92273991);
    __Pyx_GIVEREF(__pyx_int_92273991);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_int_92273991);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state);
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1);
    __pyx_t_6 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef bint use_setstate
 *     state = (self.single_chans, self.vector_chans)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("pycx4.pycda.ChanFactory.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":14
 *     else:
 *         return __pyx_unpickle_ChanFactory, (type(self), 0x57ffd47, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_ChanFactory__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_11ChanFactory_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_11ChanFactory_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_11ChanFactory_6__setstate_cython__(((struct __pyx_obj_5pycx4_5pycda_ChanFactory *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_11ChanFactory_6__setstate_cython__(struct __pyx_obj_5pycx4_5pycda_ChanFactory *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":15
 *         return __pyx_unpickle_ChanFactory, (type(self), 0x57ffd47, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_ChanFactory__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 15, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_5pycx4_5pycda___pyx_unpickle_ChanFactory__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":14
 *     else:
 *         return __pyx_unpickle_ChanFactory, (type(self), 0x57ffd47, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_ChanFactory__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.ChanFactory.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/gw.pxi":10
 *         object in_chan, out_chan, last_val
 * 
 *     def __init__(self, str in_name, str out_name, **kwargs):             # <<<<<<<<<<<<<<
 *         dtype = kwargs.get('dtype', 'double')
 *         max_nelems = kwargs.get('max_nelems', 1)
 */

/* Python wrapper */
static int __pyx_pw_5pycx4_5pycda_6PassGW_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5pycx4_5pycda_6PassGW_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_in_name = 0;
  PyObject *__pyx_v_out_name = 0;
  PyObject *__pyx_v_kwargs = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  __pyx_v_kwargs = PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return -1;
  __Pyx_GOTREF(__pyx_v_kwargs);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_in_name,&__pyx_n_s_out_name,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_in_name)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_out_name)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(16, 10, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, __pyx_v_kwargs, values, pos_args, "__init__") < 0)) __PYX_ERR(16, 10, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_in_name = ((PyObject*)values[0]);
    __pyx_v_out_name = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(16, 10, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_DECREF(__pyx_v_kwargs); __pyx_v_kwargs = 0;
  __Pyx_AddTraceback("pycx4.pycda.PassGW.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_in_name), (&PyUnicode_Type), 1, "in_name", 1))) __PYX_ERR(16, 10, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_out_name), (&PyUnicode_Type), 1, "out_name", 1))) __PYX_ERR(16, 10, __pyx_L1_error)
  __pyx_r = __pyx_pf_5pycx4_5pycda_6PassGW___init__(((struct __pyx_obj_5pycx4_5pycda_PassGW *)__pyx_v_self), __pyx_v_in_name, __pyx_v_out_name, __pyx_v_kwargs);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pycx4_5pycda_6PassGW___init__(struct __pyx_obj_5pycx4_5pycda_PassGW *__pyx_v_self, PyObject *__pyx_v_in_name, PyObject *__pyx_v_out_name, PyObject *__pyx_v_kwargs) {
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_v_max_nelems = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "pycx4/gw.pxi":11
 * 
 *     def __init__(self, str in_name, str out_name, **kwargs):
 *         dtype = kwargs.get('dtype', 'double')             # <<<<<<<<<<<<<<
 *         max_nelems = kwargs.get('max_nelems', 1)
 * 
 */
  __pyx_t_1 = __Pyx_PyDict_GetItemDefault(__pyx_v_kwargs, __pyx_n_u_dtype, __pyx_n_u_double); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 11, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_dtype = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pycx4/gw.pxi":12
 *     def __init__(self, str in_name, str out_name, **kwargs):
 *         dtype = kwargs.get('dtype', 'double')
 *         max_nelems = kwargs.get('max_nelems', 1)             # <<<<<<<<<<<<<<
 * 
 *         self.in_chan = cfactory.create_chan(in_name, dtype, max_nelems, **kwargs)
 */
  __pyx_t_1 = __Pyx_PyDict_GetItemDefault(__pyx_v_kwargs, __pyx_n_u_max_nelems, __pyx_int_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_max_nelems = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pycx4/gw.pxi":14
 *         max_nelems = kwargs.get('max_nelems', 1)
 * 
 *         self.in_chan = cfactory.create_chan(in_name, dtype, max_nelems, **kwargs)             # <<<<<<<<<<<<<<
 *         self.out_chan = cfactory.create_chan(out_name, dtype, max_nelems, **kwargs)
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_cfactory); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 14, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_create_chan); if (unlikely(!__pyx_t_2)) __PYX_ERR(16, 14, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 14, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_in_name);
  __Pyx_GIVEREF(__pyx_v_in_name);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_in_name);
  __Pyx_INCREF(__pyx_v_dtype);
  __Pyx_GIVEREF(__pyx_v_dtype);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_dtype);
  __Pyx_INCREF(__pyx_v_max_nelems);
  __Pyx_GIVEREF(__pyx_v_max_nelems);
  PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_max_nelems);
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_1, __pyx_v_kwargs); if (unlikely(!__pyx_t_3)) __PYX_ERR(16, 14, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->in_chan);
  __Pyx_DECREF(__pyx_v_self->in_chan);
  __pyx_v_self->in_chan = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pycx4/gw.pxi":15
 * 
 *         self.in_chan = cfactory.create_chan(in_name, dtype, max_nelems, **kwargs)
 *         self.out_chan = cfactory.create_chan(out_name, dtype, max_nelems, **kwargs)             # <<<<<<<<<<<<<<
 * 
 *         self.in_chan.valueMeasured.connect(self.in_c_proc)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_cfactory); if (unlikely(!__pyx_t_3)) __PYX_ERR(16, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_create_chan); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(16, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_out_name);
  __Pyx_GIVEREF(__pyx_v_out_name);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_out_name);
  __Pyx_INCREF(__pyx_v_dtype);
  __Pyx_GIVEREF(__pyx_v_dtype);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v_dtype);
  __Pyx_INCREF(__pyx_v_max_nelems);
  __Pyx_GIVEREF(__pyx_v_max_nelems);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_max_nelems);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_3, __pyx_v_kwargs); if (unlikely(!__pyx_t_2)) __PYX_ERR(16, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->out_chan);
  __Pyx_DECREF(__pyx_v_self->out_chan);
  __pyx_v_self->out_chan = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "pycx4/gw.pxi":17
 *         self.out_chan = cfactory.create_chan(out_name, dtype, max_nelems, **kwargs)
 * 
 *         self.in_chan.valueMeasured.connect(self.in_c_proc)             # <<<<<<<<<<<<<<
 *         self.out_chan.valueMeasured.connect(self.out_c_proc)
 * 
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->in_chan, __pyx_n_s_valueMeasured); if (unlikely(!__pyx_t_3)) __PYX_ERR(16, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_connect); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_in_c_proc); if (unlikely(!__pyx_t_3)) __PYX_ERR(16, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_4, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(16, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pycx4/gw.pxi":18
 * 
 *         self.in_chan.valueMeasured.connect(self.in_c_proc)
 *         self.out_chan.valueMeasured.connect(self.out_c_proc)             # <<<<<<<<<<<<<<
 * 
 *         self.last_val = None
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->out_chan, __pyx_n_s_valueMeasured); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_connect); if (unlikely(!__pyx_t_3)) __PYX_ERR(16, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_out_c_proc); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_t_1) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(16, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pycx4/gw.pxi":20
 *         self.out_chan.valueMeasured.connect(self.out_c_proc)
 * 
 *         self.last_val = None             # <<<<<<<<<<<<<<
 * 
 *     cpdef void in_c_proc(self, chan):
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->last_val);
  __Pyx_DECREF(__pyx_v_self->last_val);
  __pyx_v_self->last_val = Py_None;

  /* "pycx4/gw.pxi":10
 *         object in_chan, out_chan, last_val
 * 
 *     def __init__(self, str in_name, str out_name, **kwargs):             # <<<<<<<<<<<<<<
 *         dtype = kwargs.get('dtype', 'double')
 *         max_nelems = kwargs.get('max_nelems', 1)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pycx4.pycda.PassGW.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_max_nelems);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/gw.pxi":22
 *         self.last_val = None
 * 
 *     cpdef void in_c_proc(self, chan):             # <<<<<<<<<<<<<<
 *         self.last_val = self.in_chan.val
 *         self.out_chan.setValue(self.in_chan.val)
 */

static PyObject *__pyx_pw_5pycx4_5pycda_6PassGW_3in_c_proc(PyObject *__pyx_v_self, PyObject *__pyx_v_chan); /*proto*/
static void __pyx_f_5pycx4_5pycda_6PassGW_in_c_proc(struct __pyx_obj_5pycx4_5pycda_PassGW *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_chan, int __pyx_skip_dispatch) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("in_c_proc", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    static PY_UINT64_T tp_dict_version = 0, obj_dict_version = 0;
    if (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict && tp_dict_version == __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) && (!Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset || obj_dict_version == __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))))));
    else {
      PY_UINT64_T type_dict_guard = (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict)) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_in_c_proc); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 22, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_5pycx4_5pycda_6PassGW_3in_c_proc)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_chan) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_chan);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(16, 22, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
      tp_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      obj_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset) ? __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))) : 0;
      if (unlikely(type_dict_guard != tp_dict_version)) {
        tp_dict_version = obj_dict_version = 0;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    }
    #endif
  }

  /* "pycx4/gw.pxi":23
 * 
 *     cpdef void in_c_proc(self, chan):
 *         self.last_val = self.in_chan.val             # <<<<<<<<<<<<<<
 *         self.out_chan.setValue(self.in_chan.val)
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->in_chan, __pyx_n_s_val); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 23, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->last_val);
  __Pyx_DECREF(__pyx_v_self->last_val);
  __pyx_v_self->last_val = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pycx4/gw.pxi":24
 *     cpdef void in_c_proc(self, chan):
 *         self.last_val = self.in_chan.val
 *         self.out_chan.setValue(self.in_chan.val)             # <<<<<<<<<<<<<<
 * 
 *     cpdef void out_c_proc(self, chan):
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->out_chan, __pyx_n_s_setValue); if (unlikely(!__pyx_t_2)) __PYX_ERR(16, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->in_chan, __pyx_n_s_val); if (unlikely(!__pyx_t_3)) __PYX_ERR(16, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/gw.pxi":22
 *         self.last_val = None
 * 
 *     cpdef void in_c_proc(self, chan):             # <<<<<<<<<<<<<<
 *         self.last_val = self.in_chan.val
 *         self.out_chan.setValue(self.in_chan.val)
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("pycx4.pycda.PassGW.in_c_proc", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_6PassGW_3in_c_proc(PyObject *__pyx_v_self, PyObject *__pyx_v_chan); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_6PassGW_3in_c_proc(PyObject *__pyx_v_self, PyObject *__pyx_v_chan) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("in_c_proc (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_6PassGW_2in_c_proc(((struct __pyx_obj_5pycx4_5pycda_PassGW *)__pyx_v_self), ((PyObject *)__pyx_v_chan));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_6PassGW_2in_c_proc(struct __pyx_obj_5pycx4_5pycda_PassGW *__pyx_v_self, PyObject *__pyx_v_chan) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("in_c_proc", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_void_to_None(__pyx_f_5pycx4_5pycda_6PassGW_in_c_proc(__pyx_v_self, __pyx_v_chan, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.PassGW.in_c_proc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/gw.pxi":26
 *         self.out_chan.setValue(self.in_chan.val)
 * 
 *     cpdef void out_c_proc(self, chan):             # <<<<<<<<<<<<<<
 *         if chan.val == self.last_val or self.last_val is None:
 *             return
 */

static PyObject *__pyx_pw_5pycx4_5pycda_6PassGW_5out_c_proc(PyObject *__pyx_v_self, PyObject *__pyx_v_chan); /*proto*/
static void __pyx_f_5pycx4_5pycda_6PassGW_out_c_proc(struct __pyx_obj_5pycx4_5pycda_PassGW *__pyx_v_self, PyObject *__pyx_v_chan, int __pyx_skip_dispatch) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  __Pyx_RefNannySetupContext("out_c_proc", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    static PY_UINT64_T tp_dict_version = 0, obj_dict_version = 0;
    if (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict && tp_dict_version == __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) && (!Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset || obj_dict_version == __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))))));
    else {
      PY_UINT64_T type_dict_guard = (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict)) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_out_c_proc); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 26, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_5pycx4_5pycda_6PassGW_5out_c_proc)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_chan) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_chan);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(16, 26, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
      tp_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      obj_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset) ? __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))) : 0;
      if (unlikely(type_dict_guard != tp_dict_version)) {
        tp_dict_version = obj_dict_version = 0;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    }
    #endif
  }

  /* "pycx4/gw.pxi":27
 * 
 *     cpdef void out_c_proc(self, chan):
 *         if chan.val == self.last_val or self.last_val is None:             # <<<<<<<<<<<<<<
 *             return
 *         self.out_chan.setValue(self.last_val)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_chan, __pyx_n_s_val); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_RichCompare(__pyx_t_1, __pyx_v_self->last_val, Py_EQ); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(16, 27, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(16, 27, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (!__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = (__pyx_v_self->last_val == Py_None);
  __pyx_t_7 = (__pyx_t_6 != 0);
  __pyx_t_5 = __pyx_t_7;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_5) {

    /* "pycx4/gw.pxi":28
 *     cpdef void out_c_proc(self, chan):
 *         if chan.val == self.last_val or self.last_val is None:
 *             return             # <<<<<<<<<<<<<<
 *         self.out_chan.setValue(self.last_val)
 * 
 */
    goto __pyx_L0;

    /* "pycx4/gw.pxi":27
 * 
 *     cpdef void out_c_proc(self, chan):
 *         if chan.val == self.last_val or self.last_val is None:             # <<<<<<<<<<<<<<
 *             return
 *         self.out_chan.setValue(self.last_val)
 */
  }

  /* "pycx4/gw.pxi":29
 *         if chan.val == self.last_val or self.last_val is None:
 *             return
 *         self.out_chan.setValue(self.last_val)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->out_chan, __pyx_n_s_setValue); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 29, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_2 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_3, __pyx_v_self->last_val) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_v_self->last_val);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(16, 29, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pycx4/gw.pxi":26
 *         self.out_chan.setValue(self.in_chan.val)
 * 
 *     cpdef void out_c_proc(self, chan):             # <<<<<<<<<<<<<<
 *         if chan.val == self.last_val or self.last_val is None:
 *             return
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("pycx4.pycda.PassGW.out_c_proc", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_6PassGW_5out_c_proc(PyObject *__pyx_v_self, PyObject *__pyx_v_chan); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_6PassGW_5out_c_proc(PyObject *__pyx_v_self, PyObject *__pyx_v_chan) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("out_c_proc (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_6PassGW_4out_c_proc(((struct __pyx_obj_5pycx4_5pycda_PassGW *)__pyx_v_self), ((PyObject *)__pyx_v_chan));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_6PassGW_4out_c_proc(struct __pyx_obj_5pycx4_5pycda_PassGW *__pyx_v_self, PyObject *__pyx_v_chan) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("out_c_proc", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_void_to_None(__pyx_f_5pycx4_5pycda_6PassGW_out_c_proc(__pyx_v_self, __pyx_v_chan, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.PassGW.out_c_proc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef bint use_setstate
 *     state = (self.in_chan, self.last_val, self.out_chan)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_6PassGW_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_6PassGW_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_6PassGW_6__reduce_cython__(((struct __pyx_obj_5pycx4_5pycda_PassGW *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_6PassGW_6__reduce_cython__(struct __pyx_obj_5pycx4_5pycda_PassGW *__pyx_v_self) {
  int __pyx_v_use_setstate;
  PyObject *__pyx_v_state = NULL;
  PyObject *__pyx_v__dict = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     cdef bint use_setstate
 *     state = (self.in_chan, self.last_val, self.out_chan)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->in_chan);
  __Pyx_GIVEREF(__pyx_v_self->in_chan);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_self->in_chan);
  __Pyx_INCREF(__pyx_v_self->last_val);
  __Pyx_GIVEREF(__pyx_v_self->last_val);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_self->last_val);
  __Pyx_INCREF(__pyx_v_self->out_chan);
  __Pyx_GIVEREF(__pyx_v_self->out_chan);
  PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_self->out_chan);
  __pyx_v_state = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":4
 *     cdef bint use_setstate
 *     state = (self.in_chan, self.last_val, self.out_chan)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_1 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v__dict = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":5
 *     state = (self.in_chan, self.last_val, self.out_chan)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_2 = (__pyx_v__dict != Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "(tree fragment)":6
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v__dict);
    __pyx_t_4 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "(tree fragment)":7
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.in_chan is not None or self.last_val is not None or self.out_chan is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":5
 *     state = (self.in_chan, self.last_val, self.out_chan)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":9
 *         use_setstate = True
 *     else:
 *         use_setstate = self.in_chan is not None or self.last_val is not None or self.out_chan is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_PassGW, (type(self), 0x5a33863, None), state
 */
  /*else*/ {
    __pyx_t_2 = (__pyx_v_self->in_chan != Py_None);
    __pyx_t_5 = (__pyx_t_2 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_3 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (__pyx_v_self->last_val != Py_None);
    __pyx_t_2 = (__pyx_t_5 != 0);
    if (!__pyx_t_2) {
    } else {
      __pyx_t_3 = __pyx_t_2;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_2 = (__pyx_v_self->out_chan != Py_None);
    __pyx_t_5 = (__pyx_t_2 != 0);
    __pyx_t_3 = __pyx_t_5;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_3;
  }
  __pyx_L3:;

  /* "(tree fragment)":10
 *     else:
 *         use_setstate = self.in_chan is not None or self.last_val is not None or self.out_chan is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_PassGW, (type(self), 0x5a33863, None), state
 *     else:
 */
  __pyx_t_3 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_3) {

    /* "(tree fragment)":11
 *         use_setstate = self.in_chan is not None or self.last_val is not None or self.out_chan is not None
 *     if use_setstate:
 *         return __pyx_unpickle_PassGW, (type(self), 0x5a33863, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_PassGW, (type(self), 0x5a33863, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_PassGW); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_94582883);
    __Pyx_GIVEREF(__pyx_int_94582883);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_int_94582883);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_1, 2, Py_None);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);
    __pyx_t_4 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_6;
    __pyx_t_6 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":10
 *     else:
 *         use_setstate = self.in_chan is not None or self.last_val is not None or self.out_chan is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_PassGW, (type(self), 0x5a33863, None), state
 *     else:
 */
  }

  /* "(tree fragment)":13
 *         return __pyx_unpickle_PassGW, (type(self), 0x5a33863, None), state
 *     else:
 *         return __pyx_unpickle_PassGW, (type(self), 0x5a33863, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_PassGW__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_pyx_unpickle_PassGW); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_94582883);
    __Pyx_GIVEREF(__pyx_int_94582883);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_int_94582883);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state);
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1);
    __pyx_t_6 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef bint use_setstate
 *     state = (self.in_chan, self.last_val, self.out_chan)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("pycx4.pycda.PassGW.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":14
 *     else:
 *         return __pyx_unpickle_PassGW, (type(self), 0x5a33863, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_PassGW__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_6PassGW_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_6PassGW_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_6PassGW_8__setstate_cython__(((struct __pyx_obj_5pycx4_5pycda_PassGW *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_6PassGW_8__setstate_cython__(struct __pyx_obj_5pycx4_5pycda_PassGW *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":15
 *         return __pyx_unpickle_PassGW, (type(self), 0x5a33863, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_PassGW__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 15, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_5pycx4_5pycda___pyx_unpickle_PassGW__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":14
 *     else:
 *         return __pyx_unpickle_PassGW, (type(self), 0x5a33863, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_PassGW__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.PassGW.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/gw.pxi":36
 *         object in_chan, out_chan
 * 
 *     def __init__(self, str in_name, str out_name, **kwargs):             # <<<<<<<<<<<<<<
 *         dtype = kwargs.get('dtype', 'double')
 *         max_nelems = kwargs.get('max_nelems', 1)
 */

/* Python wrapper */
static int __pyx_pw_5pycx4_5pycda_6ProcGW_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5pycx4_5pycda_6ProcGW_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_in_name = 0;
  PyObject *__pyx_v_out_name = 0;
  PyObject *__pyx_v_kwargs = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  __pyx_v_kwargs = PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return -1;
  __Pyx_GOTREF(__pyx_v_kwargs);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_in_name,&__pyx_n_s_out_name,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_in_name)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_out_name)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(16, 36, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, __pyx_v_kwargs, values, pos_args, "__init__") < 0)) __PYX_ERR(16, 36, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_in_name = ((PyObject*)values[0]);
    __pyx_v_out_name = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(16, 36, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_DECREF(__pyx_v_kwargs); __pyx_v_kwargs = 0;
  __Pyx_AddTraceback("pycx4.pycda.ProcGW.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_in_name), (&PyUnicode_Type), 1, "in_name", 1))) __PYX_ERR(16, 36, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_out_name), (&PyUnicode_Type), 1, "out_name", 1))) __PYX_ERR(16, 36, __pyx_L1_error)
  __pyx_r = __pyx_pf_5pycx4_5pycda_6ProcGW___init__(((struct __pyx_obj_5pycx4_5pycda_ProcGW *)__pyx_v_self), __pyx_v_in_name, __pyx_v_out_name, __pyx_v_kwargs);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pycx4_5pycda_6ProcGW___init__(struct __pyx_obj_5pycx4_5pycda_ProcGW *__pyx_v_self, PyObject *__pyx_v_in_name, PyObject *__pyx_v_out_name, PyObject *__pyx_v_kwargs) {
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_v_max_nelems = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "pycx4/gw.pxi":37
 * 
 *     def __init__(self, str in_name, str out_name, **kwargs):
 *         dtype = kwargs.get('dtype', 'double')             # <<<<<<<<<<<<<<
 *         max_nelems = kwargs.get('max_nelems', 1)
 * 
 */
  __pyx_t_1 = __Pyx_PyDict_GetItemDefault(__pyx_v_kwargs, __pyx_n_u_dtype, __pyx_n_u_double); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 37, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_dtype = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pycx4/gw.pxi":38
 *     def __init__(self, str in_name, str out_name, **kwargs):
 *         dtype = kwargs.get('dtype', 'double')
 *         max_nelems = kwargs.get('max_nelems', 1)             # <<<<<<<<<<<<<<
 * 
 *         self.in_chan = cfactory.create_chan(in_name, dtype, max_nelems, **kwargs)
 */
  __pyx_t_1 = __Pyx_PyDict_GetItemDefault(__pyx_v_kwargs, __pyx_n_u_max_nelems, __pyx_int_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 38, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_max_nelems = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pycx4/gw.pxi":40
 *         max_nelems = kwargs.get('max_nelems', 1)
 * 
 *         self.in_chan = cfactory.create_chan(in_name, dtype, max_nelems, **kwargs)             # <<<<<<<<<<<<<<
 *         self.out_chan = cfactory.create_chan(out_name, dtype, max_nelems, **kwargs)
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_cfactory); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_create_chan); if (unlikely(!__pyx_t_2)) __PYX_ERR(16, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_in_name);
  __Pyx_GIVEREF(__pyx_v_in_name);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_in_name);
  __Pyx_INCREF(__pyx_v_dtype);
  __Pyx_GIVEREF(__pyx_v_dtype);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_dtype);
  __Pyx_INCREF(__pyx_v_max_nelems);
  __Pyx_GIVEREF(__pyx_v_max_nelems);
  PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_max_nelems);
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_1, __pyx_v_kwargs); if (unlikely(!__pyx_t_3)) __PYX_ERR(16, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->in_chan);
  __Pyx_DECREF(__pyx_v_self->in_chan);
  __pyx_v_self->in_chan = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pycx4/gw.pxi":41
 * 
 *         self.in_chan = cfactory.create_chan(in_name, dtype, max_nelems, **kwargs)
 *         self.out_chan = cfactory.create_chan(out_name, dtype, max_nelems, **kwargs)             # <<<<<<<<<<<<<<
 * 
 *         self.in_chan.valueMeasured.connect(self.in_c_proc)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_cfactory); if (unlikely(!__pyx_t_3)) __PYX_ERR(16, 41, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_create_chan); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 41, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(16, 41, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_out_name);
  __Pyx_GIVEREF(__pyx_v_out_name);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_out_name);
  __Pyx_INCREF(__pyx_v_dtype);
  __Pyx_GIVEREF(__pyx_v_dtype);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v_dtype);
  __Pyx_INCREF(__pyx_v_max_nelems);
  __Pyx_GIVEREF(__pyx_v_max_nelems);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_max_nelems);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_3, __pyx_v_kwargs); if (unlikely(!__pyx_t_2)) __PYX_ERR(16, 41, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->out_chan);
  __Pyx_DECREF(__pyx_v_self->out_chan);
  __pyx_v_self->out_chan = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "pycx4/gw.pxi":43
 *         self.out_chan = cfactory.create_chan(out_name, dtype, max_nelems, **kwargs)
 * 
 *         self.in_chan.valueMeasured.connect(self.in_c_proc)             # <<<<<<<<<<<<<<
 *         self.out_chan.valueMeasured.connect(self.out_c_proc)
 * 
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->in_chan, __pyx_n_s_valueMeasured); if (unlikely(!__pyx_t_3)) __PYX_ERR(16, 43, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_connect); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 43, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_in_c_proc); if (unlikely(!__pyx_t_3)) __PYX_ERR(16, 43, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_4, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(16, 43, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pycx4/gw.pxi":44
 * 
 *         self.in_chan.valueMeasured.connect(self.in_c_proc)
 *         self.out_chan.valueMeasured.connect(self.out_c_proc)             # <<<<<<<<<<<<<<
 * 
 *     cpdef void in_c_proc(self, chan):
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->out_chan, __pyx_n_s_valueMeasured); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 44, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_connect); if (unlikely(!__pyx_t_3)) __PYX_ERR(16, 44, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_out_c_proc); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 44, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_t_1) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(16, 44, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pycx4/gw.pxi":36
 *         object in_chan, out_chan
 * 
 *     def __init__(self, str in_name, str out_name, **kwargs):             # <<<<<<<<<<<<<<
 *         dtype = kwargs.get('dtype', 'double')
 *         max_nelems = kwargs.get('max_nelems', 1)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pycx4.pycda.ProcGW.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_max_nelems);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/gw.pxi":46
 *         self.out_chan.valueMeasured.connect(self.out_c_proc)
 * 
 *     cpdef void in_c_proc(self, chan):             # <<<<<<<<<<<<<<
 *         pass
 * 
 */

static PyObject *__pyx_pw_5pycx4_5pycda_6ProcGW_3in_c_proc(PyObject *__pyx_v_self, PyObject *__pyx_v_chan); /*proto*/
static void __pyx_f_5pycx4_5pycda_6ProcGW_in_c_proc(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_ProcGW *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_chan, int __pyx_skip_dispatch) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("in_c_proc", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    static PY_UINT64_T tp_dict_version = 0, obj_dict_version = 0;
    if (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict && tp_dict_version == __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) && (!Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset || obj_dict_version == __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))))));
    else {
      PY_UINT64_T type_dict_guard = (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict)) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_in_c_proc); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 46, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_5pycx4_5pycda_6ProcGW_3in_c_proc)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_chan) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_chan);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(16, 46, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
      tp_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      obj_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset) ? __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))) : 0;
      if (unlikely(type_dict_guard != tp_dict_version)) {
        tp_dict_version = obj_dict_version = 0;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    }
    #endif
  }

  /* "pycx4/gw.pxi":47
 * 
 *     cpdef void in_c_proc(self, chan):
 *         pass             # <<<<<<<<<<<<<<
 * 
 *     cpdef void out_c_proc(self, chan):
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("pycx4.pycda.ProcGW.in_c_proc", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/gw.pxi":46
 *         self.out_chan.valueMeasured.connect(self.out_c_proc)
 * 
 *     cpdef void in_c_proc(self, chan):             # <<<<<<<<<<<<<<
 *         pass
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_6ProcGW_3in_c_proc(PyObject *__pyx_v_self, PyObject *__pyx_v_chan); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_6ProcGW_3in_c_proc(PyObject *__pyx_v_self, PyObject *__pyx_v_chan) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("in_c_proc (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_6ProcGW_2in_c_proc(((struct __pyx_obj_5pycx4_5pycda_ProcGW *)__pyx_v_self), ((PyObject *)__pyx_v_chan));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_6ProcGW_2in_c_proc(struct __pyx_obj_5pycx4_5pycda_ProcGW *__pyx_v_self, PyObject *__pyx_v_chan) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("in_c_proc", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_void_to_None(__pyx_f_5pycx4_5pycda_6ProcGW_in_c_proc(__pyx_v_self, __pyx_v_chan, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 46, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.ProcGW.in_c_proc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/gw.pxi":49
 *         pass
 * 
 *     cpdef void out_c_proc(self, chan):             # <<<<<<<<<<<<<<
 *         pass
 * 
 */

static PyObject *__pyx_pw_5pycx4_5pycda_6ProcGW_5out_c_proc(PyObject *__pyx_v_self, PyObject *__pyx_v_chan); /*proto*/
static void __pyx_f_5pycx4_5pycda_6ProcGW_out_c_proc(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_ProcGW *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_chan, int __pyx_skip_dispatch) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("out_c_proc", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    static PY_UINT64_T tp_dict_version = 0, obj_dict_version = 0;
    if (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict && tp_dict_version == __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) && (!Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset || obj_dict_version == __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))))));
    else {
      PY_UINT64_T type_dict_guard = (likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict)) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_out_c_proc); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 49, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_5pycx4_5pycda_6ProcGW_5out_c_proc)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_chan) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_chan);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(16, 49, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
      tp_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) ? __PYX_GET_DICT_VERSION(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dict) : 0;
      obj_dict_version = likely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset) ? __PYX_GET_DICT_VERSION(_PyObject_GetDictPtr(((PyObject *)__pyx_v_self))) : 0;
      if (unlikely(type_dict_guard != tp_dict_version)) {
        tp_dict_version = obj_dict_version = 0;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP
    }
    #endif
  }

  /* "pycx4/gw.pxi":50
 * 
 *     cpdef void out_c_proc(self, chan):
 *         pass             # <<<<<<<<<<<<<<
 * 
 *     def in2out(in_val):
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("pycx4.pycda.ProcGW.out_c_proc", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/gw.pxi":49
 *         pass
 * 
 *     cpdef void out_c_proc(self, chan):             # <<<<<<<<<<<<<<
 *         pass
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_6ProcGW_5out_c_proc(PyObject *__pyx_v_self, PyObject *__pyx_v_chan); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_6ProcGW_5out_c_proc(PyObject *__pyx_v_self, PyObject *__pyx_v_chan) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("out_c_proc (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_6ProcGW_4out_c_proc(((struct __pyx_obj_5pycx4_5pycda_ProcGW *)__pyx_v_self), ((PyObject *)__pyx_v_chan));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_6ProcGW_4out_c_proc(struct __pyx_obj_5pycx4_5pycda_ProcGW *__pyx_v_self, PyObject *__pyx_v_chan) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("out_c_proc", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_void_to_None(__pyx_f_5pycx4_5pycda_6ProcGW_out_c_proc(__pyx_v_self, __pyx_v_chan, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 49, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.ProcGW.out_c_proc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/gw.pxi":52
 *         pass
 * 
 *     def in2out(in_val):             # <<<<<<<<<<<<<<
 *         pass
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_6ProcGW_7in2out(PyObject *__pyx_v_in_val, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_6ProcGW_7in2out(PyObject *__pyx_v_in_val, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("in2out (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_6ProcGW_6in2out(((struct __pyx_obj_5pycx4_5pycda_ProcGW *)__pyx_v_in_val));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_6ProcGW_6in2out(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_ProcGW *__pyx_v_in_val) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("in2out", 0);

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef bint use_setstate
 *     state = (self.in_chan, self.out_chan)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_6ProcGW_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_6ProcGW_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_6ProcGW_8__reduce_cython__(((struct __pyx_obj_5pycx4_5pycda_ProcGW *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_6ProcGW_8__reduce_cython__(struct __pyx_obj_5pycx4_5pycda_ProcGW *__pyx_v_self) {
  int __pyx_v_use_setstate;
  PyObject *__pyx_v_state = NULL;
  PyObject *__pyx_v__dict = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     cdef bint use_setstate
 *     state = (self.in_chan, self.out_chan)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->in_chan);
  __Pyx_GIVEREF(__pyx_v_self->in_chan);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_self->in_chan);
  __Pyx_INCREF(__pyx_v_self->out_chan);
  __Pyx_GIVEREF(__pyx_v_self->out_chan);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_self->out_chan);
  __pyx_v_state = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":4
 *     cdef bint use_setstate
 *     state = (self.in_chan, self.out_chan)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_1 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v__dict = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":5
 *     state = (self.in_chan, self.out_chan)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_2 = (__pyx_v__dict != Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "(tree fragment)":6
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v__dict);
    __pyx_t_4 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "(tree fragment)":7
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.in_chan is not None or self.out_chan is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":5
 *     state = (self.in_chan, self.out_chan)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":9
 *         use_setstate = True
 *     else:
 *         use_setstate = self.in_chan is not None or self.out_chan is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_ProcGW, (type(self), 0x777971d, None), state
 */
  /*else*/ {
    __pyx_t_2 = (__pyx_v_self->in_chan != Py_None);
    __pyx_t_5 = (__pyx_t_2 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_3 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (__pyx_v_self->out_chan != Py_None);
    __pyx_t_2 = (__pyx_t_5 != 0);
    __pyx_t_3 = __pyx_t_2;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_3;
  }
  __pyx_L3:;

  /* "(tree fragment)":10
 *     else:
 *         use_setstate = self.in_chan is not None or self.out_chan is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_ProcGW, (type(self), 0x777971d, None), state
 *     else:
 */
  __pyx_t_3 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_3) {

    /* "(tree fragment)":11
 *         use_setstate = self.in_chan is not None or self.out_chan is not None
 *     if use_setstate:
 *         return __pyx_unpickle_ProcGW, (type(self), 0x777971d, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_ProcGW, (type(self), 0x777971d, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_ProcGW); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_125277981);
    __Pyx_GIVEREF(__pyx_int_125277981);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_int_125277981);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_1, 2, Py_None);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);
    __pyx_t_4 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_6;
    __pyx_t_6 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":10
 *     else:
 *         use_setstate = self.in_chan is not None or self.out_chan is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_ProcGW, (type(self), 0x777971d, None), state
 *     else:
 */
  }

  /* "(tree fragment)":13
 *         return __pyx_unpickle_ProcGW, (type(self), 0x777971d, None), state
 *     else:
 *         return __pyx_unpickle_ProcGW, (type(self), 0x777971d, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_ProcGW__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_pyx_unpickle_ProcGW); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_125277981);
    __Pyx_GIVEREF(__pyx_int_125277981);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_int_125277981);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state);
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1);
    __pyx_t_6 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef bint use_setstate
 *     state = (self.in_chan, self.out_chan)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("pycx4.pycda.ProcGW.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":14
 *     else:
 *         return __pyx_unpickle_ProcGW, (type(self), 0x777971d, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_ProcGW__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_6ProcGW_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_6ProcGW_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_6ProcGW_10__setstate_cython__(((struct __pyx_obj_5pycx4_5pycda_ProcGW *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_6ProcGW_10__setstate_cython__(struct __pyx_obj_5pycx4_5pycda_ProcGW *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":15
 *         return __pyx_unpickle_ProcGW, (type(self), 0x777971d, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_ProcGW__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 15, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_5pycx4_5pycda___pyx_unpickle_ProcGW__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":14
 *     else:
 *         return __pyx_unpickle_ProcGW, (type(self), 0x777971d, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_ProcGW__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.ProcGW.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_Timer(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0xef1b4d4:
 *         from pickle import PickleError as __pyx_PickleError
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_7__pyx_unpickle_Timer(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5pycx4_5pycda_7__pyx_unpickle_Timer = {"__pyx_unpickle_Timer", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_5pycx4_5pycda_7__pyx_unpickle_Timer, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5pycx4_5pycda_7__pyx_unpickle_Timer(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_Timer (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Timer", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Timer", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_Timer") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Timer", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pycx4.pycda.__pyx_unpickle_Timer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pycx4_5pycda_6__pyx_unpickle_Timer(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_6__pyx_unpickle_Timer(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = NULL;
  PyObject *__pyx_v___pyx_result = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  __Pyx_RefNannySetupContext("__pyx_unpickle_Timer", 0);

  /* "(tree fragment)":2
 * def __pyx_unpickle_Timer(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0xef1b4d4:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xef1b4d4 = (active, repeat, tid, timeout, usec))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0xef1b4d4) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":3
 * def __pyx_unpickle_Timer(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0xef1b4d4:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xef1b4d4 = (active, repeat, tid, timeout, usec))" % __pyx_checksum)
 *     __pyx_result = Timer.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":4
 *     if __pyx_checksum != 0xef1b4d4:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xef1b4d4 = (active, repeat, tid, timeout, usec))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = Timer.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 4, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0xef, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 4, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 4, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 4, __pyx_L1_error)

    /* "(tree fragment)":2
 * def __pyx_unpickle_Timer(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0xef1b4d4:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xef1b4d4 = (active, repeat, tid, timeout, usec))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":5
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xef1b4d4 = (active, repeat, tid, timeout, usec))" % __pyx_checksum)
 *     __pyx_result = Timer.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Timer__set_state(<Timer> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_5pycx4_5pycda_Timer), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":6
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xef1b4d4 = (active, repeat, tid, timeout, usec))" % __pyx_checksum)
 *     __pyx_result = Timer.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_Timer__set_state(<Timer> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":7
 *     __pyx_result = Timer.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Timer__set_state(<Timer> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_Timer__set_state(Timer __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 7, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_5pycx4_5pycda___pyx_unpickle_Timer__set_state(((struct __pyx_obj_5pycx4_5pycda_Timer *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xef1b4d4 = (active, repeat, tid, timeout, usec))" % __pyx_checksum)
 *     __pyx_result = Timer.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_Timer__set_state(<Timer> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":8
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Timer__set_state(<Timer> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_Timer__set_state(Timer __pyx_result, tuple __pyx_state):
 *     __pyx_result.active = __pyx_state[0]; __pyx_result.repeat = __pyx_state[1]; __pyx_result.tid = __pyx_state[2]; __pyx_result.timeout = __pyx_state[3]; __pyx_result.usec = __pyx_state[4]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_Timer(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0xef1b4d4:
 *         from pickle import PickleError as __pyx_PickleError
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pycx4.pycda.__pyx_unpickle_Timer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":9
 *         __pyx_unpickle_Timer__set_state(<Timer> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_Timer__set_state(Timer __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.active = __pyx_state[0]; __pyx_result.repeat = __pyx_state[1]; __pyx_result.tid = __pyx_state[2]; __pyx_result.timeout = __pyx_state[3]; __pyx_result.usec = __pyx_state[4]
 *     if len(__pyx_state) > 5 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_5pycx4_5pycda___pyx_unpickle_Timer__set_state(struct __pyx_obj_5pycx4_5pycda_Timer *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  sl_tid_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  __Pyx_RefNannySetupContext("__pyx_unpickle_Timer__set_state", 0);

  /* "(tree fragment)":10
 *     return __pyx_result
 * cdef __pyx_unpickle_Timer__set_state(Timer __pyx_result, tuple __pyx_state):
 *     __pyx_result.active = __pyx_state[0]; __pyx_result.repeat = __pyx_state[1]; __pyx_result.tid = __pyx_state[2]; __pyx_result.timeout = __pyx_state[3]; __pyx_result.usec = __pyx_state[4]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 5 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[5])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyInt_As_int(PyTuple_GET_ITEM(__pyx_v___pyx_state, 0)); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 10, __pyx_L1_error)
  __pyx_v___pyx_result->active = __pyx_t_1;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyInt_As_int(PyTuple_GET_ITEM(__pyx_v___pyx_state, 1)); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 10, __pyx_L1_error)
  __pyx_v___pyx_result->repeat = __pyx_t_1;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_2 = __Pyx_PyInt_As_sl_tid_t(PyTuple_GET_ITEM(__pyx_v___pyx_state, 2)); if (unlikely((__pyx_t_2 == ((sl_tid_t)-1)) && PyErr_Occurred())) __PYX_ERR(1, 10, __pyx_L1_error)
  __pyx_v___pyx_result->tid = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  if (!(likely(((PyTuple_GET_ITEM(__pyx_v___pyx_state, 3)) == Py_None) || likely(__Pyx_TypeTest(PyTuple_GET_ITEM(__pyx_v___pyx_state, 3), __pyx_ptype_5pycx4_5pycda_Signal))))) __PYX_ERR(1, 10, __pyx_L1_error)
  __pyx_t_3 = PyTuple_GET_ITEM(__pyx_v___pyx_state, 3);
  __Pyx_INCREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v___pyx_result->timeout);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->timeout));
  __pyx_v___pyx_result->timeout = ((struct __pyx_obj_5pycx4_5pycda_Signal *)__pyx_t_3);
  __pyx_t_3 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyInt_As_int(PyTuple_GET_ITEM(__pyx_v___pyx_state, 4)); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 10, __pyx_L1_error)
  __pyx_v___pyx_result->usec = __pyx_t_1;

  /* "(tree fragment)":11
 * cdef __pyx_unpickle_Timer__set_state(Timer __pyx_result, tuple __pyx_state):
 *     __pyx_result.active = __pyx_state[0]; __pyx_result.repeat = __pyx_state[1]; __pyx_result.tid = __pyx_state[2]; __pyx_result.timeout = __pyx_state[3]; __pyx_result.usec = __pyx_state[4]
 *     if len(__pyx_state) > 5 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[5])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 11, __pyx_L1_error)
  }
  __pyx_t_5 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(1, 11, __pyx_L1_error)
  __pyx_t_6 = ((__pyx_t_5 > 5) != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_4 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(1, 11, __pyx_L1_error)
  __pyx_t_7 = (__pyx_t_6 != 0);
  __pyx_t_4 = __pyx_t_7;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_4) {

    /* "(tree fragment)":12
 *     __pyx_result.active = __pyx_state[0]; __pyx_result.repeat = __pyx_state[1]; __pyx_result.tid = __pyx_state[2]; __pyx_result.timeout = __pyx_state[3]; __pyx_result.usec = __pyx_state[4]
 *     if len(__pyx_state) > 5 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[5])             # <<<<<<<<<<<<<<
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_update); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 12, __pyx_L1_error)
    }
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_9);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_9, function);
      }
    }
    __pyx_t_3 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_9, __pyx_t_8, PyTuple_GET_ITEM(__pyx_v___pyx_state, 5)) : __Pyx_PyObject_CallOneArg(__pyx_t_9, PyTuple_GET_ITEM(__pyx_v___pyx_state, 5));
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":11
 * cdef __pyx_unpickle_Timer__set_state(Timer __pyx_result, tuple __pyx_state):
 *     __pyx_result.active = __pyx_state[0]; __pyx_result.repeat = __pyx_state[1]; __pyx_result.tid = __pyx_state[2]; __pyx_result.timeout = __pyx_state[3]; __pyx_result.usec = __pyx_state[4]
 *     if len(__pyx_state) > 5 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[5])
 */
  }

  /* "(tree fragment)":9
 *         __pyx_unpickle_Timer__set_state(<Timer> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_Timer__set_state(Timer __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.active = __pyx_state[0]; __pyx_result.repeat = __pyx_state[1]; __pyx_result.tid = __pyx_state[2]; __pyx_result.timeout = __pyx_state[3]; __pyx_result.usec = __pyx_state[4]
 *     if len(__pyx_state) > 5 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("pycx4.pycda.__pyx_unpickle_Timer__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_FdEvent(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0x183067a:
 *         from pickle import PickleError as __pyx_PickleError
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_9__pyx_unpickle_FdEvent(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5pycx4_5pycda_9__pyx_unpickle_FdEvent = {"__pyx_unpickle_FdEvent", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_5pycx4_5pycda_9__pyx_unpickle_FdEvent, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5pycx4_5pycda_9__pyx_unpickle_FdEvent(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_FdEvent (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_FdEvent", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_FdEvent", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_FdEvent") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_FdEvent", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pycx4.pycda.__pyx_unpickle_FdEvent", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pycx4_5pycda_8__pyx_unpickle_FdEvent(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_8__pyx_unpickle_FdEvent(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = NULL;
  PyObject *__pyx_v___pyx_result = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  __Pyx_RefNannySetupContext("__pyx_unpickle_FdEvent", 0);

  /* "(tree fragment)":2
 * def __pyx_unpickle_FdEvent(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0x183067a:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x183067a = (fhd, file, mask, ready))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x183067a) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":3
 * def __pyx_unpickle_FdEvent(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0x183067a:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x183067a = (fhd, file, mask, ready))" % __pyx_checksum)
 *     __pyx_result = FdEvent.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":4
 *     if __pyx_checksum != 0x183067a:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x183067a = (fhd, file, mask, ready))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = FdEvent.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 4, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x18, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 4, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 4, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 4, __pyx_L1_error)

    /* "(tree fragment)":2
 * def __pyx_unpickle_FdEvent(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0x183067a:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x183067a = (fhd, file, mask, ready))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":5
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x183067a = (fhd, file, mask, ready))" % __pyx_checksum)
 *     __pyx_result = FdEvent.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_FdEvent__set_state(<FdEvent> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_5pycx4_5pycda_FdEvent), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":6
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x183067a = (fhd, file, mask, ready))" % __pyx_checksum)
 *     __pyx_result = FdEvent.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_FdEvent__set_state(<FdEvent> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":7
 *     __pyx_result = FdEvent.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_FdEvent__set_state(<FdEvent> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_FdEvent__set_state(FdEvent __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 7, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_5pycx4_5pycda___pyx_unpickle_FdEvent__set_state(((struct __pyx_obj_5pycx4_5pycda_FdEvent *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x183067a = (fhd, file, mask, ready))" % __pyx_checksum)
 *     __pyx_result = FdEvent.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_FdEvent__set_state(<FdEvent> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":8
 *     if __pyx_state is not None:
 *         __pyx_unpickle_FdEvent__set_state(<FdEvent> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_FdEvent__set_state(FdEvent __pyx_result, tuple __pyx_state):
 *     __pyx_result.fhd = __pyx_state[0]; __pyx_result.file = __pyx_state[1]; __pyx_result.mask = __pyx_state[2]; __pyx_result.ready = __pyx_state[3]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_FdEvent(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0x183067a:
 *         from pickle import PickleError as __pyx_PickleError
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pycx4.pycda.__pyx_unpickle_FdEvent", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":9
 *         __pyx_unpickle_FdEvent__set_state(<FdEvent> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_FdEvent__set_state(FdEvent __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.fhd = __pyx_state[0]; __pyx_result.file = __pyx_state[1]; __pyx_result.mask = __pyx_state[2]; __pyx_result.ready = __pyx_state[3]
 *     if len(__pyx_state) > 4 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_5pycx4_5pycda___pyx_unpickle_FdEvent__set_state(struct __pyx_obj_5pycx4_5pycda_FdEvent *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  sl_fdh_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  __Pyx_RefNannySetupContext("__pyx_unpickle_FdEvent__set_state", 0);

  /* "(tree fragment)":10
 *     return __pyx_result
 * cdef __pyx_unpickle_FdEvent__set_state(FdEvent __pyx_result, tuple __pyx_state):
 *     __pyx_result.fhd = __pyx_state[0]; __pyx_result.file = __pyx_state[1]; __pyx_result.mask = __pyx_state[2]; __pyx_result.ready = __pyx_state[3]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 4 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[4])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyInt_As_sl_fdh_t(PyTuple_GET_ITEM(__pyx_v___pyx_state, 0)); if (unlikely((__pyx_t_1 == ((sl_fdh_t)-1)) && PyErr_Occurred())) __PYX_ERR(1, 10, __pyx_L1_error)
  __pyx_v___pyx_result->fhd = __pyx_t_1;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_2 = PyTuple_GET_ITEM(__pyx_v___pyx_state, 1);
  __Pyx_INCREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v___pyx_result->file);
  __Pyx_DECREF(__pyx_v___pyx_result->file);
  __pyx_v___pyx_result->file = __pyx_t_2;
  __pyx_t_2 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_3 = __Pyx_PyInt_As_int(PyTuple_GET_ITEM(__pyx_v___pyx_state, 2)); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 10, __pyx_L1_error)
  __pyx_v___pyx_result->mask = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_2 = PyTuple_GET_ITEM(__pyx_v___pyx_state, 3);
  __Pyx_INCREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v___pyx_result->ready);
  __Pyx_DECREF(__pyx_v___pyx_result->ready);
  __pyx_v___pyx_result->ready = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "(tree fragment)":11
 * cdef __pyx_unpickle_FdEvent__set_state(FdEvent __pyx_result, tuple __pyx_state):
 *     __pyx_result.fhd = __pyx_state[0]; __pyx_result.file = __pyx_state[1]; __pyx_result.mask = __pyx_state[2]; __pyx_result.ready = __pyx_state[3]
 *     if len(__pyx_state) > 4 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[4])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 11, __pyx_L1_error)
  }
  __pyx_t_5 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(1, 11, __pyx_L1_error)
  __pyx_t_6 = ((__pyx_t_5 > 4) != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_4 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(1, 11, __pyx_L1_error)
  __pyx_t_7 = (__pyx_t_6 != 0);
  __pyx_t_4 = __pyx_t_7;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_4) {

    /* "(tree fragment)":12
 *     __pyx_result.fhd = __pyx_state[0]; __pyx_result.file = __pyx_state[1]; __pyx_result.mask = __pyx_state[2]; __pyx_result.ready = __pyx_state[3]
 *     if len(__pyx_state) > 4 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[4])             # <<<<<<<<<<<<<<
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_update); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 12, __pyx_L1_error)
    }
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_9);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_9, function);
      }
    }
    __pyx_t_2 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_9, __pyx_t_8, PyTuple_GET_ITEM(__pyx_v___pyx_state, 4)) : __Pyx_PyObject_CallOneArg(__pyx_t_9, PyTuple_GET_ITEM(__pyx_v___pyx_state, 4));
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "(tree fragment)":11
 * cdef __pyx_unpickle_FdEvent__set_state(FdEvent __pyx_result, tuple __pyx_state):
 *     __pyx_result.fhd = __pyx_state[0]; __pyx_result.file = __pyx_state[1]; __pyx_result.mask = __pyx_state[2]; __pyx_result.ready = __pyx_state[3]
 *     if len(__pyx_state) > 4 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[4])
 */
  }

  /* "(tree fragment)":9
 *         __pyx_unpickle_FdEvent__set_state(<FdEvent> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_FdEvent__set_state(FdEvent __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.fhd = __pyx_state[0]; __pyx_result.file = __pyx_state[1]; __pyx_result.mask = __pyx_state[2]; __pyx_result.ready = __pyx_state[3]
 *     if len(__pyx_state) > 4 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("pycx4.pycda.__pyx_unpickle_FdEvent__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_ChanFactory(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0x57ffd47:
 *         from pickle import PickleError as __pyx_PickleError
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_11__pyx_unpickle_ChanFactory(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5pycx4_5pycda_11__pyx_unpickle_ChanFactory = {"__pyx_unpickle_ChanFactory", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_5pycx4_5pycda_11__pyx_unpickle_ChanFactory, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5pycx4_5pycda_11__pyx_unpickle_ChanFactory(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_ChanFactory (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ChanFactory", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ChanFactory", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_ChanFactory") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ChanFactory", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pycx4.pycda.__pyx_unpickle_ChanFactory", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pycx4_5pycda_10__pyx_unpickle_ChanFactory(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_10__pyx_unpickle_ChanFactory(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = NULL;
  PyObject *__pyx_v___pyx_result = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  __Pyx_RefNannySetupContext("__pyx_unpickle_ChanFactory", 0);

  /* "(tree fragment)":2
 * def __pyx_unpickle_ChanFactory(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0x57ffd47:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x57ffd47 = (single_chans, vector_chans))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x57ffd47) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":3
 * def __pyx_unpickle_ChanFactory(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0x57ffd47:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x57ffd47 = (single_chans, vector_chans))" % __pyx_checksum)
 *     __pyx_result = ChanFactory.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":4
 *     if __pyx_checksum != 0x57ffd47:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x57ffd47 = (single_chans, vector_chans))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = ChanFactory.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 4, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x57, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 4, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 4, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 4, __pyx_L1_error)

    /* "(tree fragment)":2
 * def __pyx_unpickle_ChanFactory(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0x57ffd47:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x57ffd47 = (single_chans, vector_chans))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":5
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x57ffd47 = (single_chans, vector_chans))" % __pyx_checksum)
 *     __pyx_result = ChanFactory.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ChanFactory__set_state(<ChanFactory> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_5pycx4_5pycda_ChanFactory), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":6
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x57ffd47 = (single_chans, vector_chans))" % __pyx_checksum)
 *     __pyx_result = ChanFactory.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_ChanFactory__set_state(<ChanFactory> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":7
 *     __pyx_result = ChanFactory.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ChanFactory__set_state(<ChanFactory> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_ChanFactory__set_state(ChanFactory __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 7, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_5pycx4_5pycda___pyx_unpickle_ChanFactory__set_state(((struct __pyx_obj_5pycx4_5pycda_ChanFactory *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x57ffd47 = (single_chans, vector_chans))" % __pyx_checksum)
 *     __pyx_result = ChanFactory.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_ChanFactory__set_state(<ChanFactory> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":8
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ChanFactory__set_state(<ChanFactory> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_ChanFactory__set_state(ChanFactory __pyx_result, tuple __pyx_state):
 *     __pyx_result.single_chans = __pyx_state[0]; __pyx_result.vector_chans = __pyx_state[1]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_ChanFactory(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0x57ffd47:
 *         from pickle import PickleError as __pyx_PickleError
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pycx4.pycda.__pyx_unpickle_ChanFactory", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":9
 *         __pyx_unpickle_ChanFactory__set_state(<ChanFactory> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_ChanFactory__set_state(ChanFactory __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.single_chans = __pyx_state[0]; __pyx_result.vector_chans = __pyx_state[1]
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_5pycx4_5pycda___pyx_unpickle_ChanFactory__set_state(struct __pyx_obj_5pycx4_5pycda_ChanFactory *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  __Pyx_RefNannySetupContext("__pyx_unpickle_ChanFactory__set_state", 0);

  /* "(tree fragment)":10
 *     return __pyx_result
 * cdef __pyx_unpickle_ChanFactory__set_state(ChanFactory __pyx_result, tuple __pyx_state):
 *     __pyx_result.single_chans = __pyx_state[0]; __pyx_result.vector_chans = __pyx_state[1]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[2])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  if (!(likely(PyDict_CheckExact(PyTuple_GET_ITEM(__pyx_v___pyx_state, 0)))||((PyTuple_GET_ITEM(__pyx_v___pyx_state, 0)) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(PyTuple_GET_ITEM(__pyx_v___pyx_state, 0))->tp_name), 0))) __PYX_ERR(1, 10, __pyx_L1_error)
  __pyx_t_1 = PyTuple_GET_ITEM(__pyx_v___pyx_state, 0);
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->single_chans);
  __Pyx_DECREF(__pyx_v___pyx_result->single_chans);
  __pyx_v___pyx_result->single_chans = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  if (!(likely(PyDict_CheckExact(PyTuple_GET_ITEM(__pyx_v___pyx_state, 1)))||((PyTuple_GET_ITEM(__pyx_v___pyx_state, 1)) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(PyTuple_GET_ITEM(__pyx_v___pyx_state, 1))->tp_name), 0))) __PYX_ERR(1, 10, __pyx_L1_error)
  __pyx_t_1 = PyTuple_GET_ITEM(__pyx_v___pyx_state, 1);
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->vector_chans);
  __Pyx_DECREF(__pyx_v___pyx_result->vector_chans);
  __pyx_v___pyx_result->vector_chans = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":11
 * cdef __pyx_unpickle_ChanFactory__set_state(ChanFactory __pyx_result, tuple __pyx_state):
 *     __pyx_result.single_chans = __pyx_state[0]; __pyx_result.vector_chans = __pyx_state[1]
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[2])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 11, __pyx_L1_error)
  }
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(1, 11, __pyx_L1_error)
  __pyx_t_4 = ((__pyx_t_3 > 2) != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 11, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_4 != 0);
  __pyx_t_2 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":12
 *     __pyx_result.single_chans = __pyx_state[0]; __pyx_result.vector_chans = __pyx_state[1]
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[2])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 12, __pyx_L1_error)
    }
    __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_1 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_6, PyTuple_GET_ITEM(__pyx_v___pyx_state, 2)) : __Pyx_PyObject_CallOneArg(__pyx_t_7, PyTuple_GET_ITEM(__pyx_v___pyx_state, 2));
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":11
 * cdef __pyx_unpickle_ChanFactory__set_state(ChanFactory __pyx_result, tuple __pyx_state):
 *     __pyx_result.single_chans = __pyx_state[0]; __pyx_result.vector_chans = __pyx_state[1]
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[2])
 */
  }

  /* "(tree fragment)":9
 *         __pyx_unpickle_ChanFactory__set_state(<ChanFactory> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_ChanFactory__set_state(ChanFactory __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.single_chans = __pyx_state[0]; __pyx_result.vector_chans = __pyx_state[1]
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("pycx4.pycda.__pyx_unpickle_ChanFactory__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_PassGW(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0x5a33863:
 *         from pickle import PickleError as __pyx_PickleError
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_13__pyx_unpickle_PassGW(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5pycx4_5pycda_13__pyx_unpickle_PassGW = {"__pyx_unpickle_PassGW", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_5pycx4_5pycda_13__pyx_unpickle_PassGW, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5pycx4_5pycda_13__pyx_unpickle_PassGW(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_PassGW (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_PassGW", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_PassGW", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_PassGW") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_PassGW", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pycx4.pycda.__pyx_unpickle_PassGW", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pycx4_5pycda_12__pyx_unpickle_PassGW(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_12__pyx_unpickle_PassGW(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = NULL;
  PyObject *__pyx_v___pyx_result = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  __Pyx_RefNannySetupContext("__pyx_unpickle_PassGW", 0);

  /* "(tree fragment)":2
 * def __pyx_unpickle_PassGW(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0x5a33863:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x5a33863 = (in_chan, last_val, out_chan))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x5a33863) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":3
 * def __pyx_unpickle_PassGW(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0x5a33863:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x5a33863 = (in_chan, last_val, out_chan))" % __pyx_checksum)
 *     __pyx_result = PassGW.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":4
 *     if __pyx_checksum != 0x5a33863:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x5a33863 = (in_chan, last_val, out_chan))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = PassGW.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 4, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x5a, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 4, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 4, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 4, __pyx_L1_error)

    /* "(tree fragment)":2
 * def __pyx_unpickle_PassGW(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0x5a33863:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x5a33863 = (in_chan, last_val, out_chan))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":5
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x5a33863 = (in_chan, last_val, out_chan))" % __pyx_checksum)
 *     __pyx_result = PassGW.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_PassGW__set_state(<PassGW> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_5pycx4_5pycda_PassGW), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":6
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x5a33863 = (in_chan, last_val, out_chan))" % __pyx_checksum)
 *     __pyx_result = PassGW.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_PassGW__set_state(<PassGW> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":7
 *     __pyx_result = PassGW.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_PassGW__set_state(<PassGW> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_PassGW__set_state(PassGW __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 7, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_5pycx4_5pycda___pyx_unpickle_PassGW__set_state(((struct __pyx_obj_5pycx4_5pycda_PassGW *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x5a33863 = (in_chan, last_val, out_chan))" % __pyx_checksum)
 *     __pyx_result = PassGW.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_PassGW__set_state(<PassGW> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":8
 *     if __pyx_state is not None:
 *         __pyx_unpickle_PassGW__set_state(<PassGW> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_PassGW__set_state(PassGW __pyx_result, tuple __pyx_state):
 *     __pyx_result.in_chan = __pyx_state[0]; __pyx_result.last_val = __pyx_state[1]; __pyx_result.out_chan = __pyx_state[2]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_PassGW(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0x5a33863:
 *         from pickle import PickleError as __pyx_PickleError
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pycx4.pycda.__pyx_unpickle_PassGW", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":9
 *         __pyx_unpickle_PassGW__set_state(<PassGW> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_PassGW__set_state(PassGW __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.in_chan = __pyx_state[0]; __pyx_result.last_val = __pyx_state[1]; __pyx_result.out_chan = __pyx_state[2]
 *     if len(__pyx_state) > 3 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_5pycx4_5pycda___pyx_unpickle_PassGW__set_state(struct __pyx_obj_5pycx4_5pycda_PassGW *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  __Pyx_RefNannySetupContext("__pyx_unpickle_PassGW__set_state", 0);

  /* "(tree fragment)":10
 *     return __pyx_result
 * cdef __pyx_unpickle_PassGW__set_state(PassGW __pyx_result, tuple __pyx_state):
 *     __pyx_result.in_chan = __pyx_state[0]; __pyx_result.last_val = __pyx_state[1]; __pyx_result.out_chan = __pyx_state[2]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 3 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[3])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = PyTuple_GET_ITEM(__pyx_v___pyx_state, 0);
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->in_chan);
  __Pyx_DECREF(__pyx_v___pyx_result->in_chan);
  __pyx_v___pyx_result->in_chan = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = PyTuple_GET_ITEM(__pyx_v___pyx_state, 1);
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->last_val);
  __Pyx_DECREF(__pyx_v___pyx_result->last_val);
  __pyx_v___pyx_result->last_val = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = PyTuple_GET_ITEM(__pyx_v___pyx_state, 2);
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->out_chan);
  __Pyx_DECREF(__pyx_v___pyx_result->out_chan);
  __pyx_v___pyx_result->out_chan = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":11
 * cdef __pyx_unpickle_PassGW__set_state(PassGW __pyx_result, tuple __pyx_state):
 *     __pyx_result.in_chan = __pyx_state[0]; __pyx_result.last_val = __pyx_state[1]; __pyx_result.out_chan = __pyx_state[2]
 *     if len(__pyx_state) > 3 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[3])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 11, __pyx_L1_error)
  }
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(1, 11, __pyx_L1_error)
  __pyx_t_4 = ((__pyx_t_3 > 3) != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 11, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_4 != 0);
  __pyx_t_2 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":12
 *     __pyx_result.in_chan = __pyx_state[0]; __pyx_result.last_val = __pyx_state[1]; __pyx_result.out_chan = __pyx_state[2]
 *     if len(__pyx_state) > 3 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[3])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 12, __pyx_L1_error)
    }
    __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_1 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_6, PyTuple_GET_ITEM(__pyx_v___pyx_state, 3)) : __Pyx_PyObject_CallOneArg(__pyx_t_7, PyTuple_GET_ITEM(__pyx_v___pyx_state, 3));
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":11
 * cdef __pyx_unpickle_PassGW__set_state(PassGW __pyx_result, tuple __pyx_state):
 *     __pyx_result.in_chan = __pyx_state[0]; __pyx_result.last_val = __pyx_state[1]; __pyx_result.out_chan = __pyx_state[2]
 *     if len(__pyx_state) > 3 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[3])
 */
  }

  /* "(tree fragment)":9
 *         __pyx_unpickle_PassGW__set_state(<PassGW> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_PassGW__set_state(PassGW __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.in_chan = __pyx_state[0]; __pyx_result.last_val = __pyx_state[1]; __pyx_result.out_chan = __pyx_state[2]
 *     if len(__pyx_state) > 3 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("pycx4.pycda.__pyx_unpickle_PassGW__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_ProcGW(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0x777971d:
 *         from pickle import PickleError as __pyx_PickleError
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_15__pyx_unpickle_ProcGW(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5pycx4_5pycda_15__pyx_unpickle_ProcGW = {"__pyx_unpickle_ProcGW", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_5pycx4_5pycda_15__pyx_unpickle_ProcGW, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5pycx4_5pycda_15__pyx_unpickle_ProcGW(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_ProcGW (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ProcGW", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ProcGW", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_ProcGW") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ProcGW", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pycx4.pycda.__pyx_unpickle_ProcGW", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pycx4_5pycda_14__pyx_unpickle_ProcGW(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_14__pyx_unpickle_ProcGW(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = NULL;
  PyObject *__pyx_v___pyx_result = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  __Pyx_RefNannySetupContext("__pyx_unpickle_ProcGW", 0);

  /* "(tree fragment)":2
 * def __pyx_unpickle_ProcGW(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0x777971d:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x777971d = (in_chan, out_chan))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x777971d) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":3
 * def __pyx_unpickle_ProcGW(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0x777971d:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x777971d = (in_chan, out_chan))" % __pyx_checksum)
 *     __pyx_result = ProcGW.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":4
 *     if __pyx_checksum != 0x777971d:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x777971d = (in_chan, out_chan))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = ProcGW.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 4, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x77, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 4, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 4, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 4, __pyx_L1_error)

    /* "(tree fragment)":2
 * def __pyx_unpickle_ProcGW(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0x777971d:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x777971d = (in_chan, out_chan))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":5
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x777971d = (in_chan, out_chan))" % __pyx_checksum)
 *     __pyx_result = ProcGW.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ProcGW__set_state(<ProcGW> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_5pycx4_5pycda_ProcGW), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":6
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x777971d = (in_chan, out_chan))" % __pyx_checksum)
 *     __pyx_result = ProcGW.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_ProcGW__set_state(<ProcGW> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":7
 *     __pyx_result = ProcGW.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ProcGW__set_state(<ProcGW> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_ProcGW__set_state(ProcGW __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 7, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_5pycx4_5pycda___pyx_unpickle_ProcGW__set_state(((struct __pyx_obj_5pycx4_5pycda_ProcGW *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x777971d = (in_chan, out_chan))" % __pyx_checksum)
 *     __pyx_result = ProcGW.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_ProcGW__set_state(<ProcGW> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":8
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ProcGW__set_state(<ProcGW> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_ProcGW__set_state(ProcGW __pyx_result, tuple __pyx_state):
 *     __pyx_result.in_chan = __pyx_state[0]; __pyx_result.out_chan = __pyx_state[1]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_ProcGW(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0x777971d:
 *         from pickle import PickleError as __pyx_PickleError
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pycx4.pycda.__pyx_unpickle_ProcGW", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":9
 *         __pyx_unpickle_ProcGW__set_state(<ProcGW> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_ProcGW__set_state(ProcGW __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.in_chan = __pyx_state[0]; __pyx_result.out_chan = __pyx_state[1]
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_5pycx4_5pycda___pyx_unpickle_ProcGW__set_state(struct __pyx_obj_5pycx4_5pycda_ProcGW *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  __Pyx_RefNannySetupContext("__pyx_unpickle_ProcGW__set_state", 0);

  /* "(tree fragment)":10
 *     return __pyx_result
 * cdef __pyx_unpickle_ProcGW__set_state(ProcGW __pyx_result, tuple __pyx_state):
 *     __pyx_result.in_chan = __pyx_state[0]; __pyx_result.out_chan = __pyx_state[1]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[2])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = PyTuple_GET_ITEM(__pyx_v___pyx_state, 0);
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->in_chan);
  __Pyx_DECREF(__pyx_v___pyx_result->in_chan);
  __pyx_v___pyx_result->in_chan = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = PyTuple_GET_ITEM(__pyx_v___pyx_state, 1);
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->out_chan);
  __Pyx_DECREF(__pyx_v___pyx_result->out_chan);
  __pyx_v___pyx_result->out_chan = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":11
 * cdef __pyx_unpickle_ProcGW__set_state(ProcGW __pyx_result, tuple __pyx_state):
 *     __pyx_result.in_chan = __pyx_state[0]; __pyx_result.out_chan = __pyx_state[1]
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[2])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 11, __pyx_L1_error)
  }
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(1, 11, __pyx_L1_error)
  __pyx_t_4 = ((__pyx_t_3 > 2) != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 11, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_4 != 0);
  __pyx_t_2 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":12
 *     __pyx_result.in_chan = __pyx_state[0]; __pyx_result.out_chan = __pyx_state[1]
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[2])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 12, __pyx_L1_error)
    }
    __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_1 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_6, PyTuple_GET_ITEM(__pyx_v___pyx_state, 2)) : __Pyx_PyObject_CallOneArg(__pyx_t_7, PyTuple_GET_ITEM(__pyx_v___pyx_state, 2));
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":11
 * cdef __pyx_unpickle_ProcGW__set_state(ProcGW __pyx_result, tuple __pyx_state):
 *     __pyx_result.in_chan = __pyx_state[0]; __pyx_result.out_chan = __pyx_state[1]
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[2])
 */
  }

  /* "(tree fragment)":9
 *         __pyx_unpickle_ProcGW__set_state(<ProcGW> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_ProcGW__set_state(ProcGW __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.in_chan = __pyx_state[0]; __pyx_result.out_chan = __pyx_state[1]
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("pycx4.pycda.__pyx_unpickle_ProcGW__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":258
 *         # experimental exception made for __getbuffer__ and __releasebuffer__
 *         # -- the details of this may change.
 *         def __getbuffer__(ndarray self, Py_buffer* info, int flags):             # <<<<<<<<<<<<<<
 *             # This implementation of getbuffer is geared towards Cython
 *             # requirements, and does not yet fulfill the PEP.
 */

/* Python wrapper */
static CYTHON_UNUSED int __pyx_pw_5numpy_7ndarray_1__getbuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /*proto*/
static CYTHON_UNUSED int __pyx_pw_5numpy_7ndarray_1__getbuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getbuffer__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5numpy_7ndarray___getbuffer__(((PyArrayObject *)__pyx_v_self), ((Py_buffer *)__pyx_v_info), ((int)__pyx_v_flags));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5numpy_7ndarray___getbuffer__(PyArrayObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {
  int __pyx_v_i;
  int __pyx_v_ndim;
  int __pyx_v_endian_detector;
  int __pyx_v_little_endian;
  int __pyx_v_t;
  char *__pyx_v_f;
  PyArray_Descr *__pyx_v_descr = 0;
  int __pyx_v_offset;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyArray_Descr *__pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  char *__pyx_t_9;
  if (__pyx_v_info == NULL) {
    PyErr_SetString(PyExc_BufferError, "PyObject_GetBuffer: view==NULL argument is obsolete");
    return -1;
  }
  __Pyx_RefNannySetupContext("__getbuffer__", 0);
  __pyx_v_info->obj = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(__pyx_v_info->obj);

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":265
 * 
 *             cdef int i, ndim
 *             cdef int endian_detector = 1             # <<<<<<<<<<<<<<
 *             cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)
 * 
 */
  __pyx_v_endian_detector = 1;

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":266
 *             cdef int i, ndim
 *             cdef int endian_detector = 1
 *             cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)             # <<<<<<<<<<<<<<
 * 
 *             ndim = PyArray_NDIM(self)
 */
  __pyx_v_little_endian = ((((char *)(&__pyx_v_endian_detector))[0]) != 0);

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":268
 *             cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)
 * 
 *             ndim = PyArray_NDIM(self)             # <<<<<<<<<<<<<<
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)
 */
  __pyx_v_ndim = PyArray_NDIM(__pyx_v_self);

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":270
 *             ndim = PyArray_NDIM(self)
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_ARRAY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")
 */
  __pyx_t_2 = (((__pyx_v_flags & PyBUF_C_CONTIGUOUS) == PyBUF_C_CONTIGUOUS) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":271
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_ARRAY_C_CONTIGUOUS)):             # <<<<<<<<<<<<<<
 *                 raise ValueError(u"ndarray is not C contiguous")
 * 
 */
  __pyx_t_2 = ((!(PyArray_CHKFLAGS(__pyx_v_self, NPY_ARRAY_C_CONTIGUOUS) != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":270
 *             ndim = PyArray_NDIM(self)
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_ARRAY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")
 */
  if (unlikely(__pyx_t_1)) {

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":272
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_ARRAY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")             # <<<<<<<<<<<<<<
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__35, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(17, 272, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(17, 272, __pyx_L1_error)

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":270
 *             ndim = PyArray_NDIM(self)
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_ARRAY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")
 */
  }

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":274
 *                 raise ValueError(u"ndarray is not C contiguous")
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_ARRAY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 */
  __pyx_t_2 = (((__pyx_v_flags & PyBUF_F_CONTIGUOUS) == PyBUF_F_CONTIGUOUS) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L7_bool_binop_done;
  }

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":275
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_ARRAY_F_CONTIGUOUS)):             # <<<<<<<<<<<<<<
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 * 
 */
  __pyx_t_2 = ((!(PyArray_CHKFLAGS(__pyx_v_self, NPY_ARRAY_F_CONTIGUOUS) != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L7_bool_binop_done:;

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":274
 *                 raise ValueError(u"ndarray is not C contiguous")
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_ARRAY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 */
  if (unlikely(__pyx_t_1)) {

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":276
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_ARRAY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")             # <<<<<<<<<<<<<<
 * 
 *             info.buf = PyArray_DATA(self)
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__36, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(17, 276, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(17, 276, __pyx_L1_error)

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":274
 *                 raise ValueError(u"ndarray is not C contiguous")
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_ARRAY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 */
  }

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":278
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 * 
 *             info.buf = PyArray_DATA(self)             # <<<<<<<<<<<<<<
 *             info.ndim = ndim
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 */
  __pyx_v_info->buf = PyArray_DATA(__pyx_v_self);

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":279
 * 
 *             info.buf = PyArray_DATA(self)
 *             info.ndim = ndim             # <<<<<<<<<<<<<<
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 *                 # Allocate new buffer for strides and shape info.
 */
  __pyx_v_info->ndim = __pyx_v_ndim;

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":280
 *             info.buf = PyArray_DATA(self)
 *             info.ndim = ndim
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):             # <<<<<<<<<<<<<<
 *                 # Allocate new buffer for strides and shape info.
 *                 # This is allocated as one block, strides first.
 */
  __pyx_t_1 = (((sizeof(npy_intp)) != (sizeof(Py_ssize_t))) != 0);
  if (__pyx_t_1) {

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":283
 *                 # Allocate new buffer for strides and shape info.
 *                 # This is allocated as one block, strides first.
 *                 info.strides = <Py_ssize_t*>PyObject_Malloc(sizeof(Py_ssize_t) * 2 * <size_t>ndim)             # <<<<<<<<<<<<<<
 *                 info.shape = info.strides + ndim
 *                 for i in range(ndim):
 */
    __pyx_v_info->strides = ((Py_ssize_t *)PyObject_Malloc((((sizeof(Py_ssize_t)) * 2) * ((size_t)__pyx_v_ndim))));

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":284
 *                 # This is allocated as one block, strides first.
 *                 info.strides = <Py_ssize_t*>PyObject_Malloc(sizeof(Py_ssize_t) * 2 * <size_t>ndim)
 *                 info.shape = info.strides + ndim             # <<<<<<<<<<<<<<
 *                 for i in range(ndim):
 *                     info.strides[i] = PyArray_STRIDES(self)[i]
 */
    __pyx_v_info->shape = (__pyx_v_info->strides + __pyx_v_ndim);

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":285
 *                 info.strides = <Py_ssize_t*>PyObject_Malloc(sizeof(Py_ssize_t) * 2 * <size_t>ndim)
 *                 info.shape = info.strides + ndim
 *                 for i in range(ndim):             # <<<<<<<<<<<<<<
 *                     info.strides[i] = PyArray_STRIDES(self)[i]
 *                     info.shape[i] = PyArray_DIMS(self)[i]
 */
    __pyx_t_4 = __pyx_v_ndim;
    __pyx_t_5 = __pyx_t_4;
    for (__pyx_t_6 = 0; __pyx_t_6 < __pyx_t_5; __pyx_t_6+=1) {
      __pyx_v_i = __pyx_t_6;

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":286
 *                 info.shape = info.strides + ndim
 *                 for i in range(ndim):
 *                     info.strides[i] = PyArray_STRIDES(self)[i]             # <<<<<<<<<<<<<<
 *                     info.shape[i] = PyArray_DIMS(self)[i]
 *             else:
 */
      (__pyx_v_info->strides[__pyx_v_i]) = (PyArray_STRIDES(__pyx_v_self)[__pyx_v_i]);

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":287
 *                 for i in range(ndim):
 *                     info.strides[i] = PyArray_STRIDES(self)[i]
 *                     info.shape[i] = PyArray_DIMS(self)[i]             # <<<<<<<<<<<<<<
 *             else:
 *                 info.strides = <Py_ssize_t*>PyArray_STRIDES(self)
 */
      (__pyx_v_info->shape[__pyx_v_i]) = (PyArray_DIMS(__pyx_v_self)[__pyx_v_i]);
    }

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":280
 *             info.buf = PyArray_DATA(self)
 *             info.ndim = ndim
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):             # <<<<<<<<<<<<<<
 *                 # Allocate new buffer for strides and shape info.
 *                 # This is allocated as one block, strides first.
 */
    goto __pyx_L9;
  }

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":289
 *                     info.shape[i] = PyArray_DIMS(self)[i]
 *             else:
 *                 info.strides = <Py_ssize_t*>PyArray_STRIDES(self)             # <<<<<<<<<<<<<<
 *                 info.shape = <Py_ssize_t*>PyArray_DIMS(self)
 *             info.suboffsets = NULL
 */
  /*else*/ {
    __pyx_v_info->strides = ((Py_ssize_t *)PyArray_STRIDES(__pyx_v_self));

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":290
 *             else:
 *                 info.strides = <Py_ssize_t*>PyArray_STRIDES(self)
 *                 info.shape = <Py_ssize_t*>PyArray_DIMS(self)             # <<<<<<<<<<<<<<
 *             info.suboffsets = NULL
 *             info.itemsize = PyArray_ITEMSIZE(self)
 */
    __pyx_v_info->shape = ((Py_ssize_t *)PyArray_DIMS(__pyx_v_self));
  }
  __pyx_L9:;

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":291
 *                 info.strides = <Py_ssize_t*>PyArray_STRIDES(self)
 *                 info.shape = <Py_ssize_t*>PyArray_DIMS(self)
 *             info.suboffsets = NULL             # <<<<<<<<<<<<<<
 *             info.itemsize = PyArray_ITEMSIZE(self)
 *             info.readonly = not PyArray_ISWRITEABLE(self)
 */
  __pyx_v_info->suboffsets = NULL;

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":292
 *                 info.shape = <Py_ssize_t*>PyArray_DIMS(self)
 *             info.suboffsets = NULL
 *             info.itemsize = PyArray_ITEMSIZE(self)             # <<<<<<<<<<<<<<
 *             info.readonly = not PyArray_ISWRITEABLE(self)
 * 
 */
  __pyx_v_info->itemsize = PyArray_ITEMSIZE(__pyx_v_self);

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":293
 *             info.suboffsets = NULL
 *             info.itemsize = PyArray_ITEMSIZE(self)
 *             info.readonly = not PyArray_ISWRITEABLE(self)             # <<<<<<<<<<<<<<
 * 
 *             cdef int t
 */
  __pyx_v_info->readonly = (!(PyArray_ISWRITEABLE(__pyx_v_self) != 0));

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":296
 * 
 *             cdef int t
 *             cdef char* f = NULL             # <<<<<<<<<<<<<<
 *             cdef dtype descr = <dtype>PyArray_DESCR(self)
 *             cdef int offset
 */
  __pyx_v_f = NULL;

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":297
 *             cdef int t
 *             cdef char* f = NULL
 *             cdef dtype descr = <dtype>PyArray_DESCR(self)             # <<<<<<<<<<<<<<
 *             cdef int offset
 * 
 */
  __pyx_t_7 = PyArray_DESCR(__pyx_v_self);
  __pyx_t_3 = ((PyObject *)__pyx_t_7);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_v_descr = ((PyArray_Descr *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":300
 *             cdef int offset
 * 
 *             info.obj = self             # <<<<<<<<<<<<<<
 * 
 *             if not PyDataType_HASFIELDS(descr):
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  __Pyx_GOTREF(__pyx_v_info->obj);
  __Pyx_DECREF(__pyx_v_info->obj);
  __pyx_v_info->obj = ((PyObject *)__pyx_v_self);

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":302
 *             info.obj = self
 * 
 *             if not PyDataType_HASFIELDS(descr):             # <<<<<<<<<<<<<<
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or
 */
  __pyx_t_1 = ((!(PyDataType_HASFIELDS(__pyx_v_descr) != 0)) != 0);
  if (__pyx_t_1) {

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":303
 * 
 *             if not PyDataType_HASFIELDS(descr):
 *                 t = descr.type_num             # <<<<<<<<<<<<<<
 *                 if ((descr.byteorder == c'>' and little_endian) or
 *                     (descr.byteorder == c'<' and not little_endian)):
 */
    __pyx_t_4 = __pyx_v_descr->type_num;
    __pyx_v_t = __pyx_t_4;

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":304
 *             if not PyDataType_HASFIELDS(descr):
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")
 */
    __pyx_t_2 = ((__pyx_v_descr->byteorder == '>') != 0);
    if (!__pyx_t_2) {
      goto __pyx_L15_next_or;
    } else {
    }
    __pyx_t_2 = (__pyx_v_little_endian != 0);
    if (!__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L14_bool_binop_done;
    }
    __pyx_L15_next_or:;

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":305
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or
 *                     (descr.byteorder == c'<' and not little_endian)):             # <<<<<<<<<<<<<<
 *                     raise ValueError(u"Non-native byte order not supported")
 *                 if   t == NPY_BYTE:        f = "b"
 */
    __pyx_t_2 = ((__pyx_v_descr->byteorder == '<') != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L14_bool_binop_done;
    }
    __pyx_t_2 = ((!(__pyx_v_little_endian != 0)) != 0);
    __pyx_t_1 = __pyx_t_2;
    __pyx_L14_bool_binop_done:;

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":304
 *             if not PyDataType_HASFIELDS(descr):
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")
 */
    if (unlikely(__pyx_t_1)) {

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":306
 *                 if ((descr.byteorder == c'>' and little_endian) or
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")             # <<<<<<<<<<<<<<
 *                 if   t == NPY_BYTE:        f = "b"
 *                 elif t == NPY_UBYTE:       f = "B"
 */
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__37, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(17, 306, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __PYX_ERR(17, 306, __pyx_L1_error)

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":304
 *             if not PyDataType_HASFIELDS(descr):
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")
 */
    }

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":307
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")
 *                 if   t == NPY_BYTE:        f = "b"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_UBYTE:       f = "B"
 *                 elif t == NPY_SHORT:       f = "h"
 */
    switch (__pyx_v_t) {
      case NPY_BYTE:
      __pyx_v_f = ((char *)"b");
      break;
      case NPY_UBYTE:

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":308
 *                     raise ValueError(u"Non-native byte order not supported")
 *                 if   t == NPY_BYTE:        f = "b"
 *                 elif t == NPY_UBYTE:       f = "B"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_SHORT:       f = "h"
 *                 elif t == NPY_USHORT:      f = "H"
 */
      __pyx_v_f = ((char *)"B");
      break;
      case NPY_SHORT:

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":309
 *                 if   t == NPY_BYTE:        f = "b"
 *                 elif t == NPY_UBYTE:       f = "B"
 *                 elif t == NPY_SHORT:       f = "h"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_USHORT:      f = "H"
 *                 elif t == NPY_INT:         f = "i"
 */
      __pyx_v_f = ((char *)"h");
      break;
      case NPY_USHORT:

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":310
 *                 elif t == NPY_UBYTE:       f = "B"
 *                 elif t == NPY_SHORT:       f = "h"
 *                 elif t == NPY_USHORT:      f = "H"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_INT:         f = "i"
 *                 elif t == NPY_UINT:        f = "I"
 */
      __pyx_v_f = ((char *)"H");
      break;
      case NPY_INT:

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":311
 *                 elif t == NPY_SHORT:       f = "h"
 *                 elif t == NPY_USHORT:      f = "H"
 *                 elif t == NPY_INT:         f = "i"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_UINT:        f = "I"
 *                 elif t == NPY_LONG:        f = "l"
 */
      __pyx_v_f = ((char *)"i");
      break;
      case NPY_UINT:

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":312
 *                 elif t == NPY_USHORT:      f = "H"
 *                 elif t == NPY_INT:         f = "i"
 *                 elif t == NPY_UINT:        f = "I"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_LONG:        f = "l"
 *                 elif t == NPY_ULONG:       f = "L"
 */
      __pyx_v_f = ((char *)"I");
      break;
      case NPY_LONG:

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":313
 *                 elif t == NPY_INT:         f = "i"
 *                 elif t == NPY_UINT:        f = "I"
 *                 elif t == NPY_LONG:        f = "l"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_ULONG:       f = "L"
 *                 elif t == NPY_LONGLONG:    f = "q"
 */
      __pyx_v_f = ((char *)"l");
      break;
      case NPY_ULONG:

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":314
 *                 elif t == NPY_UINT:        f = "I"
 *                 elif t == NPY_LONG:        f = "l"
 *                 elif t == NPY_ULONG:       f = "L"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_LONGLONG:    f = "q"
 *                 elif t == NPY_ULONGLONG:   f = "Q"
 */
      __pyx_v_f = ((char *)"L");
      break;
      case NPY_LONGLONG:

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":315
 *                 elif t == NPY_LONG:        f = "l"
 *                 elif t == NPY_ULONG:       f = "L"
 *                 elif t == NPY_LONGLONG:    f = "q"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_ULONGLONG:   f = "Q"
 *                 elif t == NPY_FLOAT:       f = "f"
 */
      __pyx_v_f = ((char *)"q");
      break;
      case NPY_ULONGLONG:

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":316
 *                 elif t == NPY_ULONG:       f = "L"
 *                 elif t == NPY_LONGLONG:    f = "q"
 *                 elif t == NPY_ULONGLONG:   f = "Q"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_FLOAT:       f = "f"
 *                 elif t == NPY_DOUBLE:      f = "d"
 */
      __pyx_v_f = ((char *)"Q");
      break;
      case NPY_FLOAT:

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":317
 *                 elif t == NPY_LONGLONG:    f = "q"
 *                 elif t == NPY_ULONGLONG:   f = "Q"
 *                 elif t == NPY_FLOAT:       f = "f"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_DOUBLE:      f = "d"
 *                 elif t == NPY_LONGDOUBLE:  f = "g"
 */
      __pyx_v_f = ((char *)"f");
      break;
      case NPY_DOUBLE:

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":318
 *                 elif t == NPY_ULONGLONG:   f = "Q"
 *                 elif t == NPY_FLOAT:       f = "f"
 *                 elif t == NPY_DOUBLE:      f = "d"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_LONGDOUBLE:  f = "g"
 *                 elif t == NPY_CFLOAT:      f = "Zf"
 */
      __pyx_v_f = ((char *)"d");
      break;
      case NPY_LONGDOUBLE:

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":319
 *                 elif t == NPY_FLOAT:       f = "f"
 *                 elif t == NPY_DOUBLE:      f = "d"
 *                 elif t == NPY_LONGDOUBLE:  f = "g"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_CFLOAT:      f = "Zf"
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 */
      __pyx_v_f = ((char *)"g");
      break;
      case NPY_CFLOAT:

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":320
 *                 elif t == NPY_DOUBLE:      f = "d"
 *                 elif t == NPY_LONGDOUBLE:  f = "g"
 *                 elif t == NPY_CFLOAT:      f = "Zf"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"
 */
      __pyx_v_f = ((char *)"Zf");
      break;
      case NPY_CDOUBLE:

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":321
 *                 elif t == NPY_LONGDOUBLE:  f = "g"
 *                 elif t == NPY_CFLOAT:      f = "Zf"
 *                 elif t == NPY_CDOUBLE:     f = "Zd"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"
 *                 elif t == NPY_OBJECT:      f = "O"
 */
      __pyx_v_f = ((char *)"Zd");
      break;
      case NPY_CLONGDOUBLE:

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":322
 *                 elif t == NPY_CFLOAT:      f = "Zf"
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_OBJECT:      f = "O"
 *                 else:
 */
      __pyx_v_f = ((char *)"Zg");
      break;
      case NPY_OBJECT:

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":323
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"
 *                 elif t == NPY_OBJECT:      f = "O"             # <<<<<<<<<<<<<<
 *                 else:
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 */
      __pyx_v_f = ((char *)"O");
      break;
      default:

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":325
 *                 elif t == NPY_OBJECT:      f = "O"
 *                 else:
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)             # <<<<<<<<<<<<<<
 *                 info.format = f
 *                 return
 */
      __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_t); if (unlikely(!__pyx_t_3)) __PYX_ERR(17, 325, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_8 = PyUnicode_Format(__pyx_kp_u_unknown_dtype_code_in_numpy_pxd, __pyx_t_3); if (unlikely(!__pyx_t_8)) __PYX_ERR(17, 325, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_8); if (unlikely(!__pyx_t_3)) __PYX_ERR(17, 325, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __PYX_ERR(17, 325, __pyx_L1_error)
      break;
    }

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":326
 *                 else:
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 *                 info.format = f             # <<<<<<<<<<<<<<
 *                 return
 *             else:
 */
    __pyx_v_info->format = __pyx_v_f;

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":327
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 *                 info.format = f
 *                 return             # <<<<<<<<<<<<<<
 *             else:
 *                 info.format = <char*>PyObject_Malloc(_buffer_format_string_len)
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":302
 *             info.obj = self
 * 
 *             if not PyDataType_HASFIELDS(descr):             # <<<<<<<<<<<<<<
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or
 */
  }

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":329
 *                 return
 *             else:
 *                 info.format = <char*>PyObject_Malloc(_buffer_format_string_len)             # <<<<<<<<<<<<<<
 *                 info.format[0] = c'^' # Native data types, manual alignment
 *                 offset = 0
 */
  /*else*/ {
    __pyx_v_info->format = ((char *)PyObject_Malloc(0xFF));

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":330
 *             else:
 *                 info.format = <char*>PyObject_Malloc(_buffer_format_string_len)
 *                 info.format[0] = c'^' # Native data types, manual alignment             # <<<<<<<<<<<<<<
 *                 offset = 0
 *                 f = _util_dtypestring(descr, info.format + 1,
 */
    (__pyx_v_info->format[0]) = '^';

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":331
 *                 info.format = <char*>PyObject_Malloc(_buffer_format_string_len)
 *                 info.format[0] = c'^' # Native data types, manual alignment
 *                 offset = 0             # <<<<<<<<<<<<<<
 *                 f = _util_dtypestring(descr, info.format + 1,
 *                                       info.format + _buffer_format_string_len,
 */
    __pyx_v_offset = 0;

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":332
 *                 info.format[0] = c'^' # Native data types, manual alignment
 *                 offset = 0
 *                 f = _util_dtypestring(descr, info.format + 1,             # <<<<<<<<<<<<<<
 *                                       info.format + _buffer_format_string_len,
 *                                       &offset)
 */
    __pyx_t_9 = __pyx_f_5numpy__util_dtypestring(__pyx_v_descr, (__pyx_v_info->format + 1), (__pyx_v_info->format + 0xFF), (&__pyx_v_offset)); if (unlikely(__pyx_t_9 == ((char *)NULL))) __PYX_ERR(17, 332, __pyx_L1_error)
    __pyx_v_f = __pyx_t_9;

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":335
 *                                       info.format + _buffer_format_string_len,
 *                                       &offset)
 *                 f[0] = c'\0' # Terminate format string             # <<<<<<<<<<<<<<
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):
 */
    (__pyx_v_f[0]) = '\x00';
  }

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":258
 *         # experimental exception made for __getbuffer__ and __releasebuffer__
 *         # -- the details of this may change.
 *         def __getbuffer__(ndarray self, Py_buffer* info, int flags):             # <<<<<<<<<<<<<<
 *             # This implementation of getbuffer is geared towards Cython
 *             # requirements, and does not yet fulfill the PEP.
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.ndarray.__getbuffer__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  if (__pyx_v_info->obj != NULL) {
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj); __pyx_v_info->obj = 0;
  }
  goto __pyx_L2;
  __pyx_L0:;
  if (__pyx_v_info->obj == Py_None) {
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj); __pyx_v_info->obj = 0;
  }
  __pyx_L2:;
  __Pyx_XDECREF((PyObject *)__pyx_v_descr);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":337
 *                 f[0] = c'\0' # Terminate format string
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):             # <<<<<<<<<<<<<<
 *             if PyArray_HASFIELDS(self):
 *                 PyObject_Free(info.format)
 */

/* Python wrapper */
static CYTHON_UNUSED void __pyx_pw_5numpy_7ndarray_3__releasebuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info); /*proto*/
static CYTHON_UNUSED void __pyx_pw_5numpy_7ndarray_3__releasebuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__releasebuffer__ (wrapper)", 0);
  __pyx_pf_5numpy_7ndarray_2__releasebuffer__(((PyArrayObject *)__pyx_v_self), ((Py_buffer *)__pyx_v_info));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_5numpy_7ndarray_2__releasebuffer__(PyArrayObject *__pyx_v_self, Py_buffer *__pyx_v_info) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__releasebuffer__", 0);

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":338
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):
 *             if PyArray_HASFIELDS(self):             # <<<<<<<<<<<<<<
 *                 PyObject_Free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 */
  __pyx_t_1 = (PyArray_HASFIELDS(__pyx_v_self) != 0);
  if (__pyx_t_1) {

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":339
 *         def __releasebuffer__(ndarray self, Py_buffer* info):
 *             if PyArray_HASFIELDS(self):
 *                 PyObject_Free(info.format)             # <<<<<<<<<<<<<<
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 *                 PyObject_Free(info.strides)
 */
    PyObject_Free(__pyx_v_info->format);

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":338
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):
 *             if PyArray_HASFIELDS(self):             # <<<<<<<<<<<<<<
 *                 PyObject_Free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 */
  }

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":340
 *             if PyArray_HASFIELDS(self):
 *                 PyObject_Free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):             # <<<<<<<<<<<<<<
 *                 PyObject_Free(info.strides)
 *                 # info.shape was stored after info.strides in the same block
 */
  __pyx_t_1 = (((sizeof(npy_intp)) != (sizeof(Py_ssize_t))) != 0);
  if (__pyx_t_1) {

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":341
 *                 PyObject_Free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 *                 PyObject_Free(info.strides)             # <<<<<<<<<<<<<<
 *                 # info.shape was stored after info.strides in the same block
 * 
 */
    PyObject_Free(__pyx_v_info->strides);

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":340
 *             if PyArray_HASFIELDS(self):
 *                 PyObject_Free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):             # <<<<<<<<<<<<<<
 *                 PyObject_Free(info.strides)
 *                 # info.shape was stored after info.strides in the same block
 */
  }

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":337
 *                 f[0] = c'\0' # Terminate format string
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):             # <<<<<<<<<<<<<<
 *             if PyArray_HASFIELDS(self):
 *                 PyObject_Free(info.format)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":821
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew1(PyObject *__pyx_v_a) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew1", 0);

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":822
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 *     return PyArray_MultiIterNew(1, <void*>a)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(1, ((void *)__pyx_v_a)); if (unlikely(!__pyx_t_1)) __PYX_ERR(17, 822, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":821
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew1", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":824
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew2(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew2", 0);

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":825
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(2, ((void *)__pyx_v_a), ((void *)__pyx_v_b)); if (unlikely(!__pyx_t_1)) __PYX_ERR(17, 825, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":824
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew2", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":827
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew3(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew3", 0);

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":828
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(3, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c)); if (unlikely(!__pyx_t_1)) __PYX_ERR(17, 828, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":827
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew3", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":830
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew4(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew4", 0);

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":831
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(4, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d)); if (unlikely(!__pyx_t_1)) __PYX_ERR(17, 831, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":830
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew4", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":833
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew5(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d, PyObject *__pyx_v_e) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew5", 0);

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":834
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)             # <<<<<<<<<<<<<<
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(5, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d), ((void *)__pyx_v_e)); if (unlikely(!__pyx_t_1)) __PYX_ERR(17, 834, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":833
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew5", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":836
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):             # <<<<<<<<<<<<<<
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyDataType_SHAPE(PyArray_Descr *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("PyDataType_SHAPE", 0);

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":837
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):             # <<<<<<<<<<<<<<
 *         return <tuple>d.subarray.shape
 *     else:
 */
  __pyx_t_1 = (PyDataType_HASSUBARRAY(__pyx_v_d) != 0);
  if (__pyx_t_1) {

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":838
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape             # <<<<<<<<<<<<<<
 *     else:
 *         return ()
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject*)__pyx_v_d->subarray->shape));
    __pyx_r = ((PyObject*)__pyx_v_d->subarray->shape);
    goto __pyx_L0;

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":837
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):             # <<<<<<<<<<<<<<
 *         return <tuple>d.subarray.shape
 *     else:
 */
  }

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":840
 *         return <tuple>d.subarray.shape
 *     else:
 *         return ()             # <<<<<<<<<<<<<<
 * 
 * cdef inline char* _util_dtypestring(dtype descr, char* f, char* end, int* offset) except NULL:
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_empty_tuple);
    __pyx_r = __pyx_empty_tuple;
    goto __pyx_L0;
  }

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":836
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):             # <<<<<<<<<<<<<<
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":842
 *         return ()
 * 
 * cdef inline char* _util_dtypestring(dtype descr, char* f, char* end, int* offset) except NULL:             # <<<<<<<<<<<<<<
 *     # Recursive utility function used in __getbuffer__ to get format
 *     # string. The new location in the format string is returned.
 */

static CYTHON_INLINE char *__pyx_f_5numpy__util_dtypestring(PyArray_Descr *__pyx_v_descr, char *__pyx_v_f, char *__pyx_v_end, int *__pyx_v_offset) {
  PyArray_Descr *__pyx_v_child = 0;
  int __pyx_v_endian_detector;
  int __pyx_v_little_endian;
  PyObject *__pyx_v_fields = 0;
  PyObject *__pyx_v_childname = NULL;
  PyObject *__pyx_v_new_offset = NULL;
  PyObject *__pyx_v_t = NULL;
  char *__pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  long __pyx_t_8;
  char *__pyx_t_9;
  __Pyx_RefNannySetupContext("_util_dtypestring", 0);

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":847
 * 
 *     cdef dtype child
 *     cdef int endian_detector = 1             # <<<<<<<<<<<<<<
 *     cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)
 *     cdef tuple fields
 */
  __pyx_v_endian_detector = 1;

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":848
 *     cdef dtype child
 *     cdef int endian_detector = 1
 *     cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)             # <<<<<<<<<<<<<<
 *     cdef tuple fields
 * 
 */
  __pyx_v_little_endian = ((((char *)(&__pyx_v_endian_detector))[0]) != 0);

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":851
 *     cdef tuple fields
 * 
 *     for childname in descr.names:             # <<<<<<<<<<<<<<
 *         fields = descr.fields[childname]
 *         child, new_offset = fields
 */
  if (unlikely(__pyx_v_descr->names == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(17, 851, __pyx_L1_error)
  }
  __pyx_t_1 = __pyx_v_descr->names; __Pyx_INCREF(__pyx_t_1); __pyx_t_2 = 0;
  for (;;) {
    if (__pyx_t_2 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_3); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(17, 851, __pyx_L1_error)
    #else
    __pyx_t_3 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_3)) __PYX_ERR(17, 851, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_childname, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":852
 * 
 *     for childname in descr.names:
 *         fields = descr.fields[childname]             # <<<<<<<<<<<<<<
 *         child, new_offset = fields
 * 
 */
    if (unlikely(__pyx_v_descr->fields == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(17, 852, __pyx_L1_error)
    }
    __pyx_t_3 = __Pyx_PyDict_GetItem(__pyx_v_descr->fields, __pyx_v_childname); if (unlikely(!__pyx_t_3)) __PYX_ERR(17, 852, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(PyTuple_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_t_3)->tp_name), 0))) __PYX_ERR(17, 852, __pyx_L1_error)
    __Pyx_XDECREF_SET(__pyx_v_fields, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":853
 *     for childname in descr.names:
 *         fields = descr.fields[childname]
 *         child, new_offset = fields             # <<<<<<<<<<<<<<
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:
 */
    if (likely(__pyx_v_fields != Py_None)) {
      PyObject* sequence = __pyx_v_fields;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(17, 853, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_4 = PyTuple_GET_ITEM(sequence, 1); 
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      #else
      __pyx_t_3 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(17, 853, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(17, 853, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      #endif
    } else {
      __Pyx_RaiseNoneNotIterableError(); __PYX_ERR(17, 853, __pyx_L1_error)
    }
    if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5numpy_dtype))))) __PYX_ERR(17, 853, __pyx_L1_error)
    __Pyx_XDECREF_SET(__pyx_v_child, ((PyArray_Descr *)__pyx_t_3));
    __pyx_t_3 = 0;
    __Pyx_XDECREF_SET(__pyx_v_new_offset, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":855
 *         child, new_offset = fields
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:             # <<<<<<<<<<<<<<
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 */
    __pyx_t_4 = __Pyx_PyInt_From_int((__pyx_v_offset[0])); if (unlikely(!__pyx_t_4)) __PYX_ERR(17, 855, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = PyNumber_Subtract(__pyx_v_new_offset, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(17, 855, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(17, 855, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_6 = ((((__pyx_v_end - __pyx_v_f) - ((int)__pyx_t_5)) < 15) != 0);
    if (unlikely(__pyx_t_6)) {

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":856
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")             # <<<<<<<<<<<<<<
 * 
 *         if ((child.byteorder == c'>' and little_endian) or
 */
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__38, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(17, 856, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __PYX_ERR(17, 856, __pyx_L1_error)

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":855
 *         child, new_offset = fields
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:             # <<<<<<<<<<<<<<
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 */
    }

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":858
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 *         if ((child.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")
 */
    __pyx_t_7 = ((__pyx_v_child->byteorder == '>') != 0);
    if (!__pyx_t_7) {
      goto __pyx_L8_next_or;
    } else {
    }
    __pyx_t_7 = (__pyx_v_little_endian != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_L8_next_or:;

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":859
 * 
 *         if ((child.byteorder == c'>' and little_endian) or
 *             (child.byteorder == c'<' and not little_endian)):             # <<<<<<<<<<<<<<
 *             raise ValueError(u"Non-native byte order not supported")
 *             # One could encode it in the format string and have Cython
 */
    __pyx_t_7 = ((__pyx_v_child->byteorder == '<') != 0);
    if (__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_t_7 = ((!(__pyx_v_little_endian != 0)) != 0);
    __pyx_t_6 = __pyx_t_7;
    __pyx_L7_bool_binop_done:;

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":858
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 *         if ((child.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")
 */
    if (unlikely(__pyx_t_6)) {

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":860
 *         if ((child.byteorder == c'>' and little_endian) or
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")             # <<<<<<<<<<<<<<
 *             # One could encode it in the format string and have Cython
 *             # complain instead, BUT: < and > in format strings also imply
 */
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__37, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(17, 860, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __PYX_ERR(17, 860, __pyx_L1_error)

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":858
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 *         if ((child.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")
 */
    }

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":870
 * 
 *         # Output padding bytes
 *         while offset[0] < new_offset:             # <<<<<<<<<<<<<<
 *             f[0] = 120 # "x"; pad byte
 *             f += 1
 */
    while (1) {
      __pyx_t_3 = __Pyx_PyInt_From_int((__pyx_v_offset[0])); if (unlikely(!__pyx_t_3)) __PYX_ERR(17, 870, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_t_3, __pyx_v_new_offset, Py_LT); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(17, 870, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(17, 870, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (!__pyx_t_6) break;

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":871
 *         # Output padding bytes
 *         while offset[0] < new_offset:
 *             f[0] = 120 # "x"; pad byte             # <<<<<<<<<<<<<<
 *             f += 1
 *             offset[0] += 1
 */
      (__pyx_v_f[0]) = 0x78;

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":872
 *         while offset[0] < new_offset:
 *             f[0] = 120 # "x"; pad byte
 *             f += 1             # <<<<<<<<<<<<<<
 *             offset[0] += 1
 * 
 */
      __pyx_v_f = (__pyx_v_f + 1);

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":873
 *             f[0] = 120 # "x"; pad byte
 *             f += 1
 *             offset[0] += 1             # <<<<<<<<<<<<<<
 * 
 *         offset[0] += child.itemsize
 */
      __pyx_t_8 = 0;
      (__pyx_v_offset[__pyx_t_8]) = ((__pyx_v_offset[__pyx_t_8]) + 1);
    }

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":875
 *             offset[0] += 1
 * 
 *         offset[0] += child.itemsize             # <<<<<<<<<<<<<<
 * 
 *         if not PyDataType_HASFIELDS(child):
 */
    __pyx_t_8 = 0;
    (__pyx_v_offset[__pyx_t_8]) = ((__pyx_v_offset[__pyx_t_8]) + __pyx_v_child->elsize);

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":877
 *         offset[0] += child.itemsize
 * 
 *         if not PyDataType_HASFIELDS(child):             # <<<<<<<<<<<<<<
 *             t = child.type_num
 *             if end - f < 5:
 */
    __pyx_t_6 = ((!(PyDataType_HASFIELDS(__pyx_v_child) != 0)) != 0);
    if (__pyx_t_6) {

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":878
 * 
 *         if not PyDataType_HASFIELDS(child):
 *             t = child.type_num             # <<<<<<<<<<<<<<
 *             if end - f < 5:
 *                 raise RuntimeError(u"Format string allocated too short.")
 */
      __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_child->type_num); if (unlikely(!__pyx_t_4)) __PYX_ERR(17, 878, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_XDECREF_SET(__pyx_v_t, __pyx_t_4);
      __pyx_t_4 = 0;

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":879
 *         if not PyDataType_HASFIELDS(child):
 *             t = child.type_num
 *             if end - f < 5:             # <<<<<<<<<<<<<<
 *                 raise RuntimeError(u"Format string allocated too short.")
 * 
 */
      __pyx_t_6 = (((__pyx_v_end - __pyx_v_f) < 5) != 0);
      if (unlikely(__pyx_t_6)) {

        /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":880
 *             t = child.type_num
 *             if end - f < 5:
 *                 raise RuntimeError(u"Format string allocated too short.")             # <<<<<<<<<<<<<<
 * 
 *             # Until ticket #99 is fixed, use integers to avoid warnings
 */
        __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__39, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(17, 880, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_Raise(__pyx_t_4, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __PYX_ERR(17, 880, __pyx_L1_error)

        /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":879
 *         if not PyDataType_HASFIELDS(child):
 *             t = child.type_num
 *             if end - f < 5:             # <<<<<<<<<<<<<<
 *                 raise RuntimeError(u"Format string allocated too short.")
 * 
 */
      }

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":883
 * 
 *             # Until ticket #99 is fixed, use integers to avoid warnings
 *             if   t == NPY_BYTE:        f[0] =  98 #"b"             # <<<<<<<<<<<<<<
 *             elif t == NPY_UBYTE:       f[0] =  66 #"B"
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_BYTE); if (unlikely(!__pyx_t_4)) __PYX_ERR(17, 883, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(17, 883, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(17, 883, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 98;
        goto __pyx_L15;
      }

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":884
 *             # Until ticket #99 is fixed, use integers to avoid warnings
 *             if   t == NPY_BYTE:        f[0] =  98 #"b"
 *             elif t == NPY_UBYTE:       f[0] =  66 #"B"             # <<<<<<<<<<<<<<
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_UBYTE); if (unlikely(!__pyx_t_3)) __PYX_ERR(17, 884, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(17, 884, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(17, 884, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 66;
        goto __pyx_L15;
      }

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":885
 *             if   t == NPY_BYTE:        f[0] =  98 #"b"
 *             elif t == NPY_UBYTE:       f[0] =  66 #"B"
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"             # <<<<<<<<<<<<<<
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"
 *             elif t == NPY_INT:         f[0] = 105 #"i"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_SHORT); if (unlikely(!__pyx_t_4)) __PYX_ERR(17, 885, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(17, 885, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(17, 885, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x68;
        goto __pyx_L15;
      }

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":886
 *             elif t == NPY_UBYTE:       f[0] =  66 #"B"
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"             # <<<<<<<<<<<<<<
 *             elif t == NPY_INT:         f[0] = 105 #"i"
 *             elif t == NPY_UINT:        f[0] =  73 #"I"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_USHORT); if (unlikely(!__pyx_t_3)) __PYX_ERR(17, 886, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(17, 886, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(17, 886, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 72;
        goto __pyx_L15;
      }

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":887
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"
 *             elif t == NPY_INT:         f[0] = 105 #"i"             # <<<<<<<<<<<<<<
 *             elif t == NPY_UINT:        f[0] =  73 #"I"
 *             elif t == NPY_LONG:        f[0] = 108 #"l"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_INT); if (unlikely(!__pyx_t_4)) __PYX_ERR(17, 887, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(17, 887, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(17, 887, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x69;
        goto __pyx_L15;
      }

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":888
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"
 *             elif t == NPY_INT:         f[0] = 105 #"i"
 *             elif t == NPY_UINT:        f[0] =  73 #"I"             # <<<<<<<<<<<<<<
 *             elif t == NPY_LONG:        f[0] = 108 #"l"
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_UINT); if (unlikely(!__pyx_t_3)) __PYX_ERR(17, 888, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(17, 888, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(17, 888, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 73;
        goto __pyx_L15;
      }

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":889
 *             elif t == NPY_INT:         f[0] = 105 #"i"
 *             elif t == NPY_UINT:        f[0] =  73 #"I"
 *             elif t == NPY_LONG:        f[0] = 108 #"l"             # <<<<<<<<<<<<<<
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_LONG); if (unlikely(!__pyx_t_4)) __PYX_ERR(17, 889, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(17, 889, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(17, 889, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x6C;
        goto __pyx_L15;
      }

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":890
 *             elif t == NPY_UINT:        f[0] =  73 #"I"
 *             elif t == NPY_LONG:        f[0] = 108 #"l"
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"             # <<<<<<<<<<<<<<
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_ULONG); if (unlikely(!__pyx_t_3)) __PYX_ERR(17, 890, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(17, 890, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(17, 890, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 76;
        goto __pyx_L15;
      }

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":891
 *             elif t == NPY_LONG:        f[0] = 108 #"l"
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"             # <<<<<<<<<<<<<<
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_LONGLONG); if (unlikely(!__pyx_t_4)) __PYX_ERR(17, 891, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(17, 891, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(17, 891, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x71;
        goto __pyx_L15;
      }

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":892
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"             # <<<<<<<<<<<<<<
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_ULONGLONG); if (unlikely(!__pyx_t_3)) __PYX_ERR(17, 892, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(17, 892, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(17, 892, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 81;
        goto __pyx_L15;
      }

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":893
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"             # <<<<<<<<<<<<<<
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_FLOAT); if (unlikely(!__pyx_t_4)) __PYX_ERR(17, 893, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(17, 893, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(17, 893, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x66;
        goto __pyx_L15;
      }

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":894
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"             # <<<<<<<<<<<<<<
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_DOUBLE); if (unlikely(!__pyx_t_3)) __PYX_ERR(17, 894, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(17, 894, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(17, 894, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x64;
        goto __pyx_L15;
      }

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":895
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"             # <<<<<<<<<<<<<<
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_LONGDOUBLE); if (unlikely(!__pyx_t_4)) __PYX_ERR(17, 895, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(17, 895, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(17, 895, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x67;
        goto __pyx_L15;
      }

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":896
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf             # <<<<<<<<<<<<<<
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd
 *             elif t == NPY_CLONGDOUBLE: f[0] = 90; f[1] = 103; f += 1 # Zg
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_CFLOAT); if (unlikely(!__pyx_t_3)) __PYX_ERR(17, 896, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(17, 896, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(17, 896, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 90;
        (__pyx_v_f[1]) = 0x66;
        __pyx_v_f = (__pyx_v_f + 1);
        goto __pyx_L15;
      }

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":897
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd             # <<<<<<<<<<<<<<
 *             elif t == NPY_CLONGDOUBLE: f[0] = 90; f[1] = 103; f += 1 # Zg
 *             elif t == NPY_OBJECT:      f[0] = 79 #"O"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_CDOUBLE); if (unlikely(!__pyx_t_4)) __PYX_ERR(17, 897, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(17, 897, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(17, 897, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 90;
        (__pyx_v_f[1]) = 0x64;
        __pyx_v_f = (__pyx_v_f + 1);
        goto __pyx_L15;
      }

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":898
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd
 *             elif t == NPY_CLONGDOUBLE: f[0] = 90; f[1] = 103; f += 1 # Zg             # <<<<<<<<<<<<<<
 *             elif t == NPY_OBJECT:      f[0] = 79 #"O"
 *             else:
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_CLONGDOUBLE); if (unlikely(!__pyx_t_3)) __PYX_ERR(17, 898, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(17, 898, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(17, 898, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 90;
        (__pyx_v_f[1]) = 0x67;
        __pyx_v_f = (__pyx_v_f + 1);
        goto __pyx_L15;
      }

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":899
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd
 *             elif t == NPY_CLONGDOUBLE: f[0] = 90; f[1] = 103; f += 1 # Zg
 *             elif t == NPY_OBJECT:      f[0] = 79 #"O"             # <<<<<<<<<<<<<<
 *             else:
 *                 raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_OBJECT); if (unlikely(!__pyx_t_4)) __PYX_ERR(17, 899, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(17, 899, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(17, 899, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (likely(__pyx_t_6)) {
        (__pyx_v_f[0]) = 79;
        goto __pyx_L15;
      }

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":901
 *             elif t == NPY_OBJECT:      f[0] = 79 #"O"
 *             else:
 *                 raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)             # <<<<<<<<<<<<<<
 *             f += 1
 *         else:
 */
      /*else*/ {
        __pyx_t_3 = __Pyx_PyUnicode_FormatSafe(__pyx_kp_u_unknown_dtype_code_in_numpy_pxd, __pyx_v_t); if (unlikely(!__pyx_t_3)) __PYX_ERR(17, 901, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(17, 901, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_Raise(__pyx_t_4, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __PYX_ERR(17, 901, __pyx_L1_error)
      }
      __pyx_L15:;

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":902
 *             else:
 *                 raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 *             f += 1             # <<<<<<<<<<<<<<
 *         else:
 *             # Cython ignores struct boundary information ("T{...}"),
 */
      __pyx_v_f = (__pyx_v_f + 1);

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":877
 *         offset[0] += child.itemsize
 * 
 *         if not PyDataType_HASFIELDS(child):             # <<<<<<<<<<<<<<
 *             t = child.type_num
 *             if end - f < 5:
 */
      goto __pyx_L13;
    }

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":906
 *             # Cython ignores struct boundary information ("T{...}"),
 *             # so don't output it
 *             f = _util_dtypestring(child, f, end, offset)             # <<<<<<<<<<<<<<
 *     return f
 * 
 */
    /*else*/ {
      __pyx_t_9 = __pyx_f_5numpy__util_dtypestring(__pyx_v_child, __pyx_v_f, __pyx_v_end, __pyx_v_offset); if (unlikely(__pyx_t_9 == ((char *)NULL))) __PYX_ERR(17, 906, __pyx_L1_error)
      __pyx_v_f = __pyx_t_9;
    }
    __pyx_L13:;

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":851
 *     cdef tuple fields
 * 
 *     for childname in descr.names:             # <<<<<<<<<<<<<<
 *         fields = descr.fields[childname]
 *         child, new_offset = fields
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":907
 *             # so don't output it
 *             f = _util_dtypestring(child, f, end, offset)
 *     return f             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_f;
  goto __pyx_L0;

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":842
 *         return ()
 * 
 * cdef inline char* _util_dtypestring(dtype descr, char* f, char* end, int* offset) except NULL:             # <<<<<<<<<<<<<<
 *     # Recursive utility function used in __getbuffer__ to get format
 *     # string. The new location in the format string is returned.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("numpy._util_dtypestring", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_child);
  __Pyx_XDECREF(__pyx_v_fields);
  __Pyx_XDECREF(__pyx_v_childname);
  __Pyx_XDECREF(__pyx_v_new_offset);
  __Pyx_XDECREF(__pyx_v_t);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1022
 *     int _import_umath() except -1
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)
 */

static CYTHON_INLINE void __pyx_f_5numpy_set_array_base(PyArrayObject *__pyx_v_arr, PyObject *__pyx_v_base) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_array_base", 0);

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1023
 * 
 * cdef inline void set_array_base(ndarray arr, object base):
 *     Py_INCREF(base) # important to do this before stealing the reference below!             # <<<<<<<<<<<<<<
 *     PyArray_SetBaseObject(arr, base)
 * 
 */
  Py_INCREF(__pyx_v_base);

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1024
 * cdef inline void set_array_base(ndarray arr, object base):
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object get_array_base(ndarray arr):
 */
  (void)(PyArray_SetBaseObject(__pyx_v_arr, __pyx_v_base));

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1022
 *     int _import_umath() except -1
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1026
 *     PyArray_SetBaseObject(arr, base)
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_get_array_base(PyArrayObject *__pyx_v_arr) {
  PyObject *__pyx_v_base;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("get_array_base", 0);

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1027
 * 
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)             # <<<<<<<<<<<<<<
 *     if base is NULL:
 *         return None
 */
  __pyx_v_base = PyArray_BASE(__pyx_v_arr);

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1028
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)
 *     if base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     return <object>base
 */
  __pyx_t_1 = ((__pyx_v_base == NULL) != 0);
  if (__pyx_t_1) {

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1029
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 *         return None             # <<<<<<<<<<<<<<
 *     return <object>base
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1028
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)
 *     if base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     return <object>base
 */
  }

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1030
 *     if base is NULL:
 *         return None
 *     return <object>base             # <<<<<<<<<<<<<<
 * 
 * # Versions of the import_* functions which are more suitable for
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_base));
  __pyx_r = ((PyObject *)__pyx_v_base);
  goto __pyx_L0;

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1026
 *     PyArray_SetBaseObject(arr, base)
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1034
 * # Versions of the import_* functions which are more suitable for
 * # Cython code.
 * cdef inline int import_array() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_array()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_array(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  __Pyx_RefNannySetupContext("import_array", 0);

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1035
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_array()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1036
 * cdef inline int import_array() except -1:
 *     try:
 *         _import_array()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")
 */
      __pyx_t_4 = _import_array(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(17, 1036, __pyx_L3_error)

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1035
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_array()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1037
 *     try:
 *         _import_array()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(17, 1037, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1038
 *         _import_array()
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_umath() except -1:
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__40, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(17, 1038, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(17, 1038, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1035
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_array()
 *     except Exception:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1034
 * # Versions of the import_* functions which are more suitable for
 * # Cython code.
 * cdef inline int import_array() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_array()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1040
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 * cdef inline int import_umath() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_umath(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  __Pyx_RefNannySetupContext("import_umath", 0);

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1041
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1042
 * cdef inline int import_umath() except -1:
 *     try:
 *         _import_umath()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")
 */
      __pyx_t_4 = _import_umath(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(17, 1042, __pyx_L3_error)

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1041
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1043
 *     try:
 *         _import_umath()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_umath", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(17, 1043, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1044
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_ufunc() except -1:
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__41, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(17, 1044, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(17, 1044, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1041
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1040
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 * cdef inline int import_umath() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_umath", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1046
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 * cdef inline int import_ufunc() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_ufunc(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  __Pyx_RefNannySetupContext("import_ufunc", 0);

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1047
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1048
 * cdef inline int import_ufunc() except -1:
 *     try:
 *         _import_umath()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")
 */
      __pyx_t_4 = _import_umath(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(17, 1048, __pyx_L3_error)

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1047
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1049
 *     try:
 *         _import_umath()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.umath failed to import")
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_ufunc", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(17, 1049, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1050
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__41, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(17, 1050, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(17, 1050, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1047
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1046
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 * cdef inline int import_ufunc() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_ufunc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static struct __pyx_vtabstruct_5pycx4_5pycda_Signal __pyx_vtable_5pycx4_5pycda_Signal;

static PyObject *__pyx_tp_new_5pycx4_5pycda_Signal(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_5pycx4_5pycda_Signal *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5pycx4_5pycda_Signal *)o);
  p->__pyx_vtab = __pyx_vtabptr_5pycx4_5pycda_Signal;
  if (unlikely(__pyx_pw_5pycx4_5pycda_6Signal_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_5pycx4_5pycda_Signal(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_pw_5pycx4_5pycda_6Signal_3__dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  (*Py_TYPE(o)->tp_free)(o);
}

static PyMethodDef __pyx_methods_5pycx4_5pycda_Signal[] = {
  {"connect", (PyCFunction)__pyx_pw_5pycx4_5pycda_6Signal_5connect, METH_O, 0},
  {"disconnect", (PyCFunction)__pyx_pw_5pycx4_5pycda_6Signal_7disconnect, METH_O, 0},
  {"emit", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_5pycx4_5pycda_6Signal_9emit, METH_VARARGS|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_5pycx4_5pycda_6Signal_11__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_5pycx4_5pycda_6Signal_13__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5pycx4_5pycda_Signal = {
  PyVarObject_HEAD_INIT(0, 0)
  "pycx4.pycda.Signal", /*tp_name*/
  sizeof(struct __pyx_obj_5pycx4_5pycda_Signal), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5pycx4_5pycda_Signal, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5pycx4_5pycda_Signal, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5pycx4_5pycda_Signal, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_5pycx4_5pycda_Timer __pyx_vtable_5pycx4_5pycda_Timer;

static PyObject *__pyx_tp_new_5pycx4_5pycda_Timer(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_5pycx4_5pycda_Timer *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5pycx4_5pycda_Timer *)o);
  p->__pyx_vtab = __pyx_vtabptr_5pycx4_5pycda_Timer;
  p->timeout = ((struct __pyx_obj_5pycx4_5pycda_Signal *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_5pycx4_5pycda_Timer(PyObject *o) {
  struct __pyx_obj_5pycx4_5pycda_Timer *p = (struct __pyx_obj_5pycx4_5pycda_Timer *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->timeout);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_5pycx4_5pycda_Timer(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5pycx4_5pycda_Timer *p = (struct __pyx_obj_5pycx4_5pycda_Timer *)o;
  if (p->timeout) {
    e = (*v)(((PyObject *)p->timeout), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5pycx4_5pycda_Timer(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5pycx4_5pycda_Timer *p = (struct __pyx_obj_5pycx4_5pycda_Timer *)o;
  tmp = ((PyObject*)p->timeout);
  p->timeout = ((struct __pyx_obj_5pycx4_5pycda_Signal *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_5pycx4_5pycda_5Timer_usec(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_5Timer_4usec_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_5Timer_repeat(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_5Timer_6repeat_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_5Timer_active(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_5Timer_6active_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_5Timer_tid(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_5Timer_3tid_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_5Timer_timeout(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_5Timer_7timeout_1__get__(o);
}

static PyMethodDef __pyx_methods_5pycx4_5pycda_Timer[] = {
  {"stop", (PyCFunction)__pyx_pw_5pycx4_5pycda_5Timer_3stop, METH_NOARGS, 0},
  {"start", (PyCFunction)__pyx_pw_5pycx4_5pycda_5Timer_5start, METH_NOARGS, 0},
  {"singleShot", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_5pycx4_5pycda_5Timer_7singleShot, METH_VARARGS|METH_KEYWORDS, 0},
  {"interval", (PyCFunction)__pyx_pw_5pycx4_5pycda_5Timer_9interval, METH_NOARGS, 0},
  {"setInterval", (PyCFunction)__pyx_pw_5pycx4_5pycda_5Timer_11setInterval, METH_O, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_5pycx4_5pycda_5Timer_13__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_5pycx4_5pycda_5Timer_15__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_5pycx4_5pycda_Timer[] = {
  {(char *)"usec", __pyx_getprop_5pycx4_5pycda_5Timer_usec, 0, (char *)0, 0},
  {(char *)"repeat", __pyx_getprop_5pycx4_5pycda_5Timer_repeat, 0, (char *)0, 0},
  {(char *)"active", __pyx_getprop_5pycx4_5pycda_5Timer_active, 0, (char *)0, 0},
  {(char *)"tid", __pyx_getprop_5pycx4_5pycda_5Timer_tid, 0, (char *)0, 0},
  {(char *)"timeout", __pyx_getprop_5pycx4_5pycda_5Timer_timeout, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5pycx4_5pycda_Timer = {
  PyVarObject_HEAD_INIT(0, 0)
  "pycx4.pycda.Timer", /*tp_name*/
  sizeof(struct __pyx_obj_5pycx4_5pycda_Timer), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5pycx4_5pycda_Timer, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5pycx4_5pycda_Timer, /*tp_traverse*/
  __pyx_tp_clear_5pycx4_5pycda_Timer, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5pycx4_5pycda_Timer, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_5pycx4_5pycda_Timer, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_5pycx4_5pycda_5Timer_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5pycx4_5pycda_Timer, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyObject *__pyx_tp_new_5pycx4_5pycda_FdEvent(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_5pycx4_5pycda_FdEvent *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5pycx4_5pycda_FdEvent *)o);
  p->file = Py_None; Py_INCREF(Py_None);
  p->ready = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_5pycx4_5pycda_FdEvent(PyObject *o) {
  struct __pyx_obj_5pycx4_5pycda_FdEvent *p = (struct __pyx_obj_5pycx4_5pycda_FdEvent *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_pw_5pycx4_5pycda_7FdEvent_3__dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->file);
  Py_CLEAR(p->ready);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_5pycx4_5pycda_FdEvent(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5pycx4_5pycda_FdEvent *p = (struct __pyx_obj_5pycx4_5pycda_FdEvent *)o;
  if (p->file) {
    e = (*v)(p->file, a); if (e) return e;
  }
  if (p->ready) {
    e = (*v)(p->ready, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5pycx4_5pycda_FdEvent(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5pycx4_5pycda_FdEvent *p = (struct __pyx_obj_5pycx4_5pycda_FdEvent *)o;
  tmp = ((PyObject*)p->file);
  p->file = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->ready);
  p->ready = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_5pycx4_5pycda_7FdEvent_file(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_7FdEvent_4file_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_7FdEvent_ready(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_7FdEvent_5ready_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_7FdEvent_mask(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_7FdEvent_4mask_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_7FdEvent_fhd(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_7FdEvent_3fhd_1__get__(o);
}

static PyMethodDef __pyx_methods_5pycx4_5pycda_FdEvent[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_5pycx4_5pycda_7FdEvent_5__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_5pycx4_5pycda_7FdEvent_7__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_5pycx4_5pycda_FdEvent[] = {
  {(char *)"file", __pyx_getprop_5pycx4_5pycda_7FdEvent_file, 0, (char *)0, 0},
  {(char *)"ready", __pyx_getprop_5pycx4_5pycda_7FdEvent_ready, 0, (char *)0, 0},
  {(char *)"mask", __pyx_getprop_5pycx4_5pycda_7FdEvent_mask, 0, (char *)0, 0},
  {(char *)"fhd", __pyx_getprop_5pycx4_5pycda_7FdEvent_fhd, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5pycx4_5pycda_FdEvent = {
  PyVarObject_HEAD_INIT(0, 0)
  "pycx4.pycda.FdEvent", /*tp_name*/
  sizeof(struct __pyx_obj_5pycx4_5pycda_FdEvent), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5pycx4_5pycda_FdEvent, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5pycx4_5pycda_FdEvent, /*tp_traverse*/
  __pyx_tp_clear_5pycx4_5pycda_FdEvent, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5pycx4_5pycda_FdEvent, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_5pycx4_5pycda_FdEvent, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_5pycx4_5pycda_7FdEvent_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5pycx4_5pycda_FdEvent, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_5pycx4_5pycda_CdaObject __pyx_vtable_5pycx4_5pycda_CdaObject;

static PyObject *__pyx_tp_new_5pycx4_5pycda_CdaObject(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_5pycx4_5pycda_CdaObject *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5pycx4_5pycda_CdaObject *)o);
  p->__pyx_vtab = __pyx_vtabptr_5pycx4_5pycda_CdaObject;
  return o;
}

static void __pyx_tp_dealloc_5pycx4_5pycda_CdaObject(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_pw_5pycx4_5pycda_9CdaObject_3__dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  (*Py_TYPE(o)->tp_free)(o);
}

static PyMethodDef __pyx_methods_5pycx4_5pycda_CdaObject[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_5pycx4_5pycda_9CdaObject_5__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_5pycx4_5pycda_9CdaObject_7__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5pycx4_5pycda_CdaObject = {
  PyVarObject_HEAD_INIT(0, 0)
  "pycx4.pycda.CdaObject", /*tp_name*/
  sizeof(struct __pyx_obj_5pycx4_5pycda_CdaObject), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5pycx4_5pycda_CdaObject, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5pycx4_5pycda_CdaObject, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_5pycx4_5pycda_9CdaObject_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5pycx4_5pycda_CdaObject, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_5pycx4_5pycda_Context __pyx_vtable_5pycx4_5pycda_Context;

static PyObject *__pyx_tp_new_5pycx4_5pycda_Context(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_5pycx4_5pycda_Context *p;
  PyObject *o = __pyx_tp_new_5pycx4_5pycda_CdaObject(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5pycx4_5pycda_Context *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_5pycx4_5pycda_CdaObject*)__pyx_vtabptr_5pycx4_5pycda_Context;
  p->defpfx = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->serverCycle = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_5pycx4_5pycda_Context(PyObject *o) {
  struct __pyx_obj_5pycx4_5pycda_Context *p = (struct __pyx_obj_5pycx4_5pycda_Context *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_pw_5pycx4_5pycda_7Context_3__dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->defpfx);
  Py_CLEAR(p->serverCycle);
  #if CYTHON_USE_TYPE_SLOTS
  if (PyType_IS_GC(Py_TYPE(o)->tp_base))
  #endif
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_5pycx4_5pycda_CdaObject(o);
}

static int __pyx_tp_traverse_5pycx4_5pycda_Context(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5pycx4_5pycda_Context *p = (struct __pyx_obj_5pycx4_5pycda_Context *)o;
  e = ((likely(__pyx_ptype_5pycx4_5pycda_CdaObject)) ? ((__pyx_ptype_5pycx4_5pycda_CdaObject->tp_traverse) ? __pyx_ptype_5pycx4_5pycda_CdaObject->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_5pycx4_5pycda_Context)); if (e) return e;
  if (p->serverCycle) {
    e = (*v)(p->serverCycle, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5pycx4_5pycda_Context(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5pycx4_5pycda_Context *p = (struct __pyx_obj_5pycx4_5pycda_Context *)o;
  if (likely(__pyx_ptype_5pycx4_5pycda_CdaObject)) { if (__pyx_ptype_5pycx4_5pycda_CdaObject->tp_clear) __pyx_ptype_5pycx4_5pycda_CdaObject->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_5pycx4_5pycda_Context);
  tmp = ((PyObject*)p->serverCycle);
  p->serverCycle = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_5pycx4_5pycda_7Context_cid(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_7Context_3cid_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_7Context_defpfx(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_7Context_6defpfx_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_7Context_serverCycle(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_7Context_11serverCycle_1__get__(o);
}

static int __pyx_setprop_5pycx4_5pycda_7Context_serverCycle(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pycx4_5pycda_7Context_11serverCycle_3__set__(o, v);
  }
  else {
    return __pyx_pw_5pycx4_5pycda_7Context_11serverCycle_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_5pycx4_5pycda_Context[] = {
  {"enable_serverCycle", (PyCFunction)__pyx_pw_5pycx4_5pycda_7Context_7enable_serverCycle, METH_NOARGS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_5pycx4_5pycda_7Context_9__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_5pycx4_5pycda_7Context_11__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_5pycx4_5pycda_Context[] = {
  {(char *)"cid", __pyx_getprop_5pycx4_5pycda_7Context_cid, 0, (char *)0, 0},
  {(char *)"defpfx", __pyx_getprop_5pycx4_5pycda_7Context_defpfx, 0, (char *)0, 0},
  {(char *)"serverCycle", __pyx_getprop_5pycx4_5pycda_7Context_serverCycle, __pyx_setprop_5pycx4_5pycda_7Context_serverCycle, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5pycx4_5pycda_Context = {
  PyVarObject_HEAD_INIT(0, 0)
  "pycx4.pycda.Context", /*tp_name*/
  sizeof(struct __pyx_obj_5pycx4_5pycda_Context), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5pycx4_5pycda_Context, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_5pycx4_5pycda_7Context_5__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5pycx4_5pycda_Context, /*tp_traverse*/
  __pyx_tp_clear_5pycx4_5pycda_Context, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5pycx4_5pycda_Context, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_5pycx4_5pycda_Context, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_5pycx4_5pycda_7Context_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5pycx4_5pycda_Context, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_5pycx4_5pycda_BaseChan __pyx_vtable_5pycx4_5pycda_BaseChan;

static PyObject *__pyx_tp_new_5pycx4_5pycda_BaseChan(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_5pycx4_5pycda_BaseChan *p;
  PyObject *o = __pyx_tp_new_5pycx4_5pycda_CdaObject(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5pycx4_5pycda_BaseChan *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_5pycx4_5pycda_CdaObject*)__pyx_vtabptr_5pycx4_5pycda_BaseChan;
  p->name = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->rslv_str = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->context = ((struct __pyx_obj_5pycx4_5pycda_Context *)Py_None); Py_INCREF(Py_None);
  p->rng = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->valueMeasured = Py_None; Py_INCREF(Py_None);
  p->valueChanged = Py_None; Py_INCREF(Py_None);
  p->resolve = Py_None; Py_INCREF(Py_None);
  p->stringsUpdate = Py_None; Py_INCREF(Py_None);
  p->lockStateUpdate = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_5pycx4_5pycda_BaseChan(PyObject *o) {
  struct __pyx_obj_5pycx4_5pycda_BaseChan *p = (struct __pyx_obj_5pycx4_5pycda_BaseChan *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_pw_5pycx4_5pycda_8BaseChan_3__dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->name);
  Py_CLEAR(p->rslv_str);
  Py_CLEAR(p->context);
  Py_CLEAR(p->rng);
  Py_CLEAR(p->valueMeasured);
  Py_CLEAR(p->valueChanged);
  Py_CLEAR(p->resolve);
  Py_CLEAR(p->stringsUpdate);
  Py_CLEAR(p->lockStateUpdate);
  #if CYTHON_USE_TYPE_SLOTS
  if (PyType_IS_GC(Py_TYPE(o)->tp_base))
  #endif
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_5pycx4_5pycda_CdaObject(o);
}

static int __pyx_tp_traverse_5pycx4_5pycda_BaseChan(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5pycx4_5pycda_BaseChan *p = (struct __pyx_obj_5pycx4_5pycda_BaseChan *)o;
  e = ((likely(__pyx_ptype_5pycx4_5pycda_CdaObject)) ? ((__pyx_ptype_5pycx4_5pycda_CdaObject->tp_traverse) ? __pyx_ptype_5pycx4_5pycda_CdaObject->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_5pycx4_5pycda_BaseChan)); if (e) return e;
  if (p->context) {
    e = (*v)(((PyObject *)p->context), a); if (e) return e;
  }
  if (p->rng) {
    e = (*v)(p->rng, a); if (e) return e;
  }
  if (p->valueMeasured) {
    e = (*v)(p->valueMeasured, a); if (e) return e;
  }
  if (p->valueChanged) {
    e = (*v)(p->valueChanged, a); if (e) return e;
  }
  if (p->resolve) {
    e = (*v)(p->resolve, a); if (e) return e;
  }
  if (p->stringsUpdate) {
    e = (*v)(p->stringsUpdate, a); if (e) return e;
  }
  if (p->lockStateUpdate) {
    e = (*v)(p->lockStateUpdate, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5pycx4_5pycda_BaseChan(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5pycx4_5pycda_BaseChan *p = (struct __pyx_obj_5pycx4_5pycda_BaseChan *)o;
  if (likely(__pyx_ptype_5pycx4_5pycda_CdaObject)) { if (__pyx_ptype_5pycx4_5pycda_CdaObject->tp_clear) __pyx_ptype_5pycx4_5pycda_CdaObject->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_5pycx4_5pycda_BaseChan);
  tmp = ((PyObject*)p->context);
  p->context = ((struct __pyx_obj_5pycx4_5pycda_Context *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->rng);
  p->rng = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->valueMeasured);
  p->valueMeasured = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->valueChanged);
  p->valueChanged = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->resolve);
  p->resolve = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->stringsUpdate);
  p->stringsUpdate = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->lockStateUpdate);
  p->lockStateUpdate = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_5pycx4_5pycda_8BaseChan_ref(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_8BaseChan_3ref_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_8BaseChan_name(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_8BaseChan_4name_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_8BaseChan_max_nelems(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_8BaseChan_10max_nelems_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_8BaseChan_dtype(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_8BaseChan_5dtype_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_8BaseChan_itemsize(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_8BaseChan_8itemsize_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_8BaseChan_time(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_8BaseChan_4time_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_8BaseChan_prev_time(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_8BaseChan_9prev_time_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_8BaseChan_rslv_stat(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_8BaseChan_9rslv_stat_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_8BaseChan_rslv_str(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_8BaseChan_8rslv_str_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_8BaseChan_quant(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_8BaseChan_5quant_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_8BaseChan_registered(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_8BaseChan_10registered_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_8BaseChan_first_cycle(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_8BaseChan_11first_cycle_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_8BaseChan_lock_state(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_8BaseChan_10lock_state_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_8BaseChan_context(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_8BaseChan_7context_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_8BaseChan_rflags(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_8BaseChan_6rflags_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_8BaseChan_rng(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_8BaseChan_3rng_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_8BaseChan_ident(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_8BaseChan_5ident_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_8BaseChan_label(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_8BaseChan_5label_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_8BaseChan_tip(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_8BaseChan_3tip_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_8BaseChan_comment(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_8BaseChan_7comment_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_8BaseChan_geoinfo(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_8BaseChan_7geoinfo_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_8BaseChan_rsrvd6(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_8BaseChan_6rsrvd6_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_8BaseChan_units(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_8BaseChan_5units_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_8BaseChan_dpyfmt(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_8BaseChan_6dpyfmt_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_8BaseChan_valueMeasured(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_8BaseChan_13valueMeasured_1__get__(o);
}

static int __pyx_setprop_5pycx4_5pycda_8BaseChan_valueMeasured(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pycx4_5pycda_8BaseChan_13valueMeasured_3__set__(o, v);
  }
  else {
    return __pyx_pw_5pycx4_5pycda_8BaseChan_13valueMeasured_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5pycx4_5pycda_8BaseChan_valueChanged(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_8BaseChan_12valueChanged_1__get__(o);
}

static int __pyx_setprop_5pycx4_5pycda_8BaseChan_valueChanged(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pycx4_5pycda_8BaseChan_12valueChanged_3__set__(o, v);
  }
  else {
    return __pyx_pw_5pycx4_5pycda_8BaseChan_12valueChanged_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5pycx4_5pycda_8BaseChan_resolve(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_8BaseChan_7resolve_1__get__(o);
}

static int __pyx_setprop_5pycx4_5pycda_8BaseChan_resolve(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pycx4_5pycda_8BaseChan_7resolve_3__set__(o, v);
  }
  else {
    return __pyx_pw_5pycx4_5pycda_8BaseChan_7resolve_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5pycx4_5pycda_8BaseChan_stringsUpdate(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_8BaseChan_13stringsUpdate_1__get__(o);
}

static int __pyx_setprop_5pycx4_5pycda_8BaseChan_stringsUpdate(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pycx4_5pycda_8BaseChan_13stringsUpdate_3__set__(o, v);
  }
  else {
    return __pyx_pw_5pycx4_5pycda_8BaseChan_13stringsUpdate_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5pycx4_5pycda_8BaseChan_lockStateUpdate(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_8BaseChan_15lockStateUpdate_1__get__(o);
}

static int __pyx_setprop_5pycx4_5pycda_8BaseChan_lockStateUpdate(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5pycx4_5pycda_8BaseChan_15lockStateUpdate_3__set__(o, v);
  }
  else {
    return __pyx_pw_5pycx4_5pycda_8BaseChan_15lockStateUpdate_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_5pycx4_5pycda_BaseChan[] = {
  {"short_name", (PyCFunction)__pyx_pw_5pycx4_5pycda_8BaseChan_7short_name, METH_NOARGS, 0},
  {"is_available", (PyCFunction)__pyx_pw_5pycx4_5pycda_8BaseChan_9is_available, METH_NOARGS, 0},
  {"rflags_text", (PyCFunction)__pyx_pw_5pycx4_5pycda_8BaseChan_11rflags_text, METH_NOARGS, 0},
  {"get_range", (PyCFunction)__pyx_pw_5pycx4_5pycda_8BaseChan_13get_range, METH_NOARGS, 0},
  {"get_strings", (PyCFunction)__pyx_pw_5pycx4_5pycda_8BaseChan_15get_strings, METH_NOARGS, 0},
  {"get_lockstate", (PyCFunction)__pyx_pw_5pycx4_5pycda_8BaseChan_17get_lockstate, METH_NOARGS, 0},
  {"lock", (PyCFunction)__pyx_pw_5pycx4_5pycda_8BaseChan_19lock, METH_NOARGS, 0},
  {"unlock", (PyCFunction)__pyx_pw_5pycx4_5pycda_8BaseChan_21unlock, METH_NOARGS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_5pycx4_5pycda_8BaseChan_23__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_5pycx4_5pycda_8BaseChan_25__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_5pycx4_5pycda_BaseChan[] = {
  {(char *)"ref", __pyx_getprop_5pycx4_5pycda_8BaseChan_ref, 0, (char *)0, 0},
  {(char *)"name", __pyx_getprop_5pycx4_5pycda_8BaseChan_name, 0, (char *)0, 0},
  {(char *)"max_nelems", __pyx_getprop_5pycx4_5pycda_8BaseChan_max_nelems, 0, (char *)0, 0},
  {(char *)"dtype", __pyx_getprop_5pycx4_5pycda_8BaseChan_dtype, 0, (char *)0, 0},
  {(char *)"itemsize", __pyx_getprop_5pycx4_5pycda_8BaseChan_itemsize, 0, (char *)0, 0},
  {(char *)"time", __pyx_getprop_5pycx4_5pycda_8BaseChan_time, 0, (char *)0, 0},
  {(char *)"prev_time", __pyx_getprop_5pycx4_5pycda_8BaseChan_prev_time, 0, (char *)0, 0},
  {(char *)"rslv_stat", __pyx_getprop_5pycx4_5pycda_8BaseChan_rslv_stat, 0, (char *)0, 0},
  {(char *)"rslv_str", __pyx_getprop_5pycx4_5pycda_8BaseChan_rslv_str, 0, (char *)0, 0},
  {(char *)"quant", __pyx_getprop_5pycx4_5pycda_8BaseChan_quant, 0, (char *)0, 0},
  {(char *)"registered", __pyx_getprop_5pycx4_5pycda_8BaseChan_registered, 0, (char *)0, 0},
  {(char *)"first_cycle", __pyx_getprop_5pycx4_5pycda_8BaseChan_first_cycle, 0, (char *)0, 0},
  {(char *)"lock_state", __pyx_getprop_5pycx4_5pycda_8BaseChan_lock_state, 0, (char *)0, 0},
  {(char *)"context", __pyx_getprop_5pycx4_5pycda_8BaseChan_context, 0, (char *)0, 0},
  {(char *)"rflags", __pyx_getprop_5pycx4_5pycda_8BaseChan_rflags, 0, (char *)0, 0},
  {(char *)"rng", __pyx_getprop_5pycx4_5pycda_8BaseChan_rng, 0, (char *)0, 0},
  {(char *)"ident", __pyx_getprop_5pycx4_5pycda_8BaseChan_ident, 0, (char *)0, 0},
  {(char *)"label", __pyx_getprop_5pycx4_5pycda_8BaseChan_label, 0, (char *)0, 0},
  {(char *)"tip", __pyx_getprop_5pycx4_5pycda_8BaseChan_tip, 0, (char *)0, 0},
  {(char *)"comment", __pyx_getprop_5pycx4_5pycda_8BaseChan_comment, 0, (char *)0, 0},
  {(char *)"geoinfo", __pyx_getprop_5pycx4_5pycda_8BaseChan_geoinfo, 0, (char *)0, 0},
  {(char *)"rsrvd6", __pyx_getprop_5pycx4_5pycda_8BaseChan_rsrvd6, 0, (char *)0, 0},
  {(char *)"units", __pyx_getprop_5pycx4_5pycda_8BaseChan_units, 0, (char *)0, 0},
  {(char *)"dpyfmt", __pyx_getprop_5pycx4_5pycda_8BaseChan_dpyfmt, 0, (char *)0, 0},
  {(char *)"valueMeasured", __pyx_getprop_5pycx4_5pycda_8BaseChan_valueMeasured, __pyx_setprop_5pycx4_5pycda_8BaseChan_valueMeasured, (char *)0, 0},
  {(char *)"valueChanged", __pyx_getprop_5pycx4_5pycda_8BaseChan_valueChanged, __pyx_setprop_5pycx4_5pycda_8BaseChan_valueChanged, (char *)0, 0},
  {(char *)"resolve", __pyx_getprop_5pycx4_5pycda_8BaseChan_resolve, __pyx_setprop_5pycx4_5pycda_8BaseChan_resolve, (char *)0, 0},
  {(char *)"stringsUpdate", __pyx_getprop_5pycx4_5pycda_8BaseChan_stringsUpdate, __pyx_setprop_5pycx4_5pycda_8BaseChan_stringsUpdate, (char *)0, 0},
  {(char *)"lockStateUpdate", __pyx_getprop_5pycx4_5pycda_8BaseChan_lockStateUpdate, __pyx_setprop_5pycx4_5pycda_8BaseChan_lockStateUpdate, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5pycx4_5pycda_BaseChan = {
  PyVarObject_HEAD_INIT(0, 0)
  "pycx4.pycda.BaseChan", /*tp_name*/
  sizeof(struct __pyx_obj_5pycx4_5pycda_BaseChan), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5pycx4_5pycda_BaseChan, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_5pycx4_5pycda_8BaseChan_5__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5pycx4_5pycda_BaseChan, /*tp_traverse*/
  __pyx_tp_clear_5pycx4_5pycda_BaseChan, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5pycx4_5pycda_BaseChan, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_5pycx4_5pycda_BaseChan, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_5pycx4_5pycda_8BaseChan_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5pycx4_5pycda_BaseChan, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_5pycx4_5pycda_DChan __pyx_vtable_5pycx4_5pycda_DChan;

static PyObject *__pyx_tp_new_5pycx4_5pycda_DChan(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_5pycx4_5pycda_DChan *p;
  PyObject *o = __pyx_tp_new_5pycx4_5pycda_BaseChan(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5pycx4_5pycda_DChan *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_5pycx4_5pycda_CdaObject*)__pyx_vtabptr_5pycx4_5pycda_DChan;
  return o;
}

static PyObject *__pyx_getprop_5pycx4_5pycda_5DChan_val(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_5DChan_3val_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_5DChan_prev_val(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_5DChan_8prev_val_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_5DChan_tolerance(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_5DChan_9tolerance_1__get__(o);
}

static PyMethodDef __pyx_methods_5pycx4_5pycda_DChan[] = {
  {"setValue", (PyCFunction)__pyx_pw_5pycx4_5pycda_5DChan_3setValue, METH_O, 0},
  {"setTolerance", (PyCFunction)__pyx_pw_5pycx4_5pycda_5DChan_5setTolerance, METH_O, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_5pycx4_5pycda_5DChan_7__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_5pycx4_5pycda_5DChan_9__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_5pycx4_5pycda_DChan[] = {
  {(char *)"val", __pyx_getprop_5pycx4_5pycda_5DChan_val, 0, (char *)0, 0},
  {(char *)"prev_val", __pyx_getprop_5pycx4_5pycda_5DChan_prev_val, 0, (char *)0, 0},
  {(char *)"tolerance", __pyx_getprop_5pycx4_5pycda_5DChan_tolerance, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5pycx4_5pycda_DChan = {
  PyVarObject_HEAD_INIT(0, 0)
  "pycx4.pycda.DChan", /*tp_name*/
  sizeof(struct __pyx_obj_5pycx4_5pycda_DChan), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5pycx4_5pycda_BaseChan, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  #if CYTHON_COMPILING_IN_PYPY
  __pyx_pw_5pycx4_5pycda_8BaseChan_5__str__, /*tp_str*/
  #else
  0, /*tp_str*/
  #endif
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    One-double channel\n\n    val - double last-known value\n    prev_val - previous value\n    tolerance - tolerance of emitting valueChanged signal\n    ", /*tp_doc*/
  __pyx_tp_traverse_5pycx4_5pycda_BaseChan, /*tp_traverse*/
  __pyx_tp_clear_5pycx4_5pycda_BaseChan, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5pycx4_5pycda_DChan, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_5pycx4_5pycda_DChan, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_5pycx4_5pycda_5DChan_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5pycx4_5pycda_DChan, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_5pycx4_5pycda_IChan __pyx_vtable_5pycx4_5pycda_IChan;

static PyObject *__pyx_tp_new_5pycx4_5pycda_IChan(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_5pycx4_5pycda_IChan *p;
  PyObject *o = __pyx_tp_new_5pycx4_5pycda_BaseChan(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5pycx4_5pycda_IChan *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_5pycx4_5pycda_CdaObject*)__pyx_vtabptr_5pycx4_5pycda_IChan;
  return o;
}

static PyObject *__pyx_getprop_5pycx4_5pycda_5IChan_val(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_5IChan_3val_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_5IChan_prev_val(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_5IChan_8prev_val_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_5IChan_tolerance(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_5IChan_9tolerance_1__get__(o);
}

static PyMethodDef __pyx_methods_5pycx4_5pycda_IChan[] = {
  {"setValue", (PyCFunction)__pyx_pw_5pycx4_5pycda_5IChan_3setValue, METH_O, 0},
  {"setTolerance", (PyCFunction)__pyx_pw_5pycx4_5pycda_5IChan_5setTolerance, METH_O, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_5pycx4_5pycda_5IChan_7__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_5pycx4_5pycda_5IChan_9__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_5pycx4_5pycda_IChan[] = {
  {(char *)"val", __pyx_getprop_5pycx4_5pycda_5IChan_val, 0, (char *)0, 0},
  {(char *)"prev_val", __pyx_getprop_5pycx4_5pycda_5IChan_prev_val, 0, (char *)0, 0},
  {(char *)"tolerance", __pyx_getprop_5pycx4_5pycda_5IChan_tolerance, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5pycx4_5pycda_IChan = {
  PyVarObject_HEAD_INIT(0, 0)
  "pycx4.pycda.IChan", /*tp_name*/
  sizeof(struct __pyx_obj_5pycx4_5pycda_IChan), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5pycx4_5pycda_BaseChan, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  #if CYTHON_COMPILING_IN_PYPY
  __pyx_pw_5pycx4_5pycda_8BaseChan_5__str__, /*tp_str*/
  #else
  0, /*tp_str*/
  #endif
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5pycx4_5pycda_BaseChan, /*tp_traverse*/
  __pyx_tp_clear_5pycx4_5pycda_BaseChan, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5pycx4_5pycda_IChan, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_5pycx4_5pycda_IChan, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_5pycx4_5pycda_5IChan_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5pycx4_5pycda_IChan, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_5pycx4_5pycda_Chan __pyx_vtable_5pycx4_5pycda_Chan;

static PyObject *__pyx_tp_new_5pycx4_5pycda_Chan(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_5pycx4_5pycda_Chan *p;
  PyObject *o = __pyx_tp_new_5pycx4_5pycda_BaseChan(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5pycx4_5pycda_Chan *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_5pycx4_5pycda_CdaObject*)__pyx_vtabptr_5pycx4_5pycda_Chan;
  p->val = Py_None; Py_INCREF(Py_None);
  p->prev_val = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_5pycx4_5pycda_Chan(PyObject *o) {
  struct __pyx_obj_5pycx4_5pycda_Chan *p = (struct __pyx_obj_5pycx4_5pycda_Chan *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->val);
  Py_CLEAR(p->prev_val);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_5pycx4_5pycda_BaseChan(o);
}

static int __pyx_tp_traverse_5pycx4_5pycda_Chan(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5pycx4_5pycda_Chan *p = (struct __pyx_obj_5pycx4_5pycda_Chan *)o;
  e = __pyx_tp_traverse_5pycx4_5pycda_BaseChan(o, v, a); if (e) return e;
  if (p->val) {
    e = (*v)(p->val, a); if (e) return e;
  }
  if (p->prev_val) {
    e = (*v)(p->prev_val, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5pycx4_5pycda_Chan(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5pycx4_5pycda_Chan *p = (struct __pyx_obj_5pycx4_5pycda_Chan *)o;
  __pyx_tp_clear_5pycx4_5pycda_BaseChan(o);
  tmp = ((PyObject*)p->val);
  p->val = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->prev_val);
  p->prev_val = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_5pycx4_5pycda_4Chan_val(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_4Chan_3val_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_4Chan_prev_val(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_4Chan_8prev_val_1__get__(o);
}

static PyMethodDef __pyx_methods_5pycx4_5pycda_Chan[] = {
  {"setValue", (PyCFunction)__pyx_pw_5pycx4_5pycda_4Chan_3setValue, METH_O, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_5pycx4_5pycda_4Chan_5__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_5pycx4_5pycda_4Chan_7__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_5pycx4_5pycda_Chan[] = {
  {(char *)"val", __pyx_getprop_5pycx4_5pycda_4Chan_val, 0, (char *)0, 0},
  {(char *)"prev_val", __pyx_getprop_5pycx4_5pycda_4Chan_prev_val, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5pycx4_5pycda_Chan = {
  PyVarObject_HEAD_INIT(0, 0)
  "pycx4.pycda.Chan", /*tp_name*/
  sizeof(struct __pyx_obj_5pycx4_5pycda_Chan), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5pycx4_5pycda_Chan, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  #if CYTHON_COMPILING_IN_PYPY
  __pyx_pw_5pycx4_5pycda_8BaseChan_5__str__, /*tp_str*/
  #else
  0, /*tp_str*/
  #endif
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5pycx4_5pycda_Chan, /*tp_traverse*/
  __pyx_tp_clear_5pycx4_5pycda_Chan, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5pycx4_5pycda_Chan, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_5pycx4_5pycda_Chan, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_5pycx4_5pycda_4Chan_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5pycx4_5pycda_Chan, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_5pycx4_5pycda_VChan __pyx_vtable_5pycx4_5pycda_VChan;

static PyObject *__pyx_tp_new_5pycx4_5pycda_VChan(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_5pycx4_5pycda_VChan *p;
  PyObject *o = __pyx_tp_new_5pycx4_5pycda_BaseChan(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5pycx4_5pycda_VChan *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_5pycx4_5pycda_CdaObject*)__pyx_vtabptr_5pycx4_5pycda_VChan;
  p->val = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  p->npdtype = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_5pycx4_5pycda_VChan(PyObject *o) {
  struct __pyx_obj_5pycx4_5pycda_VChan *p = (struct __pyx_obj_5pycx4_5pycda_VChan *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->val);
  Py_CLEAR(p->npdtype);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_5pycx4_5pycda_BaseChan(o);
}

static int __pyx_tp_traverse_5pycx4_5pycda_VChan(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5pycx4_5pycda_VChan *p = (struct __pyx_obj_5pycx4_5pycda_VChan *)o;
  e = __pyx_tp_traverse_5pycx4_5pycda_BaseChan(o, v, a); if (e) return e;
  if (p->val) {
    e = (*v)(((PyObject *)p->val), a); if (e) return e;
  }
  if (p->npdtype) {
    e = (*v)(p->npdtype, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5pycx4_5pycda_VChan(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5pycx4_5pycda_VChan *p = (struct __pyx_obj_5pycx4_5pycda_VChan *)o;
  __pyx_tp_clear_5pycx4_5pycda_BaseChan(o);
  tmp = ((PyObject*)p->val);
  p->val = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->npdtype);
  p->npdtype = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_5pycx4_5pycda_5VChan_val(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_5VChan_3val_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_5VChan_npdtype(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_5VChan_7npdtype_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_5VChan_nelems(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_5VChan_6nelems_1__get__(o);
}

static PyMethodDef __pyx_methods_5pycx4_5pycda_VChan[] = {
  {"setValue", (PyCFunction)__pyx_pw_5pycx4_5pycda_5VChan_3setValue, METH_O, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_5pycx4_5pycda_5VChan_5__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_5pycx4_5pycda_5VChan_7__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_5pycx4_5pycda_VChan[] = {
  {(char *)"val", __pyx_getprop_5pycx4_5pycda_5VChan_val, 0, (char *)0, 0},
  {(char *)"npdtype", __pyx_getprop_5pycx4_5pycda_5VChan_npdtype, 0, (char *)0, 0},
  {(char *)"nelems", __pyx_getprop_5pycx4_5pycda_5VChan_nelems, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5pycx4_5pycda_VChan = {
  PyVarObject_HEAD_INIT(0, 0)
  "pycx4.pycda.VChan", /*tp_name*/
  sizeof(struct __pyx_obj_5pycx4_5pycda_VChan), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5pycx4_5pycda_VChan, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  #if CYTHON_COMPILING_IN_PYPY
  __pyx_pw_5pycx4_5pycda_8BaseChan_5__str__, /*tp_str*/
  #else
  0, /*tp_str*/
  #endif
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5pycx4_5pycda_VChan, /*tp_traverse*/
  __pyx_tp_clear_5pycx4_5pycda_VChan, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5pycx4_5pycda_VChan, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_5pycx4_5pycda_VChan, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_5pycx4_5pycda_5VChan_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5pycx4_5pycda_VChan, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_5pycx4_5pycda_StrChan __pyx_vtable_5pycx4_5pycda_StrChan;

static PyObject *__pyx_tp_new_5pycx4_5pycda_StrChan(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_5pycx4_5pycda_StrChan *p;
  PyObject *o = __pyx_tp_new_5pycx4_5pycda_BaseChan(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5pycx4_5pycda_StrChan *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_5pycx4_5pycda_CdaObject*)__pyx_vtabptr_5pycx4_5pycda_StrChan;
  p->val = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->prev_val = ((PyObject*)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_5pycx4_5pycda_StrChan(PyObject *o) {
  struct __pyx_obj_5pycx4_5pycda_StrChan *p = (struct __pyx_obj_5pycx4_5pycda_StrChan *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_pw_5pycx4_5pycda_7StrChan_3__dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->val);
  Py_CLEAR(p->prev_val);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_5pycx4_5pycda_BaseChan(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_7StrChan_val(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_7StrChan_3val_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_7StrChan_prev_val(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_7StrChan_8prev_val_1__get__(o);
}

static PyMethodDef __pyx_methods_5pycx4_5pycda_StrChan[] = {
  {"setValue", (PyCFunction)__pyx_pw_5pycx4_5pycda_7StrChan_5setValue, METH_O, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_5pycx4_5pycda_7StrChan_7__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_5pycx4_5pycda_7StrChan_9__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_5pycx4_5pycda_StrChan[] = {
  {(char *)"val", __pyx_getprop_5pycx4_5pycda_7StrChan_val, 0, (char *)0, 0},
  {(char *)"prev_val", __pyx_getprop_5pycx4_5pycda_7StrChan_prev_val, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5pycx4_5pycda_StrChan = {
  PyVarObject_HEAD_INIT(0, 0)
  "pycx4.pycda.StrChan", /*tp_name*/
  sizeof(struct __pyx_obj_5pycx4_5pycda_StrChan), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5pycx4_5pycda_StrChan, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  #if CYTHON_COMPILING_IN_PYPY
  __pyx_pw_5pycx4_5pycda_8BaseChan_5__str__, /*tp_str*/
  #else
  0, /*tp_str*/
  #endif
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5pycx4_5pycda_BaseChan, /*tp_traverse*/
  __pyx_tp_clear_5pycx4_5pycda_BaseChan, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5pycx4_5pycda_StrChan, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_5pycx4_5pycda_StrChan, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_5pycx4_5pycda_7StrChan_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5pycx4_5pycda_StrChan, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyObject *__pyx_tp_new_5pycx4_5pycda_ChanFactory(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_5pycx4_5pycda_ChanFactory *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5pycx4_5pycda_ChanFactory *)o);
  p->single_chans = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->vector_chans = ((PyObject*)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_5pycx4_5pycda_ChanFactory(PyObject *o) {
  struct __pyx_obj_5pycx4_5pycda_ChanFactory *p = (struct __pyx_obj_5pycx4_5pycda_ChanFactory *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->single_chans);
  Py_CLEAR(p->vector_chans);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_5pycx4_5pycda_ChanFactory(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5pycx4_5pycda_ChanFactory *p = (struct __pyx_obj_5pycx4_5pycda_ChanFactory *)o;
  if (p->single_chans) {
    e = (*v)(p->single_chans, a); if (e) return e;
  }
  if (p->vector_chans) {
    e = (*v)(p->vector_chans, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5pycx4_5pycda_ChanFactory(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5pycx4_5pycda_ChanFactory *p = (struct __pyx_obj_5pycx4_5pycda_ChanFactory *)o;
  tmp = ((PyObject*)p->single_chans);
  p->single_chans = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->vector_chans);
  p->vector_chans = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_5pycx4_5pycda_ChanFactory[] = {
  {"create_chan", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_5pycx4_5pycda_11ChanFactory_3create_chan, METH_VARARGS|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_5pycx4_5pycda_11ChanFactory_5__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_5pycx4_5pycda_11ChanFactory_7__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5pycx4_5pycda_ChanFactory = {
  PyVarObject_HEAD_INIT(0, 0)
  "pycx4.pycda.ChanFactory", /*tp_name*/
  sizeof(struct __pyx_obj_5pycx4_5pycda_ChanFactory), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5pycx4_5pycda_ChanFactory, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5pycx4_5pycda_ChanFactory, /*tp_traverse*/
  __pyx_tp_clear_5pycx4_5pycda_ChanFactory, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5pycx4_5pycda_ChanFactory, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_5pycx4_5pycda_11ChanFactory_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5pycx4_5pycda_ChanFactory, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_5pycx4_5pycda_PassGW __pyx_vtable_5pycx4_5pycda_PassGW;

static PyObject *__pyx_tp_new_5pycx4_5pycda_PassGW(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_5pycx4_5pycda_PassGW *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5pycx4_5pycda_PassGW *)o);
  p->__pyx_vtab = __pyx_vtabptr_5pycx4_5pycda_PassGW;
  p->in_chan = Py_None; Py_INCREF(Py_None);
  p->out_chan = Py_None; Py_INCREF(Py_None);
  p->last_val = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_5pycx4_5pycda_PassGW(PyObject *o) {
  struct __pyx_obj_5pycx4_5pycda_PassGW *p = (struct __pyx_obj_5pycx4_5pycda_PassGW *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->in_chan);
  Py_CLEAR(p->out_chan);
  Py_CLEAR(p->last_val);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_5pycx4_5pycda_PassGW(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5pycx4_5pycda_PassGW *p = (struct __pyx_obj_5pycx4_5pycda_PassGW *)o;
  if (p->in_chan) {
    e = (*v)(p->in_chan, a); if (e) return e;
  }
  if (p->out_chan) {
    e = (*v)(p->out_chan, a); if (e) return e;
  }
  if (p->last_val) {
    e = (*v)(p->last_val, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5pycx4_5pycda_PassGW(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5pycx4_5pycda_PassGW *p = (struct __pyx_obj_5pycx4_5pycda_PassGW *)o;
  tmp = ((PyObject*)p->in_chan);
  p->in_chan = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->out_chan);
  p->out_chan = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->last_val);
  p->last_val = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_5pycx4_5pycda_PassGW[] = {
  {"in_c_proc", (PyCFunction)__pyx_pw_5pycx4_5pycda_6PassGW_3in_c_proc, METH_O, 0},
  {"out_c_proc", (PyCFunction)__pyx_pw_5pycx4_5pycda_6PassGW_5out_c_proc, METH_O, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_5pycx4_5pycda_6PassGW_7__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_5pycx4_5pycda_6PassGW_9__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5pycx4_5pycda_PassGW = {
  PyVarObject_HEAD_INIT(0, 0)
  "pycx4.pycda.PassGW", /*tp_name*/
  sizeof(struct __pyx_obj_5pycx4_5pycda_PassGW), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5pycx4_5pycda_PassGW, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5pycx4_5pycda_PassGW, /*tp_traverse*/
  __pyx_tp_clear_5pycx4_5pycda_PassGW, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5pycx4_5pycda_PassGW, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_5pycx4_5pycda_6PassGW_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5pycx4_5pycda_PassGW, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_5pycx4_5pycda_ProcGW __pyx_vtable_5pycx4_5pycda_ProcGW;

static PyObject *__pyx_tp_new_5pycx4_5pycda_ProcGW(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_5pycx4_5pycda_ProcGW *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5pycx4_5pycda_ProcGW *)o);
  p->__pyx_vtab = __pyx_vtabptr_5pycx4_5pycda_ProcGW;
  p->in_chan = Py_None; Py_INCREF(Py_None);
  p->out_chan = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_5pycx4_5pycda_ProcGW(PyObject *o) {
  struct __pyx_obj_5pycx4_5pycda_ProcGW *p = (struct __pyx_obj_5pycx4_5pycda_ProcGW *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->in_chan);
  Py_CLEAR(p->out_chan);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_5pycx4_5pycda_ProcGW(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5pycx4_5pycda_ProcGW *p = (struct __pyx_obj_5pycx4_5pycda_ProcGW *)o;
  if (p->in_chan) {
    e = (*v)(p->in_chan, a); if (e) return e;
  }
  if (p->out_chan) {
    e = (*v)(p->out_chan, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5pycx4_5pycda_ProcGW(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5pycx4_5pycda_ProcGW *p = (struct __pyx_obj_5pycx4_5pycda_ProcGW *)o;
  tmp = ((PyObject*)p->in_chan);
  p->in_chan = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->out_chan);
  p->out_chan = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_5pycx4_5pycda_ProcGW[] = {
  {"in_c_proc", (PyCFunction)__pyx_pw_5pycx4_5pycda_6ProcGW_3in_c_proc, METH_O, 0},
  {"out_c_proc", (PyCFunction)__pyx_pw_5pycx4_5pycda_6ProcGW_5out_c_proc, METH_O, 0},
  {"in2out", (PyCFunction)__pyx_pw_5pycx4_5pycda_6ProcGW_7in2out, METH_NOARGS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_5pycx4_5pycda_6ProcGW_9__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_5pycx4_5pycda_6ProcGW_11__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5pycx4_5pycda_ProcGW = {
  PyVarObject_HEAD_INIT(0, 0)
  "pycx4.pycda.ProcGW", /*tp_name*/
  sizeof(struct __pyx_obj_5pycx4_5pycda_ProcGW), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5pycx4_5pycda_ProcGW, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5pycx4_5pycda_ProcGW, /*tp_traverse*/
  __pyx_tp_clear_5pycx4_5pycda_ProcGW, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5pycx4_5pycda_ProcGW, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_5pycx4_5pycda_6ProcGW_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5pycx4_5pycda_ProcGW, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {"main_loop", (PyCFunction)__pyx_pw_5pycx4_5pycda_1main_loop, METH_NOARGS, 0},
  {"break_", (PyCFunction)__pyx_pw_5pycx4_5pycda_3break_, METH_NOARGS, 0},
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_pycda(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_pycda},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "pycda",
    0, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_u_A_function_was_expected, __pyx_k_A_function_was_expected, sizeof(__pyx_k_A_function_was_expected), 0, 1, 0, 0},
  {&__pyx_kp_u_Alarm, __pyx_k_Alarm, sizeof(__pyx_k_Alarm), 0, 1, 0, 0},
  {&__pyx_n_s_BaseChan, __pyx_k_BaseChan, sizeof(__pyx_k_BaseChan), 0, 0, 1, 1},
  {&__pyx_n_u_C, __pyx_k_C, sizeof(__pyx_k_C), 0, 1, 0, 1},
  {&__pyx_kp_u_CdaContext_cid, __pyx_k_CdaContext_cid, sizeof(__pyx_k_CdaContext_cid), 0, 1, 0, 0},
  {&__pyx_n_s_CdaObject, __pyx_k_CdaObject, sizeof(__pyx_k_CdaObject), 0, 0, 1, 1},
  {&__pyx_n_s_Chan, __pyx_k_Chan, sizeof(__pyx_k_Chan), 0, 0, 1, 1},
  {&__pyx_n_u_Chan, __pyx_k_Chan, sizeof(__pyx_k_Chan), 0, 1, 0, 1},
  {&__pyx_n_s_ChanFactory, __pyx_k_ChanFactory, sizeof(__pyx_k_ChanFactory), 0, 0, 1, 1},
  {&__pyx_n_u_ChanFactory, __pyx_k_ChanFactory, sizeof(__pyx_k_ChanFactory), 0, 1, 0, 1},
  {&__pyx_kp_u_Channel_not_found, __pyx_k_Channel_not_found, sizeof(__pyx_k_Channel_not_found), 0, 1, 0, 0},
  {&__pyx_kp_u_Channel_was_programmatically_cha, __pyx_k_Channel_was_programmatically_cha, sizeof(__pyx_k_Channel_was_programmatically_cha), 0, 1, 0, 0},
  {&__pyx_kp_u_Configuration_problem, __pyx_k_Configuration_problem, sizeof(__pyx_k_Configuration_problem), 0, 1, 0, 0},
  {&__pyx_n_s_Context, __pyx_k_Context, sizeof(__pyx_k_Context), 0, 0, 1, 1},
  {&__pyx_n_u_Context, __pyx_k_Context, sizeof(__pyx_k_Context), 0, 1, 0, 1},
  {&__pyx_n_s_DChan, __pyx_k_DChan, sizeof(__pyx_k_DChan), 0, 0, 1, 1},
  {&__pyx_n_u_DChan, __pyx_k_DChan, sizeof(__pyx_k_DChan), 0, 1, 0, 1},
  {&__pyx_n_s_DTYPE_DOUBLE, __pyx_k_DTYPE_DOUBLE, sizeof(__pyx_k_DTYPE_DOUBLE), 0, 0, 1, 1},
  {&__pyx_n_u_DTYPE_DOUBLE, __pyx_k_DTYPE_DOUBLE, sizeof(__pyx_k_DTYPE_DOUBLE), 0, 1, 0, 1},
  {&__pyx_n_s_DTYPE_INT16, __pyx_k_DTYPE_INT16, sizeof(__pyx_k_DTYPE_INT16), 0, 0, 1, 1},
  {&__pyx_n_u_DTYPE_INT16, __pyx_k_DTYPE_INT16, sizeof(__pyx_k_DTYPE_INT16), 0, 1, 0, 1},
  {&__pyx_n_s_DTYPE_INT32, __pyx_k_DTYPE_INT32, sizeof(__pyx_k_DTYPE_INT32), 0, 0, 1, 1},
  {&__pyx_n_u_DTYPE_INT32, __pyx_k_DTYPE_INT32, sizeof(__pyx_k_DTYPE_INT32), 0, 1, 0, 1},
  {&__pyx_n_s_DTYPE_INT64, __pyx_k_DTYPE_INT64, sizeof(__pyx_k_DTYPE_INT64), 0, 0, 1, 1},
  {&__pyx_n_u_DTYPE_INT64, __pyx_k_DTYPE_INT64, sizeof(__pyx_k_DTYPE_INT64), 0, 1, 0, 1},
  {&__pyx_n_s_DTYPE_INT8, __pyx_k_DTYPE_INT8, sizeof(__pyx_k_DTYPE_INT8), 0, 0, 1, 1},
  {&__pyx_n_u_DTYPE_INT8, __pyx_k_DTYPE_INT8, sizeof(__pyx_k_DTYPE_INT8), 0, 1, 0, 1},
  {&__pyx_n_s_DTYPE_SINGLE, __pyx_k_DTYPE_SINGLE, sizeof(__pyx_k_DTYPE_SINGLE), 0, 0, 1, 1},
  {&__pyx_n_u_DTYPE_SINGLE, __pyx_k_DTYPE_SINGLE, sizeof(__pyx_k_DTYPE_SINGLE), 0, 1, 0, 1},
  {&__pyx_n_s_DTYPE_TEXT, __pyx_k_DTYPE_TEXT, sizeof(__pyx_k_DTYPE_TEXT), 0, 0, 1, 1},
  {&__pyx_n_u_DTYPE_TEXT, __pyx_k_DTYPE_TEXT, sizeof(__pyx_k_DTYPE_TEXT), 0, 1, 0, 1},
  {&__pyx_n_s_DTYPE_UCTEXT, __pyx_k_DTYPE_UCTEXT, sizeof(__pyx_k_DTYPE_UCTEXT), 0, 0, 1, 1},
  {&__pyx_n_u_DTYPE_UCTEXT, __pyx_k_DTYPE_UCTEXT, sizeof(__pyx_k_DTYPE_UCTEXT), 0, 1, 0, 1},
  {&__pyx_n_s_DTYPE_UINT16, __pyx_k_DTYPE_UINT16, sizeof(__pyx_k_DTYPE_UINT16), 0, 0, 1, 1},
  {&__pyx_n_u_DTYPE_UINT16, __pyx_k_DTYPE_UINT16, sizeof(__pyx_k_DTYPE_UINT16), 0, 1, 0, 1},
  {&__pyx_n_s_DTYPE_UINT32, __pyx_k_DTYPE_UINT32, sizeof(__pyx_k_DTYPE_UINT32), 0, 0, 1, 1},
  {&__pyx_n_u_DTYPE_UINT32, __pyx_k_DTYPE_UINT32, sizeof(__pyx_k_DTYPE_UINT32), 0, 1, 0, 1},
  {&__pyx_n_s_DTYPE_UINT64, __pyx_k_DTYPE_UINT64, sizeof(__pyx_k_DTYPE_UINT64), 0, 0, 1, 1},
  {&__pyx_n_u_DTYPE_UINT64, __pyx_k_DTYPE_UINT64, sizeof(__pyx_k_DTYPE_UINT64), 0, 1, 0, 1},
  {&__pyx_n_s_DTYPE_UINT8, __pyx_k_DTYPE_UINT8, sizeof(__pyx_k_DTYPE_UINT8), 0, 0, 1, 1},
  {&__pyx_n_u_DTYPE_UINT8, __pyx_k_DTYPE_UINT8, sizeof(__pyx_k_DTYPE_UINT8), 0, 1, 0, 1},
  {&__pyx_n_s_DTYPE_UNKNOWN, __pyx_k_DTYPE_UNKNOWN, sizeof(__pyx_k_DTYPE_UNKNOWN), 0, 0, 1, 1},
  {&__pyx_n_u_DTYPE_UNKNOWN, __pyx_k_DTYPE_UNKNOWN, sizeof(__pyx_k_DTYPE_UNKNOWN), 0, 1, 0, 1},
  {&__pyx_kp_u_Defunct_channel, __pyx_k_Defunct_channel, sizeof(__pyx_k_Defunct_channel), 0, 1, 0, 0},
  {&__pyx_kp_u_Device_is_offline, __pyx_k_Device_is_offline, sizeof(__pyx_k_Device_is_offline), 0, 1, 0, 0},
  {&__pyx_kp_u_Driver_internal_problem, __pyx_k_Driver_internal_problem, sizeof(__pyx_k_Driver_internal_problem), 0, 1, 0, 0},
  {&__pyx_kp_u_Driver_loading_problem, __pyx_k_Driver_loading_problem, sizeof(__pyx_k_Driver_loading_problem), 0, 1, 0, 0},
  {&__pyx_n_s_FdEvent, __pyx_k_FdEvent, sizeof(__pyx_k_FdEvent), 0, 0, 1, 1},
  {&__pyx_n_u_FdEvent, __pyx_k_FdEvent, sizeof(__pyx_k_FdEvent), 0, 1, 0, 1},
  {&__pyx_kp_u_Format_string_allocated_too_shor, __pyx_k_Format_string_allocated_too_shor, sizeof(__pyx_k_Format_string_allocated_too_shor), 0, 1, 0, 0},
  {&__pyx_kp_u_Format_string_allocated_too_shor_2, __pyx_k_Format_string_allocated_too_shor_2, sizeof(__pyx_k_Format_string_allocated_too_shor_2), 0, 1, 0, 0},
  {&__pyx_kp_u_Formula_calculation_error, __pyx_k_Formula_calculation_error, sizeof(__pyx_k_Formula_calculation_error), 0, 1, 0, 0},
  {&__pyx_n_s_IChan, __pyx_k_IChan, sizeof(__pyx_k_IChan), 0, 0, 1, 1},
  {&__pyx_n_u_IChan, __pyx_k_IChan, sizeof(__pyx_k_IChan), 0, 1, 0, 1},
  {&__pyx_kp_u_I_O_timeout_expired, __pyx_k_I_O_timeout_expired, sizeof(__pyx_k_I_O_timeout_expired), 0, 1, 0, 0},
  {&__pyx_n_s_ImportError, __pyx_k_ImportError, sizeof(__pyx_k_ImportError), 0, 0, 1, 1},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x18, __pyx_k_Incompatible_checksums_s_vs_0x18, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x18), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x57, __pyx_k_Incompatible_checksums_s_vs_0x57, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x57), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x5a, __pyx_k_Incompatible_checksums_s_vs_0x5a, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x5a), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x77, __pyx_k_Incompatible_checksums_s_vs_0x77, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x77), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0xef, __pyx_k_Incompatible_checksums_s_vs_0xef, sizeof(__pyx_k_Incompatible_checksums_s_vs_0xef), 0, 0, 1, 0},
  {&__pyx_kp_u_Input_channel_overload, __pyx_k_Input_channel_overload, sizeof(__pyx_k_Input_channel_overload), 0, 1, 0, 0},
  {&__pyx_kp_u_Invalid_parameter, __pyx_k_Invalid_parameter, sizeof(__pyx_k_Invalid_parameter), 0, 1, 0, 0},
  {&__pyx_n_s_MemoryError, __pyx_k_MemoryError, sizeof(__pyx_k_MemoryError), 0, 0, 1, 1},
  {&__pyx_kp_u_No_Q_from_CAMAC_device, __pyx_k_No_Q_from_CAMAC_device, sizeof(__pyx_k_No_Q_from_CAMAC_device), 0, 1, 0, 0},
  {&__pyx_kp_u_No_X_from_CAMAC_device, __pyx_k_No_X_from_CAMAC_device, sizeof(__pyx_k_No_X_from_CAMAC_device), 0, 1, 0, 0},
  {&__pyx_kp_u_Non_native_byte_order_not_suppor, __pyx_k_Non_native_byte_order_not_suppor, sizeof(__pyx_k_Non_native_byte_order_not_suppor), 0, 1, 0, 0},
  {&__pyx_kp_u_None, __pyx_k_None, sizeof(__pyx_k_None), 0, 1, 0, 0},
  {&__pyx_kp_u_Other_operator_is_active, __pyx_k_Other_operator_is_active, sizeof(__pyx_k_Other_operator_is_active), 0, 1, 0, 0},
  {&__pyx_n_s_PassGW, __pyx_k_PassGW, sizeof(__pyx_k_PassGW), 0, 0, 1, 1},
  {&__pyx_n_u_PassGW, __pyx_k_PassGW, sizeof(__pyx_k_PassGW), 0, 1, 0, 1},
  {&__pyx_n_s_PickleError, __pyx_k_PickleError, sizeof(__pyx_k_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_ProcGW, __pyx_k_ProcGW, sizeof(__pyx_k_ProcGW), 0, 0, 1, 1},
  {&__pyx_n_s_RSLVSTAT_FOUND, __pyx_k_RSLVSTAT_FOUND, sizeof(__pyx_k_RSLVSTAT_FOUND), 0, 0, 1, 1},
  {&__pyx_n_u_RSLVSTAT_FOUND, __pyx_k_RSLVSTAT_FOUND, sizeof(__pyx_k_RSLVSTAT_FOUND), 0, 1, 0, 1},
  {&__pyx_n_s_RSLVSTAT_NOTFOUND, __pyx_k_RSLVSTAT_NOTFOUND, sizeof(__pyx_k_RSLVSTAT_NOTFOUND), 0, 0, 1, 1},
  {&__pyx_n_u_RSLVSTAT_NOTFOUND, __pyx_k_RSLVSTAT_NOTFOUND, sizeof(__pyx_k_RSLVSTAT_NOTFOUND), 0, 1, 0, 1},
  {&__pyx_n_s_RSLVSTAT_SEARCHING, __pyx_k_RSLVSTAT_SEARCHING, sizeof(__pyx_k_RSLVSTAT_SEARCHING), 0, 0, 1, 1},
  {&__pyx_n_u_RSLVSTAT_SEARCHING, __pyx_k_RSLVSTAT_SEARCHING, sizeof(__pyx_k_RSLVSTAT_SEARCHING), 0, 1, 0, 1},
  {&__pyx_kp_u_Relaxing_after_alarm, __pyx_k_Relaxing_after_alarm, sizeof(__pyx_k_Relaxing_after_alarm), 0, 1, 0, 0},
  {&__pyx_kp_u_Remote_controller_problem, __pyx_k_Remote_controller_problem, sizeof(__pyx_k_Remote_controller_problem), 0, 1, 0, 0},
  {&__pyx_n_s_RuntimeError, __pyx_k_RuntimeError, sizeof(__pyx_k_RuntimeError), 0, 0, 1, 1},
  {&__pyx_n_s_Signal, __pyx_k_Signal, sizeof(__pyx_k_Signal), 0, 0, 1, 1},
  {&__pyx_n_u_Signal, __pyx_k_Signal, sizeof(__pyx_k_Signal), 0, 1, 0, 1},
  {&__pyx_n_s_StrChan, __pyx_k_StrChan, sizeof(__pyx_k_StrChan), 0, 0, 1, 1},
  {&__pyx_n_u_StrChan, __pyx_k_StrChan, sizeof(__pyx_k_StrChan), 0, 1, 0, 1},
  {&__pyx_n_s_Timer, __pyx_k_Timer, sizeof(__pyx_k_Timer), 0, 0, 1, 1},
  {&__pyx_n_u_Timer, __pyx_k_Timer, sizeof(__pyx_k_Timer), 0, 1, 0, 1},
  {&__pyx_n_s_TypeError, __pyx_k_TypeError, sizeof(__pyx_k_TypeError), 0, 0, 1, 1},
  {&__pyx_kp_u_Unsupported_feature_channel, __pyx_k_Unsupported_feature_channel, sizeof(__pyx_k_Unsupported_feature_channel), 0, 1, 0, 0},
  {&__pyx_n_s_VChan, __pyx_k_VChan, sizeof(__pyx_k_VChan), 0, 0, 1, 1},
  {&__pyx_n_u_VChan, __pyx_k_VChan, sizeof(__pyx_k_VChan), 0, 1, 0, 1},
  {&__pyx_n_s_ValueError, __pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 0, 1, 1},
  {&__pyx_kp_u_Value_in_red_zone, __pyx_k_Value_in_red_zone, sizeof(__pyx_k_Value_in_red_zone), 0, 1, 0, 0},
  {&__pyx_kp_u_Value_in_yellow_zone, __pyx_k_Value_in_yellow_zone, sizeof(__pyx_k_Value_in_yellow_zone), 0, 1, 0, 0},
  {&__pyx_kp_u_Value_is_weird, __pyx_k_Value_is_weird, sizeof(__pyx_k_Value_is_weird), 0, 1, 0, 0},
  {&__pyx_kp_u_Wrong_device, __pyx_k_Wrong_device, sizeof(__pyx_k_Wrong_device), 0, 1, 0, 0},
  {&__pyx_kp_u__10, __pyx_k__10, sizeof(__pyx_k__10), 0, 1, 0, 0},
  {&__pyx_kp_u__11, __pyx_k__11, sizeof(__pyx_k__11), 0, 1, 0, 0},
  {&__pyx_kp_u__19, __pyx_k__19, sizeof(__pyx_k__19), 0, 1, 0, 0},
  {&__pyx_kp_u__20, __pyx_k__20, sizeof(__pyx_k__20), 0, 1, 0, 0},
  {&__pyx_kp_u__32, __pyx_k__32, sizeof(__pyx_k__32), 0, 1, 0, 0},
  {&__pyx_n_s_all, __pyx_k_all, sizeof(__pyx_k_all), 0, 0, 1, 1},
  {&__pyx_n_u_ascii, __pyx_k_ascii, sizeof(__pyx_k_ascii), 0, 1, 0, 1},
  {&__pyx_n_u_break, __pyx_k_break, sizeof(__pyx_k_break), 0, 1, 0, 1},
  {&__pyx_kp_u_cda_chan_error_cname, __pyx_k_cda_chan_error_cname, sizeof(__pyx_k_cda_chan_error_cname), 0, 1, 0, 0},
  {&__pyx_kp_u_cda_chan_ref, __pyx_k_cda_chan_ref, sizeof(__pyx_k_cda_chan_ref), 0, 1, 0, 0},
  {&__pyx_kp_u_cda_context_error_cid, __pyx_k_cda_context_error_cid, sizeof(__pyx_k_cda_context_error_cid), 0, 1, 0, 0},
  {&__pyx_n_s_cfactory, __pyx_k_cfactory, sizeof(__pyx_k_cfactory), 0, 0, 1, 1},
  {&__pyx_n_u_cfactory, __pyx_k_cfactory, sizeof(__pyx_k_cfactory), 0, 1, 0, 1},
  {&__pyx_n_u_change_sign, __pyx_k_change_sign, sizeof(__pyx_k_change_sign), 0, 1, 0, 1},
  {&__pyx_kp_u_channum, __pyx_k_channum, sizeof(__pyx_k_channum), 0, 1, 0, 0},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_connect, __pyx_k_connect, sizeof(__pyx_k_connect), 0, 0, 1, 1},
  {&__pyx_n_u_context, __pyx_k_context, sizeof(__pyx_k_context), 0, 1, 0, 1},
  {&__pyx_n_s_create_chan, __pyx_k_create_chan, sizeof(__pyx_k_create_chan), 0, 0, 1, 1},
  {&__pyx_kp_u_cx, __pyx_k_cx, sizeof(__pyx_k_cx), 0, 1, 0, 0},
  {&__pyx_n_s_cx_dtype_map, __pyx_k_cx_dtype_map, sizeof(__pyx_k_cx_dtype_map), 0, 0, 1, 1},
  {&__pyx_n_u_d, __pyx_k_d, sizeof(__pyx_k_d), 0, 1, 0, 1},
  {&__pyx_n_u_debug, __pyx_k_debug, sizeof(__pyx_k_debug), 0, 1, 0, 1},
  {&__pyx_n_s_defpfx, __pyx_k_defpfx, sizeof(__pyx_k_defpfx), 0, 0, 1, 1},
  {&__pyx_kp_u_defpfx_2, __pyx_k_defpfx_2, sizeof(__pyx_k_defpfx_2), 0, 1, 0, 0},
  {&__pyx_n_s_dict, __pyx_k_dict, sizeof(__pyx_k_dict), 0, 0, 1, 1},
  {&__pyx_n_s_disconnect, __pyx_k_disconnect, sizeof(__pyx_k_disconnect), 0, 0, 1, 1},
  {&__pyx_n_s_double, __pyx_k_double, sizeof(__pyx_k_double), 0, 0, 1, 1},
  {&__pyx_n_u_double, __pyx_k_double, sizeof(__pyx_k_double), 0, 1, 0, 1},
  {&__pyx_n_s_dsize, __pyx_k_dsize, sizeof(__pyx_k_dsize), 0, 0, 1, 1},
  {&__pyx_n_s_dtype, __pyx_k_dtype, sizeof(__pyx_k_dtype), 0, 0, 1, 1},
  {&__pyx_n_u_dtype, __pyx_k_dtype, sizeof(__pyx_k_dtype), 0, 1, 0, 1},
  {&__pyx_n_s_emit, __pyx_k_emit, sizeof(__pyx_k_emit), 0, 0, 1, 1},
  {&__pyx_n_s_encode, __pyx_k_encode, sizeof(__pyx_k_encode), 0, 0, 1, 1},
  {&__pyx_kp_u_errcode, __pyx_k_errcode, sizeof(__pyx_k_errcode), 0, 1, 0, 0},
  {&__pyx_n_s_file, __pyx_k_file, sizeof(__pyx_k_file), 0, 0, 1, 1},
  {&__pyx_n_s_fileno, __pyx_k_fileno, sizeof(__pyx_k_fileno), 0, 0, 1, 1},
  {&__pyx_n_u_find_only, __pyx_k_find_only, sizeof(__pyx_k_find_only), 0, 1, 0, 1},
  {&__pyx_n_u_float, __pyx_k_float, sizeof(__pyx_k_float), 0, 1, 0, 1},
  {&__pyx_n_u_found, __pyx_k_found, sizeof(__pyx_k_found), 0, 1, 0, 1},
  {&__pyx_n_s_get, __pyx_k_get, sizeof(__pyx_k_get), 0, 0, 1, 1},
  {&__pyx_n_u_get_curval, __pyx_k_get_curval, sizeof(__pyx_k_get_curval), 0, 1, 0, 1},
  {&__pyx_n_s_get_lockstate, __pyx_k_get_lockstate, sizeof(__pyx_k_get_lockstate), 0, 0, 1, 1},
  {&__pyx_n_s_get_range, __pyx_k_get_range, sizeof(__pyx_k_get_range), 0, 0, 1, 1},
  {&__pyx_n_s_get_strings, __pyx_k_get_strings, sizeof(__pyx_k_get_strings), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_n_u_ignore_update, __pyx_k_ignore_update, sizeof(__pyx_k_ignore_update), 0, 1, 0, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_in_c_proc, __pyx_k_in_c_proc, sizeof(__pyx_k_in_c_proc), 0, 0, 1, 1},
  {&__pyx_n_s_in_name, __pyx_k_in_name, sizeof(__pyx_k_in_name), 0, 0, 1, 1},
  {&__pyx_n_s_init, __pyx_k_init, sizeof(__pyx_k_init), 0, 0, 1, 1},
  {&__pyx_n_u_int, __pyx_k_int, sizeof(__pyx_k_int), 0, 1, 0, 1},
  {&__pyx_n_s_int16, __pyx_k_int16, sizeof(__pyx_k_int16), 0, 0, 1, 1},
  {&__pyx_n_u_int16, __pyx_k_int16, sizeof(__pyx_k_int16), 0, 1, 0, 1},
  {&__pyx_n_s_int32, __pyx_k_int32, sizeof(__pyx_k_int32), 0, 0, 1, 1},
  {&__pyx_n_u_int32, __pyx_k_int32, sizeof(__pyx_k_int32), 0, 1, 0, 1},
  {&__pyx_n_s_int64, __pyx_k_int64, sizeof(__pyx_k_int64), 0, 0, 1, 1},
  {&__pyx_n_u_int64, __pyx_k_int64, sizeof(__pyx_k_int64), 0, 1, 0, 1},
  {&__pyx_n_s_int8, __pyx_k_int8, sizeof(__pyx_k_int8), 0, 0, 1, 1},
  {&__pyx_n_u_int8, __pyx_k_int8, sizeof(__pyx_k_int8), 0, 1, 0, 1},
  {&__pyx_n_s_interval, __pyx_k_interval, sizeof(__pyx_k_interval), 0, 0, 1, 1},
  {&__pyx_n_s_is_available, __pyx_k_is_available, sizeof(__pyx_k_is_available), 0, 0, 1, 1},
  {&__pyx_n_s_lock, __pyx_k_lock, sizeof(__pyx_k_lock), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_u_main_loop, __pyx_k_main_loop, sizeof(__pyx_k_main_loop), 0, 1, 0, 1},
  {&__pyx_n_s_mask, __pyx_k_mask, sizeof(__pyx_k_mask), 0, 0, 1, 1},
  {&__pyx_n_u_max_nelems, __pyx_k_max_nelems, sizeof(__pyx_k_max_nelems), 0, 1, 0, 1},
  {&__pyx_n_s_msec, __pyx_k_msec, sizeof(__pyx_k_msec), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_kp_u_name_2, __pyx_k_name_2, sizeof(__pyx_k_name_2), 0, 1, 0, 0},
  {&__pyx_n_s_name_3, __pyx_k_name_3, sizeof(__pyx_k_name_3), 0, 0, 1, 1},
  {&__pyx_kp_u_ndarray_is_not_C_contiguous, __pyx_k_ndarray_is_not_C_contiguous, sizeof(__pyx_k_ndarray_is_not_C_contiguous), 0, 1, 0, 0},
  {&__pyx_kp_u_ndarray_is_not_Fortran_contiguou, __pyx_k_ndarray_is_not_Fortran_contiguou, sizeof(__pyx_k_ndarray_is_not_Fortran_contiguou), 0, 1, 0, 0},
  {&__pyx_n_s_new, __pyx_k_new, sizeof(__pyx_k_new), 0, 0, 1, 1},
  {&__pyx_kp_s_no_default___reduce___due_to_non, __pyx_k_no_default___reduce___due_to_non, sizeof(__pyx_k_no_default___reduce___due_to_non), 0, 0, 1, 0},
  {&__pyx_n_u_no_rd_conv, __pyx_k_no_rd_conv, sizeof(__pyx_k_no_rd_conv), 0, 1, 0, 1},
  {&__pyx_n_u_no_wr_wait, __pyx_k_no_wr_wait, sizeof(__pyx_k_no_wr_wait), 0, 1, 0, 1},
  {&__pyx_kp_u_not_found, __pyx_k_not_found, sizeof(__pyx_k_not_found), 0, 1, 0, 0},
  {&__pyx_n_s_np, __pyx_k_np, sizeof(__pyx_k_np), 0, 0, 1, 1},
  {&__pyx_n_s_numpy, __pyx_k_numpy, sizeof(__pyx_k_numpy), 0, 0, 1, 1},
  {&__pyx_kp_u_numpy_core_multiarray_failed_to, __pyx_k_numpy_core_multiarray_failed_to, sizeof(__pyx_k_numpy_core_multiarray_failed_to), 0, 1, 0, 0},
  {&__pyx_kp_u_numpy_core_umath_failed_to_impor, __pyx_k_numpy_core_umath_failed_to_impor, sizeof(__pyx_k_numpy_core_umath_failed_to_impor), 0, 1, 0, 0},
  {&__pyx_n_s_object, __pyx_k_object, sizeof(__pyx_k_object), 0, 0, 1, 1},
  {&__pyx_n_u_on_update, __pyx_k_on_update, sizeof(__pyx_k_on_update), 0, 1, 0, 1},
  {&__pyx_n_s_order, __pyx_k_order, sizeof(__pyx_k_order), 0, 0, 1, 1},
  {&__pyx_n_u_other_opp_flag, __pyx_k_other_opp_flag, sizeof(__pyx_k_other_opp_flag), 0, 1, 0, 1},
  {&__pyx_n_s_out_c_proc, __pyx_k_out_c_proc, sizeof(__pyx_k_out_c_proc), 0, 0, 1, 1},
  {&__pyx_n_s_out_name, __pyx_k_out_name, sizeof(__pyx_k_out_name), 0, 0, 1, 1},
  {&__pyx_n_s_pickle, __pyx_k_pickle, sizeof(__pyx_k_pickle), 0, 0, 1, 1},
  {&__pyx_n_u_private, __pyx_k_private, sizeof(__pyx_k_private), 0, 1, 0, 1},
  {&__pyx_n_s_proc, __pyx_k_proc, sizeof(__pyx_k_proc), 0, 0, 1, 1},
  {&__pyx_n_s_pycx4_pycda, __pyx_k_pycx4_pycda, sizeof(__pyx_k_pycx4_pycda), 0, 0, 1, 1},
  {&__pyx_kp_s_pycx4_scheduler_pxi, __pyx_k_pycx4_scheduler_pxi, sizeof(__pyx_k_pycx4_scheduler_pxi), 0, 0, 1, 0},
  {&__pyx_n_s_pyx_PickleError, __pyx_k_pyx_PickleError, sizeof(__pyx_k_pyx_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_checksum, __pyx_k_pyx_checksum, sizeof(__pyx_k_pyx_checksum), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_result, __pyx_k_pyx_result, sizeof(__pyx_k_pyx_result), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_state, __pyx_k_pyx_state, sizeof(__pyx_k_pyx_state), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_type, __pyx_k_pyx_type, sizeof(__pyx_k_pyx_type), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_ChanFactory, __pyx_k_pyx_unpickle_ChanFactory, sizeof(__pyx_k_pyx_unpickle_ChanFactory), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_FdEvent, __pyx_k_pyx_unpickle_FdEvent, sizeof(__pyx_k_pyx_unpickle_FdEvent), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_PassGW, __pyx_k_pyx_unpickle_PassGW, sizeof(__pyx_k_pyx_unpickle_PassGW), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_ProcGW, __pyx_k_pyx_unpickle_ProcGW, sizeof(__pyx_k_pyx_unpickle_ProcGW), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_Timer, __pyx_k_pyx_unpickle_Timer, sizeof(__pyx_k_pyx_unpickle_Timer), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_n_s_resize, __pyx_k_resize, sizeof(__pyx_k_resize), 0, 0, 1, 1},
  {&__pyx_n_s_rflags_text, __pyx_k_rflags_text, sizeof(__pyx_k_rflags_text), 0, 0, 1, 1},
  {&__pyx_n_u_searching, __pyx_k_searching, sizeof(__pyx_k_searching), 0, 1, 0, 1},
  {&__pyx_n_s_select_interrupt_action, __pyx_k_select_interrupt_action, sizeof(__pyx_k_select_interrupt_action), 0, 0, 1, 1},
  {&__pyx_kp_s_self_chans_self_events_cannot_be, __pyx_k_self_chans_self_events_cannot_be, sizeof(__pyx_k_self_chans_self_events_cannot_be), 0, 0, 1, 0},
  {&__pyx_kp_s_self_events_cannot_be_converted, __pyx_k_self_events_cannot_be_converted, sizeof(__pyx_k_self_events_cannot_be_converted), 0, 0, 1, 0},
  {&__pyx_n_s_setInterval, __pyx_k_setInterval, sizeof(__pyx_k_setInterval), 0, 0, 1, 1},
  {&__pyx_n_s_setTolerance, __pyx_k_setTolerance, sizeof(__pyx_k_setTolerance), 0, 0, 1, 1},
  {&__pyx_n_s_setValue, __pyx_k_setValue, sizeof(__pyx_k_setValue), 0, 0, 1, 1},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_n_u_shy, __pyx_k_shy, sizeof(__pyx_k_shy), 0, 1, 0, 1},
  {&__pyx_n_s_single, __pyx_k_single, sizeof(__pyx_k_single), 0, 0, 1, 1},
  {&__pyx_n_u_single, __pyx_k_single, sizeof(__pyx_k_single), 0, 1, 0, 1},
  {&__pyx_n_s_singleShot, __pyx_k_singleShot, sizeof(__pyx_k_singleShot), 0, 0, 1, 1},
  {&__pyx_n_s_size, __pyx_k_size, sizeof(__pyx_k_size), 0, 0, 1, 1},
  {&__pyx_n_s_split, __pyx_k_split, sizeof(__pyx_k_split), 0, 0, 1, 1},
  {&__pyx_n_s_start, __pyx_k_start, sizeof(__pyx_k_start), 0, 0, 1, 1},
  {&__pyx_n_s_stop, __pyx_k_stop, sizeof(__pyx_k_stop), 0, 0, 1, 1},
  {&__pyx_n_u_str, __pyx_k_str, sizeof(__pyx_k_str), 0, 1, 0, 1},
  {&__pyx_kp_s_stringsource, __pyx_k_stringsource, sizeof(__pyx_k_stringsource), 0, 0, 1, 0},
  {&__pyx_n_s_super, __pyx_k_super, sizeof(__pyx_k_super), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_u_text, __pyx_k_text, sizeof(__pyx_k_text), 0, 1, 0, 1},
  {&__pyx_n_u_uctext, __pyx_k_uctext, sizeof(__pyx_k_uctext), 0, 1, 0, 1},
  {&__pyx_n_u_uint, __pyx_k_uint, sizeof(__pyx_k_uint), 0, 1, 0, 1},
  {&__pyx_n_s_uint16, __pyx_k_uint16, sizeof(__pyx_k_uint16), 0, 0, 1, 1},
  {&__pyx_n_u_uint16, __pyx_k_uint16, sizeof(__pyx_k_uint16), 0, 1, 0, 1},
  {&__pyx_n_s_uint32, __pyx_k_uint32, sizeof(__pyx_k_uint32), 0, 0, 1, 1},
  {&__pyx_n_u_uint32, __pyx_k_uint32, sizeof(__pyx_k_uint32), 0, 1, 0, 1},
  {&__pyx_n_s_uint64, __pyx_k_uint64, sizeof(__pyx_k_uint64), 0, 0, 1, 1},
  {&__pyx_n_u_uint64, __pyx_k_uint64, sizeof(__pyx_k_uint64), 0, 1, 0, 1},
  {&__pyx_n_s_uint8, __pyx_k_uint8, sizeof(__pyx_k_uint8), 0, 0, 1, 1},
  {&__pyx_n_u_uint8, __pyx_k_uint8, sizeof(__pyx_k_uint8), 0, 1, 0, 1},
  {&__pyx_kp_u_unknown_dtype_code_in_numpy_pxd, __pyx_k_unknown_dtype_code_in_numpy_pxd, sizeof(__pyx_k_unknown_dtype_code_in_numpy_pxd), 0, 1, 0, 0},
  {&__pyx_n_s_unlock, __pyx_k_unlock, sizeof(__pyx_k_unlock), 0, 0, 1, 1},
  {&__pyx_n_s_update, __pyx_k_update, sizeof(__pyx_k_update), 0, 0, 1, 1},
  {&__pyx_n_s_val, __pyx_k_val, sizeof(__pyx_k_val), 0, 0, 1, 1},
  {&__pyx_n_s_valueMeasured, __pyx_k_valueMeasured, sizeof(__pyx_k_valueMeasured), 0, 0, 1, 1},
  {&__pyx_kp_u_value_size_greater_than_channel, __pyx_k_value_size_greater_than_channel, sizeof(__pyx_k_value_size_greater_than_channel), 0, 1, 0, 0},
  {&__pyx_n_s_zeros, __pyx_k_zeros, sizeof(__pyx_k_zeros), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) __PYX_ERR(0, 16, __pyx_L1_error)
  __pyx_builtin_MemoryError = __Pyx_GetBuiltinName(__pyx_n_s_MemoryError); if (!__pyx_builtin_MemoryError) __PYX_ERR(0, 34, __pyx_L1_error)
  __pyx_builtin_TypeError = __Pyx_GetBuiltinName(__pyx_n_s_TypeError); if (!__pyx_builtin_TypeError) __PYX_ERR(1, 2, __pyx_L1_error)
  __pyx_builtin_object = __Pyx_GetBuiltinName(__pyx_n_s_object); if (!__pyx_builtin_object) __PYX_ERR(2, 23, __pyx_L1_error)
  __pyx_builtin_super = __Pyx_GetBuiltinName(__pyx_n_s_super); if (!__pyx_builtin_super) __PYX_ERR(3, 20, __pyx_L1_error)
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_n_s_ValueError); if (!__pyx_builtin_ValueError) __PYX_ERR(17, 272, __pyx_L1_error)
  __pyx_builtin_RuntimeError = __Pyx_GetBuiltinName(__pyx_n_s_RuntimeError); if (!__pyx_builtin_RuntimeError) __PYX_ERR(17, 856, __pyx_L1_error)
  __pyx_builtin_ImportError = __Pyx_GetBuiltinName(__pyx_n_s_ImportError); if (!__pyx_builtin_ImportError) __PYX_ERR(17, 1038, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "pycx4/signal.pxi":29
 *             callback = slot
 *         else:
 *             raise Exception('A function was expected')             # <<<<<<<<<<<<<<
 *         for ind in range(self.cnum):
 *             if callback == <object>self.callbacks[ind]:
 */
  __pyx_tuple_ = PyTuple_Pack(1, __pyx_kp_u_A_function_was_expected); if (unlikely(!__pyx_tuple_)) __PYX_ERR(0, 29, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__2 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__2)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__3 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__3)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "pycx4/timer.pxi":23
 *         self.setInterval(msec)
 *         self.repeat = 1
 *         self.timeout = Signal(object)             # <<<<<<<<<<<<<<
 * 
 *     cpdef stop(self):
 */
  __pyx_tuple__4 = PyTuple_Pack(1, __pyx_builtin_object); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(2, 23, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);

  /* "pycx4/fd_event.pxi":17
 *         self.file = file
 *         self.mask = mask
 *         self.ready = Signal(object)             # <<<<<<<<<<<<<<
 *         self.fhd = sl_add_fd(0, <void*>self, file.fileno(), mask, fd_event_proc, NULL)
 * 
 */
  __pyx_tuple__6 = PyTuple_Pack(1, __pyx_builtin_object); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(5, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__7 = PyTuple_Pack(1, __pyx_kp_s_self_events_cannot_be_converted); if (unlikely(!__pyx_tuple__7)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__7);
  __Pyx_GIVEREF(__pyx_tuple__7);

  /* "(tree fragment)":4
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__8 = PyTuple_Pack(1, __pyx_kp_s_self_events_cannot_be_converted); if (unlikely(!__pyx_tuple__8)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__8);
  __Pyx_GIVEREF(__pyx_tuple__8);

  /* "pycx4/context.pxi":40
 *         self.cid, self.defpfx, self.chans, self.channum = ret, defpfx, NULL, 0
 * 
 *         self.serverCycle = Signal(object)             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
 */
  __pyx_tuple__9 = PyTuple_Pack(1, __pyx_builtin_object); if (unlikely(!__pyx_tuple__9)) __PYX_ERR(3, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__9);
  __Pyx_GIVEREF(__pyx_tuple__9);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.chans,self.events cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.chans,self.events cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__12 = PyTuple_Pack(1, __pyx_kp_s_self_chans_self_events_cannot_be); if (unlikely(!__pyx_tuple__12)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__12);
  __Pyx_GIVEREF(__pyx_tuple__12);

  /* "(tree fragment)":4
 *     raise TypeError("self.chans,self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.chans,self.events cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__13 = PyTuple_Pack(1, __pyx_kp_s_self_chans_self_events_cannot_be); if (unlikely(!__pyx_tuple__13)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__13);
  __Pyx_GIVEREF(__pyx_tuple__13);

  /* "pycx4/basechan.pxi":108
 *             self.dtype = dtype
 * 
 *         self.valueChanged, self.valueMeasured, self.resolve = Signal(object), Signal(object), Signal(object)             # <<<<<<<<<<<<<<
 *         self.stringsUpdate, self.lockStateUpdate = Signal(object), Signal(object)
 * 
 */
  __pyx_tuple__14 = PyTuple_Pack(1, __pyx_builtin_object); if (unlikely(!__pyx_tuple__14)) __PYX_ERR(9, 108, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__14);
  __Pyx_GIVEREF(__pyx_tuple__14);
  __pyx_tuple__15 = PyTuple_Pack(1, __pyx_builtin_object); if (unlikely(!__pyx_tuple__15)) __PYX_ERR(9, 108, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__15);
  __Pyx_GIVEREF(__pyx_tuple__15);
  __pyx_tuple__16 = PyTuple_Pack(1, __pyx_builtin_object); if (unlikely(!__pyx_tuple__16)) __PYX_ERR(9, 108, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__16);
  __Pyx_GIVEREF(__pyx_tuple__16);

  /* "pycx4/basechan.pxi":109
 * 
 *         self.valueChanged, self.valueMeasured, self.resolve = Signal(object), Signal(object), Signal(object)
 *         self.stringsUpdate, self.lockStateUpdate = Signal(object), Signal(object)             # <<<<<<<<<<<<<<
 * 
 *         b_name = name.encode("ascii")
 */
  __pyx_tuple__17 = PyTuple_Pack(1, __pyx_builtin_object); if (unlikely(!__pyx_tuple__17)) __PYX_ERR(9, 109, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__17);
  __Pyx_GIVEREF(__pyx_tuple__17);
  __pyx_tuple__18 = PyTuple_Pack(1, __pyx_builtin_object); if (unlikely(!__pyx_tuple__18)) __PYX_ERR(9, 109, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__18);
  __Pyx_GIVEREF(__pyx_tuple__18);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__21 = PyTuple_Pack(1, __pyx_kp_s_self_events_cannot_be_converted); if (unlikely(!__pyx_tuple__21)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__21);
  __Pyx_GIVEREF(__pyx_tuple__21);

  /* "(tree fragment)":4
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__22 = PyTuple_Pack(1, __pyx_kp_s_self_events_cannot_be_converted); if (unlikely(!__pyx_tuple__22)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__22);
  __Pyx_GIVEREF(__pyx_tuple__22);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__23 = PyTuple_Pack(1, __pyx_kp_s_self_events_cannot_be_converted); if (unlikely(!__pyx_tuple__23)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__23);
  __Pyx_GIVEREF(__pyx_tuple__23);

  /* "(tree fragment)":4
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__24 = PyTuple_Pack(1, __pyx_kp_s_self_events_cannot_be_converted); if (unlikely(!__pyx_tuple__24)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__24);
  __Pyx_GIVEREF(__pyx_tuple__24);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__25 = PyTuple_Pack(1, __pyx_kp_s_self_events_cannot_be_converted); if (unlikely(!__pyx_tuple__25)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__25);
  __Pyx_GIVEREF(__pyx_tuple__25);

  /* "(tree fragment)":4
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__26 = PyTuple_Pack(1, __pyx_kp_s_self_events_cannot_be_converted); if (unlikely(!__pyx_tuple__26)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__26);
  __Pyx_GIVEREF(__pyx_tuple__26);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__27 = PyTuple_Pack(1, __pyx_kp_s_self_events_cannot_be_converted); if (unlikely(!__pyx_tuple__27)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__27);
  __Pyx_GIVEREF(__pyx_tuple__27);

  /* "(tree fragment)":4
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__28 = PyTuple_Pack(1, __pyx_kp_s_self_events_cannot_be_converted); if (unlikely(!__pyx_tuple__28)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__28);
  __Pyx_GIVEREF(__pyx_tuple__28);

  /* "pycx4/vchan.pxi":33
 *     cpdef void setValue(self, np.ndarray value):
 *         if value.size > self.max_nelems:
 *             raise Exception('value size greater than channel.max_nelems')             # <<<<<<<<<<<<<<
 *         dtype = np2cxdtype(value.dtype)
 *         self.snd_data(dtype, value.size, <void*>value.data)
 */
  __pyx_tuple__29 = PyTuple_Pack(1, __pyx_kp_u_value_size_greater_than_channel); if (unlikely(!__pyx_tuple__29)) __PYX_ERR(13, 33, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__29);
  __Pyx_GIVEREF(__pyx_tuple__29);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__30 = PyTuple_Pack(1, __pyx_kp_s_self_events_cannot_be_converted); if (unlikely(!__pyx_tuple__30)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__30);
  __Pyx_GIVEREF(__pyx_tuple__30);

  /* "(tree fragment)":4
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__31 = PyTuple_Pack(1, __pyx_kp_s_self_events_cannot_be_converted); if (unlikely(!__pyx_tuple__31)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__31);
  __Pyx_GIVEREF(__pyx_tuple__31);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__33 = PyTuple_Pack(1, __pyx_kp_s_self_events_cannot_be_converted); if (unlikely(!__pyx_tuple__33)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__33);
  __Pyx_GIVEREF(__pyx_tuple__33);

  /* "(tree fragment)":4
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.events cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__34 = PyTuple_Pack(1, __pyx_kp_s_self_events_cannot_be_converted); if (unlikely(!__pyx_tuple__34)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__34);
  __Pyx_GIVEREF(__pyx_tuple__34);

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":272
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_ARRAY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")             # <<<<<<<<<<<<<<
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 */
  __pyx_tuple__35 = PyTuple_Pack(1, __pyx_kp_u_ndarray_is_not_C_contiguous); if (unlikely(!__pyx_tuple__35)) __PYX_ERR(17, 272, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__35);
  __Pyx_GIVEREF(__pyx_tuple__35);

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":276
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_ARRAY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")             # <<<<<<<<<<<<<<
 * 
 *             info.buf = PyArray_DATA(self)
 */
  __pyx_tuple__36 = PyTuple_Pack(1, __pyx_kp_u_ndarray_is_not_Fortran_contiguou); if (unlikely(!__pyx_tuple__36)) __PYX_ERR(17, 276, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__36);
  __Pyx_GIVEREF(__pyx_tuple__36);

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":306
 *                 if ((descr.byteorder == c'>' and little_endian) or
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")             # <<<<<<<<<<<<<<
 *                 if   t == NPY_BYTE:        f = "b"
 *                 elif t == NPY_UBYTE:       f = "B"
 */
  __pyx_tuple__37 = PyTuple_Pack(1, __pyx_kp_u_Non_native_byte_order_not_suppor); if (unlikely(!__pyx_tuple__37)) __PYX_ERR(17, 306, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__37);
  __Pyx_GIVEREF(__pyx_tuple__37);

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":856
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")             # <<<<<<<<<<<<<<
 * 
 *         if ((child.byteorder == c'>' and little_endian) or
 */
  __pyx_tuple__38 = PyTuple_Pack(1, __pyx_kp_u_Format_string_allocated_too_shor); if (unlikely(!__pyx_tuple__38)) __PYX_ERR(17, 856, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__38);
  __Pyx_GIVEREF(__pyx_tuple__38);

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":860
 *         if ((child.byteorder == c'>' and little_endian) or
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")             # <<<<<<<<<<<<<<
 *             # One could encode it in the format string and have Cython
 *             # complain instead, BUT: < and > in format strings also imply
 */
  __pyx_tuple__37 = PyTuple_Pack(1, __pyx_kp_u_Non_native_byte_order_not_suppor); if (unlikely(!__pyx_tuple__37)) __PYX_ERR(17, 860, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__37);
  __Pyx_GIVEREF(__pyx_tuple__37);

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":880
 *             t = child.type_num
 *             if end - f < 5:
 *                 raise RuntimeError(u"Format string allocated too short.")             # <<<<<<<<<<<<<<
 * 
 *             # Until ticket #99 is fixed, use integers to avoid warnings
 */
  __pyx_tuple__39 = PyTuple_Pack(1, __pyx_kp_u_Format_string_allocated_too_shor_2); if (unlikely(!__pyx_tuple__39)) __PYX_ERR(17, 880, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__39);
  __Pyx_GIVEREF(__pyx_tuple__39);

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1038
 *         _import_array()
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_umath() except -1:
 */
  __pyx_tuple__40 = PyTuple_Pack(1, __pyx_kp_u_numpy_core_multiarray_failed_to); if (unlikely(!__pyx_tuple__40)) __PYX_ERR(17, 1038, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__40);
  __Pyx_GIVEREF(__pyx_tuple__40);

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1044
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_ufunc() except -1:
 */
  __pyx_tuple__41 = PyTuple_Pack(1, __pyx_kp_u_numpy_core_umath_failed_to_impor); if (unlikely(!__pyx_tuple__41)) __PYX_ERR(17, 1044, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__41);
  __Pyx_GIVEREF(__pyx_tuple__41);

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1050
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__41 = PyTuple_Pack(1, __pyx_kp_u_numpy_core_umath_failed_to_impor); if (unlikely(!__pyx_tuple__41)) __PYX_ERR(17, 1050, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__41);
  __Pyx_GIVEREF(__pyx_tuple__41);

  /* "pycx4/scheduler.pxi":11
 * 
 * #  required to react to signals in case of daemon
 * def select_interrupt_action():             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
  __pyx_codeobj__42 = (PyObject*)__Pyx_PyCode_New(0, 0, 0, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_pycx4_scheduler_pxi, __pyx_n_s_select_interrupt_action, 11, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__42)) __PYX_ERR(4, 11, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __pyx_unpickle_Timer(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0xef1b4d4:
 *         from pickle import PickleError as __pyx_PickleError
 */
  __pyx_tuple__43 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__43)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__43);
  __Pyx_GIVEREF(__pyx_tuple__43);
  __pyx_codeobj__44 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__43, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_Timer, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__44)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__45 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__45)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__45);
  __Pyx_GIVEREF(__pyx_tuple__45);
  __pyx_codeobj__46 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__45, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_FdEvent, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__46)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__47 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__47)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__47);
  __Pyx_GIVEREF(__pyx_tuple__47);
  __pyx_codeobj__48 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__47, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_ChanFactory, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__48)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__49 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__49)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__49);
  __Pyx_GIVEREF(__pyx_tuple__49);
  __pyx_codeobj__50 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__49, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_PassGW, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__50)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__51 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__51)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__51);
  __Pyx_GIVEREF(__pyx_tuple__51);
  __pyx_codeobj__52 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__51, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_ProcGW, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__52)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void) {
  __pyx_umethod_PyDict_Type_get.type = (PyObject*)&PyDict_Type;
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(18, 1, __pyx_L1_error);
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) __PYX_ERR(18, 1, __pyx_L1_error)
  __pyx_int_25364090 = PyInt_FromLong(25364090L); if (unlikely(!__pyx_int_25364090)) __PYX_ERR(18, 1, __pyx_L1_error)
  __pyx_int_92273991 = PyInt_FromLong(92273991L); if (unlikely(!__pyx_int_92273991)) __PYX_ERR(18, 1, __pyx_L1_error)
  __pyx_int_94582883 = PyInt_FromLong(94582883L); if (unlikely(!__pyx_int_94582883)) __PYX_ERR(18, 1, __pyx_L1_error)
  __pyx_int_125277981 = PyInt_FromLong(125277981L); if (unlikely(!__pyx_int_125277981)) __PYX_ERR(18, 1, __pyx_L1_error)
  __pyx_int_250721492 = PyInt_FromLong(250721492L); if (unlikely(!__pyx_int_250721492)) __PYX_ERR(18, 1, __pyx_L1_error)
  __pyx_int_neg_1 = PyInt_FromLong(-1); if (unlikely(!__pyx_int_neg_1)) __PYX_ERR(18, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __pyx_v_5pycx4_5pycda_default_context = ((struct __pyx_obj_5pycx4_5pycda_Context *)Py_None); Py_INCREF(Py_None);
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __pyx_vtabptr_5pycx4_5pycda_Signal = &__pyx_vtable_5pycx4_5pycda_Signal;
  __pyx_vtable_5pycx4_5pycda_Signal.connect = (PyObject *(*)(struct __pyx_obj_5pycx4_5pycda_Signal *, PyObject *, int __pyx_skip_dispatch))__pyx_f_5pycx4_5pycda_6Signal_connect;
  __pyx_vtable_5pycx4_5pycda_Signal.disconnect = (PyObject *(*)(struct __pyx_obj_5pycx4_5pycda_Signal *, PyObject *, int __pyx_skip_dispatch))__pyx_f_5pycx4_5pycda_6Signal_disconnect;
  if (PyType_Ready(&__pyx_type_5pycx4_5pycda_Signal) < 0) __PYX_ERR(0, 4, __pyx_L1_error)
  __pyx_type_5pycx4_5pycda_Signal.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_5pycx4_5pycda_Signal.tp_dictoffset && __pyx_type_5pycx4_5pycda_Signal.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_5pycx4_5pycda_Signal.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_5pycx4_5pycda_Signal.tp_dict, __pyx_vtabptr_5pycx4_5pycda_Signal) < 0) __PYX_ERR(0, 4, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_Signal, (PyObject *)&__pyx_type_5pycx4_5pycda_Signal) < 0) __PYX_ERR(0, 4, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_5pycx4_5pycda_Signal) < 0) __PYX_ERR(0, 4, __pyx_L1_error)
  __pyx_ptype_5pycx4_5pycda_Signal = &__pyx_type_5pycx4_5pycda_Signal;
  __pyx_vtabptr_5pycx4_5pycda_Timer = &__pyx_vtable_5pycx4_5pycda_Timer;
  __pyx_vtable_5pycx4_5pycda_Timer.stop = (PyObject *(*)(struct __pyx_obj_5pycx4_5pycda_Timer *, int __pyx_skip_dispatch))__pyx_f_5pycx4_5pycda_5Timer_stop;
  __pyx_vtable_5pycx4_5pycda_Timer.start = (PyObject *(*)(struct __pyx_obj_5pycx4_5pycda_Timer *, int __pyx_skip_dispatch))__pyx_f_5pycx4_5pycda_5Timer_start;
  __pyx_vtable_5pycx4_5pycda_Timer.singleShot = (PyObject *(*)(struct __pyx_obj_5pycx4_5pycda_Timer *, int __pyx_skip_dispatch, struct __pyx_opt_args_5pycx4_5pycda_5Timer_singleShot *__pyx_optional_args))__pyx_f_5pycx4_5pycda_5Timer_singleShot;
  __pyx_vtable_5pycx4_5pycda_Timer.interval = (int (*)(struct __pyx_obj_5pycx4_5pycda_Timer *, int __pyx_skip_dispatch))__pyx_f_5pycx4_5pycda_5Timer_interval;
  __pyx_vtable_5pycx4_5pycda_Timer.setInterval = (void (*)(struct __pyx_obj_5pycx4_5pycda_Timer *, int, int __pyx_skip_dispatch))__pyx_f_5pycx4_5pycda_5Timer_setInterval;
  if (PyType_Ready(&__pyx_type_5pycx4_5pycda_Timer) < 0) __PYX_ERR(2, 12, __pyx_L1_error)
  __pyx_type_5pycx4_5pycda_Timer.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_5pycx4_5pycda_Timer.tp_dictoffset && __pyx_type_5pycx4_5pycda_Timer.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_5pycx4_5pycda_Timer.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_5pycx4_5pycda_Timer.tp_dict, __pyx_vtabptr_5pycx4_5pycda_Timer) < 0) __PYX_ERR(2, 12, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_Timer, (PyObject *)&__pyx_type_5pycx4_5pycda_Timer) < 0) __PYX_ERR(2, 12, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_5pycx4_5pycda_Timer) < 0) __PYX_ERR(2, 12, __pyx_L1_error)
  __pyx_ptype_5pycx4_5pycda_Timer = &__pyx_type_5pycx4_5pycda_Timer;
  if (PyType_Ready(&__pyx_type_5pycx4_5pycda_FdEvent) < 0) __PYX_ERR(5, 8, __pyx_L1_error)
  __pyx_type_5pycx4_5pycda_FdEvent.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_5pycx4_5pycda_FdEvent.tp_dictoffset && __pyx_type_5pycx4_5pycda_FdEvent.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_5pycx4_5pycda_FdEvent.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_FdEvent, (PyObject *)&__pyx_type_5pycx4_5pycda_FdEvent) < 0) __PYX_ERR(5, 8, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_5pycx4_5pycda_FdEvent) < 0) __PYX_ERR(5, 8, __pyx_L1_error)
  __pyx_ptype_5pycx4_5pycda_FdEvent = &__pyx_type_5pycx4_5pycda_FdEvent;
  __pyx_vtabptr_5pycx4_5pycda_CdaObject = &__pyx_vtable_5pycx4_5pycda_CdaObject;
  __pyx_vtable_5pycx4_5pycda_CdaObject.add_event = (int (*)(struct __pyx_obj_5pycx4_5pycda_CdaObject *, int, void *, void *, void *))__pyx_f_5pycx4_5pycda_9CdaObject_add_event;
  __pyx_vtable_5pycx4_5pycda_CdaObject.del_event = (int (*)(struct __pyx_obj_5pycx4_5pycda_CdaObject *, int, void *, void *, void *))__pyx_f_5pycx4_5pycda_9CdaObject_del_event;
  __pyx_vtable_5pycx4_5pycda_CdaObject.register_event = (void (*)(struct __pyx_obj_5pycx4_5pycda_CdaObject *, __pyx_t_5pycx4_5pycda_event *))__pyx_f_5pycx4_5pycda_9CdaObject_register_event;
  __pyx_vtable_5pycx4_5pycda_CdaObject.unregister_event = (void (*)(struct __pyx_obj_5pycx4_5pycda_CdaObject *, __pyx_t_5pycx4_5pycda_event *))__pyx_f_5pycx4_5pycda_9CdaObject_unregister_event;
  if (PyType_Ready(&__pyx_type_5pycx4_5pycda_CdaObject) < 0) __PYX_ERR(8, 3, __pyx_L1_error)
  __pyx_type_5pycx4_5pycda_CdaObject.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_5pycx4_5pycda_CdaObject.tp_dictoffset && __pyx_type_5pycx4_5pycda_CdaObject.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_5pycx4_5pycda_CdaObject.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_5pycx4_5pycda_CdaObject.tp_dict, __pyx_vtabptr_5pycx4_5pycda_CdaObject) < 0) __PYX_ERR(8, 3, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_CdaObject, (PyObject *)&__pyx_type_5pycx4_5pycda_CdaObject) < 0) __PYX_ERR(8, 3, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_5pycx4_5pycda_CdaObject) < 0) __PYX_ERR(8, 3, __pyx_L1_error)
  __pyx_ptype_5pycx4_5pycda_CdaObject = &__pyx_type_5pycx4_5pycda_CdaObject;
  __pyx_vtabptr_5pycx4_5pycda_Context = &__pyx_vtable_5pycx4_5pycda_Context;
  __pyx_vtable_5pycx4_5pycda_Context.__pyx_base = *__pyx_vtabptr_5pycx4_5pycda_CdaObject;
  __pyx_vtable_5pycx4_5pycda_Context.__pyx_base.register_event = (void (*)(struct __pyx_obj_5pycx4_5pycda_CdaObject *, __pyx_t_5pycx4_5pycda_event *))__pyx_f_5pycx4_5pycda_7Context_register_event;
  __pyx_vtable_5pycx4_5pycda_Context.__pyx_base.unregister_event = (void (*)(struct __pyx_obj_5pycx4_5pycda_CdaObject *, __pyx_t_5pycx4_5pycda_event *))__pyx_f_5pycx4_5pycda_7Context_unregister_event;
  __pyx_vtable_5pycx4_5pycda_Context.save_chan = (void (*)(struct __pyx_obj_5pycx4_5pycda_Context *, void *))__pyx_f_5pycx4_5pycda_7Context_save_chan;
  __pyx_vtable_5pycx4_5pycda_Context.drop_chan = (void (*)(struct __pyx_obj_5pycx4_5pycda_Context *, void *))__pyx_f_5pycx4_5pycda_7Context_drop_chan;
  __pyx_vtable_5pycx4_5pycda_Context.check_exception = (void (*)(struct __pyx_obj_5pycx4_5pycda_Context *, int))__pyx_f_5pycx4_5pycda_7Context_check_exception;
  __pyx_type_5pycx4_5pycda_Context.tp_base = __pyx_ptype_5pycx4_5pycda_CdaObject;
  if (PyType_Ready(&__pyx_type_5pycx4_5pycda_Context) < 0) __PYX_ERR(3, 9, __pyx_L1_error)
  __pyx_type_5pycx4_5pycda_Context.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_5pycx4_5pycda_Context.tp_dictoffset && __pyx_type_5pycx4_5pycda_Context.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_5pycx4_5pycda_Context.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_5pycx4_5pycda_Context.tp_dict, __pyx_vtabptr_5pycx4_5pycda_Context) < 0) __PYX_ERR(3, 9, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_Context, (PyObject *)&__pyx_type_5pycx4_5pycda_Context) < 0) __PYX_ERR(3, 9, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_5pycx4_5pycda_Context) < 0) __PYX_ERR(3, 9, __pyx_L1_error)
  __pyx_ptype_5pycx4_5pycda_Context = &__pyx_type_5pycx4_5pycda_Context;
  __pyx_vtabptr_5pycx4_5pycda_BaseChan = &__pyx_vtable_5pycx4_5pycda_BaseChan;
  __pyx_vtable_5pycx4_5pycda_BaseChan.__pyx_base = *__pyx_vtabptr_5pycx4_5pycda_CdaObject;
  __pyx_vtable_5pycx4_5pycda_BaseChan.__pyx_base.register_event = (void (*)(struct __pyx_obj_5pycx4_5pycda_CdaObject *, __pyx_t_5pycx4_5pycda_event *))__pyx_f_5pycx4_5pycda_8BaseChan_register_event;
  __pyx_vtable_5pycx4_5pycda_BaseChan.__pyx_base.unregister_event = (void (*)(struct __pyx_obj_5pycx4_5pycda_CdaObject *, __pyx_t_5pycx4_5pycda_event *))__pyx_f_5pycx4_5pycda_8BaseChan_unregister_event;
  __pyx_vtable_5pycx4_5pycda_BaseChan.cb = (void (*)(struct __pyx_obj_5pycx4_5pycda_BaseChan *))__pyx_f_5pycx4_5pycda_8BaseChan_cb;
  __pyx_vtable_5pycx4_5pycda_BaseChan.snd_data = (int (*)(struct __pyx_obj_5pycx4_5pycda_BaseChan *, cxdtype_t, int, void *))__pyx_f_5pycx4_5pycda_8BaseChan_snd_data;
  __pyx_vtable_5pycx4_5pycda_BaseChan.get_data = (int (*)(struct __pyx_obj_5pycx4_5pycda_BaseChan *, size_t, size_t, void *))__pyx_f_5pycx4_5pycda_8BaseChan_get_data;
  __pyx_vtable_5pycx4_5pycda_BaseChan.current_nelems = (int (*)(struct __pyx_obj_5pycx4_5pycda_BaseChan *))__pyx_f_5pycx4_5pycda_8BaseChan_current_nelems;
  __pyx_vtable_5pycx4_5pycda_BaseChan.get_src = (void (*)(struct __pyx_obj_5pycx4_5pycda_BaseChan *, char const **))__pyx_f_5pycx4_5pycda_8BaseChan_get_src;
  __pyx_vtable_5pycx4_5pycda_BaseChan.check_exception = (void (*)(struct __pyx_obj_5pycx4_5pycda_BaseChan *, int))__pyx_f_5pycx4_5pycda_8BaseChan_check_exception;
  __pyx_vtable_5pycx4_5pycda_BaseChan.is_available = (PyObject *(*)(struct __pyx_obj_5pycx4_5pycda_BaseChan *, int __pyx_skip_dispatch))__pyx_f_5pycx4_5pycda_8BaseChan_is_available;
  __pyx_vtable_5pycx4_5pycda_BaseChan.rflags_text = (PyObject *(*)(struct __pyx_obj_5pycx4_5pycda_BaseChan *, int __pyx_skip_dispatch))__pyx_f_5pycx4_5pycda_8BaseChan_rflags_text;
  __pyx_vtable_5pycx4_5pycda_BaseChan.get_range = (PyObject *(*)(struct __pyx_obj_5pycx4_5pycda_BaseChan *, int __pyx_skip_dispatch))__pyx_f_5pycx4_5pycda_8BaseChan_get_range;
  __pyx_vtable_5pycx4_5pycda_BaseChan.get_strings = (PyObject *(*)(struct __pyx_obj_5pycx4_5pycda_BaseChan *, int __pyx_skip_dispatch))__pyx_f_5pycx4_5pycda_8BaseChan_get_strings;
  __pyx_vtable_5pycx4_5pycda_BaseChan.get_lockstate = (PyObject *(*)(struct __pyx_obj_5pycx4_5pycda_BaseChan *, int __pyx_skip_dispatch))__pyx_f_5pycx4_5pycda_8BaseChan_get_lockstate;
  __pyx_vtable_5pycx4_5pycda_BaseChan.lock = (PyObject *(*)(struct __pyx_obj_5pycx4_5pycda_BaseChan *, int __pyx_skip_dispatch))__pyx_f_5pycx4_5pycda_8BaseChan_lock;
  __pyx_vtable_5pycx4_5pycda_BaseChan.unlock = (PyObject *(*)(struct __pyx_obj_5pycx4_5pycda_BaseChan *, int __pyx_skip_dispatch))__pyx_f_5pycx4_5pycda_8BaseChan_unlock;
  __pyx_type_5pycx4_5pycda_BaseChan.tp_base = __pyx_ptype_5pycx4_5pycda_CdaObject;
  if (PyType_Ready(&__pyx_type_5pycx4_5pycda_BaseChan) < 0) __PYX_ERR(9, 71, __pyx_L1_error)
  __pyx_type_5pycx4_5pycda_BaseChan.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_5pycx4_5pycda_BaseChan.tp_dictoffset && __pyx_type_5pycx4_5pycda_BaseChan.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_5pycx4_5pycda_BaseChan.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_5pycx4_5pycda_BaseChan.tp_dict, __pyx_vtabptr_5pycx4_5pycda_BaseChan) < 0) __PYX_ERR(9, 71, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_BaseChan, (PyObject *)&__pyx_type_5pycx4_5pycda_BaseChan) < 0) __PYX_ERR(9, 71, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_5pycx4_5pycda_BaseChan) < 0) __PYX_ERR(9, 71, __pyx_L1_error)
  __pyx_ptype_5pycx4_5pycda_BaseChan = &__pyx_type_5pycx4_5pycda_BaseChan;
  __pyx_vtabptr_5pycx4_5pycda_DChan = &__pyx_vtable_5pycx4_5pycda_DChan;
  __pyx_vtable_5pycx4_5pycda_DChan.__pyx_base = *__pyx_vtabptr_5pycx4_5pycda_BaseChan;
  __pyx_vtable_5pycx4_5pycda_DChan.__pyx_base.cb = (void (*)(struct __pyx_obj_5pycx4_5pycda_BaseChan *))__pyx_f_5pycx4_5pycda_5DChan_cb;
  __pyx_vtable_5pycx4_5pycda_DChan.setValue = (void (*)(struct __pyx_obj_5pycx4_5pycda_DChan *, double, int __pyx_skip_dispatch))__pyx_f_5pycx4_5pycda_5DChan_setValue;
  __pyx_vtable_5pycx4_5pycda_DChan.setTolerance = (void (*)(struct __pyx_obj_5pycx4_5pycda_DChan *, double, int __pyx_skip_dispatch))__pyx_f_5pycx4_5pycda_5DChan_setTolerance;
  __pyx_type_5pycx4_5pycda_DChan.tp_base = __pyx_ptype_5pycx4_5pycda_BaseChan;
  if (PyType_Ready(&__pyx_type_5pycx4_5pycda_DChan) < 0) __PYX_ERR(10, 2, __pyx_L1_error)
  __pyx_type_5pycx4_5pycda_DChan.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_5pycx4_5pycda_DChan.tp_dictoffset && __pyx_type_5pycx4_5pycda_DChan.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_5pycx4_5pycda_DChan.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_5pycx4_5pycda_DChan.tp_dict, __pyx_vtabptr_5pycx4_5pycda_DChan) < 0) __PYX_ERR(10, 2, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_DChan, (PyObject *)&__pyx_type_5pycx4_5pycda_DChan) < 0) __PYX_ERR(10, 2, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_5pycx4_5pycda_DChan) < 0) __PYX_ERR(10, 2, __pyx_L1_error)
  __pyx_ptype_5pycx4_5pycda_DChan = &__pyx_type_5pycx4_5pycda_DChan;
  __pyx_vtabptr_5pycx4_5pycda_IChan = &__pyx_vtable_5pycx4_5pycda_IChan;
  __pyx_vtable_5pycx4_5pycda_IChan.__pyx_base = *__pyx_vtabptr_5pycx4_5pycda_BaseChan;
  __pyx_vtable_5pycx4_5pycda_IChan.__pyx_base.cb = (void (*)(struct __pyx_obj_5pycx4_5pycda_BaseChan *))__pyx_f_5pycx4_5pycda_5IChan_cb;
  __pyx_vtable_5pycx4_5pycda_IChan.setValue = (void (*)(struct __pyx_obj_5pycx4_5pycda_IChan *, int, int __pyx_skip_dispatch))__pyx_f_5pycx4_5pycda_5IChan_setValue;
  __pyx_vtable_5pycx4_5pycda_IChan.setTolerance = (void (*)(struct __pyx_obj_5pycx4_5pycda_IChan *, int, int __pyx_skip_dispatch))__pyx_f_5pycx4_5pycda_5IChan_setTolerance;
  __pyx_type_5pycx4_5pycda_IChan.tp_base = __pyx_ptype_5pycx4_5pycda_BaseChan;
  if (PyType_Ready(&__pyx_type_5pycx4_5pycda_IChan) < 0) __PYX_ERR(11, 2, __pyx_L1_error)
  __pyx_type_5pycx4_5pycda_IChan.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_5pycx4_5pycda_IChan.tp_dictoffset && __pyx_type_5pycx4_5pycda_IChan.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_5pycx4_5pycda_IChan.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_5pycx4_5pycda_IChan.tp_dict, __pyx_vtabptr_5pycx4_5pycda_IChan) < 0) __PYX_ERR(11, 2, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_IChan, (PyObject *)&__pyx_type_5pycx4_5pycda_IChan) < 0) __PYX_ERR(11, 2, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_5pycx4_5pycda_IChan) < 0) __PYX_ERR(11, 2, __pyx_L1_error)
  __pyx_ptype_5pycx4_5pycda_IChan = &__pyx_type_5pycx4_5pycda_IChan;
  __pyx_vtabptr_5pycx4_5pycda_Chan = &__pyx_vtable_5pycx4_5pycda_Chan;
  __pyx_vtable_5pycx4_5pycda_Chan.__pyx_base = *__pyx_vtabptr_5pycx4_5pycda_BaseChan;
  __pyx_vtable_5pycx4_5pycda_Chan.__pyx_base.cb = (void (*)(struct __pyx_obj_5pycx4_5pycda_BaseChan *))__pyx_f_5pycx4_5pycda_4Chan_cb;
  __pyx_vtable_5pycx4_5pycda_Chan.setValue = (void (*)(struct __pyx_obj_5pycx4_5pycda_Chan *, PyObject *, int __pyx_skip_dispatch))__pyx_f_5pycx4_5pycda_4Chan_setValue;
  __pyx_type_5pycx4_5pycda_Chan.tp_base = __pyx_ptype_5pycx4_5pycda_BaseChan;
  if (PyType_Ready(&__pyx_type_5pycx4_5pycda_Chan) < 0) __PYX_ERR(12, 2, __pyx_L1_error)
  __pyx_type_5pycx4_5pycda_Chan.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_5pycx4_5pycda_Chan.tp_dictoffset && __pyx_type_5pycx4_5pycda_Chan.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_5pycx4_5pycda_Chan.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_5pycx4_5pycda_Chan.tp_dict, __pyx_vtabptr_5pycx4_5pycda_Chan) < 0) __PYX_ERR(12, 2, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_Chan, (PyObject *)&__pyx_type_5pycx4_5pycda_Chan) < 0) __PYX_ERR(12, 2, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_5pycx4_5pycda_Chan) < 0) __PYX_ERR(12, 2, __pyx_L1_error)
  __pyx_ptype_5pycx4_5pycda_Chan = &__pyx_type_5pycx4_5pycda_Chan;
  __pyx_vtabptr_5pycx4_5pycda_VChan = &__pyx_vtable_5pycx4_5pycda_VChan;
  __pyx_vtable_5pycx4_5pycda_VChan.__pyx_base = *__pyx_vtabptr_5pycx4_5pycda_BaseChan;
  __pyx_vtable_5pycx4_5pycda_VChan.__pyx_base.cb = (void (*)(struct __pyx_obj_5pycx4_5pycda_BaseChan *))__pyx_f_5pycx4_5pycda_5VChan_cb;
  __pyx_vtable_5pycx4_5pycda_VChan.setValue = (void (*)(struct __pyx_obj_5pycx4_5pycda_VChan *, PyArrayObject *, int __pyx_skip_dispatch))__pyx_f_5pycx4_5pycda_5VChan_setValue;
  __pyx_type_5pycx4_5pycda_VChan.tp_base = __pyx_ptype_5pycx4_5pycda_BaseChan;
  if (PyType_Ready(&__pyx_type_5pycx4_5pycda_VChan) < 0) __PYX_ERR(13, 4, __pyx_L1_error)
  __pyx_type_5pycx4_5pycda_VChan.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_5pycx4_5pycda_VChan.tp_dictoffset && __pyx_type_5pycx4_5pycda_VChan.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_5pycx4_5pycda_VChan.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_5pycx4_5pycda_VChan.tp_dict, __pyx_vtabptr_5pycx4_5pycda_VChan) < 0) __PYX_ERR(13, 4, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_VChan, (PyObject *)&__pyx_type_5pycx4_5pycda_VChan) < 0) __PYX_ERR(13, 4, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_5pycx4_5pycda_VChan) < 0) __PYX_ERR(13, 4, __pyx_L1_error)
  __pyx_ptype_5pycx4_5pycda_VChan = &__pyx_type_5pycx4_5pycda_VChan;
  __pyx_vtabptr_5pycx4_5pycda_StrChan = &__pyx_vtable_5pycx4_5pycda_StrChan;
  __pyx_vtable_5pycx4_5pycda_StrChan.__pyx_base = *__pyx_vtabptr_5pycx4_5pycda_BaseChan;
  __pyx_vtable_5pycx4_5pycda_StrChan.__pyx_base.cb = (void (*)(struct __pyx_obj_5pycx4_5pycda_BaseChan *))__pyx_f_5pycx4_5pycda_7StrChan_cb;
  __pyx_vtable_5pycx4_5pycda_StrChan.setValue = (void (*)(struct __pyx_obj_5pycx4_5pycda_StrChan *, PyObject *, int __pyx_skip_dispatch))__pyx_f_5pycx4_5pycda_7StrChan_setValue;
  __pyx_type_5pycx4_5pycda_StrChan.tp_base = __pyx_ptype_5pycx4_5pycda_BaseChan;
  if (PyType_Ready(&__pyx_type_5pycx4_5pycda_StrChan) < 0) __PYX_ERR(14, 2, __pyx_L1_error)
  __pyx_type_5pycx4_5pycda_StrChan.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_5pycx4_5pycda_StrChan.tp_dictoffset && __pyx_type_5pycx4_5pycda_StrChan.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_5pycx4_5pycda_StrChan.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_5pycx4_5pycda_StrChan.tp_dict, __pyx_vtabptr_5pycx4_5pycda_StrChan) < 0) __PYX_ERR(14, 2, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_StrChan, (PyObject *)&__pyx_type_5pycx4_5pycda_StrChan) < 0) __PYX_ERR(14, 2, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_5pycx4_5pycda_StrChan) < 0) __PYX_ERR(14, 2, __pyx_L1_error)
  __pyx_ptype_5pycx4_5pycda_StrChan = &__pyx_type_5pycx4_5pycda_StrChan;
  if (PyType_Ready(&__pyx_type_5pycx4_5pycda_ChanFactory) < 0) __PYX_ERR(15, 2, __pyx_L1_error)
  __pyx_type_5pycx4_5pycda_ChanFactory.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_5pycx4_5pycda_ChanFactory.tp_dictoffset && __pyx_type_5pycx4_5pycda_ChanFactory.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_5pycx4_5pycda_ChanFactory.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_ChanFactory, (PyObject *)&__pyx_type_5pycx4_5pycda_ChanFactory) < 0) __PYX_ERR(15, 2, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_5pycx4_5pycda_ChanFactory) < 0) __PYX_ERR(15, 2, __pyx_L1_error)
  __pyx_ptype_5pycx4_5pycda_ChanFactory = &__pyx_type_5pycx4_5pycda_ChanFactory;
  __pyx_vtabptr_5pycx4_5pycda_PassGW = &__pyx_vtable_5pycx4_5pycda_PassGW;
  __pyx_vtable_5pycx4_5pycda_PassGW.in_c_proc = (void (*)(struct __pyx_obj_5pycx4_5pycda_PassGW *, PyObject *, int __pyx_skip_dispatch))__pyx_f_5pycx4_5pycda_6PassGW_in_c_proc;
  __pyx_vtable_5pycx4_5pycda_PassGW.out_c_proc = (void (*)(struct __pyx_obj_5pycx4_5pycda_PassGW *, PyObject *, int __pyx_skip_dispatch))__pyx_f_5pycx4_5pycda_6PassGW_out_c_proc;
  if (PyType_Ready(&__pyx_type_5pycx4_5pycda_PassGW) < 0) __PYX_ERR(16, 6, __pyx_L1_error)
  __pyx_type_5pycx4_5pycda_PassGW.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_5pycx4_5pycda_PassGW.tp_dictoffset && __pyx_type_5pycx4_5pycda_PassGW.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_5pycx4_5pycda_PassGW.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_5pycx4_5pycda_PassGW.tp_dict, __pyx_vtabptr_5pycx4_5pycda_PassGW) < 0) __PYX_ERR(16, 6, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_PassGW, (PyObject *)&__pyx_type_5pycx4_5pycda_PassGW) < 0) __PYX_ERR(16, 6, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_5pycx4_5pycda_PassGW) < 0) __PYX_ERR(16, 6, __pyx_L1_error)
  __pyx_ptype_5pycx4_5pycda_PassGW = &__pyx_type_5pycx4_5pycda_PassGW;
  __pyx_vtabptr_5pycx4_5pycda_ProcGW = &__pyx_vtable_5pycx4_5pycda_ProcGW;
  __pyx_vtable_5pycx4_5pycda_ProcGW.in_c_proc = (void (*)(struct __pyx_obj_5pycx4_5pycda_ProcGW *, PyObject *, int __pyx_skip_dispatch))__pyx_f_5pycx4_5pycda_6ProcGW_in_c_proc;
  __pyx_vtable_5pycx4_5pycda_ProcGW.out_c_proc = (void (*)(struct __pyx_obj_5pycx4_5pycda_ProcGW *, PyObject *, int __pyx_skip_dispatch))__pyx_f_5pycx4_5pycda_6ProcGW_out_c_proc;
  if (PyType_Ready(&__pyx_type_5pycx4_5pycda_ProcGW) < 0) __PYX_ERR(16, 32, __pyx_L1_error)
  __pyx_type_5pycx4_5pycda_ProcGW.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_5pycx4_5pycda_ProcGW.tp_dictoffset && __pyx_type_5pycx4_5pycda_ProcGW.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_5pycx4_5pycda_ProcGW.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_5pycx4_5pycda_ProcGW.tp_dict, __pyx_vtabptr_5pycx4_5pycda_ProcGW) < 0) __PYX_ERR(16, 32, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_ProcGW, (PyObject *)&__pyx_type_5pycx4_5pycda_ProcGW) < 0) __PYX_ERR(16, 32, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_5pycx4_5pycda_ProcGW) < 0) __PYX_ERR(16, 32, __pyx_L1_error)
  __pyx_ptype_5pycx4_5pycda_ProcGW = &__pyx_type_5pycx4_5pycda_ProcGW;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(19, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "type", 
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7cpython_4type_type) __PYX_ERR(19, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(20, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_4bool_bool = __Pyx_ImportType(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "bool", sizeof(PyBoolObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7cpython_4bool_bool) __PYX_ERR(20, 8, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(21, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_7complex_complex = __Pyx_ImportType(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "complex", sizeof(PyComplexObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7cpython_7complex_complex) __PYX_ERR(21, 15, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("numpy"); if (unlikely(!__pyx_t_1)) __PYX_ERR(17, 206, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_5numpy_dtype = __Pyx_ImportType(__pyx_t_1, "numpy", "dtype", sizeof(PyArray_Descr), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_dtype) __PYX_ERR(17, 206, __pyx_L1_error)
  __pyx_ptype_5numpy_flatiter = __Pyx_ImportType(__pyx_t_1, "numpy", "flatiter", sizeof(PyArrayIterObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_flatiter) __PYX_ERR(17, 229, __pyx_L1_error)
  __pyx_ptype_5numpy_broadcast = __Pyx_ImportType(__pyx_t_1, "numpy", "broadcast", sizeof(PyArrayMultiIterObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_broadcast) __PYX_ERR(17, 233, __pyx_L1_error)
  __pyx_ptype_5numpy_ndarray = __Pyx_ImportType(__pyx_t_1, "numpy", "ndarray", sizeof(PyArrayObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_ndarray) __PYX_ERR(17, 242, __pyx_L1_error)
  __pyx_ptype_5numpy_ufunc = __Pyx_ImportType(__pyx_t_1, "numpy", "ufunc", sizeof(PyUFuncObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_ufunc) __PYX_ERR(17, 918, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}


#if PY_MAJOR_VERSION < 3
#ifdef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC void
#else
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#endif
#else
#ifdef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC initpycda(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC initpycda(void)
#else
__Pyx_PyMODINIT_FUNC PyInit_pycda(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_pycda(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    #if PY_VERSION_HEX >= 0x030700A1
    static PY_INT64_T main_interpreter_id = -1;
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return (unlikely(current_id == -1)) ? -1 : 0;
    } else if (unlikely(main_interpreter_id != current_id))
    #else
    static PyInterpreterState *main_interpreter = NULL;
    PyInterpreterState *current_interpreter = PyThreadState_Get()->interp;
    if (!main_interpreter) {
        main_interpreter = current_interpreter;
    } else if (unlikely(main_interpreter != current_interpreter))
    #endif
    {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        result = PyDict_SetItemString(moddict, to_name, value);
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__Pyx_check_single_interpreter())
        return NULL;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__") < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_pycda(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'pycda' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit_pycda(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(18, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(18, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(18, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(18, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(18, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(18, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(18, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(18, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(18, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(18, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("pycda", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(18, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(18, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(18, 1, __pyx_L1_error)
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(18, 1, __pyx_L1_error)
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(18, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(18, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(18, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_pycx4__pycda) {
    if (PyObject_SetAttr(__pyx_m, __pyx_n_s_name_3, __pyx_n_s_main) < 0) __PYX_ERR(18, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(18, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "pycx4.pycda")) {
      if (unlikely(PyDict_SetItemString(modules, "pycx4.pycda", __pyx_m) < 0)) __PYX_ERR(18, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(18, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(18, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  (void)__Pyx_modinit_variable_export_code();
  (void)__Pyx_modinit_function_export_code();
  if (unlikely(__Pyx_modinit_type_init_code() != 0)) goto __pyx_L1_error;
  if (unlikely(__Pyx_modinit_type_import_code() != 0)) goto __pyx_L1_error;
  (void)__Pyx_modinit_variable_import_code();
  (void)__Pyx_modinit_function_import_code();
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(18, 1, __pyx_L1_error)
  #endif

  /* "pycx4/imports.pxi":1
 * import numpy as np             # <<<<<<<<<<<<<<
 * cimport numpy as np
 * from cpython.version cimport PY_MAJOR_VERSION
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(22, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_np, __pyx_t_1) < 0) __PYX_ERR(22, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/imports.pxi":9
 * from cx4.cda cimport *
 * 
 * RSLVSTAT_NOTFOUND = CDA_RSLVSTAT_NOTFOUND             # <<<<<<<<<<<<<<
 * RSLVSTAT_SEARCHING = CDA_RSLVSTAT_SEARCHING
 * RSLVSTAT_FOUND = CDA_RSLVSTAT_FOUND
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(CDA_RSLVSTAT_NOTFOUND); if (unlikely(!__pyx_t_1)) __PYX_ERR(22, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_RSLVSTAT_NOTFOUND, __pyx_t_1) < 0) __PYX_ERR(22, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/imports.pxi":10
 * 
 * RSLVSTAT_NOTFOUND = CDA_RSLVSTAT_NOTFOUND
 * RSLVSTAT_SEARCHING = CDA_RSLVSTAT_SEARCHING             # <<<<<<<<<<<<<<
 * RSLVSTAT_FOUND = CDA_RSLVSTAT_FOUND
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(CDA_RSLVSTAT_SEARCHING); if (unlikely(!__pyx_t_1)) __PYX_ERR(22, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_RSLVSTAT_SEARCHING, __pyx_t_1) < 0) __PYX_ERR(22, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/imports.pxi":11
 * RSLVSTAT_NOTFOUND = CDA_RSLVSTAT_NOTFOUND
 * RSLVSTAT_SEARCHING = CDA_RSLVSTAT_SEARCHING
 * RSLVSTAT_FOUND = CDA_RSLVSTAT_FOUND             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(CDA_RSLVSTAT_FOUND); if (unlikely(!__pyx_t_1)) __PYX_ERR(22, 11, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_RSLVSTAT_FOUND, __pyx_t_1) < 0) __PYX_ERR(22, 11, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/scheduler.pxi":11
 * 
 * #  required to react to signals in case of daemon
 * def select_interrupt_action():             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_5pycx4_5pycda_5select_interrupt_action, NULL, __pyx_n_s_pycx4_pycda); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 11, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_select_interrupt_action, __pyx_t_1) < 0) __PYX_ERR(4, 11, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/scheduler.pxi":17
 *     select_interrupt_action()
 * 
 * sl_set_select_behaviour(<sl_at_select_proc>NULL, <sl_at_select_proc>at_select, 0)             # <<<<<<<<<<<<<<
 * 
 */
  (void)(sl_set_select_behaviour(((sl_at_select_proc)NULL), ((sl_at_select_proc)__pyx_f_5pycx4_5pycda_at_select), 0));

  /* "pycx4/fd_event.pxi":14
 *         sl_fdh_t fhd
 * 
 *     def __init__(self, file, mask=SL_RD):             # <<<<<<<<<<<<<<
 *         self.file = file
 *         self.mask = mask
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SL_RD); if (unlikely(!__pyx_t_1)) __PYX_ERR(5, 14, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_k__5 = __pyx_t_1;
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pycx4/cxdtype.pxi":3
 * # data types declarations and utility functions
 * 
 * DTYPE_UNKNOWN = CXDTYPE_UNKNOWN             # <<<<<<<<<<<<<<
 * DTYPE_INT8 = CXDTYPE_INT8
 * DTYPE_INT16 = CXDTYPE_INT16
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(CXDTYPE_UNKNOWN); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_DTYPE_UNKNOWN, __pyx_t_1) < 0) __PYX_ERR(6, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/cxdtype.pxi":4
 * 
 * DTYPE_UNKNOWN = CXDTYPE_UNKNOWN
 * DTYPE_INT8 = CXDTYPE_INT8             # <<<<<<<<<<<<<<
 * DTYPE_INT16 = CXDTYPE_INT16
 * DTYPE_INT32 = CXDTYPE_INT32
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(CXDTYPE_INT8); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_DTYPE_INT8, __pyx_t_1) < 0) __PYX_ERR(6, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/cxdtype.pxi":5
 * DTYPE_UNKNOWN = CXDTYPE_UNKNOWN
 * DTYPE_INT8 = CXDTYPE_INT8
 * DTYPE_INT16 = CXDTYPE_INT16             # <<<<<<<<<<<<<<
 * DTYPE_INT32 = CXDTYPE_INT32
 * DTYPE_INT64 = CXDTYPE_INT64
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(CXDTYPE_INT16); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_DTYPE_INT16, __pyx_t_1) < 0) __PYX_ERR(6, 5, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/cxdtype.pxi":6
 * DTYPE_INT8 = CXDTYPE_INT8
 * DTYPE_INT16 = CXDTYPE_INT16
 * DTYPE_INT32 = CXDTYPE_INT32             # <<<<<<<<<<<<<<
 * DTYPE_INT64 = CXDTYPE_INT64
 * DTYPE_UINT8 = CXDTYPE_UINT8
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(CXDTYPE_INT32); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_DTYPE_INT32, __pyx_t_1) < 0) __PYX_ERR(6, 6, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/cxdtype.pxi":7
 * DTYPE_INT16 = CXDTYPE_INT16
 * DTYPE_INT32 = CXDTYPE_INT32
 * DTYPE_INT64 = CXDTYPE_INT64             # <<<<<<<<<<<<<<
 * DTYPE_UINT8 = CXDTYPE_UINT8
 * DTYPE_UINT16 = CXDTYPE_UINT16
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(CXDTYPE_INT64); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_DTYPE_INT64, __pyx_t_1) < 0) __PYX_ERR(6, 7, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/cxdtype.pxi":8
 * DTYPE_INT32 = CXDTYPE_INT32
 * DTYPE_INT64 = CXDTYPE_INT64
 * DTYPE_UINT8 = CXDTYPE_UINT8             # <<<<<<<<<<<<<<
 * DTYPE_UINT16 = CXDTYPE_UINT16
 * DTYPE_UINT32 = CXDTYPE_UINT32
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(CXDTYPE_UINT8); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_DTYPE_UINT8, __pyx_t_1) < 0) __PYX_ERR(6, 8, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/cxdtype.pxi":9
 * DTYPE_INT64 = CXDTYPE_INT64
 * DTYPE_UINT8 = CXDTYPE_UINT8
 * DTYPE_UINT16 = CXDTYPE_UINT16             # <<<<<<<<<<<<<<
 * DTYPE_UINT32 = CXDTYPE_UINT32
 * DTYPE_UINT64 = CXDTYPE_UINT64
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(CXDTYPE_UINT16); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_DTYPE_UINT16, __pyx_t_1) < 0) __PYX_ERR(6, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/cxdtype.pxi":10
 * DTYPE_UINT8 = CXDTYPE_UINT8
 * DTYPE_UINT16 = CXDTYPE_UINT16
 * DTYPE_UINT32 = CXDTYPE_UINT32             # <<<<<<<<<<<<<<
 * DTYPE_UINT64 = CXDTYPE_UINT64
 * DTYPE_SINGLE = CXDTYPE_SINGLE
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(CXDTYPE_UINT32); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_DTYPE_UINT32, __pyx_t_1) < 0) __PYX_ERR(6, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/cxdtype.pxi":11
 * DTYPE_UINT16 = CXDTYPE_UINT16
 * DTYPE_UINT32 = CXDTYPE_UINT32
 * DTYPE_UINT64 = CXDTYPE_UINT64             # <<<<<<<<<<<<<<
 * DTYPE_SINGLE = CXDTYPE_SINGLE
 * DTYPE_DOUBLE = CXDTYPE_DOUBLE
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(CXDTYPE_UINT64); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 11, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_DTYPE_UINT64, __pyx_t_1) < 0) __PYX_ERR(6, 11, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/cxdtype.pxi":12
 * DTYPE_UINT32 = CXDTYPE_UINT32
 * DTYPE_UINT64 = CXDTYPE_UINT64
 * DTYPE_SINGLE = CXDTYPE_SINGLE             # <<<<<<<<<<<<<<
 * DTYPE_DOUBLE = CXDTYPE_DOUBLE
 * DTYPE_TEXT = CXDTYPE_TEXT
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(CXDTYPE_SINGLE); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_DTYPE_SINGLE, __pyx_t_1) < 0) __PYX_ERR(6, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/cxdtype.pxi":13
 * DTYPE_UINT64 = CXDTYPE_UINT64
 * DTYPE_SINGLE = CXDTYPE_SINGLE
 * DTYPE_DOUBLE = CXDTYPE_DOUBLE             # <<<<<<<<<<<<<<
 * DTYPE_TEXT = CXDTYPE_TEXT
 * DTYPE_UCTEXT = CXDTYPE_UCTEXT
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(CXDTYPE_DOUBLE); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 13, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_DTYPE_DOUBLE, __pyx_t_1) < 0) __PYX_ERR(6, 13, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/cxdtype.pxi":14
 * DTYPE_SINGLE = CXDTYPE_SINGLE
 * DTYPE_DOUBLE = CXDTYPE_DOUBLE
 * DTYPE_TEXT = CXDTYPE_TEXT             # <<<<<<<<<<<<<<
 * DTYPE_UCTEXT = CXDTYPE_UCTEXT
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(CXDTYPE_TEXT); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 14, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_DTYPE_TEXT, __pyx_t_1) < 0) __PYX_ERR(6, 14, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/cxdtype.pxi":15
 * DTYPE_DOUBLE = CXDTYPE_DOUBLE
 * DTYPE_TEXT = CXDTYPE_TEXT
 * DTYPE_UCTEXT = CXDTYPE_UCTEXT             # <<<<<<<<<<<<<<
 * 
 * cx_dtype_map = {
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(CXDTYPE_UCTEXT); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_DTYPE_UCTEXT, __pyx_t_1) < 0) __PYX_ERR(6, 15, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/cxdtype.pxi":18
 * 
 * cx_dtype_map = {
 *     'int8': CXDTYPE_INT8,             # <<<<<<<<<<<<<<
 *     'int16': CXDTYPE_INT16,
 *     'int32': CXDTYPE_INT32,
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(16); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(CXDTYPE_INT8); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_int8, __pyx_t_2) < 0) __PYX_ERR(6, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pycx4/cxdtype.pxi":19
 * cx_dtype_map = {
 *     'int8': CXDTYPE_INT8,
 *     'int16': CXDTYPE_INT16,             # <<<<<<<<<<<<<<
 *     'int32': CXDTYPE_INT32,
 *     'int': CXDTYPE_INT32,
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(CXDTYPE_INT16); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_int16, __pyx_t_2) < 0) __PYX_ERR(6, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pycx4/cxdtype.pxi":20
 *     'int8': CXDTYPE_INT8,
 *     'int16': CXDTYPE_INT16,
 *     'int32': CXDTYPE_INT32,             # <<<<<<<<<<<<<<
 *     'int': CXDTYPE_INT32,
 *     'int64': CXDTYPE_INT64,
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(CXDTYPE_INT32); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 20, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_int32, __pyx_t_2) < 0) __PYX_ERR(6, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pycx4/cxdtype.pxi":21
 *     'int16': CXDTYPE_INT16,
 *     'int32': CXDTYPE_INT32,
 *     'int': CXDTYPE_INT32,             # <<<<<<<<<<<<<<
 *     'int64': CXDTYPE_INT64,
 *     'uint8': CXDTYPE_UINT8,
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(CXDTYPE_INT32); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_int, __pyx_t_2) < 0) __PYX_ERR(6, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pycx4/cxdtype.pxi":22
 *     'int32': CXDTYPE_INT32,
 *     'int': CXDTYPE_INT32,
 *     'int64': CXDTYPE_INT64,             # <<<<<<<<<<<<<<
 *     'uint8': CXDTYPE_UINT8,
 *     'uint16': CXDTYPE_UINT16,
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(CXDTYPE_INT64); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_int64, __pyx_t_2) < 0) __PYX_ERR(6, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pycx4/cxdtype.pxi":23
 *     'int': CXDTYPE_INT32,
 *     'int64': CXDTYPE_INT64,
 *     'uint8': CXDTYPE_UINT8,             # <<<<<<<<<<<<<<
 *     'uint16': CXDTYPE_UINT16,
 *     'uint32': CXDTYPE_UINT32,
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(CXDTYPE_UINT8); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 23, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_uint8, __pyx_t_2) < 0) __PYX_ERR(6, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pycx4/cxdtype.pxi":24
 *     'int64': CXDTYPE_INT64,
 *     'uint8': CXDTYPE_UINT8,
 *     'uint16': CXDTYPE_UINT16,             # <<<<<<<<<<<<<<
 *     'uint32': CXDTYPE_UINT32,
 *     'uint': CXDTYPE_UINT32,
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(CXDTYPE_UINT16); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_uint16, __pyx_t_2) < 0) __PYX_ERR(6, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pycx4/cxdtype.pxi":25
 *     'uint8': CXDTYPE_UINT8,
 *     'uint16': CXDTYPE_UINT16,
 *     'uint32': CXDTYPE_UINT32,             # <<<<<<<<<<<<<<
 *     'uint': CXDTYPE_UINT32,
 *     'uint64': CXDTYPE_UINT64,
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(CXDTYPE_UINT32); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_uint32, __pyx_t_2) < 0) __PYX_ERR(6, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pycx4/cxdtype.pxi":26
 *     'uint16': CXDTYPE_UINT16,
 *     'uint32': CXDTYPE_UINT32,
 *     'uint': CXDTYPE_UINT32,             # <<<<<<<<<<<<<<
 *     'uint64': CXDTYPE_UINT64,
 *     'single': CXDTYPE_SINGLE,
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(CXDTYPE_UINT32); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_uint, __pyx_t_2) < 0) __PYX_ERR(6, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pycx4/cxdtype.pxi":27
 *     'uint32': CXDTYPE_UINT32,
 *     'uint': CXDTYPE_UINT32,
 *     'uint64': CXDTYPE_UINT64,             # <<<<<<<<<<<<<<
 *     'single': CXDTYPE_SINGLE,
 *     'float': CXDTYPE_SINGLE,
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(CXDTYPE_UINT64); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_uint64, __pyx_t_2) < 0) __PYX_ERR(6, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pycx4/cxdtype.pxi":28
 *     'uint': CXDTYPE_UINT32,
 *     'uint64': CXDTYPE_UINT64,
 *     'single': CXDTYPE_SINGLE,             # <<<<<<<<<<<<<<
 *     'float': CXDTYPE_SINGLE,
 *     'double': CXDTYPE_DOUBLE,
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(CXDTYPE_SINGLE); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_single, __pyx_t_2) < 0) __PYX_ERR(6, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pycx4/cxdtype.pxi":29
 *     'uint64': CXDTYPE_UINT64,
 *     'single': CXDTYPE_SINGLE,
 *     'float': CXDTYPE_SINGLE,             # <<<<<<<<<<<<<<
 *     'double': CXDTYPE_DOUBLE,
 *     'text': CXDTYPE_TEXT,
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(CXDTYPE_SINGLE); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 29, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_float, __pyx_t_2) < 0) __PYX_ERR(6, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pycx4/cxdtype.pxi":30
 *     'single': CXDTYPE_SINGLE,
 *     'float': CXDTYPE_SINGLE,
 *     'double': CXDTYPE_DOUBLE,             # <<<<<<<<<<<<<<
 *     'text': CXDTYPE_TEXT,
 *     'str': CXDTYPE_TEXT,
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(CXDTYPE_DOUBLE); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 30, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_double, __pyx_t_2) < 0) __PYX_ERR(6, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pycx4/cxdtype.pxi":31
 *     'float': CXDTYPE_SINGLE,
 *     'double': CXDTYPE_DOUBLE,
 *     'text': CXDTYPE_TEXT,             # <<<<<<<<<<<<<<
 *     'str': CXDTYPE_TEXT,
 *     'uctext': CXDTYPE_UCTEXT,
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(CXDTYPE_TEXT); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_text, __pyx_t_2) < 0) __PYX_ERR(6, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pycx4/cxdtype.pxi":32
 *     'double': CXDTYPE_DOUBLE,
 *     'text': CXDTYPE_TEXT,
 *     'str': CXDTYPE_TEXT,             # <<<<<<<<<<<<<<
 *     'uctext': CXDTYPE_UCTEXT,
 *     #'unknown': CXDTYPE_UNKNOWN,
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(CXDTYPE_TEXT); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 32, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_str, __pyx_t_2) < 0) __PYX_ERR(6, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pycx4/cxdtype.pxi":33
 *     'text': CXDTYPE_TEXT,
 *     'str': CXDTYPE_TEXT,
 *     'uctext': CXDTYPE_UCTEXT,             # <<<<<<<<<<<<<<
 *     #'unknown': CXDTYPE_UNKNOWN,
 * }
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(CXDTYPE_UCTEXT); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 33, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_uctext, __pyx_t_2) < 0) __PYX_ERR(6, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_cx_dtype_map, __pyx_t_1) < 0) __PYX_ERR(6, 17, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/cda_common.pxi":12
 * include 'context.pxi'
 * 
 * cdef Context default_context=Context()             # <<<<<<<<<<<<<<
 * 
 * include 'basechan.pxi'
 */
  __pyx_t_1 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_5pycx4_5pycda_Context)); if (unlikely(!__pyx_t_1)) __PYX_ERR(23, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_XGOTREF(((PyObject *)__pyx_v_5pycx4_5pycda_default_context));
  __Pyx_DECREF_SET(__pyx_v_5pycx4_5pycda_default_context, ((struct __pyx_obj_5pycx4_5pycda_Context *)__pyx_t_1));
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pycx4/vchan.pxi":1
 * np.import_array()             # <<<<<<<<<<<<<<
 * 
 * # vector-data channel class
 */
  __pyx_t_3 = __pyx_f_5numpy_import_array(); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(13, 1, __pyx_L1_error)

  /* "pycx4/chan_factory.pxi":28
 *         return VChan(name, cx_dtype_map[dtype], dsize, **kwargs)
 * 
 * cfactory = ChanFactory()             # <<<<<<<<<<<<<<
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_5pycx4_5pycda_ChanFactory)); if (unlikely(!__pyx_t_1)) __PYX_ERR(15, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_cfactory, __pyx_t_1) < 0) __PYX_ERR(15, 28, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/cda_all.pxi":1
 * __all__ = ['Context', 'StrChan', 'DChan', 'IChan', 'Chan', 'VChan',             # <<<<<<<<<<<<<<
 *            'Timer', 'Signal',
 *            'DTYPE_UNKNOWN', 'DTYPE_INT8', 'DTYPE_INT16', 'DTYPE_INT32', 'DTYPE_INT64', 'DTYPE_UINT8','DTYPE_UINT16',
 */
  __pyx_t_1 = PyList_New(26); if (unlikely(!__pyx_t_1)) __PYX_ERR(24, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_u_Context);
  __Pyx_GIVEREF(__pyx_n_u_Context);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_u_Context);
  __Pyx_INCREF(__pyx_n_u_StrChan);
  __Pyx_GIVEREF(__pyx_n_u_StrChan);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_u_StrChan);
  __Pyx_INCREF(__pyx_n_u_DChan);
  __Pyx_GIVEREF(__pyx_n_u_DChan);
  PyList_SET_ITEM(__pyx_t_1, 2, __pyx_n_u_DChan);
  __Pyx_INCREF(__pyx_n_u_IChan);
  __Pyx_GIVEREF(__pyx_n_u_IChan);
  PyList_SET_ITEM(__pyx_t_1, 3, __pyx_n_u_IChan);
  __Pyx_INCREF(__pyx_n_u_Chan);
  __Pyx_GIVEREF(__pyx_n_u_Chan);
  PyList_SET_ITEM(__pyx_t_1, 4, __pyx_n_u_Chan);
  __Pyx_INCREF(__pyx_n_u_VChan);
  __Pyx_GIVEREF(__pyx_n_u_VChan);
  PyList_SET_ITEM(__pyx_t_1, 5, __pyx_n_u_VChan);
  __Pyx_INCREF(__pyx_n_u_Timer);
  __Pyx_GIVEREF(__pyx_n_u_Timer);
  PyList_SET_ITEM(__pyx_t_1, 6, __pyx_n_u_Timer);
  __Pyx_INCREF(__pyx_n_u_Signal);
  __Pyx_GIVEREF(__pyx_n_u_Signal);
  PyList_SET_ITEM(__pyx_t_1, 7, __pyx_n_u_Signal);
  __Pyx_INCREF(__pyx_n_u_DTYPE_UNKNOWN);
  __Pyx_GIVEREF(__pyx_n_u_DTYPE_UNKNOWN);
  PyList_SET_ITEM(__pyx_t_1, 8, __pyx_n_u_DTYPE_UNKNOWN);
  __Pyx_INCREF(__pyx_n_u_DTYPE_INT8);
  __Pyx_GIVEREF(__pyx_n_u_DTYPE_INT8);
  PyList_SET_ITEM(__pyx_t_1, 9, __pyx_n_u_DTYPE_INT8);
  __Pyx_INCREF(__pyx_n_u_DTYPE_INT16);
  __Pyx_GIVEREF(__pyx_n_u_DTYPE_INT16);
  PyList_SET_ITEM(__pyx_t_1, 10, __pyx_n_u_DTYPE_INT16);
  __Pyx_INCREF(__pyx_n_u_DTYPE_INT32);
  __Pyx_GIVEREF(__pyx_n_u_DTYPE_INT32);
  PyList_SET_ITEM(__pyx_t_1, 11, __pyx_n_u_DTYPE_INT32);
  __Pyx_INCREF(__pyx_n_u_DTYPE_INT64);
  __Pyx_GIVEREF(__pyx_n_u_DTYPE_INT64);
  PyList_SET_ITEM(__pyx_t_1, 12, __pyx_n_u_DTYPE_INT64);
  __Pyx_INCREF(__pyx_n_u_DTYPE_UINT8);
  __Pyx_GIVEREF(__pyx_n_u_DTYPE_UINT8);
  PyList_SET_ITEM(__pyx_t_1, 13, __pyx_n_u_DTYPE_UINT8);
  __Pyx_INCREF(__pyx_n_u_DTYPE_UINT16);
  __Pyx_GIVEREF(__pyx_n_u_DTYPE_UINT16);
  PyList_SET_ITEM(__pyx_t_1, 14, __pyx_n_u_DTYPE_UINT16);
  __Pyx_INCREF(__pyx_n_u_DTYPE_UINT32);
  __Pyx_GIVEREF(__pyx_n_u_DTYPE_UINT32);
  PyList_SET_ITEM(__pyx_t_1, 15, __pyx_n_u_DTYPE_UINT32);
  __Pyx_INCREF(__pyx_n_u_DTYPE_UINT64);
  __Pyx_GIVEREF(__pyx_n_u_DTYPE_UINT64);
  PyList_SET_ITEM(__pyx_t_1, 16, __pyx_n_u_DTYPE_UINT64);
  __Pyx_INCREF(__pyx_n_u_DTYPE_SINGLE);
  __Pyx_GIVEREF(__pyx_n_u_DTYPE_SINGLE);
  PyList_SET_ITEM(__pyx_t_1, 17, __pyx_n_u_DTYPE_SINGLE);
  __Pyx_INCREF(__pyx_n_u_DTYPE_DOUBLE);
  __Pyx_GIVEREF(__pyx_n_u_DTYPE_DOUBLE);
  PyList_SET_ITEM(__pyx_t_1, 18, __pyx_n_u_DTYPE_DOUBLE);
  __Pyx_INCREF(__pyx_n_u_DTYPE_TEXT);
  __Pyx_GIVEREF(__pyx_n_u_DTYPE_TEXT);
  PyList_SET_ITEM(__pyx_t_1, 19, __pyx_n_u_DTYPE_TEXT);
  __Pyx_INCREF(__pyx_n_u_DTYPE_UCTEXT);
  __Pyx_GIVEREF(__pyx_n_u_DTYPE_UCTEXT);
  PyList_SET_ITEM(__pyx_t_1, 20, __pyx_n_u_DTYPE_UCTEXT);
  __Pyx_INCREF(__pyx_n_u_RSLVSTAT_NOTFOUND);
  __Pyx_GIVEREF(__pyx_n_u_RSLVSTAT_NOTFOUND);
  PyList_SET_ITEM(__pyx_t_1, 21, __pyx_n_u_RSLVSTAT_NOTFOUND);
  __Pyx_INCREF(__pyx_n_u_RSLVSTAT_SEARCHING);
  __Pyx_GIVEREF(__pyx_n_u_RSLVSTAT_SEARCHING);
  PyList_SET_ITEM(__pyx_t_1, 22, __pyx_n_u_RSLVSTAT_SEARCHING);
  __Pyx_INCREF(__pyx_n_u_RSLVSTAT_FOUND);
  __Pyx_GIVEREF(__pyx_n_u_RSLVSTAT_FOUND);
  PyList_SET_ITEM(__pyx_t_1, 23, __pyx_n_u_RSLVSTAT_FOUND);
  __Pyx_INCREF(__pyx_n_u_ChanFactory);
  __Pyx_GIVEREF(__pyx_n_u_ChanFactory);
  PyList_SET_ITEM(__pyx_t_1, 24, __pyx_n_u_ChanFactory);
  __Pyx_INCREF(__pyx_n_u_cfactory);
  __Pyx_GIVEREF(__pyx_n_u_cfactory);
  PyList_SET_ITEM(__pyx_t_1, 25, __pyx_n_u_cfactory);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_all, __pyx_t_1) < 0) __PYX_ERR(24, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/pycda.pyx":10
 * 
 * 
 * __all__ += ['main_loop', 'break_', 'FdEvent', 'PassGW']             # <<<<<<<<<<<<<<
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_all); if (unlikely(!__pyx_t_1)) __PYX_ERR(18, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyList_New(4); if (unlikely(!__pyx_t_2)) __PYX_ERR(18, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_main_loop);
  __Pyx_GIVEREF(__pyx_n_u_main_loop);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_main_loop);
  __Pyx_INCREF(__pyx_n_u_break);
  __Pyx_GIVEREF(__pyx_n_u_break);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_n_u_break);
  __Pyx_INCREF(__pyx_n_u_FdEvent);
  __Pyx_GIVEREF(__pyx_n_u_FdEvent);
  PyList_SET_ITEM(__pyx_t_2, 2, __pyx_n_u_FdEvent);
  __Pyx_INCREF(__pyx_n_u_PassGW);
  __Pyx_GIVEREF(__pyx_n_u_PassGW);
  PyList_SET_ITEM(__pyx_t_2, 3, __pyx_n_u_PassGW);
  __pyx_t_4 = PyNumber_InPlaceAdd(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(18, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_all, __pyx_t_4) < 0) __PYX_ERR(18, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_Timer(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0xef1b4d4:
 *         from pickle import PickleError as __pyx_PickleError
 */
  __pyx_t_4 = PyCFunction_NewEx(&__pyx_mdef_5pycx4_5pycda_7__pyx_unpickle_Timer, NULL, __pyx_n_s_pycx4_pycda); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_Timer, __pyx_t_4) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "(tree fragment)":9
 *         __pyx_unpickle_Timer__set_state(<Timer> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_Timer__set_state(Timer __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.active = __pyx_state[0]; __pyx_result.repeat = __pyx_state[1]; __pyx_result.tid = __pyx_state[2]; __pyx_result.timeout = __pyx_state[3]; __pyx_result.usec = __pyx_state[4]
 *     if len(__pyx_state) > 5 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_4 = PyCFunction_NewEx(&__pyx_mdef_5pycx4_5pycda_9__pyx_unpickle_FdEvent, NULL, __pyx_n_s_pycx4_pycda); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_FdEvent, __pyx_t_4) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_ChanFactory(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0x57ffd47:
 *         from pickle import PickleError as __pyx_PickleError
 */
  __pyx_t_4 = PyCFunction_NewEx(&__pyx_mdef_5pycx4_5pycda_11__pyx_unpickle_ChanFactory, NULL, __pyx_n_s_pycx4_pycda); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_ChanFactory, __pyx_t_4) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "(tree fragment)":9
 *         __pyx_unpickle_ChanFactory__set_state(<ChanFactory> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_ChanFactory__set_state(ChanFactory __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.single_chans = __pyx_state[0]; __pyx_result.vector_chans = __pyx_state[1]
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_4 = PyCFunction_NewEx(&__pyx_mdef_5pycx4_5pycda_13__pyx_unpickle_PassGW, NULL, __pyx_n_s_pycx4_pycda); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_PassGW, __pyx_t_4) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_ProcGW(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0x777971d:
 *         from pickle import PickleError as __pyx_PickleError
 */
  __pyx_t_4 = PyCFunction_NewEx(&__pyx_mdef_5pycx4_5pycda_15__pyx_unpickle_ProcGW, NULL, __pyx_n_s_pycx4_pycda); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_ProcGW, __pyx_t_4) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pycx4/pycda.pyx":1
 * include 'imports.pxi'             # <<<<<<<<<<<<<<
 * include 'scheduler.pxi'
 * include 'signal.pxi'
 */
  __pyx_t_4 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_4)) __PYX_ERR(18, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_4) < 0) __PYX_ERR(18, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "../../../../../../../../export/ctlhomes/oper/python/py36/lib64/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1046
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 * cdef inline int import_ufunc() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init pycx4.pycda", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_CLEAR(__pyx_m);
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init pycx4.pycda");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    } else if (unlikely(PyErr_Occurred())) {
        return NULL;
    }
#else
    result = PyDict_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallNoArg */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_CyFunction_Check(func)))
#else
    if (likely(PyCFunction_Check(func)))
#endif
    {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)(void*)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)(void*)meth)) (self, args, nargs);
    }
}
#endif

/* PyObjectCallOneArg */
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (PyCFunction_GET_FLAGS(func) & METH_FASTCALL) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* WriteUnraisableException */
static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#ifdef _MSC_VER
    else state = (PyGILState_STATE)-1;
#endif
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* KeywordStringCheck */
static int __Pyx_CheckKeywordStrings(
    PyObject *kwdict,
    const char* function_name,
    int kw_allowed)
{
    PyObject* key = 0;
    Py_ssize_t pos = 0;
#if CYTHON_COMPILING_IN_PYPY
    if (!kw_allowed && PyDict_Next(kwdict, &pos, &key, 0))
        goto invalid_keyword;
    return 1;
#else
    while (PyDict_Next(kwdict, &pos, &key, 0)) {
        #if PY_MAJOR_VERSION < 3
        if (unlikely(!PyString_Check(key)))
        #endif
            if (unlikely(!PyUnicode_Check(key)))
                goto invalid_keyword_type;
    }
    if ((!kw_allowed) && unlikely(key))
        goto invalid_keyword;
    return 1;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    return 0;
#endif
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
    return 0;
}

/* PyObjectCall2Args */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args, *result = NULL;
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyFunction_FastCall(function, args, 2);
    }
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyCFunction_FastCall(function, args, 2);
    }
    #endif
    args = PyTuple_New(2);
    if (unlikely(!args)) goto done;
    Py_INCREF(arg1);
    PyTuple_SET_ITEM(args, 0, arg1);
    Py_INCREF(arg2);
    PyTuple_SET_ITEM(args, 1, arg2);
    Py_INCREF(function);
    result = __Pyx_PyObject_Call(function, args, NULL);
    Py_DECREF(args);
    Py_DECREF(function);
done:
    return result;
}

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_CheckExact(key)) || likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (PyUnicode_GET_SIZE(**name) != PyUnicode_GET_SIZE(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (PyUnicode_GET_SIZE(**argname) != PyUnicode_GET_SIZE(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* PyErrExceptionMatches */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    PyObject *exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
    if (unlikely(PyTuple_Check(err)))
        return __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    return __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
}
#endif

/* GetAttr */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *o, PyObject *n) {
#if CYTHON_USE_TYPE_SLOTS
#if PY_MAJOR_VERSION >= 3
    if (likely(PyUnicode_Check(n)))
#else
    if (likely(PyString_Check(n)))
#endif
        return __Pyx_PyObject_GetAttrStr(o, n);
#endif
    return PyObject_GetAttr(o, n);
}

/* GetAttr3 */
static PyObject *__Pyx_GetAttr3Default(PyObject *d) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    Py_INCREF(d);
    return d;
}
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *o, PyObject *n, PyObject *d) {
    PyObject *r = __Pyx_GetAttr(o, n);
    return (likely(r)) ? r : __Pyx_GetAttr3Default(d);
}

/* DictGetItem */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key) {
    PyObject *value;
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (!PyErr_Occurred()) {
            if (unlikely(PyTuple_Check(key))) {
                PyObject* args = PyTuple_Pack(1, key);
                if (likely(args)) {
                    PyErr_SetObject(PyExc_KeyError, args);
                    Py_DECREF(args);
                }
            } else {
                PyErr_SetObject(PyExc_KeyError, key);
            }
        }
        return NULL;
    }
    Py_INCREF(value);
    return value;
}
#endif

/* PyObjectFormat */
#if CYTHON_USE_UNICODE_WRITER
static PyObject* __Pyx_PyObject_Format(PyObject* obj, PyObject* format_spec) {
    int ret;
    _PyUnicodeWriter writer;
    if (likely(PyFloat_CheckExact(obj))) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x03040000
        _PyUnicodeWriter_Init(&writer, 0);
#else
        _PyUnicodeWriter_Init(&writer);
#endif
        ret = _PyFloat_FormatAdvancedWriter(
            &writer,
            obj,
            format_spec, 0, PyUnicode_GET_LENGTH(format_spec));
    } else if (likely(PyLong_CheckExact(obj))) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x03040000
        _PyUnicodeWriter_Init(&writer, 0);
#else
        _PyUnicodeWriter_Init(&writer);
#endif
        ret = _PyLong_FormatAdvancedWriter(
            &writer,
            obj,
            format_spec, 0, PyUnicode_GET_LENGTH(format_spec));
    } else {
        return PyObject_Format(obj, format_spec);
    }
    if (unlikely(ret == -1)) {
        _PyUnicodeWriter_Dealloc(&writer);
        return NULL;
    }
    return _PyUnicodeWriter_Finish(&writer);
}
#endif

/* PyUnicode_Unicode */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_Unicode(PyObject *obj) {
    if (unlikely(obj == Py_None))
        obj = __pyx_kp_u_None;
    return __Pyx_NewRef(obj);
}

/* CIntToDigits */
static const char DIGIT_PAIRS_10[2*10*10+1] = {
    "00010203040506070809"
    "10111213141516171819"
    "20212223242526272829"
    "30313233343536373839"
    "40414243444546474849"
    "50515253545556575859"
    "60616263646566676869"
    "70717273747576777879"
    "80818283848586878889"
    "90919293949596979899"
};
static const char DIGIT_PAIRS_8[2*8*8+1] = {
    "0001020304050607"
    "1011121314151617"
    "2021222324252627"
    "3031323334353637"
    "4041424344454647"
    "5051525354555657"
    "6061626364656667"
    "7071727374757677"
};
static const char DIGITS_HEX[2*16+1] = {
    "0123456789abcdef"
    "0123456789ABCDEF"
};

/* BuildPyUnicode */
static PyObject* __Pyx_PyUnicode_BuildFromAscii(Py_ssize_t ulength, char* chars, int clength,
                                                int prepend_sign, char padding_char) {
    PyObject *uval;
    Py_ssize_t uoffset = ulength - clength;
#if CYTHON_USE_UNICODE_INTERNALS
    Py_ssize_t i;
#if CYTHON_PEP393_ENABLED
    void *udata;
    uval = PyUnicode_New(ulength, 127);
    if (unlikely(!uval)) return NULL;
    udata = PyUnicode_DATA(uval);
#else
    Py_UNICODE *udata;
    uval = PyUnicode_FromUnicode(NULL, ulength);
    if (unlikely(!uval)) return NULL;
    udata = PyUnicode_AS_UNICODE(uval);
#endif
    if (uoffset > 0) {
        i = 0;
        if (prepend_sign) {
            __Pyx_PyUnicode_WRITE(PyUnicode_1BYTE_KIND, udata, 0, '-');
            i++;
        }
        for (; i < uoffset; i++) {
            __Pyx_PyUnicode_WRITE(PyUnicode_1BYTE_KIND, udata, i, padding_char);
        }
    }
    for (i=0; i < clength; i++) {
        __Pyx_PyUnicode_WRITE(PyUnicode_1BYTE_KIND, udata, uoffset+i, chars[i]);
    }
#else
    {
        uval = NULL;
        PyObject *sign = NULL, *padding = NULL;
        if (uoffset > 0) {
            prepend_sign = !!prepend_sign;
            if (uoffset > prepend_sign) {
                padding = PyUnicode_FromOrdinal(padding_char);
                if (likely(padding) && uoffset > prepend_sign + 1) {
                    PyObject *tmp;
                    PyObject *repeat = PyInt_FromSize_t(uoffset - prepend_sign);
                    if (unlikely(!repeat)) goto done_or_error;
                    tmp = PyNumber_Multiply(padding, repeat);
                    Py_DECREF(repeat);
                    Py_DECREF(padding);
                    padding = tmp;
                }
                if (unlikely(!padding)) goto done_or_error;
            }
            if (prepend_sign) {
                sign = PyUnicode_FromOrdinal('-');
                if (unlikely(!sign)) goto done_or_error;
            }
        }
        uval = PyUnicode_DecodeASCII(chars, clength, NULL);
        if (likely(uval) && padding) {
            PyObject *tmp = PyNumber_Add(padding, uval);
            Py_DECREF(uval);
            uval = tmp;
        }
        if (likely(uval) && sign) {
            PyObject *tmp = PyNumber_Add(sign, uval);
            Py_DECREF(uval);
            uval = tmp;
        }
done_or_error:
        Py_XDECREF(padding);
        Py_XDECREF(sign);
    }
#endif
    return uval;
}

/* CIntToPyUnicode */
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned short    uint16_t;
        #else
           typedef unsigned __int16  uint16_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_From_int(int value, Py_ssize_t width, char padding_char, char format_char) {
    char digits[sizeof(int)*3+2];
    char *dpos, *end = digits + sizeof(int)*3+2;
    const char *hex_digits = DIGITS_HEX;
    Py_ssize_t length, ulength;
    int prepend_sign, last_one_off;
    int remaining;
    const int neg_one = (int) ((int) 0 - (int) 1), const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (format_char == 'X') {
        hex_digits += 16;
        format_char = 'x';
    }
    remaining = value;
    last_one_off = 0;
    dpos = end;
    do {
        int digit_pos;
        switch (format_char) {
        case 'o':
            digit_pos = abs((int)(remaining % (8*8)));
            remaining = (int) (remaining / (8*8));
            dpos -= 2;
            *(uint16_t*)dpos = ((uint16_t*)DIGIT_PAIRS_8)[digit_pos];
            last_one_off = (digit_pos < 8);
            break;
        case 'd':
            digit_pos = abs((int)(remaining % (10*10)));
            remaining = (int) (remaining / (10*10));
            dpos -= 2;
            *(uint16_t*)dpos = ((uint16_t*)DIGIT_PAIRS_10)[digit_pos];
            last_one_off = (digit_pos < 10);
            break;
        case 'x':
            *(--dpos) = hex_digits[abs((int)(remaining % 16))];
            remaining = (int) (remaining / 16);
            break;
        default:
            assert(0);
            break;
        }
    } while (unlikely(remaining != 0));
    if (last_one_off) {
        assert(*dpos == '0');
        dpos++;
    }
    length = end - dpos;
    ulength = length;
    prepend_sign = 0;
    if (!is_unsigned && value <= neg_one) {
        if (padding_char == ' ' || width <= length + 1) {
            *(--dpos) = '-';
            ++length;
        } else {
            prepend_sign = 1;
        }
        ++ulength;
    }
    if (width > ulength) {
        ulength = width;
    }
    if (ulength == 1) {
        return PyUnicode_FromOrdinal(*dpos);
    }
    return __Pyx_PyUnicode_BuildFromAscii(ulength, dpos, (int) length, prepend_sign, padding_char);
}

/* JoinPyUnicode */
static PyObject* __Pyx_PyUnicode_Join(PyObject* value_tuple, Py_ssize_t value_count, Py_ssize_t result_ulength,
                                      CYTHON_UNUSED Py_UCS4 max_char) {
#if CYTHON_USE_UNICODE_INTERNALS && CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    PyObject *result_uval;
    int result_ukind;
    Py_ssize_t i, char_pos;
    void *result_udata;
#if CYTHON_PEP393_ENABLED
    result_uval = PyUnicode_New(result_ulength, max_char);
    if (unlikely(!result_uval)) return NULL;
    result_ukind = (max_char <= 255) ? PyUnicode_1BYTE_KIND : (max_char <= 65535) ? PyUnicode_2BYTE_KIND : PyUnicode_4BYTE_KIND;
    result_udata = PyUnicode_DATA(result_uval);
#else
    result_uval = PyUnicode_FromUnicode(NULL, result_ulength);
    if (unlikely(!result_uval)) return NULL;
    result_ukind = sizeof(Py_UNICODE);
    result_udata = PyUnicode_AS_UNICODE(result_uval);
#endif
    char_pos = 0;
    for (i=0; i < value_count; i++) {
        int ukind;
        Py_ssize_t ulength;
        void *udata;
        PyObject *uval = PyTuple_GET_ITEM(value_tuple, i);
        if (unlikely(__Pyx_PyUnicode_READY(uval)))
            goto bad;
        ulength = __Pyx_PyUnicode_GET_LENGTH(uval);
        if (unlikely(!ulength))
            continue;
        if (unlikely(char_pos + ulength < 0))
            goto overflow;
        ukind = __Pyx_PyUnicode_KIND(uval);
        udata = __Pyx_PyUnicode_DATA(uval);
        if (!CYTHON_PEP393_ENABLED || ukind == result_ukind) {
            memcpy((char *)result_udata + char_pos * result_ukind, udata, (size_t) (ulength * result_ukind));
        } else {
            #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030300F0 || defined(_PyUnicode_FastCopyCharacters)
            _PyUnicode_FastCopyCharacters(result_uval, char_pos, uval, 0, ulength);
            #else
            Py_ssize_t j;
            for (j=0; j < ulength; j++) {
                Py_UCS4 uchar = __Pyx_PyUnicode_READ(ukind, udata, j);
                __Pyx_PyUnicode_WRITE(result_ukind, result_udata, char_pos+j, uchar);
            }
            #endif
        }
        char_pos += ulength;
    }
    return result_uval;
overflow:
    PyErr_SetString(PyExc_OverflowError, "join() result is too long for a Python string");
bad:
    Py_DECREF(result_uval);
    return NULL;
#else
    result_ulength++;
    value_count++;
    return PyUnicode_Join(__pyx_empty_unicode, value_tuple);
#endif
}

/* PyObjectFormatAndDecref */
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatSimpleAndDecref(PyObject* s, PyObject* f) {
    if (unlikely(!s)) return NULL;
    if (likely(PyUnicode_CheckExact(s))) return s;
    #if PY_MAJOR_VERSION < 3
    if (likely(PyString_CheckExact(s))) {
        PyObject *result = PyUnicode_FromEncodedObject(s, NULL, "strict");
        Py_DECREF(s);
        return result;
    }
    #endif
    return __Pyx_PyObject_FormatAndDecref(s, f);
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatAndDecref(PyObject* s, PyObject* f) {
    PyObject *result = PyObject_Format(s, f);
    Py_DECREF(s);
    return result;
}

/* ArgTypeTest */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (exact) {
        #if PY_MAJOR_VERSION == 2
        if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    }
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
    return 0;
}

/* UnpackUnboundCMethod */
static int __Pyx_TryUnpackUnboundCMethod(__Pyx_CachedCFunction* target) {
    PyObject *method;
    method = __Pyx_PyObject_GetAttrStr(target->type, *target->method_name);
    if (unlikely(!method))
        return -1;
    target->method = method;
#if CYTHON_COMPILING_IN_CPYTHON
    #if PY_MAJOR_VERSION >= 3
    if (likely(__Pyx_TypeCheck(method, &PyMethodDescr_Type)))
    #endif
    {
        PyMethodDescrObject *descr = (PyMethodDescrObject*) method;
        target->func = descr->d_method->ml_meth;
        target->flag = descr->d_method->ml_flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_STACKLESS);
    }
#endif
    return 0;
}

/* CallUnboundCMethod1 */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg) {
    if (likely(cfunc->func)) {
        int flag = cfunc->flag;
        if (flag == METH_O) {
            return (*(cfunc->func))(self, arg);
        } else if (PY_VERSION_HEX >= 0x030600B1 && flag == METH_FASTCALL) {
            if (PY_VERSION_HEX >= 0x030700A0) {
                return (*(__Pyx_PyCFunctionFast)(void*)(PyCFunction)cfunc->func)(self, &arg, 1);
            } else {
                return (*(__Pyx_PyCFunctionFastWithKeywords)(void*)(PyCFunction)cfunc->func)(self, &arg, 1, NULL);
            }
        } else if (PY_VERSION_HEX >= 0x030700A0 && flag == (METH_FASTCALL | METH_KEYWORDS)) {
            return (*(__Pyx_PyCFunctionFastWithKeywords)(void*)(PyCFunction)cfunc->func)(self, &arg, 1, NULL);
        }
    }
    return __Pyx__CallUnboundCMethod1(cfunc, self, arg);
}
#endif
static PyObject* __Pyx__CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg){
    PyObject *args, *result = NULL;
    if (unlikely(!cfunc->func && !cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_COMPILING_IN_CPYTHON
    if (cfunc->func && (cfunc->flag & METH_VARARGS)) {
        args = PyTuple_New(1);
        if (unlikely(!args)) goto bad;
        Py_INCREF(arg);
        PyTuple_SET_ITEM(args, 0, arg);
        if (cfunc->flag & METH_KEYWORDS)
            result = (*(PyCFunctionWithKeywords)(void*)(PyCFunction)cfunc->func)(self, args, NULL);
        else
            result = (*cfunc->func)(self, args);
    } else {
        args = PyTuple_New(2);
        if (unlikely(!args)) goto bad;
        Py_INCREF(self);
        PyTuple_SET_ITEM(args, 0, self);
        Py_INCREF(arg);
        PyTuple_SET_ITEM(args, 1, arg);
        result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
    }
#else
    args = PyTuple_Pack(2, self, arg);
    if (unlikely(!args)) goto bad;
    result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
#endif
bad:
    Py_XDECREF(args);
    return result;
}

/* CallUnboundCMethod2 */
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030600B1
static CYTHON_INLINE PyObject *__Pyx_CallUnboundCMethod2(__Pyx_CachedCFunction *cfunc, PyObject *self, PyObject *arg1, PyObject *arg2) {
    if (likely(cfunc->func)) {
        PyObject *args[2] = {arg1, arg2};
        if (cfunc->flag == METH_FASTCALL) {
            #if PY_VERSION_HEX >= 0x030700A0
            return (*(__Pyx_PyCFunctionFast)(void*)(PyCFunction)cfunc->func)(self, args, 2);
            #else
            return (*(__Pyx_PyCFunctionFastWithKeywords)(void*)(PyCFunction)cfunc->func)(self, args, 2, NULL);
            #endif
        }
        #if PY_VERSION_HEX >= 0x030700A0
        if (cfunc->flag == (METH_FASTCALL | METH_KEYWORDS))
            return (*(__Pyx_PyCFunctionFastWithKeywords)(void*)(PyCFunction)cfunc->func)(self, args, 2, NULL);
        #endif
    }
    return __Pyx__CallUnboundCMethod2(cfunc, self, arg1, arg2);
}
#endif
static PyObject* __Pyx__CallUnboundCMethod2(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg1, PyObject* arg2){
    PyObject *args, *result = NULL;
    if (unlikely(!cfunc->func && !cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_COMPILING_IN_CPYTHON
    if (cfunc->func && (cfunc->flag & METH_VARARGS)) {
        args = PyTuple_New(2);
        if (unlikely(!args)) goto bad;
        Py_INCREF(arg1);
        PyTuple_SET_ITEM(args, 0, arg1);
        Py_INCREF(arg2);
        PyTuple_SET_ITEM(args, 1, arg2);
        if (cfunc->flag & METH_KEYWORDS)
            result = (*(PyCFunctionWithKeywords)(void*)(PyCFunction)cfunc->func)(self, args, NULL);
        else
            result = (*cfunc->func)(self, args);
    } else {
        args = PyTuple_New(3);
        if (unlikely(!args)) goto bad;
        Py_INCREF(self);
        PyTuple_SET_ITEM(args, 0, self);
        Py_INCREF(arg1);
        PyTuple_SET_ITEM(args, 1, arg1);
        Py_INCREF(arg2);
        PyTuple_SET_ITEM(args, 2, arg2);
        result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
    }
#else
    args = PyTuple_Pack(3, self, arg1, arg2);
    if (unlikely(!args)) goto bad;
    result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
#endif
bad:
    Py_XDECREF(args);
    return result;
}

/* dict_getitem_default */
static PyObject* __Pyx_PyDict_GetItemDefault(PyObject* d, PyObject* key, PyObject* default_value) {
    PyObject* value;
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (unlikely(PyErr_Occurred()))
            return NULL;
        value = default_value;
    }
    Py_INCREF(value);
    if ((1));
#else
    if (PyString_CheckExact(key) || PyUnicode_CheckExact(key) || PyInt_CheckExact(key)) {
        value = PyDict_GetItem(d, key);
        if (unlikely(!value)) {
            value = default_value;
        }
        Py_INCREF(value);
    }
#endif
    else {
        if (default_value == Py_None)
            value = __Pyx_CallUnboundCMethod1(&__pyx_umethod_PyDict_Type_get, d, key);
        else
            value = __Pyx_CallUnboundCMethod2(&__pyx_umethod_PyDict_Type_get, d, key, default_value);
    }
    return value;
}

/* ExtTypeTest */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* ObjectGetItem */
#if CYTHON_USE_TYPE_SLOTS
static PyObject *__Pyx_PyObject_GetIndex(PyObject *obj, PyObject* index) {
    PyObject *runerr;
    Py_ssize_t key_value;
    PySequenceMethods *m = Py_TYPE(obj)->tp_as_sequence;
    if (unlikely(!(m && m->sq_item))) {
        PyErr_Format(PyExc_TypeError, "'%.200s' object is not subscriptable", Py_TYPE(obj)->tp_name);
        return NULL;
    }
    key_value = __Pyx_PyIndex_AsSsize_t(index);
    if (likely(key_value != -1 || !(runerr = PyErr_Occurred()))) {
        return __Pyx_GetItemInt_Fast(obj, key_value, 0, 1, 1);
    }
    if (PyErr_GivenExceptionMatches(runerr, PyExc_OverflowError)) {
        PyErr_Clear();
        PyErr_Format(PyExc_IndexError, "cannot fit '%.200s' into an index-sized integer", Py_TYPE(index)->tp_name);
    }
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key) {
    PyMappingMethods *m = Py_TYPE(obj)->tp_as_mapping;
    if (likely(m && m->mp_subscript)) {
        return m->mp_subscript(obj, key);
    }
    return __Pyx_PyObject_GetIndex(obj, key);
}
#endif

/* decode_c_bytes */
static CYTHON_INLINE PyObject* __Pyx_decode_c_bytes(
         const char* cstring, Py_ssize_t length, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors)) {
    if (unlikely((start < 0) | (stop < 0))) {
        if (start < 0) {
            start += length;
            if (start < 0)
                start = 0;
        }
        if (stop < 0)
            stop += length;
    }
    if (stop > length)
        stop = length;
    length = stop - start;
    if (unlikely(length <= 0))
        return PyUnicode_FromUnicode(NULL, 0);
    cstring += start;
    if (decode_func) {
        return decode_func(cstring, length, errors);
    } else {
        return PyUnicode_Decode(cstring, length, encoding, errors);
    }
}

/* BytesEquals */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
#if PY_MAJOR_VERSION < 3
    PyObject* owned_ref = NULL;
#endif
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
#if PY_MAJOR_VERSION < 3
    if ((s1_is_unicode & (!s2_is_unicode)) && PyString_CheckExact(s2)) {
        owned_ref = PyUnicode_FromObject(s2);
        if (unlikely(!owned_ref))
            return -1;
        s2 = owned_ref;
        s2_is_unicode = 1;
    } else if ((s2_is_unicode & (!s1_is_unicode)) && PyString_CheckExact(s1)) {
        owned_ref = PyUnicode_FromObject(s1);
        if (unlikely(!owned_ref))
            return -1;
        s1 = owned_ref;
        s1_is_unicode = 1;
    } else if (((!s2_is_unicode) & (!s1_is_unicode))) {
        return __Pyx_PyBytes_Equals(s1, s2, equals);
    }
#endif
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length;
        int kind;
        void *data1, *data2;
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        if (length != __Pyx_PyUnicode_GET_LENGTH(s2)) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
        #if CYTHON_PEP393_ENABLED
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
        #else
            hash1 = ((PyUnicodeObject*)s1)->hash;
            hash2 = ((PyUnicodeObject*)s2)->hash;
        #endif
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            #if PY_MAJOR_VERSION < 3
            Py_XDECREF(owned_ref);
            #endif
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        #if PY_MAJOR_VERSION < 3
        Py_XDECREF(owned_ref);
        #endif
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_EQ);
return_ne:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_NE);
#endif
}

/* PyIntCompare */
static CYTHON_INLINE PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, CYTHON_UNUSED long inplace) {
    if (op1 == op2) {
        Py_RETURN_TRUE;
    }
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long a = PyInt_AS_LONG(op1);
        if (a == b) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        int unequal;
        unsigned long uintval;
        Py_ssize_t size = Py_SIZE(op1);
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        if (intval == 0) {
            if (size == 0) Py_RETURN_TRUE; else Py_RETURN_FALSE;
        } else if (intval < 0) {
            if (size >= 0)
                Py_RETURN_FALSE;
            intval = -intval;
            size = -size;
        } else {
            if (size <= 0)
                Py_RETURN_FALSE;
        }
        uintval = (unsigned long) intval;
#if PyLong_SHIFT * 4 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 4)) {
            unequal = (size != 5) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[4] != ((uintval >> (4 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 3 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 3)) {
            unequal = (size != 4) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 2 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 2)) {
            unequal = (size != 3) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 1 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 1)) {
            unequal = (size != 2) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
            unequal = (size != 1) || (((unsigned long) digits[0]) != (uintval & (unsigned long) PyLong_MASK));
        if (unequal == 0) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
        if ((double)a == (double)b) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    return (
        PyObject_RichCompare(op1, op2, Py_EQ));
}

/* Import */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.')) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, (PyObject *)NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* ImportFrom */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* HasAttr */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!__Pyx_PyBaseString_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_GetAttr(o, n);
    if (unlikely(!r)) {
        PyErr_Clear();
        return 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}

/* RaiseTooManyValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* RaiseNeedMoreValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* RaiseNoneIterError */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
}

/* GetTopmostException */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem *
__Pyx_PyErr_GetTopmostException(PyThreadState *tstate)
{
    _PyErr_StackItem *exc_info = tstate->exc_info;
    while ((exc_info->exc_type == NULL || exc_info->exc_type == Py_None) &&
           exc_info->previous_item != NULL)
    {
        exc_info = exc_info->previous_item;
    }
    return exc_info;
}
#endif

/* SaveResetException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    *type = exc_info->exc_type;
    *value = exc_info->exc_value;
    *tb = exc_info->exc_traceback;
    #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    #endif
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = type;
    exc_info->exc_value = value;
    exc_info->exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
#endif

/* GetException */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb)
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb)
#endif
{
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if CYTHON_USE_EXC_INFO_STACK
    {
        _PyErr_StackItem *exc_info = tstate->exc_info;
        tmp_type = exc_info->exc_type;
        tmp_value = exc_info->exc_value;
        tmp_tb = exc_info->exc_traceback;
        exc_info->exc_type = local_type;
        exc_info->exc_value = local_value;
        exc_info->exc_traceback = local_tb;
    }
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

/* CallNextTpTraverse */
static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_traverse != current_tp_traverse)
        type = type->tp_base;
    while (type && type->tp_traverse == current_tp_traverse)
        type = type->tp_base;
    if (type && type->tp_traverse)
        return type->tp_traverse(obj, v, a);
    return 0;
}

/* CallNextTpClear */
static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_clear) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_clear != current_tp_clear)
        type = type->tp_base;
    while (type && type->tp_clear == current_tp_clear)
        type = type->tp_base;
    if (type && type->tp_clear)
        type->tp_clear(obj);
}

/* PyObject_GenericGetAttrNoDict */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject *__Pyx_RaiseGenericGetAttributeError(PyTypeObject *tp, PyObject *attr_name) {
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, attr_name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(attr_name));
#endif
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name) {
    PyObject *descr;
    PyTypeObject *tp = Py_TYPE(obj);
    if (unlikely(!PyString_Check(attr_name))) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    assert(!tp->tp_dictoffset);
    descr = _PyType_Lookup(tp, attr_name);
    if (unlikely(!descr)) {
        return __Pyx_RaiseGenericGetAttributeError(tp, attr_name);
    }
    Py_INCREF(descr);
    #if PY_MAJOR_VERSION < 3
    if (likely(PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_HAVE_CLASS)))
    #endif
    {
        descrgetfunc f = Py_TYPE(descr)->tp_descr_get;
        if (unlikely(f)) {
            PyObject *res = f(descr, obj, (PyObject *)tp);
            Py_DECREF(descr);
            return res;
        }
    }
    return descr;
}
#endif

/* PyObject_GenericGetAttr */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name) {
    if (unlikely(Py_TYPE(obj)->tp_dictoffset)) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    return __Pyx_PyObject_GenericGetAttrNoDict(obj, attr_name);
}
#endif

/* SetVTable */
static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* SetupReduce */
static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStr(meth, __pyx_n_s_name_3);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    if (_PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate)) goto GOOD;
#else
    if (PyObject_HasAttr(type_obj, __pyx_n_s_getstate)) goto GOOD;
#endif
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_cython); if (unlikely(!reduce_cython)) goto BAD;
            ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto BAD;
            ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto BAD;
            setstate = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate_cython); if (unlikely(!setstate_cython)) goto BAD;
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto BAD;
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto GOOD;
BAD:
    if (!PyErr_Occurred())
        PyErr_Format(PyExc_RuntimeError, "Unable to initialize pickling for %s", ((PyTypeObject*)type_obj)->tp_name);
    ret = -1;
GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* TypeImport */
#ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(PyObject *module, const char *module_name, const char *class_name,
    size_t size, enum __Pyx_ImportType_CheckSize check_size)
{
    PyObject *result = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    result = PyObject_GetAttrString(module, class_name);
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if ((size_t)basicsize < size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    if (check_size == __Pyx_ImportType_CheckSize_Error && (size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    else if (check_size == __Pyx_ImportType_CheckSize_Warn && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(result);
    return NULL;
}
#endif

/* CLineInTraceback */
#ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback))
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, (size_t)new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
    const int neg_one = (int) ((int) 0 - (int) 1), const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_sl_tid_t(sl_tid_t value) {
    const sl_tid_t neg_one = (sl_tid_t) ((sl_tid_t) 0 - (sl_tid_t) 1), const_zero = (sl_tid_t) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(sl_tid_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(sl_tid_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(sl_tid_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(sl_tid_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(sl_tid_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(sl_tid_t),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_sl_fdh_t(sl_fdh_t value) {
    const sl_fdh_t neg_one = (sl_fdh_t) ((sl_fdh_t) 0 - (sl_fdh_t) 1), const_zero = (sl_fdh_t) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(sl_fdh_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(sl_fdh_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(sl_fdh_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(sl_fdh_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(sl_fdh_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(sl_fdh_t),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int32(int32 value) {
    const int32 neg_one = (int32) ((int32) 0 - (int32) 1), const_zero = (int32) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int32) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int32) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int32) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int32) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int32) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int32),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int8(int8 value) {
    const int8 neg_one = (int8) ((int8) 0 - (int8) 1), const_zero = (int8) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int8) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int8) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int8) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int8) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int8) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int8),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int16(int16 value) {
    const int16 neg_one = (int16) ((int16) 0 - (int16) 1), const_zero = (int16) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int16) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int16) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int16) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int16) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int16) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int16),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int64(int64 value) {
    const int64 neg_one = (int64) ((int64) 0 - (int64) 1), const_zero = (int64) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int64) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int64) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int64) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int64) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int64) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int64),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint8(uint8 value) {
    const uint8 neg_one = (uint8) ((uint8) 0 - (uint8) 1), const_zero = (uint8) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(uint8) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(uint8) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(uint8) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(uint8) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(uint8) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(uint8),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint16(uint16 value) {
    const uint16 neg_one = (uint16) ((uint16) 0 - (uint16) 1), const_zero = (uint16) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(uint16) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(uint16) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(uint16) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(uint16) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(uint16) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(uint16),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint32(uint32 value) {
    const uint32 neg_one = (uint32) ((uint32) 0 - (uint32) 1), const_zero = (uint32) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(uint32) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(uint32) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(uint32) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(uint32) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(uint32) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(uint32),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint64(uint64 value) {
    const uint64 neg_one = (uint64) ((uint64) 0 - (uint64) 1), const_zero = (uint64) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(uint64) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(uint64) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(uint64) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(uint64) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(uint64) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(uint64),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_char8(char8 value) {
    const char8 neg_one = (char8) ((char8) 0 - (char8) 1), const_zero = (char8) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(char8) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(char8) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(char8) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(char8) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(char8) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(char8),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_cda_context_t(cda_context_t value) {
    const cda_context_t neg_one = (cda_context_t) ((cda_context_t) 0 - (cda_context_t) 1), const_zero = (cda_context_t) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(cda_context_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(cda_context_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(cda_context_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(cda_context_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(cda_context_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(cda_context_t),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_CxDataRef_t(CxDataRef_t value) {
    const CxDataRef_t neg_one = (CxDataRef_t) ((CxDataRef_t) 0 - (CxDataRef_t) 1), const_zero = (CxDataRef_t) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(CxDataRef_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(CxDataRef_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(CxDataRef_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(CxDataRef_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(CxDataRef_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(CxDataRef_t),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) ((long) 0 - (long) 1), const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* Declarations */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return ::std::complex< float >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return x + y*(__pyx_t_float_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      __pyx_t_float_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

/* Arithmetic */
#if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eq_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sum_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_diff_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prod_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    #if 1
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        if (b.imag == 0) {
            return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else if (fabsf(b.real) >= fabsf(b.imag)) {
            if (b.real == 0 && b.imag == 0) {
                return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.imag);
            } else {
                float r = b.imag / b.real;
                float s = 1.0 / (b.real + b.imag * r);
                return __pyx_t_float_complex_from_parts(
                    (a.real + a.imag * r) * s, (a.imag - a.real * r) * s);
            }
        } else {
            float r = b.real / b.imag;
            float s = 1.0 / (b.imag + b.real * r);
            return __pyx_t_float_complex_from_parts(
                (a.real * r + a.imag) * s, (a.imag * r - a.real) * s);
        }
    }
    #else
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        if (b.imag == 0) {
            return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else {
            float denom = b.real * b.real + b.imag * b.imag;
            return __pyx_t_float_complex_from_parts(
                (a.real * b.real + a.imag * b.imag) / denom,
                (a.imag * b.real - a.real * b.imag) / denom);
        }
    }
    #endif
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_neg_float(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero_float(__pyx_t_float_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conj_float(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE float __Pyx_c_abs_float(__pyx_t_float_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrtf(z.real*z.real + z.imag*z.imag);
          #else
            return hypotf(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_pow_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
            __pyx_t_float_complex z;
            float r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    float denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        z = __Pyx_c_prod_float(a, a);
                        return __Pyx_c_prod_float(a, a);
                    case 3:
                        z = __Pyx_c_prod_float(a, a);
                        return __Pyx_c_prod_float(z, a);
                    case 4:
                        z = __Pyx_c_prod_float(a, a);
                        return __Pyx_c_prod_float(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                } else if (b.imag == 0) {
                    z.real = powf(a.real, b.real);
                    z.imag = 0;
                    return z;
                } else if (a.real > 0) {
                    r = a.real;
                    theta = 0;
                } else {
                    r = -a.real;
                    theta = atan2f(0, -1);
                }
            } else {
                r = __Pyx_c_abs_float(a);
                theta = atan2f(a.imag, a.real);
            }
            lnr = logf(r);
            z_r = expf(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cosf(z_theta);
            z.imag = z_r * sinf(z_theta);
            return z;
        }
    #endif
#endif

/* Declarations */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return ::std::complex< double >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return x + y*(__pyx_t_double_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      __pyx_t_double_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

/* Arithmetic */
#if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eq_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    #if 1
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        if (b.imag == 0) {
            return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else if (fabs(b.real) >= fabs(b.imag)) {
            if (b.real == 0 && b.imag == 0) {
                return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.imag);
            } else {
                double r = b.imag / b.real;
                double s = 1.0 / (b.real + b.imag * r);
                return __pyx_t_double_complex_from_parts(
                    (a.real + a.imag * r) * s, (a.imag - a.real * r) * s);
            }
        } else {
            double r = b.real / b.imag;
            double s = 1.0 / (b.imag + b.real * r);
            return __pyx_t_double_complex_from_parts(
                (a.real * r + a.imag) * s, (a.imag * r - a.real) * s);
        }
    }
    #else
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        if (b.imag == 0) {
            return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else {
            double denom = b.real * b.real + b.imag * b.imag;
            return __pyx_t_double_complex_from_parts(
                (a.real * b.real + a.imag * b.imag) / denom,
                (a.imag * b.real - a.real * b.imag) / denom);
        }
    }
    #endif
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg_double(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero_double(__pyx_t_double_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj_double(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs_double(__pyx_t_double_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrt(z.real*z.real + z.imag*z.imag);
          #else
            return hypot(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
            __pyx_t_double_complex z;
            double r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    double denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        z = __Pyx_c_prod_double(a, a);
                        return __Pyx_c_prod_double(a, a);
                    case 3:
                        z = __Pyx_c_prod_double(a, a);
                        return __Pyx_c_prod_double(z, a);
                    case 4:
                        z = __Pyx_c_prod_double(a, a);
                        return __Pyx_c_prod_double(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                } else if (b.imag == 0) {
                    z.real = pow(a.real, b.real);
                    z.imag = 0;
                    return z;
                } else if (a.real > 0) {
                    r = a.real;
                    theta = 0;
                } else {
                    r = -a.real;
                    theta = atan2(0, -1);
                }
            } else {
                r = __Pyx_c_abs_double(a);
                theta = atan2(a.imag, a.real);
            }
            lnr = log(r);
            z_r = exp(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cos(z_theta);
            z.imag = z_r * sin(z_theta);
            return z;
        }
    #endif
#endif

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__NPY_TYPES(enum NPY_TYPES value) {
    const enum NPY_TYPES neg_one = (enum NPY_TYPES) ((enum NPY_TYPES) 0 - (enum NPY_TYPES) 1), const_zero = (enum NPY_TYPES) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(enum NPY_TYPES) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(enum NPY_TYPES) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enum NPY_TYPES) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(enum NPY_TYPES) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enum NPY_TYPES) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(enum NPY_TYPES),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) ((int) 0 - (int) 1), const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntFromPy */
static CYTHON_INLINE sl_tid_t __Pyx_PyInt_As_sl_tid_t(PyObject *x) {
    const sl_tid_t neg_one = (sl_tid_t) ((sl_tid_t) 0 - (sl_tid_t) 1), const_zero = (sl_tid_t) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(sl_tid_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(sl_tid_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (sl_tid_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (sl_tid_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(sl_tid_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(sl_tid_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(sl_tid_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(sl_tid_t) >= 2 * PyLong_SHIFT) {
                            return (sl_tid_t) (((((sl_tid_t)digits[1]) << PyLong_SHIFT) | (sl_tid_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(sl_tid_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(sl_tid_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(sl_tid_t) >= 3 * PyLong_SHIFT) {
                            return (sl_tid_t) (((((((sl_tid_t)digits[2]) << PyLong_SHIFT) | (sl_tid_t)digits[1]) << PyLong_SHIFT) | (sl_tid_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(sl_tid_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(sl_tid_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(sl_tid_t) >= 4 * PyLong_SHIFT) {
                            return (sl_tid_t) (((((((((sl_tid_t)digits[3]) << PyLong_SHIFT) | (sl_tid_t)digits[2]) << PyLong_SHIFT) | (sl_tid_t)digits[1]) << PyLong_SHIFT) | (sl_tid_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (sl_tid_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(sl_tid_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(sl_tid_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(sl_tid_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(sl_tid_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (sl_tid_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(sl_tid_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(sl_tid_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(sl_tid_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(sl_tid_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(sl_tid_t) - 1 > 2 * PyLong_SHIFT) {
                            return (sl_tid_t) (((sl_tid_t)-1)*(((((sl_tid_t)digits[1]) << PyLong_SHIFT) | (sl_tid_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(sl_tid_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(sl_tid_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(sl_tid_t) - 1 > 2 * PyLong_SHIFT) {
                            return (sl_tid_t) ((((((sl_tid_t)digits[1]) << PyLong_SHIFT) | (sl_tid_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(sl_tid_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(sl_tid_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(sl_tid_t) - 1 > 3 * PyLong_SHIFT) {
                            return (sl_tid_t) (((sl_tid_t)-1)*(((((((sl_tid_t)digits[2]) << PyLong_SHIFT) | (sl_tid_t)digits[1]) << PyLong_SHIFT) | (sl_tid_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(sl_tid_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(sl_tid_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(sl_tid_t) - 1 > 3 * PyLong_SHIFT) {
                            return (sl_tid_t) ((((((((sl_tid_t)digits[2]) << PyLong_SHIFT) | (sl_tid_t)digits[1]) << PyLong_SHIFT) | (sl_tid_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(sl_tid_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(sl_tid_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(sl_tid_t) - 1 > 4 * PyLong_SHIFT) {
                            return (sl_tid_t) (((sl_tid_t)-1)*(((((((((sl_tid_t)digits[3]) << PyLong_SHIFT) | (sl_tid_t)digits[2]) << PyLong_SHIFT) | (sl_tid_t)digits[1]) << PyLong_SHIFT) | (sl_tid_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(sl_tid_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(sl_tid_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(sl_tid_t) - 1 > 4 * PyLong_SHIFT) {
                            return (sl_tid_t) ((((((((((sl_tid_t)digits[3]) << PyLong_SHIFT) | (sl_tid_t)digits[2]) << PyLong_SHIFT) | (sl_tid_t)digits[1]) << PyLong_SHIFT) | (sl_tid_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(sl_tid_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(sl_tid_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(sl_tid_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(sl_tid_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            sl_tid_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (sl_tid_t) -1;
        }
    } else {
        sl_tid_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (sl_tid_t) -1;
        val = __Pyx_PyInt_As_sl_tid_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to sl_tid_t");
    return (sl_tid_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to sl_tid_t");
    return (sl_tid_t) -1;
}

/* CIntFromPy */
static CYTHON_INLINE sl_fdh_t __Pyx_PyInt_As_sl_fdh_t(PyObject *x) {
    const sl_fdh_t neg_one = (sl_fdh_t) ((sl_fdh_t) 0 - (sl_fdh_t) 1), const_zero = (sl_fdh_t) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(sl_fdh_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(sl_fdh_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (sl_fdh_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (sl_fdh_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(sl_fdh_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(sl_fdh_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(sl_fdh_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(sl_fdh_t) >= 2 * PyLong_SHIFT) {
                            return (sl_fdh_t) (((((sl_fdh_t)digits[1]) << PyLong_SHIFT) | (sl_fdh_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(sl_fdh_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(sl_fdh_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(sl_fdh_t) >= 3 * PyLong_SHIFT) {
                            return (sl_fdh_t) (((((((sl_fdh_t)digits[2]) << PyLong_SHIFT) | (sl_fdh_t)digits[1]) << PyLong_SHIFT) | (sl_fdh_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(sl_fdh_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(sl_fdh_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(sl_fdh_t) >= 4 * PyLong_SHIFT) {
                            return (sl_fdh_t) (((((((((sl_fdh_t)digits[3]) << PyLong_SHIFT) | (sl_fdh_t)digits[2]) << PyLong_SHIFT) | (sl_fdh_t)digits[1]) << PyLong_SHIFT) | (sl_fdh_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (sl_fdh_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(sl_fdh_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(sl_fdh_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(sl_fdh_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(sl_fdh_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (sl_fdh_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(sl_fdh_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(sl_fdh_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(sl_fdh_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(sl_fdh_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(sl_fdh_t) - 1 > 2 * PyLong_SHIFT) {
                            return (sl_fdh_t) (((sl_fdh_t)-1)*(((((sl_fdh_t)digits[1]) << PyLong_SHIFT) | (sl_fdh_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(sl_fdh_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(sl_fdh_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(sl_fdh_t) - 1 > 2 * PyLong_SHIFT) {
                            return (sl_fdh_t) ((((((sl_fdh_t)digits[1]) << PyLong_SHIFT) | (sl_fdh_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(sl_fdh_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(sl_fdh_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(sl_fdh_t) - 1 > 3 * PyLong_SHIFT) {
                            return (sl_fdh_t) (((sl_fdh_t)-1)*(((((((sl_fdh_t)digits[2]) << PyLong_SHIFT) | (sl_fdh_t)digits[1]) << PyLong_SHIFT) | (sl_fdh_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(sl_fdh_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(sl_fdh_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(sl_fdh_t) - 1 > 3 * PyLong_SHIFT) {
                            return (sl_fdh_t) ((((((((sl_fdh_t)digits[2]) << PyLong_SHIFT) | (sl_fdh_t)digits[1]) << PyLong_SHIFT) | (sl_fdh_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(sl_fdh_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(sl_fdh_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(sl_fdh_t) - 1 > 4 * PyLong_SHIFT) {
                            return (sl_fdh_t) (((sl_fdh_t)-1)*(((((((((sl_fdh_t)digits[3]) << PyLong_SHIFT) | (sl_fdh_t)digits[2]) << PyLong_SHIFT) | (sl_fdh_t)digits[1]) << PyLong_SHIFT) | (sl_fdh_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(sl_fdh_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(sl_fdh_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(sl_fdh_t) - 1 > 4 * PyLong_SHIFT) {
                            return (sl_fdh_t) ((((((((((sl_fdh_t)digits[3]) << PyLong_SHIFT) | (sl_fdh_t)digits[2]) << PyLong_SHIFT) | (sl_fdh_t)digits[1]) << PyLong_SHIFT) | (sl_fdh_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(sl_fdh_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(sl_fdh_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(sl_fdh_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(sl_fdh_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            sl_fdh_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (sl_fdh_t) -1;
        }
    } else {
        sl_fdh_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (sl_fdh_t) -1;
        val = __Pyx_PyInt_As_sl_fdh_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to sl_fdh_t");
    return (sl_fdh_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to sl_fdh_t");
    return (sl_fdh_t) -1;
}

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) ((long) 0 - (long) 1), const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntFromPy */
static CYTHON_INLINE int32 __Pyx_PyInt_As_int32(PyObject *x) {
    const int32 neg_one = (int32) ((int32) 0 - (int32) 1), const_zero = (int32) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int32) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int32, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int32) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int32) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int32, digit, digits[0])
                case 2:
                    if (8 * sizeof(int32) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32) >= 2 * PyLong_SHIFT) {
                            return (int32) (((((int32)digits[1]) << PyLong_SHIFT) | (int32)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int32) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32) >= 3 * PyLong_SHIFT) {
                            return (int32) (((((((int32)digits[2]) << PyLong_SHIFT) | (int32)digits[1]) << PyLong_SHIFT) | (int32)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int32) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32) >= 4 * PyLong_SHIFT) {
                            return (int32) (((((((((int32)digits[3]) << PyLong_SHIFT) | (int32)digits[2]) << PyLong_SHIFT) | (int32)digits[1]) << PyLong_SHIFT) | (int32)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int32) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int32) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int32, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int32) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int32, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int32) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int32, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int32,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int32) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32) - 1 > 2 * PyLong_SHIFT) {
                            return (int32) (((int32)-1)*(((((int32)digits[1]) << PyLong_SHIFT) | (int32)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int32) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32) - 1 > 2 * PyLong_SHIFT) {
                            return (int32) ((((((int32)digits[1]) << PyLong_SHIFT) | (int32)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int32) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32) - 1 > 3 * PyLong_SHIFT) {
                            return (int32) (((int32)-1)*(((((((int32)digits[2]) << PyLong_SHIFT) | (int32)digits[1]) << PyLong_SHIFT) | (int32)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int32) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32) - 1 > 3 * PyLong_SHIFT) {
                            return (int32) ((((((((int32)digits[2]) << PyLong_SHIFT) | (int32)digits[1]) << PyLong_SHIFT) | (int32)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int32) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32) - 1 > 4 * PyLong_SHIFT) {
                            return (int32) (((int32)-1)*(((((((((int32)digits[3]) << PyLong_SHIFT) | (int32)digits[2]) << PyLong_SHIFT) | (int32)digits[1]) << PyLong_SHIFT) | (int32)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int32) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32) - 1 > 4 * PyLong_SHIFT) {
                            return (int32) ((((((((((int32)digits[3]) << PyLong_SHIFT) | (int32)digits[2]) << PyLong_SHIFT) | (int32)digits[1]) << PyLong_SHIFT) | (int32)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int32) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int32, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int32) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int32, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int32 val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int32) -1;
        }
    } else {
        int32 val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int32) -1;
        val = __Pyx_PyInt_As_int32(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int32");
    return (int32) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int32");
    return (int32) -1;
}

/* CIntFromPy */
static CYTHON_INLINE int8 __Pyx_PyInt_As_int8(PyObject *x) {
    const int8 neg_one = (int8) ((int8) 0 - (int8) 1), const_zero = (int8) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int8) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int8, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int8) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int8) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int8, digit, digits[0])
                case 2:
                    if (8 * sizeof(int8) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int8, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int8) >= 2 * PyLong_SHIFT) {
                            return (int8) (((((int8)digits[1]) << PyLong_SHIFT) | (int8)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int8) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int8, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int8) >= 3 * PyLong_SHIFT) {
                            return (int8) (((((((int8)digits[2]) << PyLong_SHIFT) | (int8)digits[1]) << PyLong_SHIFT) | (int8)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int8) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int8, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int8) >= 4 * PyLong_SHIFT) {
                            return (int8) (((((((((int8)digits[3]) << PyLong_SHIFT) | (int8)digits[2]) << PyLong_SHIFT) | (int8)digits[1]) << PyLong_SHIFT) | (int8)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int8) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int8) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int8, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int8) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int8, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int8) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int8, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int8,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int8) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int8, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int8) - 1 > 2 * PyLong_SHIFT) {
                            return (int8) (((int8)-1)*(((((int8)digits[1]) << PyLong_SHIFT) | (int8)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int8) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int8, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int8) - 1 > 2 * PyLong_SHIFT) {
                            return (int8) ((((((int8)digits[1]) << PyLong_SHIFT) | (int8)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int8) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int8, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int8) - 1 > 3 * PyLong_SHIFT) {
                            return (int8) (((int8)-1)*(((((((int8)digits[2]) << PyLong_SHIFT) | (int8)digits[1]) << PyLong_SHIFT) | (int8)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int8) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int8, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int8) - 1 > 3 * PyLong_SHIFT) {
                            return (int8) ((((((((int8)digits[2]) << PyLong_SHIFT) | (int8)digits[1]) << PyLong_SHIFT) | (int8)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int8) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int8, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int8) - 1 > 4 * PyLong_SHIFT) {
                            return (int8) (((int8)-1)*(((((((((int8)digits[3]) << PyLong_SHIFT) | (int8)digits[2]) << PyLong_SHIFT) | (int8)digits[1]) << PyLong_SHIFT) | (int8)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int8) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int8, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int8) - 1 > 4 * PyLong_SHIFT) {
                            return (int8) ((((((((((int8)digits[3]) << PyLong_SHIFT) | (int8)digits[2]) << PyLong_SHIFT) | (int8)digits[1]) << PyLong_SHIFT) | (int8)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int8) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int8, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int8) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int8, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int8 val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int8) -1;
        }
    } else {
        int8 val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int8) -1;
        val = __Pyx_PyInt_As_int8(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int8");
    return (int8) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int8");
    return (int8) -1;
}

/* CIntFromPy */
static CYTHON_INLINE int16 __Pyx_PyInt_As_int16(PyObject *x) {
    const int16 neg_one = (int16) ((int16) 0 - (int16) 1), const_zero = (int16) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int16) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int16, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int16) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int16) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int16, digit, digits[0])
                case 2:
                    if (8 * sizeof(int16) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int16, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int16) >= 2 * PyLong_SHIFT) {
                            return (int16) (((((int16)digits[1]) << PyLong_SHIFT) | (int16)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int16) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int16, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int16) >= 3 * PyLong_SHIFT) {
                            return (int16) (((((((int16)digits[2]) << PyLong_SHIFT) | (int16)digits[1]) << PyLong_SHIFT) | (int16)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int16) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int16, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int16) >= 4 * PyLong_SHIFT) {
                            return (int16) (((((((((int16)digits[3]) << PyLong_SHIFT) | (int16)digits[2]) << PyLong_SHIFT) | (int16)digits[1]) << PyLong_SHIFT) | (int16)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int16) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int16) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int16, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int16) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int16, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int16) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int16, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int16,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int16) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int16, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int16) - 1 > 2 * PyLong_SHIFT) {
                            return (int16) (((int16)-1)*(((((int16)digits[1]) << PyLong_SHIFT) | (int16)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int16) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int16, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int16) - 1 > 2 * PyLong_SHIFT) {
                            return (int16) ((((((int16)digits[1]) << PyLong_SHIFT) | (int16)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int16) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int16, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int16) - 1 > 3 * PyLong_SHIFT) {
                            return (int16) (((int16)-1)*(((((((int16)digits[2]) << PyLong_SHIFT) | (int16)digits[1]) << PyLong_SHIFT) | (int16)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int16) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int16, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int16) - 1 > 3 * PyLong_SHIFT) {
                            return (int16) ((((((((int16)digits[2]) << PyLong_SHIFT) | (int16)digits[1]) << PyLong_SHIFT) | (int16)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int16) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int16, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int16) - 1 > 4 * PyLong_SHIFT) {
                            return (int16) (((int16)-1)*(((((((((int16)digits[3]) << PyLong_SHIFT) | (int16)digits[2]) << PyLong_SHIFT) | (int16)digits[1]) << PyLong_SHIFT) | (int16)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int16) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int16, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int16) - 1 > 4 * PyLong_SHIFT) {
                            return (int16) ((((((((((int16)digits[3]) << PyLong_SHIFT) | (int16)digits[2]) << PyLong_SHIFT) | (int16)digits[1]) << PyLong_SHIFT) | (int16)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int16) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int16, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int16) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int16, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int16 val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int16) -1;
        }
    } else {
        int16 val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int16) -1;
        val = __Pyx_PyInt_As_int16(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int16");
    return (int16) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int16");
    return (int16) -1;
}

/* CIntFromPy */
static CYTHON_INLINE int64 __Pyx_PyInt_As_int64(PyObject *x) {
    const int64 neg_one = (int64) ((int64) 0 - (int64) 1), const_zero = (int64) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int64) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int64, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int64) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int64) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int64, digit, digits[0])
                case 2:
                    if (8 * sizeof(int64) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int64, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int64) >= 2 * PyLong_SHIFT) {
                            return (int64) (((((int64)digits[1]) << PyLong_SHIFT) | (int64)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int64) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int64, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int64) >= 3 * PyLong_SHIFT) {
                            return (int64) (((((((int64)digits[2]) << PyLong_SHIFT) | (int64)digits[1]) << PyLong_SHIFT) | (int64)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int64) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int64, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int64) >= 4 * PyLong_SHIFT) {
                            return (int64) (((((((((int64)digits[3]) << PyLong_SHIFT) | (int64)digits[2]) << PyLong_SHIFT) | (int64)digits[1]) << PyLong_SHIFT) | (int64)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int64) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int64) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int64, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int64) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int64, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int64) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int64, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int64,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int64) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int64, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int64) - 1 > 2 * PyLong_SHIFT) {
                            return (int64) (((int64)-1)*(((((int64)digits[1]) << PyLong_SHIFT) | (int64)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int64) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int64, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int64) - 1 > 2 * PyLong_SHIFT) {
                            return (int64) ((((((int64)digits[1]) << PyLong_SHIFT) | (int64)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int64) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int64, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int64) - 1 > 3 * PyLong_SHIFT) {
                            return (int64) (((int64)-1)*(((((((int64)digits[2]) << PyLong_SHIFT) | (int64)digits[1]) << PyLong_SHIFT) | (int64)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int64) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int64, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int64) - 1 > 3 * PyLong_SHIFT) {
                            return (int64) ((((((((int64)digits[2]) << PyLong_SHIFT) | (int64)digits[1]) << PyLong_SHIFT) | (int64)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int64) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int64, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int64) - 1 > 4 * PyLong_SHIFT) {
                            return (int64) (((int64)-1)*(((((((((int64)digits[3]) << PyLong_SHIFT) | (int64)digits[2]) << PyLong_SHIFT) | (int64)digits[1]) << PyLong_SHIFT) | (int64)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int64) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int64, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int64) - 1 > 4 * PyLong_SHIFT) {
                            return (int64) ((((((((((int64)digits[3]) << PyLong_SHIFT) | (int64)digits[2]) << PyLong_SHIFT) | (int64)digits[1]) << PyLong_SHIFT) | (int64)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int64) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int64, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int64) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int64, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int64 val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int64) -1;
        }
    } else {
        int64 val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int64) -1;
        val = __Pyx_PyInt_As_int64(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int64");
    return (int64) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int64");
    return (int64) -1;
}

/* CIntFromPy */
static CYTHON_INLINE uint8 __Pyx_PyInt_As_uint8(PyObject *x) {
    const uint8 neg_one = (uint8) ((uint8) 0 - (uint8) 1), const_zero = (uint8) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(uint8) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(uint8, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (uint8) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint8) 0;
                case  1: __PYX_VERIFY_RETURN_INT(uint8, digit, digits[0])
                case 2:
                    if (8 * sizeof(uint8) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8) >= 2 * PyLong_SHIFT) {
                            return (uint8) (((((uint8)digits[1]) << PyLong_SHIFT) | (uint8)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint8) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8) >= 3 * PyLong_SHIFT) {
                            return (uint8) (((((((uint8)digits[2]) << PyLong_SHIFT) | (uint8)digits[1]) << PyLong_SHIFT) | (uint8)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint8) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8) >= 4 * PyLong_SHIFT) {
                            return (uint8) (((((((((uint8)digits[3]) << PyLong_SHIFT) | (uint8)digits[2]) << PyLong_SHIFT) | (uint8)digits[1]) << PyLong_SHIFT) | (uint8)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (uint8) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(uint8) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint8, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(uint8) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint8, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint8) 0;
                case -1: __PYX_VERIFY_RETURN_INT(uint8, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(uint8,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(uint8) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8) - 1 > 2 * PyLong_SHIFT) {
                            return (uint8) (((uint8)-1)*(((((uint8)digits[1]) << PyLong_SHIFT) | (uint8)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(uint8) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8) - 1 > 2 * PyLong_SHIFT) {
                            return (uint8) ((((((uint8)digits[1]) << PyLong_SHIFT) | (uint8)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(uint8) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8) - 1 > 3 * PyLong_SHIFT) {
                            return (uint8) (((uint8)-1)*(((((((uint8)digits[2]) << PyLong_SHIFT) | (uint8)digits[1]) << PyLong_SHIFT) | (uint8)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint8) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8) - 1 > 3 * PyLong_SHIFT) {
                            return (uint8) ((((((((uint8)digits[2]) << PyLong_SHIFT) | (uint8)digits[1]) << PyLong_SHIFT) | (uint8)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(uint8) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8) - 1 > 4 * PyLong_SHIFT) {
                            return (uint8) (((uint8)-1)*(((((((((uint8)digits[3]) << PyLong_SHIFT) | (uint8)digits[2]) << PyLong_SHIFT) | (uint8)digits[1]) << PyLong_SHIFT) | (uint8)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint8) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8) - 1 > 4 * PyLong_SHIFT) {
                            return (uint8) ((((((((((uint8)digits[3]) << PyLong_SHIFT) | (uint8)digits[2]) << PyLong_SHIFT) | (uint8)digits[1]) << PyLong_SHIFT) | (uint8)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(uint8) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint8, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(uint8) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint8, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            uint8 val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (uint8) -1;
        }
    } else {
        uint8 val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (uint8) -1;
        val = __Pyx_PyInt_As_uint8(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to uint8");
    return (uint8) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to uint8");
    return (uint8) -1;
}

/* CIntFromPy */
static CYTHON_INLINE uint16 __Pyx_PyInt_As_uint16(PyObject *x) {
    const uint16 neg_one = (uint16) ((uint16) 0 - (uint16) 1), const_zero = (uint16) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(uint16) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(uint16, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (uint16) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint16) 0;
                case  1: __PYX_VERIFY_RETURN_INT(uint16, digit, digits[0])
                case 2:
                    if (8 * sizeof(uint16) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint16, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint16) >= 2 * PyLong_SHIFT) {
                            return (uint16) (((((uint16)digits[1]) << PyLong_SHIFT) | (uint16)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint16) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint16, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint16) >= 3 * PyLong_SHIFT) {
                            return (uint16) (((((((uint16)digits[2]) << PyLong_SHIFT) | (uint16)digits[1]) << PyLong_SHIFT) | (uint16)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint16) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint16, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint16) >= 4 * PyLong_SHIFT) {
                            return (uint16) (((((((((uint16)digits[3]) << PyLong_SHIFT) | (uint16)digits[2]) << PyLong_SHIFT) | (uint16)digits[1]) << PyLong_SHIFT) | (uint16)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (uint16) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(uint16) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint16, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(uint16) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint16, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint16) 0;
                case -1: __PYX_VERIFY_RETURN_INT(uint16, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(uint16,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(uint16) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint16, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint16) - 1 > 2 * PyLong_SHIFT) {
                            return (uint16) (((uint16)-1)*(((((uint16)digits[1]) << PyLong_SHIFT) | (uint16)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(uint16) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint16, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint16) - 1 > 2 * PyLong_SHIFT) {
                            return (uint16) ((((((uint16)digits[1]) << PyLong_SHIFT) | (uint16)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(uint16) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint16, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint16) - 1 > 3 * PyLong_SHIFT) {
                            return (uint16) (((uint16)-1)*(((((((uint16)digits[2]) << PyLong_SHIFT) | (uint16)digits[1]) << PyLong_SHIFT) | (uint16)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint16) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint16, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint16) - 1 > 3 * PyLong_SHIFT) {
                            return (uint16) ((((((((uint16)digits[2]) << PyLong_SHIFT) | (uint16)digits[1]) << PyLong_SHIFT) | (uint16)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(uint16) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint16, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint16) - 1 > 4 * PyLong_SHIFT) {
                            return (uint16) (((uint16)-1)*(((((((((uint16)digits[3]) << PyLong_SHIFT) | (uint16)digits[2]) << PyLong_SHIFT) | (uint16)digits[1]) << PyLong_SHIFT) | (uint16)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint16) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint16, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint16) - 1 > 4 * PyLong_SHIFT) {
                            return (uint16) ((((((((((uint16)digits[3]) << PyLong_SHIFT) | (uint16)digits[2]) << PyLong_SHIFT) | (uint16)digits[1]) << PyLong_SHIFT) | (uint16)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(uint16) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint16, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(uint16) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint16, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            uint16 val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (uint16) -1;
        }
    } else {
        uint16 val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (uint16) -1;
        val = __Pyx_PyInt_As_uint16(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to uint16");
    return (uint16) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to uint16");
    return (uint16) -1;
}

/* CIntFromPy */
static CYTHON_INLINE uint32 __Pyx_PyInt_As_uint32(PyObject *x) {
    const uint32 neg_one = (uint32) ((uint32) 0 - (uint32) 1), const_zero = (uint32) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(uint32) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(uint32, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (uint32) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint32) 0;
                case  1: __PYX_VERIFY_RETURN_INT(uint32, digit, digits[0])
                case 2:
                    if (8 * sizeof(uint32) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32) >= 2 * PyLong_SHIFT) {
                            return (uint32) (((((uint32)digits[1]) << PyLong_SHIFT) | (uint32)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint32) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32) >= 3 * PyLong_SHIFT) {
                            return (uint32) (((((((uint32)digits[2]) << PyLong_SHIFT) | (uint32)digits[1]) << PyLong_SHIFT) | (uint32)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint32) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32) >= 4 * PyLong_SHIFT) {
                            return (uint32) (((((((((uint32)digits[3]) << PyLong_SHIFT) | (uint32)digits[2]) << PyLong_SHIFT) | (uint32)digits[1]) << PyLong_SHIFT) | (uint32)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (uint32) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(uint32) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint32, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(uint32) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint32, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint32) 0;
                case -1: __PYX_VERIFY_RETURN_INT(uint32, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(uint32,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(uint32) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32) - 1 > 2 * PyLong_SHIFT) {
                            return (uint32) (((uint32)-1)*(((((uint32)digits[1]) << PyLong_SHIFT) | (uint32)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(uint32) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32) - 1 > 2 * PyLong_SHIFT) {
                            return (uint32) ((((((uint32)digits[1]) << PyLong_SHIFT) | (uint32)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(uint32) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32) - 1 > 3 * PyLong_SHIFT) {
                            return (uint32) (((uint32)-1)*(((((((uint32)digits[2]) << PyLong_SHIFT) | (uint32)digits[1]) << PyLong_SHIFT) | (uint32)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint32) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32) - 1 > 3 * PyLong_SHIFT) {
                            return (uint32) ((((((((uint32)digits[2]) << PyLong_SHIFT) | (uint32)digits[1]) << PyLong_SHIFT) | (uint32)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(uint32) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32) - 1 > 4 * PyLong_SHIFT) {
                            return (uint32) (((uint32)-1)*(((((((((uint32)digits[3]) << PyLong_SHIFT) | (uint32)digits[2]) << PyLong_SHIFT) | (uint32)digits[1]) << PyLong_SHIFT) | (uint32)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint32) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32) - 1 > 4 * PyLong_SHIFT) {
                            return (uint32) ((((((((((uint32)digits[3]) << PyLong_SHIFT) | (uint32)digits[2]) << PyLong_SHIFT) | (uint32)digits[1]) << PyLong_SHIFT) | (uint32)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(uint32) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint32, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(uint32) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint32, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            uint32 val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (uint32) -1;
        }
    } else {
        uint32 val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (uint32) -1;
        val = __Pyx_PyInt_As_uint32(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to uint32");
    return (uint32) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to uint32");
    return (uint32) -1;
}

/* CIntFromPy */
static CYTHON_INLINE uint64 __Pyx_PyInt_As_uint64(PyObject *x) {
    const uint64 neg_one = (uint64) ((uint64) 0 - (uint64) 1), const_zero = (uint64) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(uint64) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(uint64, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (uint64) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint64) 0;
                case  1: __PYX_VERIFY_RETURN_INT(uint64, digit, digits[0])
                case 2:
                    if (8 * sizeof(uint64) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64) >= 2 * PyLong_SHIFT) {
                            return (uint64) (((((uint64)digits[1]) << PyLong_SHIFT) | (uint64)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint64) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64) >= 3 * PyLong_SHIFT) {
                            return (uint64) (((((((uint64)digits[2]) << PyLong_SHIFT) | (uint64)digits[1]) << PyLong_SHIFT) | (uint64)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint64) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64) >= 4 * PyLong_SHIFT) {
                            return (uint64) (((((((((uint64)digits[3]) << PyLong_SHIFT) | (uint64)digits[2]) << PyLong_SHIFT) | (uint64)digits[1]) << PyLong_SHIFT) | (uint64)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (uint64) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(uint64) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint64, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(uint64) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint64, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint64) 0;
                case -1: __PYX_VERIFY_RETURN_INT(uint64, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(uint64,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(uint64) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64) - 1 > 2 * PyLong_SHIFT) {
                            return (uint64) (((uint64)-1)*(((((uint64)digits[1]) << PyLong_SHIFT) | (uint64)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(uint64) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64) - 1 > 2 * PyLong_SHIFT) {
                            return (uint64) ((((((uint64)digits[1]) << PyLong_SHIFT) | (uint64)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(uint64) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64) - 1 > 3 * PyLong_SHIFT) {
                            return (uint64) (((uint64)-1)*(((((((uint64)digits[2]) << PyLong_SHIFT) | (uint64)digits[1]) << PyLong_SHIFT) | (uint64)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint64) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64) - 1 > 3 * PyLong_SHIFT) {
                            return (uint64) ((((((((uint64)digits[2]) << PyLong_SHIFT) | (uint64)digits[1]) << PyLong_SHIFT) | (uint64)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(uint64) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64) - 1 > 4 * PyLong_SHIFT) {
                            return (uint64) (((uint64)-1)*(((((((((uint64)digits[3]) << PyLong_SHIFT) | (uint64)digits[2]) << PyLong_SHIFT) | (uint64)digits[1]) << PyLong_SHIFT) | (uint64)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint64) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64) - 1 > 4 * PyLong_SHIFT) {
                            return (uint64) ((((((((((uint64)digits[3]) << PyLong_SHIFT) | (uint64)digits[2]) << PyLong_SHIFT) | (uint64)digits[1]) << PyLong_SHIFT) | (uint64)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(uint64) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint64, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(uint64) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint64, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            uint64 val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (uint64) -1;
        }
    } else {
        uint64 val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (uint64) -1;
        val = __Pyx_PyInt_As_uint64(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to uint64");
    return (uint64) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to uint64");
    return (uint64) -1;
}

/* CIntFromPy */
static CYTHON_INLINE char8 __Pyx_PyInt_As_char8(PyObject *x) {
    const char8 neg_one = (char8) ((char8) 0 - (char8) 1), const_zero = (char8) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(char8) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(char8, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (char8) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (char8) 0;
                case  1: __PYX_VERIFY_RETURN_INT(char8, digit, digits[0])
                case 2:
                    if (8 * sizeof(char8) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char8, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char8) >= 2 * PyLong_SHIFT) {
                            return (char8) (((((char8)digits[1]) << PyLong_SHIFT) | (char8)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(char8) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char8, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char8) >= 3 * PyLong_SHIFT) {
                            return (char8) (((((((char8)digits[2]) << PyLong_SHIFT) | (char8)digits[1]) << PyLong_SHIFT) | (char8)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(char8) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char8, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char8) >= 4 * PyLong_SHIFT) {
                            return (char8) (((((((((char8)digits[3]) << PyLong_SHIFT) | (char8)digits[2]) << PyLong_SHIFT) | (char8)digits[1]) << PyLong_SHIFT) | (char8)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (char8) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(char8) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(char8, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(char8) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(char8, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (char8) 0;
                case -1: __PYX_VERIFY_RETURN_INT(char8, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(char8,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(char8) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char8, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char8) - 1 > 2 * PyLong_SHIFT) {
                            return (char8) (((char8)-1)*(((((char8)digits[1]) << PyLong_SHIFT) | (char8)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(char8) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char8, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char8) - 1 > 2 * PyLong_SHIFT) {
                            return (char8) ((((((char8)digits[1]) << PyLong_SHIFT) | (char8)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(char8) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char8, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char8) - 1 > 3 * PyLong_SHIFT) {
                            return (char8) (((char8)-1)*(((((((char8)digits[2]) << PyLong_SHIFT) | (char8)digits[1]) << PyLong_SHIFT) | (char8)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(char8) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char8, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char8) - 1 > 3 * PyLong_SHIFT) {
                            return (char8) ((((((((char8)digits[2]) << PyLong_SHIFT) | (char8)digits[1]) << PyLong_SHIFT) | (char8)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(char8) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char8, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char8) - 1 > 4 * PyLong_SHIFT) {
                            return (char8) (((char8)-1)*(((((((((char8)digits[3]) << PyLong_SHIFT) | (char8)digits[2]) << PyLong_SHIFT) | (char8)digits[1]) << PyLong_SHIFT) | (char8)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(char8) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char8, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char8) - 1 > 4 * PyLong_SHIFT) {
                            return (char8) ((((((((((char8)digits[3]) << PyLong_SHIFT) | (char8)digits[2]) << PyLong_SHIFT) | (char8)digits[1]) << PyLong_SHIFT) | (char8)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(char8) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(char8, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(char8) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(char8, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            char8 val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (char8) -1;
        }
    } else {
        char8 val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (char8) -1;
        val = __Pyx_PyInt_As_char8(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to char8");
    return (char8) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to char8");
    return (char8) -1;
}

/* CIntFromPy */
static CYTHON_INLINE size_t __Pyx_PyInt_As_size_t(PyObject *x) {
    const size_t neg_one = (size_t) ((size_t) 0 - (size_t) 1), const_zero = (size_t) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(size_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(size_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (size_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (size_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(size_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(size_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 2 * PyLong_SHIFT) {
                            return (size_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(size_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 3 * PyLong_SHIFT) {
                            return (size_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(size_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 4 * PyLong_SHIFT) {
                            return (size_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (size_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(size_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(size_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (size_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(size_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(size_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(size_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(size_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                            return (size_t) ((((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(size_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                            return (size_t) ((((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(size_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT) {
                            return (size_t) ((((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(size_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(size_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            size_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (size_t) -1;
        }
    } else {
        size_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (size_t) -1;
        val = __Pyx_PyInt_As_size_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to size_t");
    return (size_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to size_t");
    return (size_t) -1;
}

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* InitStrings */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(b);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
